﻿
-- =============================================
-- Description:	Szótár elemeket létrehozó script 
-- =============================================

DROP PROCEDURE IF EXISTS [dev].uspCreateDictionaryItems
GO

CREATE PROCEDURE [dev].uspCreateDictionaryItems
    @IntezmenyId int = NULL,
    @pTanevNev nvarchar(10) = NULL
AS
BEGIN
  SET NOCOUNT ON;
  DECLARE @EnvironmentName nvarchar(1000), @LastTanevSorszam int
  SET @EnvironmentName = (SELECT [Value] FROM dev.EnvironmentInfo WHERE [Key] = 'Dictionary')

  IF (@EnvironmentName IS NULL OR @EnvironmentName NOT IN ('KK','NSZFH','HOI','AZURE'))
  BEGIN  INSERT INTO T_DICTIONARYITEMBASE (C_VALUE) VALUES ('Nincs Dictionary ertek a dev.EnvironmentInfo tablaban') END -- elszall a migracio ha nincs
  
  SET @LastTanevSorszam  = (SELECT MAX(C_SORSZAM) FROM T_TANEV WHERE TOROLT = 'F' AND (C_KOVETKEZO = 'T' OR C_AKTIV = 'T') AND @IntezmenyId IS NULL OR C_INTEZMENYID = @IntezmenyId)

  DROP TABLE IF EXISTS #Tanevek
  CREATE TABLE #Tanevek (IntezmenyId int, TanevId int)
  IF @IntezmenyId IS NULL BEGIN
      INSERT INTO #Tanevek (IntezmenyId, TanevId)
      SELECT
         i.ID
        ,t.ID
      FROM T_INTEZMENY i
        INNER JOIN T_TANEV t ON t.C_INTEZMENYID = i.ID
          AND C_SORSZAM <= @LastTanevSorszam
          AND (@pTanevNev IS NULL OR t.C_NEV = @pTanevNev)
      WHERE i.C_ISARCHIV = 'F'
    END
    ELSE BEGIN
      INSERT INTO #Tanevek (IntezmenyId, TanevId)
      SELECT
         i.ID
        ,t.ID
      FROM T_INTEZMENY i
        INNER JOIN T_TANEV t ON t.C_INTEZMENYID = i.ID
          AND t.C_INTEZMENYID = i.ID
          AND C_SORSZAM <= @LastTanevSorszam
          AND (@pTanevNev IS NULL OR t.C_NEV = @pTanevNev)
      WHERE i.ID = @IntezmenyId
        AND i.C_ISARCHIV = 'F'
    END
    
  DROP TABLE IF EXISTS #DICTIONARYTYPE
  CREATE TABLE #DICTIONARYTYPE(
     ID INT PRIMARY KEY
    ,C_CODENAME NVARCHAR(255) INDEX IX_CODENAME NONCLUSTERED
    ,C_NAME NVARCHAR(255)
    ,C_ISCUSTOMITEMALLOWED NVARCHAR(1)
    ,C_ISCOLOREDITABLE NVARCHAR(1)
    ,C_DESCRIPTION NVARCHAR(255)
  )
  DROP TABLE IF EXISTS #DICTIONARYITEMBASE
  CREATE TABLE #DICTIONARYITEMBASE (
     ID INT not null PRIMARY KEY
    ,C_NAME NVARCHAR(250) not null
    ,C_NAME_1 NVARCHAR(250) null
    ,C_NAME_2 NVARCHAR(250) null
    ,C_NAME_3 NVARCHAR(250) null
    ,C_NAME_4 NVARCHAR(250) null
    ,C_TYPE NVARCHAR(32) not null INDEX IX_TYPE NONCLUSTERED
    ,C_DICTIONARYTYPEID INT not null
    ,C_PROTECTED CHAR(1) null
    ,C_VISIBLE CHAR(1) not null
    ,C_ORDER INT null
    ,C_CODE NVARCHAR(20) null
    ,C_VALUE INT null
    ,C_COLOR NVARCHAR(6) null
  )

    -- DictionaryType elemek 

    INSERT INTO #DICTIONARYTYPE (ID, C_CODENAME, C_NAME, C_ISCUSTOMITEMALLOWED, C_ISCOLOREDITABLE, C_DESCRIPTION) 
        VALUES
            (186, N'NyugdijTipus', N'Nyugdíj típusai', 'F', 'F', N'NULL'),
            (185, N'DualisHelyszinTipus', N'Duális helyszín típusai', 'F', 'F', N'NULL'),
            (181, N'AgazatAlapVizsgaEredTipus', N'Ágazati alapvizsga eredmény típusok', 'F', 'F', N'NULL'),
            (184, N'JogvKeletkezeseJogcimTipus', N'Jogviszony keletkezésének jogcím típusai', 'F', 'F', N'NULL'),
            (180, N'AgazatAlapOktMegnevTipus', N'Ágazati alapoktatás megnevezések', 'F', 'F', N'NULL'),
            (183, N'TbJogviszonyTipus', N'Jogviszony típusok', 'F', 'F', N'NULL'),
            (182, N'JogvMegszuneseJogcimTipus', N'Jogviszony megszűnésének jogcím típusok', 'F', 'F', N'NULL'),
            (179, N'EszkozIgenylesStatusz', N'Eszközigénylés státusz', 'F', 'F', N'NULL'),
            (178, N'ReszSzakmaTipus', N'Részszakma', 'T', 'F', N'NULL'),
            (176, N'AgazatReszSzakmaTipus', N'Ágazat - Új Szkt. – részszakmához', 'T', 'F', N'NULL'),
            (177, N'SzakmaReszSzakmaTipus', N'Szakma – részszakmához', 'T', 'F', N'NULL'),
            (175, N'FogyatekossagTipus', N'Fogyatékosság típus', 'F', 'F', N'NULL'),
            (173, N'TartJogOkmanyTipus', N'Tartózkodási jogot igazoló okmány', 'F', 'F', N'NULL'),
            (170, N'TartozkodasJogcimTipus', N'Tartózkodás jogcíme', 'F', 'F', N'NULL'),
            (167, N'TKJogalapTipus', N'Törvényes képviselő jogalap', 'F', 'F', N'NULL'),
            (169, N'FelekezetTipus', N'Egyházak', 'F', 'F', N'NULL'),
            (168, N'NemzetisegTipus', N'Nemzetiség', 'F', 'F', N'NULL'),
            (172, N'TanszakTipus', N'Tanszak', 'T', 'F', N'NULL'),
            (171, N'MufajTipus', N'Műfaj', 'T', 'F', N'NULL'),
            (166, N'DigTamEszkozTipus', N'Digitális eszköz típus', 'F', 'F', N'NULL'),
            (165, N'DigPlatformTipus', N'Digitális platform típus', 'F', 'F', N'NULL'),
            (164, N'DigEszkozTipus', N'Digitális eszköz típus', 'F', 'F', N'NULL'),
            (162, N'SzakkepesitesNktTipus', N'Szakképesítés - Nkt.', 'T', 'F', N'NULL'),
            (161, N'TanulmanyiTeruletNktTipus', N'Tanulmányi terület', 'T', 'F', N'NULL'),
            (160, N'TulajdonTipus', N'Tulajdon típusa', 'F', 'F', N'NULL'),
            (163, N'SzakiranyNktTipus', N'Szakirány', 'T', 'F', N'NULL'),
            (159, N'OraFileTipus', N'Órafile típus', 'F', 'F', N'NULL'),
            (158, N'SzirStatTipus', N'SZIR-Stat dokumentum típus', 'F', 'F', N'NULL'),
            (157, N'AlkalmazottMunkaKorTipus', N'Munkakör kategóriák', 'T', 'F', N'NULL'),
            (156, N'MunkaugyTorvenyiHatalyTipus', N'Munkaügy törvényi hatály', 'F', 'F', N'NULL'),
            (155, N'EgyebTevekenysegekTipus', N'Egyéb tevékenységek', 'T', 'F', N'NULL'),
            (149, N'AgazatUjSzktTipus', N'Ágazat - Új Szkt.', 'T', 'F', N'NULL'),
            (150, N'SzakmaTipus', N'Szakma', 'T', 'F', N'NULL'),
            (151, N'SzakmairanyTipus', N'Szakmairány', 'T', 'F', N'NULL'),
            (154, N'JuttatasTipus', N'Szakképzési juttatás', 'F', 'F', N'NULL'),
            (152, N'JogviszonyTipus', N'Jogviszony típusok', 'F', 'F', N'NULL'),
            (153, N'BankszamlaTulajdonosTipus', N'Bankszámla tulajdonos típusok', 'F', 'F', N'NULL'),
            (148, N'AmiJellemzoTipus', N'Tantervi jellemző', 'F', 'F', N'NULL'),
            (147, N'TanterviJellemzoTipus', N'Tantervi jellemző', 'F', 'F', N'NULL'),
            (145, N'AranyJanosProgramTipus', N'Arany János Program típus', 'F', 'F', N'NULL'),
            (144, N'SniTanuloNevelesiFormaTipus', N'SNI tanuló nevelesi forma típus', 'F', 'F', N'NULL'),
            (142, N'BeszerzesiIgenySatusz', N'Beszerzési igény státusz', 'F', 'F', N'NULL'),
            (143, N'MennyisegiEgyseg', N'Mennyiségi egység', 'F', 'F', N'NULL'),
            (141, N'AMIFotargy', N'AMI főtárgy', 'F', 'F', N'NULL'),
            (140, N'AMITantargyKotelezoseg', N'AMI tantárgy kötelezőség', 'F', 'F', N'NULL'),
            (139, N'AMITagozat', N'AMI tagozat', 'F', 'F', N'NULL'),
            (137, N'MuveszetiAg', N'Művészeti ág', 'F', 'F', N'NULL'),
            (138, N'ZenemuveszetiAgTipus', N'Zeneművészeti ág típus', 'F', 'F', N'NULL'),
            (136, N'ESLTantargykategoria', N'ESL tantárgykategória', 'F', 'F', N'NULL'),
            (134, N'TavolletTipus', N'Távollét típus', 'F', 'F', N'NULL'),
            (135, N'TavolletStatusz', N'Távollét státusz', 'F', 'F', N'NULL'),
            (133, N'DokumentumElemiSzint', N'Dokumentum elemi szint', 'F', 'F', N'NULL'),
            (132, N'OktNevelesiKategoria', N'Oktatási-nevelési feladat kategória', 'F', 'F', N'NULL'),
            (131, N'DokumentumTipus', N'Dokumentum típus', 'F', 'F', N'NULL'),
            (128, N'DokumentumStatusz', N'Dokumentum státusz', 'F', 'F', N'NULL'),
            (130, N'DokumentumKulcsszoTipus', N'Dokumentum kulcsszó típus', 'F', 'F', N'NULL'),
            (129, N'DokumentumKategoria', N'Dokumentum kategória', 'F', 'F', N'NULL'),
            (127, N'ErtMondatbankTipus', N'Tanuló értékelés mondatbank típus', 'F', 'F', N'NULL'),
            (119, N'KKKepesitesJellTipus', N'Képesítés jellemző típus', 'F', 'F', N'NULL'),
            (115, N'KKVegzettsegTipus', N'Végzettség típus', 'F', 'F', N'NULL'),
            (122, N'KKAMINepzeneTipus', N'AMI Népzene típus', 'F', 'F', N'NULL'),
            (124, N'KKAMIElektroakuTipus', N'AMI Elektroakusztikus zene típus', 'F', 'F', N'NULL'),
            (116, N'KKAMIVegzettsegTipus', N'AMI Végzettség típus', 'F', 'F', N'NULL'),
            (118, N'KKAMITeruletTipus', N'AMI Terület típus', 'F', 'F', N'NULL'),
            (123, N'KKAMIJazzZeneTipus', N'AMI Jazz-zene típus', 'F', 'F', N'NULL'),
            (125, N'KKTanitoMuvTerTipus', N'Tanító műveltségi terület típus', 'F', 'F', N'NULL'),
            (121, N'KKAMIKlasszikusTipus', N'AMI Klasszikus zene típus', 'F', 'F', N'NULL'),
            (126, N'KKGyogypedTerTipus', N'Gyógypedagógus terület típus', 'F', 'F', N'NULL'),
            (117, N'KKKepesitesTipus', N'Képesítés típus', 'F', 'F', N'NULL'),
            (114, N'KKSzakTipus', N'Szak típus', 'F', 'F', N'NULL'),
            (120, N'KKAMIZenemuvKatTipus', N'AMI Zeneművészeti kategória típus', 'F', 'F', N'NULL'),
            (113, N'KirSzolgaltatasTipus', N'KIR szolgáltatás típus', 'F', 'F', N'NULL'),
            (112, N'SzakTipus', N'Szak típus', 'F', 'F', N'NULL'),
            (111, N'GimnaziumiEvfolyamTipus', N'Gimnáziumi évfolyam típus', 'F', 'F', N'NULL'),
            (109, N'LicenceFunkciokTipus', N'Licence funkciók típus', 'F', 'F', N'NULL'),
            (108, N'PenznemTipus', N'Pénznem típus', 'F', 'F', N'NULL'),
            (106, N'MerohelyTipus', N'Mérőhely típus', 'F', 'F', N'NULL'),
            (107, N'KozmuSzamlaTipus', N'Közmű számla típus', 'F', 'F', N'NULL'),
            (105, N'HittanTipus', N'Hittan típus', 'T', 'F', N'NULL'),
            (103, N'VegzettsegTipus', N'Végzettség típus', 'F', 'F', N'NULL'),
            (102, N'OraTulajdonsagTipus', N'Óra tulajdonság típus', 'F', 'F', N'NULL'),
            (101, N'KozossegiSzolgalatTipus', N'Közösségi szolgálat típus', 'F', 'F', N'NULL'),
            (96, N'VezetoiOraszamokTipus', N'Vezetői óraszám típusa', 'F', 'F', N'NULL'),
            (95, N'FoglalkoztatasTipusa', N'Foglalkoztatás típusa', 'T', 'F', N'NULL'),
            (97, N'ErettsegiSzint', N'Érettségi szint', 'F', 'F', N'NULL'),
            (98, N'ErettsegiTipus', N'Érettségi tipus', 'F', 'F', N'NULL'),
            (99, N'ErettsegiTantargy', N'Érettségi tantárgy', 'F', 'F', N'NULL'),
            (94, N'ReszszakkepesitesTipus', N'Részszakképesítés', 'T', 'F', N'NULL'),
            (91, N'AgazatTipus', N'Ágazat', 'T', 'F', N'NULL'),
            (93, N'SzakmacsoportTipus', N'Szakmacsoport', 'T', 'F', N'NULL'),
            (92, N'SzakkepesitesTipus', N'Szakképesítés', 'T', 'F', N'NULL'),
            (90, N'MukodesiHelyTipus', N'Működési hely típus', 'F', 'F', N'NULL'),
            (87, N'IKTEszkozhasznalatMod', N'IKT eszközhasználat módja', 'F', 'F', N'NULL'),
            (88, N'ElsodlegesIKTEszkoz', N'Pedagógus elsődleges IKT eszköze', 'F', 'F', N'NULL'),
            (89, N'WiFiEleres', N'WiFi elérés', 'F', 'F', N'NULL'),
            (86, N'IKTKompetenciaSzint', N'IKT kompetencia szint', 'F', 'F', N'NULL'),
            (85, N'ENaploHasznalat', N'Használnak-e az intézményben elektronikus naplót?', 'F', 'F', N'NULL'),
            (83, N'TagozatTipus', N'Tagozat típus', 'F', 'F', N'NULL'),
            (84, N'FelvetelStatuszaTipus', N'Felvétel státusza', 'F', 'F', N'NULL'),
            (80, N'Idokeret', N'Időkeret', 'F', 'F', N'NULL'),
            (81, N'OsszegTullepesTipus', N'Összeg túllépés típusa', 'F', 'F', N'NULL'),
            (77, N'HibajegyStatusz', N'Hibajegy státusz', 'F', 'F', N'NULL'),
            (78, N'Prioritas', N'Prioritás', 'F', 'F', N'NULL'),
            (79, N'Forras', N'Forrás', 'F', 'F', N'NULL'),
            (76, N'SzerzodoSzemelyPozicio', N'Szerződő személy pozíciója', 'F', 'F', N'NULL'),
            (73, N'ENaploTipus', N'Elektronikus napló típusa', 'F', 'F', N'NULL'),
            (75, N'ENaploForras', N'Elektronikus napló forrása', 'F', 'F', N'NULL'),
            (74, N'ENaploInfra', N'Elektronikus napló infrastruktúrája', 'F', 'F', N'NULL'),
            (72, N'ESLAdatTipus', N'ESL adat típus', 'F', 'F', N'NULL'),
            (41, N'SorolasOkaTipus', N'Záradékok', 'T', 'F', N'NULL'),
            (71, N'MunkaidoKedvezmenyOka', N'Feladattal terhelt óraszám oka', 'F', 'F', N'NULL'),
            (70, N'BesorolasiFokozatTipus', N'Besorolási fokozat', 'F', 'F', N'NULL'),
            (69, N'PedagogusStatusz', N'Pedagógus státusz', 'F', 'F', N'NULL'),
            (68, N'KretaTipus', N'Kréta típus', 'F', 'F', N'NULL'),
            (67, N'SzerepkorTipus', N'Szerepkör típusok', 'F', 'F', N'NULL'),
            (62, N'SystemSettingsControlType', N'Rendszer beállításokhoz használt vezérlő típusok', 'F', 'F', N'NULL'),
            (60, N'MulasztasStatisztikaTipus', N'Mulasztás statisztika típusok', 'F', 'F', N'NULL'),
            (61, N'RendszerBeallitasTipus', N'Rendszer beállítás típusok', 'F', 'F', N'NULL'),
            (59, N'KizaroCsoportTipus', N'Kizáró csoport típusa', 'F', 'F', N'NULL'),
            (57, N'ErtesitesTipus', N'Értesítés típusok', 'F', 'F', N'NULL'),
            (56, N'ErtesitesSzinTipus', N'Értesítés színtípusok', 'F', 'F', N'NULL'),
            (58, N'TevekenysegTipus', N'Tevékenységek típusai', 'T', 'F', N'NULL'),
            (53, N'EletpalyamodellFokozatTipus', N'Pedagógus életpályamodell fokozatok', 'F', 'F', N'NULL'),
            (54, N'EletpalyamodellStatuszok', N'Pedagógus életpályamodell portfólió státuszok', 'F', 'F', N'NULL'),
            (50, N'MagatartasErtekelesTipus', N'Magatartás típusok', 'F', 'F', N'NULL'),
            (51, N'SzorgalomErtekelesTipus', N'Szorgalom típusok', 'F', 'F', N'NULL'),
            (48, N'EsemenyTipus', N'Esemény típusok', 'T', 'F', N'NULL'),
            (47, N'IgazolasTipus', N'Igazolás típusok', 'T', 'F', N'NULL'),
            (43, N'MulasztasTipus', N'Mulasztás típusok', 'F', 'F', N'NULL'),
            (45, N'ErtekelesMod', N'Értékelés módjai', 'T', 'F', N'NULL'),
            (44, N'OsztalyzatTipus', N'Osztályzat', 'F', 'F', N'NULL'),
            (46, N'ErtekelesTipus', N'Értékelés típusai', 'T', 'F', N'NULL'),
            (42, N'HelyettesitesTipus', N'Helyettesítés típus', 'F', 'F', N'NULL'),
            (40, N'HetNapjaTipus', N'Hét napjai', 'F', 'F', N'NULL'),
            (49, N'HetiRendTipus', N'Hetirend típusai', 'T', 'F', N'NULL'),
            (39, N'NapTipus', N'Nap típusok', 'T', 'F', N'NULL'),
            (38, N'FoglalkozasTipus', N'Foglalkozás típusok', 'F', 'F', N'NULL'),
            (36, N'TargyKategoriaTipus', N'Tárgy kategóriák', 'T', 'F', N'NULL'),
            (37, N'EvfolyamTipus', N'Évfolyam típusok', 'T', 'F', N'NULL'),
            (35, N'OktatasiNevelesiFeladat', N'Köznevelési feladatok', 'T', 'F', N'NULL'),
            (31, N'ErtelmiFogyatekosTipus', N'Értelmi fogyatékos típusok', 'F', 'F', N'NULL'),
            (32, N'HallasserultTipus', N'Hallássérült típusok', 'F', 'F', N'NULL'),
            (33, N'LatasserultTipus', N'Látássérült típusok', 'F', 'F', N'NULL'),
            (28, N'EtkezesiKedvezmenyTipus', N'Étkezési kedvezmény típusok', 'F', 'F', N'NULL'),
            (29, N'HatranyosHelyzetTipus', N'Hátrányos helyzet típusok', 'F', 'F', N'NULL'),
            (30, N'Vallas', N'Vallások', 'T', 'F', N'NULL'),
            (27, N'TanterviJelleg', N'Tantervi jellegek', 'T', 'F', N'NULL'),
            (26, N'TestnevelesTipus', N'Testnevelés típusok', 'F', 'F', N'NULL'),
            (23, N'EszkozJelleg', N'Eszköz jellegek', 'F', 'F', N'NULL'),
            (24, N'EszkozCel', N'Eszköz céljai', 'F', 'F', N'NULL'),
            (25, N'MertekegysegTipus', N'Mértékegységek', 'F', 'F', N'NULL'),
            (19, N'FenntartoTipus', N'Fenntartó típusai', 'F', 'F', N'NULL'),
            (22, N'CsoportTipus', N'Csoport típusok', 'T', 'F', N'NULL'),
            (18, N'RokonsagiFok', N'Rokonsági fokok', 'F', 'F', N'NULL'),
            (12, N'MunkakorTipus', N'Munkakörök', 'T', 'F', N'NULL'),
            (10, N'Anyanyelv', N'Anyanyelv', 'T', 'F', N'NULL'),
            (11, N'IgazolvanyTipus', N'Igazolvány típusok', 'F', 'F', N'NULL'),
            (13, N'MunkaviszonyTipus', N'Munkaviszony típusok', 'T', 'F', N'NULL'),
            (6, N'EszkozTipus', N'Eszköz típus', 'T', 'F', N'NULL'),
            (7, N'EszkozKategoria', N'Eszköz kategória', 'F', 'F', N'NULL'),
            (2, N'IngyenesTankonyvEllatas', N'Ingyenes tankönyvellátás típusa', 'F', 'F', N'NULL'),
            (8, N'TeremTipus', N'Terem típusai', 'T', 'F', N'NULL'),
            (14, N'MagantanulosagOka', N'Magántanulóság okai', 'F', 'F', N'NULL'),
            (55, N'Honapok', N'Hónapok', 'F', 'F', N'NULL'),
            (34, N'DiakigazolvanyIgenyTipus', N'Diákigazolvány igény típusok', 'F', 'F', N'NULL'),
            (17, N'KozteruletJelleg', N'Közterület jellege', 'F', 'F', N'NULL'),
            (52, N'DiakigazolvanyIgenyStatusz', N'Diákigazolvány igénylés hiba státuszok', 'F', 'F', N'NULL'),
            (15, N'OrszagTipus', N'Országok', 'T', 'F', N'NULL'),
            (4, N'EsemenyStatusz', N'Esemény státuszok', 'F', 'F', N'NULL'),
            (21, N'EmailTipus', N'E-mail típusai', 'F', 'F', N'NULL'),
            (20, N'TelefonTipus', N'Telefon típusai', 'F', 'F', N'NULL'),
            (9, N'Allampolgarsag', N'Állampolgárság', 'T', 'F', N'NULL'),
            (16, N'CimTipus', N'Címek típusai', 'T', 'F', N'NULL'),
            (1, N'Nem', N'Felhasználó neme', 'F', 'F', N'NULL')
    SET IDENTITY_INSERT T_DICTIONARYTYPE ON

    INSERT INTO T_DICTIONARYTYPE (ID, C_CODENAME, C_NAME, C_ISCUSTOMITEMALLOWED, C_ISCOLOREDITABLE, C_DESCRIPTION, C_INTEZMENYID, C_TANEVID)
      SELECT
         ID
        ,C_CODENAME
        ,C_NAME
        ,C_ISCUSTOMITEMALLOWED
        ,C_ISCOLOREDITABLE
        ,C_DESCRIPTION
        ,tn.IntezmenyId AS C_INTEZMENYID
        ,tn.TanevId AS C_TANEVID
      FROM #DICTIONARYTYPE tdt
        CROSS JOIN #Tanevek tn
      WHERE NOT EXISTS (SELECT 1 FROM dbo.T_DICTIONARYTYPE dt WHERE dt.ID = tdt.ID AND dt.C_INTEZMENYID = tn.IntezmenyId AND dt.C_TANEVID = tn.TanevId) 

    SET IDENTITY_INSERT T_DICTIONARYTYPE OFF

    -- DictionaryItemBase elemek 
    
    -- A 'KK' sajatos elemek kezdete
    IF (@EnvironmentName IS NOT NULL AND @EnvironmentName = 'KK')
    BEGIN 
   
    INSERT INTO #DICTIONARYITEMBASE (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
        VALUES
            (7624, N'Fizetés nélküli szabadság biztosítás nem szünetel', N'Fizetés nélküli szabadság biztosítás nem szünetel', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 20, NULL, 1, NULL),
            (9088, N'Alkotói szabadság díjazás nélkül', N'Alkotói szabadság díjazás nélkül', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 55, NULL, 2, NULL),
            (9105, N'Születésnapi szabadság', N'Születésnapi szabadság', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 53, NULL, 3, NULL),
            (7575, N'Elfogadva átadva középirányítónak', N'Elfogadva átadva középirányítónak', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 2, NULL, 1, NULL),
            (7652, N'Középirányító által elfogadva', N'Középirányító által elfogadva', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 5, NULL, 2, NULL),
            (7653, N'Középirányító által elutasítva', N'Középirányító által elutasítva', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 6, NULL, 3, NULL),
            (7654, N'Középirányító által hiánypótlásra visszaküldve', N'Középirányító által hiánypótlásra visszaküldve', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 7, NULL, 4, NULL),
            (7681, N'Engedélyezve átadva középirányítónak', N'Engedélyezve átadva középirányítónak', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 8, NULL, 5, NULL),
            (7682, N'Tudomásul véve átadva középirányítónak', N'Tudomásul véve átadva középirányítónak', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 9, NULL, 6, NULL),
            (1518, N'Évközi jegy/értékelés', N'Évközi jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 20, NULL, 1, NULL),
            (1519, N'Félévi jegy/értékelés', N'Félévi jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 30, NULL, 2, NULL),
            (1520, N'Év végi jegy/értékelés', N'Év végi jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 40, NULL, 3, NULL),
            (1521, N'Modulzáró vizsga', N'Modulzáró vizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 600, NULL, 4, NULL),
            (1522, N'I. né. jegy/értékelés', N'I. né. jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 50, NULL, 5, NULL),
            (1523, N'III. né. jegy/értékelés', N'III. né. jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 70, NULL, 6, NULL),
            (1524, N'II. né. jegy/értékelés', N'II. né. jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 60, NULL, 7, NULL),
            (1525, N'IV. né. jegy/értékelés', N'IV. né. jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 80, NULL, 8, NULL),
            (1526, N'Pótvizsga', N'Pótvizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 610, NULL, 9, NULL),
            (1527, N'Osztályozó vizsga', N'Osztályozó vizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 100, NULL, 10, NULL),
            (1528, N'Különbözeti vizsga', N'Különbözeti vizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 110, NULL, 11, NULL),
            (6916, N'Pótló vizsga', N'Pótló vizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 120, NULL, 12, NULL),
            (6917, N'Javítóvizsga', N'Javítóvizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 130, NULL, 13, NULL),
            (6918, N'Beszámoltató vizsga', N'Beszámoltató vizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 140, NULL, 14, NULL),
            (7897, N'Ágazati alapvizsga', N'Ágazati alapvizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 200, NULL, 15, NULL),
            (7898, N'Szakmai vizsga', N'Szakmai vizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 210, NULL, 16, NULL),
            (7899, N'Művészeti alapvizsga', N'Művészeti alapvizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 300, NULL, 17, NULL),
            (7900, N'Művészeti záróvizsga', N'Művészeti záróvizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 310, NULL, 18, NULL),
            (8394, N'Ksz/13 4-es szintű szakképző iskola közismeret nélküli', N'Ksz/13', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4560, N'004', 1, NULL),
            (8395, N'3/15 5-ös szintű technikum', N'3/15.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4345, N'003', 2, NULL),
            (8396, N'9. szakgimnázium NKT (két tanítási nyelvű)', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2328, NULL, 3, NULL),
            (8397, N'10. szakgimnázium NKT (két tanítási nyelvű)', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2428, NULL, 4, NULL),
            (8398, N'11. szakgimnázium NKT (két tanítási nyelvű)', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2528, NULL, 5, NULL),
            (8399, N'12. szakgimnázium NKT (két tanítási nyelvű)', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2628, NULL, 6, NULL),
            (8408, N'Ksz/13', N'Ksz/13', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4545, NULL, 7, NULL),
            (8409, N'Ksz/14', N'Ksz/14', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4550, NULL, 8, NULL),
            (6812, N'fejlesztő pedagógus', N'fejlesztő pedagógus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 20, NULL, 1, NULL),
            (566, N'ápoló', N'ápoló', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 500, NULL, 2, NULL),
            (560, N'dajka', N'dajka', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1165, NULL, 3, NULL),
            (569, N'gazdasági dolgozó', N'gazdasági dolgozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 800, NULL, 4, NULL),
            (561, N'gondozónő és takarító', N'gondozónő és takarító', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 520, NULL, 5, NULL),
            (559, N'gyógypedagógiai asszisztens', N'gyógypedagógiai asszisztens', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 540, NULL, 6, NULL),
            (597, N'gyógypedagógus', N'gyógypedagógus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 50, NULL, 7, NULL),
            (633, N'gyógytornász', N'gyógytornász', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1497, NULL, 8, NULL),
            (567, N'hangszerkarbantartó', N'hangszerkarbantartó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1505, NULL, 9, NULL),
            (584, N'iskolatitkár', N'iskolatitkár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 595, NULL, 10, NULL),
            (572, N'kisegítő dolgozó', N'kisegítő dolgozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 810, NULL, 11, NULL),
            (598, N'konduktor', N'konduktor', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1547, NULL, 12, NULL),
            (3026, N'könyvtáros', N'könyvtáros', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 600, NULL, 13, NULL),
            (582, N'könyvtáros asszisztens', N'könyvtáros asszisztens', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1665, NULL, 14, NULL),
            (581, N'könyvtáros technikus', N'könyvtáros technikus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1667, NULL, 15, NULL),
            (589, N'laboráns', N'laboráns', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 610, NULL, 16, NULL),
            (555, N'logopédus', N'logopédus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1715, NULL, 17, NULL),
            (571, N'műszaki dolgozó', N'műszaki dolgozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 850, NULL, 18, NULL),
            (565, N'műszaki vezető', N'műszaki vezető', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 620, NULL, 19, NULL),
            (586, N'oktatástechnikus', N'oktatástechnikus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 860, NULL, 20, NULL),
            (595, N'óvodapedagógus', N'óvodapedagógus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1873, NULL, 21, NULL),
            (583, N'óvodatitkár', N'óvodatitkár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1877, NULL, 22, NULL),
            (585, N'pedagógiai asszisztens', N'pedagógiai asszisztens', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 640, NULL, 23, NULL),
            (562, N'pedagógiai felügyelő', N'pedagógiai felügyelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1895, NULL, 24, NULL),
            (578, N'pszichológus', N'pszichológus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1945, NULL, 25, NULL),
            (594, N'rendszergazda', N'rendszergazda', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 670, NULL, 26, NULL),
            (564, N'szabadidő-szervező', N'szabadidő-szervező', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 680, NULL, 27, NULL),
            (557, N'szakorvos', N'szakorvos', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2067, NULL, 28, NULL),
            (587, N'számítógép-kezelő', N'számítógép-kezelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2085, NULL, 29, NULL),
            (588, N'számítógép-rendszerprogramozó', N'számítógép-rendszerprogramozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2087, NULL, 30, NULL),
            (580, N'szociálpedagógus', N'szociálpedagógus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2155, NULL, 31, NULL),
            (568, N'úszómester', N'úszómester', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 710, NULL, 32, NULL),
            (570, N'ügyviteli dolgozó', N'ügyviteli dolgozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 890, NULL, 33, NULL),
            (590, N'ügyviteli gépkezelő', N'ügyviteli gépkezelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 900, NULL, 34, NULL),
            (6348, N'együttnevelést segítő pedagógus', N'együttnevelést segítő pedagógus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1210, NULL, 35, NULL),
            (6352, N'ének-zene tanár', N'ének-zene tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 15, NULL, 36, NULL),
            (6361, N'gimnáziumi tanár', N'gimnáziumi tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 30, NULL, 37, NULL),
            (6370, N'gyógytestnevelő tanár', N'gyógytestnevelő tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1495, NULL, 38, NULL),
            (6375, N'idegennyelvtanár', N'idegennyelvtanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 90, NULL, 39, NULL),
            (6377, N'iskola- és óvodapszichológia koordinátora', N'iskola- és óvodapszichológia koordinátora', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1545, NULL, 40, NULL),
            (6382, N'jelmez- és viselettáros', N'jelmez- és viselettáros', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1585, NULL, 41, NULL),
            (6388, N'kollégiumi titkár', N'kollégiumi titkár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1545, NULL, 42, NULL),
            (6392, N'közismereti tantárgyat nemzetiségi nyelven oktató tanár', N'közismereti tantárgyat nemzetiségi nyelven oktató tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 150, NULL, 43, NULL),
            (6404, N'munkaügyi, személyzeti előadó', N'munkaügyi, személyzeti előadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 840, NULL, 44, NULL),
            (6406, N'nemzetiségi nyelvtanár', N'nemzetiségi nyelvtanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1775, NULL, 45, NULL),
            (6407, N'nemzetiségi óvodapedagógus', N'nemzetiségi óvodapedagógus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1780, NULL, 46, NULL),
            (6408, N'nemzetiségi tanító', N'nemzetiségi tanító', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1790, NULL, 47, NULL),
            (6414, N'iskolapszichológus', N'iskolapszichológus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 110, NULL, 48, NULL),
            (6421, N'pszichopedagógus', N'pszichopedagógus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 660, NULL, 49, NULL),
            (6432, N'szakmai elméleti tantárgyat oktató tanár', N'szakmai elméleti tantárgyat oktató tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 230, NULL, 50, NULL),
            (6434, N'szakmai tantárgyat tanító tanár', N'szakmai tantárgyat tanító tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2045, NULL, 51, NULL),
            (6436, N'szakoktató', N'szakoktató', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2065, NULL, 52, NULL),
            (6438, N'szakszolgálati titkár', N'szakszolgálati titkár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2075, NULL, 53, NULL),
            (6447, N'tanácsadó pedagógus', N'tanácsadó pedagógus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2175, NULL, 54, NULL),
            (6453, N'tehetséggondozó koordinátor', N'tehetséggondozó koordinátor', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2225, NULL, 55, NULL),
            (6457, N'testnevelő', N'testnevelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 415, NULL, 56, NULL),
            (6461, N'kollégiumi nevelőtanár', N'kollégiumi nevelőtanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 120, NULL, 57, NULL),
            (6725, N'tanító', N'tanító', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2195, NULL, 58, NULL),
            (6914, N'általános iskolai tanár', N'általános iskolai tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 10, NULL, 59, NULL),
            (6915, N'óvodapszichológus', N'óvodapszichológus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1875, NULL, 60, NULL),
            (6968, N'szakmai tanár', N'szakmai tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 240, NULL, 61, NULL),
            (6969, N'gyakorlati oktató', N'gyakorlati oktató', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 40, NULL, 62, NULL),
            (8349, N'idegennyelv-tantágykategóriát oktató oktató', N'idegennyelv-tantágykategóriát oktató oktató', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 92, NULL, 63, NULL),
            (8350, N'könyvtáros oktató (kizárólag könyvtárosi feladatokat lát el)', N'könyvtáros oktató (kizárólag könyvtárosi feladatokat lát el)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 136, NULL, 64, NULL),
            (8351, N'könyvtáros oktató (oktatói feladatokat is ellát)', N'könyvtáros oktató (oktatói feladatokat is ellát)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 138, NULL, 65, NULL),
            (8352, N'közismereti tantárgyat oktató középiskolai oktató', N'közismereti tantárgyat oktató középiskolai oktató', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 158, NULL, 66, NULL),
            (8353, N'közismereti tantárgyat oktató oktató', N'közismereti tantárgyat oktató oktató', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 168, NULL, 67, NULL),
            (8354, N'szakmai oktató (felsőfokú végzettséggel)', N'szakmai oktató (felsőfokú végzettséggel)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 232, NULL, 68, NULL),
            (8355, N'szakmai oktató (középfokú végzettséggel)', N'szakmai oktató (középfokú végzettséggel)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 234, NULL, 69, NULL)
    END
    -- A 'KK' sajatos elemek vege

    -- A 'NSZFH' sajatos elemek kezdete
    IF (@EnvironmentName IS NOT NULL AND @EnvironmentName = 'NSZFH')
    BEGIN 
   
    INSERT INTO #DICTIONARYITEMBASE (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
        VALUES
            (8996, N'Teljesítette', N'Teljesítette', NULL, NULL, NULL, N'AgazatAlapVizsgaEredTipus', 181, N'T', N'T', 1, NULL, 1, NULL),
            (8997, N'Nem teljesítette', N'Nem teljesítette', NULL, NULL, NULL, N'AgazatAlapVizsgaEredTipus', 181, N'T', N'T', 2, NULL, 2, NULL),
            (8998, N'Mentesítve, előtanulmányok beszámításával', N'Mentesítve, előtanulmányok beszámításával', NULL, NULL, NULL, N'AgazatAlapVizsgaEredTipus', 181, N'T', N'T', 3, NULL, 3, NULL),
            (8973, N'Műszaki ágazati alapoktatás', N'Műszaki ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 1, NULL, 1, NULL),
            (8974, N'Egészségügyi technika ágazati alapoktatás', N'Egészségügyi technika ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 2, NULL, 2, NULL),
            (8975, N'Egészségügy ágazati alapoktatás', N'Egészségügy ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 3, NULL, 3, NULL),
            (8976, N'Élelmiszeripar ágazati alapoktatás', N'Élelmiszeripar ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 4, NULL, 4, NULL),
            (8977, N'Építőipar ágazati alapoktatás', N'Építőipar ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 5, NULL, 5, NULL),
            (8978, N'Épületgépészet ágazati alapoktatás', N'Épületgépészet ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 6, NULL, 6, NULL),
            (8979, N'Fa- és bútoripar ágazati alapoktatás', N'Fa- és bútoripar ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 7, NULL, 7, NULL),
            (8980, N'Gazdálkodás és menedzsment ágazati alapoktatás', N'Gazdálkodás és menedzsment ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 8, NULL, 8, NULL),
            (8981, N'Honvédelem ágazati alapoktatás', N'Honvédelem ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 9, NULL, 9, NULL),
            (8982, N'Informatika és távközlés ágazati alapoktatás', N'Informatika és távközlés ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 10, NULL, 10, NULL),
            (8983, N'Kereskedelem ágazati alapoktatás', N'Kereskedelem ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 11, NULL, 11, NULL),
            (8984, N'Környezetvédelem és vízügy ágazati alapoktatás', N'Környezetvédelem és vízügy ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 12, NULL, 12, NULL),
            (8985, N'Kreatív ipari ágazati alapoktatás', N'Kreatív ipari ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 13, NULL, 13, NULL),
            (8986, N'Vizuális ágazati alapoktatás', N'Vizuális ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 14, NULL, 14, NULL),
            (8987, N'Színház és hangtechnika ágazati alapoktatás', N'Színház és hangtechnika ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 15, NULL, 15, NULL),
            (8988, N'Mezőgazdaság és erdészet ágazati alapoktatás', N'Mezőgazdaság és erdészet ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 16, NULL, 16, NULL),
            (8989, N'Rendészet és közszolgálat alapoktatás', N'Rendészet és közszolgálat alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 17, NULL, 17, NULL),
            (8990, N'Rendészet és közszolgálat (rendőri) alapoktatás', N'Rendészet és közszolgálat (rendőri) alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 18, NULL, 18, NULL),
            (8991, N'Sport ágazati alapoktatás', N'Sport ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 19, NULL, 19, NULL),
            (8992, N'Szépészet ágazati alapoktatás', N'Szépészet ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 20, NULL, 20, NULL),
            (8993, N'Szociális ágazati alapoktatás', N'Szociális ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 21, NULL, 21, NULL),
            (8994, N'Turizmus-vendéglátás ágazati alapoktatás', N'Turizmus-vendéglátás ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 22, NULL, 22, NULL),
            (8995, N'Vegyipar ágazati alapoktatás', N'Vegyipar ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 23, NULL, 23, NULL),
            (9058, N'Oktatás ágazati alapoktatás', N'Oktatás ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'T', N'T', 24, NULL, 24, NULL),
            (7913, N'FejlesztőpedagógusI tevékenység', N'FejlesztőpedagógusI tevékenység', NULL, NULL, NULL, N'EgyebTevekenysegekTipus', 155, N'F', N'T', 100, NULL, 1, NULL),
            (7914, N'Gyakorlati oktatásszervezőI tevékenység', N'Gyakorlati oktatásszervezőI tevékenység', NULL, NULL, NULL, N'EgyebTevekenysegekTipus', 155, N'F', N'T', 110, NULL, 2, NULL),
            (7915, N'Könyvtári tevékenység', N'Könyvtári tevékenység', NULL, NULL, NULL, N'EgyebTevekenysegekTipus', 155, N'F', N'T', 120, NULL, 3, NULL),
            (7916, N'Mentori tevékenység', N'Mentori tevékenység', NULL, NULL, NULL, N'EgyebTevekenysegekTipus', 155, N'F', N'T', 130, NULL, 4, NULL),
            (7917, N'Pályázatírói tevékenység', N'Pályázatírói tevékenység', NULL, NULL, NULL, N'EgyebTevekenysegekTipus', 155, N'F', N'T', 140, NULL, 5, NULL),
            (7918, N'Pszichológusi tevékenység', N'Pszichológusi tevékenység', NULL, NULL, NULL, N'EgyebTevekenysegekTipus', 155, N'F', N'T', 150, NULL, 6, NULL),
            (7919, N'Egyéb tevékenység/tevékenységek', N'Egyéb tevékenység/tevékenységek', NULL, NULL, NULL, N'EgyebTevekenysegekTipus', 155, N'F', N'T', 200, NULL, 7, NULL),
            (8370, N'Diákönkormányzati munkát segítő', N'Diákönkormányzati munkát segítő', NULL, NULL, NULL, N'EgyebTevekenysegekTipus', 155, N'F', N'T', 160, NULL, 8, NULL),
            (8371, N'Munkaközösség-vezető', N'Munkaközösség-vezető', NULL, NULL, NULL, N'EgyebTevekenysegekTipus', 155, N'F', N'T', 170, NULL, 9, NULL),
            (7827, N'Érettségire felkészítő évfolyam Waldorf-intézményben', N'Érettségire felkészítő évfolyam Waldorf-intézményben', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 360, NULL, 1, NULL),
            (7842, N'Szakgimnáziumi nevelés-oktatás (9-12. évfolyam - 2020.09.01-től kifutó)', N'Szakgimnáziumi nevelés-oktatás (9-12. évfolyam - 2020.09.01-től kifutó)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 2000, NULL, 2, NULL),
            (7843, N'Szakgimnáziumi nevelés-oktatás (kizárólag szakképzési évfolyamok - 2020.09.01-től kifutó)', N'Szakgimnáziumi nevelés-oktatás (kizárólag szakképzési évfolyamok - 2020.09.01-től kifutó)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 2010, NULL, 3, NULL),
            (7850, N'Szakközépiskolai nevelés-oktatás (9-11. évfolyam - 2020.09.01-től kifutó)', N'Szakközépiskolai nevelés-oktatás (9-11. évfolyam - 2020.09.01-től kifutó)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 2100, NULL, 4, NULL),
            (7851, N'Szakközépiskolai nevelés-oktatás (érettségire felkészítő évfolyamok - 2020.09.01-től kifutó)', N'Szakközépiskolai nevelés-oktatás (érettségire felkészítő évfolyamok - 2020.09.01-től kifutó)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 2110, NULL, 5, NULL),
            (7852, N'Szakközépiskolai nevelés-oktatás (kizárólag szakmai oktatás, 2016.09.01-től felmenő - 2020.09.01-től kifutó rendszerben)', N'Szakközépiskolai nevelés-oktatás (kizárólag szakmai oktatás, 2016.09.01-től felmenő - 2020.09.01-től kifutó rendszerben)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 2120, NULL, 6, NULL),
            (8779, N'Szakképző iskolai oktatás (kizárólag szakképzési évfolyamok)', NULL, NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'F', 1110, NULL, 7, NULL),
            (8780, N'Előkészítő technikumi oktatás', NULL, NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'F', 1200, NULL, 8, NULL),
            (8782, N'Technikumi oktatás (kizárólag szakképzési évfolyamok)', NULL, NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'F', 1220, NULL, 9, NULL),
            (8792, N'Technikumi oktatás (érettségire felkészítő évfolyamok)', N'Technikumi oktatás (érettségire felkészítő évfolyamok)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'F', 1230, NULL, 10, NULL),
            (8793, N'Szakgimnáziumi nevelés-oktatás (érettségire felkészítő évfolyamok)', N'Szakgimnáziumi nevelés-oktatás (érettségire felkészítő évfolyamok)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'F', 1340, NULL, 11, NULL),
            (7606, N'Rendes szabadság', N'Rendes szabadság', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 2, NULL, 1, NULL),
            (7575, N'Elfogadva átadva középirányítónak', N'Elfogadva átadva középirányítónak', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 2, NULL, 1, NULL),
            (7652, N'Középirányító által elfogadva', N'Középirányító által elfogadva', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 5, NULL, 2, NULL),
            (7653, N'Középirányító által elutasítva', N'Középirányító által elutasítva', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 6, NULL, 3, NULL),
            (7654, N'Középirányító által hiánypótlásra visszaküldve', N'Középirányító által hiánypótlásra visszaküldve', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 7, NULL, 4, NULL),
            (7681, N'Engedélyezve átadva középirányítónak', N'Engedélyezve átadva középirányítónak', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 8, NULL, 5, NULL),
            (7682, N'Tudomásul véve átadva középirányítónak', N'Tudomásul véve átadva középirányítónak', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 9, NULL, 6, NULL),
            (6572, N'intézményvezető', N'intézményvezető', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 10, NULL, 1, NULL),
            (6573, N'tagintézmény-vezető', N'tagintézmény-vezető', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 20, NULL, 2, NULL),
            (6574, N'intézményegység-vezető', N'Head of the Institutional Unit', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 40, NULL, 3, NULL),
            (6575, N'intézményvezető-helyettes', N'intézményvezető-helyettes', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 60, NULL, 4, NULL),
            (6576, N'tagintézményvezető-helyettes', N'tagintézményvezető-helyettes', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 70, NULL, 5, NULL),
            (6577, N'intézményegységvezető-helyettes', N'intézményegységvezető-helyettes', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 90, NULL, 6, NULL),
            (6578, N'gyakorlati oktatásvezető', N'gyakorlati oktatásvezető', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 110, NULL, 7, NULL),
            (7599, N'másik KRÉTA rendszerben vezető', N'másik KRÉTA rendszerben vezető', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 130, NULL, 8, NULL),
            (7755, N'igazgató', N'Director', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 30, NULL, 9, NULL),
            (7756, N'igazgatóhelyettes', N'igazgatóhelyettes', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 80, NULL, 10, NULL),
            (7757, N'kollégiumvezető', N'Head of Dormitory', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 50, NULL, 11, NULL),
            (7758, N'főigazgató', N'főigazgató', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 120, NULL, 12, NULL),
            (7854, N'főigazgató-helyettes', N'főigazgató-helyettes', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'F', N'T', 140, NULL, 13, NULL),
            (8348, N'szakirányú oktatásért felelős igazgatóhelyettes', N'szakirányú oktatásért felelős igazgatóhelyettes', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 150, NULL, 14, NULL),
            (3073, N'egyéb', N'egyéb', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 50, NULL, 1, NULL),
            (3069, N'osztályfőnök', N'osztályfőnök', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 5, NULL, 2, NULL),
            (3071, N'diákönkormányzati munkát segítő', N'diákönkormányzati munkát segítő', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 15, NULL, 3, NULL),
            (3070, N'munkaközösség vezető', N'munkaközösség vezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 10, NULL, 4, NULL),
            (6483, N'osztályfőnök és diákönkormányzati munkát segítő', N'osztályfőnök és diákönkormányzati munkát segítő', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 25, NULL, 5, NULL),
            (3072, N'osztályfőnök és munkaközösség vezető', N'osztályfőnök és munkaközösség vezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 20, NULL, 6, NULL),
            (6484, N'diákönkormányzati munkát segítő és munkaközösség vezető', N'diákönkormányzati munkát segítő és munkaközösség vezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 30, NULL, 7, NULL),
            (6485, N'osztályfőnök és diákönkormányzati munkát segítő és munkaközösség vezető', N'osztályfőnök és diákönkormányzati munkát segítő és munkaközösség vezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 35, NULL, 8, NULL),
            (6971, N'osztályfőnök és egyéb', N'osztályfőnök és egyéb', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 55, NULL, 9, NULL),
            (6972, N'munkaközösség vezető és egyéb', N'munkaközösség vezető és egyéb', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 60, NULL, 10, NULL),
            (6973, N'diákönkormányzati munkát segítő és egyéb', N'diákönkormányzati munkát segítő és egyéb', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 65, NULL, 11, NULL),
            (7798, N'tanszakvezető', N'tanszakvezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 70, NULL, 12, NULL),
            (7799, N'tanszakvezető és munkaközösség vezető', N'tanszakvezető és munkaközösség vezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 75, NULL, 13, NULL),
            (9106, N'tanszakvezető és osztályfőnök', N'tanszakvezető és osztályfőnök', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'F', 15, NULL, 14, NULL),
            (9107, N'tanszakvezető és diákönkormányzati munkát segítő', N'tanszakvezető és diákönkormányzati munkát segítő', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'F', 16, NULL, 15, NULL),
            (9108, N'tanszakvezető, osztályfőnök, diákönkormányzati munkát segítő és munkaközösség vezető', N'tanszakvezető, osztályfőnök, diákönkormányzati munkát segítő és munkaközösség vezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'F', 17, NULL, 16, NULL),
            (9109, N'tanszakvezető, osztályfőnök és diákönkormányzati munkát segítő', N'tanszakvezető, osztályfőnök és diákönkormányzati munkát segítő', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'F', 18, NULL, 17, NULL),
            (9110, N'tanszakvezető, osztályfőnök és munkaközösségvezető', N'tanszakvezető, osztályfőnök és munkaközösségvezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'F', 19, NULL, 18, NULL),
            (9111, N'tanszakvezető, diákönkormányzati munkát segítő és munkaközösségvezető', N'tanszakvezető, diákönkormányzati munkát segítő és munkaközösségvezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'F', 20, NULL, 19, NULL),
            (8828, N'FELTÁR - Eset kezelő', N'FELTÁR - Eset kezelő', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 27, NULL, 1, NULL),
            (8829, N'FELTÁR - Eszköz menedzser', N'FELTÁR - Eszköz menedzser', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 29, NULL, 2, NULL),
            (9061, N'FELTÁR - Szerződés menedzser', N'FELTÁR - Szerződés menedzser', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 32, NULL, 3, NULL),
            (9062, N'FELTÁR - Eset jóváhagyó', N'FELTÁR - Eset jóváhagyó', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 33, NULL, 4, NULL),
            (9063, N'FELTÁR - Eset közreműködő', N'FELTÁR - Eset közreműködő', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 34, NULL, 5, NULL),
            (9064, N'FELTÁR - Munkavégző', N'FELTÁR - Munkavégző', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 35, NULL, 6, NULL),
            (9000, N'Szabadság', N'Szabadság', NULL, NULL, NULL, N'IgazolasTipus', 47, N'T', N'T', 12, NULL, 1, NULL),
            (9115, N'Pótlás', N'Pótlás', NULL, NULL, NULL, N'IgazolasTipus', 47, N'T', N'T', 13, NULL, 2, NULL),
            (9116, N'Betegszabadság', N'Betegszabadság', NULL, NULL, NULL, N'IgazolasTipus', 47, N'T', N'T', 14, NULL, 3, NULL),
            (1518, N'Évközi jegy/értékelés', N'Évközi jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 20, NULL, 1, NULL),
            (1519, N'Félévi jegy/értékelés', N'Félévi jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 30, NULL, 2, NULL),
            (1520, N'Év végi jegy/értékelés', N'Év végi jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 40, NULL, 3, NULL),
            (1521, N'Modulzáró vizsga', N'Modulzáró vizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 600, NULL, 4, NULL),
            (1522, N'I. né. jegy/értékelés', N'I. né. jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 50, NULL, 5, NULL),
            (1523, N'III. né. jegy/értékelés', N'III. né. jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 70, NULL, 6, NULL),
            (1524, N'II. né. jegy/értékelés', N'II. né. jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 60, NULL, 7, NULL),
            (1525, N'IV. né. jegy/értékelés', N'IV. né. jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 80, NULL, 8, NULL),
            (1526, N'Pótvizsga', N'Pótvizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'F', 610, NULL, 9, NULL),
            (1527, N'Osztályozó vizsga', N'Osztályozó vizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 100, NULL, 10, NULL),
            (1528, N'Különbözeti vizsga', N'Különbözeti vizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 110, NULL, 11, NULL),
            (6916, N'Pótló vizsga', N'Pótló vizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 120, NULL, 12, NULL),
            (6917, N'Javítóvizsga', N'Javítóvizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 130, NULL, 13, NULL),
            (6918, N'Beszámoltató vizsga', N'Beszámoltató vizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 140, NULL, 14, NULL),
            (7897, N'Ágazati alapvizsga', N'Ágazati alapvizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 200, NULL, 15, NULL),
            (7898, N'Szakmai vizsga', N'Szakmai vizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 210, NULL, 16, NULL),
            (7899, N'Művészeti alapvizsga', N'Művészeti alapvizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'F', 300, NULL, 17, NULL),
            (7900, N'Művészeti záróvizsga', N'Művészeti záróvizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'F', 310, NULL, 18, NULL),
            (8999, N'Duális foglalkozás', N'Duális foglalkozás', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'T', N'T', 49, NULL, 1, NULL),
            (8271, N'Történelem és állampolgári ismeretek', N'Történelem és állampolgári ismeretek', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 1025, NULL, 1, NULL),
            (8272, N'Természettudomány és földrajz', N'Természettudomány és földrajz', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 1105, NULL, 2, NULL),
            (8273, N'Technológia', N'Technológia', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 1610, NULL, 3, NULL),
            (8274, N'Testnevelés és egészségfejlesztés', N'Testnevelés és egészségfejlesztés', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 1810, NULL, 4, NULL),
            (8275, N'Szakmai elmélet (Bányászat és kohászat - 01 (2020))', N'Szakmai elmélet (Bányászat és kohászat - 01 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8010, NULL, 5, NULL),
            (8276, N'Szakmai elmélet (Egészségügyi technika - 02 (2020))', N'Szakmai elmélet (Egészségügyi technika - 02 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8020, NULL, 6, NULL),
            (8277, N'Szakmai elmélet (Egészségügy - 03 (2020))', N'Szakmai elmélet (Egészségügy - 03 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8030, NULL, 7, NULL),
            (8278, N'Szakmai elmélet (Elektronika és elektrotechnika - 04 (2020))', N'Szakmai elmélet (Elektronika és elektrotechnika - 04 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8040, NULL, 8, NULL),
            (8279, N'Szakmai elmélet (Élelmiszeripar - 05 (2020))', N'Szakmai elmélet (Élelmiszeripar - 05 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8050, NULL, 9, NULL),
            (8280, N'Szakmai elmélet (Építőipar - 06 (2020))', N'Szakmai elmélet (Építőipar - 06 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8060, NULL, 10, NULL),
            (8281, N'Szakmai elmélet (Épületgépészet - 07 (2020))', N'Szakmai elmélet (Épületgépészet - 07 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8070, NULL, 11, NULL),
            (8282, N'Szakmai elmélet (Fa- és bútoripar - 08 (2020))', N'Szakmai elmélet (Fa- és bútoripar - 08 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8080, NULL, 12, NULL),
            (8283, N'Szakmai elmélet (Gazdálkodás és menedzsment - 09 (2020))', N'Szakmai elmélet (Gazdálkodás és menedzsment - 09 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8090, NULL, 13, NULL),
            (8284, N'Szakmai elmélet (Gépészet - 10 (2020))', N'Szakmai elmélet (Gépészet - 10 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8100, NULL, 14, NULL),
            (8285, N'Szakmai elmélet (Honvédelem - 11 (2020))', N'Szakmai elmélet (Honvédelem - 11 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8110, NULL, 15, NULL),
            (8286, N'Szakmai elmélet (Informatika és távközlés - 12 (2020))', N'Szakmai elmélet (Informatika és távközlés - 12 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8120, NULL, 16, NULL),
            (8287, N'Szakmai elmélet (Kereskedelem - 13 (2020))', N'Szakmai elmélet (Kereskedelem - 13 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8130, NULL, 17, NULL),
            (8288, N'Szakmai elmélet (Környezetvédelem és vízügy - 14 (2020))', N'Szakmai elmélet (Környezetvédelem és vízügy - 14 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8140, NULL, 18, NULL),
            (8289, N'Szakmai elmélet (Közlekedés és szállítmányozás - 15 (2020))', N'Szakmai elmélet (Közlekedés és szállítmányozás - 15 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8150, NULL, 19, NULL),
            (8290, N'Szakmai elmélet (Kreatív - 16 (2020))', N'Szakmai elmélet (Kreatív - 16 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8160, NULL, 20, NULL),
            (8291, N'Szakmai elmélet (Mezőgazdaság és erdészet - 17 (2020))', N'Szakmai elmélet (Mezőgazdaság és erdészet - 17 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8170, NULL, 21, NULL),
            (8292, N'Szakmai elmélet (Rendészet és közszolgálat - 18 (2020))', N'Szakmai elmélet (Rendészet és közszolgálat - 18 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8180, NULL, 22, NULL),
            (8293, N'Szakmai elmélet (Specializált gép- és járműgyártás - 19 (2020))', N'Szakmai elmélet (Specializált gép- és járműgyártás - 19 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8190, NULL, 23, NULL),
            (8294, N'Szakmai elmélet (Sport - 20 (2020))', N'Szakmai elmélet (Sport - 20 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8200, NULL, 24, NULL),
            (8295, N'Szakmai elmélet (Szépészet - 21 (2020))', N'Szakmai elmélet (Szépészet - 21 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8210, NULL, 25, NULL),
            (8296, N'Szakmai elmélet (Szociális - 22 (2020))', N'Szakmai elmélet (Szociális - 22 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8220, NULL, 26, NULL),
            (8297, N'Szakmai elmélet (Turizmus-vendéglátás - 23 (2020))', N'Szakmai elmélet (Turizmus-vendéglátás - 23 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8230, NULL, 27, NULL),
            (8298, N'Szakmai elmélet (Vegyipar - 24 (2020))', N'Szakmai elmélet (Vegyipar - 24 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8240, NULL, 28, NULL),
            (8299, N'Szakmai gyakorlat (Bányászat és kohászat - 01 (2020))', N'Szakmai gyakorlat (Bányászat és kohászat - 01 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8510, NULL, 29, NULL),
            (8300, N'Szakmai gyakorlat (Egészségügyi technika - 02 (2020))', N'Szakmai gyakorlat (Egészségügyi technika - 02 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8520, NULL, 30, NULL),
            (8301, N'Szakmai gyakorlat (Egészségügy - 03 (2020))', N'Szakmai gyakorlat (Egészségügy - 03 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8530, NULL, 31, NULL),
            (8302, N'Szakmai gyakorlat (Elektronika és elektrotechnika - 04 (2020))', N'Szakmai gyakorlat (Elektronika és elektrotechnika - 04 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8540, NULL, 32, NULL),
            (8303, N'Szakmai gyakorlat (Élelmiszeripar - 05 (2020))', N'Szakmai gyakorlat (Élelmiszeripar - 05 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8550, NULL, 33, NULL),
            (8304, N'Szakmai gyakorlat (Építőipar - 06 (2020))', N'Szakmai gyakorlat (Építőipar - 06 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8560, NULL, 34, NULL),
            (8305, N'Szakmai gyakorlat (Épületgépészet - 07 (2020))', N'Szakmai gyakorlat (Épületgépészet - 07 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8570, NULL, 35, NULL),
            (8306, N'Szakmai gyakorlat (Fa- és bútoripar - 08 (2020))', N'Szakmai gyakorlat (Fa- és bútoripar - 08 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8580, NULL, 36, NULL),
            (8307, N'Szakmai gyakorlat (Gazdálkodás és menedzsment - 09 (2020))', N'Szakmai gyakorlat (Gazdálkodás és menedzsment - 09 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8590, NULL, 37, NULL),
            (8308, N'Szakmai gyakorlat (Gépészet - 10 (2020))', N'Szakmai gyakorlat (Gépészet - 10 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8600, NULL, 38, NULL),
            (8309, N'Szakmai gyakorlat (Honvédelem - 11 (2020))', N'Szakmai gyakorlat (Honvédelem - 11 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8610, NULL, 39, NULL),
            (8310, N'Szakmai gyakorlat (Informatika és távközlés - 12 (2020))', N'Szakmai gyakorlat (Informatika és távközlés - 12 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8620, NULL, 40, NULL),
            (8311, N'Szakmai gyakorlat (Kereskedelem - 13 (2020))', N'Szakmai gyakorlat (Kereskedelem - 13 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8630, NULL, 41, NULL),
            (8312, N'Szakmai gyakorlat (Környezetvédelem és vízügy - 14 (2020))', N'Szakmai gyakorlat (Környezetvédelem és vízügy - 14 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8640, NULL, 42, NULL),
            (8313, N'Szakmai gyakorlat (Közlekedés és szállítmányozás - 15 (2020))', N'Szakmai gyakorlat (Közlekedés és szállítmányozás - 15 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8650, NULL, 43, NULL),
            (8314, N'Szakmai gyakorlat (Kreatív - 16 (2020))', N'Szakmai gyakorlat (Kreatív - 16 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8660, NULL, 44, NULL),
            (8315, N'Szakmai gyakorlat (Mezőgazdaság és erdészet - 17 (2020))', N'Szakmai gyakorlat (Mezőgazdaság és erdészet - 17 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8670, NULL, 45, NULL),
            (8316, N'Szakmai gyakorlat (Rendészet és közszolgálat - 18 (2020))', N'Szakmai gyakorlat (Rendészet és közszolgálat - 18 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8680, NULL, 46, NULL),
            (8317, N'Szakmai gyakorlat (Specializált gép- és járműgyártás - 19 (2020))', N'Szakmai gyakorlat (Specializált gép- és járműgyártás - 19 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8690, NULL, 47, NULL),
            (8318, N'Szakmai gyakorlat (Sport - 20 (2020))', N'Szakmai gyakorlat (Sport - 20 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8700, NULL, 48, NULL),
            (8319, N'Szakmai gyakorlat (Szépészet - 21 (2020))', N'Szakmai gyakorlat (Szépészet - 21 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8710, NULL, 49, NULL),
            (8320, N'Szakmai gyakorlat (Szociális - 22 (2020))', N'Szakmai gyakorlat (Szociális - 22 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8720, NULL, 50, NULL),
            (8321, N'Szakmai gyakorlat (Turizmus-vendéglátás - 23 (2020))', N'Szakmai gyakorlat (Turizmus-vendéglátás - 23 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8730, NULL, 51, NULL),
            (8322, N'Szakmai gyakorlat (Vegyipar - 24 (2020))', N'Szakmai gyakorlat (Vegyipar - 24 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 8740, NULL, 52, NULL),
            (8323, N'Szakmai', N'Szakmai', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9000, NULL, 53, NULL),
            (8324, N'Szakmai (Bányászat és kohászat - 01 (2020))', N'Szakmai (Bányászat és kohászat - 01 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9010, NULL, 54, NULL),
            (8325, N'Szakmai (Egészségügyi technika - 02 (2020))', N'Szakmai (Egészségügyi technika - 02 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9020, NULL, 55, NULL),
            (8326, N'Szakmai (Egészségügy - 03 (2020))', N'Szakmai (Egészségügy - 03 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9030, NULL, 56, NULL),
            (8327, N'Szakmai (Elektronika és elektrotechnika - 04 (2020))', N'Szakmai (Elektronika és elektrotechnika - 04 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9040, NULL, 57, NULL),
            (8328, N'Szakmai (Élelmiszeripar - 05 (2020))', N'Szakmai (Élelmiszeripar - 05 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9050, NULL, 58, NULL),
            (8329, N'Szakmai (Építőipar - 06 (2020))', N'Szakmai (Építőipar - 06 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9060, NULL, 59, NULL),
            (8330, N'Szakmai (Épületgépészet - 07 (2020))', N'Szakmai (Épületgépészet - 07 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9070, NULL, 60, NULL),
            (8331, N'Szakmai (Fa- és bútoripar - 08 (2020))', N'Szakmai (Fa- és bútoripar - 08 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9080, NULL, 61, NULL),
            (8332, N'Szakmai (Gazdálkodás és menedzsment - 09 (2020))', N'Szakmai (Gazdálkodás és menedzsment - 09 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9090, NULL, 62, NULL),
            (8333, N'Szakmai (Gépészet - 10 (2020))', N'Szakmai (Gépészet - 10 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9100, NULL, 63, NULL),
            (8334, N'Szakmai (Honvédelem - 11 (2020))', N'Szakmai (Honvédelem - 11 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9110, NULL, 64, NULL),
            (8335, N'Szakmai (Informatika és távközlés - 12 (2020))', N'Szakmai (Informatika és távközlés - 12 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9120, NULL, 65, NULL),
            (8336, N'Szakmai (Kereskedelem - 13 (2020))', N'Szakmai (Kereskedelem - 13 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9130, NULL, 66, NULL),
            (8337, N'Szakmai (Környezetvédelem és vízügy - 14 (2020))', N'Szakmai (Környezetvédelem és vízügy - 14 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9140, NULL, 67, NULL),
            (8338, N'Szakmai (Közlekedés és szállítmányozás - 15 (2020))', N'Szakmai (Közlekedés és szállítmányozás - 15 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9150, NULL, 68, NULL),
            (8339, N'Szakmai (Kreatív - 16 (2020))', N'Szakmai (Kreatív - 16 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9160, NULL, 69, NULL),
            (8340, N'Szakmai (Mezőgazdaság és erdészet - 17 (2020))', N'Szakmai (Mezőgazdaság és erdészet - 17 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9170, NULL, 70, NULL),
            (8341, N'Szakmai (Rendészet és közszolgálat - 18 (2020))', N'Szakmai (Rendészet és közszolgálat - 18 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9180, NULL, 71, NULL),
            (8342, N'Szakmai (Specializált gép- és járműgyártás - 19 (2020))', N'Szakmai (Specializált gép- és járműgyártás - 19 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9190, NULL, 72, NULL),
            (8343, N'Szakmai (Sport - 20 (2020))', N'Szakmai (Sport - 20 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9200, NULL, 73, NULL),
            (8344, N'Szakmai (Szépészet - 21 (2020))', N'Szakmai (Szépészet - 21 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9210, NULL, 74, NULL),
            (8345, N'Szakmai (Szociális - 22 (2020))', N'Szakmai (Szociális - 22 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9220, NULL, 75, NULL),
            (8346, N'Szakmai (Turizmus-vendéglátás - 23 (2020))', N'Szakmai (Turizmus-vendéglátás - 23 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9230, NULL, 76, NULL),
            (8347, N'Szakmai (Vegyipar - 24 (2020))', N'Szakmai (Vegyipar - 24 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 9240, NULL, 77, NULL),
            (8789, N'Hit- és erkölcstan', NULL, NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1044, NULL, 78, NULL),
            (8790, N'Etika', NULL, NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1042, NULL, 79, NULL),
            (1296, N'Na', N'Na', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 1, NULL, 1, NULL),
            (1297, N'Kis', N'Kis', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 1010, NULL, 2, NULL),
            (1298, N'Középső', N'Középső', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 1020, NULL, 3, NULL),
            (6470, N'Nagy középső', N'Nagy középső', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 1030, NULL, 4, NULL),
            (1299, N'Nagy', N'Nagy', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 1040, NULL, 5, NULL),
            (1300, N'Óvodai vegyes', N'Óvodai vegyes', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 1050, NULL, 6, NULL),
            (1301, N'Előképző 1', N'Előképző 1.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 1060, NULL, 7, NULL),
            (1302, N'Előképző 2', N'Előképző 2.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 1070, NULL, 8, NULL),
            (1303, N'1 ált isk', N'1.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2110, NULL, 9, NULL),
            (1304, N'2 ált isk', N'2.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2120, NULL, 10, NULL),
            (1305, N'3 ált isk', N'3.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2130, NULL, 11, NULL),
            (1306, N'4 ált isk', N'4.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2140, NULL, 12, NULL),
            (1307, N'5 ált isk', N'5.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2150, NULL, 13, NULL),
            (1308, N'6 ált isk', N'6.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2160, NULL, 14, NULL),
            (1309, N'7 ált isk', N'7.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2170, NULL, 15, NULL),
            (1310, N'8 ált isk', N'8.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2180, NULL, 16, NULL),
            (1311, N'9', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2310, NULL, 17, NULL),
            (1312, N'10', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2410, NULL, 18, NULL),
            (1313, N'11', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2510, NULL, 19, NULL),
            (1314, N'12', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2610, NULL, 20, NULL),
            (1315, N'13', N'13.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2710, NULL, 21, NULL),
            (1330, N'5/Ny', N'5/Ny', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3010, NULL, 22, NULL),
            (1331, N'7/Ny', N'7/Ny', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3020, NULL, 23, NULL),
            (1332, N'9/Ny', N'9/Ny', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3040, NULL, 24, NULL),
            (1333, N'9/Kny', N'9/Kny', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3120, NULL, 25, NULL),
            (1334, N'5/N', N'5/N', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3210, NULL, 26, NULL),
            (1335, N'7/N', N'7/N', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3220, NULL, 27, NULL),
            (1336, N'9/N', N'9/N', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3230, NULL, 28, NULL),
            (3019, N'9/AJKP', N'9/AJKP', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3310, NULL, 29, NULL),
            (3018, N'9/AJTP', N'9/AJTP', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3320, NULL, 30, NULL),
            (3020, N'9/E szakiskola elokészíto enyhe értelmi fogyatékos', N'9/E', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3420, NULL, 31, NULL),
            (6471, N'KH/1/1', N'KH', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3510, NULL, 32, NULL),
            (6472, N'KH/2/1/C', N'KH', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3520, NULL, 33, NULL),
            (6473, N'KH/2/1/A', N'KH', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3530, NULL, 34, NULL),
            (6474, N'KH/2/2/A', N'KH', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3540, NULL, 35, NULL),
            (6475, N'KH/2/1/B', N'KH', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3550, NULL, 36, NULL),
            (6476, N'KH/2/2/B', N'KH', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3560, NULL, 37, NULL),
            (6477, N'SZH/1', N'SZH/1', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3610, NULL, 38, NULL),
            (6478, N'SZH/2', N'SZH/2', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3620, NULL, 39, NULL),
            (6479, N'SZH/1/A', N'SZH/1', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3630, NULL, 40, NULL),
            (6480, N'SZH/2/A', N'SZH/2', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3640, NULL, 41, NULL),
            (6481, N'SZH/1/B', N'SZH/1', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3650, NULL, 42, NULL),
            (6482, N'SZH/2/B', N'SZH/2', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3660, NULL, 43, NULL),
            (1324, N'1/8', N'1/8.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4020, NULL, 44, NULL),
            (1316, N'1/9', N'1/9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4140, NULL, 45, NULL),
            (1323, N'2/10', N'2/10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4150, NULL, 46, NULL),
            (1325, N'3/11', N'3/11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4160, NULL, 47, NULL),
            (6520, N'4/12', N'4/12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4170, NULL, 48, NULL),
            (6469, N'5/13', N'5/13.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4190, NULL, 49, NULL),
            (1317, N'1/11', N'1/11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4210, NULL, 50, NULL),
            (1318, N'2/12', N'2/12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4220, NULL, 51, NULL),
            (1322, N'3/13', N'3/13.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4230, NULL, 52, NULL),
            (1319, N'1/13', N'1/13.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4320, NULL, 53, NULL),
            (1320, N'2/14', N'2/14.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4340, NULL, 54, NULL),
            (1321, N'3/15', N'3/15.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4350, NULL, 55, NULL),
            (6468, N'4/16', N'4/16.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4360, NULL, 56, NULL),
            (1326, N'1/12', N'1/12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4420, NULL, 57, NULL),
            (1337, N'1/15', N'1/15.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4440, NULL, 58, NULL),
            (3016, N'Ksz/11', N'Ksz/11', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4520, NULL, 59, NULL),
            (3017, N'Ksz/12', N'Ksz/12', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4540, NULL, 60, NULL),
            (3021, N'Szél/12/1', N'Szél/12/1', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 640, NULL, 61, NULL),
            (3022, N'Szél/12/2', N'Szél/12/2', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 650, NULL, 62, NULL),
            (6702, N'9/S szakiskola enyhe értelmi fogyatékosok', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4610, NULL, 63, NULL),
            (6703, N'10/S szakiskola enyhe értelmi fogyatékosok', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4620, NULL, 64, NULL),
            (6704, N'11/S szakiskola enyhe értelmi fogyatékosok', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4630, NULL, 65, NULL),
            (6705, N'12/S szakiskola enyhe értelmi fogyatékosok', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4640, NULL, 66, NULL),
            (6818, N'Kk 12', N'Kk 12', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4710, NULL, 67, NULL),
            (6819, N'Kk 13', N'Kk 13', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4720, NULL, 68, NULL),
            (6908, N'Összevont 1-2', N'Összevont 1-2', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 5020, NULL, 69, NULL),
            (6909, N'Összevont 1-3', N'Összevont 1-3', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 5030, NULL, 70, NULL),
            (6910, N'Összevont 1-4', N'Összevont 1-4', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 5040, NULL, 71, NULL),
            (6911, N'Összevont 2-3', N'Összevont 2-3', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 5050, NULL, 72, NULL),
            (6912, N'Összevont 2-4', N'Összevont 2-4', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 5060, NULL, 73, NULL),
            (6913, N'Összevont 3-4', N'Összevont 3-4', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 5070, NULL, 74, NULL),
            (7109, N'7 gimnázium', N'7.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2190, NULL, 75, NULL),
            (7110, N'8 gimnázium', N'8.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2200, NULL, 76, NULL),
            (7111, N'9 szakgimnázium', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2320, NULL, 77, NULL),
            (7112, N'9 gimnázium', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2330, NULL, 78, NULL),
            (7113, N'9 szakiskola', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2340, NULL, 79, NULL),
            (7114, N'9 készségfejlesztő iskola', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2350, NULL, 80, NULL),
            (7115, N'10 szakgimnázium', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2420, NULL, 81, NULL),
            (7116, N'10 gimnázium', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2430, NULL, 82, NULL),
            (7117, N'10 szakiskola', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2440, NULL, 83, NULL),
            (7118, N'10 készségfejlesztő iskola', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2450, NULL, 84, NULL),
            (7119, N'11 szakgimnázium', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2520, NULL, 85, NULL),
            (7120, N'11 gimnázium', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2530, NULL, 86, NULL),
            (7121, N'11 szakiskola', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2540, NULL, 87, NULL),
            (7122, N'11 készségfejlesztő iskola', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2550, NULL, 88, NULL),
            (7123, N'12 szakgimnázium', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2620, NULL, 89, NULL),
            (7124, N'12 gimnázium', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2630, NULL, 90, NULL),
            (7125, N'12 szakiskola', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2640, NULL, 91, NULL),
            (7126, N'12 készségfejlesztő iskola', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2650, NULL, 92, NULL),
            (7127, N'9/Ny gimnázium nyelvi előkészítő', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3050, NULL, 93, NULL),
            (7128, N'9/Kny gimnázium két tanítási nyelvű előkészítő', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3130, NULL, 94, NULL),
            (7129, N'Kollégium', N'Kollégium', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 6010, NULL, 95, NULL),
            (7366, N'Összevont évfolyam', N'Összevont', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 5010, NULL, 96, NULL),
            (7862, N'1/9 4-es szintű szakképző iskola', N'1/9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4141, N'002', 97, NULL),
            (7863, N'2/10 4-es szintű szakképző iskola', N'2/10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4151, N'003', 98, NULL),
            (7864, N'3/11 4-es szintű szakképző iskola', N'3/11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4161, N'003', 99, NULL),
            (7866, N'Ksz/11 4-es szintű szakképző iskola közismeret nélküli', N'Ksz/11', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4521, N'002', 100, NULL),
            (7867, N'Ksz/12 4-es szintű szakképző iskola közismeret nélküli', N'Ksz/12', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4522, N'003', 101, NULL),
            (7868, N'9. orientációs évfolyam', N'9/Oe', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4010, N'004', 102, NULL),
            (7869, N'9/Ny technikum nyelvi előkészítő', N'9/Ny', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3042, N'001', 103, NULL),
            (7870, N'9/Kny technikum két tanítási nyelvű előkészítő', N'9/Kny', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3122, N'001', 104, NULL),
            (7871, N'9. technikum', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2322, N'001', 105, NULL),
            (7872, N'9. technikum (két tanítási nyelvű)', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2324, N'001', 106, NULL),
            (7873, N'10. technikum', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2422, N'001', 107, NULL),
            (7874, N'10. technikum (két tanítási nyelvű)', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2424, N'001', 108, NULL),
            (7875, N'11. technikum', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2522, N'003', 109, NULL),
            (7876, N'11. technikum (két tanítási nyelvű)', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2524, N'003', 110, NULL),
            (7877, N'12. technikum', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2622, N'003', 111, NULL),
            (7878, N'12. technikum (két tanítási nyelvű)', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2624, N'003', 112, NULL),
            (7883, N'1/13 5-ös szintű technikum', N'1/13.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4322, N'001', 113, NULL),
            (7884, N'2/14 5-ös szintű technikum', N'2/14.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4342, N'003', 114, NULL),
            (7901, N'1/12 35-ös szintű kifutó ráépülés szakképző iskola', N'1/12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4421, NULL, 115, NULL),
            (7902, N'Kk 12 szakképző iskola két éves érettségire felkészítő', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4711, NULL, 116, NULL),
            (7903, N'Kk 13 szakképző iskola két éves érettségire felkészítő', N'13.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4721, NULL, 117, NULL),
            (7904, N'Dobbantó évfolyam', N'E/D', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4000, N'004', 118, NULL),
            (7905, N'9/Ny szakgimnázium (művészeti, pedagógiai, közművelődési) nyelvi előkészítő', N'9/Ny', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3044, NULL, 119, NULL),
            (7906, N'9. szakgimnázium (művészeti, pedagógiai, közművelődési)', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2326, NULL, 120, NULL),
            (7907, N'10. szakgimnázium (művészeti, pedagógiai, közművelődési)', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2426, NULL, 121, NULL),
            (7908, N'11. szakgimnázium (művészeti, pedagógiai, közművelődési)', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2526, NULL, 122, NULL),
            (7909, N'12. szakgimnázium (művészeti, pedagógiai, közművelődési)', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2626, NULL, 123, NULL),
            (7910, N'5/13 5-ös szintű technikum beszámításos egy éves', N'5/13.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4192, NULL, 124, NULL),
            (7911, N'1/15 55-ös szintű kifutó ráépülés technikum', N'1/15.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4441, NULL, 125, NULL),
            (8401, N'Fejlesztő nevelés-oktatás', N'Fejlesztő nevelés-oktatás', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 6020, NULL, 126, NULL),
            (8769, N'1/13 szakgimnázium (művészeti, pedagógiai, közművelődési)', N'1/13.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4324, NULL, 127, NULL),
            (8770, N'2/14 szakgimnázium (művészeti, pedagógiai, közművelődési)', N'2/14.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4344, NULL, 128, NULL),
            (8771, N'3/15 szakgimnázium (művészeti, pedagógiai, közművelődési)', N'3/15.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4347, NULL, 129, NULL),
            (1156, N'Na', N'Na', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 1, NULL, 1, NULL),
            (1157, N'Óvoda', N'Óvoda', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 10, NULL, 2, NULL),
            (1158, N'Általános iskola', N'Általános iskola', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 20, NULL, 3, NULL),
            (1159, N'Alapfokú művészetoktatás', N'Alapfokú művészetoktatás', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 140, NULL, 4, NULL),
            (1160, N'Gimnázium', N'Gimnázium', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 30, NULL, 5, NULL),
            (1161, N'Szakközépiskola', N'Szakközépiskola', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 90, NULL, 6, NULL),
            (1162, N'Szakiskola', N'Szakiskola', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 100, NULL, 7, NULL),
            (5435, N'Szakgimnázium', N'Szakgimnázium', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 40, NULL, 8, NULL),
            (1165, N'Kollégium', N'Kollégium', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 130, NULL, 9, NULL),
            (1166, N'Gyermekotthoni feladatok', N'Gyermekotthoni feladatok', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 200, NULL, 10, NULL),
            (1173, N'Konduktív pedagógiai ellátás', N'Konduktív pedagógiai ellátás', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 190, NULL, 11, NULL),
            (1176, N'Felnőttképzés', N'Felnőttképzés', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 1000, NULL, 12, NULL),
            (6463, N'Felnőttoktatás', N'Felnőttoktatás', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 120, NULL, 13, NULL),
            (1177, N'Óvoda (gyógypedagógia)', N'Óvoda (gyógypedagógia)', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 170, NULL, 14, NULL),
            (1178, N'Általános iskola (gyógypedagógia)', N'Általános iskola (gyógypedagógia)', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 180, NULL, 15, NULL),
            (1180, N'Készségfejlesztő iskola', N'Készségfejlesztő iskola', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 110, NULL, 16, NULL),
            (1195, N'Pedagógiai szakszolgálat', N'Pedagógiai szakszolgálat', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 160, NULL, 17, NULL),
            (1196, N'Kiegészítő nemzetiségi nyelvoktatás', N'Kiegészítő nemzetiségi nyelvoktatás', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 350, NULL, 18, NULL),
            (6570, N'Fejlesztő nevelés-oktatás', N'Fejlesztő nevelés-oktatás', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 150, NULL, 19, NULL),
            (6711, N'Utazó gyógypedagógusi hálózat', N'Utazó gyógypedagógusi hálózat', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 210, NULL, 20, NULL),
            (7664, N'Művészeti szakgimnázium', N'Művészeti szakgimnázium', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 50, NULL, 21, NULL),
            (7760, N'Köznevelési és Szakképzési Hídprogram', N'Köznevelési és Szakképzési Hídprogram', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 230, NULL, 22, NULL),
            (7761, N'Utazó konduktori hálózat', N'Utazó konduktori hálózat', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 220, NULL, 23, NULL),
            (7762, N'Óvoda (nemzetiségi)', N'Óvoda (nemzetiségi)', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 1100, NULL, 24, NULL),
            (7763, N'Általános iskola (nemzetiségi)', N'Általános iskola (nemzetiségi)', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 1110, NULL, 25, NULL),
            (7764, N'Gimnázium (nemzetiségi)', N'Gimnázium (nemzetiségi)', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 1120, NULL, 26, NULL),
            (7765, N'Szakgimnázium (nemzetiségi)', N'Szakgimnázium (nemzetiségi)', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 1130, NULL, 27, NULL),
            (7766, N'Szakközépiskola (nemzetiségi)', N'Szakközépiskola (nemzetiségi)', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 1140, NULL, 28, NULL),
            (7767, N'Kollégium (nemzetiségi)', N'Kollégium (nemzetiségi)', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 1150, NULL, 29, NULL),
            (7759, N'Pedagógiai-szakmai szolgáltatás', N'Pedagógiai-szakmai szolgáltatás', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'F', 500, NULL, 30, NULL),
            (7894, N'Szakgimnázium (művészeti, pedagógiai, közművelődési)', N'Szakgimnázium', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 60, NULL, 31, NULL),
            (7895, N'Technikum', N'Technikum', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 70, NULL, 32, NULL),
            (7896, N'Szakképző iskola', N'Szakképző iskola', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 80, NULL, 33, NULL),
            (1096, N'Nappali rendszerű iskolai oktatás (*)', N'Nappali rendszerű iskolai oktatás', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 10, NULL, 1, NULL),
            (1099, N'Kollégiumi nevelés (*)', N'Bentlakásos intézményi ellátás', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 90, NULL, 2, NULL),
            (5446, N'Felnőttoktatás esti rendszerű (*)', N'Esti rendszerű felnőttoktatás', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 60, NULL, 3, NULL),
            (5447, N'Felnőttoktatás levelező rendszerű (*)', N'Levelező rendszerű felnőttoktatás', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 70, NULL, 4, NULL),
            (5445, N'Felnőttoktatás nappali rendszerű (*)', N'Nappali rendszerű felnőttoktatás', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 50, NULL, 5, NULL),
            (6812, N'fejlesztő pedagógus (*)', N'fejlesztő pedagógus (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 20, NULL, 1, NULL),
            (6813, N'pedagógus (ágazatspecifikus szociális területen)', N'Pedagógus (ágazatspecifikus szociális területen)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1900, NULL, 2, NULL),
            (566, N'TOVÁBBI ápoló (*)', N'TOVÁBBI ápoló (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 500, NULL, 3, NULL),
            (639, N'családgondozó', N'Családgondozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1130, NULL, 4, NULL),
            (637, N'családsegítő', N'Családsegítő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1140, NULL, 5, NULL),
            (560, N'dajka', N'dajka', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1165, NULL, 6, NULL),
            (574, N'fűtő', N'Fűtő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1340, NULL, 7, NULL),
            (569, N'TOVÁBBI gazdasági dolgozó (*)', N'TOVÁBBI gazdasági dolgozó (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 800, NULL, 8, NULL),
            (600, N'Gondnok', N'Gondnok', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1360, NULL, 9, NULL),
            (636, N'gondozó', N'Gondozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1370, NULL, 10, NULL),
            (561, N'TOVÁBBI gondozónő és takarító (*)', N'TOVÁBBI gondozónő és takarító (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 520, NULL, 11, NULL),
            (558, N'gyermek- és ifjúságvédelmi felügyelő', N'Gyermek- és ifjúságvédelmi felügyelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1400, NULL, 12, NULL),
            (563, N'TOVÁBBI gyermek- és ifjúságvédelmi felügyelő (*)', N'TOVÁBBI gyermek- és ifjúságvédelmi felügyelő (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 530, NULL, 13, NULL),
            (635, N'gyermekfelügyelő', N'Gyermekfelügyelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1410, NULL, 14, NULL),
            (638, N'gyermekvédelmi asszisztens', N'Gyermekvédelmi asszisztens', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1450, NULL, 15, NULL),
            (642, N'gyermekvédelmi ügyintéző', N'Gyermekvédelmi ügyintéző', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1480, NULL, 16, NULL),
            (559, N'TOVÁBBI gyógypedagógiai asszisztens (*)', N'TOVÁBBI gyógypedagógiai asszisztens (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 540, NULL, 17, NULL),
            (597, N'gyógypedagógus (*)', N'gyógypedagógus (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 50, NULL, 18, NULL),
            (633, N'gyógytornász', N'gyógytornász', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1497, NULL, 19, NULL),
            (567, N'hangszerkarbantartó', N'hangszerkarbantartó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1505, NULL, 20, NULL),
            (584, N'TOVÁBBI iskolatitkár, kollégiumi titkár (*)', N'TOVÁBBI iskolatitkár, kollégiumi titkár (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 595, NULL, 21, NULL),
            (573, N'karbantartó', N'Karbantartó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1600, NULL, 22, NULL),
            (572, N'TOVÁBBI kisegítő dolgozó (*)', N'TOVÁBBI kisegítő dolgozó (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 810, NULL, 23, NULL),
            (598, N'konduktor', N'konduktor', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1547, NULL, 24, NULL),
            (3026, N'TOVÁBBI könyvtáros (*)', N'TOVÁBBI könyvtáros (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 600, NULL, 25, NULL),
            (582, N'könyvtáros asszisztens', N'könyvtáros asszisztens', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1665, NULL, 26, NULL),
            (553, N'könyvtárostanár (tanító)', N'könyvtárostanár (tanító)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 140, NULL, 27, NULL),
            (581, N'könyvtáros technikus', N'könyvtáros technikus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1667, NULL, 28, NULL),
            (589, N'TOVÁBBI laboráns (*)', N'TOVÁBBI laboráns (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 610, NULL, 29, NULL),
            (555, N'logopédus', N'logopédus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1715, NULL, 30, NULL),
            (591, N'munkaügyi előadó', N'Munkaügyi előadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1760, NULL, 31, NULL),
            (571, N'TOVÁBBI műszaki dolgozó (*)', N'TOVÁBBI műszaki dolgozó (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 850, NULL, 32, NULL),
            (565, N'TOVÁBBI műszaki vezető (*)', N'TOVÁBBI műszaki vezető (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 620, NULL, 33, NULL),
            (552, N'nevelő', N'Nevelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1800, NULL, 34, NULL),
            (641, N'növendékügyi előadó', N'Növendékügyi előadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1830, NULL, 35, NULL),
            (586, N'TOVÁBBI oktatástechnikus (*)', N'TOVÁBBI oktatástechnikus (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 860, NULL, 36, NULL),
            (577, N'orvos', N'Orvos', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1850, NULL, 37, NULL),
            (595, N'óvódapedagógus', N'óvodapedagógus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1873, NULL, 38, NULL),
            (583, N'óvodatitkár', N'óvodatitkár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1877, NULL, 39, NULL),
            (585, N'TOVÁBBI pedagógiai asszisztens (*)', N'TOVÁBBI pedagógiai asszisztens (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 640, NULL, 40, NULL),
            (562, N'pedagógiai felügyelő', N'pedagógiai felügyelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1895, NULL, 41, NULL),
            (576, N'portás', N'Portás', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1920, NULL, 42, NULL),
            (578, N'pszichológus', N'pszichológus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1945, NULL, 43, NULL),
            (594, N'TOVÁBBI rendszergazda (*)', N'TOVÁBBI rendszergazda (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 670, NULL, 44, NULL),
            (564, N'TOVÁBBI szabadidő-szervező (*)', N'TOVÁBBI szabadidő-szervező (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 680, NULL, 45, NULL),
            (557, N'szakorvos', N'szakorvos', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2067, NULL, 46, NULL),
            (587, N'számítógép-kezelő', N'számítógép-kezelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2085, NULL, 47, NULL),
            (588, N'számítógép-rendszerprogramozó', N'számítógép-rendszerprogramozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2087, NULL, 48, NULL),
            (592, N'személyzeti előadó', N'Személyzeti előadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2100, NULL, 49, NULL),
            (579, N'TOVÁBBI szociális munkás (*)', N'TOVÁBBI szociális munkás (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 705, NULL, 50, NULL),
            (580, N'szociálpedagógus', N'szociálpedagógus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2155, NULL, 51, NULL),
            (599, N'takarító', N'Takarító', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2160, NULL, 52, NULL),
            (568, N'TOVÁBBI úszómester (*)', N'TOVÁBBI úszómester (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 710, NULL, 53, NULL),
            (640, N'utógondozó', N'Utógondozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2290, NULL, 54, NULL),
            (570, N'TOVÁBBI ügyviteli dolgozó (*)', N'TOVÁBBI ügyviteli dolgozó (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 890, NULL, 55, NULL),
            (590, N'EGYÉB ügyviteli gépkezelő (*)', N'EGYÉB ügyviteli gépkezelő (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 900, NULL, 56, NULL),
            (6329, N'addiktológiai konzultáns', N'addiktológiai konzultáns', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1000, NULL, 57, NULL),
            (6330, N'addiktológiai koordinátor', N'addiktológiai koordinátor', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1010, NULL, 58, NULL),
            (6331, N'adminisztrátor', N'adminisztrátor', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1020, NULL, 59, NULL),
            (6332, N'általános ápoló', N'általános ápoló', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1030, NULL, 60, NULL),
            (6333, N'általános titkár, titkárnő', N'általános titkár, titkárnő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1040, NULL, 61, NULL),
            (6334, N'anyagnyilvántartó', N'anyagnyilvántartó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1050, NULL, 62, NULL),
            (6335, N'ápolási asszisztens', N'ápolási asszisztens', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1060, NULL, 63, NULL),
            (6336, N'ápoló kizárólagosan sajátos nevelési igényű tanulókat ellátó kollégiumban', N'ápoló kizárólagosan sajátos nevelési igényű tanulókat ellátó kollégiumban', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1070, NULL, 64, NULL),
            (6337, N'ápoló kollégiumban', N'ápoló kollégiumban', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1080, NULL, 65, NULL),
            (6338, N'asszisztens', N'asszisztens', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1090, NULL, 66, NULL),
            (6339, N'bér- és társadalombiztosítási ügyintéző', N'bér- és társadalombiztosítási ügyintéző', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1100, NULL, 67, NULL),
            (6340, N'bérelszámoló', N'bérelszámoló', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1110, NULL, 68, NULL),
            (6341, N'betanított munkás', N'betanított munkás', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1120, NULL, 69, NULL),
            (6342, N'csecsemő- és gyermekápoló', N'csecsemő- és gyermekápoló', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1150, NULL, 70, NULL),
            (6343, N'csecsemő- és gyermekgyógyászat szakorvos', N'csecsemő- és gyermekgyógyászat szakorvos', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1160, NULL, 71, NULL),
            (6344, N'diétás nővér', N'diétás nővér', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1170, NULL, 72, NULL),
            (6345, N'egyéb irodai és ügyviteli munkakör', N'egyéb irodai és ügyviteli munkakör', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1180, NULL, 73, NULL),
            (6346, N'egyéb kisegítő', N'egyéb kisegítő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1190, NULL, 74, NULL),
            (6347, N'egyéb ügyintéző', N'egyéb ügyintéző', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1200, NULL, 75, NULL),
            (6348, N'együttnevelést segítő pedagógus (*)', N'együttnevelést segítő pedagógus (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1210, NULL, 76, NULL),
            (6349, N'éjjeliőr', N'éjjeliőr', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1220, NULL, 77, NULL),
            (6350, N'élelmezésvezető', N'élelmezésvezető', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1230, NULL, 78, NULL),
            (6351, N'elhelyezési ügyintéző', N'elhelyezési ügyintéző', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1240, NULL, 79, NULL),
            (6352, N'ének-zene tanár', N'ének-zene tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 15, NULL, 80, NULL),
            (6353, N'esetfelelős', N'esetfelelős', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1260, NULL, 81, NULL),
            (6354, N'esetmenedzser', N'esetmenedzser', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1270, NULL, 82, NULL),
            (6355, N'falugondnok', N'falugondnok', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1280, NULL, 83, NULL),
            (6356, N'foglalkoztatás-szervező', N'foglalkoztatás-szervező', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1300, NULL, 84, NULL),
            (6357, N'foglalkoztatás-vezető', N'foglalkoztatás-vezető', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1310, NULL, 85, NULL),
            (6358, N'foglalkoztató', N'foglalkoztató', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1320, NULL, 86, NULL),
            (6359, N'fogyatékosság típusának megfelelő szakorvos', N'fogyatékosság típusának megfelelő szakorvos', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1330, NULL, 87, NULL),
            (6360, N'gépkocsivezető', N'gépkocsivezető', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1350, NULL, 88, NULL),
            (6361, N'gimnáziumi tanár (*)', N'gimnáziumi tanár (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 30, NULL, 89, NULL),
            (6362, N'gondozó (irányító gondozó)', N'gondozó (irányító gondozó)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1380, NULL, 90, NULL),
            (6363, N'gyermek- és ifjúsági pszichiátria szakorvos', N'gyermek- és ifjúsági pszichiátria szakorvos', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1390, NULL, 91, NULL),
            (6364, N'gyermekgondozó', N'gyermekgondozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1420, NULL, 92, NULL),
            (6365, N'gyermekneurológia szakorvos', N'gyermekneurológia szakorvos', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1430, NULL, 93, NULL),
            (6366, N'gyermekotthon vezető', N'gyermekotthon vezető', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1440, NULL, 94, NULL),
            (6367, N'gyermekvédelmi gyám', N'gyermekvédelmi gyám', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1460, NULL, 95, NULL),
            (6368, N'gyermekvédelmi szakértői bizottság családgondozója', N'gyermekvédelmi szakértői bizottság családgondozója', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1470, NULL, 96, NULL),
            (6369, N'gyógypedagógiai/fejlesztőpedagógiai tanácsadó', N'gyógypedagógiai/fejlesztőpedagógiai tanácsadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1490, NULL, 97, NULL),
            (6370, N'gyógytestnevelő tanár', N'gyógytestnevelő tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1495, NULL, 98, NULL),
            (6371, N'gyors- és gépíró', N'gyors- és gépíró', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1500, NULL, 99, NULL),
            (6372, N'helyettes szülői tanácsadó', N'helyettes szülői tanácsadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1510, NULL, 100, NULL),
            (6373, N'hivatalsegéd', N'hivatalsegéd', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1520, NULL, 101, NULL),
            (6374, N'humán erőgazdálkodási ügyintéző', N'humán erőgazdálkodási ügyintéző', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1530, NULL, 102, NULL),
            (6375, N'idegennyelvtanár', N'idegennyelvtanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 90, NULL, 103, NULL),
            (6376, N'iratkezelő', N'iratkezelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1540, NULL, 104, NULL),
            (6377, N'iskola- és óvodapszichológia koordinátora', N'iskola- és óvodapszichológia koordinátora', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1545, NULL, 105, NULL),
            (6378, N'iskolai szociális munkás', N'iskolai szociális munkás', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1550, NULL, 106, NULL),
            (6379, N'javítóintézeti ápoló', N'javítóintézeti ápoló', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1560, NULL, 107, NULL),
            (6380, N'javítóintézeti rendész', N'javítóintézeti rendész', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1570, NULL, 108, NULL),
            (6381, N'javítóintézeti utógondozó', N'javítóintézeti utógondozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1580, NULL, 109, NULL),
            (6382, N'jelmez- és viselettáros', N'jelmez- és viselettáros', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1585, NULL, 110, NULL),
            (6383, N'jogász', N'jogász', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1590, NULL, 111, NULL),
            (6384, N'kazánfűtő', N'kazánfűtő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1610, NULL, 112, NULL),
            (6385, N'kézbesítő', N'kézbesítő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1620, NULL, 113, NULL),
            (6386, N'kísérő támogatást biztosító munkatárs', N'kísérő támogatást biztosító munkatárs', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1630, NULL, 114, NULL),
            (6387, N'kisgyermeknevelő', N'kisgyermeknevelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1640, NULL, 115, NULL),
            (6388, N'kollégiumi titkár', N'kollégiumi titkár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1545, NULL, 116, NULL),
            (6389, N'konyhai kisegítő', N'konyhai kisegítő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1650, NULL, 117, NULL),
            (6390, N'könyvelő', N'könyvelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1660, NULL, 118, NULL),
            (6392, N'közismereti tantárgyat nemzetiségi nyelven oktató tanár', N'közismereti tantárgyat nemzetiségi nyelven oktató tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 150, NULL, 119, NULL),
            (6393, N'közismereti tantárgyat oktató középiskolai tanár (szakgimnázium)', N'közismereti tantárgyat oktató középiskolai tanár (szakgimnázium)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 160, NULL, 120, NULL),
            (6394, N'közismereti tantárgyat oktató tanár (szakközépiskola)', N'közismereti tantárgyat oktató tanár (szakközépiskola)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 170, NULL, 121, NULL),
            (6395, N'közösségi gondozó', N'közösségi gondozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1670, NULL, 122, NULL),
            (6396, N'közösségi koordinátor', N'közösségi koordinátor', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1680, NULL, 123, NULL),
            (6397, N'közvetítő', N'közvetítő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1690, NULL, 124, NULL),
            (6398, N'leltárellenőr', N'leltárellenőr', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1700, NULL, 125, NULL),
            (6399, N'leltározó', N'leltározó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1710, NULL, 126, NULL),
            (6400, N'mentálhigiénés munkatárs', N'mentálhigiénés munkatárs', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1720, NULL, 127, NULL),
            (6401, N'módszertani munkatárs', N'módszertani munkatárs', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1730, NULL, 128, NULL),
            (6402, N'mozgásterapeuta', N'mozgásterapeuta', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1740, NULL, 129, NULL),
            (6403, N'munkaoktató', N'munkaoktató', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1750, NULL, 130, NULL),
            (6404, N'TOVÁBBI munkaügyi, személyzeti előadó (*)', N'TOVÁBBI munkaügyi, személyzeti előadó (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 840, NULL, 131, NULL),
            (6405, N'munkavezető', N'munkavezető', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1770, NULL, 132, NULL),
            (6406, N'nemzetiségi nyelvtanár', N'nemzetiségi nyelvtanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1775, NULL, 133, NULL),
            (6407, N'nemzetiségi óvodapedagógus (*)', N'nemzetiségi óvodapedagógus (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1780, NULL, 134, NULL),
            (6408, N'nemzetiségi tanító (*)', N'nemzetiségi tanító (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1790, NULL, 135, NULL),
            (6409, N'nevelőszülői hálózat szakmai vezetője', N'nevelőszülői hálózat szakmai vezetője', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1810, NULL, 136, NULL),
            (6410, N'nevelőszülői tanácsadó', N'nevelőszülői tanácsadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1820, NULL, 137, NULL),
            (6411, N'nyilvántartó pénztáros', N'nyilvántartó pénztáros', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1840, NULL, 138, NULL),
            (6412, N'orvos konzultáns', N'orvos konzultáns', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1860, NULL, 139, NULL),
            (6413, N'otthonvezető javítóintézetben', N'otthonvezető javítóintézetben', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1870, NULL, 140, NULL),
            (6414, N'iskolapszichológus (*)', N'Iskolapszichológus (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 110, NULL, 141, NULL),
            (6415, N'őr/telepőr', N'őr/telepőr', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1880, NULL, 142, NULL),
            (6416, N'örökbefogadási tanácsadó', N'örökbefogadási tanácsadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1890, NULL, 143, NULL),
            (6418, N'pénzügyi ügyintéző', N'pénzügyi ügyintéző', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1910, NULL, 144, NULL),
            (6419, N'pszichiáter', N'pszichiáter', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1930, NULL, 145, NULL),
            (6420, N'pszichológiai tanácsadó', N'pszichológiai tanácsadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1940, NULL, 146, NULL),
            (6421, N'TOVÁBBI pszichopedagógus (*)', N'TOVÁBBI pszichopedagógus (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 660, NULL, 147, NULL),
            (6422, N'raktárkezelő', N'raktárkezelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1950, NULL, 148, NULL),
            (6423, N'raktáros', N'raktáros', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1960, NULL, 149, NULL),
            (6424, N'rendész', N'rendész', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1970, NULL, 150, NULL),
            (6425, N'segédápoló', N'segédápoló', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1980, NULL, 151, NULL),
            (6426, N'segédgondozó', N'segédgondozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1990, NULL, 152, NULL),
            (6427, N'segédmunkás', N'segédmunkás', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2000, NULL, 153, NULL),
            (6428, N'statisztikai ügyintéző', N'statisztikai ügyintéző', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2010, NULL, 154, NULL),
            (6429, N'szakápoló', N'szakápoló', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2020, NULL, 155, NULL),
            (6430, N'szakasszisztens', N'szakasszisztens', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2030, NULL, 156, NULL),
            (6431, N'szakgondozó', N'szakgondozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2040, NULL, 157, NULL),
            (6432, N'szakmai elméleti tantárgyat oktató tanár', N'szakmai elméleti tantárgyat oktató tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 230, NULL, 158, NULL),
            (6433, N'szakmai tanár, szakoktató, gyakorlati oktató', N'szakmai tanár, szakoktató, gyakorlati oktató', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 405, NULL, 159, NULL),
            (6434, N'szakmai tantárgyat tanító tanár', N'szakmai tantárgyat tanító tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2045, NULL, 160, NULL),
            (6435, N'szakmunkás', N'szakmunkás', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2060, NULL, 161, NULL),
            (6436, N'szakoktató', N'szakoktató', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2065, NULL, 162, NULL),
            (6437, N'szakorvos kizárólag sajátos nevelési igényű gyermekeket nevelő óvodában', N'szakorvos kizárólag sajátos nevelési igényű gyermekeket nevelő óvodában', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2070, NULL, 163, NULL),
            (6438, N'szakszolgálati titkár', N'szakszolgálati titkár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2075, NULL, 164, NULL),
            (6439, N'szaktanácsadó', N'szaktanácsadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2080, NULL, 165, NULL),
            (6440, N'számviteli ügyintéző', N'számviteli ügyintéző', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2090, NULL, 166, NULL),
            (6441, N'szociális gondozó', N'szociális gondozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2110, NULL, 167, NULL),
            (6442, N'szociális munkatárs/szakértő', N'szociális munkatárs/szakértő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2130, NULL, 168, NULL),
            (6443, N'szociális segítő', N'szociális segítő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2140, NULL, 169, NULL),
            (6445, N'szociális ügyintéző', N'szociális ügyintéző', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2150, NULL, 170, NULL),
            (6446, N'tanácsadó', N'tanácsadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2170, NULL, 171, NULL),
            (6447, N'tanácsadó pedagógus', N'tanácsadó pedagógus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2175, NULL, 172, NULL),
            (6448, N'tanár', N'tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2180, NULL, 173, NULL),
            (6449, N'tanár a szakmai elméleti oktatásban', N'tanár a szakmai elméleti oktatásban', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2190, NULL, 174, NULL),
            (6450, N'tanító, tanár', N'tanító, tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2200, NULL, 175, NULL),
            (6451, N'tanyagondnok', N'tanyagondnok', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2210, NULL, 176, NULL),
            (6452, N'technikus', N'technikus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2220, NULL, 177, NULL),
            (6453, N'tehetséggondozó koordinátor', N'tehetséggondozó koordinátor', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2225, NULL, 178, NULL),
            (6454, N'tejkonyhavezető', N'tejkonyhavezető', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2230, NULL, 179, NULL),
            (6455, N'telefon-, telex- és telefaxkezelő', N'telefon-, telex- és telefaxkezelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2240, NULL, 180, NULL),
            (6456, N'terápiás munkatárs', N'terápiás munkatárs', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2250, NULL, 181, NULL),
            (6457, N'testnevelő (*)', N'testnevelő (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 415, NULL, 182, NULL),
            (6458, N'utcai szociális munkás', N'utcai szociális munkás', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2270, NULL, 183, NULL),
            (6459, N'utcai szolgálat koordinátor', N'utcai szolgálat koordinátor', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2280, NULL, 184, NULL),
            (6460, N'fejlesztőpedagógus', N'Fejlesztőpedagógus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1290, NULL, 185, NULL),
            (6461, N'kollégiumi nevelőtanár (*)', N'kollégiumi nevelőtanár (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 120, NULL, 186, NULL),
            (6709, N'duális képzőhely (*)', N'duális képzőhely (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 180, NULL, 187, NULL),
            (6710, N'hittanoktató', N'hittanoktató', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1515, NULL, 188, NULL),
            (6725, N'tanító', N'tanító', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2195, NULL, 189, NULL),
            (6914, N'általános iskolai tanár (*)', N'általános iskolai tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 10, NULL, 190, NULL),
            (6915, N'óvodapszichológus', N'óvodapszichológus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1875, NULL, 191, NULL),
            (6968, N'szakmai tanár', N'szakmai tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 240, NULL, 192, NULL),
            (6969, N'gyakorlati oktató', N'gyakorlati oktató', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 40, NULL, 193, NULL),
            (7356, N'NOKS pedagógiai felügyelő (*)', N'pedagógiai felügyelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 650, NULL, 194, NULL),
            (7772, N'könyvtárostanár', N'könyvtárostanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 142, NULL, 195, NULL),
            (7773, N'könyvtárostanító', N'könyvtárostanító', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 144, NULL, 196, NULL),
            (7774, N'utazó gyógypedagógus', N'utazó gyógypedagógus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 310, NULL, 197, NULL),
            (7775, N'utazó konduktor', N'utazó konduktor', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 320, NULL, 198, NULL),
            (7783, N'lelkész', N'lelkész', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 1695, NULL, 199, NULL),
            (8349, N'idegennyelv-tantágykategóriát oktató oktató (*)', N'idegennyelv-tantágykategóriát oktató oktató', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 92, NULL, 200, NULL),
            (8350, N'könyvtáros oktató (kizárólag könyvtárosi feladatokat lát el) (*)', N'könyvtáros oktató (kizárólag könyvtárosi feladatokat lát el)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 136, NULL, 201, NULL),
            (8351, N'könyvtáros oktató (oktatói feladatokat is ellát) (*)', N'könyvtáros oktató (oktatói feladatokat is ellát)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 138, NULL, 202, NULL),
            (8352, N'közismereti tantárgyat oktató középiskolai oktató (*)', N'közismereti tantárgyat oktató középiskolai oktató', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 158, NULL, 203, NULL),
            (8353, N'közismereti tantárgyat oktató oktató (*)', N'közismereti tantárgyat oktató oktató', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 168, NULL, 204, NULL),
            (8354, N'szakmai oktató (felsőfokú végzettséggel) (*)', N'szakmai oktató (felsőfokú végzettséggel)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 232, NULL, 205, NULL),
            (8355, N'szakmai oktató (középfokú végzettséggel) (*)', N'szakmai oktató (középfokú végzettséggel)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 234, NULL, 206, NULL),
            (8736, N'közismereti tantárgyat oktató középiskola', N'közismereti tantárgyat oktató középiskola', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2301, NULL, 207, NULL),
            (8737, N'közismereti tantárgyat oktató szakgimnáziumi tanár', N'közismereti tantárgyat oktató szakgimnáziumi tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2302, NULL, 208, NULL),
            (8738, N'özismereti tantárgyat oktató tanár', N'özismereti tantárgyat oktató tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2303, NULL, 209, NULL),
            (8739, N'művészeti szakmai tantárgyat oktató tanár', N'művészeti szakmai tantárgyat oktató tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2304, NULL, 210, NULL),
            (8740, N'szakmai tantárgyat oktató tanár', N'szakmai tantárgyat oktató tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2305, NULL, 211, NULL),
            (644, N'közalkalmazotti jogviszony (határozatlan időre)', N'közalkalmazotti jogviszony (határozatlan időre)', NULL, NULL, NULL, N'MunkaviszonyTipus', 13, N'T', N'F', 2, NULL, 1, NULL),
            (645, N'közalkalmazotti jogviszony (határozott időre)', N'közalkalmazotti jogviszony (határozott időre)', NULL, NULL, NULL, N'MunkaviszonyTipus', 13, N'T', N'F', 3, NULL, 2, NULL),
            (651, N'munkaviszony (határozott időre)', N'munkaviszony (határozott időre)', NULL, NULL, NULL, N'MunkaviszonyTipus', 13, N'T', N'T', 5, NULL, 3, NULL),
            (652, N'munkaviszony (határozatlan időre)', N'munkaviszony (határozatlan időre)', NULL, NULL, NULL, N'MunkaviszonyTipus', 13, N'T', N'T', 4, NULL, 4, NULL),
            (6762, N'egyházi szolgálati jogviszony', N'egyházi szolgálati jogviszony', NULL, NULL, NULL, N'MunkaviszonyTipus', 13, N'T', N'T', 8, NULL, 5, NULL)
    END
    -- A 'NSZFH' sajatos elemek vege
    
    -- A 'HOI' sajatos elemek kezdete
    IF (@EnvironmentName IS NOT NULL AND @EnvironmentName = 'HOI')
    BEGIN 
   
    INSERT INTO #DICTIONARYITEMBASE (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
        VALUES
            (9106, N'tanszakvezető és osztályfőnök', N'tanszakvezető és osztályfőnök', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'F', 15, NULL, 1, NULL),
            (9107, N'tanszakvezető és diákönkormányzati munkát segítő', N'tanszakvezető és diákönkormányzati munkát segítő', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'F', 16, NULL, 2, NULL),
            (9108, N'tanszakvezető, osztályfőnök, diákönkormányzati munkát segítő és munkaközösség vezető', N'tanszakvezető, osztályfőnök, diákönkormányzati munkát segítő és munkaközösség vezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'F', 17, NULL, 3, NULL),
            (9109, N'tanszakvezető, osztályfőnök és diákönkormányzati munkát segítő', N'tanszakvezető, osztályfőnök és diákönkormányzati munkát segítő', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'F', 18, NULL, 4, NULL),
            (9110, N'tanszakvezető, osztályfőnök és munkaközösségvezető', N'tanszakvezető, osztályfőnök és munkaközösségvezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'F', 19, NULL, 5, NULL),
            (9111, N'tanszakvezető, diákönkormányzati munkát segítő és munkaközösségvezető', N'tanszakvezető, diákönkormányzati munkát segítő és munkaközösségvezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'F', 20, NULL, 6, NULL),
            (1311, N'9', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2310, NULL, 1, NULL),
            (1312, N'10', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2410, NULL, 2, NULL),
            (1313, N'11', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2510, NULL, 3, NULL),
            (1314, N'12', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2610, NULL, 4, NULL),
            (1332, N'9/Ny', N'9/Ny', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3040, NULL, 5, NULL),
            (1316, N'1/9', N'1/9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4140, NULL, 6, NULL),
            (1323, N'2/10', N'2/10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4150, NULL, 7, NULL),
            (1325, N'3/11', N'3/11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4160, NULL, 8, NULL),
            (6469, N'5/13', N'5/13.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4190, NULL, 9, NULL),
            (1319, N'1/13', N'1/13.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4320, NULL, 10, NULL),
            (1320, N'2/14', N'2/14.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4340, NULL, 11, NULL),
            (1326, N'1/12', N'1/12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4420, NULL, 12, NULL),
            (1337, N'1/15', N'1/15.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4440, NULL, 13, NULL),
            (3016, N'Ksz/11', N'Ksz/11', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4520, NULL, 14, NULL),
            (3017, N'Ksz/12', N'Ksz/12', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4540, NULL, 15, NULL),
            (6818, N'Kk 12', N'Kk 12', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4710, NULL, 16, NULL),
            (6819, N'Kk 13', N'Kk 13', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4720, NULL, 17, NULL),
            (1160, N'Gimnázium', N'Gimnázium', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 30, NULL, 1, NULL),
            (1161, N'Szakközépiskola', N'Szakközépiskola', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 90, NULL, 2, NULL),
            (1162, N'Szakiskola', N'Szakiskola', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 100, NULL, 3, NULL),
            (5435, N'Szakgimnázium', N'Szakgimnázium', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 40, NULL, 4, NULL),
            (1165, N'Kollégium', N'Kollégium', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 130, NULL, 5, NULL),
            (6463, N'Felnőttoktatás', N'Felnőttoktatás', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 120, NULL, 6, NULL),
            (1096, N'Nappali rendszerű iskolai oktatás (*)', N'Nappali rendszerű iskolai oktatás', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 10, NULL, 1, NULL),
            (5446, N'Felnőttoktatás esti rendszerű (*)', N'Esti rendszerű felnőttoktatás', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 60, NULL, 2, NULL),
            (5447, N'Felnőttoktatás levelező rendszerű (*)', N'Levelező rendszerű felnőttoktatás', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 70, NULL, 3, NULL),
            (5445, N'Felnőttoktatás nappali rendszerű (*)', N'Nappali rendszerű felnőttoktatás', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 50, NULL, 4, NULL),
            (5448, N'Speciális szakképzés (*)', N'Speciális szakképzés', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 210, NULL, 5, NULL),
            (8736, N'közismereti tantárgyat oktató középiskola', N'közismereti tantárgyat oktató középiskola', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2301, NULL, 1, NULL),
            (8737, N'közismereti tantárgyat oktató szakgimnáziumi tanár', N'közismereti tantárgyat oktató szakgimnáziumi tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2302, NULL, 2, NULL),
            (8738, N'özismereti tantárgyat oktató tanár', N'özismereti tantárgyat oktató tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2303, NULL, 3, NULL),
            (8739, N'művészeti szakmai tantárgyat oktató tanár', N'művészeti szakmai tantárgyat oktató tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2304, NULL, 4, NULL),
            (8740, N'szakmai tantárgyat oktató tanár', N'szakmai tantárgyat oktató tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2305, NULL, 5, NULL)
    END
    -- A 'HOI' sajatos elemek vege
        
    -- A 'AZURE' sajatos elemek kezdete
    IF (@EnvironmentName IS NOT NULL AND @EnvironmentName = 'AZURE')
    BEGIN 
   
    INSERT INTO #DICTIONARYITEMBASE (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
        VALUES
            (8408, N'Ksz/13', N'Ksz/13', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4545, NULL, 1, NULL),
            (8409, N'Ksz/14', N'Ksz/14', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4550, NULL, 2, NULL),
            (8736, N'közismereti tantárgyat oktató középiskola', N'közismereti tantárgyat oktató középiskola', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2301, NULL, 1, NULL),
            (8737, N'közismereti tantárgyat oktató szakgimnáziumi tanár', N'közismereti tantárgyat oktató szakgimnáziumi tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2302, NULL, 2, NULL),
            (8738, N'özismereti tantárgyat oktató tanár', N'özismereti tantárgyat oktató tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2303, NULL, 3, NULL),
            (8739, N'művészeti szakmai tantárgyat oktató tanár', N'művészeti szakmai tantárgyat oktató tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2304, NULL, 4, NULL),
            (8740, N'szakmai tantárgyat oktató tanár', N'szakmai tantárgyat oktató tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 2305, NULL, 5, NULL)
    END
    -- A 'AZURE' sajatos elemek vege
    
    -- A 'Default' elemek kezdete
    INSERT INTO #DICTIONARYITEMBASE (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
        SELECT * FROM ( VALUES
            (9080, N'Öregségi teljes nyugdíj', N'Öregségi teljes nyugdíj', NULL, NULL, NULL, N'NyugdijTipus', 186, N'F', N'T', 1, NULL, 1, NULL),
            (9081, N'Öregségi résznyugdíj', N'Öregségi résznyugdíj', NULL, NULL, NULL, N'NyugdijTipus', 186, N'F', N'T', 2, NULL, 2, NULL),
            (9082, N'Nők kedvezményes öregségi nyugdíja', N'Nők kedvezményes öregségi nyugdíja', NULL, NULL, NULL, N'NyugdijTipus', 186, N'F', N'T', 3, NULL, 3, NULL),
            (9083, N'Hozzátartozói nyugellátás (özvegyi nyugdíj, özvegyi járadék, árvaellátás vagy szülői nyugdíj)', N'Hozzátartozói nyugellátás (özvegyi nyugdíj, özvegyi járadék, árvaellátás vagy szülői nyugdíj)', NULL, NULL, NULL, N'NyugdijTipus', 186, N'F', N'T', 4, NULL, 4, NULL),
            (9072, N'Na', N'Na', NULL, NULL, NULL, N'DualisHelyszinTipus', 185, N'T', N'T', 1, NULL, 1, NULL),
            (9073, N'Székhely', N'Székhely', NULL, NULL, NULL, N'DualisHelyszinTipus', 185, N'T', N'T', 2, NULL, 2, NULL),
            (9074, N'Telephely', N'Telephely', NULL, NULL, NULL, N'DualisHelyszinTipus', 185, N'T', N'T', 3, NULL, 3, NULL),
            (9075, N'Egyéb foglalkoztató', N'Egyéb foglalkoztató', NULL, NULL, NULL, N'DualisHelyszinTipus', 185, N'T', N'T', 4, NULL, 4, NULL),
            (8996, N'Teljesítette', N'Teljesítette', NULL, NULL, NULL, N'AgazatAlapVizsgaEredTipus', 181, N'F', N'F', 1, NULL, 1, NULL),
            (8997, N'Nem teljesítette', N'Nem teljesítette', NULL, NULL, NULL, N'AgazatAlapVizsgaEredTipus', 181, N'F', N'F', 2, NULL, 2, NULL),
            (8998, N'Mentesítve, előtanulmányok beszámításával', N'Mentesítve, előtanulmányok beszámításával', NULL, NULL, NULL, N'AgazatAlapVizsgaEredTipus', 181, N'F', N'F', 3, NULL, 3, NULL),
            (9056, N'Felvétellel', N'Felvétellel', NULL, NULL, NULL, N'JogvKeletkezeseJogcimTipus', 184, N'T', N'T', 1, NULL, 1, NULL),
            (9057, N'Átvétellel', N'Átvétellel', NULL, NULL, NULL, N'JogvKeletkezeseJogcimTipus', 184, N'T', N'T', 2, NULL, 2, NULL),
            (8973, N'Műszaki ágazati alapoktatás', N'Műszaki ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 1, NULL, 1, NULL),
            (8974, N'Egészségügyi technika ágazati alapoktatás', N'Egészségügyi technika ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 2, NULL, 2, NULL),
            (8975, N'Egészségügy ágazati alapoktatás', N'Egészségügy ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 3, NULL, 3, NULL),
            (8976, N'Élelmiszeripar ágazati alapoktatás', N'Élelmiszeripar ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 4, NULL, 4, NULL),
            (8977, N'Építőipar ágazati alapoktatás', N'Építőipar ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 5, NULL, 5, NULL),
            (8978, N'Épületgépészet ágazati alapoktatás', N'Épületgépészet ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 6, NULL, 6, NULL),
            (8979, N'Fa- és bútoripar ágazati alapoktatás', N'Fa- és bútoripar ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 7, NULL, 7, NULL),
            (8980, N'Gazdálkodás és menedzsment ágazati alapoktatás', N'Gazdálkodás és menedzsment ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 8, NULL, 8, NULL),
            (8981, N'Honvédelem ágazati alapoktatás', N'Honvédelem ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 9, NULL, 9, NULL),
            (8982, N'Informatika és távközlés ágazati alapoktatás', N'Informatika és távközlés ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 10, NULL, 10, NULL),
            (8983, N'Kereskedelem ágazati alapoktatás', N'Kereskedelem ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 11, NULL, 11, NULL),
            (8984, N'Környezetvédelem és vízügy ágazati alapoktatás', N'Környezetvédelem és vízügy ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 12, NULL, 12, NULL),
            (8985, N'Kreatív ipari ágazati alapoktatás', N'Kreatív ipari ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 13, NULL, 13, NULL),
            (8986, N'Vizuális ágazati alapoktatás', N'Vizuális ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 14, NULL, 14, NULL),
            (8987, N'Színház és hangtechnika ágazati alapoktatás', N'Színház és hangtechnika ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 15, NULL, 15, NULL),
            (8988, N'Mezőgazdaság és erdészet ágazati alapoktatás', N'Mezőgazdaság és erdészet ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 16, NULL, 16, NULL),
            (8989, N'Rendészet és közszolgálat alapoktatás', N'Rendészet és közszolgálat alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 17, NULL, 17, NULL),
            (8990, N'Rendészet és közszolgálat (rendőri) alapoktatás', N'Rendészet és közszolgálat (rendőri) alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 18, NULL, 18, NULL),
            (8991, N'Sport ágazati alapoktatás', N'Sport ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 19, NULL, 19, NULL),
            (8992, N'Szépészet ágazati alapoktatás', N'Szépészet ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 20, NULL, 20, NULL),
            (8993, N'Szociális ágazati alapoktatás', N'Szociális ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 21, NULL, 21, NULL),
            (8994, N'Turizmus-vendéglátás ágazati alapoktatás', N'Turizmus-vendéglátás ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 22, NULL, 22, NULL),
            (8995, N'Vegyipar ágazati alapoktatás', N'Vegyipar ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 23, NULL, 23, NULL),
            (9058, N'Oktatás ágazati alapoktatás', N'Oktatás ágazati alapoktatás', NULL, NULL, NULL, N'AgazatAlapOktMegnevTipus', 180, N'F', N'F', 24, NULL, 24, NULL),
            (9050, N'Tanulói jogviszony', N'Tanulói jogviszony', NULL, NULL, NULL, N'TbJogviszonyTipus', 183, N'T', N'T', 1, NULL, 1, NULL),
            (9051, N'Felnőttképzési jogviszony', N'Felnőttképzési jogviszony', NULL, NULL, NULL, N'TbJogviszonyTipus', 183, N'T', N'T', 2, NULL, 2, NULL),
            (9052, N'Óvodai nevelési jogviszony', N'Óvodai nevelési jogviszony', NULL, NULL, NULL, N'TbJogviszonyTipus', 183, N'T', N'T', 3, NULL, 3, NULL),
            (9053, N'Kollégium tagsági viszony', N'Kollégium tagsági viszony', NULL, NULL, NULL, N'TbJogviszonyTipus', 183, N'T', N'T', 4, NULL, 4, NULL),
            (9054, N'Vendégtanulói jogviszony', N'Vendégtanulói jogviszony', NULL, NULL, NULL, N'TbJogviszonyTipus', 183, N'T', N'T', 5, NULL, 5, NULL),
            (9055, N'Tanulói jogviszony (Alapfokú művészeti iskola)', N'Tanulói jogviszony (Alapfokú művészeti iskola)', NULL, NULL, NULL, N'TbJogviszonyTipus', 183, N'T', N'T', 6, NULL, 6, NULL),
            (9002, N'a felmentést engedélyező szerv a szülő kérelmére engedélyt adott a gyermek óvodából történő kimaradására', N'a felmentést engedélyező szerv a szülő kérelmére engedélyt adott a gyermek óvodából történő kimaradására', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 1, NULL, 1, NULL),
            (9003, N'a gimnáziumi tanulmányok esetén az utolsó évfolyamát elvégezte', N'a gimnáziumi tanulmányok esetén az utolsó évfolyamát elvégezte', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 2, NULL, 2, NULL),
            (9004, N'a gyermeket másik óvoda átvette', N'a gyermeket másik óvoda átvette', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 3, NULL, 3, NULL),
            (9005, N'a tankötelezettség megszűnése után – ha a szülő a tanuló egyetértésével, nagykorú tanuló esetén a tanuló írásban bejelenti, hogy nem folytatja tanulmányait az iskolában', N'a tankötelezettség megszűnése után – ha a szülő a tanuló egyetértésével, nagykorú tanuló esetén a tanuló írásban bejelenti, hogy nem folytatja tanulmányait az iskolában', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 4, NULL, 4, NULL),
            (9006, N'a tankötelezettség megszűnése után – kiskorú tanuló a törvényes képviselője egyetértésével – írásban bejelenti, hogy kimarad', N'a tankötelezettség megszűnése után – kiskorú tanuló a törvényes képviselője egyetértésével – írásban bejelenti, hogy kimarad', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 5, NULL, 5, NULL),
            (9007, N'a tanuló – a tanköteles tanuló kivételével – a szakképző intézmény kötelező foglalkozásairól a jogszabályban meghatározott időnél igazolatlanul többet mulasztott', N'a tanuló – a tanköteles tanuló kivételével – a szakképző intézmény kötelező foglalkozásairól a jogszabályban meghatározott időnél igazolatlanul többet mulasztott', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 6, NULL, 6, NULL),
            (9008, N'a tanuló – a tanköteles tanuló kivételével – az iskola kötelező foglalkozásairól a jogszabályban meghatározott időnél igazolatlanul többet mulasztott', N'a tanuló – a tanköteles tanuló kivételével – az iskola kötelező foglalkozásairól a jogszabályban meghatározott időnél igazolatlanul többet mulasztott', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 7, NULL, 7, NULL),
            (9009, N'tanuló jogviszony – a tanköteles és a hátrányos helyzetű tanuló kivételével – fizetési hátralék miatt, tanuló eredménytelen felszólítása és a tanuló szociális helyzetének vizsgálata után megszűnik', N'tanuló jogviszony – a tanköteles és a hátrányos helyzetű tanuló kivételével – fizetési hátralék miatt, tanuló eredménytelen felszólítása és a tanuló szociális helyzetének vizsgálata után megszűnik', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 8, NULL, 8, NULL),
            (9010, N'tanuló jogviszony – a tanköteles és a hátrányos helyzetű tanuló kivételével – fizetési hátralék miatt, kiskorú tanuló esetén a törvényes képviselőjének eredménytelen felszólítása és a tanuló szociális helyzetének vizsgálata után megszűnik', N'tanuló jogviszony – a tanköteles és a hátrányos helyzetű tanuló kivételével – fizetési hátralék miatt, kiskorú tanuló esetén a törvényes képviselőjének eredménytelen felszólítása és a tanuló szociális helyzetének vizsgálata után megszűnik', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 9, NULL, 9, NULL),
            (9011, N'a tanuló kizárása az iskolából fegyelmi határozattal', N'a tanuló kizárása az iskolából fegyelmi határozattal', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 10, NULL, 10, NULL),
            (9012, N'a tanuló, kiskorú tanuló esetén a szülő a tanuló kollégiumi tagságáról írásban lemond', N'a tanuló, kiskorú tanuló esetén a szülő a tanuló kollégiumi tagságáról írásban lemond', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 11, NULL, 11, NULL),
            (9013, N'a tanulót másik iskola átvette', N'a tanulót másik iskola átvette', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 12, NULL, 12, NULL),
            (9014, N'a tanulót másik kollégium átvette', N'a tanulót másik kollégium átvette', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 13, NULL, 13, NULL),
            (9015, N'a tanulóval szemben kizárás a szakképző intézményből fegyelmi határozatot hoztak', N'a tanulóval szemben kizárás a szakképző intézményből fegyelmi határozatot hoztak', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 14, NULL, 14, NULL),
            (9016, N'alapfokú művészeti iskolában az utolsó alapfokú évfolyamát elvégezte, nem tesz művészeti alapvizsgát', N'alapfokú művészeti iskolában az utolsó alapfokú évfolyamát elvégezte, nem tesz művészeti alapvizsgát', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 15, NULL, 15, NULL),
            (9017, N'alapfokú művészeti iskolában az utolsó továbbképző évfolyamát elvégezte', N'alapfokú művészeti iskolában az utolsó továbbképző évfolyamát elvégezte', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 16, NULL, 16, NULL),
            (9018, N'alapfokú művészeti iskolában kiskorú tanuló esetén a szülő vagy a nagykorú tanuló írásban bejelenti, hogy nem folytatja tanulmányait az iskolában', N'alapfokú művészeti iskolában kiskorú tanuló esetén a szülő vagy a nagykorú tanuló írásban bejelenti, hogy nem folytatja tanulmányait az iskolában', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 17, NULL, 17, NULL),
            (9019, N'az általános iskola utolsó évfolyamát elvégezte', N'az általános iskola utolsó évfolyamát elvégezte', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 18, NULL, 18, NULL),
            (9020, N'az iskola egyoldalú nyilatkozattal megszünteti nem tanköteles tanulónak a jogviszonyát, ugyanannak az évfolyamnak a tanulmányi követelményeit második alkalommal nem teljesítette', N'az iskola egyoldalú nyilatkozattal megszünteti nem tanköteles tanulónak a jogviszonyát, ugyanannak az évfolyamnak a tanulmányi követelményeit második alkalommal nem teljesítette', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 19, NULL, 19, NULL),
            (9021, N'az iskola utolsó évfolyamának elvégzése', N'az iskola utolsó évfolyamának elvégzése', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 20, NULL, 20, NULL),
            (9022, N'az óvodába járási kötelezettségét külföldön teljesítő gyermek eléri a tanköteles kort', N'az óvodába járási kötelezettségét külföldön teljesítő gyermek eléri a tanköteles kort', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 21, NULL, 21, NULL),
            (9023, N'az utolsó évfolyamot elvégezte', N'az utolsó évfolyamot elvégezte', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 22, NULL, 22, NULL),
            (9024, N'gyermeket felvették az iskolába', N'gyermeket felvették az iskolába', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 23, NULL, 23, NULL),
            (9025, N'készségfejlesztő iskolában, az utolsó gyakorlati évfolyamát elvégezte', N'készségfejlesztő iskolában, az utolsó gyakorlati évfolyamát elvégezte', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 24, NULL, 24, NULL),
            (9026, N'készségfejlesztő iskolában, tanulmányainak folytatására egészségileg alkalmatlanná vált és az iskolában nem folyik másik megfelelő képzés', N'készségfejlesztő iskolában, tanulmányainak folytatására egészségileg alkalmatlanná vált és az iskolában nem folyik másik megfelelő képzés', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 25, NULL, 25, NULL),
            (9027, N'készségfejlesztő iskolában, tanulmányainak folytatására egészségileg alkalmatlanná vált és a tanuló nem kíván továbbtanulni', N'készségfejlesztő iskolában, tanulmányainak folytatására egészségileg alkalmatlanná vált és a tanuló nem kíván továbbtanulni', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 26, NULL, 26, NULL),
            (9028, N'készségfejlesztő iskolában, tanulmányainak folytatására egészségileg alkalmatlanná vált és a továbbtanuláshoz szükséges feltételek hiányában nem tanulhat tovább', N'készségfejlesztő iskolában, tanulmányainak folytatására egészségileg alkalmatlanná vált és a továbbtanuláshoz szükséges feltételek hiányában nem tanulhat tovább', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 27, NULL, 27, NULL),
            (9029, N'kollégiumi tagsága megszűnik a tanuló a kollégiumból kizáró fegyelmi határozattal', N'kollégiumi tagsága megszűnik a tanuló a kollégiumból kizáró fegyelmi határozattal', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 28, NULL, 28, NULL),
            (9030, N'kollégiumi tagsága megszűnik a tanulói jogviszony megszűnésével', N'kollégiumi tagsága megszűnik a tanulói jogviszony megszűnésével', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 29, NULL, 29, NULL),
            (9031, N'kollégiumi tagsága fizetési hátralék miatt; a szülő, nagykorú tanuló esetében a tanuló eredménytelen felszólítása és a tanuló szociális helyzetének vizsgálata után megszűnik', N'kollégiumi tagsága fizetési hátralék miatt; a szülő, nagykorú tanuló esetében a tanuló eredménytelen felszólítása és a tanuló szociális helyzetének vizsgálata után megszűnik', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 30, NULL, 30, NULL),
            (9032, N'másik szakképző intézmény vagy köznevelési intézmény átvette', N'másik szakképző intézmény vagy köznevelési intézmény átvette', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 31, NULL, 31, NULL),
            (9033, N'nem magyar állampolgárságú gyerek, tanuló külföldre vagy saját országába távozik', N'nem magyar állampolgárságú gyerek, tanuló külföldre vagy saját országába távozik', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 32, NULL, 32, NULL),
            (9034, N'nem magyar állampolgárságú tanuló külföldre távozik', N'nem magyar állampolgárságú tanuló külföldre távozik', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 33, NULL, 33, NULL),
            (9035, N'szakgimnáziumi tanulmányok esetén a szakképzésben nem kíván továbbtanulni, vagy a továbbtanuláshoz szükséges feltételek hiányában nem tanulhat tovább', N'szakgimnáziumi tanulmányok esetén a szakképzésben nem kíván továbbtanulni, vagy a továbbtanuláshoz szükséges feltételek hiányában nem tanulhat tovább', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 34, NULL, 34, NULL),
            (9036, N'szakgimnáziumi tanulmányok esetén államilag elismert szakképesítést szerzett tanuló döntése, hogy nem tanul tovább az érettségi vizsgára felkészítő 12. és 13. évfolyamon', N'szakgimnáziumi tanulmányok esetén államilag elismert szakképesítést szerzett tanuló döntése, hogy nem tanul tovább az érettségi vizsgára felkészítő 12. és 13. évfolyamon', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 35, NULL, 35, NULL),
            (9037, N'szakgimnáziumi tanulmányok esetén államilag elismert szakképesítést szerzett tanuló döntése, hogy továbbtanul az érettségi vizsgára felkészítő 12. és 13. évfolyamon', N'szakgimnáziumi tanulmányok esetén államilag elismert szakképesítést szerzett tanuló döntése, hogy továbbtanul az érettségi vizsgára felkészítő 12. és 13. évfolyamon', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 36, NULL, 36, NULL),
            (9038, N'szakgimnáziumi tanulmányok esetén az utolsó évfolyamát elvégezte', N'szakgimnáziumi tanulmányok esetén az utolsó évfolyamát elvégezte', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 37, NULL, 37, NULL),
            (9039, N'szakiskolai tanulmányok esetén az utolsó évfolyamát elvégezte, a tanuló jelentkezik szakmai vizsgára', N'szakiskolai tanulmányok esetén az utolsó évfolyamát elvégezte, a tanuló jelentkezik szakmai vizsgára', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 38, NULL, 38, NULL),
            (9040, N'szakiskolai tanulmányok esetén az utolsó évfolyamát elvégezte, a tanuló nem jelentkezik szakmai vizsgára vagy képesítő vizsgára', N'szakiskolai tanulmányok esetén az utolsó évfolyamát elvégezte, a tanuló nem jelentkezik szakmai vizsgára vagy képesítő vizsgára', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 39, NULL, 39, NULL),
            (9041, N'szakiskolai tanulmányok esetén az utolsó évfolyamát elvégezte, jelentkezik képesítő vizsgára', N'szakiskolai tanulmányok esetén az utolsó évfolyamát elvégezte, jelentkezik képesítő vizsgára', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 40, NULL, 40, NULL),
            (9042, N'szakiskolai tanulmányok esetén tanulmányainak folytatására egészségileg alkalmatlanná vált és az iskolában nem folyik másik megfelelő szakképzés', N'szakiskolai tanulmányok esetén tanulmányainak folytatására egészségileg alkalmatlanná vált és az iskolában nem folyik másik megfelelő szakképzés', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 41, NULL, 41, NULL),
            (9043, N'szakiskolai tanulmányok esetén tanulmányainak folytatására egészségileg alkalmatlanná vált és a tanuló nem kíván továbbtanulni', N'szakiskolai tanulmányok esetén tanulmányainak folytatására egészségileg alkalmatlanná vált és a tanuló nem kíván továbbtanulni', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 42, NULL, 42, NULL),
            (9044, N'szakiskolai tanulmányok esetén tanulmányainak folytatására egészségileg alkalmatlanná vált és a továbbtanuláshoz szükséges feltételek hiányában nem tanulhat tovább', N'szakiskolai tanulmányok esetén tanulmányainak folytatására egészségileg alkalmatlanná vált és a továbbtanuláshoz szükséges feltételek hiányában nem tanulhat tovább', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 43, NULL, 43, NULL),
            (9045, N'tanulmányainak folytatására egészségileg alkalmatlanná vált és a továbbtanuláshoz szükséges feltételek a szakképző intézményben nem adottak', N'tanulmányainak folytatására egészségileg alkalmatlanná vált és a továbbtanuláshoz szükséges feltételek a szakképző intézményben nem adottak', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 44, NULL, 44, NULL),
            (9046, N'tanulmányainak folytatására egészségileg alkalmatlanná vált és a szakképző intézményben nem folyik másik megfelelő szakképzés', N'tanulmányainak folytatására egészségileg alkalmatlanná vált és a szakképző intézményben nem folyik másik megfelelő szakképzés', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 45, NULL, 45, NULL),
            (9047, N'tanulmányainak folytatására egészségileg alkalmatlanná vált és a tanuló a tanulmányait nem kívánja a szakképző intézményben tovább folytatni', N'tanulmányainak folytatására egészségileg alkalmatlanná vált és a tanuló a tanulmányait nem kívánja a szakképző intézményben tovább folytatni', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 46, NULL, 46, NULL),
            (9048, N'tanulói jogviszony – a tanköteles tanuló kivételével – fizetési hátralék miatt a szülő, nagykorú tanuló esetén a tanuló eredménytelen felszólítása és a tanuló szociális helyzetének vizsgálata után megszűnik', N'tanulói jogviszony – a tanköteles tanuló kivételével – fizetési hátralék miatt a szülő, nagykorú tanuló esetén a tanuló eredménytelen felszólítása és a tanuló szociális helyzetének vizsgálata után megszűnik', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 47, NULL, 47, NULL),
            (9049, N'Téves besorolás', N'Téves besorolás', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 48, NULL, 48, NULL),
            (9060, N'Na', N'Na', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 49, NULL, 49, NULL),
            (9117, N'a tanulót másik szakképző intézmény vagy köznevelési intézmény átvette', N'a tanulót másik szakképző intézmény vagy köznevelési intézmény átvette', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 50, NULL, 50, NULL),
            (9118, N'a tanuló az utolsó évfolyamot elvégezte', N'a tanuló az utolsó évfolyamot elvégezte', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 51, NULL, 51, NULL),
            (9119, N'technikumi tanulmányok esetén az utolsó évfolyamát elvégezte', N'technikumi tanulmányok esetén az utolsó évfolyamát elvégezte', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 52, NULL, 52, NULL),
            (9120, N'szakképző iskolai tanulmányok esetén az utolsó évfolyamát elvégezte', N'szakképző iskolai tanulmányok esetén az utolsó évfolyamát elvégezte', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 53, NULL, 53, NULL),
            (9121, N'a tanuló az utolsó évfolyamot követő első szakmai vizsgaidőszakban szakmai vizsgát tett', N'a tanuló az utolsó évfolyamot követő első szakmai vizsgaidőszakban szakmai vizsgát tett', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 54, NULL, 54, NULL),
            (9122, N'a tankötelezettség megszűnése után a tanuló írásban bejelentette, hogy kimarad', N'a tankötelezettség megszűnése után a tanuló írásban bejelentette, hogy kimarad', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 55, NULL, 55, NULL),
            (9123, N'a tanuló a technikum kizárólag szakmai vizsgára felkészítő évfolyamának az első félév utolsó tanítási napjáig érettségi végzettséget nem szerzett', N'a tanuló a technikum kizárólag szakmai vizsgára felkészítő évfolyamának az első félév utolsó tanítási napjáig érettségi végzettséget nem szerzett', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 56, NULL, 56, NULL),
            (9124, N'a tanuló elhunyt', N'a tanuló elhunyt', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 57, NULL, 57, NULL),
            (9125, N'a képzésben részt vevő személy elhunyt', N'a képzésben részt vevő személy elhunyt', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 58, NULL, 58, NULL),
            (9126, N'a képzésben részt vevő személy a technikum kizárólag szakmai vizsgára felkészítő évfolyamának az első félév utolsó tanítási napjáig érettségi végzettséget nem szerzett', N'a képzésben részt vevő személy a technikum kizárólag szakmai vizsgára felkészítő évfolyamának az első félév utolsó tanítási napjáig érettségi végzettséget nem szerzett', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 59, NULL, 59, NULL),
            (9127, N'a képzésben részt vevő személy a felnőttképzési szerződésben foglaltak szerint a szakmai oktatás követelményeit teljesítette', N'a képzésben részt vevő személy a felnőttképzési szerződésben foglaltak szerint a szakmai oktatás követelményeit teljesítette', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 60, NULL, 60, NULL),
            (9128, N'a képzésben részt vevő személy a szakképző intézményben a jogszabályban meghatározott foglalkozásnál igazolatlanul többet mulaszt', N'a képzésben részt vevő személy a szakképző intézményben a jogszabályban meghatározott foglalkozásnál igazolatlanul többet mulaszt', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 61, NULL, 61, NULL),
            (9129, N'a képzésben részt vevő személy felnőttképzési jogviszonya kimaradással megszűnt', N'a képzésben részt vevő személy felnőttképzési jogviszonya kimaradással megszűnt', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 62, NULL, 62, NULL),
            (9130, N'a képzésben részt vevő személy felnőttképzési jogviszonya a szakmai oktatás befejezésével megszűnt', N'a képzésben részt vevő személy felnőttképzési jogviszonya a szakmai oktatás befejezésével megszűnt', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 63, NULL, 63, NULL),
            (9131, N'a képzésben részt vevő személy felnőttképzési jogviszonya a szakma megszerzésével megszűnt', N'a képzésben részt vevő személy felnőttképzési jogviszonya a szakma megszerzésével megszűnt', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 64, NULL, 64, NULL),
            (9132, N'a képzésben részt vevő személy felnőttképzési jogviszonya a szakma megszerzése nélkül megszűnt', N'a képzésben részt vevő személy felnőttképzési jogviszonya a szakma megszerzése nélkül megszűnt', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 65, NULL, 65, NULL),
            (9133, N'a kollégiumi tagsági viszonya kérésére megszűnt', N'a kollégiumi tagsági viszonya kérésére megszűnt', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 66, NULL, 66, NULL),
            (9134, N'a felnőttképzési szerződést a felek közös megegyezéssel megszüntették', N'a felnőttképzési szerződést a felek közös megegyezéssel megszüntették', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 67, NULL, 67, NULL),
            (9135, N'a felnőttképzési szerződést a képzésben részt vevő személy felmondta', N'a felnőttképzési szerződést a képzésben részt vevő személy felmondta', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 68, NULL, 68, NULL),
            (9136, N'a felnőttképzési szerződést az intézmény/képzésben részt vevő azonnali hatállyal felmondta, mert képzésben részt vevő személy az oktatásban való részvételre egészségileg alkalmatlanná vált', N'a felnőttképzési szerződést az intézmény/képzésben részt vevő azonnali hatállyal felmondta, mert képzésben részt vevő személy az oktatásban való részvételre egészségileg alkalmatlanná vált', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 69, NULL, 69, NULL),
            (9137, N'a felnőttképzési szerződést az intézmény azonnali hatállyal felmondta, mert a képzésben részt vevő személy a fizetési hátralékát a felszólítás ellenére sem teljesítette', N'a felnőttképzési szerződést az intézmény azonnali hatállyal felmondta, mert a képzésben részt vevő személy a fizetési hátralékát a felszólítás ellenére sem teljesítette', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 70, NULL, 70, NULL),
            (9138, N'a felnőttképzési szerződést az intézmény azonnali hatállyal felmondta, mert a képzésben részt vevő személy a kontaktórákról a képzési programban meghatározott időnél igazolatlanul többet mulasztott', N'a felnőttképzési szerződést az intézmény azonnali hatállyal felmondta, mert a képzésben részt vevő személy a kontaktórákról a képzési programban meghatározott időnél igazolatlanul többet mulasztott', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 71, NULL, 71, NULL),
            (9139, N'tankötelezettsége a korhatár betöltésével megszűnt', N'tankötelezettsége a korhatár betöltésével megszűnt', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 72, NULL, 72, NULL),
            (9140, N'sajátos nevelési igénye megszűnt', N'sajátos nevelési igénye megszűnt', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 73, NULL, 73, NULL),
            (9141, N'alapfokú művészeti iskolában technikai lezárás osztályváltás miatt', N'alapfokú művészeti iskolában technikai lezárás osztályváltás miatt', NULL, NULL, NULL, N'JogvMegszuneseJogcimTipus', 182, N'T', N'T', 74, NULL, 74, NULL),
            (8962, N'Jogosult', N'Jogosult', NULL, NULL, NULL, N'EszkozIgenylesStatusz', 179, N'T', N'T', 1, NULL, 1, NULL),
            (8963, N'Regisztrációt benyújtotta', N'Regisztrációt benyújtotta', NULL, NULL, NULL, N'EszkozIgenylesStatusz', 179, N'T', N'T', 10, NULL, 2, NULL),
            (8964, N'Eszköz kiosztva', N'Eszköz kiosztva', NULL, NULL, NULL, N'EszkozIgenylesStatusz', 179, N'T', N'T', 20, NULL, 3, NULL),
            (8965, N'Nincs rögzítve gondviselő', N'Nincs rögzítve gondviselő', NULL, NULL, NULL, N'EszkozIgenylesStatusz', 179, N'T', N'T', 30, NULL, 4, NULL),
            (8966, N'Gondviselőnek nincs belépési adata', N'Gondviselőnek nincs belépési adata', NULL, NULL, NULL, N'EszkozIgenylesStatusz', 179, N'T', N'T', 40, NULL, 5, NULL),
            (8967, N'Gondviselő nincs törvényes képviselőnek jelölve', N'Gondviselő nincs törvényes képviselőnek jelölve', NULL, NULL, NULL, N'EszkozIgenylesStatusz', 179, N'T', N'T', 50, NULL, 6, NULL),
            (8968, N'Gondviselő nem fogadta el az ASZF-t', N'Gondviselő nem fogadta el az ASZF-t', NULL, NULL, NULL, N'EszkozIgenylesStatusz', 179, N'T', N'T', 60, NULL, 7, NULL),
            (8969, N'Munkakör alapján nem jogosult pedagógus', N'Munkakör alapján nem jogosult pedagógus', NULL, NULL, NULL, N'EszkozIgenylesStatusz', 179, N'T', N'T', 70, NULL, 8, NULL),
            (8970, N'Pedagógusnak nincs belépési adata', N'Pedagógusnak nincs belépési adata', NULL, NULL, NULL, N'EszkozIgenylesStatusz', 179, N'T', N'T', 80, NULL, 9, NULL),
            (8971, N'Pedagógus nem rendelkezik napló joggal', N'Pedagógus nem rendelkezik napló joggal', NULL, NULL, NULL, N'EszkozIgenylesStatusz', 179, N'T', N'T', 90, NULL, 10, NULL),
            (8972, N'Pedagógus nem fogadta el az ASZF-t', N'Pedagógus nem fogadta el az ASZF-t', NULL, NULL, NULL, N'EszkozIgenylesStatusz', 179, N'T', N'T', 100, NULL, 11, NULL),
            (8895, N'Na', N'Na', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 1, NULL, 1, NULL),
            (8896, N'Állatgondozó', N'Állatgondozó', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 10, NULL, 2, NULL),
            (8897, N'Aranykalászos gazda', N'Aranykalászos gazda', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 20, NULL, 3, NULL),
            (8898, N'Asztalosipari szerelő', N'Asztalosipari szerelő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 30, NULL, 4, NULL),
            (8899, N'Betonacél-szerelő', N'Betonacél-szerelő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 40, NULL, 5, NULL),
            (8900, N'Bolti előkészítő', N'Bolti előkészítő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 50, NULL, 6, NULL),
            (8901, N'Bőripari szabász', N'Bőripari szabász', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 60, NULL, 7, NULL),
            (8902, N'Bőrtárgy készítő', N'Bőrtárgy készítő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 70, NULL, 8, NULL),
            (8903, N'Cipőipari szabász', N'Cipőipari szabász', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 80, NULL, 9, NULL),
            (8904, N'Cukrászsegéd', N'Cukrászsegéd', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 90, NULL, 10, NULL),
            (8905, N'Csokoládétermék gyártó', N'Csokoládétermék gyártó', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 100, NULL, 11, NULL),
            (8906, N'Csontozó munkás', N'Csontozó munkás', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 110, NULL, 12, NULL),
            (8907, N'Csőhálózat-szerelő', N'Csőhálózat-szerelő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 120, NULL, 13, NULL),
            (8908, N'Digitális nyomóforma készítő', N'Digitális nyomóforma készítő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 130, NULL, 14, NULL),
            (8909, N'Égéstermék elvezető szerelő', N'Égéstermék elvezető szerelő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 140, NULL, 15, NULL),
            (8910, N'Élelmiszeripari csomagológép kezelő', N'Élelmiszeripari csomagológép kezelő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 150, NULL, 16, NULL),
            (8911, N'Erdőművelő', N'Erdőművelő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 160, NULL, 17, NULL),
            (8912, N'Faiskolai kertész', N'Faiskolai kertész', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 170, NULL, 18, NULL),
            (8913, N'Falazó kőműves', N'Falazó kőműves', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 180, NULL, 19, NULL),
            (8914, N'Famegmunkáló', N'Famegmunkáló', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 190, NULL, 20, NULL),
            (8915, N'Felépítő gép kezelő', N'Felépítő gép kezelő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 200, NULL, 21, NULL),
            (8916, N'Fémipari gyártás előkészítő', N'Fémipari gyártás előkészítő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 210, NULL, 22, NULL),
            (8917, N'Gépi vakoló', N'Gépi vakoló', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 220, NULL, 23, NULL),
            (8918, N'Gipszkartonszerelő', N'Gipszkartonszerelő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 230, NULL, 24, NULL),
            (8919, N'Hangosító', N'Hangosító', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 240, NULL, 25, NULL),
            (8920, N'Háztartásvezető', N'Háztartásvezető', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 250, NULL, 26, NULL),
            (8921, N'Hő- és hangszigetelő', N'Hő- és hangszigetelő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 260, NULL, 27, NULL),
            (8922, N'Hulladékgyűjtő és -szállító', N'Hulladékgyűjtő és -szállító', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 270, NULL, 28, NULL),
            (8923, N'Hulladékválogató és -feldolgozó', N'Hulladékválogató és -feldolgozó', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 280, NULL, 29, NULL),
            (8924, N'Húskészítmény gyártó', N'Húskészítmény gyártó', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 290, NULL, 30, NULL),
            (8925, N'Kárpit behúzó', N'Kárpit behúzó', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 300, NULL, 31, NULL),
            (8926, N'Keksz- és ostyagyártó', N'Keksz- és ostyagyártó', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 310, NULL, 32, NULL),
            (8927, N'Kerti munkás', N'Kerti munkás', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 320, NULL, 33, NULL),
            (8928, N'Kézi könyvkötő', N'Kézi könyvkötő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 330, NULL, 34, NULL),
            (8929, N'Kishajóalkatrész-javító', N'Kishajóalkatrész-javító', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 340, NULL, 35, NULL),
            (8930, N'Konzervgyártó', N'Konzervgyártó', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 350, NULL, 36, NULL),
            (8931, N'Lakástextil-készítő', N'Lakástextil-készítő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 360, NULL, 37, NULL),
            (8932, N'Logisztikai feldolgozó', N'Logisztikai feldolgozó', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 370, NULL, 38, NULL),
            (8933, N'Mezőgazdasági munkás', N'Mezőgazdasági munkás', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 380, NULL, 39, NULL),
            (8934, N'Növényházi munkás', N'Növényházi munkás', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 390, NULL, 40, NULL),
            (8935, N'Nyomdai grafikai kép és szövegszerkesztő', N'Nyomdai grafikai kép és szövegszerkesztő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 400, NULL, 41, NULL),
            (8936, N'Nyomdász segéd', N'Nyomdász segéd', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 410, NULL, 42, NULL),
            (8937, N'Papír feldolgozó', N'Papír feldolgozó', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 420, NULL, 43, NULL),
            (8938, N'Parkgondozó', N'Parkgondozó', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 430, NULL, 44, NULL),
            (8939, N'Pénztáros', N'Pénztáros', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 440, NULL, 45, NULL),
            (8940, N'Pincemunkás', N'Pincemunkás', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 450, NULL, 46, NULL),
            (8941, N'Pincérsegéd', N'Pincérsegéd', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 460, NULL, 47, NULL),
            (8942, N'Raktáros', N'Raktáros', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 470, NULL, 48, NULL),
            (8943, N'Segédgondozó', N'Segédgondozó', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 480, NULL, 49, NULL),
            (8944, N'Sírkő és műkőkészítő', N'Sírkő és műkőkészítő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 490, NULL, 50, NULL),
            (8945, N'Sörgyártó', N'Sörgyártó', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 500, NULL, 51, NULL),
            (8946, N'Süteménykészítő', N'Süteménykészítő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 510, NULL, 52, NULL),
            (8947, N'Sütőipari és gyorspékségi munkás', N'Sütőipari és gyorspékségi munkás', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 520, NULL, 53, NULL),
            (8948, N'Szabász', N'Szabász', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 530, NULL, 54, NULL),
            (8949, N'Szakácssegéd', N'Szakácssegéd', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 540, NULL, 55, NULL),
            (8950, N'Színpadi gépkezelő és berendező', N'Színpadi gépkezelő és berendező', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 550, NULL, 56, NULL),
            (8951, N'Szobafestő', N'Szobafestő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 560, NULL, 57, NULL),
            (8952, N'Szőlőfeldolgozó', N'Szőlőfeldolgozó', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 570, NULL, 58, NULL),
            (8953, N'Tejipari munkás', N'Tejipari munkás', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 580, NULL, 59, NULL),
            (8954, N'Textiltermék-összeállító', N'Textiltermék-összeállító', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 590, NULL, 60, NULL),
            (8955, N'Útfenntartó', N'Útfenntartó', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 600, NULL, 61, NULL),
            (8956, N'Üdítőital-ipari termékkészítő', N'Üdítőital-ipari termékkészítő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 610, NULL, 62, NULL),
            (8957, N'Vágóhídi munkás', N'Vágóhídi munkás', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 620, NULL, 63, NULL),
            (8958, N'Villamosipari előkészítő', N'Villamosipari előkészítő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 630, NULL, 64, NULL),
            (8959, N'Vízszigetelő', N'Vízszigetelő', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 640, NULL, 65, NULL),
            (8960, N'Zsaluzó, állványozó', N'Zsaluzó, állványozó', NULL, NULL, NULL, N'ReszSzakmaTipus', 178, N'T', N'T', 650, NULL, 66, NULL),
            (8832, N'Na', N'Na', NULL, NULL, NULL, N'AgazatReszSzakmaTipus', 176, N'T', N'T', 1, NULL, 1, NULL),
            (8833, N'Elektronika és elektrotechnika', N'Elektronika és elektrotechnika', NULL, NULL, NULL, N'AgazatReszSzakmaTipus', 176, N'T', N'T', 10, NULL, 2, NULL),
            (8834, N'Élelmiszeripar', N'Élelmiszeripar', NULL, NULL, NULL, N'AgazatReszSzakmaTipus', 176, N'T', N'T', 20, NULL, 3, NULL),
            (8835, N'Építőipar', N'Építőipar', NULL, NULL, NULL, N'AgazatReszSzakmaTipus', 176, N'T', N'T', 30, NULL, 4, NULL),
            (8836, N'Épületgépészet', N'Épületgépészet', NULL, NULL, NULL, N'AgazatReszSzakmaTipus', 176, N'T', N'T', 40, NULL, 5, NULL),
            (8837, N'Fa- és bútoripar', N'Fa- és bútoripar', NULL, NULL, NULL, N'AgazatReszSzakmaTipus', 176, N'T', N'T', 50, NULL, 6, NULL),
            (8838, N'Gépészet', N'Gépészet', NULL, NULL, NULL, N'AgazatReszSzakmaTipus', 176, N'T', N'T', 60, NULL, 7, NULL),
            (8839, N'Kereskedelem', N'Kereskedelem', NULL, NULL, NULL, N'AgazatReszSzakmaTipus', 176, N'T', N'T', 70, NULL, 8, NULL),
            (8840, N'Környezetvédelem és vízügy', N'Környezetvédelem és vízügy', NULL, NULL, NULL, N'AgazatReszSzakmaTipus', 176, N'T', N'T', 80, NULL, 9, NULL),
            (8841, N'Közlekedés és szállítmányozás - közlekedés alágazat', N'Közlekedés és szállítmányozás - közlekedés alágazat', NULL, NULL, NULL, N'AgazatReszSzakmaTipus', 176, N'T', N'T', 90, NULL, 10, NULL),
            (8842, N'Közlekedés és szállítmányozás - szállítmányozás alágazat', N'Közlekedés és szállítmányozás - szállítmányozás alágazat', NULL, NULL, NULL, N'AgazatReszSzakmaTipus', 176, N'T', N'T', 100, NULL, 11, NULL),
            (8843, N'Kreatív - kreatív színház- és hangtechnika alágazat', N'Kreatív - kreatív színház- és hangtechnika alágazat', NULL, NULL, NULL, N'AgazatReszSzakmaTipus', 176, N'T', N'T', 110, NULL, 12, NULL),
            (8844, N'Kreatív - kreatív vizuális alágazat', N'Kreatív - kreatív vizuális alágazat', NULL, NULL, NULL, N'AgazatReszSzakmaTipus', 176, N'T', N'T', 120, NULL, 13, NULL),
            (8845, N'Kreatív - kreatívipar alágazat', N'Kreatív - kreatívipar alágazat', NULL, NULL, NULL, N'AgazatReszSzakmaTipus', 176, N'T', N'T', 130, NULL, 14, NULL),
            (8846, N'Mezőgazdaság és erdészet', N'Mezőgazdaság és erdészet', NULL, NULL, NULL, N'AgazatReszSzakmaTipus', 176, N'T', N'T', 140, NULL, 15, NULL),
            (8847, N'Szociális', N'Szociális', NULL, NULL, NULL, N'AgazatReszSzakmaTipus', 176, N'T', N'T', 150, NULL, 16, NULL),
            (8848, N'Turizmus-vendéglátás', N'Turizmus-vendéglátás', NULL, NULL, NULL, N'AgazatReszSzakmaTipus', 176, N'T', N'T', 160, NULL, 17, NULL),
            (8849, N'Vegyipar', N'Vegyipar', NULL, NULL, NULL, N'AgazatReszSzakmaTipus', 176, N'T', N'T', 170, NULL, 18, NULL),
            (8850, N'Na', N'Na', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 1, NULL, 1, NULL),
            (8851, N'Abroncsgyártó', N'Abroncsgyártó', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 10, NULL, 2, NULL),
            (8852, N'Ács', N'Ács', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 20, NULL, 3, NULL),
            (8853, N'Asztalos', N'Asztalos', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 30, NULL, 4, NULL),
            (8854, N'Bőrtermékkészítő', N'Bőrtermékkészítő', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 40, NULL, 5, NULL),
            (8855, N'Cukrász', N'Cukrász', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 50, NULL, 6, NULL),
            (8856, N'Divatszabó', N'Divatszabó', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 60, NULL, 7, NULL),
            (8857, N'Édességkészítő', N'Édességkészítő', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 70, NULL, 8, NULL),
            (8858, N'Élelmiszeripari gépkezelő', N'Élelmiszeripari gépkezelő', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 80, NULL, 9, NULL),
            (8859, N'Épületszobrász és műköves', N'Épületszobrász és műköves', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 90, NULL, 10, NULL),
            (8860, N'Erdőművelő-fakitermelő', N'Erdőművelő-fakitermelő', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 100, NULL, 11, NULL),
            (8861, N'Erjedés- és üdítőital-ipari termékkészítő', N'Erjedés- és üdítőital-ipari termékkészítő', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 110, NULL, 12, NULL),
            (8862, N'Festő, mázoló, tapétázó', N'Festő, mázoló, tapétázó', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 120, NULL, 13, NULL),
            (8863, N'Gazda', N'Gazda', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 130, NULL, 14, NULL),
            (8864, N'Gyermek- és ifjúsági felügyelő', N'Gyermek- és ifjúsági felügyelő', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 140, NULL, 15, NULL),
            (8865, N'Hangtechnikus', N'Hangtechnikus', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 150, NULL, 16, NULL),
            (8866, N'Hegesztő', N'Hegesztő', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 160, NULL, 17, NULL),
            (8867, N'Hentes és húskészítmény-készítő', N'Hentes és húskészítmény-készítő', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 170, NULL, 18, NULL),
            (8868, N'Hulladékfeldolgozó munkatárs', N'Hulladékfeldolgozó munkatárs', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 180, NULL, 19, NULL),
            (8869, N'Kárpitos', N'Kárpitos', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 190, NULL, 20, NULL),
            (8870, N'Kereskedelmi értékesítő', N'Kereskedelmi értékesítő', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 200, NULL, 21, NULL),
            (8871, N'Kertész', N'Kertész', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 210, NULL, 22, NULL),
            (8872, N'Kertész technikus', N'Kertész technikus', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 220, NULL, 23, NULL),
            (8873, N'Kishajóépítő és -karbantartó', N'Kishajóépítő és -karbantartó', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 230, NULL, 24, NULL),
            (8874, N'Kőműves', N'Kőműves', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 240, NULL, 25, NULL),
            (8875, N'Könnyűipari technikus', N'Könnyűipari technikus', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 250, NULL, 26, NULL),
            (8876, N'Központifűtés- és gázhálózatrendszer-szerelő', N'Központifűtés- és gázhálózatrendszer-szerelő', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 260, NULL, 27, NULL),
            (8877, N'Logisztikai technikus', N'Logisztikai technikus', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 270, NULL, 28, NULL),
            (8878, N'Nyomdász', N'Nyomdász', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 280, NULL, 29, NULL),
            (8879, N'Papírgyártó és -feldolgozó, csomagolószer-gyártó', N'Papírgyártó és -feldolgozó, csomagolószer-gyártó', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 290, NULL, 30, NULL),
            (8880, N'Pék', N'Pék', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 300, NULL, 31, NULL),
            (8881, N'Pék-cukrász', N'Pék-cukrász', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 310, NULL, 32, NULL),
            (8882, N'Pincér - vendégtéri szakember', N'Pincér - vendégtéri szakember', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 320, NULL, 33, NULL),
            (8883, N'Szakács', N'Szakács', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 330, NULL, 34, NULL),
            (8884, N'Szárazépítő', N'Szárazépítő', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 340, NULL, 35, NULL),
            (8885, N'Szerkezetépítő és -szerelő', N'Szerkezetépítő és -szerelő', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 350, NULL, 36, NULL),
            (8886, N'Szigetelő', N'Szigetelő', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 360, NULL, 37, NULL),
            (8887, N'Színház- és rendezvénytechnikus', N'Színház- és rendezvénytechnikus', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 370, NULL, 38, NULL),
            (8888, N'Szociális ápoló és gondozó', N'Szociális ápoló és gondozó', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 380, NULL, 39, NULL),
            (8889, N'Szőlész-borász', N'Szőlész-borász', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 390, NULL, 40, NULL),
            (8890, N'Tartósítóipari termék-készítő', N'Tartósítóipari termék-készítő', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 400, NULL, 41, NULL),
            (8891, N'Tejtermékkészítő', N'Tejtermékkészítő', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 410, NULL, 42, NULL),
            (8892, N'Útépítő és útfenntartó', N'Útépítő és útfenntartó', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 420, NULL, 43, NULL),
            (8893, N'Villanyszerelő', N'Villanyszerelő', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 430, NULL, 44, NULL),
            (8894, N'Víz- és csatornarendszer-szerelő', N'Víz- és csatornarendszer-szerelő', NULL, NULL, NULL, N'SzakmaReszSzakmaTipus', 177, N'T', N'T', 440, NULL, 45, NULL),
            (8796, N'Na', N'Na', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 1, NULL, 1, NULL),
            (8797, N'Mozgásszervi fogyatékos', N'Mozgásszervi fogyatékos', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 10, NULL, 2, NULL),
            (8798, N'Vak', N'Vak', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 20, NULL, 3, NULL),
            (8799, N'Gyengénlátó', N'Gyengénlátó', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 30, NULL, 4, NULL),
            (8800, N'Siket', N'Siket', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 40, NULL, 5, NULL),
            (8801, N'Nagyothalló', N'Nagyothalló', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 50, NULL, 6, NULL),
            (8802, N'Enyhén értelmi fogyatékos', N'Enyhén értelmi fogyatékos', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 60, NULL, 7, NULL),
            (8803, N'Középsúlyosan értelmi fogyatékos', N'Középsúlyosan értelmi fogyatékos', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 70, NULL, 8, NULL),
            (8804, N'Beszédfogyatékos', N'Beszédfogyatékos', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 80, NULL, 9, NULL),
            (8805, N'Autizmus spektrum zavarral küzdő', N'Autizmus spektrum zavarral küzdő', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 90, NULL, 10, NULL),
            (8806, N'Egyéb pszichés fejlődési zavar - Súlyos tanulási zavar', N'Egyéb pszichés fejlődési zavar - Súlyos tanulási zavar', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 100, NULL, 11, NULL),
            (8807, N'Egyéb pszichés fejlődési zavar - Súlyos figyelemszabályozási zavar', N'Egyéb pszichés fejlődési zavar - Súlyos figyelemszabályozási zavar', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 110, NULL, 12, NULL),
            (8808, N'Egyéb pszichés fejlődési zavar - Súlyos magatartásszabályozási zavar', N'Egyéb pszichés fejlődési zavar - Súlyos magatartásszabályozási zavar', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 120, NULL, 13, NULL),
            (8809, N'Siket és vak', N'Siket és vak', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 130, NULL, 14, NULL),
            (8810, N'Enyhén értelmi fogyatékos és mozgásszervi fogyatékos', N'Enyhén értelmi fogyatékos és mozgásszervi fogyatékos', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 140, NULL, 15, NULL),
            (8811, N'Enyhén értelmi fogyatékos és vak', N'Enyhén értelmi fogyatékos és vak', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 150, NULL, 16, NULL),
            (8812, N'Enyhén értelmi fogyatékos és gyengénlátó', N'Enyhén értelmi fogyatékos és gyengénlátó', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 160, NULL, 17, NULL),
            (8813, N'Enyhén értelmi fogyatékos és siket', N'Enyhén értelmi fogyatékos és siket', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 170, NULL, 18, NULL),
            (8814, N'Enyhén értelmi fogyatékos és nagyothalló', N'Enyhén értelmi fogyatékos és nagyothalló', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 180, NULL, 19, NULL),
            (8815, N'Enyhén értelmi fogyatékos és autizmus spektrum zavarral küzdő', N'Enyhén értelmi fogyatékos és autizmus spektrum zavarral küzdő', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 190, NULL, 20, NULL),
            (8816, N'Középsúlyosan értelmi fogyatékos és mozgásszervi fogyatékos', N'Középsúlyosan értelmi fogyatékos és mozgásszervi fogyatékos', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 200, NULL, 21, NULL),
            (8817, N'Középsúlyosan értelmi fogyatékos és vak', N'Középsúlyosan értelmi fogyatékos és vak', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 210, NULL, 22, NULL),
            (8818, N'Középsúlyosan értelmi fogyatékos és siket', N'Középsúlyosan értelmi fogyatékos és siket', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 220, NULL, 23, NULL),
            (8819, N'Középsúlyosan értelmi fogyatékos és autizmus spektrum zavarral küzdő', N'Középsúlyosan értelmi fogyatékos és autizmus spektrum zavarral küzdő', NULL, NULL, NULL, N'FogyatekossagTipus', 175, N'T', N'T', 230, NULL, 24, NULL),
            (8733, N'Regisztrációs igazolás', N'Regisztrációs igazolás', NULL, NULL, NULL, N'TartJogOkmanyTipus', 173, N'T', N'T', 10, NULL, 1, NULL),
            (8734, N'Tartózkodási kártya', N'Tartózkodási kártya', NULL, NULL, NULL, N'TartJogOkmanyTipus', 173, N'T', N'T', 20, NULL, 2, NULL),
            (8735, N'Állandó tartózkodási kártya', N'Állandó tartózkodási kártya', NULL, NULL, NULL, N'TartJogOkmanyTipus', 173, N'T', N'T', 30, NULL, 3, NULL),
            (8667, N'Na', N'Na', NULL, NULL, NULL, N'TartozkodasJogcimTipus', 170, N'T', N'T', 1, NULL, 1, NULL),
            (8668, N'Tartózkodási vízum', N'Tartózkodási vízum', NULL, NULL, NULL, N'TartozkodasJogcimTipus', 170, N'T', N'T', 10, NULL, 2, NULL),
            (8669, N'Tartózkodási engedély', N'Tartózkodási engedély', NULL, NULL, NULL, N'TartozkodasJogcimTipus', 170, N'T', N'T', 20, NULL, 3, NULL),
            (8670, N'EU kék kártya', N'EU kék kártya', NULL, NULL, NULL, N'TartozkodasJogcimTipus', 170, N'T', N'T', 30, NULL, 4, NULL),
            (8671, N'Bevándorlási engedély', N'Bevándorlási engedély', NULL, NULL, NULL, N'TartozkodasJogcimTipus', 170, N'T', N'T', 40, NULL, 5, NULL),
            (8672, N'Letelepedési engedély', N'Letelepedési engedély', NULL, NULL, NULL, N'TartozkodasJogcimTipus', 170, N'T', N'T', 50, NULL, 6, NULL),
            (8673, N'Ideiglenes letelepedési engedély', N'Ideiglenes letelepedési engedély', NULL, NULL, NULL, N'TartozkodasJogcimTipus', 170, N'T', N'T', 60, NULL, 7, NULL),
            (8674, N'Nemzeti letelepedési engedély', N'Nemzeti letelepedési engedély', NULL, NULL, NULL, N'TartozkodasJogcimTipus', 170, N'T', N'T', 70, NULL, 8, NULL),
            (8675, N'EK letelepedési engedély', N'EK letelepedési engedély', NULL, NULL, NULL, N'TartozkodasJogcimTipus', 170, N'T', N'T', 80, NULL, 9, NULL),
            (8676, N'Menekültként elismert', N'Menekültként elismert', NULL, NULL, NULL, N'TartozkodasJogcimTipus', 170, N'T', N'T', 90, NULL, 10, NULL),
            (8622, N'Szülő', N'Szülő', NULL, NULL, NULL, N'TKJogalapTipus', 167, N'T', N'F', 5, N'SZULO', 1, NULL),
            (8623, N'Gyám', N'Gyám', NULL, NULL, NULL, N'TKJogalapTipus', 167, N'T', N'F', 10, N'GYAM', 2, NULL),
            (8637, N'Magyar Katolikus Egyház', N'Magyar Katolikus Egyház', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 5, NULL, 1, NULL),
            (8638, N'Magyarországi Református Egyház', N'Magyarországi Református Egyház', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 10, NULL, 2, NULL),
            (8639, N'Magyarországi Evangélikus Egyház', N'Magyarországi Evangélikus Egyház', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 15, NULL, 3, NULL),
            (8640, N'Magyarországi Zsidó Hitközségek Szövetsége', N'Magyarországi Zsidó Hitközségek Szövetsége', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 20, NULL, 4, NULL),
            (8641, N'Egységes Magyarországi Izraelita Hitközség (Statusquo Ante)', N'Egységes Magyarországi Izraelita Hitközség (Statusquo Ante)', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 25, NULL, 5, NULL),
            (8642, N'Magyarországi Autonóm Orthodox Izraelita Hitközség', N'Magyarországi Autonóm Orthodox Izraelita Hitközség', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 30, NULL, 6, NULL),
            (8643, N'Budai Szerb Ortodox Egyházmegye', N'Budai Szerb Ortodox Egyházmegye', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 35, NULL, 7, NULL),
            (8644, N'Konstantinápolyi Egyetemes Patriarchátus – Magyarországi Ortodox Exarchátus', N'Konstantinápolyi Egyetemes Patriarchátus – Magyarországi Ortodox Exarchátus', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 40, NULL, 8, NULL),
            (8645, N'Magyarországi Bolgár Ortodox Egyház', N'Magyarországi Bolgár Ortodox Egyház', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 45, NULL, 9, NULL),
            (8646, N'Magyarországi Román Ortodox Egyházmegye', N'Magyarországi Román Ortodox Egyházmegye', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 50, NULL, 10, NULL),
            (8647, N'Orosz Ortodox Egyház Magyar Egyházmegyéje (Moszkvai Patriarchátus)', N'Orosz Ortodox Egyház Magyar Egyházmegyéje (Moszkvai Patriarchátus)', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 55, NULL, 11, NULL),
            (8648, N'Magyar Unitárius Egyház Magyarországi Egyházkerülete (Magyarországi Unitárius Egyház)', N'Magyar Unitárius Egyház Magyarországi Egyházkerülete (Magyarországi Unitárius Egyház)', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 60, NULL, 12, NULL),
            (8649, N'Magyarországi Baptista Egyház', N'Magyarországi Baptista Egyház', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 65, NULL, 13, NULL),
            (8650, N'HIT Gyülekezete', N'HIT Gyülekezete', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 70, NULL, 14, NULL),
            (8651, N'Magyarországi Metodista Egyház', N'Magyarországi Metodista Egyház', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 75, NULL, 15, NULL),
            (8652, N'Magyar Pünkösdi Egyház', N'Magyar Pünkösdi Egyház', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 80, NULL, 16, NULL),
            (8653, N'Szent Margit Anglikán/Episzkopális Egyház', N'Szent Margit Anglikán/Episzkopális Egyház', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 85, NULL, 17, NULL),
            (8654, N'Erdélyi Gyülekezet', N'Erdélyi Gyülekezet', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 90, NULL, 18, NULL),
            (8655, N'Hetednapi Adventista Egyház', N'Hetednapi Adventista Egyház', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 95, NULL, 19, NULL),
            (8656, N'Magyarországi Kopt Ortodox Egyház', N'Magyarországi Kopt Ortodox Egyház', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 100, NULL, 20, NULL),
            (8657, N'Magyarországi Iszlám Tanács', N'Magyarországi Iszlám Tanács', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 105, NULL, 21, NULL),
            (8658, N'Krisztusban Hívő Nazarénus Gyülekezetek', N'Krisztusban Hívő Nazarénus Gyülekezetek', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 110, NULL, 22, NULL),
            (8659, N'Magyarországi Krisna-tudatú Hívők Közössége', N'Magyarországi Krisna-tudatú Hívők Közössége', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 115, NULL, 23, NULL),
            (8660, N'Az Üdvhadsereg Szabadegyház Magyarország', N'Az Üdvhadsereg Szabadegyház Magyarország', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 120, NULL, 24, NULL),
            (8661, N'Az Utolsó Napok Szentjeinek Jézus Krisztus Egyháza', N'Az Utolsó Napok Szentjeinek Jézus Krisztus Egyháza', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 125, NULL, 25, NULL),
            (8662, N'Magyarországi Jehova Tanúi Egyház', N'Magyarországi Jehova Tanúi Egyház', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 130, NULL, 26, NULL),
            (8663, N'Buddhista vallási közösségek', N'Buddhista vallási közösségek', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 135, NULL, 27, NULL),
            (8664, N'(Nincs adat)', N'(Nincs adat)', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'F', 140, NULL, 28, NULL),
            (9070, N'Római Katolikus Egyház', N'Római Katolikus Egyház', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'T', 145, NULL, 29, NULL),
            (9071, N'Görög Katolikus Egyház', N'Görög Katolikus Egyház', NULL, NULL, NULL, N'FelekezetTipus', 169, N'T', N'T', 150, NULL, 30, NULL),
            (8624, N'bolgár', N'bolgár', NULL, NULL, NULL, N'NemzetisegTipus', 168, N'T', N'F', 5, N'BOLGAR', 1, NULL),
            (8625, N'görög', N'görög', NULL, NULL, NULL, N'NemzetisegTipus', 168, N'T', N'F', 10, N'GOROG', 2, NULL),
            (8626, N'horvát', N'horvát', NULL, NULL, NULL, N'NemzetisegTipus', 168, N'T', N'F', 15, N'HORVAT', 3, NULL),
            (8627, N'lengyel', N'lengyel', NULL, NULL, NULL, N'NemzetisegTipus', 168, N'T', N'F', 20, N'LENGYEL', 4, NULL),
            (8628, N'német', N'német', NULL, NULL, NULL, N'NemzetisegTipus', 168, N'T', N'F', 25, N'NEMET', 5, NULL),
            (8629, N'örmény', N'örmény', NULL, NULL, NULL, N'NemzetisegTipus', 168, N'T', N'F', 30, N'ORMENY', 6, NULL),
            (8630, N'roma', N'roma', NULL, NULL, NULL, N'NemzetisegTipus', 168, N'T', N'F', 35, N'ROMA', 7, NULL),
            (8631, N'román', N'román', NULL, NULL, NULL, N'NemzetisegTipus', 168, N'T', N'F', 40, N'ROMAN', 8, NULL),
            (8632, N'ruszin', N'ruszin', NULL, NULL, NULL, N'NemzetisegTipus', 168, N'T', N'F', 45, N'RUSZIN', 9, NULL),
            (8633, N'szerb', N'szerb', NULL, NULL, NULL, N'NemzetisegTipus', 168, N'T', N'F', 50, N'SZERB', 10, NULL),
            (8634, N'szlovák', N'szlovák', NULL, NULL, NULL, N'NemzetisegTipus', 168, N'T', N'F', 55, N'SZLOVAK', 11, NULL),
            (8635, N'szlovén', N'szlovén', NULL, NULL, NULL, N'NemzetisegTipus', 168, N'T', N'F', 60, N'SZLOVEN', 12, NULL),
            (8636, N'ukrán', N'ukrán', NULL, NULL, NULL, N'NemzetisegTipus', 168, N'T', N'F', 65, N'UKRAN', 13, NULL),
            (8677, N'Na', N'Na', NULL, NULL, NULL, N'TanszakTipus', 172, N'F', N'T', 1, NULL, 1, NULL),
            (8706, N'Akkordikus', N'Akkordikus', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 1000, NULL, 2, NULL),
            (8707, N'Billentyűs', N'Billentyűs', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 1010, NULL, 3, NULL),
            (8708, N'Fafúvós', N'Fafúvós', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 1020, NULL, 4, NULL),
            (8709, N'Fúvós', N'Fúvós', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 1030, NULL, 5, NULL),
            (8710, N'Kamarazene', N'Kamarazene', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 1040, NULL, 6, NULL),
            (8711, N'Pengetős', N'Pengetős', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 1050, NULL, 7, NULL),
            (8712, N'Rézfúvós', N'Rézfúvós', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 1060, NULL, 8, NULL),
            (8713, N'Ütős', N'Ütős', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 1070, NULL, 9, NULL),
            (8714, N'Vokális', N'Vokális', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 1080, NULL, 10, NULL),
            (8715, N'Vonós és tekerő', N'Vonós és tekerő', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 1090, NULL, 11, NULL),
            (8716, N'Vonós', N'Vonós', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 1100, NULL, 12, NULL),
            (8717, N'Zeneismeret', N'Zeneismeret', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 1110, NULL, 13, NULL),
            (8718, N'Balett', N'Balett', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 2000, NULL, 14, NULL),
            (8719, N'Kortárstánc', N'Kortárstánc', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 2010, NULL, 15, NULL),
            (8720, N'Moderntánc', N'Moderntánc', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 2020, NULL, 16, NULL),
            (8721, N'Néptánc', N'Néptánc', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 2030, NULL, 17, NULL),
            (8722, N'Társastánc', N'Társastánc', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 2040, NULL, 18, NULL),
            (8723, N'Fém- és zománcműves', N'Fém- és zománcműves', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 3010, NULL, 19, NULL),
            (8724, N'Fotó és film', N'Fotó és film', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 3020, NULL, 20, NULL),
            (8725, N'Grafika és festészet', N'Grafika és festészet', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 3030, NULL, 21, NULL),
            (8726, N'Képzőművészeti', N'Képzőművészeti', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 3040, NULL, 22, NULL),
            (8727, N'Környezet- és kézműveskultúra', N'Környezet- és kézműveskultúra', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 3050, NULL, 23, NULL),
            (8728, N'Szobrászat és kerámia', N'Szobrászat és kerámia', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 3060, NULL, 24, NULL),
            (8729, N'Textil- és bőrműves', N'Textil- és bőrműves', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 3070, NULL, 25, NULL),
            (8730, N'Színjáték', N'Színjáték', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 4000, NULL, 26, NULL),
            (8731, N'Bábjáték', N'Bábjáték', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 4010, NULL, 27, NULL),
            (8732, N'Több tanszak', N'Több tanszak', NULL, NULL, NULL, N'TanszakTipus', 172, N'T', N'T', 10000, NULL, 28, NULL),
            (8678, N'Klasszikus zene', N'Klasszikus zene', NULL, NULL, NULL, N'MufajTipus', 171, N'T', N'T', 10, NULL, 1, NULL),
            (8679, N'Népzene', N'Népzene', NULL, NULL, NULL, N'MufajTipus', 171, N'T', N'T', 20, NULL, 2, NULL),
            (8680, N'Jazz-zene', N'Jazz-zene', NULL, NULL, NULL, N'MufajTipus', 171, N'T', N'T', 30, NULL, 3, NULL),
            (8681, N'Elektroakusztikus zene', N'Elektroakusztikus zene', NULL, NULL, NULL, N'MufajTipus', 171, N'T', N'T', 40, NULL, 4, NULL),
            (8704, N'Na', N'Na', NULL, NULL, NULL, N'MufajTipus', 171, N'T', N'T', 1, NULL, 5, NULL),
            (8705, N'Több műfaj', N'Több műfaj', NULL, NULL, NULL, N'MufajTipus', 171, N'T', N'T', 1000, NULL, 6, NULL),
            (8600, N'Na', N'Na', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 1, NULL, 1, NULL),
            (8601, N'YouTube videók', N'YouTube videók', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 5, NULL, 2, NULL),
            (8602, N'Zanza.tv', N'Zanza.tv', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 10, NULL, 3, NULL),
            (8603, N'Okosdoboz.hu', N'Okosdoboz.hu', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 15, NULL, 4, NULL),
            (8604, N'Mozaweb, Mozabook', N'Mozaweb, Mozabook', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 20, NULL, 5, NULL),
            (8605, N'Kahoot', N'Kahoot', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 25, NULL, 6, NULL),
            (8606, N'Wordwall', N'Wordwall', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 30, NULL, 7, NULL),
            (8607, N'Quizlet', N'Quizlet', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 35, NULL, 8, NULL),
            (8608, N'Sulinet Tudásbázis', N'Sulinet Tudásbázis', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 40, NULL, 9, NULL),
            (8609, N'NKP (Okostankönyv)', N'NKP (Okostankönyv)', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 45, NULL, 10, NULL),
            (8610, N'Videotanar.hu', N'Videotanar.hu', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 50, NULL, 11, NULL),
            (8611, N'Socrative', N'Socrative', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 55, NULL, 12, NULL),
            (8612, N'Egyéb', N'Egyéb', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 200, NULL, 13, NULL),
            (8613, N'nem használok egyéb tanulástámogató eszközt', N'nem használok egyéb tanulástámogató eszközt', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 210, NULL, 14, NULL),
            (8666, N'Saját videó', N'Saját videó', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 57, NULL, 15, NULL),
            (8683, N'Sutori', N'Sutori', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 60, NULL, 16, NULL),
            (8684, N'LearningApps.org', N'LearningApps.org', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 70, NULL, 17, NULL),
            (8685, N'Matific', N'Matific', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 80, NULL, 18, NULL),
            (8686, N'Genially', N'Genially', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 90, NULL, 19, NULL),
            (8687, N'Nearpod', N'Nearpod', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 100, NULL, 20, NULL),
            (8688, N'TED-Ed', N'TED-Ed', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 110, NULL, 21, NULL),
            (8689, N'Biteable', N'Biteable', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 120, NULL, 22, NULL),
            (8690, N'Flipgrid', N'Flipgrid', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 130, NULL, 23, NULL),
            (8691, N'Flippity', N'Flippity', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 140, NULL, 24, NULL),
            (8692, N'Mentimeter', N'Mentimeter', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 150, NULL, 25, NULL),
            (8693, N'Baamboozle', N'Baamboozle', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 160, NULL, 26, NULL),
            (8694, N'Padlet', N'Padlet', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 170, NULL, 27, NULL),
            (8695, N'edpuzzle', N'edpuzzle', NULL, NULL, NULL, N'DigTamEszkozTipus', 166, N'T', N'T', 180, NULL, 28, NULL),
            (8573, N'Na', N'Na', NULL, NULL, NULL, N'DigPlatformTipus', 165, N'T', N'T', 1, NULL, 1, NULL),
            (8574, N'KRÉTA DKT', N'KRÉTA DKT', NULL, NULL, NULL, N'DigPlatformTipus', 165, N'T', N'T', 5, NULL, 2, NULL),
            (8575, N'Google Classroom', N'Google Classroom', NULL, NULL, NULL, N'DigPlatformTipus', 165, N'T', N'T', 10, NULL, 3, NULL),
            (8576, N'Google Drive', N'Google Drive', NULL, NULL, NULL, N'DigPlatformTipus', 165, N'T', N'T', 15, NULL, 4, NULL),
            (8577, N'Microsoft Teams', N'Microsoft Teams', NULL, NULL, NULL, N'DigPlatformTipus', 165, N'T', N'T', 20, NULL, 5, NULL),
            (8578, N'Redmenta', N'Redmenta', NULL, NULL, NULL, N'DigPlatformTipus', 165, N'T', N'T', 25, NULL, 6, NULL),
            (8579, N'Moodle', N'Moodle', NULL, NULL, NULL, N'DigPlatformTipus', 165, N'T', N'T', 30, NULL, 7, NULL),
            (8580, N'Canvas', N'Canvas', NULL, NULL, NULL, N'DigPlatformTipus', 165, N'T', N'T', 35, NULL, 8, NULL),
            (8581, N'LearningApps', N'LearningApps', NULL, NULL, NULL, N'DigPlatformTipus', 165, N'T', N'T', 40, NULL, 9, NULL),
            (8582, N'NeoLMS', N'NeoLMS', NULL, NULL, NULL, N'DigPlatformTipus', 165, N'T', N'T', 45, NULL, 10, NULL),
            (8583, N'e-mail küldése', N'e-mail küldése', NULL, NULL, NULL, N'DigPlatformTipus', 165, N'T', N'T', 50, NULL, 11, NULL),
            (8584, N'Facebook', N'Facebook', NULL, NULL, NULL, N'DigPlatformTipus', 165, N'T', N'T', 55, NULL, 12, NULL),
            (8585, N'Egyéb', N'Egyéb', NULL, NULL, NULL, N'DigPlatformTipus', 165, N'T', N'T', 60, NULL, 13, NULL),
            (8586, N'nem használok digitális platformot', N'nem használok digitális platformot', NULL, NULL, NULL, N'DigPlatformTipus', 165, N'T', N'T', 65, NULL, 14, NULL),
            (8587, N'Na', N'Na', NULL, NULL, NULL, N'DigEszkozTipus', 164, N'T', N'T', 1, NULL, 1, NULL),
            (8588, N'KRÉTA DKT', N'KRÉTA DKT', NULL, NULL, NULL, N'DigEszkozTipus', 164, N'T', N'T', 5, NULL, 2, NULL),
            (8589, N'Zoom', N'Zoom', NULL, NULL, NULL, N'DigEszkozTipus', 164, N'T', N'T', 10, NULL, 3, NULL),
            (8590, N'Skype', N'Skype', NULL, NULL, NULL, N'DigEszkozTipus', 164, N'T', N'T', 15, NULL, 4, NULL),
            (8591, N'WhatsApp', N'WhatsApp', NULL, NULL, NULL, N'DigEszkozTipus', 164, N'T', N'T', 20, NULL, 5, NULL),
            (8592, N'Google Meet', N'Google Meet', NULL, NULL, NULL, N'DigEszkozTipus', 164, N'T', N'T', 25, NULL, 6, NULL),
            (8593, N'Microsoft Teams', N'Microsoft Teams', NULL, NULL, NULL, N'DigEszkozTipus', 164, N'T', N'T', 30, NULL, 7, NULL),
            (8594, N'Discord', N'Discord', NULL, NULL, NULL, N'DigEszkozTipus', 164, N'T', N'T', 35, NULL, 8, NULL),
            (8595, N'Viber', N'Viber', NULL, NULL, NULL, N'DigEszkozTipus', 164, N'T', N'T', 40, NULL, 9, NULL),
            (8596, N'Hangouts', N'Hangouts', NULL, NULL, NULL, N'DigEszkozTipus', 164, N'T', N'T', 45, NULL, 10, NULL),
            (8597, N'Facebook Messenger', N'Facebook Messenger', NULL, NULL, NULL, N'DigEszkozTipus', 164, N'T', N'T', 50, NULL, 11, NULL),
            (8598, N'Egyéb', N'Egyéb', NULL, NULL, NULL, N'DigEszkozTipus', 164, N'T', N'T', 55, NULL, 12, NULL),
            (8599, N'nem használok on-line kapcsolattartást', N'nem használok on-line kapcsolattartást', NULL, NULL, NULL, N'DigEszkozTipus', 164, N'T', N'T', 60, NULL, 13, NULL),
            (8665, N'Cisco Webex', N'Cisco Webex', NULL, NULL, NULL, N'DigEszkozTipus', 164, N'T', N'T', 52, NULL, 14, NULL),
            (8464, N'Na', N'Na', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 1, NULL, 1, NULL),
            (8465, N'Aranyműves - 5 0213 01 (2020)', N'Aranyműves - 5 0213 01', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 10, NULL, 2, NULL),
            (8466, N'Artista I. - 5 0215 01 (2020)', N'Artista I. - 5 0215 01', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 20, NULL, 3, NULL),
            (8467, N'Artista II. - 4 0215 01 (2020)', N'Artista II. - 4 0215 01', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 30, NULL, 4, NULL),
            (8468, N'Bronzműves és szoboröntő - 5 0213 02 (2020)', N'Bronzműves és szoboröntő - 5 0213 02', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 40, NULL, 5, NULL),
            (8469, N'Díszítő festő - 5 0213 03 (2020)', N'Díszítő festő - 5 0213 03', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 50, NULL, 6, NULL),
            (8470, N'Egyházzenész I. - 5 0215 02 (2020)', N'Egyházzenész I. - 5 0215 02', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 60, NULL, 7, NULL),
            (8471, N'Egyházzenész II. - 4 0215 02 (2020)', N'Egyházzenész II. - 4 0215 02', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 70, NULL, 8, NULL),
            (8472, N'Ezüstműves - 5 0213 04 (2020)', N'Ezüstműves - 5 0213 04', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 80, NULL, 9, NULL),
            (8473, N'Hangszerkészítő és -javító I. - 5 0214 03 (2020)', N'Hangszerkészítő és -javító I. - 5 0215 03', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 90, NULL, 10, NULL),
            (8474, N'Hangszerkészítő és -javító II. - 4 0214 03 (2020)', N'Hangszerkészítő és -javító II. - 4 0215 03', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 100, NULL, 11, NULL),
            (8475, N'Jazz-zenész I. - 5 0215 04 (2020)', N'Jazz-zenész I. - 5 0215 04', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 110, NULL, 12, NULL),
            (8476, N'Jazz-zenész II. - 4 0215 04 (2020)', N'Jazz-zenész II. - 4 0215 04', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 120, NULL, 13, NULL),
            (8477, N'Képző- és iparművészeti munkatárs - 4 0213 01 (2020)', N'Képző- és iparművészeti munkatárs - 4 0213 01', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 130, NULL, 14, NULL),
            (8478, N'Klasszikus zenész I. - 5 0215 05 (2020)', N'Klasszikus zenész I. - 5 0215 05', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 140, NULL, 15, NULL),
            (8479, N'Klasszikus zenész II. - 4 0215 05 (2020)', N'Klasszikus zenész II. - 4 0215 05', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 150, NULL, 16, NULL),
            (8480, N'Közművelődési munkatárs - 4 0413 01 (2020)', N'Közművelődési munkatárs - 4 0413 01', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 160, NULL, 17, NULL),
            (8481, N'Népi kézműves - 3 0214 01 (2020)', N'Népi kézműves - 3 0214 01', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 170, NULL, 18, NULL),
            (8482, N'Népzenész I. - 5 0215 06 (2020)', N'Népzenész I. - 5 0215 06', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 180, NULL, 19, NULL),
            (8483, N'Népzenész II. - 4 0215 06 (2020)', N'Népzenész II. - 4 0215 06', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 190, NULL, 20, NULL),
            (8484, N'Pedagógiai munkatárs - 4 0119 01 (2020)', N'Pedagógiai munkatárs - 4 0119 01', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 200, NULL, 21, NULL),
            (8485, N'Színész I. - 5 0215 07 (2020)', N'Színész I. - 5 0215 07', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 210, NULL, 22, NULL),
            (8486, N'Színész II. - 4 0215 07 (2020)', N'Színész II. - 4 0215 07', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 220, NULL, 23, NULL),
            (8487, N'Szórakoztató zenész I. - 5 0215 08 (2020)', N'Szórakoztató zenész I. - 5 0215 08', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 230, NULL, 24, NULL),
            (8488, N'Szórakoztató zenész II. - 4 0215 08 (2020)', N'Szórakoztató zenész II. - 4 0215 08', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 240, NULL, 25, NULL),
            (8489, N'Táncos I. - 5 0215 09 (2020)', N'Táncos I. - 5 0215 09', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 250, NULL, 26, NULL),
            (8490, N'Táncos II. - 4 0215 09 (2020)', N'Táncos II. - 4 0215 09', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 260, NULL, 27, NULL),
            (8491, N'Vésnök - 5 0213 05 (2020)', N'Vésnök - 5 0213 05', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'T', 270, NULL, 28, NULL),
            (8492, N'Vegyes', N'Vegyes', NULL, NULL, NULL, N'SzakkepesitesNktTipus', 162, N'T', N'F', 1000, NULL, 29, NULL),
            (8459, N'Na', N'Na', NULL, NULL, NULL, N'TanulmanyiTeruletNktTipus', 161, N'T', N'T', 1, NULL, 1, NULL),
            (8460, N'Művészet (2020)', N'Művészet', NULL, NULL, NULL, N'TanulmanyiTeruletNktTipus', 161, N'T', N'T', 10, NULL, 2, NULL),
            (8461, N'Pedagógia (2020)', N'Pedagógia', NULL, NULL, NULL, N'TanulmanyiTeruletNktTipus', 161, N'T', N'T', 20, NULL, 3, NULL),
            (8462, N'Közművelődés (2020)', N'Közművelődés', NULL, NULL, NULL, N'TanulmanyiTeruletNktTipus', 161, N'T', N'T', 30, NULL, 4, NULL),
            (8463, N'Vegyes', N'Vegyes', NULL, NULL, NULL, N'TanulmanyiTeruletNktTipus', 161, N'T', N'F', 100, NULL, 5, NULL),
            (8453, N'Állami tulajdonban lévő ingatlan', N'Állami tulajdonban lévő ingatlan', NULL, NULL, NULL, N'TulajdonTipus', 160, N'T', N'T', 1, NULL, 1, NULL),
            (8454, N'Önkormányzati tulajdonban lévő ingatlan', N'Önkormányzati tulajdonban lévő ingatlan', NULL, NULL, NULL, N'TulajdonTipus', 160, N'T', N'T', 2, NULL, 2, NULL),
            (8493, N'Na', N'Na', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 1, NULL, 1, NULL),
            (8494, N'Akrobata', N'Akrobata', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 10, NULL, 2, NULL),
            (8495, N'Bábszínész', N'Bábszínész', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 20, NULL, 3, NULL),
            (8496, N'Bohóc', N'Bohóc', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 30, NULL, 4, NULL),
            (8497, N'Cimbalom', N'Cimbalom', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 40, NULL, 5, NULL),
            (8498, N'Csipkekészítő', N'Csipkekészítő', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 50, NULL, 6, NULL),
            (8499, N'Divatstílus- és jelmeztervező', N'Divatstílus- és jelmeztervező', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 60, NULL, 7, NULL),
            (8500, N'Egyensúlyozó', N'Egyensúlyozó', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 70, NULL, 8, NULL),
            (8501, N'Énekes szólista', N'Énekes szólista', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 80, NULL, 9, NULL),
            (8502, N'Fafúvós', N'Fafúvós', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 90, NULL, 10, NULL),
            (8503, N'Faműves, fajátékkészítő', N'Faműves, fajátékkészítő', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 100, NULL, 11, NULL),
            (8504, N'Fazekas', N'Fazekas', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 110, NULL, 12, NULL),
            (8505, N'Festő', N'Festő', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 120, NULL, 13, NULL),
            (8506, N'Fonottbútor készítő', N'Fonottbútor készítő', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 130, NULL, 14, NULL),
            (8507, N'Gyékény-, szalma- és csuhétárgykészítő', N'Gyékény-, szalma- és csuhétárgykészítő', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 140, NULL, 15, NULL),
            (8508, N'Gyógypedagógiai asszisztens', N'Gyógypedagógiai asszisztens', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 150, NULL, 16, NULL),
            (8509, N'Hangkultúra', N'Hangkultúra', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 160, NULL, 17, NULL),
            (8510, N'Jazz-énekes', N'Jazz-énekes', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 170, NULL, 18, NULL),
            (8511, N'Jazz-zenész (Billentyűs)', N'Jazz-zenész (Billentyűs)', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 180, NULL, 19, NULL),
            (8512, N'Jazz-zenész (Fafúvós)', N'Jazz-zenész (Fafúvós)', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 190, NULL, 20, NULL),
            (8513, N'Jazz-zenész (Húros/vonós)', N'Jazz-zenész (Húros/vonós)', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 200, NULL, 21, NULL),
            (8514, N'Jazz-zenész (Rézfúvós)', N'Jazz-zenész (Rézfúvós)', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 210, NULL, 22, NULL),
            (8515, N'Jazz-zenész (Ütős)', N'Jazz-zenész (Ütős)', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 220, NULL, 23, NULL),
            (8516, N'Kántor-énekvezető', N'Kántor-énekvezető', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 230, NULL, 24, NULL),
            (8517, N'Kántor-gitáros', N'Kántor-gitáros', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 240, NULL, 25, NULL),
            (8518, N'Kántor-kórusvezető', N'Kántor-kórusvezető', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 250, NULL, 26, NULL),
            (8519, N'Kántor-orgonista', N'Kántor-orgonista', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 260, NULL, 27, NULL),
            (8520, N'Kékfestő', N'Kékfestő', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 270, NULL, 28, NULL),
            (8521, N'Kerámiaműves', N'Kerámiaműves', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 280, NULL, 29, NULL),
            (8522, N'Kézi és gépi hímző', N'Kézi és gépi hímző', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 290, NULL, 30, NULL),
            (8523, N'Klasszikus balett-táncos', N'Klasszikus balett-táncos', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 300, NULL, 31, NULL),
            (8524, N'Klasszikus zenész (Billentyűs)', N'Klasszikus zenész (Billentyűs)', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 310, NULL, 32, NULL),
            (8525, N'Klasszikus zenész (Fafúvós)', N'Klasszikus zenész (Fafúvós)', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 320, NULL, 33, NULL),
            (8526, N'Klasszikus zenész (Húros/vonós)', N'Klasszikus zenész (Húros/vonós)', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 330, NULL, 34, NULL),
            (8527, N'Klasszikus zenész (Rézfúvós)', N'Klasszikus zenész (Rézfúvós)', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 340, NULL, 35, NULL),
            (8528, N'Klasszikus zenész (Ütős)', N'Klasszikus zenész (Ütős)', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 350, NULL, 36, NULL),
            (8529, N'Kortárs-, modern táncos', N'Kortárs-, modern táncos', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 360, NULL, 37, NULL),
            (8530, N'Légtornász', N'Légtornász', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 370, NULL, 38, NULL),
            (8531, N'Magánénekes', N'Magánénekes', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 380, NULL, 39, NULL),
            (8532, N'Mézeskalács díszítő', N'Mézeskalács díszítő', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 390, NULL, 40, NULL),
            (8533, N'Mozgókép és animációtervező', N'Mozgókép és animációtervező', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 400, NULL, 41, NULL),
            (8534, N'Művészeti és médiafotográfus', N'Művészeti és médiafotográfus', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 410, NULL, 42, NULL),
            (8535, N'Művészeti grafikus', N'Művészeti grafikus', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 420, NULL, 43, NULL),
            (8536, N'Nemezkészítő', N'Nemezkészítő', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 430, NULL, 44, NULL),
            (8537, N'Népi bőrműves', N'Népi bőrműves', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 440, NULL, 45, NULL),
            (8538, N'Népi énekes', N'Népi énekes', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 450, NULL, 46, NULL),
            (8539, N'Néptáncos', N'Néptáncos', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 460, NULL, 47, NULL),
            (8540, N'Népzenész (Billentyűs)', N'Népzenész (Billentyűs)', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 470, NULL, 48, NULL),
            (8541, N'Népzenész (Fafúvós)', N'Népzenész (Fafúvós)', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 480, NULL, 49, NULL),
            (8542, N'Népzenész (Húros/vonós)', N'Népzenész (Húros/vonós)', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 490, NULL, 50, NULL),
            (8543, N'Népzenész (Ütős)', N'Népzenész (Ütős)', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 500, NULL, 51, NULL),
            (8544, N'Orgonaépítő', N'Orgonaépítő', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 510, NULL, 52, NULL),
            (8545, N'Ötvös', N'Ötvös', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 520, NULL, 53, NULL),
            (8546, N'Pantomimes', N'Pantomimes', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 530, NULL, 54, NULL),
            (8547, N'Papucskészítő', N'Papucskészítő', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 540, NULL, 55, NULL),
            (8548, N'Pedagógiai asszisztens', N'Pedagógiai asszisztens', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 550, NULL, 56, NULL),
            (8549, N'Pengetős', N'Pengetős', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 560, NULL, 57, NULL),
            (8550, N'Rézfúvós', N'Rézfúvós', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 570, NULL, 58, NULL),
            (8551, N'Szíjgyártó-nyerges', N'Szíjgyártó-nyerges', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 580, NULL, 59, NULL),
            (8552, N'Színházi- és filmszínész', N'Színházi- és filmszínész', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 590, NULL, 60, NULL),
            (8553, N'Színházi táncos', N'Színházi táncos', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 600, NULL, 61, NULL),
            (8554, N'Szobrász', N'Szobrász', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 610, NULL, 62, NULL),
            (8555, N'Szórakoztató zenész (Billentyűs)', N'Szórakoztató zenész (Billentyűs)', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 620, NULL, 63, NULL),
            (8556, N'Szórakoztató zenész (Fafúvós)', N'Szórakoztató zenész (Fafúvós)', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 630, NULL, 64, NULL),
            (8557, N'Szórakoztató zenész (Húros/vonós)', N'Szórakoztató zenész (Húros/vonós)', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 640, NULL, 65, NULL),
            (8558, N'Szórakoztató zenész (Rézfúvós)', N'Szórakoztató zenész (Rézfúvós)', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 650, NULL, 66, NULL),
            (8559, N'Szórakoztató zenész (Ütős)', N'Szórakoztató zenész (Ütős)', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 660, NULL, 67, NULL),
            (8560, N'Szőnyegszövő', N'Szőnyegszövő', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 670, NULL, 68, NULL),
            (8561, N'Takács', N'Takács', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 680, NULL, 69, NULL),
            (8562, N'Társulati artista', N'Társulati artista', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 690, NULL, 70, NULL),
            (8563, N'Textilműves', N'Textilműves', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 700, NULL, 71, NULL),
            (8564, N'Ütős', N'Ütős', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 710, NULL, 72, NULL),
            (8565, N'Üvegműves', N'Üvegműves', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 720, NULL, 73, NULL),
            (8566, N'Vonókészítő', N'Vonókészítő', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 730, NULL, 74, NULL),
            (8567, N'Vonós', N'Vonós', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 740, NULL, 75, NULL),
            (8568, N'Zeneelmélet-szolfézs', N'Zeneelmélet-szolfézs', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 750, NULL, 76, NULL),
            (8569, N'Zeneszerzés', N'Zeneszerzés', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 760, NULL, 77, NULL),
            (8570, N'Zongora', N'Zongora', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 770, NULL, 78, NULL),
            (8571, N'Zsonglőr', N'Zsonglőr', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'T', 780, NULL, 79, NULL),
            (8572, N'Vegyes', N'Vegyes', NULL, NULL, NULL, N'SzakiranyNktTipus', 163, N'T', N'F', 2000, NULL, 80, NULL),
            (8415, N'OpenBoard', N'OpenBoard', NULL, NULL, NULL, N'OraFileTipus', 159, N'F', N'T', 1, NULL, 1, NULL),
            (8410, N'OSA1INT', N'OSA1INT - Intézmény', NULL, NULL, NULL, N'SzirStatTipus', 158, N'F', N'T', 1, NULL, 1, NULL),
            (8411, N'OSA2FELH', N'OSA2FELH - Adatszolgáltatás a feladatellátásról', NULL, NULL, NULL, N'SzirStatTipus', 158, N'F', N'T', 2, NULL, 2, NULL),
            (8412, N'OSA4TANU', N'OSA4TANU - Adatszolgáltatás az egy tanterv szerint tanulók létszámáról', NULL, NULL, NULL, N'SzirStatTipus', 158, N'F', N'T', 3, NULL, 3, NULL),
            (8413, N'OSA5SZAK', N'OSA5SZAK', NULL, NULL, NULL, N'SzirStatTipus', 158, N'F', N'T', 4, NULL, 4, NULL),
            (8414, N'OSA11MUV', N'OSA11MUV', NULL, NULL, NULL, N'SzirStatTipus', 158, N'F', N'T', 5, NULL, 5, NULL),
            (8384, N'Tanár', N'Tanár', NULL, NULL, NULL, N'AlkalmazottMunkaKorTipus', 157, N'T', N'T', 1, NULL, 1, NULL),
            (8385, N'NOKS', N'NOKS', NULL, NULL, NULL, N'AlkalmazottMunkaKorTipus', 157, N'T', N'T', 2, NULL, 2, NULL),
            (8386, N'Egyéb', N'Egyéb', NULL, NULL, NULL, N'AlkalmazottMunkaKorTipus', 157, N'T', N'T', 3, NULL, 3, NULL),
            (7913, N'FejlesztőpedagógusI tevékenység', N'FejlesztőpedagógusI tevékenység', NULL, NULL, NULL, N'EgyebTevekenysegekTipus', 155, N'F', N'F', 100, NULL, 1, NULL),
            (7914, N'Gyakorlati oktatásszervezőI tevékenység', N'Gyakorlati oktatásszervezőI tevékenység', NULL, NULL, NULL, N'EgyebTevekenysegekTipus', 155, N'F', N'F', 110, NULL, 2, NULL),
            (7915, N'Könyvtári tevékenység', N'Könyvtári tevékenység', NULL, NULL, NULL, N'EgyebTevekenysegekTipus', 155, N'F', N'F', 120, NULL, 3, NULL),
            (7916, N'Mentori tevékenység', N'Mentori tevékenység', NULL, NULL, NULL, N'EgyebTevekenysegekTipus', 155, N'F', N'F', 130, NULL, 4, NULL),
            (7917, N'Pályázatírói tevékenység', N'Pályázatírói tevékenység', NULL, NULL, NULL, N'EgyebTevekenysegekTipus', 155, N'F', N'F', 140, NULL, 5, NULL),
            (7918, N'Pszichológusi tevékenység', N'Pszichológusi tevékenység', NULL, NULL, NULL, N'EgyebTevekenysegekTipus', 155, N'F', N'F', 150, NULL, 6, NULL),
            (7919, N'Egyéb tevékenység/tevékenységek', N'Egyéb tevékenység/tevékenységek', NULL, NULL, NULL, N'EgyebTevekenysegekTipus', 155, N'F', N'F', 200, NULL, 7, NULL),
            (8370, N'Diákönkormányzati munkát segítő', N'Diákönkormányzati munkát segítő', NULL, NULL, NULL, N'EgyebTevekenysegekTipus', 155, N'F', N'F', 160, NULL, 8, NULL),
            (8371, N'Munkaközösség-vezető', N'Munkaközösség-vezető', NULL, NULL, NULL, N'EgyebTevekenysegekTipus', 155, N'F', N'F', 170, NULL, 9, NULL),
            (7920, N'Na', N'Na', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 1, NULL, 1, NULL),
            (7921, N'Bányászat és kohászat - 01 (2020)', N'Bányászat és kohászat - 01', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 110, NULL, 2, NULL),
            (7922, N'Egészségügyi technika - 02 (2020)', N'Egészségügyi technika - 02', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 120, NULL, 3, NULL),
            (7923, N'Egészségügy - 03 (2020)', N'Egészségügy - 03', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 130, NULL, 4, NULL),
            (7924, N'Elektronika és elektrotechnika - 04 (2020)', N'Elektronika és elektrotechnika - 04', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 140, NULL, 5, NULL),
            (7925, N'Élelmiszeripar - 05 (2020)', N'Élelmiszeripar - 05', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 150, NULL, 6, NULL),
            (7926, N'Építőipar - 06 (2020)', N'Építőipar - 06', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 160, NULL, 7, NULL),
            (7927, N'Épületgépészet - 07 (2020)', N'Épületgépészet - 07', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 170, NULL, 8, NULL),
            (7928, N'Fa- és bútoripar - 08 (2020)', N'Fa- és bútoripar - 08', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 180, NULL, 9, NULL),
            (7929, N'Gazdálkodás és menedzsment - 09 (2020)', N'Gazdálkodás és menedzsment - 09', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 190, NULL, 10, NULL),
            (7930, N'Gépészet - 10 (2020)', N'Gépészet - 10', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 200, NULL, 11, NULL),
            (7931, N'Honvédelem - 11 (2020)', N'Honvédelem - 11', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 210, NULL, 12, NULL),
            (7932, N'Informatika és távközlés - 12 (2020)', N'Informatika és távközlés - 12', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 220, NULL, 13, NULL),
            (7933, N'Kereskedelem - 13 (2020)', N'Kereskedelem - 13', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 230, NULL, 14, NULL),
            (7934, N'Környezetvédelem és vízügy - 14 (2020)', N'Környezetvédelem és vízügy - 14', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 240, NULL, 15, NULL),
            (7935, N'Közlekedés és szállítmányozás - 15 (2020)', N'Közlekedés és szállítmányozás - 15', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 250, NULL, 16, NULL),
            (7936, N'Kreatív - 16 (2020)', N'Kreatív - 16', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 260, NULL, 17, NULL),
            (7937, N'Mezőgazdaság és erdészet - 17 (2020)', N'Mezőgazdaság és erdészet - 17', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 270, NULL, 18, NULL),
            (7938, N'Rendészet és közszolgálat - 18 (2020)', N'Rendészet és közszolgálat - 18', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 280, NULL, 19, NULL),
            (7939, N'Specializált gép- és járműgyártás - 19 (2020)', N'Specializált gép- és járműgyártás - 19', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 290, NULL, 20, NULL),
            (7940, N'Sport - 20 (2020)', N'Sport - 20', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 300, NULL, 21, NULL),
            (7941, N'Szépészet - 21 (2020)', N'Szépészet - 21', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 310, NULL, 22, NULL),
            (7942, N'Szociális - 22 (2020)', N'Szociális - 22', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 320, NULL, 23, NULL),
            (7943, N'Turizmus-vendéglátás - 23 (2020)', N'Turizmus-vendéglátás - 23', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 330, NULL, 24, NULL),
            (7944, N'Vegyipar - 24 (2020)', N'Vegyipar - 24', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 340, NULL, 25, NULL),
            (7945, N'Több ágazat', N'Több ágazat', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'F', 500, NULL, 26, NULL),
            (9065, N'25. Oktatás', N'25. Oktatás', NULL, NULL, NULL, N'AgazatUjSzktTipus', 149, N'T', N'T', 350, NULL, 27, NULL),
            (7946, N'Na', N'Na', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 1, NULL, 1, NULL),
            (7947, N'Bányaipari technikus - 5 0724 01 01 (2020)', N'Bányaipari technikus - 5 0724 01 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 1100, NULL, 2, NULL),
            (7948, N'Bányaművelő - 4 0724 01 02 (2020)', N'Bányaművelő - 4 0724 01 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 1150, NULL, 3, NULL),
            (7949, N'Fémelőállító - 4 0724 01 03 (2020)', N'Fémelőállító - 4 0724 01 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 1200, NULL, 4, NULL),
            (7950, N'Fluidumkitermelő - 4 0724 01 04 (2020)', N'Fluidumkitermelő - 4 0724 01 04', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 1250, NULL, 5, NULL),
            (7951, N'Fluidumkitermelő technikus - 5 0724 01 05 (2020)', N'Fluidumkitermelő technikus - 5 0724 01 05', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 1300, NULL, 6, NULL),
            (7952, N'Kohász- és öntésztechnikus - 5 0715 01 06 (2020)', N'Kohász- és öntésztechnikus - 5 0715 01 06', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 1350, NULL, 7, NULL),
            (7953, N'Öntvénykészítő - 4 0715 01 07 (2020)', N'Öntvénykészítő - 4 0715 01 07', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 1400, NULL, 8, NULL),
            (7954, N'Fogtechnikus - 5 0911 02 01 (2020)', N'Fogtechnikus - 5 0911 02 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 2100, NULL, 9, NULL),
            (7955, N'Optikaitermék-készítő - 4 0914 02 02 (2020)', N'Optikaitermék-készítő - 4 0914 02 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 2150, NULL, 10, NULL),
            (7956, N'Optikus - 5 0914 02 03 (2020)', N'Optikus - 5 0914 02 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 2200, NULL, 11, NULL),
            (7957, N'Ápoló - 5 0913 03 01 (2020)', N'Ápoló - 5 0913 03 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 3100, NULL, 12, NULL),
            (7958, N'Egészségügyi asszisztens - 5 0913 03 02 (2020)', N'Egészségügyi asszisztens - 5 0913 03 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 3150, NULL, 13, NULL),
            (7959, N'Egészségügyi laboráns - 5 0914 03 03 (2020)', N'Egészségügyi laboráns - 5 0914 03 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 3200, NULL, 14, NULL),
            (7960, N'Gyakorló ápoló - 5 0913 03 04 (2020)', N'Gyakorló ápoló - 5 0913 03 04', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 3250, NULL, 15, NULL),
            (7961, N'Klinikai laboratóriumi szakasszisztens - 5 0914 03 05 (2020)', N'Klinikai laboratóriumi szakasszisztens - 5 0914 03 05', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 3300, NULL, 16, NULL),
            (7962, N'Ortopédiai műszerész - 5 0915 03 06 (2020)', N'Ortopédiai műszerész - 5 0915 03 06', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 3350, NULL, 17, NULL),
            (7963, N'Perioperatív szakasszisztens - 5 0910 03 07 (2020)', N'Perioperatív szakasszisztens - 5 0910 03 07', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 3400, NULL, 18, NULL),
            (7964, N'Radiográfiai szakasszisztens - 5 0914 03 08 (2020)', N'Radiográfiai szakasszisztens - 5 0914 03 08', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 3450, NULL, 19, NULL),
            (7965, N'Rehabilitációs terapeuta - 5 0923 03 09 (2020)', N'Rehabilitációs terapeuta - 5 0923 03 09', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 3500, NULL, 20, NULL),
            (7966, N'Szövettani szakasszisztens - 5 0914 03 10 (2020)', N'Szövettani szakasszisztens - 5 0914 03 10', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 3550, NULL, 21, NULL),
            (7967, N'Automatikai technikus - 5 0714 04 01 (2020)', N'Automatikai technikus - 5 0714 04 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 4100, NULL, 22, NULL),
            (7968, N'Elektronikai műszerész - 4 0713 04 02 (2020)', N'Elektronikai műszerész - 4 0713 04 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 4150, NULL, 23, NULL),
            (7969, N'Elektronikai technikus - 5 0714 04 03 (2020)', N'Elektronikai technikus - 5 0714 04 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 4200, NULL, 24, NULL),
            (7970, N'Erősáramú elektrotechnikus - 5 0713 04 04 (2020)', N'Erősáramú elektrotechnikus - 5 0713 04 04', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 4250, NULL, 25, NULL),
            (7971, N'Ipari informatikai technikus - 5 0714 04 05 (2020)', N'Ipari informatikai technikus - 5 0714 04 05', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 4300, NULL, 26, NULL),
            (7972, N'Közlekedésautomatikai technikus - 5 0714 04 06 (2020)', N'Közlekedésautomatikai technikus - 5 0714 04 06', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 4350, NULL, 27, NULL),
            (7973, N'Villanyszerelő - 4 0713 04 07 (2020)', N'Villanyszerelő - 4 0713 04 07', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 4400, NULL, 28, NULL),
            (7974, N'Bor- és pezsgőgyártó technikus - 5 0721 05 01 (2020)', N'Bor- és pezsgőgyártó technikus - 5 0721 05 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 5100, NULL, 29, NULL),
            (7975, N'Édességkészítő - 4 0721 05 02 (2020)', N'Édességkészítő - 4 0721 05 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 5150, NULL, 30, NULL),
            (7976, N'Élelmiszer-ellenőrzési technikus - 5 0721 05 03 (2020)', N'Élelmiszer-ellenőrzési technikus - 5 0721 05 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 5200, NULL, 31, NULL),
            (7977, N'Élelmiszeripari gépésztechnikus - 5 0721 05 04 (2020)', N'Élelmiszeripari gépésztechnikus - 5 0721 05 04', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 5250, NULL, 32, NULL),
            (7978, N'Élelmiszeripari gépkezelő - 4 0721 05 05 (2020)', N'Élelmiszeripari gépkezelő - 4 0721 05 05', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 5300, NULL, 33, NULL),
            (7979, N'Erjedés- és üdítőital-ipari technikus - 5 0721 05 06 (2020)', N'Erjedés- és üdítőital-ipari technikus - 5 0721 05 06', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 5350, NULL, 34, NULL),
            (7980, N'Erjedés- és üdítőital-ipari termékkészítő - 4 0721 05 07 (2020)', N'Erjedés- és üdítőital-ipari termékkészítő - 4 0721 05 07', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 5400, NULL, 35, NULL),
            (7981, N'Hentes és húskészítmény-készítő - 4 0721 05 08 (2020)', N'Hentes és húskészítmény-készítő - 4 0721 05 08', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 5450, NULL, 36, NULL),
            (7982, N'Húsipari technikus - 5 0721 05 09 (2020)', N'Húsipari technikus - 5 0721 05 09', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 5500, NULL, 37, NULL),
            (7983, N'Kistermelői élelmiszer-előállító - 4 0721 05 10 (2020)', N'Kistermelői élelmiszer-előállító - 4 0721 05 10', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 5550, NULL, 38, NULL),
            (7984, N'Pék - 4 0721 05 11 (2020)', N'Pék - 4 0721 05 11', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 5600, NULL, 39, NULL),
            (7985, N'Pék-cukrász - 4 0721 05 12 (2020)', N'Pék-cukrász - 4 0721 05 12', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 5650, NULL, 40, NULL),
            (7986, N'Sütő- és cukrászipari technikus - 5 0721 05 13 (2020)', N'Sütő- és cukrászipari technikus - 5 0721 05 13', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 5700, NULL, 41, NULL),
            (7987, N'Szőlész-borász - 4 0721 05 14 (2020)', N'Szőlész-borász - 4 0721 05 14', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 5750, NULL, 42, NULL),
            (7988, N'Tartósítóipari technikus - 5 0721 05 15 (2020)', N'Tartósítóipari technikus - 5 0721 05 15', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 5800, NULL, 43, NULL),
            (7989, N'Tartósítóipari termék-készítő - 4 0721 05 16 (2020)', N'Tartósítóipari termék-készítő - 4 0721 05 16', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 5850, NULL, 44, NULL),
            (7990, N'Tejipari technikus - 5 0721 05 17 (2020)', N'Tejipari technikus - 5 0721 05 17', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 5900, NULL, 45, NULL),
            (7991, N'Tejtermékkészítő - 4 0721 05 18 (2020)', N'Tejtermékkészítő - 4 0721 05 18', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 5950, NULL, 46, NULL),
            (7992, N'Ács - 4 0732 06 01 (2020)', N'Ács - 4 0732 06 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 6100, NULL, 47, NULL),
            (7993, N'Bádogos - 4 0732 06 02 (2020)', N'Bádogos - 4 0732 06 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 6150, NULL, 48, NULL),
            (7994, N'Burkoló - 4 0732 06 03 (2020)', N'Burkoló - 4 0732 06 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 6200, NULL, 49, NULL),
            (7995, N'Épületszobrász és műköves - 4 0732 06 04 (2020)', N'Épületszobrász és műköves - 4 0732 06 04', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 6250, NULL, 50, NULL),
            (7996, N'Festő, mázoló, tapétázó - 4 0732 06 05 (2020)', N'Festő, mázoló, tapétázó - 4 0732 06 05', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 6300, NULL, 51, NULL),
            (7997, N'Hídépítő és -fenntartó technikus - 5 0732 06 06 (2020)', N'Hídépítő és -fenntartó technikus - 5 0732 06 06', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 6350, NULL, 52, NULL),
            (7998, N'Kőfaragó - 4 0732 06 07 (2020)', N'Kőfaragó - 4 0732 06 07', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 6400, NULL, 53, NULL),
            (7999, N'Kőműves - 4 0732 06 08 (2020)', N'Kőműves - 4 0732 06 08', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 6450, NULL, 54, NULL),
            (8000, N'Magasépítő technikus - 5 0732 06 09 (2020)', N'Magasépítő technikus - 5 0732 06 09', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 6500, NULL, 55, NULL),
            (8001, N'Mélyépítő technikus - 5 0732 06 10 (2020)', N'Mélyépítő technikus - 5 0732 06 10', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 6550, NULL, 56, NULL),
            (8002, N'Szárazépítő - 4 0732 06 11 (2020)', N'Szárazépítő - 4 0732 06 11', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 6600, NULL, 57, NULL),
            (8003, N'Szerkezetépítő és -szerelő - 4 0732 06 12 (2020)', N'Szerkezetépítő és -szerelő - 4 0732 06 12', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 6650, NULL, 58, NULL),
            (8004, N'Szigetelő - 4 0732 06 13 (2020)', N'Szigetelő - 4 0732 06 13', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 6700, NULL, 59, NULL),
            (8005, N'Tetőfedő - 4 0732 06 14 (2020)', N'Tetőfedő - 4 0732 06 14', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 6750, NULL, 60, NULL),
            (8006, N'Útépítő és útfenntartó - 4 0732 06 15 (2020)', N'Útépítő és útfenntartó - 4 0732 06 15', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 6800, NULL, 61, NULL),
            (8007, N'Útépítő, vasútépítő és -fenntartó technikus - 5 0732 06 16 (2020)', N'Útépítő, vasútépítő és -fenntartó technikus - 5 0732 06 16', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 6850, NULL, 62, NULL),
            (8008, N'Épületgépész technikus - 5 0732 07 01 (2020)', N'Épületgépész technikus - 5 0732 07 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 7100, NULL, 63, NULL),
            (8009, N'Hűtő- és szellőzésrendszer-szerelő - 4 0732 07 02 (2020)', N'Hűtő- és szellőzésrendszer-szerelő - 4 0732 07 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 7150, NULL, 64, NULL),
            (8010, N'Központifűtés- és gázhálózatrendszer-szerelő - 4 0732 07 03 (2020)', N'Központifűtés- és gázhálózatrendszer-szerelő - 4 0732 07 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 7200, NULL, 65, NULL),
            (8011, N'Víz- és csatornarendszer-szerelő - 4 0732 07 04 (2020)', N'Víz- és csatornarendszer-szerelő - 4 0732 07 04', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 7250, NULL, 66, NULL),
            (8012, N'Asztalos - 4 0722 08 01 (2020)', N'Asztalos - 4 0722 08 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 8100, NULL, 67, NULL),
            (8013, N'Faipari technikus - 5 0722 08 02 (2020)', N'Faipari technikus - 5 0722 08 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 8150, NULL, 68, NULL),
            (8014, N'Kárpitos - 4 0723 08 03 (2020)', N'Kárpitos - 4 0723 08 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 8200, NULL, 69, NULL),
            (8015, N'Pénzügyi-számviteli ügyintéző - 5 0411 09 01 (2020)', N'Pénzügyi-számviteli ügyintéző - 5 0411 09 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 9100, NULL, 70, NULL),
            (8016, N'Vállalkozási ügyviteli ügyintéző - 5 0411 09 02 (2020)', N'Vállalkozási ügyviteli ügyintéző - 5 0411 09 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 9150, NULL, 71, NULL),
            (8017, N'CNC-programozó - 4 0715 10 01 (2020)', N'CNC-programozó - 4 0715 10 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 10100, NULL, 72, NULL),
            (8018, N'Építő-, szállító- és munkagép-szerelő - 4 0715 10 02 (2020)', N'Építő-, szállító- és munkagép-szerelő - 4 0715 10 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 10150, NULL, 73, NULL),
            (8019, N'Épület- és szerkezetlakatos - 4 0732 10 03 (2020)', N'Épület- és szerkezetlakatos - 4 0732 10 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 10200, NULL, 74, NULL),
            (8020, N'Finommechanikai műszerész - 4 0715 10 04 (2020)', N'Finommechanikai műszerész - 4 0715 10 04', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 10250, NULL, 75, NULL),
            (8021, N'Gépész technikus - 5 0715 10 05 (2020)', N'Gépész technikus - 5 0715 10 05', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 10300, NULL, 76, NULL),
            (8022, N'Gépgyártás-technológiai technikus - 5 0715 10 06 (2020)', N'Gépgyártás-technológiai technikus - 5 0715 10 06', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 10350, NULL, 77, NULL),
            (8023, N'Gépi és CNC forgácsoló - 4 0715 10 07 (2020)', N'Gépi és CNC forgácsoló - 4 0715 10 07', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 10400, NULL, 78, NULL),
            (8024, N'Hegesztő - 4 0715 10 08 (2020)', N'Hegesztő - 4 0715 10 08', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 10450, NULL, 79, NULL),
            (8025, N'Ipari gépész - 4 0715 10 09 (2020)', N'Ipari gépész - 4 0715 10 09', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 10500, NULL, 80, NULL),
            (8026, N'Légijármű-műszerész technikus - 5 0716 10 10 (2020)', N'Légijármű-műszerész technikus - 5 0716 10 10', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 10550, NULL, 81, NULL),
            (8027, N'Légijármű-szerelő technikus - 5 0716 10 11 (2020)', N'Légijármű-szerelő technikus - 5 0716 10 11', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 10600, NULL, 82, NULL),
            (8028, N'Szerszám- és készülékgyártó - 4 0715 10 12 (2020)', N'Szerszám- és készülékgyártó - 4 0715 10 12', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 10650, NULL, 83, NULL),
            (8029, N'Vasútijármű-szerelő technikus - 5 0716 10 13 (2020)', N'Vasútijármű-szerelő technikus - 5 0716 10 13', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 10700, NULL, 84, NULL),
            (8030, N'Fegyvergyártó szaktechnikus - 5 1031 11 01 (2020)', N'Fegyvergyártó szaktechnikus - 5 1031 11 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 11100, NULL, 85, NULL),
            (8031, N'Fegyverműszerész technikus - 5 1031 11 02 (2020)', N'Fegyverműszerész technikus - 5 1031 11 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 11150, NULL, 86, NULL),
            (8032, N'Fegyveroptikai szaktechnikus - 5 1031 11 03 (2020)', N'Fegyveroptikai szaktechnikus - 5 1031 11 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 11200, NULL, 87, NULL),
            (8033, N'Honvéd altiszt - 5 1031 11 04 (2020)', N'Honvéd altiszt - 5 1031 11 04', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 11250, NULL, 88, NULL),
            (8034, N'Honvéd kadét - 5 1031 11 05 (2020)', N'Honvéd kadét - 5 1031 11 05', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 11300, NULL, 89, NULL),
            (8035, N'Infokommunikációs hálózatépítő és -üzemeltető technikus - 5 0612 12 01 (2020)', N'Infokommunikációs hálózatépítő és -üzemeltető technikus - 5 0612 12 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 12100, NULL, 90, NULL),
            (8036, N'Informatikai rendszer- és alkalmazás-üzemeltető technikus - 5 0612 12 02 (2020)', N'Informatikai rendszer- és alkalmazás-üzemeltető technikus - 5 0612 12 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 12150, NULL, 91, NULL),
            (8037, N'Szoftverfejlesztő és -tesztelő - 5 0613 12 03 (2020)', N'Szoftverfejlesztő és -tesztelő - 5 0613 12 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 12200, NULL, 92, NULL),
            (8038, N'Távközlési technikus - 5 0714 12 04 (2020)', N'Távközlési technikus - 5 0714 12 04', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 12250, NULL, 93, NULL),
            (8039, N'Idegen nyelvű ipari és kereskedelmi technikus - 5 0417 13 01 (2020)', N'Idegen nyelvű ipari és kereskedelmi technikus - 5 0417 13 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 13100, NULL, 94, NULL),
            (8040, N'Kereskedelmi értékesítő - 4 0416 13 02 (2020)', N'Kereskedelmi értékesítő - 4 0416 13 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 13150, NULL, 95, NULL),
            (8041, N'Kereskedő és webáruházi technikus - 5 0416 13 03 (2020)', N'Kereskedő és webáruházi technikus - 5 0416 13 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 13200, NULL, 96, NULL),
            (8042, N'Hulladékfeldolgozó munkatárs - 4 0712 14 01 (2020)', N'Hulladékfeldolgozó munkatárs - 4 0712 14 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 14100, NULL, 97, NULL),
            (8043, N'Környezetvédelmi technikus - 5 0712 14 02 (2020)', N'Környezetvédelmi technikus - 5 0712 14 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 14150, NULL, 98, NULL),
            (8044, N'Vízügyi munkatárs - 4 1021 14 03 (2020)', N'Vízügyi munkatárs - 4 1021 14 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 14200, NULL, 99, NULL),
            (8045, N'Vízügyi technikus - 5 1021 14 04 (2020)', N'Vízügyi technikus - 5 1021 14 04', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 14250, NULL, 100, NULL),
            (8046, N'Hajózási technikus - 5 1041 15 01 (2020)', N'Hajózási technikus - 5 1041 15 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 15100, NULL, 101, NULL),
            (8047, N'Képesített hajós - 4 1041 15 02 (2020)', N'Képesített hajós - 4 1041 15 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 15150, NULL, 102, NULL),
            (8048, N'Kishajóépítő és -karbantartó - 4 0722 15 03 (2020)', N'Kishajóépítő és -karbantartó - 4 0722 15 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 15200, NULL, 103, NULL),
            (8049, N'Kocsivizsgáló technikus - 5 0716 15 04 (2020)', N'Kocsivizsgáló technikus - 5 0716 15 04', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 15250, NULL, 104, NULL),
            (8050, N'Közlekedésüzemvitel-ellátó technikus - 5 1041 15 05 (2020)', N'Közlekedésüzemvitel-ellátó technikus - 5 1041 15 05', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 15300, NULL, 105, NULL),
            (8051, N'Logisztikai technikus - 5 1041 15 06 (2020)', N'Logisztikai technikus - 5 1041 15 06', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 15350, NULL, 106, NULL),
            (8052, N'Postai üzleti ügyintéző - 5 1041 15 07 (2020)', N'Postai üzleti ügyintéző - 5 1041 15 07', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 15400, NULL, 107, NULL),
            (8053, N'Vasútforgalmi szolgálattevő technikus - 5 1041 15 08 (2020)', N'Vasútforgalmi szolgálattevő technikus - 5 1041 15 08', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 15450, NULL, 108, NULL),
            (8054, N'Bőrtermékkészítő - 4 0723 16 01 (2020)', N'Bőrtermékkészítő - 4 0723 16 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 16100, NULL, 109, NULL),
            (8055, N'Divat-, jelmez- és díszlettervező - 5 0212 16 02 (2020)', N'Divat-, jelmez- és díszlettervező - 5 0212 16 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 16150, NULL, 110, NULL),
            (8056, N'Divatszabó - 4 0723 16 03 (2020)', N'Divatszabó - 4 0723 16 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 16200, NULL, 111, NULL),
            (8057, N'Kerámia- és porcelánkészítő - 4 0214 16 04 (2020)', N'Kerámia- és porcelánkészítő - 4 0214 16 04', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 16250, NULL, 112, NULL),
            (8058, N'Könnyűipari technikus - 5 0723 16 05 (2020)', N'Könnyűipari technikus - 5 0723 16 05', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 16300, NULL, 113, NULL),
            (8059, N'Textilgyártó - 4 0723 16 06 (2020)', N'Textilgyártó - 4 0723 16 06', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 16350, NULL, 114, NULL),
            (8060, N'Dekoratőr - 5 0212 16 07 (2020)', N'Dekoratőr - 5 0212 16 07', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 16400, NULL, 115, NULL),
            (8061, N'Fotográfus - 5 0213 16 08 (2020)', N'Fotográfus - 5 0213 16 08', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 16450, NULL, 116, NULL),
            (8062, N'Grafikus - 5 0213 16 09 (2020)', N'Grafikus - 5 0213 16 09', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 16500, NULL, 117, NULL),
            (8063, N'Mozgókép- és animációkészítő - 5 0211 16 10 (2020)', N'Mozgókép- és animációkészítő - 5 0211 16 10', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 16550, NULL, 118, NULL),
            (8064, N'Nyomdaipari technikus - 5 0211 16 11 (2020)', N'Nyomdaipari technikus - 5 0211 16 11', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 16600, NULL, 119, NULL),
            (8065, N'Nyomdász - 4 0211 16 12 (2020)', N'Nyomdász - 4 0211 16 12', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 16650, NULL, 120, NULL),
            (8066, N'Hangtechnikus - 5 0211 16 13 (2020)', N'Hangtechnikus - 5 0211 16 13', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 16700, NULL, 121, NULL),
            (8067, N'Színház- és rendezvénytechnikus - 5 0413 16 14 (2020)', N'Színház- és rendezvénytechnikus - 5 0413 16 14', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 16750, NULL, 122, NULL),
            (8068, N'Erdőművelő-fakitermelő - 4 0821 17 01 (2020)', N'Erdőművelő-fakitermelő - 4 0821 17 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 17100, NULL, 123, NULL),
            (8069, N'Erdésztechnikus - 5 0821 17 02 (2020)', N'Erdésztechnikus - 5 0821 17 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 17150, NULL, 124, NULL),
            (8070, N'Földmérő, földügyi és térinformatikai technikus - 5 0810 17 03 (2020)', N'Földmérő, földügyi és térinformatikai technikus - 5 0810 17 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 17200, NULL, 125, NULL),
            (8071, N'Gazda - 4 0811 17 04 (2020)', N'Gazda - 4 0811 17 04', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 17250, NULL, 126, NULL),
            (8072, N'Kertész - 4 0812 17 05 (2020)', N'Kertész - 4 0812 17 05', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 17300, NULL, 127, NULL),
            (8073, N'Kertésztechnikus - 5 0812 17 06 (2020)', N'Kertésztechnikus - 5 0812 17 06', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 17350, NULL, 128, NULL),
            (8074, N'Mezőgazdasági gépész - 4 0810 17 07 (2020)', N'Mezőgazdasági gépész - 4 0810 17 07', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 17400, NULL, 129, NULL),
            (8075, N'Mezőgazdasági gépésztechnikus - 5 0810 17 08 (2020)', N'Mezőgazdasági gépésztechnikus - 5 0810 17 08', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 17450, NULL, 130, NULL),
            (8076, N'Mezőgazdasági technikus - 5 0811 17 09 (2020)', N'Mezőgazdasági technikus - 5 0811 17 09', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 17500, NULL, 131, NULL),
            (8077, N'Közszolgálati technikus - 5 0413 18 01 (2020)', N'Közszolgálati technikus - 5 0413 18 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 18100, NULL, 132, NULL),
            (8078, N'Rendészeti őr - 4 1032 18 02 (2020)', N'Rendészeti őr - 4 1032 18 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 18150, NULL, 133, NULL),
            (8079, N'Rendőr tiszthelyettes - 5 1032 18 03 (2020)', N'Rendőr tiszthelyettes - 5 1032 18 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 18200, NULL, 134, NULL),
            (8080, N'Alternatív járműhajtási technikus - 5 0716 19 01 (2020)', N'Alternatív járműhajtási technikus - 5 0716 19 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 19100, NULL, 135, NULL),
            (8081, N'Autógyártó - 4 0716 19 02 (2020)', N'Autógyártó - 4 0716 19 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 19150, NULL, 136, NULL),
            (8082, N'Fémszerkezetfelület-bevonó - 4 0715 19 03 (2020)', N'Fémszerkezetfelület-bevonó - 4 0715 19 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 19200, NULL, 137, NULL),
            (8083, N'Gépjármű-mechatronikai technikus - 5 0716 19 04 (2020)', N'Gépjármű-mechatronikai technikus - 5 0716 19 04', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 19250, NULL, 138, NULL),
            (8084, N'Gépjármű mechatronikus - 4 0716 19 05 (2020)', N'Gépjármű mechatronikus - 4 0716 19 05', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 19300, NULL, 139, NULL),
            (8085, N'Gyártósori gépbeállító - 4 0715 19 06 (2020)', N'Gyártósori gépbeállító - 4 0715 19 06', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 19350, NULL, 140, NULL),
            (8086, N'Ipari szerviztechnikus - 5 0715 19 07 (2020)', N'Ipari szerviztechnikus - 5 0715 19 07', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 19400, NULL, 141, NULL),
            (8087, N'Járműfényező - 4 0716 19 08 (2020)', N'Járműfényező - 4 0716 19 08', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 19450, NULL, 142, NULL),
            (8088, N'Járműipari karbantartó technikus - 5 0714 19 09 (2020)', N'Járműipari karbantartó technikus - 5 0714 19 09', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 19500, NULL, 143, NULL),
            (8089, N'Járműkarosszéria-előkészítő, felületbevonó - 4 0716 19 10 (2020)', N'Járműkarosszéria-előkészítő, felületbevonó - 4 0716 19 10', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 19550, NULL, 144, NULL),
            (8090, N'Karosszérialakatos - 4 0716 19 11 (2020)', N'Karosszérialakatos - 4 0716 19 11', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 19600, NULL, 145, NULL),
            (8091, N'Mechatronikai technikus - 5 0714 19 12 (2020)', N'Mechatronikai technikus - 5 0714 19 12', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 19650, NULL, 146, NULL),
            (8092, N'Mechatronikus karbantartó - 4 0714 19 13 (2020)', N'Mechatronikus karbantartó - 4 0714 19 13', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 19700, NULL, 147, NULL),
            (8093, N'Fitness-wellness instruktor - 5 1014 20 01 (2020)', N'Fitness-wellness instruktor - 5 1014 20 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 20100, NULL, 148, NULL),
            (8094, N'Sportedző (a sportág megjelölésével) - sportszervező - 5 1014 20 02 (2020)', N'Sportedző (a sportág megjelölésével) - sportszervező - 5 1014 20 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 20150, NULL, 149, NULL),
            (8095, N'Fodrász - 5 1012 21 01 (2020)', N'Fodrász - 5 1012 21 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 21100, NULL, 150, NULL),
            (8096, N'Kéz- és lábápoló technikus - 5 1012 21 02 (2020)', N'Kéz- és lábápoló technikus - 5 1012 21 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 21150, NULL, 151, NULL),
            (8097, N'Kozmetikus technikus - 5 1012 21 03 (2020)', N'Kozmetikus technikus - 5 1012 21 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 21200, NULL, 152, NULL),
            (8098, N'Gyermek- és ifjúsági felügyelő - 4 0922 22 01 (2020)', N'Gyermek- és ifjúsági felügyelő - 4 0922 22 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 22100, NULL, 153, NULL),
            (8099, N'Kisgyermekgondozó, -nevelő - 5 0922 22 02 (2020)', N'Kisgyermekgondozó, -nevelő - 5 0922 22 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 22150, NULL, 154, NULL),
            (8100, N'Szociális ápoló és gondozó - 4 0923 22 03 (2020)', N'Szociális ápoló és gondozó - 4 0923 22 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 22200, NULL, 155, NULL),
            (8101, N'Szociális és gyermekvédelmi szakasszisztens - 5 0923 22 04 (2020)', N'Szociális és gyermekvédelmi szakasszisztens - 5 0923 22 04', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 22250, NULL, 156, NULL),
            (8102, N'Szociális és mentálhigiénés szakgondozó - 5 0923 22 05 (2020)', N'Szociális és mentálhigiénés szakgondozó - 5 0923 22 05', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 22300, NULL, 157, NULL),
            (8103, N'Szociális és rehabilitációs szakgondozó - 5 0923 22 06 (2020)', N'Szociális és rehabilitációs szakgondozó - 5 0923 22 06', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 22350, NULL, 158, NULL),
            (8104, N'Cukrász - 4 1013 23 01 (2020)', N'Cukrász - 4 1013 23 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 23100, NULL, 159, NULL),
            (8105, N'Cukrász szaktechnikus - 5 1013 23 02 (2020)', N'Cukrász szaktechnikus - 5 1013 23 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 23150, NULL, 160, NULL),
            (8106, N'Panziós-fogadós - 4 1013 23 03 (2020)', N'Panziós-fogadós - 4 1013 23 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 23200, NULL, 161, NULL),
            (8107, N'Pincér - vendégtéri szakember - 4 1013 23 04 (2020)', N'Pincér - vendégtéri szakember - 4 1013 23 04', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 23250, NULL, 162, NULL),
            (8108, N'Szakács - 4 1013 23 05 (2020)', N'Szakács - 4 1013 23 05', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 23300, NULL, 163, NULL),
            (8109, N'Szakács szaktechnikus - 5 1013 23 06 (2020)', N'Szakács szaktechnikus - 5 1013 23 06', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 23350, NULL, 164, NULL),
            (8110, N'Turisztikai technikus - 5 1015 23 07 (2020)', N'Turisztikai technikus - 5 1015 23 07', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 23400, NULL, 165, NULL),
            (8111, N'Vendégtéri szaktechnikus - 5 1013 23 08 (2020)', N'Vendégtéri szaktechnikus - 5 1013 23 08', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 23450, NULL, 166, NULL),
            (8112, N'Abroncsgyártó - 4 0722 24 01 (2020)', N'Abroncsgyártó - 4 0722 24 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 24100, NULL, 167, NULL),
            (8113, N'Gumiipari technikus - 5 0722 24 02 (2020)', N'Gumiipari technikus - 5 0722 24 02', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 24150, NULL, 168, NULL),
            (8114, N'Gyógyszerkészítmény-gyártó - 4 0711 24 03 (2020)', N'Gyógyszerkészítmény-gyártó - 4 0711 24 03', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 24200, NULL, 169, NULL),
            (8115, N'Műanyag-feldolgozó - 4 0722 24 04 (2020)', N'Műanyag-feldolgozó - 4 0722 24 04', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 24250, NULL, 170, NULL),
            (8116, N'Műanyag-feldolgozó technikus - 5 0722 24 05 (2020)', N'Műanyag-feldolgozó technikus - 5 0722 24 05', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 24300, NULL, 171, NULL),
            (8117, N'Papírgyártó és -feldolgozó, csomagolószer-gyártó - 4 0722 24 06 (2020)', N'Papírgyártó és -feldolgozó, csomagolószer-gyártó - 4 0722 24 06', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 24350, NULL, 172, NULL),
            (8118, N'Papírgyártó és -feldolgozó, csomagolószer-gyártó technikus - 5 0722 24 07 (2020)', N'Papírgyártó és -feldolgozó, csomagolószer-gyártó technikus - 5 0722 24 07', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 24400, NULL, 173, NULL),
            (8119, N'Vegyész technikus - 5 0711 24 08 (2020)', N'Vegyész technikus - 5 0711 24 08', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 24450, NULL, 174, NULL),
            (8120, N'Vegyipari rendszerkezelő - 4 0711 24 09 (2020)', N'Vegyipari rendszerkezelő - 4 0711 24 09', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 24500, NULL, 175, NULL),
            (8121, N'Több szakma', N'Több szakma', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'F', 30000, NULL, 176, NULL),
            (8389, N'Általános ápoló - 5 0913 03 01 (2020)', N'Általános ápoló - 5 0913 03 01', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 3050, NULL, 177, NULL),
            (8390, N'Mentőápoló - 5 0913 03 11 (2020)', N'Mentőápoló - 5 0913 03 11', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 3400, NULL, 178, NULL),
            (9066, N'Csecsemő- és gyermekápoló (5 0913 03 04)', N'Csecsemő- és gyermekápoló (5 0913 03 04)', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 24550, NULL, 179, NULL),
            (9067, N'Alapápolási munkatárs (4 0913 03 12)', N'Alapápolási munkatárs (4 0913 03 12)', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 24600, NULL, 180, NULL),
            (9068, N'Hibrid és elektromos gépjármű-mechatronikus (4 0716 19 14)', N'Hibrid és elektromos gépjármű-mechatronikus (4 0716 19 14)', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 24650, NULL, 181, NULL),
            (9069, N'Oktatási szakasszisztens (5 0188 25 01)', N'Oktatási szakasszisztens (5 0188 25 01)', NULL, NULL, NULL, N'SzakmaTipus', 150, N'T', N'T', 24700, NULL, 182, NULL),
            (8122, N'Na', N'Na', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 1, NULL, 1, NULL),
            (8123, N'Kohász (2020)', N'Kohász', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 1351, NULL, 2, NULL),
            (8124, N'Öntész (2020)', N'Öntész', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 1352, NULL, 3, NULL),
            (8125, N'Könnyűfém (2020)', N'Könnyűfém', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 1401, NULL, 4, NULL),
            (8126, N'Vas és acél (2020)', N'Vas és acél', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 1402, NULL, 5, NULL),
            (8127, N'Alapellátási közösségi szakápoló (2020)', N'Alapellátási közösségi szakápoló', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3101, NULL, 6, NULL),
            (8128, N'Diabetológiai szakápoló és edukátor (2020)', N'Diabetológiai szakápoló és edukátor', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3102, NULL, 7, NULL),
            (8129, N'Epidemiológiai szakápoló (2020)', N'Epidemiológiai szakápoló', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3103, NULL, 8, NULL),
            (8130, N'Felnőtt intenzív szakápoló (2020)', N'Felnőtt intenzív szakápoló', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3104, NULL, 9, NULL),
            (8131, N'Foglalkozás-egészségügyi szakápoló (2020)', N'Foglalkozás-egészségügyi szakápoló', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3105, NULL, 10, NULL),
            (8132, N'Geriátriai és krónikusbeteg szakápoló (2020)', N'Geriátriai és krónikusbeteg szakápoló', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3106, NULL, 11, NULL),
            (8133, N'Gyermekintenzív szakápoló (2020)', N'Gyermekintenzív szakápoló', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3107, NULL, 12, NULL),
            (8134, N'Hospice szakápoló (2020)', N'Hospice szakápoló', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3108, NULL, 13, NULL),
            (8135, N'Légzőszervi szakápoló (2020)', N'Légzőszervi szakápoló', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3109, NULL, 14, NULL),
            (8136, N'Nefrológiai szakápoló (2020)', N'Nefrológiai szakápoló', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3110, NULL, 15, NULL),
            (8137, N'Onkológiai szakápoló (2020)', N'Onkológiai szakápoló', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3111, NULL, 16, NULL),
            (8138, N'Pszichiátriai szakápoló és gyógyfoglalkoztató (2020)', N'Pszichiátriai szakápoló és gyógyfoglalkoztató', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3112, NULL, 17, NULL),
            (8139, N'Sürgősségi szakápoló (2020)', N'Sürgősségi szakápoló', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3113, NULL, 18, NULL),
            (8140, N'Audiológiai asszisztens és hallásakusztikus (2020)', N'Audiológiai asszisztens és hallásakusztikus', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3151, NULL, 19, NULL),
            (8141, N'Endoszkópos asszisztens (2020)', N'Endoszkópos asszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3152, NULL, 20, NULL),
            (8142, N'Fogászati asszisztens (2020)', N'Fogászati asszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3153, NULL, 21, NULL),
            (8143, N'Gyógyszertári asszisztens (2020)', N'Gyógyszertári asszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3154, NULL, 22, NULL),
            (8144, N'Kardiológiai és angiológiai asszisztens (2020)', N'Kardiológiai és angiológiai asszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3155, NULL, 23, NULL),
            (8145, N'Klinikai neurofiziológiai asszisztens (2020)', N'Klinikai neurofiziológiai asszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3156, NULL, 24, NULL),
            (8146, N'Perioperatív asszisztens (2020)', N'Perioperatív asszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3157, NULL, 25, NULL),
            (8147, N'Radiográfiai asszisztens (2020)', N'Radiográfiai asszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3158, NULL, 26, NULL),
            (8148, N'Klinikai laboratóriumi asszisztens (2020)', N'Klinikai laboratóriumi asszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3201, NULL, 27, NULL),
            (8149, N'Szövettani asszisztens (2020)', N'Szövettani asszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3202, NULL, 28, NULL),
            (8150, N'Csecsemő- és gyermekápoló (2020)', N'Csecsemő- és gyermekápoló', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3251, NULL, 29, NULL),
            (8151, N'Felnőttbeteg-ápoló (2020)', N'Felnőttbeteg-ápoló', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3252, NULL, 30, NULL),
            (8152, N'Mentőápoló (2020)', N'Mentőápoló', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3253, NULL, 31, NULL),
            (8153, N'Hematológiai és transzfuziológiai szakasszisztens (2020)', N'Hematológiai és transzfuziológiai szakasszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3301, NULL, 32, NULL),
            (8154, N'Kémiai laboratóriumi szakasszisztens (2020)', N'Kémiai laboratóriumi szakasszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3302, NULL, 33, NULL),
            (8155, N'Mikrobiológiai szakasszisztens (2020)', N'Mikrobiológiai szakasszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3303, NULL, 34, NULL),
            (8156, N'Aneszteziológiai szakasszisztens (2020)', N'Aneszteziológiai szakasszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3401, NULL, 35, NULL),
            (8157, N'Intervenciós szakasszisztens (2020)', N'Intervenciós szakasszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3454, NULL, 36, NULL),
            (8158, N'Műtéti szakasszisztens (2020)', N'Műtéti szakasszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3403, NULL, 37, NULL),
            (8159, N'CT/MR szakasszisztens (2020)', N'CT/MR szakasszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3451, NULL, 38, NULL),
            (8160, N'Nukleáris szakasszisztens (2020)', N'Nukleáris szakasszisztens (2020)', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3453, NULL, 39, NULL),
            (8161, N'Sugárterápiás szakasszisztens (2020)', N'Sugárterápiás szakasszisztens (2020)', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3455, NULL, 40, NULL),
            (8162, N'Ergoterapeuta (2020)', N'Ergoterapeuta', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3501, NULL, 41, NULL),
            (8163, N'Fizioterápiás asszisztens (2020)', N'Fizioterápiás asszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3502, NULL, 42, NULL),
            (8164, N'Gyógymasszőr (2020)', N'Gyógymasszőr', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3503, NULL, 43, NULL),
            (8165, N'Citológiai szakasszisztens (2020)', N'Citológiai szakasszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3551, NULL, 44, NULL),
            (8166, N'Immunhisztokémiai, hisztokémiai és molekuláris biológiai szakasszisztens (2020)', N'Immunhisztokémiai, hisztokémiai és molekuláris biológiai szakasszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3552, NULL, 45, NULL),
            (8167, N'Autóipar (2020)', N'Autóipar', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 4101, NULL, 46, NULL),
            (8168, N'Energetika és petrolkémia (2020)', N'Energetika és petrolkémia', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 4102, NULL, 47, NULL),
            (8169, N'Épületautomatizálás (2020)', N'Épületautomatizálás', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 4103, NULL, 48, NULL),
            (8170, N'Gyártástechnika (2020)', N'Gyártástechnika', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 4104, NULL, 49, NULL),
            (8171, N'Épületvillamosság (2020)', N'Épületvillamosság', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 4401, NULL, 50, NULL),
            (8172, N'Villamos hálózat (2020)', N'Villamos hálózat', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 4402, NULL, 51, NULL),
            (8173, N'Villamos készülék és berendezés (2020)', N'Villamos készülék és berendezés', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 4403, NULL, 52, NULL),
            (8174, N'CAD-CAM (2020)', N'CAD-CAM', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 10301, NULL, 53, NULL),
            (8175, N'Ipar (2020)', N'Ipar', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 10302, NULL, 54, NULL),
            (8176, N'Vegyipar (2020)', N'Vegyipar', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 10303, NULL, 55, NULL),
            (8177, N'Állami légijármű műszerész (2020)', N'Állami légijármű műszerész', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11251, NULL, 56, NULL),
            (8178, N'Állami légijármű szerelő (2020)', N'Állami légijármű szerelő', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11252, NULL, 57, NULL),
            (8179, N'Híradó (2020)', N'Híradó', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11253, NULL, 58, NULL),
            (8180, N'Katonai informatikai rendszer-üzemeltető (2020)', N'Katonai informatikai rendszer-üzemeltető', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11254, NULL, 59, NULL),
            (8181, N'Katonai pénzügyi (2020)', N'Katonai pénzügyi', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11255, NULL, 60, NULL),
            (8182, N'Légi vezetés (2020)', N'Légi vezetés', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11256, NULL, 61, NULL),
            (8183, N'Műszerész, fegyverműszerész (2020)', N'Műszerész, fegyverműszerész', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11257, NULL, 62, NULL),
            (8184, N'Műszerész, páncéltörő-rakéta műszerész (2020)', N'Műszerész, páncéltörő-rakéta műszerész', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11258, NULL, 63, NULL),
            (8185, N'Parancsnok, ABV védelmi (2020)', N'Parancsnok, ABV védelmi', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11259, NULL, 64, NULL),
            (8186, N'Parancsnok, Légvédelmi rakéta és tüzér (2020)', N'Parancsnok, Légvédelmi rakéta és tüzér', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11260, NULL, 65, NULL),
            (8187, N'Repülésbiztosító (2020)', N'Repülésbiztosító', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11261, NULL, 66, NULL),
            (8188, N'Speciális felderítő, Elektronikai hadviselés (2020)', N'Speciális felderítő, Elektronikai hadviselés', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11262, NULL, 67, NULL),
            (8189, N'Speciális felderítő, Rádióelektronikai felderítő (2020)', N'Speciális felderítő, Rádióelektronikai felderítő', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11263, NULL, 68, NULL),
            (8190, N'Szerelő, Műszakigép-szerelő (2020)', N'Szerelő, Műszakigép-szerelő', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11264, NULL, 69, NULL),
            (8191, N'Szerelő, Páncélos- és gépjárműszerelő (2020)', N'Szerelő, Páncélos- és gépjárműszerelő', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11265, NULL, 70, NULL),
            (8192, N'Gépjármű mechatronikai technikus (szerviz) (2020)', N'Gépjármű mechatronikai technikus (szerviz)', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11301, NULL, 71, NULL),
            (8193, N'Elektronikai technikus (2020)', N'Elektronikai technikus', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11302, NULL, 72, NULL),
            (8194, N'Honvédelmi igazgatási ügyintéző (2020)', N'Honvédelmi igazgatási ügyintéző', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11303, NULL, 73, NULL),
            (8195, N'Infokommunikációs hálózatépítő és -üzemeltető technikus (2020)', N'Infokommunikációs hálózatépítő és -üzemeltető technikus', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11304, NULL, 74, NULL),
            (8196, N'Informatikai rendszer- és alkalmazás-üzemeltető (2020)', N'Informatikai rendszer- és alkalmazás-üzemeltető', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11305, NULL, 75, NULL),
            (8197, N'Környezetvédelmi technikus (környezetvédelem) (2020)', N'Környezetvédelmi technikus (környezetvédelem)', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11306, NULL, 76, NULL),
            (8198, N'Logisztikai technikus (2020)', N'Logisztikai technikus', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 11307, NULL, 77, NULL),
            (8199, N'Általános gyártás (2020)', N'Általános gyártás', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 13101, NULL, 78, NULL),
            (8200, N'Gépjárműgyártás (2020)', N'Gépjárműgyártás', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 13102, NULL, 79, NULL),
            (8201, N'Kereskedelmi logisztika (2020)', N'Kereskedelmi logisztika', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 13103, NULL, 80, NULL),
            (8202, N'Hulladékhasznosító, -feldolgozó (2020)', N'Hulladékhasznosító, -feldolgozó', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 14151, NULL, 81, NULL),
            (8203, N'Igazgatás (2020)', N'Igazgatás', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 14152, NULL, 82, NULL),
            (8204, N'Környezetvédelem (2020)', N'Környezetvédelem', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 14153, NULL, 83, NULL),
            (8205, N'Természetvédelem (2020)', N'Természetvédelem', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 14154, NULL, 84, NULL),
            (8206, N'Területi vízgazdálkodó (2020)', N'Területi vízgazdálkodó', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 14251, NULL, 85, NULL),
            (8207, N'Települési vízgazdálkodó (2020)', N'Települési vízgazdálkodó', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 14252, NULL, 86, NULL),
            (8208, N'Vízgépészet (2020)', N'Vízgépészet', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 14253, NULL, 87, NULL),
            (8209, N'Hajózási (2020)', N'Hajózási', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 15301, NULL, 88, NULL),
            (8210, N'Közúti (2020)', N'Közúti', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 15302, NULL, 89, NULL),
            (8211, N'Vasúti (2020)', N'Vasúti', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 15303, NULL, 90, NULL),
            (8212, N'Légi (2020)', N'Légi', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 15304, NULL, 91, NULL),
            (8213, N'Logisztika és szállítmányozás (2020)', N'Logisztika és szállítmányozás', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 15351, NULL, 92, NULL),
            (8214, N'Vasúti árufuvarozás (2020)', N'Vasúti árufuvarozás', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 15352, NULL, 93, NULL),
            (8215, N'Bőrdíszműves (2020)', N'Bőrdíszműves', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16101, NULL, 94, NULL),
            (8216, N'Cipőkészítő (2020)', N'Cipőkészítő', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16102, NULL, 95, NULL),
            (8217, N'Ortopédiai cipész (2020)', N'Ortopédiai cipész', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16103, NULL, 96, NULL),
            (8218, N'Divattervező (2020)', N'Divattervező', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16151, NULL, 97, NULL),
            (8219, N'Jelmez- és díszlettervező (2020)', N'Jelmez- és díszlettervező', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16152, NULL, 98, NULL),
            (8220, N'Férfiszabó (2020)', N'Férfiszabó', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16201, NULL, 99, NULL),
            (8221, N'Női szabó (2020)', N'Női szabó', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16202, NULL, 100, NULL),
            (8222, N'Gipszmodell-készítő (2020)', N'Gipszmodell-készítő', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16251, NULL, 101, NULL),
            (8223, N'Kerámia- és porcelántárgy-készítő (2020)', N'Kerámia- és porcelántárgy-készítő', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16252, NULL, 102, NULL),
            (8224, N'Porcelánfestő (2020)', N'Porcelánfestő', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16253, NULL, 103, NULL),
            (8225, N'Bőrfeldolgozó-ipar (2020)', N'Bőrfeldolgozó-ipar', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16301, NULL, 104, NULL),
            (8226, N'Ruhaipar (2020)', N'Ruhaipar', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16304, NULL, 105, NULL),
            (8227, N'Textilipar (2020)', N'Textilipar', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16305, NULL, 106, NULL),
            (8228, N'Fonó (2020)', N'Fonó', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16351, NULL, 107, NULL),
            (8229, N'Kötő (2020)', N'Kötő', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16352, NULL, 108, NULL),
            (8230, N'Nemszőtt-termék gyártó (2020)', N'Nemszőtt-termék gyártó', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16353, NULL, 109, NULL),
            (8231, N'Szövő (2020)', N'Szövő', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16354, NULL, 110, NULL),
            (8232, N'Kreatív fotográfus (2020)', N'Kreatív fotográfus', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16451, NULL, 111, NULL),
            (8233, N'Művészeti fotográfus (2020)', N'Művészeti fotográfus', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16452, NULL, 112, NULL),
            (8234, N'Nyomdaipari előkészítő (2020)', N'Nyomdaipari előkészítő', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16601, NULL, 113, NULL),
            (8235, N'Nyomdaipari gépmester (2020)', N'Nyomdaipari gépmester', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16602, NULL, 114, NULL),
            (8236, N'Nyomtatványfeldolgozó (2020)', N'Nyomtatványfeldolgozó', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16603, NULL, 115, NULL),
            (8237, N'Porondtechnikus (2020)', N'Porondtechnikus', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16751, NULL, 116, NULL),
            (8238, N'Színpadi hangtechnikus (2020)', N'Színpadi hangtechnikus', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16752, NULL, 117, NULL),
            (8239, N'Színpadi világítástechnikus (2020)', N'Színpadi világítástechnikus', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16753, NULL, 118, NULL),
            (8240, N'Színpadi vizuáltechnikus (2020)', N'Színpadi vizuáltechnikus', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16754, NULL, 119, NULL),
            (8241, N'Színpadtechnikus (2020)', N'Színpadtechnikus', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 16755, NULL, 120, NULL),
            (8242, N'Vadgazdálkodás (2020)', N'Vadgazdálkodás', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 17151, NULL, 121, NULL),
            (8243, N'Erdőgazdálkodás (2020)', N'Erdőgazdálkodás', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 17152, NULL, 122, NULL),
            (8244, N'Állattenyésztő (2020)', N'Állattenyésztő', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 17251, NULL, 123, NULL),
            (8245, N'Lovász (2020)', N'Lovász', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 17252, NULL, 124, NULL),
            (8246, N'Növénytermesztő (2020)', N'Növénytermesztő', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 17253, NULL, 125, NULL),
            (8247, N'Dísznövénytermesztő, virágkötő (2020)', N'Dísznövénytermesztő, virágkötő', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 17351, NULL, 126, NULL),
            (8248, N'Gyógynövénytermesztő (2020)', N'Gyógynövénytermesztő', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 17352, NULL, 127, NULL),
            (8249, N'Gyümölcstermesztő (2020)', N'Gyümölcstermesztő', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 17353, NULL, 128, NULL)
                ) x (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
                WHERE NOT EXISTS (SELECT 1 FROM #DICTIONARYITEMBASE d WHERE x.ID = d.ID)
                
    INSERT INTO #DICTIONARYITEMBASE (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
        SELECT * FROM ( VALUES
            (8250, N'Parképítő és -fenntartó (2020)', N'Parképítő és -fenntartó', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 17354, NULL, 129, NULL),
            (8251, N'Zöldségtermesztő (2020)', N'Zöldségtermesztő', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 17355, NULL, 130, NULL),
            (8252, N'Közigazgatási ügyintéző (2020)', N'Közigazgatási ügyintéző', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 18101, NULL, 131, NULL),
            (8253, N'Rendészeti technikus (2020)', N'Rendészeti technikus', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 18102, NULL, 132, NULL),
            (8254, N'Bűnügyi (2020)', N'Bűnügyi', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 18201, NULL, 133, NULL),
            (8255, N'Határrendészeti (2020)', N'Határrendészeti', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 18202, NULL, 134, NULL),
            (8256, N'Közlekedési (2020)', N'Közlekedési', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 18203, NULL, 135, NULL),
            (8257, N'Közrendvédelmi (2020)', N'Közrendvédelmi', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 18204, NULL, 136, NULL),
            (8258, N'Gyártás (2020)', N'Gyártás', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 19251, NULL, 137, NULL),
            (8259, N'Motorkerékpár- és versenymotor-szerelés (2020)', N'Motorkerékpár- és versenymotor-szerelés', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 19252, NULL, 138, NULL),
            (8260, N'Szerviz (2020)', N'Szerviz', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 19253, NULL, 139, NULL),
            (8261, N'Motorkerékpár karbantartás (2020)', N'Motorkerékpár karbantartás', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 19302, NULL, 140, NULL),
            (8262, N'Kézápoló és körömkozmetikus (2020)', N'Kézápoló és körömkozmetikus', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 21151, NULL, 141, NULL),
            (8263, N'Speciális lábápoló (2020)', N'Speciális lábápoló', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 21152, NULL, 142, NULL),
            (8264, N'Idegenvezető (2020)', N'Idegenvezető', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 23401, NULL, 143, NULL),
            (8265, N'Turisztikai szervező (2020)', N'Turisztikai szervező', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 23402, NULL, 144, NULL),
            (8266, N'Csomagolószer-gyártó (2020)', N'Csomagolószer-gyártó', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 24351, NULL, 145, NULL),
            (8267, N'Papírgyártó és -feldolgozó (2020)', N'Papírgyártó és -feldolgozó', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 24352, NULL, 146, NULL),
            (8268, N'Általános laboráns (2020)', N'Általános laboráns', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 24450, NULL, 147, NULL),
            (8269, N'Termelési folyamatirányító (2020)', N'Termelési folyamatirányító', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 24451, NULL, 148, NULL),
            (8270, N'Több szakmairány', N'Több szakmairány', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'F', 31000, NULL, 149, NULL),
            (8391, N'Audiológiai asszisztens (2020)', N'Audiológiai asszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3150, NULL, 150, NULL),
            (8392, N'Nukleáris medicina szakasszisztens (2020)', N'Nukleáris medicina szakasszisztens', NULL, NULL, NULL, N'SzakmairanyTipus', 151, N'T', N'T', 3452, NULL, 151, NULL),
            (7888, N'Szakképzési juttatás', N'Szakképzési juttatás', NULL, NULL, NULL, N'JuttatasTipus', 154, N'T', N'T', 1, NULL, 1, NULL),
            (8617, N'Egyszeri pályakezdési juttatás', N'Egyszeri pályakezdési juttatás', NULL, NULL, NULL, N'JuttatasTipus', 154, N'T', N'T', 5, NULL, 2, NULL),
            (8618, N'Apáczai-ösztöndíj', N'Apáczai-ösztöndíj', NULL, NULL, NULL, N'JuttatasTipus', 154, N'T', N'F', 10, NULL, 3, NULL),
            (7860, N'Tanulói jogviszony', N'Tanulói jogviszony', NULL, NULL, NULL, N'JogviszonyTipus', 152, N'T', N'T', 1, NULL, 1, NULL),
            (7861, N'Felnőttképzési jogviszony', N'Felnőttképzési jogviszony', NULL, NULL, NULL, N'JogviszonyTipus', 152, N'T', N'T', 2, NULL, 2, NULL),
            (7885, N'saját', N'saját', NULL, NULL, NULL, N'BankszamlaTulajdonosTipus', 153, N'T', N'T', 1, NULL, 1, NULL),
            (7886, N'szülői', N'szülői', NULL, NULL, NULL, N'BankszamlaTulajdonosTipus', 153, N'T', N'T', 2, NULL, 2, NULL),
            (7887, N'egyéb', N'egyéb', NULL, NULL, NULL, N'BankszamlaTulajdonosTipus', 153, N'T', N'T', 3, NULL, 3, NULL),
            (7856, N'A pedagógiai felügyelői munkakör bevezetésre került', N'A pedagógiai felügyelői munkakör bevezetésre került', NULL, NULL, NULL, N'AmiJellemzoTipus', 148, N'F', N'T', 1, NULL, 1, NULL),
            (7857, N'A pedagógiai felügyelői munkakör nem került bevezetésre', N'A pedagógiai felügyelői munkakör nem került bevezetésre', NULL, NULL, NULL, N'AmiJellemzoTipus', 148, N'F', N'T', 2, NULL, 2, NULL),
            (7816, N'4 évfolyamos gimnáziumi nevelés-oktatás', N'4 évfolyamos gimnáziumi nevelés-oktatás', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 300, NULL, 1, NULL),
            (7817, N'5 évfolyamos gimnáziumi nevelés-oktatás', N'5 évfolyamos gimnáziumi nevelés-oktatás', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 310, NULL, 2, NULL),
            (7818, N'6 évfolyamos gimnáziumi nevelés-oktatás', N'6 évfolyamos gimnáziumi nevelés-oktatás', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 320, NULL, 3, NULL),
            (7819, N'8 évfolyamos gimnáziumi nevelés-oktatás', N'8 évfolyamos gimnáziumi nevelés-oktatás', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 340, NULL, 4, NULL),
            (7820, N'Alapfokú művészetoktatás képző- és iparművészeti ágon', N'Alapfokú művészetoktatás képző- és iparművészeti ágon', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 200, NULL, 5, NULL),
            (7821, N'Alapfokú művészetoktatás szín- és bábművészeti ágon', N'Alapfokú művészetoktatás szín- és bábművészeti ágon', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 210, NULL, 6, NULL),
            (7822, N'Alapfokú művészetoktatás táncművészeti ágon', N'Alapfokú művészetoktatás táncművészeti ágon', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 220, NULL, 7, NULL),
            (7823, N'Alapfokú művészetoktatás zeneművészeti ágon', N'Alapfokú művészetoktatás zeneművészeti ágon', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 230, NULL, 8, NULL),
            (7824, N'Általános iskolai nevelés-oktatás (alsó tagozat)', N'Általános iskolai nevelés-oktatás (alsó tagozat)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 150, NULL, 9, NULL),
            (7825, N'Általános iskolai nevelés-oktatás (felső tagozat)', N'Általános iskolai nevelés-oktatás (felső tagozat)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 160, NULL, 10, NULL),
            (7826, N'Előkészítő speciális szakiskolai nevelés-oktatás', N'Előkészítő speciális szakiskolai nevelés-oktatás', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 3000, NULL, 11, NULL),
            (7827, N'Érettségire felkészítő évfolyam Waldorf-intézményben', N'Érettségire felkészítő évfolyam Waldorf-intézményben', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 360, NULL, 12, NULL),
            (7828, N'Készségfejlesztő iskolai nevelés-oktatás (2016.09.01-től felmenő rendszerben)', N'Készségfejlesztő iskolai nevelés-oktatás (2016.09.01-től felmenő rendszerben)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 1500, NULL, 13, NULL),
            (7829, N'Készségfejlesztő speciális szakiskola (közismereti képzés - kifutó rendszerben)', N'Készségfejlesztő speciális szakiskola (közismereti képzés - kifutó rendszerben)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 3100, NULL, 14, NULL),
            (7830, N'Készségfejlesztő speciális szakiskola (szakképzés - kifutó rendszerben)', N'Készségfejlesztő speciális szakiskola (szakképzés - kifutó rendszerben)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 3110, NULL, 15, NULL),
            (7831, N'Készségfejlesztő speciális szakiskolai nevelés-oktatás (párhuzamos közismereti és szakmai képzés - 2016.09.01-től kifutó rendszerben)', N'Készségfejlesztő speciális szakiskolai nevelés-oktatás (párhuzamos közismereti és szakmai képzés - 2016.09.01-től kifutó rendszerben)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 3120, NULL, 16, NULL),
            (7832, N'Kiegészítő nemzetiségi nevelés-oktatás', N'Kiegészítő nemzetiségi nevelés-oktatás', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 1700, NULL, 17, NULL),
            (7833, N'Kollégiumi nevelés-oktatás', N'Kollégiumi nevelés-oktatás', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 1710, NULL, 18, NULL),
            (7834, N'Köznevelési Hídprogram keretén belül zajló nevelés-oktatás', N'Köznevelési Hídprogram keretén belül zajló nevelés-oktatás', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 3410, NULL, 19, NULL),
            (7835, N'Köznevelési Hídprogram keretén belül zajló nevelés-oktatás (HÍD II. - kifutó rendszerben)', N'Köznevelési Hídprogram keretén belül zajló nevelés-oktatás (HÍD II. - kifutó rendszerben)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 3420, NULL, 20, NULL),
            (7836, N'Óvodai nevelés', N'Óvodai nevelés', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 100, NULL, 21, NULL),
            (7837, N'Párhuzamos művészeti nevelés-oktatás (szakgimnázium - 2016.09.01. előtt szakközépiskola)', N'Párhuzamos művészeti nevelés-oktatás (szakgimnázium - 2016.09.01. előtt szakközépiskola)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 1600, NULL, 22, NULL),
            (7838, N'Párhuzamos művészeti nevelés-oktatás (szakközépiskola - 2016.09.01. előtt szakiskola)', N'Párhuzamos művészeti nevelés-oktatás (szakközépiskola - 2016.09.01. előtt szakiskola)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 1610, NULL, 23, NULL),
            (7839, N'Speciális szakiskola (közismereti képzés - kifutó rendszerben)', N'Speciális szakiskola (közismereti képzés - kifutó rendszerben)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 3200, NULL, 24, NULL),
            (7840, N'Speciális szakiskola (szakképzés - kifutó rendszerben)', N'Speciális szakiskola (szakképzés - kifutó rendszerben)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 3210, NULL, 25, NULL),
            (7841, N'Speciális szakiskolai nevelés-oktatás (párhuzamos közismereti és szakmai képzés - 2016.09.01-től kifutó rendszerben)', N'Speciális szakiskolai nevelés-oktatás (párhuzamos közismereti és szakmai képzés - 2016.09.01-től kifutó rendszerben)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 3220, NULL, 26, NULL),
            (7842, N'Szakgimnáziumi nevelés-oktatás (9-12. évfolyam - 2020.09.01-től kifutó)', N'Szakgimnáziumi nevelés-oktatás (9-12. évfolyam - 2020.09.01-től kifutó)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 2000, NULL, 27, NULL),
            (7843, N'Szakgimnáziumi nevelés-oktatás (kizárólag szakképzési évfolyamok - 2020.09.01-től kifutó)', N'Szakgimnáziumi nevelés-oktatás (kizárólag szakképzési évfolyamok - 2020.09.01-től kifutó)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 2010, NULL, 28, NULL),
            (7844, N'Szakiskolai nevelés-oktatás (2016.09.01-től felmenő rendszerben)', N'Szakiskolai nevelés-oktatás (2016.09.01-től felmenő rendszerben)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 1000, NULL, 29, NULL),
            (7845, N'Szakiskolai nevelés-oktatás (közismereti képzés - kifutó rendszerben)', N'Szakiskolai nevelés-oktatás (közismereti képzés - kifutó rendszerben)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 3230, NULL, 30, NULL),
            (7846, N'Szakiskolai nevelés-oktatás (párhuzamos közismereti és szakmai képzés - 2016.09.01-től kifutó rendszerben)', N'Szakiskolai nevelés-oktatás (párhuzamos közismereti és szakmai képzés - 2016.09.01-től kifutó rendszerben)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 3240, NULL, 31, NULL),
            (7847, N'Szakiskolai nevelés-oktatás (szakképzés - kifutó rendszerben)', N'Szakiskolai nevelés-oktatás (szakképzés - kifutó rendszerben)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 3250, NULL, 32, NULL),
            (7848, N'Szakképzési Hídprogram keretén belül zajló nevelés-oktatás', N'Szakképzési Hídprogram keretén belül zajló nevelés-oktatás', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 3260, NULL, 33, NULL),
            (7849, N'Szakközépiskolai nevelés-oktatás (2016.09.01-től kifutó rendszerben)', N'Szakközépiskolai nevelés-oktatás (2016.09.01-től kifutó rendszerben)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 3270, NULL, 34, NULL),
            (7850, N'Szakközépiskolai nevelés-oktatás (9-11. évfolyam - 2020.09.01-től kifutó)', N'Szakközépiskolai nevelés-oktatás (9-11. évfolyam - 2020.09.01-től kifutó)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 2100, NULL, 35, NULL),
            (7851, N'Szakközépiskolai nevelés-oktatás (érettségire felkészítő évfolyamok - 2020.09.01-től kifutó)', N'Szakközépiskolai nevelés-oktatás (érettségire felkészítő évfolyamok - 2020.09.01-től kifutó)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 2110, NULL, 36, NULL),
            (7852, N'Szakközépiskolai nevelés-oktatás (kizárólag szakmai oktatás, 2016.09.01-től felmenő - 2020.09.01-től kifutó rendszerben)', N'Szakközépiskolai nevelés-oktatás (kizárólag szakmai oktatás, 2016.09.01-től felmenő - 2020.09.01-től kifutó rendszerben)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'T', 2120, NULL, 37, NULL),
            (7853, N'Szakközépiskolai nevelés-oktatás (szakképzés - 2016.09.01-től kifutó rendszerben)', N'Szakközépiskolai nevelés-oktatás (szakképzés - 2016.09.01-től kifutó rendszerben)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'F', N'F', 3400, NULL, 38, NULL),
            (8772, N'Alapfokú művészetoktatás népművészeti ágon', NULL, NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'T', 240, NULL, 39, NULL),
            (8773, N'7 évfolyamos gimnáziumi nevelés-oktatás', NULL, NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'T', 330, NULL, 40, NULL),
            (8774, N'9 évfolyamos gimnáziumi nevelés-oktatás', NULL, NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'T', 350, NULL, 41, NULL),
            (8775, N'Előkészítő szakiskolai nevelés-oktatás (2020.09.01-től felmenő rendszerben)', NULL, NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'T', 1010, NULL, 42, NULL),
            (8776, N'Szakiskolai nevelés-oktatás (2020.09.01-től felmenő rendszerben)', NULL, NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'T', 1020, NULL, 43, NULL),
            (8777, N'Szakiskolai nevelés-oktatás (kizárólag szakmai oktatás - 2020.09.01-től felmenő rendszerben)', NULL, NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'T', 1030, NULL, 44, NULL),
            (8778, N'Szakképző iskolai oktatás', NULL, NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'T', 1100, NULL, 45, NULL),
            (8779, N'Szakképző iskolai oktatás (kizárólag szakképzési évfolyamok)', NULL, NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'T', 1110, NULL, 46, NULL),
            (8780, N'Előkészítő technikumi oktatás', NULL, NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'T', 1200, NULL, 47, NULL),
            (8781, N'Technikumi oktatás', NULL, NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'T', 1210, NULL, 48, NULL),
            (8782, N'Technikumi oktatás (kizárólag szakképzési évfolyamok)', NULL, NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'T', 1220, NULL, 49, NULL),
            (8783, N'Szakgimnáziumi nevelés-oktatás (művészeti képzés)', NULL, NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'T', 1300, NULL, 50, NULL),
            (8784, N'Szakgimnáziumi nevelés-oktatás (pedagógiai képzés)', NULL, NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'T', 1310, NULL, 51, NULL),
            (8785, N'Szakgimnáziumi nevelés-oktatás (közművelődési képzés)', NULL, NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'T', 1320, NULL, 52, NULL),
            (8786, N'Műhelyiskola', NULL, NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'T', 1400, NULL, 53, NULL),
            (8787, N'Orientációs fejlesztés (előkészítő)', NULL, NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'T', 1410, NULL, 54, NULL),
            (8788, N'Dobbantó program (alapkompetencia-fejlesztés)', NULL, NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'T', 1420, NULL, 55, NULL),
            (8792, N'Technikumi oktatás (érettségire felkészítő évfolyamok)', N'Technikumi oktatás (érettségire felkészítő évfolyamok)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'T', 1230, NULL, 56, NULL),
            (8793, N'Szakgimnáziumi nevelés-oktatás (érettségire felkészítő évfolyamok)', N'Szakgimnáziumi nevelés-oktatás (érettségire felkészítő évfolyamok)', NULL, NULL, NULL, N'TanterviJellemzoTipus', 147, N'T', N'T', 1340, NULL, 57, NULL),
            (7776, N'AJTP - Arany János Tehetséggondozó Program', N'AJTP - Arany János Tehetséggondozó Program', NULL, NULL, NULL, N'AranyJanosProgramTipus', 145, N'T', N'T', 1, NULL, 1, NULL),
            (7777, N'AJKP - Arany János Kollégiumi Program', N'AJKP - Arany János Kollégiumi Program', NULL, NULL, NULL, N'AranyJanosProgramTipus', 145, N'T', N'T', 2, NULL, 2, NULL),
            (7778, N'AJKSzP - Arany János Kollégiumi-Szakközépiskolai Program', N'AJKSzP - Arany János Kollégiumi-Szakközépiskolai Program', NULL, NULL, NULL, N'AranyJanosProgramTipus', 145, N'T', N'T', 3, NULL, 3, NULL),
            (7723, N'Többségi (együttnevelő) nevelési-oktatási intézmény', N'Többségi (együttnevelő) nevelési-oktatási intézmény', NULL, NULL, NULL, N'SniTanuloNevelesiFormaTipus', 144, N'T', N'T', 1, NULL, 1, NULL),
            (7724, N'Gyógypedagógiai, konduktív pedagógiai (különnevelő) nevelési-oktatási intézmény', N'Gyógypedagógiai, konduktív pedagógiai (különnevelő) nevelési-oktatási intézmény', NULL, NULL, NULL, N'SniTanuloNevelesiFormaTipus', 144, N'T', N'T', 2, NULL, 2, NULL),
            (7858, N'Pedagógiai szakszolgálat', N'Pedagógiai szakszolgálat', NULL, NULL, NULL, N'SniTanuloNevelesiFormaTipus', 144, N'F', N'T', 3, NULL, 3, NULL),
            (7711, N'Függő', N'Függő', NULL, NULL, NULL, N'BeszerzesiIgenySatusz', 142, N'T', N'T', 1, NULL, 1, NULL),
            (7712, N'Jóváhagyva átadva tankerületnek', N'Jóváhagyva átadva tankerületnek', NULL, NULL, NULL, N'BeszerzesiIgenySatusz', 142, N'T', N'T', 2, NULL, 2, NULL),
            (7713, N'Elutasítva', N'Elutasítva', NULL, NULL, NULL, N'BeszerzesiIgenySatusz', 142, N'T', N'T', 3, NULL, 3, NULL),
            (7714, N'Visszaküldve hiánypótlásra', N'Visszaküldve hiánypótlásra', NULL, NULL, NULL, N'BeszerzesiIgenySatusz', 142, N'T', N'T', 4, NULL, 4, NULL),
            (7671, N'főtárgy', N'főtárgy', NULL, NULL, NULL, N'AMIFotargy', 141, N'T', N'T', 1, NULL, 1, NULL),
            (7672, N'melléktárgy', N'melléktárgy', NULL, NULL, NULL, N'AMIFotargy', 141, N'T', N'T', 2, NULL, 2, NULL),
            (7668, N'kötelező tárgy', N'kötelező tárgy', NULL, NULL, NULL, N'AMITantargyKotelezoseg', 140, N'T', N'T', 1, NULL, 1, NULL),
            (7669, N'kötelezően választható tárgy', N'kötelezően választható tárgy', NULL, NULL, NULL, N'AMITantargyKotelezoseg', 140, N'T', N'T', 2, NULL, 2, NULL),
            (7670, N'választható tárgy', N'választható tárgy', NULL, NULL, NULL, N'AMITantargyKotelezoseg', 140, N'T', N'T', 3, NULL, 3, NULL),
            (7665, N'A tagozat', N'A tagozat', NULL, NULL, NULL, N'AMITagozat', 139, N'T', N'T', 2, NULL, 1, NULL),
            (7666, N'B tagozat', N'B tagozat', NULL, NULL, NULL, N'AMITagozat', 139, N'T', N'T', 3, NULL, 2, NULL),
            (7780, N'Na', N'Na', NULL, NULL, NULL, N'AMITagozat', 139, N'T', N'T', 1, NULL, 3, NULL),
            (7655, N'Zeneművészeti ág', N'Zeneművészeti ág', NULL, NULL, NULL, N'MuveszetiAg', 137, N'T', N'T', 100, NULL, 1, NULL),
            (7656, N'Képző- és iparművészeti ág', N'Képző- és iparművészeti ág', NULL, NULL, NULL, N'MuveszetiAg', 137, N'T', N'T', 120, NULL, 2, NULL),
            (7657, N'Táncművészeti ág', N'Táncművészeti ág', NULL, NULL, NULL, N'MuveszetiAg', 137, N'T', N'T', 110, NULL, 3, NULL),
            (7658, N'Szín- és bábművészeti ág', N'Szín- és bábművészeti ág', NULL, NULL, NULL, N'MuveszetiAg', 137, N'T', N'T', 130, NULL, 4, NULL),
            (8754, N'Népművészeti ág', N'Népművészeti ág', NULL, NULL, NULL, N'MuveszetiAg', 137, N'T', N'T', 140, NULL, 5, NULL),
            (7659, N'Egyéni', N'Egyéni', NULL, NULL, NULL, N'ZenemuveszetiAgTipus', 138, N'T', N'T', 1, NULL, 1, NULL),
            (7660, N'Csoport', N'Csoport', NULL, NULL, NULL, N'ZenemuveszetiAgTipus', 138, N'T', N'T', 2, NULL, 2, NULL),
            (7855, N'Na', N'Na', NULL, NULL, NULL, N'ZenemuveszetiAgTipus', 138, N'F', N'T', 0, NULL, 3, NULL),
            (7578, N'Angol nyelv', N'Angol nyelv', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 1, NULL, 1, NULL),
            (7579, N'Német nyelv', N'Német nyelv', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 2, NULL, 2, NULL),
            (7580, N'Francia nyelv', N'Francia nyelv', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 3, NULL, 3, NULL),
            (7581, N'Egyéb idegen nyelv', N'Egyéb idegen nyelv', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 4, NULL, 4, NULL),
            (7582, N'Nemzetiségi nyelv', N'Nemzetiségi nyelv', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 5, NULL, 5, NULL),
            (7583, N'Matematika', N'Matematika', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 6, NULL, 6, NULL),
            (7584, N'Magyar nyelv', N'Magyar nyelv', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 7, NULL, 7, NULL),
            (7585, N'Magyar irodalom', N'Magyar irodalom', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 8, NULL, 8, NULL),
            (7586, N'Nemzetiségi nyelv és irodalom', N'Nemzetiségi nyelv és irodalom', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 9, NULL, 9, NULL),
            (7587, N'Történelem', N'Történelem', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 10, NULL, 10, NULL),
            (7588, N'Fizika', N'Fizika', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 11, NULL, 11, NULL),
            (7589, N'Kémia', N'Kémia', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 12, NULL, 12, NULL),
            (7590, N'Biológia', N'Biológia', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 13, NULL, 13, NULL),
            (7591, N'Földrajz', N'Földrajz', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 14, NULL, 14, NULL),
            (7592, N'Ének-zene', N'Ének-zene', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 15, NULL, 15, NULL),
            (7593, N'Számítástechnika, informatika', N'Számítástechnika, informatika', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 16, NULL, 16, NULL),
            (7594, N'Technika', N'Technika', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 17, NULL, 17, NULL),
            (7595, N'Testnevelés', N'Testnevelés', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 18, NULL, 18, NULL),
            (7596, N'Szakmai elmélet', N'Szakmai elmélet', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 19, NULL, 19, NULL),
            (7597, N'Szakmai gyakorlat', N'Szakmai gyakorlat', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 20, NULL, 20, NULL),
            (7598, N'Egyéb tantárgy', N'Egyéb tantárgy', NULL, NULL, NULL, N'ESLTantargykategoria', 136, N'T', N'T', 21, NULL, 21, NULL),
            (7573, N'Na', N'Na', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 1, NULL, 1, NULL),
            (7606, N'Szabadság', N'Szabadság', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 2, NULL, 2, NULL),
            (7608, N'Hozzátartozó halálakor mentesülés munkavégzés alól', N'Hozzátartozó halálakor mentesülés munkavégzés alól', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 4, NULL, 3, NULL),
            (7609, N'Apát megillető pótszabadság gyermeke születése esetén', N'Apát megillető pótszabadság gyermeke születése esetén', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 5, NULL, 4, NULL),
            (7610, N'Tanulmányi munkaidő-kedvezmény', N'Tanulmányi munkaidő-kedvezmény', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 6, NULL, 5, NULL),
            (7611, N'Fizetés nélküli szabadság kérelemre 30 napot meghaladó tartamú biztosítás szünetel', N'Fizetés nélküli szabadság kérelemre 30 napot meghaladó tartamú biztosítás szünetel', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 7, NULL, 6, NULL),
            (7612, N'Fizetés nélküli szabadság kérelemre 30 napot nem meghaladó tartamú biztosítás szünetel', N'Fizetés nélküli szabadság kérelemre 30 napot nem meghaladó tartamú biztosítás szünetel', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 8, NULL, 7, NULL),
            (7614, N'Egyéb hiányzás díjazással', N'Egyéb hiányzás díjazással', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 10, NULL, 8, NULL),
            (7615, N'Egyéb hiányzás díjazás nélkül', N'Egyéb hiányzás díjazás nélkül', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 11, NULL, 9, NULL),
            (7616, N'Mentesítés a munkavégzés alól felmentési/felmondási idő tartamára', N'Mentesítés a munkavégzés alól felmentési/felmondási idő tartamára', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 12, NULL, 10, NULL),
            (7617, N'Szakszervezeti tisztségviselő munkaidő-kedvezménye', N'Szakszervezeti tisztségviselő munkaidő-kedvezménye', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 13, NULL, 11, NULL),
            (7619, N'Keresőképtelenség', N'Keresőképtelenség', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 15, NULL, 12, NULL),
            (7620, N'Sztrájk', N'Sztrájk', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 16, NULL, 13, NULL),
            (7621, N'Különös méltánylást érdemlő személyi, családi vagy elháríthatatlan ok miatt indokolt távollét tartamára mentesülés munkavégzés alól', N'Különös méltánylást érdemlő személyi, családi vagy elháríthatatlan ok miatt indokolt távollét tartamára mentesülés munkavégzés alól', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 17, NULL, 14, NULL),
            (7622, N'Munkaszüneti nap miatti távollét óra- vagy teljesítménybérezés esetén', N'Munkaszüneti nap miatti távollét óra- vagy teljesítménybérezés esetén', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 18, NULL, 15, NULL),
            (7623, N'Szülési szabadság', N'Szülési szabadság', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 19, NULL, 16, NULL),
            (7624, N'Fizetés nélküli szabadság (GYED)', N'Fizetés nélküli szabadság (GYED)', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 20, NULL, 17, NULL),
            (7627, N'Önkéntes tartalékos katonai szolgálatteljesítés', N'Önkéntes tartalékos katonai szolgálatteljesítés', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 23, NULL, 18, NULL),
            (7628, N'Jogviszony/folyósítás', N'Jogviszony/folyósítás', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 24, NULL, 19, NULL),
            (7629, N'Kötelező orvosi vizsgálat tartamára mentesülés munkavégzés alól', N'Kötelező orvosi vizsgálat tartamára mentesülés munkavégzés alól', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 25, NULL, 20, NULL),
            (7630, N'Véradáshoz szükséges időtartamra mentesülés munkavégzés alól', N'Véradáshoz szükséges időtartamra mentesülés munkavégzés alól', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 26, NULL, 21, NULL),
            (7631, N'Szoptatás időtartamára mentesülés munkavégzés alól', N'Szoptatás időtartamára mentesülés munkavégzés alól', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 27, NULL, 22, NULL),
            (7632, N'Választás miatt mentesülés munkavégzés alól', N'Választás miatt mentesülés munkavégzés alól', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 28, NULL, 23, NULL),
            (7633, N'Tanúkénti meghallgatás', N'Tanúkénti meghallgatás', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 29, NULL, 24, NULL),
            (7634, N'Kirendelés a fogadó munkáltató fizet', N'Kirendelés a fogadó munkáltató fizet', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 30, NULL, 25, NULL),
            (7636, N'Képviselői feladatok ellátása', N'Képviselői feladatok ellátása', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 32, NULL, 26, NULL),
            (7637, N'Emberi reprodukciós eljárással összefüggő, egészségügyi intézményben történő kezelés tartamára mentesülés munkavégzés alól', N'Emberi reprodukciós eljárással összefüggő, egészségügyi intézményben történő kezelés tartamára mentesülés munkavégzés alól', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 33, NULL, 27, NULL),
            (7638, N'Képzésben, továbbképzésben felek megállapodása alapján való részvételhez szükséges időtartamra mentesülés munkavégzés alól', N'Képzésben, továbbképzésben felek megállapodása alapján való részvételhez szükséges időtartamra mentesülés munkavégzés alól', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 34, NULL, 28, NULL),
            (7639, N'Önkéntes vagy létesítményi tűzoltói szolgálat ellátásának tartamára mentesülés munkavégzés alól', N'Önkéntes vagy létesítményi tűzoltói szolgálat ellátásának tartamára mentesülés munkavégzés alól', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 35, NULL, 29, NULL),
            (7640, N'Bíróság, hatóság felhívására vagy az eljárásban való részvételhez szükséges időtartamra mentesülés munkavégzés alól', N'Bíróság, hatóság felhívására vagy az eljárásban való részvételhez szükséges időtartamra mentesülés munkavégzés alól', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 36, NULL, 30, NULL),
            (7641, N'Munkáltató engedélye alapján', N'Munkáltató engedélye alapján', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 37, NULL, 31, NULL),
            (7642, N'Állásidő díjazással', N'Állásidő díjazással', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 38, NULL, 32, NULL),
            (7643, N'Felfüggesztés jogviszonyt megszüntető határozat kézbesítése', N'Felfüggesztés jogviszonyt megszüntető határozat kézbesítése', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 39, NULL, 33, NULL),
            (7645, N'Munkaszüneti nap miatti távollét', N'Munkaszüneti nap miatti távollét', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 41, NULL, 34, NULL),
            (7646, N'Kötelezettségszegés körülményei kivizsgálásának időtartamára mentesítés munkavégzés alól (max. 30 nap)', N'Kötelezettségszegés körülményei kivizsgálásának időtartamára mentesítés munkavégzés alól (max. 30 nap)', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 42, NULL, 35, NULL),
            (7647, N'Fizetés nélküli szabadság külszolgálatba helyezés időtartamára', N'Fizetés nélküli szabadság külszolgálatba helyezés időtartamára', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 43, NULL, 36, NULL),
            (7648, N'Előzetes letartóztatás, szabadságvesztés', N'Előzetes letartóztatás, szabadságvesztés', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 44, NULL, 37, NULL),
            (7649, N'Állásidő díjazás nélkül', N'Állásidő díjazás nélkül', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 45, NULL, 38, NULL),
            (7690, N'Távollétigény visszavonása', N'Távollétigény visszavonása', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 46, NULL, 39, NULL),
            (8403, N'Nagyszülőt megillető pótszabadság', N'Nagyszülőt megillető pótszabadság', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 47, NULL, 40, NULL),
            (8404, N'Első házasság utáni pótszabadság', N'Első házasság utáni pótszabadság', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 48, NULL, 41, NULL),
            (8405, N'Igazolatlan törtnapi', N'Igazolatlan törtnapi', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 49, NULL, 42, NULL),
            (8406, N'Veszélyhelyzet miatti fizetés nélküli szabadság biztosítás szünetel', N'Veszélyhelyzet miatti fizetés nélküli szabadság biztosítás szünetel', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 50, NULL, 43, NULL),
            (9085, N'Tanulmányi szabadság illetmény számfejtéssel', N'Tanulmányi szabadság illetmény számfejtéssel', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 54, NULL, 44, NULL),
            (9086, N'Egyéb hiányzás díjazás nélkül (óra)', N'Egyéb hiányzás díjazás nélkül (óra)', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 111, NULL, 45, NULL),
            (9087, N'Szakszervezeti tisztségviselő munkaidő-kedvezménye (óra)', N'Szakszervezeti tisztségviselő munkaidő-kedvezménye (óra)', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 113, NULL, 46, NULL),
            (9088, N'Alkotói szabadság díjazás nélkül', N'Alkotói szabadság díjazás nélkül', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 55, NULL, 47, NULL),
            (9089, N'Keresőképtelenség (óra)', N'Keresőképtelenség (óra)', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 115, NULL, 48, NULL),
            (9090, N'Sztrájk (óra)', N'Sztrájk (óra)', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 116, NULL, 49, NULL),
            (9091, N'Különös méltánylást érdemlő személyi, családi vagy elháríthatatlan ok miatt indokolt távollét tartamára mentesülés munkavégzés alól (óra)', N'Különös méltánylást érdemlő személyi, családi vagy elháríthatatlan ok miatt indokolt távollét tartamára mentesülés munkavégzés alól (óra)', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 117, NULL, 50, NULL),
            (9092, N'Kötelező orvosi vizsgálat tartamára mentesülés munkavégzés alól (óra)', N'Kötelező orvosi vizsgálat tartamára mentesülés munkavégzés alól (óra)', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 125, NULL, 51, NULL),
            (9093, N'Véradáshoz szükséges időtartamra mentesülés munkavégzés alól (óra)', N'Véradáshoz szükséges időtartamra mentesülés munkavégzés alól (óra)', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 126, NULL, 52, NULL),
            (9094, N'Szoptatás időtartamára mentesülés munkavégzés alól (óra)', N'Szoptatás időtartamára mentesülés munkavégzés alól (óra)', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 127, NULL, 53, NULL),
            (9095, N'Tanúkénti meghallgatás (óra)', N'Tanúkénti meghallgatás (óra)', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 129, NULL, 54, NULL),
            (9096, N'Képzésben, továbbképzésben felek megállapodása alapján való részvételhez szükséges időtartamra mentesülés munkavégzés alól (óra)', N'Képzésben, továbbképzésben felek megállapodása alapján való részvételhez szükséges időtartamra mentesülés munkavégzés alól (óra)', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 134, NULL, 55, NULL),
            (9097, N'Önkéntes vagy létesítményi tűzoltói szolgálat ellátásának tartamára mentesülés munkavégzés alól (óra)', N'Önkéntes vagy létesítményi tűzoltói szolgálat ellátásának tartamára mentesülés munkavégzés alól (óra)', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 135, NULL, 56, NULL),
            (9098, N'Bíróság, hatóság felhívására vagy az eljárásban való részvételhez szükséges időtartamra mentesülés munkavégzés alól (óra)', N'Bíróság, hatóság felhívására vagy az eljárásban való részvételhez szükséges időtartamra mentesülés munkavégzés alól (óra)', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 136, NULL, 57, NULL),
            (9099, N'Munkáltató engedélye alapján (óra)', N'Munkáltató engedélye alapján (óra)', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 137, NULL, 58, NULL),
            (9100, N'Állásidő díjazással (óra)', N'Állásidő díjazással (óra)', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 138, NULL, 59, NULL),
            (9101, N'Kötelezettségszegés körülményei kivizsgálásának időtartamára mentesítés munkavégzés alól (max. 30 nap) (óra)', N'Kötelezettségszegés körülményei kivizsgálásának időtartamára mentesítés munkavégzés alól (max. 30 nap) (óra)', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 142, NULL, 60, NULL),
            (9102, N'Állásidő díjazás nélkül (óra)', N'Állásidő díjazás nélkül (óra)', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 145, NULL, 61, NULL),
            (9103, N'Örökbefogadás előkészítése céljából', N'Örökbefogadás előkészítése céljából', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 51, NULL, 62, NULL),
            (9104, N'Védőoltás hiánya miatti fizetés nélküli szabadság', N'Védőoltás hiánya miatti fizetés nélküli szabadság', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'T', 52, NULL, 63, NULL),
            (9105, N'Születésnapi szabadság', N'Születésnapi szabadság', NULL, NULL, NULL, N'TavolletTipus', 134, N'T', N'F', 53, NULL, 64, NULL),
            (7574, N'Függő', N'Függő', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 1, NULL, 1, NULL),
            (7575, N'Elfogadva átadva tankerületnek', N'Elfogadva átadva tankerületnek', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 2, NULL, 2, NULL),
            (7576, N'Elutasítva', N'Elutasítva', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 3, NULL, 3, NULL),
            (7577, N'Hiánypótlásra visszaküldve', N'Hiánypótlásra visszaküldve', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 4, NULL, 4, NULL),
            (7652, N'Tankerület által elfogadva', N'Tankerület által elfogadva', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 5, NULL, 5, NULL),
            (7653, N'Tankerület által elutasítva', N'Tankerület által elutasítva', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 6, NULL, 6, NULL),
            (7654, N'Tankerület által hiánypótlásra visszaküldve', N'Tankerület által hiánypótlásra visszaküldve', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 7, NULL, 7, NULL),
            (7681, N'Engedélyezve átadva tankerületnek', N'Engedélyezve átadva tankerületnek', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 8, NULL, 8, NULL),
            (7682, N'Tudomásul véve átadva tankerületnek', N'Tudomásul véve átadva tankerületnek', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 9, NULL, 9, NULL),
            (7691, N'Munkaügynek átadva', N'Munkaügynek átadva', NULL, NULL, NULL, N'TavolletStatusz', 135, N'T', N'T', 10, NULL, 10, NULL),
            (7557, N'Tanuló', N'Tanuló', NULL, NULL, NULL, N'DokumentumElemiSzint', 133, N'T', N'T', 1, NULL, 1, NULL),
            (7558, N'Csoport', N'Csoport', NULL, NULL, NULL, N'DokumentumElemiSzint', 133, N'T', N'T', 2, NULL, 2, NULL),
            (7559, N'Osztály', N'Osztály', NULL, NULL, NULL, N'DokumentumElemiSzint', 133, N'T', N'T', 3, NULL, 3, NULL),
            (7560, N'Intézmény', N'Intézmény', NULL, NULL, NULL, N'DokumentumElemiSzint', 133, N'T', N'T', 4, NULL, 4, NULL),
            (7678, N'Terem', N'Terem', NULL, NULL, NULL, N'DokumentumElemiSzint', 133, N'T', N'T', 5, NULL, 5, NULL),
            (7679, N'Gondviselő', N'Gondviselő', NULL, NULL, NULL, N'DokumentumElemiSzint', 133, N'T', N'T', 6, NULL, 6, NULL),
            (7553, N'Nevelés-oktatás', N'Nevelés-oktatás', NULL, NULL, NULL, N'OktNevelesiKategoria', 132, N'T', N'T', 1, NULL, 1, NULL),
            (7554, N'Felnőttképzés', N'Felnőttképzés', NULL, NULL, NULL, N'OktNevelesiKategoria', 132, N'T', N'T', 2, NULL, 2, NULL),
            (7555, N'Alapfokú művészetoktatás', N'Alapfokú művészetoktatás', NULL, NULL, NULL, N'OktNevelesiKategoria', 132, N'T', N'T', 3, NULL, 3, NULL),
            (7556, N'Kollégium', N'Kollégium', NULL, NULL, NULL, N'OktNevelesiKategoria', 132, N'T', N'T', 4, NULL, 4, NULL),
            (7717, N'EGYMI', N'EGYMI', NULL, NULL, NULL, N'OktNevelesiKategoria', 132, N'T', N'T', 5, NULL, 5, NULL),
            (7421, N'Osztálynapló', N'Osztálynapló', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 1, N'16', 1, NULL),
            (7422, N'Haladási napló', N'Haladási napló', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 2, N'16', 2, NULL),
            (7423, N'Osztályozó napló', N'Osztályozó napló', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 3, N'16', 3, NULL),
            (7424, N'Csoport napló', N'Csoport napló', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 4, N'16', 4, NULL),
            (7425, N'Napközis csoport naplója', N'Napközis csoport naplója', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 5, N'16', 5, NULL),
            (7426, N'Gyakorlati foglalkozásról vezetett napló', N'Gyakorlati foglalkozásról vezetett napló', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 6, N'16', 6, NULL),
            (7427, N'Egyéb foglalkozási napló', N'Egyéb foglalkozási napló', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 7, N'16', 7, NULL),
            (7428, N'Osztályok órarendje', N'Osztályok órarendje', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 8, N'47', 8, NULL),
            (7429, N'Pedagógus órarendje', N'Pedagógus órarendje', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 9, N'47', 9, NULL),
            (7430, N'Termek órarendje', N'Termek órarendje', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 10, N'47', 10, NULL),
            (7431, N'Közösségi szolgálati jelentkezési lap', N'Közösségi szolgálati jelentkezési lap', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 11, N'27', 11, NULL),
            (7432, N'Osztály tanulóinak ellenőrzőkönyve', N'Osztály tanulóinak ellenőrzőkönyve', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 12, N'47', 12, NULL),
            (7433, N'Tanulói adatlapok', N'Tanulói adatlapok', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 13, N'47', 13, NULL),
            (7434, N'Értékelések részletezése', N'Értékelések részletezése', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 14, N'47', 14, NULL),
            (7435, N'Tantárgyi mulasztások részletezése', N'Tantárgyi mulasztások részletezése', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 15, N'47', 15, NULL),
            (7436, N'Feljegyzések részletezése', N'Feljegyzések részletezése', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 16, N'47', 16, NULL),
            (7437, N'Felmentések / határozatok', N'Felmentések / határozatok', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 17, N'24', 17, NULL),
            (7438, N'Haladási munkafüzet osztályonként', N'Haladási munkafüzet osztályonként', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 18, N'47', 18, NULL),
            (7439, N'Haladási munkafüzet tanáronként', N'Haladási munkafüzet tanáronként', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 19, N'47', 19, NULL),
            (7440, N'Pedagógus által adott osztályzatok / értékelések', N'Pedagógus által adott osztályzatok / értékelések', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 20, N'47', 20, NULL),
            (7441, N'Munkaidő elszámolólap (+ Hónap)', N'Munkaidő elszámolólap (+ Hónap)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 21, N'3', 21, NULL),
            (7442, N'Nem kötött munkaidő részletező (heti bontás)', N'Nem kötött munkaidő részletező (heti bontás)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 22, N'47', 22, NULL),
            (7443, N'Nem kötött munkaidő részletező (havi bontás) (+ Hónap)', N'Nem kötött munkaidő részletező (havi bontás) (+ Hónap)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 23, N'47', 23, NULL),
            (7444, N'Helyettesítések listája (+ Hét sorszáma)', N'Helyettesítések listája (+ Hét sorszáma)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 24, N'3', 24, NULL),
            (7445, N'Helyettesítési napló', N'Helyettesítési napló', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 25, N'3', 25, NULL),
            (7446, N'Iskolalátogatási lap', N'Iskolalátogatási lap', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 26, N'24', 26, NULL),
            (7447, N'Részletes alkalmazott adatok', N'Részletes alkalmazott adatok', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 27, N'47', 27, NULL),
            (7448, N'Tanulók tantárgyi mulasztása tanórák alapján (osztályonként)', N'Tanulók tantárgyi mulasztása tanórák alapján (osztályonként)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 28, N'24', 28, NULL),
            (7449, N'Tanulók gyakorlati mulasztása tanórák alapján (osztályonként)', N'Tanulók gyakorlati mulasztása tanórák alapján (osztályonként)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 29, N'24', 29, NULL),
            (7450, N'Osztály tanulóinak havi mulasztásai összesítője', N'Osztály tanulóinak havi mulasztásai összesítője', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 30, N'24', 30, NULL),
            (7451, N'Osztály félévi statisztika', N'Osztály félévi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 31, N'47', 31, NULL),
            (7452, N'Osztály év végi statisztika', N'Osztály év végi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 32, N'10', 32, NULL),
            (7453, N'Osztály I. negyedévi statisztika', N'Osztály I. negyedévi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 33, N'47', 33, NULL),
            (7454, N'Osztály II. negyedévi statisztika', N'Osztály II. negyedévi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 34, N'47', 34, NULL),
            (7455, N'Osztály III. negyedévi statisztika', N'Osztály III. negyedévi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 35, N'47', 35, NULL),
            (7456, N'Osztály IV. negyedévi statisztika', N'Osztály IV. negyedévi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 36, N'47', 36, NULL),
            (7457, N'Osztályok félévi tantárgyi statisztika', N'Osztályok félévi tantárgyi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 37, N'47', 37, NULL),
            (7458, N'Osztályok év végi tantárgyi statisztika', N'Osztályok év végi tantárgyi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 38, N'47', 38, NULL),
            (7459, N'Osztályok I. negyedévi tantárgyi statisztika', N'Osztályok I. negyedévi tantárgyi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 39, N'47', 39, NULL),
            (7460, N'Osztályok II. negyedévi tantárgyi statisztika', N'Osztályok II. negyedévi tantárgyi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 40, N'47', 40, NULL),
            (7461, N'Osztályok III. negyedévi tantárgyi statisztika', N'Osztályok III. negyedévi tantárgyi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 41, N'47', 41, NULL),
            (7462, N'Osztályok IV. negyedévi tantárgyi statisztika', N'Osztályok IV. negyedévi tantárgyi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 42, N'47', 42, NULL),
            (7463, N'Osztályok/csoportok félévi tantárgyi statisztika', N'Osztályok/csoportok félévi tantárgyi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 43, N'47', 43, NULL),
            (7464, N'Osztályok/csoportok év végi tantárgyi statisztika', N'Osztályok/csoportok év végi tantárgyi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 44, N'47', 44, NULL),
            (7465, N'Osztályok/csoportok I. negyedévi tantárgyi statisztika', N'Osztályok/csoportok I. negyedévi tantárgyi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 45, N'47', 45, NULL),
            (7466, N'Osztályok/csoportok II. negyedévi tantárgyi statisztika', N'Osztályok/csoportok II. negyedévi tantárgyi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 46, N'47', 46, NULL),
            (7467, N'Osztályok/csoportok III. negyedévi tantárgyi statisztika', N'Osztályok/csoportok III. negyedévi tantárgyi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 47, N'47', 47, NULL),
            (7468, N'Osztályok/csoportok IV. negyedévi tantárgyi statisztika', N'Osztályok/csoportok IV. negyedévi tantárgyi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 48, N'47', 48, NULL),
            (7469, N'Tanulók félévi tantárgyi statisztika', N'Tanulók félévi tantárgyi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 49, N'47', 49, NULL),
            (7470, N'Tanulók év végi tantárgyi statisztika', N'Tanulók év végi tantárgyi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 50, N'47', 50, NULL),
            (7471, N'Tanulók I. negyedévi tantárgyi statisztika', N'Tanulók I. negyedévi tantárgyi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 51, N'47', 51, NULL),
            (7472, N'Tanulók II. negyedévi tantárgyi statisztika', N'Tanulók II. negyedévi tantárgyi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 52, N'47', 52, NULL),
            (7473, N'Tanulók III. negyedévi tantárgyi statisztika', N'Tanulók III. negyedévi tantárgyi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 53, N'47', 53, NULL),
            (7474, N'Tanulók IV. negyedévi tantárgyi statisztika', N'Tanulók IV. negyedévi tantárgyi statisztika', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 54, N'47', 54, NULL),
            (7475, N'Félévi értesítő', N'Félévi értesítő', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 55, N'36', 55, NULL),
            (7476, N'Félévi értesítő (ellenőrzőben elhelyezhető)', N'Félévi értesítő (ellenőrzőben elhelyezhető)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 56, N'36', 56, NULL),
            (7477, N'Félévi értesítő (ellenőrzőben elhelyezhető A/5 méretben)', N'Félévi értesítő (ellenőrzőben elhelyezhető A/5 méretben)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 57, N'36', 57, NULL),
            (7478, N'Félévi szöveges értesítő (1-4. osztály)', N'Félévi szöveges értesítő (1-4. osztály)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 58, N'36', 58, NULL),
            (7479, N'Félévi szöveges értesítő füzet (1-4. osztály)', N'Félévi szöveges értesítő füzet (1-4. osztály)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 59, N'36', 59, NULL),
            (7480, N'Félévi értesítő (művészoktatásban)', N'Félévi értesítő (művészoktatásban)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 60, N'36', 60, NULL),
            (7481, N'Év végi értesítő', N'Év végi értesítő', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 61, N'36', 61, NULL),
            (7482, N'Év végi értesítő (ellenőrzőben elhelyezhető)', N'Év végi értesítő (ellenőrzőben elhelyezhető)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 62, N'36', 62, NULL),
            (7483, N'Év végi értesítő (ellenőrzőben elhelyezhető A/5 méretben)', N'Év végi értesítő (ellenőrzőben elhelyezhető A/5 méretben)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 63, N'36', 63, NULL),
            (7484, N'Év végi szöveges értesítő (1-4. osztály)', N'Év végi szöveges értesítő (1-4. osztály)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 64, N'36', 64, NULL),
            (7485, N'Év végi szöveges értesítő füzet (1-4. osztály)', N'Év végi szöveges értesítő füzet (1-4. osztály)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 65, N'36', 65, NULL),
            (7486, N'Évi értesítő (művészoktatásban)', N'Évi értesítő (művészoktatásban)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 66, N'36', 66, NULL),
            (7487, N'I. negyedévi értesítő', N'I. negyedévi értesítő', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 67, N'36', 67, NULL),
            (7488, N'I. negyedévi értesítő (ellenőrzőben elhelyezhető)', N'I. negyedévi értesítő (ellenőrzőben elhelyezhető)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 68, N'36', 68, NULL),
            (7489, N'I. negyedévi értesítő (ellenőrzőben elhelyezhető A/5 méretben)', N'I. negyedévi értesítő (ellenőrzőben elhelyezhető A/5 méretben)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 69, N'36', 69, NULL),
            (7490, N'I. negyedévi szöveges értesítő (1-4. osztály)', N'I. negyedévi szöveges értesítő (1-4. osztály)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 70, N'36', 70, NULL),
            (7491, N'I. negyedévi szöveges értesítő füzet (1-4. osztály)', N'I. negyedévi szöveges értesítő füzet (1-4. osztály)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 71, N'36', 71, NULL),
            (7492, N'I. negyedévi értesítő (művészoktatásban)', N'I. negyedévi értesítő (művészoktatásban)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 72, N'36', 72, NULL),
            (7493, N'II. negyedévi értesítő', N'II. negyedévi értesítő', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 73, N'36', 73, NULL),
            (7494, N'II. negyedévi értesítő (ellenőrzőben elhelyezhető)', N'II. negyedévi értesítő (ellenőrzőben elhelyezhető)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 74, N'36', 74, NULL),
            (7495, N'II. negyedévi értesítő (ellenőrzőben elhelyezhető A/5 méretben)', N'II. negyedévi értesítő (ellenőrzőben elhelyezhető A/5 méretben)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 75, N'36', 75, NULL),
            (7496, N'II. negyedévi szöveges értesítő (1-4. osztály)', N'II. negyedévi szöveges értesítő (1-4. osztály)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 76, N'36', 76, NULL),
            (7497, N'II. negyedévi szöveges értesítő füzet (1-4. osztály)', N'II. negyedévi szöveges értesítő füzet (1-4. osztály)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 77, N'36', 77, NULL),
            (7498, N'II. negyedévi értesítő (művészoktatásban)', N'II. negyedévi értesítő (művészoktatásban)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 78, N'36', 78, NULL),
            (7499, N'III. negyedévi értesítő', N'III. negyedévi értesítő', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 79, N'36', 79, NULL),
            (7500, N'III. negyedévi értesítő (ellenőrzőben elhelyezhető)', N'III. negyedévi értesítő (ellenőrzőben elhelyezhető)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 80, N'36', 80, NULL),
            (7501, N'III. negyedévi értesítő (ellenőrzőben elhelyezhető A/5 méretben)', N'III. negyedévi értesítő (ellenőrzőben elhelyezhető A/5 méretben)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 81, N'36', 81, NULL),
            (7502, N'III. negyedévi szöveges értesítő (1-4. osztály)', N'III. negyedévi szöveges értesítő (1-4. osztály)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 82, N'36', 82, NULL),
            (7503, N'III. negyedévi szöveges értesítő füzet (1-4. osztály)', N'III. negyedévi szöveges értesítő füzet (1-4. osztály)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 83, N'36', 83, NULL),
            (7504, N'III. negyedévi értesítő (művészoktatásban)', N'III. negyedévi értesítő (művészoktatásban)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 84, N'36', 84, NULL),
            (7505, N'IV. negyedévi értesítő', N'IV. negyedévi értesítő', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 85, N'36', 85, NULL),
            (7506, N'IV. negyedévi értesítő (ellenőrzőben elhelyezhető)', N'IV. negyedévi értesítő (ellenőrzőben elhelyezhető)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 86, N'36', 86, NULL),
            (7507, N'IV. negyedévi értesítő (ellenőrzőben elhelyezhető A/5 méretben)', N'IV. negyedévi értesítő (ellenőrzőben elhelyezhető A/5 méretben)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 87, N'36', 87, NULL),
            (7508, N'IV. negyedévi szöveges értesítő (1-4. osztály)', N'IV. negyedévi szöveges értesítő (1-4. osztály)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 88, N'36', 88, NULL),
            (7509, N'IV. negyedévi szöveges értesítő füzet (1-4. osztály)', N'IV. negyedévi szöveges értesítő füzet (1-4. osztály)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 89, N'36', 89, NULL),
            (7510, N'IV. negyedévi értesítő (művészoktatásban)', N'IV. negyedévi értesítő (művészoktatásban)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 90, N'36', 90, NULL),
            (7511, N'Bizonyítvány pótlap az 1. évfolyamon A.Tü. 800/gépi r.sz.', N'Bizonyítvány pótlap az 1. évfolyamon A.Tü. 800/gépi r.sz.', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 91, N'13', 91, NULL),
            (7512, N'Bizonyítvány pótlap a 2. évfolyamtól A.Tü. 800/gépi r.sz.', N'Bizonyítvány pótlap a 2. évfolyamtól A.Tü. 800/gépi r.sz.', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 92, N'13', 92, NULL),
            (7513, N'Bizonyítvány pótlap az 1. évfolyamon A.Tü. 800/gépi r.sz. (német)', N'Bizonyítvány pótlap az 1. évfolyamon A.Tü. 800/gépi r.sz. (német)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 93, N'13', 93, NULL),
            (7514, N'Bizonyítvány pótlap a 2. évfolyamtól A.Tü. 800/gépi r.sz.(német)', N'Bizonyítvány pótlap a 2. évfolyamtól A.Tü. 800/gépi r.sz.(német)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 94, N'13', 94, NULL),
            (7515, N'Bizonyítvány pótlap az 1. évfolyamon A.Tü. 800/gépi r.sz. (angol)', N'Bizonyítvány pótlap az 1. évfolyamon A.Tü. 800/gépi r.sz. (angol)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 95, N'13', 95, NULL),
            (7516, N'Bizonyítvány pótlap a 2. évfolyamtól A.Tü. 800/gépi r.sz. (angol)', N'Bizonyítvány pótlap a 2. évfolyamtól A.Tü. 800/gépi r.sz. (angol)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 96, N'13', 96, NULL),
            (7517, N'Törzslap az 1.-3. és felsőbb évfolyamok szöveges minősítéséhez', N'Törzslap az 1.-3. és felsőbb évfolyamok szöveges minősítéséhez', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 97, N'13', 97, NULL),
            (7518, N'Törzslap az 1.-3. és felsőbb évfolyamok szöveges minősítéséhez (művészetoktatásban)', N'Törzslap az 1.-3. és felsőbb évfolyamok szöveges minősítéséhez (művészetoktatásban)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 98, N'13', 98, NULL),
            (7519, N'Törzslapkivonat a 4. és felsőbb évfolyamokhoz (gépi)', N'Törzslapkivonat a 4. és felsőbb évfolyamokhoz (gépi)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 99, N'13', 99, NULL),
            (7520, N'Törzslapkivonat a 4. és felsőbb évfolyamokhoz (gépi, művészetoktatásban)', N'Törzslapkivonat a 4. és felsőbb évfolyamokhoz (gépi, művészetoktatásban)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 100, N'13', 100, NULL),
            (7521, N'Törzslap az 1.-3. és felsőbb évfolyamok szöveges minősítéséhez (német, gépi)', N'Törzslap az 1.-3. és felsőbb évfolyamok szöveges minősítéséhez (német, gépi)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 101, N'13', 101, NULL),
            (7522, N'Törzslapkivonat a 4. és felsőbb évfolyamokhoz (német, gépi)', N'Törzslapkivonat a 4. és felsőbb évfolyamokhoz (német, gépi)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 102, N'13', 102, NULL),
            (7523, N'Törzslap az 1.-3. és felsőbb évfolyamok szöveges minősítéséhez (angol, gépi)', N'Törzslap az 1.-3. és felsőbb évfolyamok szöveges minősítéséhez (angol, gépi)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 103, N'13', 103, NULL),
            (7524, N'Törzslapkivonat a 4. és felsőbb évfolyamokhoz (angol, gépi)', N'Törzslapkivonat a 4. és felsőbb évfolyamokhoz (angol, gépi)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 104, N'13', 104, NULL),
            (7525, N'Üzenőfüzet értékelőlap', N'Üzenőfüzet értékelőlap', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 105, N'47', 105, NULL),
            (7526, N'Félévi Tanulói értesítő (Waldorf iskoláknak)', N'Félévi Tanulói értesítő (Waldorf iskoláknak)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 106, N'36', 106, NULL),
            (7527, N'Év végi Tanulói értesítő (Waldorf iskoláknak)', N'Év végi Tanulói értesítő (Waldorf iskoláknak)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 107, N'36', 107, NULL),
            (7528, N'I. negyedévi Tanulói értesítő (Waldorf iskoláknak)', N'I. negyedévi Tanulói értesítő (Waldorf iskoláknak)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 108, N'36', 108, NULL),
            (7529, N'II. negyedévi Tanulói értesítő (Waldorf iskoláknak)', N'II. negyedévi Tanulói értesítő (Waldorf iskoláknak)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 109, N'36', 109, NULL),
            (7530, N'III. negyedévi Tanulói értesítő (Waldorf iskoláknak)', N'III. negyedévi Tanulói értesítő (Waldorf iskoláknak)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 110, N'36', 110, NULL),
            (7531, N'IV. negyedévi Tanulói értesítő (Waldorf Iskoláknak)', N'IV. negyedévi Tanulói értesítő (Waldorf Iskoláknak)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 111, N'36', 111, NULL),
            (7532, N'Tanulói jogviszony igazoló lap', N'Tanulói jogviszony igazoló lap', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 112, N'24', 112, NULL),
            (7533, N'Értesítés szülőnek (tanköteles)', N'Értesítés szülőnek (tanköteles)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 113, N'24', 113, NULL),
            (7534, N'Értesítés szülőnek (nem tanköteles)', N'Értesítés szülőnek (nem tanköteles)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 114, N'24', 114, NULL),
            (7535, N'Értesítés kormányhivatal részére (tanköteles)', N'Értesítés kormányhivatal részére (tanköteles)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 115, N'24', 115, NULL),
            (7536, N'Értesítés kormányhivatal részére (nem tanköteles)', N'Értesítés kormányhivatal részére (nem tanköteles)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 116, N'24', 116, NULL),
            (7537, N'Értesítés gyermekjóléti szolgálat részére (tanköteles)', N'Értesítés gyermekjóléti szolgálat részére (tanköteles)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 117, N'24', 117, NULL),
            (7538, N'Értesítés gyermekjóléti szolgálat részére (nem tanköteles)', N'Értesítés gyermekjóléti szolgálat részére (nem tanköteles)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 118, N'24', 118, NULL),
            (7539, N'Értesítés gyermekvédelmi szolgálat részére (tanköteles)', N'Értesítés gyermekvédelmi szolgálat részére (tanköteles)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 119, N'24', 119, NULL),
            (7540, N'Értesítés gyermekvédelmi szolgálat részére (nem tanköteles)', N'Értesítés gyermekvédelmi szolgálat részére (nem tanköteles)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 120, N'24', 120, NULL),
            (7541, N'Tanév közben kilépett tanulók', N'Tanév közben kilépett tanulók', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 121, N'47', 121, NULL),
            (7542, N'Osztályok éves óraszámai (összesítő segédlet a törzslapon megjelenő éves óraszámok ellenőrzéséhez)', N'Osztályok éves óraszámai (összesítő segédlet a törzslapon megjelenő éves óraszámok ellenőrzéséhez)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 122, N'47', 122, NULL),
            (7543, N'Naplóba be nem jegyzett foglalkozások (órarend alapján)', N'Naplóba be nem jegyzett foglalkozások (órarend alapján)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 123, N'47', 123, NULL),
            (7544, N'Pedagógusok naplóba be nem jegyzett órái (összes, órarend alapján)', N'Pedagógusok naplóba be nem jegyzett órái (összes, órarend alapján)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 124, N'47', 124, NULL),
            (7545, N'Véglegesített tantárgyfelosztások', N'Véglegesített tantárgyfelosztások', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 125, N'23', 125, NULL),
            (7546, N'Tantárgyfelosztás formázott Excel', N'Tantárgyfelosztás formázott Excel', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 126, N'47', 126, NULL),
            (7547, N'Tantárgyfelosztás formázott PDF', N'Tantárgyfelosztás formázott PDF', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 127, N'47', 127, NULL),
            (7548, N'Tantárgyfelosztás kiegészített adatok', N'Tantárgyfelosztás kiegészített adatok', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 128, N'47', 128, NULL),
            (7549, N'Osztályok száma riport', N'Osztályok száma riport', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 129, N'47', 129, NULL),
            (7550, N'Csoportok száma riport', N'Csoportok száma riport', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 130, N'47', 130, NULL),
            (7551, N'Feladatellátási helyek riport', N'Feladatellátási helyek riport', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 131, N'47', 131, NULL),
            (7552, N'Tanulók megoszlása évfolyamok és ágazatok szerint', N'Tanulók megoszlása évfolyamok és ágazatok szerint', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 132, N'47', 132, NULL),
            (7567, N'Félévi szöveges értesítő füzet (1-4. osztály) kétoldalas nyomtatáshoz', N'Félévi szöveges értesítő füzet (1-4. osztály) kétoldalas nyomtatáshoz', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 133, N'36', 133, NULL),
            (7569, N'Év végi szöveges értesítő füzet - nyomtatványra (1-4. osztály)', N'Év végi szöveges értesítő füzet - nyomtatványra (1-4. osztály)', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 134, N'36', 134, NULL),
            (7675, N'Elfogadott tantárgyfelosztás formázott Excel', N'Elfogadott tantárgyfelosztás formázott Excel', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 135, N'23', 135, NULL),
            (7676, N'Elfogadott tantárgyfelosztás formázott PDF', N'Elfogadott tantárgyfelosztás formázott PDF', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 136, N'23', 136, NULL),
            (7677, N'Részletes tanulói adatok', N'Részletes tanulói adatok', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 137, N'47', 137, NULL),
            (7689, N'Tanulók mulasztásai', N'Tanulók mulasztásai', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 135, N'47', 138, NULL),
            (7693, N'Alapító okirat', N'Alapító okirat', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 138, N'1', 139, NULL),
            (7694, N'SZMSZ', N'SZMSZ', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 139, N'8', 140, NULL),
            (7695, N'Pedagógiai program/Szakmai program', N'Pedagógiai program/Szakmai program', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 140, N'8', 141, NULL),
            (7696, N'Házirend', N'Házirend', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 141, N'8', 142, NULL),
            (7697, N'Munkaterv', N'Munkaterv', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 142, N'10', 143, NULL),
            (7809, N'Távollét', N'Távollét', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 143, N'3', 144, NULL),
            (8616, N'Működési engedély', N'Működési engedély', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 143, N'1', 145, NULL),
            (8619, N'Pedagógusigazolvány helyett kiállított igazolás', N'Pedagógusigazolvány helyett kiállított igazolás', NULL, NULL, NULL, N'DokumentumTipus', 131, N'T', N'T', 144, N'3', 146, NULL),
            (7387, N'Iktatószámra vár', N'Iktatószámra vár', NULL, NULL, NULL, N'DokumentumStatusz', 128, N'T', N'T', 1, NULL, 1, NULL),
            (7388, N'Iktatásra vár', N'Iktatásra vár', NULL, NULL, NULL, N'DokumentumStatusz', 128, N'T', N'T', 2, NULL, 2, NULL),
            (7389, N'Iktatva', N'Iktatva', NULL, NULL, NULL, N'DokumentumStatusz', 128, N'T', N'T', 3, NULL, 3, NULL),
            (7566, N'Hibás', N'Hibás', NULL, NULL, NULL, N'DokumentumStatusz', 128, N'T', N'T', 4, NULL, 4, NULL),
            (7399, N'Osztály név', N'Osztály név', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 1, NULL, 1, NULL),
            (7400, N'Évfolyam', N'Évfolyam', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 10, NULL, 2, NULL),
            (7401, N'Feladatellátási hely', N'Feladatellátási hely', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 20, NULL, 3, NULL),
            (7402, N'Képzés típus', N'Képzés típus', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 30, NULL, 4, NULL),
            (7403, N'Osztályfőnök', N'Osztályfőnök', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 40, NULL, 5, NULL),
            (7404, N'Tanterv', N'Tanterv', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 50, NULL, 6, NULL),
            (7405, N'Osztályjellemző', N'Osztályjellemző', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 60, NULL, 7, NULL),
            (7406, N'Szakcsoport', N'Szakcsoport', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 130, NULL, 8, NULL),
            (7407, N'Ágazat', N'Ágazat', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 140, NULL, 9, NULL),
            (7408, N'Szakképesítés', N'Szakképesítés', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 150, NULL, 10, NULL),
            (7409, N'Részszakképesítés', N'Részszakképesítés', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 160, NULL, 11, NULL),
            (7410, N'Csoport név', N'Csoport név', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 170, NULL, 12, NULL),
            (7411, N'Csoport típus', N'Csoport típus', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 180, NULL, 13, NULL),
            (7412, N'Csoport vezető', N'Csoport vezető', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 190, NULL, 14, NULL),
            (7413, N'Csoportjellemző', N'Csoportjellemző', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 200, NULL, 15, NULL),
            (7414, N'Tanuló', N'Tanuló', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 210, NULL, 16, NULL),
            (7415, N'Alkalmazott', N'Alkalmazott', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 220, NULL, 17, NULL),
            (7416, N'Gondviselő', N'Gondviselő', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 230, NULL, 18, NULL),
            (7417, N'Tanuló oktatási azonosító', N'Tanuló oktatási azonosító', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 240, NULL, 19, NULL),
            (7418, N'Alkalmazott oktatási azonosító', N'Alkalmazott oktatási azonosító', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 250, NULL, 20, NULL),
            (7419, N'Dokumentum típus', N'Dokumentum típus', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 260, NULL, 21, NULL),
            (7420, N'Iktató személy', N'Iktató személy', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 270, NULL, 22, NULL),
            (7561, N'Tanuló egyedi azonosító', N'Tanuló egyedi azonosító', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 280, NULL, 23, NULL),
            (7562, N'Alkalmazott egyedi azonosító', N'Alkalmazott egyedi azonosító', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 290, NULL, 24, NULL),
            (7674, N'Terem', N'Terem', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 300, NULL, 25, NULL),
            (7810, N'Távollét oka', N'Távollét oka', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 310, NULL, 26, NULL),
            (7811, N'Igénylő SZTSZ száma', N'Igénylő SZTSZ száma', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 320, NULL, 27, NULL),
            (7812, N'Igénylő neve', N'Igénylő neve', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 330, NULL, 28, NULL),
            (7813, N'Rögzítő neve', N'Rögzítő neve', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 340, NULL, 29, NULL),
            (7814, N'Távollét napok száma', N'Távollét napok száma', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 350, NULL, 30, NULL),
            (8620, N'Munkakör', N'Munkakör', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 360, NULL, 31, NULL),
            (8621, N'Foglalkoztatási jogviszony', N'Foglalkoztatási jogviszony', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 370, NULL, 32, NULL),
            (8696, N'Új Szkt. - Ágazat', N'Új Szkt. - Ágazat', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 70, NULL, 33, NULL),
            (8697, N'Új Szkt. - Szakma', N'Új Szkt. - Szakma', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 80, NULL, 34, NULL),
            (8698, N'Új Szkt. - Szakmairány', N'Új Szkt. - Szakmairány', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 90, NULL, 35, NULL),
            (8699, N'Nkt. - Tanulmányi terület', N'Nkt. - Tanulmányi terület', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 100, NULL, 36, NULL),
            (8700, N'Nkt. - Szakképesítés', N'Nkt. - Szakképesítés', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 110, NULL, 37, NULL),
            (8701, N'Nkt. - Szakirány', N'Nkt. - Szakirány', NULL, NULL, NULL, N'DokumentumKulcsszoTipus', 130, N'T', N'T', 120, NULL, 38, NULL),
            (7390, N'Naplók', N'Naplók', NULL, NULL, NULL, N'DokumentumKategoria', 129, N'T', N'T', 1, NULL, 1, NULL),
            (7391, N'Órarendek', N'Órarendek', NULL, NULL, NULL, N'DokumentumKategoria', 129, N'T', N'T', 2, NULL, 2, NULL),
            (7392, N'Tanulók', N'Tanulók', NULL, NULL, NULL, N'DokumentumKategoria', 129, N'T', N'T', 3, NULL, 3, NULL),
            (7393, N'Pedagógusok', N'Pedagógusok', NULL, NULL, NULL, N'DokumentumKategoria', 129, N'T', N'T', 4, NULL, 4, NULL),
            (7394, N'Termek', N'Termek', NULL, NULL, NULL, N'DokumentumKategoria', 129, N'T', N'T', 5, NULL, 5, NULL),
            (7395, N'Statisztikák', N'Statisztikák', NULL, NULL, NULL, N'DokumentumKategoria', 129, N'T', N'T', 6, NULL, 6, NULL),
            (7396, N'Értesítők, törzslap, bizonyítvány', N'Értesítők, törzslap, bizonyítvány', NULL, NULL, NULL, N'DokumentumKategoria', 129, N'T', N'T', 7, NULL, 7, NULL),
            (7397, N'Igazolások, mulasztási értesítők', N'Igazolások, mulasztási értesítők', NULL, NULL, NULL, N'DokumentumKategoria', 129, N'T', N'T', 8, NULL, 8, NULL),
            (7398, N'Tanügyi ellenőrzés', N'Tanügyi ellenőrzés', NULL, NULL, NULL, N'DokumentumKategoria', 129, N'T', N'T', 9, NULL, 9, NULL),
            (7692, N'Alapdokumentumok', N'Alapdokumentumok', NULL, NULL, NULL, N'DokumentumKategoria', 129, N'T', N'T', 10, NULL, 10, NULL),
            (7808, N'Személyzeti ügyek', N'Személyzeti ügyek', NULL, NULL, NULL, N'DokumentumKategoria', 129, N'T', N'T', 11, NULL, 11, NULL),
            (7357, N'Tantárgy', N'Tantárgy', NULL, NULL, NULL, N'ErtMondatbankTipus', 127, N'T', N'T', 1, NULL, 1, NULL),
            (7358, N'Magatartás', N'Magatartás', NULL, NULL, NULL, N'ErtMondatbankTipus', 127, N'T', N'T', 2, NULL, 2, NULL),
            (7359, N'Szorgalom', N'Szorgalom', NULL, NULL, NULL, N'ErtMondatbankTipus', 127, N'T', N'T', 3, NULL, 3, NULL),
            (7251, N'Tanár', N'Tanár', NULL, NULL, NULL, N'KKKepesitesJellTipus', 119, N'T', N'T', 1, NULL, 1, NULL),
            (7252, N'Tanár (AMI)', N'Tanár (AMI)', NULL, NULL, NULL, N'KKKepesitesJellTipus', 119, N'T', N'T', 2, NULL, 2, NULL),
            (7253, N'Tanító', N'Tanító', NULL, NULL, NULL, N'KKKepesitesJellTipus', 119, N'T', N'T', 3, NULL, 3, NULL),
            (7254, N'Gyógypedagógus', N'Gyógypedagógus', NULL, NULL, NULL, N'KKKepesitesJellTipus', 119, N'T', N'T', 4, NULL, 4, NULL),
            (7229, N'Na', N'Na', NULL, NULL, NULL, N'KKVegzettsegTipus', 115, N'T', N'T', 1, NULL, 1, NULL),
            (7230, N'Alapfokozat', N'Alapfokozat', NULL, NULL, NULL, N'KKVegzettsegTipus', 115, N'T', N'T', 2, NULL, 2, NULL),
            (7231, N'Alapfokozat BA', N'Alapfokozat BA', NULL, NULL, NULL, N'KKVegzettsegTipus', 115, N'T', N'T', 3, NULL, 3, NULL),
            (7232, N'Alapfokozat BSc', N'Alapfokozat BSc', NULL, NULL, NULL, N'KKVegzettsegTipus', 115, N'T', N'T', 4, NULL, 4, NULL),
            (7233, N'Doktori fokozat DLA', N'Doktori fokozat DLA', NULL, NULL, NULL, N'KKVegzettsegTipus', 115, N'T', N'T', 5, NULL, 5, NULL),
            (7234, N'Doktori fokozat Phd', N'Doktori fokozat Phd', NULL, NULL, NULL, N'KKVegzettsegTipus', 115, N'T', N'T', 6, NULL, 6, NULL),
            (7235, N'Egyetem', N'Egyetem', NULL, NULL, NULL, N'KKVegzettsegTipus', 115, N'T', N'T', 7, NULL, 7, NULL),
            (7236, N'Főiskola', N'Főiskola', NULL, NULL, NULL, N'KKVegzettsegTipus', 115, N'T', N'T', 8, NULL, 8, NULL),
            (7237, N'Más külföldi képzéstípus szintje', N'Más külföldi képzéstípus szintje', NULL, NULL, NULL, N'KKVegzettsegTipus', 115, N'T', N'T', 9, NULL, 9, NULL),
            (7238, N'Mesterfokozat', N'Mesterfokozat', NULL, NULL, NULL, N'KKVegzettsegTipus', 115, N'T', N'T', 10, NULL, 10, NULL),
            (7239, N'Mesterfokozat MA', N'Mesterfokozat MA', NULL, NULL, NULL, N'KKVegzettsegTipus', 115, N'T', N'T', 11, NULL, 11, NULL),
            (7240, N'Mesterfokozat MSc', N'Mesterfokozat MSc', NULL, NULL, NULL, N'KKVegzettsegTipus', 115, N'T', N'T', 12, NULL, 12, NULL),
            (7287, N'Népi hegedű', N'Népi hegedű', NULL, NULL, NULL, N'KKAMINepzeneTipus', 122, N'T', N'T', 1, NULL, 1, NULL),
            (7288, N'Népi brácsa', N'Népi brácsa', NULL, NULL, NULL, N'KKAMINepzeneTipus', 122, N'T', N'T', 2, NULL, 2, NULL),
            (7289, N'Népi bőgő, cselló', N'Népi bőgő, cselló', NULL, NULL, NULL, N'KKAMINepzeneTipus', 122, N'T', N'T', 3, NULL, 3, NULL),
            (7290, N'Tekerő', N'Tekerő', NULL, NULL, NULL, N'KKAMINepzeneTipus', 122, N'T', N'T', 4, NULL, 4, NULL),
            (7291, N'Népi furulya', N'Népi furulya', NULL, NULL, NULL, N'KKAMINepzeneTipus', 122, N'T', N'T', 5, NULL, 5, NULL),
            (7292, N'Népi klarinét/ tárogató', N'Népi klarinét/ tárogató', NULL, NULL, NULL, N'KKAMINepzeneTipus', 122, N'T', N'T', 6, NULL, 6, NULL),
            (7293, N'Duda', N'Duda', NULL, NULL, NULL, N'KKAMINepzeneTipus', 122, N'T', N'T', 7, NULL, 7, NULL),
            (7294, N'Citera', N'Citera', NULL, NULL, NULL, N'KKAMINepzeneTipus', 122, N'T', N'T', 8, NULL, 8, NULL),
            (7295, N'Tambura', N'Tambura', NULL, NULL, NULL, N'KKAMINepzeneTipus', 122, N'T', N'T', 9, NULL, 9, NULL),
            (7296, N'Koboz', N'Koboz', NULL, NULL, NULL, N'KKAMINepzeneTipus', 122, N'T', N'T', 10, NULL, 10, NULL),
            (7297, N'Népi cimbalom', N'Népi cimbalom', NULL, NULL, NULL, N'KKAMINepzeneTipus', 122, N'T', N'T', 11, NULL, 11, NULL),
            (7298, N'Népi harmonika', N'Népi harmonika', NULL, NULL, NULL, N'KKAMINepzeneTipus', 122, N'T', N'T', 12, NULL, 12, NULL),
            (7299, N'Népi ritmushangszerek (ütőgardon, dob)', N'Népi ritmushangszerek (ütőgardon, dob)', NULL, NULL, NULL, N'KKAMINepzeneTipus', 122, N'T', N'T', 13, NULL, 13, NULL),
            (7300, N'Népi ének', N'Népi ének', NULL, NULL, NULL, N'KKAMINepzeneTipus', 122, N'T', N'T', 14, NULL, 14, NULL),
            (7301, N'Népzenei ismeretek', N'Népzenei ismeretek', NULL, NULL, NULL, N'KKAMINepzeneTipus', 122, N'T', N'T', 15, NULL, 15, NULL),
            (7302, N'Néprajz', N'Néprajz', NULL, NULL, NULL, N'KKAMINepzeneTipus', 122, N'T', N'T', 16, NULL, 16, NULL),
            (7303, N'Szolfézs', N'Szolfézs', NULL, NULL, NULL, N'KKAMINepzeneTipus', 122, N'T', N'T', 17, NULL, 17, NULL),
            (7304, N'Népi kamarazene (tantárgy)', N'Népi kamarazene (tantárgy)', NULL, NULL, NULL, N'KKAMINepzeneTipus', 122, N'T', N'T', 18, NULL, 18, NULL),
            (7305, N'Népi zenekar (tantárgy)', N'Népi zenekar (tantárgy)', NULL, NULL, NULL, N'KKAMINepzeneTipus', 122, N'T', N'T', 19, NULL, 19, NULL),
            (7321, N'Szintetizátor', N'Szintetizátor', NULL, NULL, NULL, N'KKAMIElektroakuTipus', 124, N'T', N'T', 1, NULL, 1, NULL),
            (7322, N'Szintetizátor-Keyboard', N'Szintetizátor-Keyboard', NULL, NULL, NULL, N'KKAMIElektroakuTipus', 124, N'T', N'T', 2, NULL, 2, NULL),
            (7323, N'Számítógépes zene', N'Számítógépes zene', NULL, NULL, NULL, N'KKAMIElektroakuTipus', 124, N'T', N'T', 3, NULL, 3, NULL),
            (7324, N'Hangszínszolfézs', N'Hangszínszolfézs', NULL, NULL, NULL, N'KKAMIElektroakuTipus', 124, N'T', N'T', 4, NULL, 4, NULL),
            (7325, N'Szolfézs', N'Szolfézs', NULL, NULL, NULL, N'KKAMIElektroakuTipus', 124, N'T', N'T', 5, NULL, 5, NULL),
            (7326, N'Zeneismeret', N'Zeneismeret', NULL, NULL, NULL, N'KKAMIElektroakuTipus', 124, N'T', N'T', 6, NULL, 6, NULL),
            (7327, N'Zenetörténet-zeneirodalom', N'Zenetörténet-zeneirodalom', NULL, NULL, NULL, N'KKAMIElektroakuTipus', 124, N'T', N'T', 7, NULL, 7, NULL),
            (7328, N'Zeneelmélet', N'Zeneelmélet', NULL, NULL, NULL, N'KKAMIElektroakuTipus', 124, N'T', N'T', 8, NULL, 8, NULL),
            (7329, N'Kamarazene', N'Kamarazene', NULL, NULL, NULL, N'KKAMIElektroakuTipus', 124, N'T', N'T', 9, NULL, 9, NULL),
            (7241, N'Na', N'Na', NULL, NULL, NULL, N'KKAMIVegzettsegTipus', 116, N'T', N'T', 1, NULL, 1, NULL),
            (7242, N'Alapfokú művészetoktatási végzettség', N'Alapfokú művészetoktatási végzettség', NULL, NULL, NULL, N'KKAMIVegzettsegTipus', 116, N'T', N'T', 2, NULL, 2, NULL),
            (7243, N'Középfokú művészetoktatási végzettség', N'Középfokú művészetoktatási végzettség', NULL, NULL, NULL, N'KKAMIVegzettsegTipus', 116, N'T', N'T', 3, NULL, 3, NULL),
            (7247, N'Zeneművészeti ág', N'Zeneművészeti ág', NULL, NULL, NULL, N'KKAMITeruletTipus', 118, N'T', N'T', 1, NULL, 1, NULL),
            (7248, N'Táncművészeti ág', N'Táncművészeti ág', NULL, NULL, NULL, N'KKAMITeruletTipus', 118, N'T', N'T', 2, NULL, 2, NULL),
            (7249, N'Képző- és iparművészeti ág', N'Képző- és iparművészeti ág', NULL, NULL, NULL, N'KKAMITeruletTipus', 118, N'T', N'T', 3, NULL, 3, NULL),
            (7250, N'Szín- és bábművészeti ág', N'Szín- és bábművészeti ág', NULL, NULL, NULL, N'KKAMITeruletTipus', 118, N'T', N'T', 4, NULL, 4, NULL),
            (7306, N'Jazz-fuvola', N'Jazz-fuvola', NULL, NULL, NULL, N'KKAMIJazzZeneTipus', 123, N'T', N'T', 1, NULL, 1, NULL),
            (7307, N'Jazz-szaxofon', N'Jazz-szaxofon', NULL, NULL, NULL, N'KKAMIJazzZeneTipus', 123, N'T', N'T', 2, NULL, 2, NULL),
            (7308, N'Jazz-trombita', N'Jazz-trombita', NULL, NULL, NULL, N'KKAMIJazzZeneTipus', 123, N'T', N'T', 3, NULL, 3, NULL),
            (7309, N'Jazz-harsona', N'Jazz-harsona', NULL, NULL, NULL, N'KKAMIJazzZeneTipus', 123, N'T', N'T', 4, NULL, 4, NULL),
            (7310, N'Jazz-zongora', N'Jazz-zongora', NULL, NULL, NULL, N'KKAMIJazzZeneTipus', 123, N'T', N'T', 5, NULL, 5, NULL),
            (7311, N'Jazz-hegedű', N'Jazz-hegedű', NULL, NULL, NULL, N'KKAMIJazzZeneTipus', 123, N'T', N'T', 6, NULL, 6, NULL),
            (7312, N'Jazz-bőgő', N'Jazz-bőgő', NULL, NULL, NULL, N'KKAMIJazzZeneTipus', 123, N'T', N'T', 7, NULL, 7, NULL),
            (7313, N'Jazz-gitár', N'Jazz-gitár', NULL, NULL, NULL, N'KKAMIJazzZeneTipus', 123, N'T', N'T', 8, NULL, 8, NULL),
            (7314, N'Jazz-basszusgitár', N'Jazz-basszusgitár', NULL, NULL, NULL, N'KKAMIJazzZeneTipus', 123, N'T', N'T', 9, NULL, 9, NULL),
            (7315, N'Jazz-dob', N'Jazz-dob', NULL, NULL, NULL, N'KKAMIJazzZeneTipus', 123, N'T', N'T', 10, NULL, 10, NULL),
            (7316, N'Jazz-ének', N'Jazz-ének', NULL, NULL, NULL, N'KKAMIJazzZeneTipus', 123, N'T', N'T', 11, NULL, 11, NULL),
            (7317, N'Jazz-szolfézs (tantárgy)', N'Jazz-szolfézs (tantárgy)', NULL, NULL, NULL, N'KKAMIJazzZeneTipus', 123, N'T', N'T', 12, NULL, 12, NULL),
            (7318, N'Jazz-irodalom (tantárgy)', N'Jazz-irodalom (tantárgy)', NULL, NULL, NULL, N'KKAMIJazzZeneTipus', 123, N'T', N'T', 13, NULL, 13, NULL),
            (7319, N'Jazz-elmélet (tantárgy)', N'Jazz-elmélet (tantárgy)', NULL, NULL, NULL, N'KKAMIJazzZeneTipus', 123, N'T', N'T', 14, NULL, 14, NULL),
            (7320, N'Zenekari gyakorlat (tantárgy)', N'Zenekari gyakorlat (tantárgy)', NULL, NULL, NULL, N'KKAMIJazzZeneTipus', 123, N'T', N'T', 15, NULL, 15, NULL),
            (7352, N'Na', N'Na', NULL, NULL, NULL, N'KKTanitoMuvTerTipus', 125, N'T', N'T', 1, NULL, 1, NULL),
            (7330, N'Idegen nyelvi', N'Idegen nyelvi', NULL, NULL, NULL, N'KKTanitoMuvTerTipus', 125, N'T', N'T', 2, NULL, 2, NULL),
            (7331, N'Ember és társadalom', N'Ember és társadalom', NULL, NULL, NULL, N'KKTanitoMuvTerTipus', 125, N'T', N'T', 3, NULL, 3, NULL),
            (7332, N'Ének-zene', N'Ének-zene', NULL, NULL, NULL, N'KKTanitoMuvTerTipus', 125, N'T', N'T', 4, NULL, 4, NULL),
            (7333, N'Informatika', N'Informatika', NULL, NULL, NULL, N'KKTanitoMuvTerTipus', 125, N'T', N'T', 5, NULL, 5, NULL),
            (7334, N'Matematika', N'Matematika', NULL, NULL, NULL, N'KKTanitoMuvTerTipus', 125, N'T', N'T', 6, NULL, 6, NULL),
            (7335, N'Magyar nyelv és irodalom', N'Magyar nyelv és irodalom', NULL, NULL, NULL, N'KKTanitoMuvTerTipus', 125, N'T', N'T', 7, NULL, 7, NULL),
            (7336, N'Természetismeret', N'Természetismeret', NULL, NULL, NULL, N'KKTanitoMuvTerTipus', 125, N'T', N'T', 8, NULL, 8, NULL),
            (7337, N'Testnevelés és sport', N'Testnevelés és sport', NULL, NULL, NULL, N'KKTanitoMuvTerTipus', 125, N'T', N'T', 9, NULL, 9, NULL),
            (7338, N'Vizuális nevelés', N'Vizuális nevelés', NULL, NULL, NULL, N'KKTanitoMuvTerTipus', 125, N'T', N'T', 10, NULL, 10, NULL),
            (7354, N'Technika és életvitel', N'Technika és életvitel', NULL, NULL, NULL, N'KKTanitoMuvTerTipus', 125, N'T', N'T', 11, NULL, 11, NULL),
            (7355, N'Neveléstudomány', N'Neveléstudomány', NULL, NULL, NULL, N'KKTanitoMuvTerTipus', 125, N'T', N'T', 12, NULL, 12, NULL),
            (7768, N'Egyéb', N'Egyéb', NULL, NULL, NULL, N'KKTanitoMuvTerTipus', 125, N'T', N'T', 13, NULL, 13, NULL),
            (7255, N'Furulya', N'Furulya', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 1, NULL, 1, NULL),
            (7256, N'Fuvola', N'Fuvola', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 2, NULL, 2, NULL),
            (7257, N'Oboa', N'Oboa', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 3, NULL, 3, NULL),
            (7258, N'Klarinét', N'Klarinét', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 4, NULL, 4, NULL),
            (7259, N'Szaxofon', N'Szaxofon', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 5, NULL, 5, NULL),
            (7260, N'Fagott', N'Fagott', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 6, NULL, 6, NULL),
            (7261, N'Kürt', N'Kürt', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 7, NULL, 7, NULL),
            (7262, N'Trombita', N'Trombita', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 8, NULL, 8, NULL),
            (7263, N'Harsona', N'Harsona', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 9, NULL, 9, NULL),
            (7264, N'Tuba', N'Tuba', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 10, NULL, 10, NULL),
            (7265, N'Hárfa', N'Hárfa', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 11, NULL, 11, NULL),
            (7266, N'Gitár', N'Gitár', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 12, NULL, 12, NULL),
            (7267, N'Cimbalom', N'Cimbalom', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 13, NULL, 13, NULL),
            (7268, N'Ütő', N'Ütő', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 14, NULL, 14, NULL),
            (7269, N'Harmonika', N'Harmonika', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 15, NULL, 15, NULL),
            (7270, N'Zongora', N'Zongora', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 16, NULL, 16, NULL),
            (7271, N'Csembaló', N'Csembaló', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 17, NULL, 17, NULL),
            (7272, N'Orgona', N'Orgona', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 18, NULL, 18, NULL),
            (7273, N'Hegedű', N'Hegedű', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 19, NULL, 19, NULL),
            (7274, N'Brácsa', N'Brácsa', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 20, NULL, 20, NULL),
            (7275, N'Gordonka', N'Gordonka', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 21, NULL, 21, NULL),
            (7276, N'Nagybőgő', N'Nagybőgő', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 22, NULL, 22, NULL),
            (7277, N'Magánének', N'Magánének', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 23, NULL, 23, NULL),
            (7278, N'Szolfézs', N'Szolfézs', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 24, NULL, 24, NULL),
            (7279, N'Zeneelmélet', N'Zeneelmélet', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 25, NULL, 25, NULL),
            (7280, N'Zenetörténet-Zeneirodalom', N'Zenetörténet-Zeneirodalom', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 26, NULL, 26, NULL),
            (7281, N'Improvizáció', N'Improvizáció', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 27, NULL, 27, NULL),
            (7282, N'Fafúvós kamara (tantárgy)', N'Fafúvós kamara (tantárgy)', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 28, NULL, 28, NULL),
            (7283, N'Rézfúvós kamara (tantárgy)', N'Rézfúvós kamara (tantárgy)', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 29, NULL, 29, NULL),
            (7284, N'Vonós kamara (tantárgy)', N'Vonós kamara (tantárgy)', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 30, NULL, 30, NULL),
            (7285, N'Zenekar (tantárgy)', N'Zenekar (tantárgy)', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 31, NULL, 31, NULL),
            (7286, N'Kórus (tantárgy)', N'Kórus (tantárgy)', NULL, NULL, NULL, N'KKAMIKlasszikusTipus', 121, N'T', N'T', 32, NULL, 32, NULL),
            (7353, N'Na', N'Na', NULL, NULL, NULL, N'KKGyogypedTerTipus', 126, N'T', N'T', 1, NULL, 1, NULL),
            (7339, N'Autizmus spektrum pedagógiája szak', N'Autizmus spektrum pedagógiája szak', NULL, NULL, NULL, N'KKGyogypedTerTipus', 126, N'T', N'T', 2, NULL, 2, NULL),
            (7340, N'Értelmileg akadályozottak pedagógiája szak', N'Értelmileg akadályozottak pedagógiája szak', NULL, NULL, NULL, N'KKGyogypedTerTipus', 126, N'T', N'T', 3, NULL, 3, NULL),
            (7341, N'Logopédus', N'Logopédus', NULL, NULL, NULL, N'KKGyogypedTerTipus', 126, N'T', N'T', 4, NULL, 4, NULL),
            (7342, N'Oligofrénpedagógus', N'Oligofrénpedagógus', NULL, NULL, NULL, N'KKGyogypedTerTipus', 126, N'T', N'T', 5, NULL, 5, NULL),
            (7343, N'Pszichopedagógus', N'Pszichopedagógus', NULL, NULL, NULL, N'KKGyogypedTerTipus', 126, N'T', N'T', 6, NULL, 6, NULL),
            (7344, N'Szomatopedagógus', N'Szomatopedagógus', NULL, NULL, NULL, N'KKGyogypedTerTipus', 126, N'T', N'T', 7, NULL, 7, NULL),
            (7345, N'Szurdopedagógus', N'Szurdopedagógus', NULL, NULL, NULL, N'KKGyogypedTerTipus', 126, N'T', N'T', 8, NULL, 8, NULL),
            (7346, N'Tanulásban akadályozottak pedagógiája szak', N'Tanulásban akadályozottak pedagógiája szak', NULL, NULL, NULL, N'KKGyogypedTerTipus', 126, N'T', N'T', 9, NULL, 9, NULL),
            (7347, N'Tiflopedagógus', N'Tiflopedagógus', NULL, NULL, NULL, N'KKGyogypedTerTipus', 126, N'T', N'T', 10, NULL, 10, NULL),
            (7244, N'Pedagógus', N'Pedagógus', NULL, NULL, NULL, N'KKKepesitesTipus', 117, N'T', N'T', 2, NULL, 1, NULL),
            (7245, N'Középfokú, képesítés nélküli végzettséggel rendelkező', N'Középfokú, képesítés nélküli végzettséggel rendelkező', NULL, NULL, NULL, N'KKKepesitesTipus', 117, N'T', N'T', 3, NULL, 2, NULL),
            (7246, N'Felsőfokú, képesítés nélküli végzettséggel rendelkező', N'Felsőfokú, képesítés nélküli végzettséggel rendelkező', NULL, NULL, NULL, N'KKKepesitesTipus', 117, N'T', N'T', 4, NULL, 3, NULL),
            (7133, N'Na', N'Na', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 1, NULL, 1, NULL),
            (7134, N'Magyar nyelv és irodalom', N'Magyar nyelv és irodalom', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 2, NULL, 2, NULL),
            (7135, N'Nemzetiségi nyelv és irodalom', N'Nemzetiségi nyelv és irodalom', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 3, NULL, 3, NULL),
            (7136, N'Két tanítási nyelvű célnyelv', N'Két tanítási nyelvű célnyelv', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 4, NULL, 4, NULL),
            (7137, N'Angol nyelv', N'Angol nyelv', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 5, NULL, 5, NULL),
            (7138, N'Német nyelv', N'Német nyelv', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 6, NULL, 6, NULL),
            (7139, N'Francia nyelv', N'Francia nyelv', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 7, NULL, 7, NULL),
            (7140, N'Olasz nyelv', N'Olasz nyelv', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 8, NULL, 8, NULL),
            (7141, N'Spanyol nyelv', N'Spanyol nyelv', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 9, NULL, 9, NULL),
            (7142, N'Báb- és színművészet', N'Báb- és színművészet', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 10, NULL, 10, NULL),
            (7143, N'Ember és természet', N'Ember és természet', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 11, NULL, 11, NULL),
            (7144, N'Képző- és iparművészet', N'Képző- és iparművészet', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 12, NULL, 12, NULL),
            (7145, N'Szakmai elmélet (Környezetvédelem - 14.)', N'Szakmai elmélet (Környezetvédelem - 14.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 13, NULL, 13, NULL),
            (7146, N'Szakmai elmélet (Közgazdaság - 15.)', N'Szakmai elmélet (Közgazdaság - 15.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 14, NULL, 14, NULL),
            (7147, N'Szakmai elmélet (Rendészet, honvédelem és közszolgálat - 22.)', N'Szakmai elmélet (Rendészet, honvédelem és közszolgálat - 22.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 15, NULL, 15, NULL),
            (7148, N'Szakmai elmélet (Vízügy - 23.)', N'Szakmai elmélet (Vízügy - 23.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 16, NULL, 16, NULL),
            (7149, N'Szakmai gyakorlat (Környezetvédelem - 14.)', N'Szakmai gyakorlat (Környezetvédelem - 14.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 17, NULL, 17, NULL),
            (7150, N'Szakmai gyakorlat (Közgazdaság - 15.)', N'Szakmai gyakorlat (Közgazdaság - 15.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 18, NULL, 18, NULL),
            (7151, N'Szakmai gyakorlat (Rendészet, honvédelem és közszolgálat - 22.)', N'Szakmai gyakorlat (Rendészet, honvédelem és közszolgálat - 22.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 19, NULL, 19, NULL),
            (7152, N'Szakmai gyakorlat (Vízügy - 23.)', N'Szakmai gyakorlat (Vízügy - 23.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 20, NULL, 20, NULL),
            (7153, N'Táncművészet', N'Táncművészet', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 21, NULL, 21, NULL),
            (7154, N'Történelem', N'Történelem', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 22, NULL, 22, NULL),
            (7155, N'Életvitel és gyakorlat', N'Életvitel és gyakorlat', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 23, NULL, 23, NULL),
            (7156, N'Orosz nyelv', N'Orosz nyelv', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 24, NULL, 24, NULL),
            (7157, N'Latin nyelv', N'Latin nyelv', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 25, NULL, 25, NULL),
            (7158, N'Egyéb idegen nyelv', N'Egyéb idegen nyelv', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 26, NULL, 26, NULL),
            (7159, N'Matematika', N'Matematika', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 27, NULL, 27, NULL),
            (7160, N'Filozófia', N'Filozófia', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 28, NULL, 28, NULL),
            (7161, N'Etika', N'Etika', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 29, NULL, 29, NULL),
            (7162, N'Hon- és népismeret', N'Hon- és népismeret', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 30, NULL, 30, NULL),
            (7163, N'Természetismeret', N'Természetismeret', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 31, NULL, 31, NULL),
            (7164, N'Fizika', N'Fizika', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 32, NULL, 32, NULL),
            (7165, N'Kémia', N'Kémia', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 33, NULL, 33, NULL),
            (7166, N'Biológia', N'Biológia', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 34, NULL, 34, NULL),
            (7167, N'Környezetismeret', N'Környezetismeret', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 35, NULL, 35, NULL),
            (7168, N'Földünk – környezetünk, földrajz', N'Földünk – környezetünk, földrajz', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 36, NULL, 36, NULL),
            (7169, N'Ének-zene', N'Ének-zene', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 37, NULL, 37, NULL),
            (7170, N'Dráma és tánc', N'Dráma és tánc', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 38, NULL, 38, NULL),
            (7171, N'Vizuális kultúra', N'Vizuális kultúra', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 39, NULL, 39, NULL),
            (7172, N'Mozgóképkultúra és médiaismeret', N'Mozgóképkultúra és médiaismeret', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 40, NULL, 40, NULL),
            (7173, N'Informatika', N'Informatika', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 41, NULL, 41, NULL),
            (7174, N'Testnevelés és sport', N'Testnevelés és sport', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 42, NULL, 42, NULL),
            (7175, N'Pályaorientáció, szakmai alapozás', N'Pályaorientáció, szakmai alapozás', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 43, NULL, 43, NULL),
            (7176, N'Szakmai orientáció', N'Szakmai orientáció', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 44, NULL, 44, NULL),
            (7177, N'Szakmacsoportos alapozás', N'Szakmacsoportos alapozás', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 45, NULL, 45, NULL),
            (7178, N'Szakmai elmélet', N'Szakmai elmélet', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 46, NULL, 46, NULL),
            (7179, N'Szakmai gyakorlat', N'Szakmai gyakorlat', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 47, NULL, 47, NULL),
            (7180, N'Zeneművészet', N'Zeneművészet', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 48, NULL, 48, NULL),
            (7181, N'Egyéb', N'Egyéb', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 49, NULL, 49, NULL),
            (7182, N'Osztályfőnöki, élet- és pályatervezés', N'Osztályfőnöki, élet- és pályatervezés', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 50, NULL, 50, NULL),
            (7183, N'Szakmai elmélet (Egészségügy - 1.)', N'Szakmai elmélet (Egészségügy - 1.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 51, NULL, 51, NULL),
            (7184, N'Szakmai elmélet (Szociális szolgáltatások - 2.)', N'Szakmai elmélet (Szociális szolgáltatások - 2.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 52, NULL, 52, NULL),
            (7185, N'Szakmai elmélet (Oktatás - 3.)', N'Szakmai elmélet (Oktatás - 3.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 53, NULL, 53, NULL),
            (7186, N'Szakmai elmélet (Művészet, közművelődés, kommunikáció - 4.)', N'Szakmai elmélet (Művészet, közművelődés, kommunikáció - 4.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 54, NULL, 54, NULL),
            (7187, N'Szakmai elmélet (Gépészet - 5.)', N'Szakmai elmélet (Gépészet - 5.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 55, NULL, 55, NULL),
            (7188, N'Szakmai elmélet (Elektrotechnika-elektronika - 6.)', N'Szakmai elmélet (Elektrotechnika-elektronika - 6.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 56, NULL, 56, NULL),
            (7189, N'Szakmai elmélet (Informatika - 7.)', N'Szakmai elmélet (Informatika - 7.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 57, NULL, 57, NULL),
            (7190, N'Szakmai elmélet (Vegyipar - 8.)', N'Szakmai elmélet (Vegyipar - 8.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 58, NULL, 58, NULL),
            (7191, N'Szakmai elmélet (Építészet - 9.)', N'Szakmai elmélet (Építészet - 9.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 59, NULL, 59, NULL),
            (7192, N'Szakmai elmélet (Könnyűipar - 10.)', N'Szakmai elmélet (Könnyűipar - 10.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 60, NULL, 60, NULL),
            (7193, N'Szakmai elmélet (Faipar - 11.)', N'Szakmai elmélet (Faipar - 11.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 61, NULL, 61, NULL),
            (7194, N'Szakmai elmélet (Nyomdaipar - 12.)', N'Szakmai elmélet (Nyomdaipar - 12.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 62, NULL, 62, NULL),
            (7195, N'Szakmai elmélet (Közlekedés - 13.)', N'Szakmai elmélet (Közlekedés - 13.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 63, NULL, 63, NULL),
            (7196, N'Szakmai elmélet (Ügyvitel - 16.)', N'Szakmai elmélet (Ügyvitel - 16.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 64, NULL, 64, NULL),
            (7197, N'Szakmai elmélet (Kereskedelem-marketing, üzleti adminisztráció - 17.)', N'Szakmai elmélet (Kereskedelem-marketing, üzleti adminisztráció - 17.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 65, NULL, 65, NULL),
            (7198, N'Szakmai elmélet (Vendéglátás-turisztika - 18.)', N'Szakmai elmélet (Vendéglátás-turisztika - 18.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 66, NULL, 66, NULL),
            (7199, N'Szakmai elmélet (Egyéb szolgáltatások - 19.)', N'Szakmai elmélet (Egyéb szolgáltatások - 19.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 67, NULL, 67, NULL),
            (7200, N'Szakmai elmélet (Mezőgazdaság - 20.)', N'Szakmai elmélet (Mezőgazdaság - 20.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 68, NULL, 68, NULL),
            (7201, N'Szakmai elmélet (Élelmiszeripar - 21.)', N'Szakmai elmélet (Élelmiszeripar - 21.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 69, NULL, 69, NULL),
            (7202, N'Szakmai gyakorlat (Egészségügy - 1.)', N'Szakmai gyakorlat (Egészségügy - 1.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 70, NULL, 70, NULL),
            (7203, N'Szakmai gyakorlat (Szociális szolgáltatások - 2.)', N'Szakmai gyakorlat (Szociális szolgáltatások - 2.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 71, NULL, 71, NULL),
            (7204, N'Szakmai gyakorlat (Oktatás - 3.)', N'Szakmai gyakorlat (Oktatás - 3.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 72, NULL, 72, NULL),
            (7205, N'Szakmai gyakorlat (Művészet, közművelődés, kommunikáció - 4.)', N'Szakmai gyakorlat (Művészet, közművelődés, kommunikáció - 4.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 73, NULL, 73, NULL),
            (7206, N'Szakmai gyakorlat (Gépészet - 5.)', N'Szakmai gyakorlat (Gépészet - 5.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 74, NULL, 74, NULL),
            (7207, N'Szakmai gyakorlat (Elektrotechnika-elektronika - 6.)', N'Szakmai gyakorlat (Elektrotechnika-elektronika - 6.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 75, NULL, 75, NULL),
            (7208, N'Szakmai gyakorlat (Informatika - 7.)', N'Szakmai gyakorlat (Informatika - 7.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 76, NULL, 76, NULL),
            (7209, N'Szakmai gyakorlat (Vegyipar - 8.)', N'Szakmai gyakorlat (Vegyipar - 8.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 77, NULL, 77, NULL),
            (7210, N'Szakmai gyakorlat (Építészet - 9.)', N'Szakmai gyakorlat (Építészet - 9.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 78, NULL, 78, NULL),
            (7211, N'Szakmai gyakorlat (Könnyűipar - 10.)', N'Szakmai gyakorlat (Könnyűipar - 10.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 79, NULL, 79, NULL),
            (7212, N'Szakmai gyakorlat (Faipar - 11.)', N'Szakmai gyakorlat (Faipar - 11.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 80, NULL, 80, NULL),
            (7213, N'Szakmai gyakorlat (Nyomdaipar - 12.)', N'Szakmai gyakorlat (Nyomdaipar - 12.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 81, NULL, 81, NULL),
            (7214, N'Szakmai gyakorlat (Közlekedés - 13.)', N'Szakmai gyakorlat (Közlekedés - 13.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 82, NULL, 82, NULL),
            (7215, N'Szakmai gyakorlat (Ügyvitel - 16.)', N'Szakmai gyakorlat (Ügyvitel - 16.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 83, NULL, 83, NULL),
            (7216, N'Szakmai gyakorlat (Kereskedelem-marketing, üzleti adminisztráció - 17.)', N'Szakmai gyakorlat (Kereskedelem-marketing, üzleti adminisztráció - 17.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 84, NULL, 84, NULL),
            (7217, N'Szakmai gyakorlat (Vendéglátás-turisztika - 18.)', N'Szakmai gyakorlat (Vendéglátás-turisztika - 18.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 85, NULL, 85, NULL),
            (7218, N'Szakmai gyakorlat (Egyéb szolgáltatások - 19.)', N'Szakmai gyakorlat (Egyéb szolgáltatások - 19.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 86, NULL, 86, NULL),
            (7219, N'Szakmai gyakorlat (Mezőgazdaság - 20.)', N'Szakmai gyakorlat (Mezőgazdaság - 20.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 87, NULL, 87, NULL),
            (7220, N'Szakmai gyakorlat (Élelmiszeripar - 21.)', N'Szakmai gyakorlat (Élelmiszeripar - 21.)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 88, NULL, 88, NULL),
            (7221, N'Társadalmi, állampolgári és gazdasági ismeretek', N'Társadalmi, állampolgári és gazdasági ismeretek', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 89, NULL, 89, NULL),
            (7222, N'Művészetek', N'Művészetek', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 90, NULL, 90, NULL),
            (7223, N'Napközi', N'Napközi', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 91, NULL, 91, NULL),
            (7224, N'Szakmai elmélet (közgazdaságtan)', N'Szakmai elmélet (közgazdaságtan)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 92, NULL, 92, NULL),
            (7225, N'Szakmai gyakorlat (közgadaságtan)', N'Szakmai gyakorlat (közgadaságtan)', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 93, NULL, 93, NULL),
            (7226, N'Ember és társadalom', N'Ember és társadalom', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 94, NULL, 94, NULL),
            (7227, N'Könyvtárostanár', N'Könyvtárostanár', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 95, NULL, 95, NULL),
            (7228, N'Fejlesztő pedagógus', N'Fejlesztő pedagógus', NULL, NULL, NULL, N'KKSzakTipus', 114, N'T', N'T', 96, NULL, 96, NULL),
            (7348, N'Klasszikus zene', N'Klasszikus zene', NULL, NULL, NULL, N'KKAMIZenemuvKatTipus', 120, N'T', N'T', 1, NULL, 1, NULL),
            (7349, N'Népzene', N'Népzene', NULL, NULL, NULL, N'KKAMIZenemuvKatTipus', 120, N'T', N'T', 2, NULL, 2, NULL),
            (7350, N'Jazz zene', N'Jazz zene', NULL, NULL, NULL, N'KKAMIZenemuvKatTipus', 120, N'T', N'T', 3, NULL, 3, NULL),
            (7351, N'Elektroakusztikus zene', N'Elektroakusztikus zene', NULL, NULL, NULL, N'KKAMIZenemuvKatTipus', 120, N'T', N'T', 4, NULL, 4, NULL),
            (7095, N'Általános feltételek szerinti nevelés-oktatás', N'Általános feltételek szerinti nevelés-oktatás', NULL, NULL, NULL, N'KirSzolgaltatasTipus', 113, N'T', N'T', 1, N'1551', 1, NULL),
            (7096, N'Anyanyelvű nemzetiségi nevelés-oktatás', N'Anyanyelvű nemzetiségi nevelés-oktatás', NULL, NULL, NULL, N'KirSzolgaltatasTipus', 113, N'T', N'T', 2, N'1566', 2, NULL),
            (7097, N'Kétnyelvű nemzetiségi nevelés-oktatás', N'Kétnyelvű nemzetiségi nevelés-oktatás', NULL, NULL, NULL, N'KirSzolgaltatasTipus', 113, N'T', N'T', 3, N'1571', 3, NULL),
            (7098, N'Nyelvoktató nemzetiségi nevelés-oktatás', N'Nyelvoktató nemzetiségi nevelés-oktatás', NULL, NULL, NULL, N'KirSzolgaltatasTipus', 113, N'T', N'T', 4, N'1576', 4, NULL),
            (7099, N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás', N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás', NULL, NULL, NULL, N'KirSzolgaltatasTipus', 113, N'T', N'T', 5, N'1581', 5, NULL),
            (7100, N'Sajátos nevelési igényű gyermekek nevelése-oktatása', N'Sajátos nevelési igényű gyermekek nevelése-oktatása', NULL, NULL, NULL, N'KirSzolgaltatasTipus', 113, N'T', N'T', 6, N'1556', 6, NULL),
            (7101, N'Két tanítási nyelvű nevelés-oktatás', N'Két tanítási nyelvű nevelés-oktatás', NULL, NULL, NULL, N'KirSzolgaltatasTipus', 113, N'T', N'T', 7, N'1586', 7, NULL),
            (7102, N'Gyógypedagógiai nevelés-oktatás', N'Gyógypedagógiai nevelés-oktatás', NULL, NULL, NULL, N'KirSzolgaltatasTipus', 113, N'T', N'T', 8, N'1561', 8, NULL),
            (7103, N'Emelt szintű oktatás', N'Emelt szintű oktatás', NULL, NULL, NULL, N'KirSzolgaltatasTipus', 113, N'T', N'T', 9, N'1588', 9, NULL),
            (9059, N'Óvodai nevelés', N'Óvodai nevelés', NULL, NULL, NULL, N'KirSzolgaltatasTipus', 113, N'T', N'T', 10, NULL, 10, NULL),
            (6974, N'Na', N'Na', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 1, NULL, 1, NULL),
            (6975, N'Magyar nyelv és irodalom', N'Magyar nyelv és irodalom', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 2, NULL),
            (6976, N'Nemzetiségi nyelv és irodalom', N'Nemzetiségi nyelv és irodalom', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 3, NULL),
            (6977, N'Két tanítási nyelvű célnyelv', N'Két tanítási nyelvű célnyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 4, NULL),
            (6978, N'Angol nyelv', N'Angol nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 5, NULL),
            (6979, N'Német nyelv', N'Német nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 6, NULL),
            (6980, N'Francia nyelv', N'Francia nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 7, NULL),
            (6981, N'Olasz nyelv', N'Olasz nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 8, NULL),
            (6982, N'Spanyol nyelv', N'Spanyol nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 9, NULL),
            (6983, N'Báb- és színművészet', N'Báb- és színművészet', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 10, NULL),
            (6984, N'Első idegen nyelv - Angol nyelv', N'Első idegen nyelv - Angol nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 11, NULL),
            (6985, N'Első idegen nyelv - Egyéb idegen nyelv', N'Első idegen nyelv - Egyéb idegen nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 12, NULL),
            (6986, N'Első idegen nyelv - Francia nyelv', N'Első idegen nyelv - Francia nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 13, NULL),
            (6987, N'Első idegen nyelv - Latin nyelv', N'Első idegen nyelv - Latin nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 14, NULL),
            (6988, N'Első idegen nyelv - Német nyelv', N'Első idegen nyelv - Német nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 15, NULL),
            (6989, N'Első idegen nyelv - Olasz nyelv', N'Első idegen nyelv - Olasz nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 16, NULL),
            (6990, N'Első idegen nyelv - Orosz nyelv', N'Első idegen nyelv - Orosz nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 17, NULL),
            (6991, N'Első idegen nyelv - Spanyol nyelv', N'Első idegen nyelv - Spanyol nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 18, NULL),
            (6992, N'Ember és természet', N'Ember és természet', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 19, NULL),
            (6993, N'Képző- és iparművészet', N'Képző- és iparművészet', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 20, NULL),
            (6994, N'Második idegen nyelv - Angol nyelv', N'Második idegen nyelv - Angol nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 21, NULL),
            (6995, N'Második idegen nyelv - Egyéb idegen nyelv', N'Második idegen nyelv - Egyéb idegen nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 22, NULL),
            (6996, N'Második idegen nyelv - Francia nyelv', N'Második idegen nyelv - Francia nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 23, NULL),
            (6997, N'Második idegen nyelv - Latin nyelv', N'Második idegen nyelv - Latin nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 24, NULL),
            (6998, N'Második idegen nyelv - Német nyelv', N'Második idegen nyelv - Német nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 25, NULL),
            (6999, N'Második idegen nyelv - Olasz nyelv', N'Második idegen nyelv - Olasz nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 26, NULL),
            (7000, N'Második idegen nyelv - Orosz nyelv', N'Második idegen nyelv - Orosz nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 27, NULL),
            (7001, N'Második idegen nyelv - Spanyol nyelv', N'Második idegen nyelv - Spanyol nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 28, NULL),
            (7002, N'Szakmai elmélet (Környezetvédelem - 14.)', N'Szakmai elmélet (Környezetvédelem - 14.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 29, NULL),
            (7003, N'Szakmai elmélet (Közgazdaság - 15.)', N'Szakmai elmélet (Közgazdaság - 15.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 30, NULL),
            (7004, N'Szakmai elmélet (Rendészet, honvédelem és közszolgálat - 22.)', N'Szakmai elmélet (Rendészet, honvédelem és közszolgálat - 22.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 31, NULL),
            (7005, N'Szakmai elmélet (Vízügy - 23.)', N'Szakmai elmélet (Vízügy - 23.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 32, NULL),
            (7006, N'Szakmai gyakorlat (Környezetvédelem - 14.)', N'Szakmai gyakorlat (Környezetvédelem - 14.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 33, NULL),
            (7007, N'Szakmai gyakorlat (Közgazdaság - 15.)', N'Szakmai gyakorlat (Közgazdaság - 15.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 34, NULL),
            (7008, N'Szakmai gyakorlat (Rendészet, honvédelem és közszolgálat - 22.)', N'Szakmai gyakorlat (Rendészet, honvédelem és közszolgálat - 22.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 35, NULL),
            (7009, N'Szakmai gyakorlat (Vízügy - 23.)', N'Szakmai gyakorlat (Vízügy - 23.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 36, NULL),
            (7010, N'Táncművészet', N'Táncművészet', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 37, NULL),
            (7011, N'Történelem', N'Történelem', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 38, NULL),
            (7012, N'Életvitel és gyakorlat', N'Életvitel és gyakorlat', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 39, NULL),
            (7013, N'Orosz nyelv', N'Orosz nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 40, NULL),
            (7014, N'Latin nyelv', N'Latin nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 41, NULL),
            (7015, N'Egyéb idegen nyelv', N'Egyéb idegen nyelv', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 42, NULL),
            (7016, N'Matematika', N'Matematika', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 43, NULL),
            (7017, N'Filozófia', N'Filozófia', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 44, NULL),
            (7018, N'Etika, hit- és erkölcstan, hittan', N'Etika, hit- és erkölcstan, hittan', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 45, NULL),
            (7019, N'Hon- és népismeret', N'Hon- és népismeret', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 46, NULL),
            (7020, N'Természetismeret', N'Természetismeret', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 47, NULL),
            (7021, N'Fizika', N'Fizika', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 48, NULL),
            (7022, N'Kémia', N'Kémia', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 49, NULL),
            (7023, N'Biológia', N'Biológia', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 50, NULL),
            (7024, N'Környezetismeret', N'Környezetismeret', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 51, NULL),
            (7025, N'Földünk – környezetünk', N'Földünk – környezetünk', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 52, NULL),
            (7026, N'Ének-zene', N'Ének-zene', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 53, NULL),
            (7027, N'Dráma és tánc', N'Dráma és tánc', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 54, NULL),
            (7028, N'Vizuális kultúra', N'Vizuális kultúra', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 55, NULL),
            (7029, N'Mozgóképkultúra és médiaismeret', N'Mozgóképkultúra és médiaismeret', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 56, NULL),
            (7030, N'Informatika', N'Informatika', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 57, NULL),
            (7031, N'Testnevelés és sport', N'Testnevelés és sport', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 58, NULL),
            (7032, N'Pályaorientáció, szakmai alapozás', N'Pályaorientáció, szakmai alapozás', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 59, NULL),
            (7033, N'Szakmai orientáció', N'Szakmai orientáció', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 60, NULL),
            (7034, N'Szakmacsoportos alapozás', N'Szakmacsoportos alapozás', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 61, NULL),
            (7035, N'Szakmai elmélet', N'Szakmai elmélet', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 62, NULL),
            (7036, N'Szakmai gyakorlat', N'Szakmai gyakorlat', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 63, NULL),
            (7037, N'Zeneművészet', N'Zeneművészet', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 64, NULL),
            (7038, N'Egyéb', N'Egyéb', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 65, NULL),
            (7039, N'Óvodai feladat', N'Óvodai feladat', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 66, NULL),
            (7040, N'Kollégiumi feladat', N'Kollégiumi feladat', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 67, NULL),
            (7041, N'Diákotthoni feladat', N'Diákotthoni feladat', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 68, NULL),
            (7042, N'Gyermekotthoni feladat', N'Gyermekotthoni feladat', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 69, NULL),
            (7043, N'Szakértői bizottsági tevékenység', N'Szakértői bizottsági tevékenység', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 70, NULL),
            (7044, N'Osztályfőnöki, élet- és pályatervezés', N'Osztályfőnöki, élet- és pályatervezés', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 71, NULL),
            (7045, N'Szakmai elmélet (Egészségügy - 1.)', N'Szakmai elmélet (Egészségügy - 1.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 72, NULL),
            (7046, N'Szakmai elmélet (Szociális szolgáltatások - 2.)', N'Szakmai elmélet (Szociális szolgáltatások - 2.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 73, NULL),
            (7047, N'Szakmai elmélet (Oktatás - 3.)', N'Szakmai elmélet (Oktatás - 3.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 74, NULL),
            (7048, N'Szakmai elmélet (Művészet, közművelődés, kommunikáció - 4.)', N'Szakmai elmélet (Művészet, közművelődés, kommunikáció - 4.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 75, NULL),
            (7049, N'Szakmai elmélet (Gépészet - 5.)', N'Szakmai elmélet (Gépészet - 5.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 76, NULL),
            (7050, N'Szakmai elmélet (Elektrotechnika-elektronika - 6.)', N'Szakmai elmélet (Elektrotechnika-elektronika - 6.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 77, NULL),
            (7051, N'Szakmai elmélet (Informatika - 7.)', N'Szakmai elmélet (Informatika - 7.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 78, NULL),
            (7052, N'Szakmai elmélet (Vegyipar - 8.)', N'Szakmai elmélet (Vegyipar - 8.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 79, NULL),
            (7053, N'Szakmai elmélet (Építészet - 9.)', N'Szakmai elmélet (Építészet - 9.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 80, NULL),
            (7054, N'Szakmai elmélet (Könnyűipar - 10.)', N'Szakmai elmélet (Könnyűipar - 10.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 81, NULL),
            (7055, N'Szakmai elmélet (Faipar - 11.)', N'Szakmai elmélet (Faipar - 11.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 82, NULL),
            (7056, N'Szakmai elmélet (Nyomdaipar - 12.)', N'Szakmai elmélet (Nyomdaipar - 12.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 83, NULL),
            (7057, N'Szakmai elmélet (Közlekedés - 13.)', N'Szakmai elmélet (Közlekedés - 13.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 84, NULL),
            (7058, N'Szakmai elmélet (Ügyvitel - 16.)', N'Szakmai elmélet (Ügyvitel - 16.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 85, NULL),
            (7059, N'Szakmai elmélet (Kereskedelem-marketing, üzleti adminisztráció - 17.)', N'Szakmai elmélet (Kereskedelem-marketing, üzleti adminisztráció - 17.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 86, NULL),
            (7060, N'Szakmai elmélet (Vendéglátás-turisztika - 18.)', N'Szakmai elmélet (Vendéglátás-turisztika - 18.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 87, NULL),
            (7061, N'Szakmai elmélet (Egyéb szolgáltatások - 19.)', N'Szakmai elmélet (Egyéb szolgáltatások - 19.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 88, NULL),
            (7062, N'Szakmai elmélet (Mezőgazdaság - 20.)', N'Szakmai elmélet (Mezőgazdaság - 20.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 89, NULL),
            (7063, N'Szakmai elmélet (Élelmiszeripar - 21.)', N'Szakmai elmélet (Élelmiszeripar - 21.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 90, NULL),
            (7064, N'Szakmai gyakorlat (Egészségügy - 1.)', N'Szakmai gyakorlat (Egészségügy - 1.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 91, NULL),
            (7065, N'Szakmai gyakorlat (Szociális szolgáltatások - 2.)', N'Szakmai gyakorlat (Szociális szolgáltatások - 2.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 92, NULL),
            (7066, N'Szakmai gyakorlat (Oktatás - 3.)', N'Szakmai gyakorlat (Oktatás - 3.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 93, NULL),
            (7067, N'Szakmai gyakorlat (Művészet, közművelődés, kommunikáció - 4.)', N'Szakmai gyakorlat (Művészet, közművelődés, kommunikáció - 4.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 94, NULL),
            (7068, N'Szakmai gyakorlat (Gépészet - 5.)', N'Szakmai gyakorlat (Gépészet - 5.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 95, NULL),
            (7069, N'Szakmai gyakorlat (Elektrotechnika-elektronika - 6.)', N'Szakmai gyakorlat (Elektrotechnika-elektronika - 6.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 96, NULL),
            (7070, N'Szakmai gyakorlat (Informatika - 7.)', N'Szakmai gyakorlat (Informatika - 7.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 97, NULL),
            (7071, N'Szakmai gyakorlat (Vegyipar - 8.)', N'Szakmai gyakorlat (Vegyipar - 8.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 98, NULL),
            (7072, N'Szakmai gyakorlat (Építészet - 9.)', N'Szakmai gyakorlat (Építészet - 9.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 99, NULL),
            (7073, N'Szakmai gyakorlat (Könnyűipar - 10.)', N'Szakmai gyakorlat (Könnyűipar - 10.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 100, NULL),
            (7074, N'Szakmai gyakorlat (Faipar - 11.)', N'Szakmai gyakorlat (Faipar - 11.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 101, NULL),
            (7075, N'Szakmai gyakorlat (Nyomdaipar - 12.)', N'Szakmai gyakorlat (Nyomdaipar - 12.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 102, NULL),
            (7076, N'Szakmai gyakorlat (Közlekedés - 13.)', N'Szakmai gyakorlat (Közlekedés - 13.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 103, NULL),
            (7077, N'Szakmai gyakorlat (Ügyvitel - 16.)', N'Szakmai gyakorlat (Ügyvitel - 16.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 104, NULL),
            (7078, N'Szakmai gyakorlat (Kereskedelem-marketing, üzleti adminisztráció - 17.)', N'Szakmai gyakorlat (Kereskedelem-marketing, üzleti adminisztráció - 17.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 105, NULL),
            (7079, N'Szakmai gyakorlat (Vendéglátás-turisztika - 18.)', N'Szakmai gyakorlat (Vendéglátás-turisztika - 18.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 106, NULL),
            (7080, N'Szakmai gyakorlat (Egyéb szolgáltatások - 19.)', N'Szakmai gyakorlat (Egyéb szolgáltatások - 19.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 107, NULL),
            (7081, N'Szakmai gyakorlat (Mezőgazdaság - 20.)', N'Szakmai gyakorlat (Mezőgazdaság - 20.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 108, NULL),
            (7082, N'Szakmai gyakorlat (Élelmiszeripar - 21.)', N'Szakmai gyakorlat (Élelmiszeripar - 21.)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 109, NULL),
            (7083, N'Társadalmi, állampolgári és gazdasági ismeretek', N'Társadalmi, állampolgári és gazdasági ismeretek', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 110, NULL),
            (7084, N'Művészetek', N'Művészetek', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 111, NULL),
            (7085, N'Napközi', N'Napközi', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 112, NULL),
            (7086, N'Szakmai elmélet (közgazdaságtan)', N'Szakmai elmélet (közgazdaságtan)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 113, NULL),
            (7087, N'Szakmai gyakorlat (közgadaságtan)', N'Szakmai gyakorlat (közgadaságtan)', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 114, NULL),
            (7088, N'Ember és társadalom', N'Ember és társadalom', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 115, NULL),
            (7089, N'Tanító', N'Tanító', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 116, NULL),
            (7090, N'Könyvtárostanár', N'Könyvtárostanár', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 117, NULL),
            (7091, N'Fejlesztő pedagógus', N'Fejlesztő pedagógus', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 118, NULL),
            (7092, N'Gyógypedagógus', N'Gyógypedagógus', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 119, NULL),
            (7093, N'Etika', N'Etika', NULL, NULL, NULL, N'SzakTipus', 112, N'T', N'T', 2, NULL, 120, NULL),
            (6963, N'Na', N'Na', NULL, NULL, NULL, N'GimnaziumiEvfolyamTipus', 111, N'T', N'T', 1, NULL, 1, NULL),
            (6964, N'4 évfolyamos', N'4 évfolyamos', NULL, NULL, NULL, N'GimnaziumiEvfolyamTipus', 111, N'T', N'T', 2, NULL, 2, NULL),
            (6965, N'6 évfolyamos', N'6 évfolyamos', NULL, NULL, NULL, N'GimnaziumiEvfolyamTipus', 111, N'T', N'T', 3, NULL, 3, NULL),
            (6966, N'8 évfolyamos', N'8 évfolyamos', NULL, NULL, NULL, N'GimnaziumiEvfolyamTipus', 111, N'T', N'T', 4, NULL, 4, NULL),
            (6936, N'Poszeidon modul', N'Poszeidon modul', NULL, NULL, NULL, N'LicenceFunkciokTipus', 109, N'T', N'T', 1, NULL, 1, NULL),
            (6937, N'Belső levelezés modul', N'Belső levelezés modul', NULL, NULL, NULL, N'LicenceFunkciokTipus', 109, N'T', N'T', 2, NULL, 2, NULL),
            (6938, N'Érettségi applikáció', N'Érettségi applikáció', NULL, NULL, NULL, N'LicenceFunkciokTipus', 109, N'T', N'T', 3, NULL, 3, NULL),
            (6933, N'forint (HUF)', N'forint (HUF)', NULL, NULL, NULL, N'PenznemTipus', 108, N'T', N'T', 1, NULL, 1, NULL),
            (6934, N'euró (EUR)', N'euró (EUR)', NULL, NULL, NULL, N'PenznemTipus', 108, N'T', N'T', 2, NULL, 2, NULL),
            (6935, N'dollár (USD)', N'dollár (USD)', NULL, NULL, NULL, N'PenznemTipus', 108, N'T', N'T', 3, NULL, 3, NULL),
            (6919, N'Villanyóra', N'Villanyóra', NULL, NULL, NULL, N'MerohelyTipus', 106, N'T', N'T', 1, NULL, 1, NULL),
            (6920, N'Gázóra', N'Gázóra', NULL, NULL, NULL, N'MerohelyTipus', 106, N'T', N'T', 2, NULL, 2, NULL),
            (6921, N'Vízóra', N'Vízóra', NULL, NULL, NULL, N'MerohelyTipus', 106, N'T', N'T', 3, NULL, 3, NULL),
            (6922, N'Egyéb', N'Egyéb', NULL, NULL, NULL, N'MerohelyTipus', 106, N'T', N'T', 4, NULL, 4, NULL),
            (6923, N'Villany', N'Villany', NULL, NULL, NULL, N'KozmuSzamlaTipus', 107, N'T', N'T', 1, NULL, 1, NULL),
            (6924, N'Gáz', N'Gáz', NULL, NULL, NULL, N'KozmuSzamlaTipus', 107, N'T', N'T', 2, NULL, 2, NULL),
            (6925, N'Víz', N'Víz', NULL, NULL, NULL, N'KozmuSzamlaTipus', 107, N'T', N'T', 3, NULL, 3, NULL),
            (6926, N'Csatorna', N'Csatorna', NULL, NULL, NULL, N'KozmuSzamlaTipus', 107, N'T', N'T', 4, NULL, 4, NULL),
            (6927, N'Szemétszállítás', N'Szemétszállítás', NULL, NULL, NULL, N'KozmuSzamlaTipus', 107, N'T', N'T', 5, NULL, 5, NULL),
            (6928, N'Telekommunikáció', N'Telekommunikáció', NULL, NULL, NULL, N'KozmuSzamlaTipus', 107, N'T', N'T', 6, NULL, 6, NULL),
            (6929, N'Telefon', N'Telefon', NULL, NULL, NULL, N'KozmuSzamlaTipus', 107, N'T', N'T', 7, NULL, 7, NULL),
            (6930, N'Televízió', N'Televízió', NULL, NULL, NULL, N'KozmuSzamlaTipus', 107, N'T', N'T', 8, NULL, 8, NULL),
            (6931, N'Internet', N'Internet', NULL, NULL, NULL, N'KozmuSzamlaTipus', 107, N'T', N'T', 9, NULL, 9, NULL),
            (6932, N'Egyéb', N'Egyéb', NULL, NULL, NULL, N'KozmuSzamlaTipus', 107, N'T', N'T', 10, NULL, 10, NULL),
            (6878, N'Magyar Katolikus Egyház (római katolikus)', N'Magyar Katolikus Egyház (római katolikus)', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 1, NULL, 1, NULL),
            (6879, N'Magyar Katolikus Egyház (görög katolikus)', N'Magyar Katolikus Egyház (görög katolikus)', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 2, NULL, 2, NULL),
            (6880, N'Magyarországi Református Egyház', N'Magyarországi Református Egyház', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 3, NULL, 3, NULL),
            (6881, N'Magyarországi Evangélikus Egyház', N'Magyarországi Evangélikus Egyház', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 4, NULL, 4, NULL),
            (6882, N'Magyarországi Zsidó Hitközségek Szövetsége', N'Magyarországi Zsidó Hitközségek Szövetsége', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 5, NULL, 5, NULL),
            (6883, N'Egységes Magyarországi Izraelita Hitközség (Statusquo Ante)', N'Egységes Magyarországi Izraelita Hitközség (Statusquo Ante)', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 6, NULL, 6, NULL),
            (6884, N'Magyarországi Autonóm Orthodox Izraelita Hitközség', N'Magyarországi Autonóm Orthodox Izraelita Hitközség', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 7, NULL, 7, NULL),
            (6885, N'Budai Szerb Ortodox Egyházmegye', N'Budai Szerb Ortodox Egyházmegye', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 8, NULL, 8, NULL),
            (6886, N'Konstantinápolyi Egyetemes Patriarchátus – Magyarországi Ortodox Exarchátus', N'Konstantinápolyi Egyetemes Patriarchátus – Magyarországi Ortodox Exarchátus', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 9, NULL, 9, NULL),
            (6887, N'Magyarországi Bolgár Ortodox Egyház', N'Magyarországi Bolgár Ortodox Egyház', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 10, NULL, 10, NULL),
            (6888, N'Magyarországi Román Ortodox Egyházmegye', N'Magyarországi Román Ortodox Egyházmegye', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 11, NULL, 11, NULL),
            (6889, N'Orosz Ortodox Egyház Magyar Egyházmegyéje (Moszkvai Patriarchátus)', N'Orosz Ortodox Egyház Magyar Egyházmegyéje (Moszkvai Patriarchátus)', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 12, NULL, 12, NULL),
            (6890, N'Magyar Unitárius Egyház Magyarországi Egyházkerülete (Magyarországi Unitárius Egyház)', N'Magyar Unitárius Egyház Magyarországi Egyházkerülete (Magyarországi Unitárius Egyház)', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 13, NULL, 13, NULL),
            (6891, N'Magyarországi Baptista Egyház', N'Magyarországi Baptista Egyház', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 14, NULL, 14, NULL),
            (6892, N'HIT Gyülekezete', N'HIT Gyülekezete', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 15, NULL, 15, NULL),
            (6893, N'Magyarországi Metodista Egyház', N'Magyarországi Metodista Egyház', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 16, NULL, 16, NULL),
            (6894, N'Magyar Pünkösdi Egyház', N'Magyar Pünkösdi Egyház', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 17, NULL, 17, NULL),
            (6895, N'Szent Margit Anglikán/Episzkopális Egyház', N'Szent Margit Anglikán/Episzkopális Egyház', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 18, NULL, 18, NULL),
            (6896, N'Erdélyi Gyülekezet', N'Erdélyi Gyülekezet', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 19, NULL, 19, NULL),
            (6897, N'Hetednapi Adventista Egyház', N'Hetednapi Adventista Egyház', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 20, NULL, 20, NULL),
            (6898, N'Magyarországi Kopt Ortodox Egyház', N'Magyarországi Kopt Ortodox Egyház', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 21, NULL, 21, NULL),
            (6899, N'Magyar Iszlám Közösség', N'Magyar Iszlám Közösség', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 22, NULL, 22, NULL),
            (6900, N'Krisztusban Hívő Nazarénus Gyülekezetek', N'Krisztusban Hívő Nazarénus Gyülekezetek', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 23, NULL, 23, NULL),
            (6901, N'Magyarországi Krisna-tudatú Hívők Közössége', N'Magyarországi Krisna-tudatú Hívők Közössége', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 24, NULL, 24, NULL),
            (6902, N'Az Üdvhadsereg Szabadegyház Magyarország', N'Az Üdvhadsereg Szabadegyház Magyarország', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 25, NULL, 25, NULL),
            (6903, N'Az Utolsó Napok Szentjeinek Jézus Krisztus Egyháza', N'Az Utolsó Napok Szentjeinek Jézus Krisztus Egyháza', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 26, NULL, 26, NULL),
            (6904, N'Magyarországi Jehova Tanúi Egyház', N'Magyarországi Jehova Tanúi Egyház', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 27, NULL, 27, NULL),
            (6905, N'Buddhista vallási közösségek', N'Buddhista vallási közösségek', NULL, NULL, NULL, N'HittanTipus', 105, N'T', N'T', 28, NULL, 28, NULL),
            (6868, N'Alapfokozat', N'Alapfokozat', NULL, NULL, NULL, N'VegzettsegTipus', 103, N'T', N'T', 100, NULL, 9, NULL),
            (6862, N'Alapfokozat BA', N'Alapfokozat BA', NULL, NULL, NULL, N'VegzettsegTipus', 103, N'T', N'T', 90, NULL, 8, NULL),
            (6861, N'Alapfokozat BSc', N'Alapfokozat BSc', NULL, NULL, NULL, N'VegzettsegTipus', 103, N'T', N'T', 80, NULL, 7, NULL),
            (6856, N'Alapfokú művészetoktatási végzettség', N'Alapfokú művészetoktatási végzettség', NULL, NULL, NULL, N'VegzettsegTipus', 103, N'T', N'T', 140, NULL, 16, NULL),
            (6866, N'Doktori fokozat DLA', N'Doktori fokozat DLA', NULL, NULL, NULL, N'VegzettsegTipus', 103, N'T', N'T', 60, NULL, 2, NULL),
            (6865, N'Doktori fokozat PhD', N'Doktori fokozat PhD', NULL, NULL, NULL, N'VegzettsegTipus', 103, N'T', N'T', 50, NULL, 1, NULL),
            (6860, N'Egyetem', N'Egyetem', NULL, NULL, NULL, N'VegzettsegTipus', 103, N'T', N'T', 10, NULL, 6, NULL),
            (6858, N'Érettségi végzettség', N'Érettségi végzettség', NULL, NULL, NULL, N'VegzettsegTipus', 103, N'T', N'T', 110, NULL, 12, NULL),
            (6859, N'Főiskola', N'Főiskola', NULL, NULL, NULL, N'VegzettsegTipus', 103, N'T', N'T', 70, NULL, 10, NULL),
            (6857, N'Középfokú művészetoktatási végzettség', N'Középfokú művészetoktatási végzettség', NULL, NULL, NULL, N'VegzettsegTipus', 103, N'T', N'T', 130, NULL, 14, NULL),
            (6962, N'Alapfokú iskolai végzettség', N'Alapfokú iskolai végzettség', NULL, NULL, NULL, N'VegzettsegTipus', 103, N'T', N'T', 120, NULL, 15, NULL),
            (6867, N'Más külföldi képzéstípus szintje', N'Más külföldi képzéstípus szintje', NULL, NULL, NULL, N'VegzettsegTipus', 103, N'T', N'T', 150, NULL, 17, NULL),
            (6869, N'Mesterfokozat', N'Mesterfokozat', NULL, NULL, NULL, N'VegzettsegTipus', 103, N'T', N'T', 40, NULL, 5, NULL),
            (6864, N'Mesterfokozat MA', N'Mesterfokozat MA', NULL, NULL, NULL, N'VegzettsegTipus', 103, N'T', N'T', 30, NULL, 4, NULL),
            (6863, N'Mesterfokozat MSc', N'Mesterfokozat MSc', NULL, NULL, NULL, N'VegzettsegTipus', 103, N'T', N'T', 20, NULL, 3, NULL),
            (6855, N'Nincs megadva', N'Nincs megadva', NULL, NULL, NULL, N'VegzettsegTipus', 103, N'T', N'T', 1, NULL, 18, NULL),
            (8794, N'Más külföldi oklevél végzettségi szintje', N'Más külföldi oklevél végzettségi szintje', NULL, NULL, NULL, N'VegzettsegTipus', 103, N'T', N'T', 105, NULL, 11, NULL),
            (8795, N'Középfokú végzettség', N'Középfokú végzettség', NULL, NULL, NULL, N'VegzettsegTipus', 103, N'T', N'T', 115, NULL, 13, NULL),
            (6837, N'Differenciált', N'Differenciált', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 2, NULL, 1, NULL),
            (6838, N'Dráma', N'Dráma', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 2, NULL, 2, NULL),
            (6839, N'IKT tanóra', N'IKT tanóra', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 2, NULL, 3, NULL),
            (6840, N'IPR tanóra', N'IPR tanóra', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 2, NULL, 4, NULL),
            (6841, N'Kooperatív', N'Kooperatív', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 2, NULL, 5, NULL),
            (6842, N'Mérés', N'Mérés', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 2, NULL, 6, NULL),
            (6843, N'Mindennapos testnevelés', N'Mindennapos testnevelés', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 2, NULL, 7, NULL),
            (6844, N'Osztályfőnöki', N'Osztályfőnöki', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 2, NULL, 8, NULL),
            (6845, N'Nem szakrendszerű óra', N'Nem szakrendszerű óra', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 2, NULL, 9, NULL),
            (6846, N'Nemzetiségi óra', N'Nemzetiségi óra', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 2, NULL, 10, NULL),
            (6847, N'TÁMOP óra', N'TÁMOP óra', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 2, NULL, 11, NULL),
            (6848, N'Túlóra', N'Túlóra', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 2, NULL, 12, NULL),
            (6849, N'Bontott', N'Bontott', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 2, NULL, 13, NULL),
            (6850, N'Párhuzamos óra', N'Párhuzamos óra', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 2, NULL, 14, NULL),
            (6851, N'TIOP 12', N'TIOP 12', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 2, NULL, 15, NULL),
            (8356, N'2019/2020-as tanév tananyagának ismétlése', N'2019/2020-as tanév tananyagának ismétlése', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 50, NULL, 16, NULL),
            (8357, N'2019/2020-as tanév tananyagának kiegészítése', N'2019/2020-as tanév tananyagának kiegészítése', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 51, NULL, 17, NULL),
            (8358, N'A tanulók digitális készségfejlesztése', N'A tanulók digitális készségfejlesztése', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 52, NULL, 18, NULL),
            (8359, N'Projektmunka', N'Projektmunka', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 53, NULL, 19, NULL),
            (8360, N'A tanulók kompenzációs íráskészség fejlesztése', N'A tanulók kompenzációs íráskészség fejlesztése', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 54, NULL, 20, NULL),
            (8361, N'A tanulók kompenzációs olvasási képesség fejlesztése', N'A tanulók kompenzációs olvasási képesség fejlesztése', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 55, NULL, 21, NULL),
            (8362, N'A tanulók matematikai gondolkodási képességének kompenzációs fejlesztése', N'A tanulók matematikai gondolkodási képességének kompenzációs fejlesztése', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 56, NULL, 22, NULL),
            (8363, N'A tanulók nyelvi képességének kompenzációs fejlesztése', N'A tanulók nyelvi képességének kompenzációs fejlesztése', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 57, NULL, 23, NULL),
            (8364, N'A tanulók önmegismerésének segítése', N'A tanulók önmegismerésének segítése', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 58, NULL, 24, NULL),
            (8365, N'A tanulók önállóságra nevelése', N'A tanulók önállóságra nevelése', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 59, NULL, 25, NULL),
            (8366, N'A tanulók tanulásszervezési kompenzációs technikájának erősítése', N'A tanulók tanulásszervezési kompenzációs technikájának erősítése', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 60, NULL, 26, NULL),
            (8367, N'A tanulók orientációs támogatása', N'A tanulók orientációs támogatása', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 61, NULL, 27, NULL),
            (8368, N'A tanulók szocialitásának fejlesztése', N'A tanulók szocialitásának fejlesztése', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 62, NULL, 28, NULL),
            (8369, N'A tanulók továbbhaladását előkészítő tevékenységek', N'A tanulók továbbhaladását előkészítő tevékenységek', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 63, NULL, 29, NULL),
            (8377, N'Pedagógus által jelzett egyéni esetkezelés', N'Pedagógus által jelzett egyéni esetkezelés', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 64, NULL, 30, NULL),
            (8378, N'Projektmunka támogatása', N'Projektmunka támogatása', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 65, NULL, 31, NULL),
            (8379, N'Digitális készségfejlesztés', N'Digitális készségfejlesztés', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 66, NULL, 32, NULL),
            (8380, N'Közösségi munka támogatása', N'Közösségi munka támogatása', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 67, NULL, 33, NULL),
            (8381, N'Tanóra előkészítésének támogatása', N'Tanóra előkészítésének támogatása', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 68, NULL, 34, NULL),
            (8382, N'Tanóra lebonyolításának támogatása', N'Tanóra lebonyolításának támogatása', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 69, NULL, 35, NULL),
            (8383, N'Napközis tevékenység keretében egyéni fejlesztés', N'Napközis tevékenység keretében egyéni fejlesztés', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 70, NULL, 36, NULL),
            (8615, N'On-line óra', N'On-line óra', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'T', N'T', 75, NULL, 37, NULL),
            (9077, N'Egybefüggő gyakorlat / Gyakorlat', N'Egybefüggő gyakorlat / Gyakorlat', NULL, NULL, NULL, N'OraTulajdonsagTipus', 102, N'F', N'T', 3, NULL, 38, NULL),
            (6809, N'Na', N'Na', NULL, NULL, NULL, N'KozossegiSzolgalatTipus', 101, N'T', N'T', 1, NULL, 1, NULL),
            (6801, N'Egészségügyi tevékenység', N'Egészségügyi tevékenység', NULL, NULL, NULL, N'KozossegiSzolgalatTipus', 101, N'T', N'T', 2, NULL, 2, NULL),
            (6802, N'Szociális és jótékonysági tevékenység', N'Szociális és jótékonysági tevékenység', NULL, NULL, NULL, N'KozossegiSzolgalatTipus', 101, N'T', N'T', 3, NULL, 3, NULL),
            (6803, N'Oktatási tevékenység', N'Oktatási tevékenység', NULL, NULL, NULL, N'KozossegiSzolgalatTipus', 101, N'T', N'T', 4, NULL, 4, NULL),
            (6804, N'Kulturális és közösségi tevékenység', N'Kulturális és közösségi tevékenység', NULL, NULL, NULL, N'KozossegiSzolgalatTipus', 101, N'T', N'T', 5, NULL, 5, NULL),
            (6805, N'Környezet- és természetvédelemi tevékenység', N'Környezet- és természetvédelemi tevékenység', NULL, NULL, NULL, N'KozossegiSzolgalatTipus', 101, N'T', N'T', 6, NULL, 6, NULL),
            (6806, N'Katasztrófavédelmi tevékenység', N'Katasztrófavédelmi tevékenység', NULL, NULL, NULL, N'KozossegiSzolgalatTipus', 101, N'T', N'T', 7, NULL, 7, NULL),
            (6807, N'Óvodás korú, sajátos nevelési igényű gyermekekkel, tanulókkal, az idős emberekkel közös sport- és szabadidős tevékenység', N'Óvodás korú, sajátos nevelési igényű gyermekekkel, tanulókkal, az idős emberekkel közös sport- és szabadidős tevékenység', NULL, NULL, NULL, N'KozossegiSzolgalatTipus', 101, N'T', N'T', 8, NULL, 8, NULL),
            (6808, N'Egyes rendőrségi feladatok ellátására létrehozott szerveknél bűn- és baleset-megelőzési tevékenység', N'Egyes rendőrségi feladatok ellátására létrehozott szerveknél bűn- és baleset-megelőzési tevékenység', NULL, NULL, NULL, N'KozossegiSzolgalatTipus', 101, N'T', N'T', 9, NULL, 9, NULL),
            (6942, N'Felkészítő foglalkozás', N'Felkészítő foglalkozás', NULL, NULL, NULL, N'KozossegiSzolgalatTipus', 101, N'T', N'T', 10, NULL, 10, NULL),
            (6943, N'Záró foglalkozás', N'Záró foglalkozás', NULL, NULL, NULL, N'KozossegiSzolgalatTipus', 101, N'T', N'T', 11, NULL, 11, NULL),
            (6571, N'nincs', N'Na', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 1, NULL, 1, NULL),
            (6572, N'intézményvezető', N'intézményvezető', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 10, NULL, 2, NULL),
            (6573, N'tagintézmény-vezető', N'tagintézmény-vezető', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 20, NULL, 3, NULL)
                ) x (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
                WHERE NOT EXISTS (SELECT 1 FROM #DICTIONARYITEMBASE d WHERE x.ID = d.ID)
                
    INSERT INTO #DICTIONARYITEMBASE (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
        SELECT * FROM ( VALUES
            (6574, N'intézményegység-vezető', N'Head of the Institutional Unit', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 40, NULL, 4, NULL),
            (6575, N'intézményvezető-helyettes', N'intézményvezető-helyettes', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 60, NULL, 5, NULL),
            (6576, N'tagintézményvezető-helyettes', N'tagintézményvezető-helyettes', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 70, NULL, 6, NULL),
            (6577, N'intézményegységvezető-helyettes', N'intézményegységvezető-helyettes', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 90, NULL, 7, NULL),
            (6578, N'gyakorlati oktatásvezető', N'gyakorlati oktatásvezető', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 110, NULL, 8, NULL),
            (7599, N'másik KRÉTA rendszerben vezető', N'másik KRÉTA rendszerben vezető', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 130, NULL, 9, NULL),
            (7755, N'igazgató', N'Director', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 30, NULL, 10, NULL),
            (7756, N'igazgatóhelyettes', N'igazgatóhelyettes', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 80, NULL, 11, NULL),
            (7757, N'kollégiumvezető', N'Head of Dormitory', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 50, NULL, 12, NULL),
            (7758, N'főigazgató', N'főigazgató', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 120, NULL, 13, NULL),
            (7854, N'főigazgató-helyettes', N'főigazgató-helyettes', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'F', N'T', 140, NULL, 14, NULL),
            (8348, N'szakirányú oktatásért felelős igazgatóhelyettes', N'szakirányú oktatásért felelős igazgatóhelyettes', NULL, NULL, NULL, N'VezetoiOraszamokTipus', 96, N'T', N'T', 150, NULL, 15, NULL),
            (6465, N'teljes munkaidős', N'teljes munkaidős', NULL, NULL, NULL, N'FoglalkoztatasTipusa', 95, N'T', N'T', 1, NULL, 1, NULL),
            (6466, N'részmunkaidős', N'részmunkaidős', NULL, NULL, NULL, N'FoglalkoztatasTipusa', 95, N'T', N'T', 2, NULL, 2, NULL),
            (6467, N'óraadó', N'óraadó', NULL, NULL, NULL, N'FoglalkoztatasTipusa', 95, N'T', N'T', 3, NULL, 3, NULL),
            (6579, N'Közép szint', N'Közép szint', NULL, NULL, NULL, N'ErettsegiSzint', 97, N'T', N'T', 1, NULL, 1, NULL),
            (6580, N'Emelt szint', N'Emelt szint', NULL, NULL, NULL, N'ErettsegiSzint', 97, N'T', N'T', 2, NULL, 2, NULL),
            (6581, N'Rendes', N'Rendes', NULL, NULL, NULL, N'ErettsegiTipus', 98, N'T', N'T', 1, NULL, 1, NULL),
            (6582, N'Előrehozott', N'Előrehozott', NULL, NULL, NULL, N'ErettsegiTipus', 98, N'T', N'T', 2, NULL, 2, NULL),
            (6583, N'Pótló', N'Pótló', NULL, NULL, NULL, N'ErettsegiTipus', 98, N'T', N'T', 3, NULL, 3, NULL),
            (6584, N'Javító', N'Javító', NULL, NULL, NULL, N'ErettsegiTipus', 98, N'T', N'T', 4, NULL, 4, NULL),
            (6585, N'Szintemelő', N'Szintemelő', NULL, NULL, NULL, N'ErettsegiTipus', 98, N'T', N'T', 5, NULL, 5, NULL),
            (6586, N'Kiegészítő', N'Kiegészítő', NULL, NULL, NULL, N'ErettsegiTipus', 98, N'T', N'T', 6, NULL, 6, NULL),
            (6587, N'Ismétlő', N'Ismétlő', NULL, NULL, NULL, N'ErettsegiTipus', 98, N'T', N'T', 7, NULL, 7, NULL),
            (7787, N'Beszámítós', N'Beszámítós', NULL, NULL, NULL, N'ErettsegiTipus', 98, N'T', N'T', 8, NULL, 8, NULL),
            (6588, N'Magyar nyelv és irodalom', N'Magyar nyelv és irodalom', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 1, NULL, 1, NULL),
            (6589, N'Matematika', N'Matematika', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 2, NULL, 2, NULL),
            (6590, N'Történelem', N'Történelem', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 3, NULL, 3, NULL),
            (6591, N'Angol nyelv', N'Angol nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 4, NULL, 4, NULL),
            (6592, N'Német nyelv', N'Német nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 5, NULL, 5, NULL),
            (6593, N'Francia nyelv', N'Francia nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 6, NULL, 6, NULL),
            (6594, N'Spanyol nyelv', N'Spanyol nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 7, NULL, 7, NULL),
            (6595, N'Biológia', N'Biológia', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 8, NULL, 8, NULL),
            (6596, N'Fizika', N'Fizika', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 9, NULL, 9, NULL),
            (6597, N'Kémia', N'Kémia', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 10, NULL, 10, NULL),
            (6598, N'Földrajz', N'Földrajz', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 11, NULL, 11, NULL),
            (6599, N'Informatika', N'Informatika', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 12, NULL, 12, NULL),
            (6600, N'Testnevelés', N'Testnevelés', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 13, NULL, 13, NULL),
            (6601, N'Ének-zene', N'Ének-zene', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 14, NULL, 14, NULL),
            (6602, N'Mozgóképkultúra és médiaismeret', N'Mozgóképkultúra és médiaismeret', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 15, NULL, 15, NULL),
            (6603, N'Artista ismeretek', N'Artista ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 16, NULL, 16, NULL),
            (6604, N'Bányászati ismeretek', N'Bányászati ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 17, NULL, 17, NULL),
            (6605, N'Beás nyelv', N'Beás nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 18, NULL, 18, NULL),
            (6606, N'Belügyi rendészeti ismeretek', N'Belügyi rendészeti ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 19, NULL, 19, NULL),
            (6607, N'Biblia ismeret (Hit gyülekezete)', N'Biblia ismeret (Hit gyülekezete)', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 20, NULL, 20, NULL),
            (6608, N'Célnyelvi civilizáció', N'Célnyelvi civilizáció', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 21, NULL, 21, NULL),
            (6609, N'Dráma', N'Dráma', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 22, NULL, 22, NULL),
            (6610, N'Egészségügy ismeretek', N'Egészségügy ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 23, NULL, 23, NULL),
            (6611, N'Egészségügyi technika ismeretek', N'Egészségügyi technika ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 24, NULL, 24, NULL),
            (6612, N'Egyházzenész ismeretek', N'Egyházzenész ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 25, NULL, 25, NULL),
            (6613, N'Élelmiszeripar ismeretek', N'Élelmiszeripar ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 26, NULL, 26, NULL),
            (6614, N'Ember és társadalomismeret, etika', N'Ember és társadalomismeret, etika', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 27, NULL, 27, NULL),
            (6615, N'Emberismeret és etika', N'Emberismeret és etika', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 28, NULL, 28, NULL),
            (6616, N'Építőipar ismeretek', N'Építőipar ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 29, NULL, 29, NULL),
            (6617, N'Épületgépészet ismeretek', N'Épületgépészet ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 30, NULL, 30, NULL),
            (6618, N'Erdészet és vadgazdálkodás ismeretek', N'Erdészet és vadgazdálkodás ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 31, NULL, 31, NULL),
            (6619, N'Eszperantó nyelv', N'Eszperantó nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 32, NULL, 32, NULL),
            (6620, N'Evangélikus hittan', N'Evangélikus hittan', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 33, NULL, 33, NULL),
            (6621, N'Faipar ismeretek', N'Faipar ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 34, NULL, 34, NULL),
            (6622, N'Filozófia', N'Filozófia', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 35, NULL, 35, NULL),
            (6623, N'Finn nyelv', N'Finn nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 36, NULL, 36, NULL),
            (6624, N'Földmérés ismeretek', N'Földmérés ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 37, NULL, 37, NULL),
            (6625, N'Gazdasági ismeretek', N'Gazdasági ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 38, NULL, 38, NULL),
            (6626, N'Gépészet ismeretek', N'Gépészet ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 39, NULL, 39, NULL),
            (6627, N'Gyakorlatos színész ismeretek', N'Gyakorlatos színész ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 40, NULL, 40, NULL),
            (6628, N'Hang-, film és színháztechnika ismeretek', N'Hang-, film és színháztechnika ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 41, NULL, 41, NULL),
            (6629, N'Héber nyelv', N'Héber nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 42, NULL, 42, NULL),
            (6630, N'Holland nyelv', N'Holland nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 43, NULL, 43, NULL),
            (6631, N'Horvát nemzetiségi népismeret', N'Horvát nemzetiségi népismeret', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 44, NULL, 44, NULL),
            (6632, N'Horvát nemzetiségi nyelv és irodalom', N'Horvát nemzetiségi nyelv és irodalom', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 45, NULL, 45, NULL),
            (6633, N'Horvát nyelv', N'Horvát nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 46, NULL, 46, NULL),
            (6634, N'Informatika ismeretek', N'Informatika ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 47, NULL, 47, NULL),
            (6635, N'Japán nyelv', N'Japán nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 48, NULL, 48, NULL),
            (6636, N'Jazz-zenész ismeretek', N'Jazz-zenész ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 49, NULL, 49, NULL),
            (6637, N'Judasztika', N'Judasztika', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 50, NULL, 50, NULL),
            (6638, N'Katolikus hittan', N'Katolikus hittan', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 51, NULL, 51, NULL),
            (6639, N'Katonai alapismeretek', N'Katonai alapismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 52, NULL, 52, NULL),
            (6640, N'Képző- és iparművészet ismeretek', N'Képző- és iparművészet ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 53, NULL, 53, NULL),
            (6641, N'Kereskedelem ismeretek', N'Kereskedelem ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 54, NULL, 54, NULL),
            (6642, N'Kertészet és parképítés ismeretek', N'Kertészet és parképítés ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 55, NULL, 55, NULL),
            (6643, N'Kínai nyelv', N'Kínai nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 56, NULL, 56, NULL),
            (6644, N'Klasszikus zenész ismeretek', N'Klasszikus zenész ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 57, NULL, 57, NULL),
            (6645, N'Kohászat ismeretek', N'Kohászat ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 58, NULL, 58, NULL),
            (6646, N'Könnyűipar ismeretek', N'Könnyűipar ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 59, NULL, 59, NULL),
            (6647, N'Környezetvédelem-vízgazdálkodás ismeretek', N'Környezetvédelem-vízgazdálkodás ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 60, NULL, 60, NULL),
            (6648, N'Közgazdaság ismeretek', N'Közgazdaság ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 61, NULL, 61, NULL),
            (6649, N'Közlekedés ismeretek', N'Közlekedés ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 62, NULL, 62, NULL),
            (6650, N'Közlekedésépítő ismeretek', N'Közlekedésépítő ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 63, NULL, 63, NULL),
            (6651, N'Közlekedésgépész ismeretek', N'Közlekedésgépész ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 64, NULL, 64, NULL),
            (6652, N'Latin nyelv', N'Latin nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 65, NULL, 65, NULL),
            (6653, N'Lengyel nyelv', N'Lengyel nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 66, NULL, 66, NULL),
            (6654, N'Lovári nyelv', N'Lovári nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 67, NULL, 67, NULL),
            (6655, N'Mezőgazdaság ismeretek', N'Mezőgazdaság ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 68, NULL, 68, NULL),
            (6656, N'Mezőgazdasági gépész ismeretek', N'Mezőgazdasági gépész ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 69, NULL, 69, NULL),
            (6657, N'Művészettörténet', N'Művészettörténet', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 70, NULL, 70, NULL),
            (6658, N'Német nemzetiségi népismeret', N'Német nemzetiségi népismeret', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 71, NULL, 71, NULL),
            (6659, N'Német nemzetiségi nyelv és irodalom', N'Német nemzetiségi nyelv és irodalom', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 72, NULL, 72, NULL),
            (6660, N'Népművészet', N'Népművészet', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 73, NULL, 73, NULL),
            (6661, N'Népzenész ismeretek', N'Népzenész ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 74, NULL, 74, NULL),
            (6662, N'Nyomdaipar ismeretek', N'Nyomdaipar ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 75, NULL, 75, NULL),
            (6663, N'Olasz nyelv', N'Olasz nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 76, NULL, 76, NULL),
            (6664, N'Optika ismeretek', N'Optika ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 77, NULL, 77, NULL),
            (6665, N'Orosz nyelv', N'Orosz nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 78, NULL, 78, NULL),
            (6666, N'Pedagógia ismeretek', N'Pedagógia ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 79, NULL, 79, NULL),
            (6667, N'Portugál nyelv', N'Portugál nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 80, NULL, 80, NULL),
            (6668, N'Pszichológia', N'Pszichológia', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 81, NULL, 81, NULL),
            (6669, N'Református hittan', N'Református hittan', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 82, NULL, 82, NULL),
            (6670, N'Rendészet ismeretek', N'Rendészet ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 83, NULL, 83, NULL),
            (6671, N'Roma/cigány nemzetiségi népismeret', N'Roma/cigány nemzetiségi népismeret', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 84, NULL, 84, NULL),
            (6672, N'Román nemzetiségi népismeret', N'Román nemzetiségi népismeret', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 85, NULL, 85, NULL),
            (6673, N'Román nemzetiségi nyelv és irodalom', N'Román nemzetiségi nyelv és irodalom', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 86, NULL, 86, NULL),
            (6674, N'Román nyelv', N'Román nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 87, NULL, 87, NULL),
            (6675, N'Sport ismeretek', N'Sport ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 88, NULL, 88, NULL),
            (6676, N'Szépészet ismeretek', N'Szépészet ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 89, NULL, 89, NULL),
            (6677, N'Szerb nemzetiségi népismeret', N'Szerb nemzetiségi népismeret', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 90, NULL, 90, NULL),
            (6678, N'Szerb nemzetiségi nyelv és irodalom', N'Szerb nemzetiségi nyelv és irodalom', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 91, NULL, 91, NULL),
            (6679, N'Szerb nyelv', N'Szerb nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 92, NULL, 92, NULL),
            (6680, N'Szlovák nemzetiségi népismeret', N'Szlovák nemzetiségi népismeret', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 93, NULL, 93, NULL),
            (6681, N'Szlovák nemzetiségi nyelv és irodalom', N'Szlovák nemzetiségi nyelv és irodalom', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 94, NULL, 94, NULL),
            (6682, N'Szlovák nyelv', N'Szlovák nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 95, NULL, 95, NULL),
            (6683, N'Szlovén nemzetiségi népismeret', N'Szlovén nemzetiségi népismeret', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 96, NULL, 96, NULL),
            (6684, N'Szlovén nyelv', N'Szlovén nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 97, NULL, 97, NULL),
            (6685, N'Szociális ismeretek', N'Szociális ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 98, NULL, 98, NULL),
            (6686, N'Szórakoztató zenész ismeretek', N'Szórakoztató zenész ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 99, NULL, 99, NULL),
            (6687, N'Táncos ismeretek', N'Táncos ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 100, NULL, 100, NULL),
            (6688, N'Társadalomismeret', N'Társadalomismeret', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 101, NULL, 101, NULL),
            (6689, N'Távközlés ismeretek', N'Távközlés ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 102, NULL, 102, NULL),
            (6690, N'Természettudomány', N'Természettudomány', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 103, NULL, 103, NULL),
            (6691, N'Turisztika ismeretek', N'Turisztika ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 104, NULL, 104, NULL),
            (6692, N'Ukrán nyelv', N'Ukrán nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 105, NULL, 105, NULL),
            (6693, N'Utazás és turizmus', N'Utazás és turizmus', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 106, NULL, 106, NULL),
            (6694, N'Ügyvitel ismeretek', N'Ügyvitel ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 107, NULL, 107, NULL),
            (6695, N'Vegyész ismeretek', N'Vegyész ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 108, NULL, 108, NULL),
            (6696, N'Vegyipar ismeretek', N'Vegyipar ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 109, NULL, 109, NULL),
            (6697, N'Vendéglátóipar ismeretek', N'Vendéglátóipar ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 110, NULL, 110, NULL),
            (6698, N'Villamosipar és elektronika ismeretek', N'Villamosipar és elektronika ismeretek', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 111, NULL, 111, NULL),
            (6699, N'Vizuális kultúra', N'Vizuális kultúra', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 112, NULL, 112, NULL),
            (6700, N'Gazdasági ismeretek I.', N'Gazdasági ismeretek I.', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 113, NULL, 113, NULL),
            (6701, N'Gazdasági ismeretek II.', N'Gazdasági ismeretek II.', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 114, NULL, 114, NULL),
            (7788, N'Magyar nyelv, mind idegen nyelv', N'Magyar nyelv, mind idegen nyelv', NULL, NULL, NULL, N'ErettsegiTantargy', 99, N'T', N'T', 115, NULL, 115, NULL),
            (6108, N'Na', N'Na', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 1, NULL, 1, NULL),
            (6109, N'3D modellező', N'3D modellező', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 102, NULL, 2, NULL),
            (6110, N'Adóügyintéző', N'Adóügyintéző', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 103, NULL, 3, NULL),
            (6111, N'Agrár vállalkozó', N'Agrár vállalkozó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 104, NULL, 4, NULL),
            (6112, N'Alapfokú katonai vezető-helyettes', N'Alapfokú katonai vezető-helyettes', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 105, NULL, 5, NULL),
            (6113, N'Állatgondozó', N'Állatgondozó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 106, NULL, 6, NULL),
            (6114, N'Általános laboráns', N'Általános laboráns', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 107, NULL, 7, NULL),
            (6115, N'Animációkészítő', N'Animációkészítő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 108, NULL, 8, NULL),
            (6116, N'Aqua tréner', N'Aqua tréner', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 109, NULL, 9, NULL),
            (6117, N'Aranykalászos gazda', N'Aranykalászos gazda', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 110, NULL, 10, NULL),
            (6118, N'Asztalosipari szerelő', N'Asztalosipari szerelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 111, NULL, 11, NULL),
            (6119, N'Bábkészítő', N'Bábkészítő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 112, NULL, 12, NULL),
            (6120, N'Banki, befektetési termékértékesítő', N'Banki, befektetési termékértékesítő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 113, NULL, 13, NULL),
            (6121, N'Bérügyintéző', N'Bérügyintéző', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 114, NULL, 14, NULL),
            (6122, N'Betegkísérő', N'Betegkísérő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 115, NULL, 15, NULL),
            (6123, N'Bevontelektródás kézi ívhegesztő', N'Bevontelektródás kézi ívhegesztő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 116, NULL, 16, NULL),
            (6124, N'Biztosításközvetítő (függő és független)', N'Biztosításközvetítő (függő és független)', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 117, NULL, 17, NULL),
            (6125, N'Bolti hentes', N'Bolti hentes', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 118, NULL, 18, NULL),
            (6126, N'Borász', N'Borász', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 119, NULL, 19, NULL),
            (6127, N'Bőrtárgy készítő', N'Bőrtárgy készítő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 120, NULL, 20, NULL),
            (6128, N'Büntetés-végrehajtási őr', N'Büntetés-végrehajtási őr', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 122, NULL, 21, NULL),
            (6129, N'Büntetés-végrehajtási segédfelügyelő', N'Büntetés-végrehajtási segédfelügyelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 123, NULL, 22, NULL),
            (6130, N'Bútor- és szőnyegbecsüs', N'Bútor- és szőnyegbecsüs', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 121, NULL, 23, NULL),
            (6131, N'Cipőfelsőrész-készítő', N'Cipőfelsőrész-készítő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 124, NULL, 24, NULL),
            (6132, N'Cipőgyártó', N'Cipőgyártó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 125, NULL, 25, NULL),
            (6133, N'Családellátó', N'Családellátó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 126, NULL, 26, NULL),
            (6134, N'Csapatszolgálati járőrtárs', N'Csapatszolgálati járőrtárs', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 127, NULL, 27, NULL),
            (6135, N'Csatornamű-kezelő', N'Csatornamű-kezelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 128, NULL, 28, NULL),
            (6136, N'Csőhálózatszerelő', N'Csőhálózatszerelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 132, NULL, 29, NULL),
            (6137, N'Csokoládétermék-gyártó', N'Csokoládétermék-gyártó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 129, NULL, 30, NULL),
            (6138, N'Csontozó munkás', N'Csontozó munkás', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 130, NULL, 31, NULL),
            (6139, N'Csoportos fitness instruktor', N'Csoportos fitness instruktor', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 131, NULL, 32, NULL),
            (6140, N'Digitális kép- és szövegszerkesztő', N'Digitális kép- és szövegszerkesztő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 133, NULL, 33, NULL),
            (6141, N'Digitális nyomóforma készítő', N'Digitális nyomóforma készítő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 134, NULL, 34, NULL),
            (6142, N'Digitálistérkép- kezelő', N'Digitálistérkép- kezelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 135, NULL, 35, NULL),
            (6143, N'Díszletkivitelező', N'Díszletkivitelező', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 136, NULL, 36, NULL),
            (6144, N'Ékszerbecsüs', N'Ékszerbecsüs', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 137, NULL, 37, NULL),
            (6145, N'Elektromos halászgép kezelője', N'Elektromos halászgép kezelője', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 138, NULL, 38, NULL),
            (6146, N'Elektronikai gyártósori műszerész', N'Elektronikai gyártósori műszerész', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 139, NULL, 39, NULL),
            (6147, N'Élelmiszer-, vegyiáru eladó', N'Élelmiszer-, vegyiáru eladó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 140, NULL, 40, NULL),
            (6148, N'Élelmiszeripari higiéniai és minőségbiztosítási munkatárs', N'Élelmiszeripari higiéniai és minőségbiztosítási munkatárs', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 141, NULL, 41, NULL),
            (6149, N'Élelmiszeripari laboráns', N'Élelmiszeripari laboráns', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 142, NULL, 42, NULL),
            (6150, N'Előadóművészeti jegypénztáros', N'Előadóművészeti jegypénztáros', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 143, NULL, 43, NULL),
            (6151, N'Előadóművészeti közönség kezelő', N'Előadóművészeti közönség kezelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 144, NULL, 44, NULL),
            (6152, N'Előadóművészeti program- és projektszervező', N'Előadóművészeti program- és projektszervező', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 145, NULL, 45, NULL),
            (6153, N'Előadóművészeti szervező', N'Előadóművészeti szervező', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 146, NULL, 46, NULL),
            (6154, N'Építményzsaluzat és fémállvány szerelő', N'Építményzsaluzat és fémállvány szerelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 147, NULL, 47, NULL),
            (6155, N'Erdőművelő', N'Erdőművelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 148, NULL, 48, NULL),
            (6156, N'Erőművi gőzturbina gépész', N'Erőművi gőzturbina gépész', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 149, NULL, 49, NULL),
            (6157, N'Erőművi kazángépész', N'Erőművi kazángépész', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 150, NULL, 50, NULL),
            (6158, N'Extruder gép kezelő', N'Extruder gép kezelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 151, NULL, 51, NULL),
            (6159, N'Fahajó javító', N'Fahajó javító', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 152, NULL, 52, NULL),
            (6160, N'Faipari gépkezelő', N'Faipari gépkezelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 153, NULL, 53, NULL),
            (6161, N'Faiskolai kertész', N'Faiskolai kertész', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 154, NULL, 54, NULL),
            (6162, N'Fakitermelő', N'Fakitermelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 155, NULL, 55, NULL),
            (6163, N'Falusi vendéglátó', N'Falusi vendéglátó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 156, NULL, 56, NULL),
            (6164, N'Famegmunkáló', N'Famegmunkáló', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 157, NULL, 57, NULL),
            (6165, N'Fehérnemű-készítő', N'Fehérnemű-készítő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 158, NULL, 58, NULL),
            (6166, N'Felépítő gép kezelő', N'Felépítő gép kezelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 159, NULL, 59, NULL),
            (6167, N'Felvonó karbantartó-szerelő', N'Felvonó karbantartó-szerelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 160, NULL, 60, NULL),
            (6168, N'Fertőtlenítő-sterilező', N'Fertőtlenítő-sterilező', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 161, NULL, 61, NULL),
            (6169, N'Festménybecsüs', N'Festménybecsüs', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 162, NULL, 62, NULL),
            (6170, N'Fitness instruktor', N'Fitness instruktor', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 163, NULL, 63, NULL),
            (6171, N'Fitness-wellness asszisztens', N'Fitness-wellness asszisztens', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 164, NULL, 64, NULL),
            (6172, N'Fluidumkitermelő', N'Fluidumkitermelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 165, NULL, 65, NULL),
            (6173, N'Fogyóelektródás védőgázas ívhegesztő', N'Fogyóelektródás védőgázas ívhegesztő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 166, NULL, 66, NULL),
            (6174, N'Fonó', N'Fonó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 167, NULL, 67, NULL),
            (6175, N'Fröccsöntő', N'Fröccsöntő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 168, NULL, 68, NULL),
            (6176, N'Fürdőüzemi gépész', N'Fürdőüzemi gépész', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 169, NULL, 69, NULL),
            (6177, N'Gát- és csatornaőr', N'Gát- és csatornaőr', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 170, NULL, 70, NULL),
            (6178, N'Gázcseretelep-kezelő', N'Gázcseretelep-kezelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 171, NULL, 71, NULL),
            (6179, N'Gázhegesztő', N'Gázhegesztő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 172, NULL, 72, NULL),
            (6180, N'Gépi kovács', N'Gépi kovács', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 173, NULL, 73, NULL),
            (6181, N'Gondnok', N'Gondnok', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 174, NULL, 74, NULL),
            (6182, N'Gumikeverék-készítő', N'Gumikeverék-készítő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 175, NULL, 75, NULL),
            (6183, N'Gyártósori gépész', N'Gyártósori gépész', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 176, NULL, 76, NULL),
            (6184, N'Gyermek- és ifjúsági felügyelő', N'Gyermek- és ifjúsági felügyelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 177, NULL, 77, NULL),
            (6185, N'Gyümölcspálinka-gyártó', N'Gyümölcspálinka-gyártó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 178, NULL, 78, NULL),
            (6186, N'Hálózati operációsrendszer-üzemeltető', N'Hálózati operációsrendszer-üzemeltető', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 179, NULL, 79, NULL),
            (6187, N'Hálózati rendszerüzemeltető', N'Hálózati rendszerüzemeltető', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 180, NULL, 80, NULL),
            (6188, N'Hálózati támogatási munkatárs', N'Hálózati támogatási munkatárs', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 181, NULL, 81, NULL),
            (6189, N'Hangosító', N'Hangosító', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 182, NULL, 82, NULL),
            (6190, N'Házi időszakos gyermekgondozó', N'Házi időszakos gyermekgondozó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 183, NULL, 83, NULL),
            (6191, N'Háztartási gépszerelő', N'Háztartási gépszerelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 184, NULL, 84, NULL),
            (6192, N'Hengerész', N'Hengerész', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 185, NULL, 85, NULL),
            (6193, N'Horgásztókezelő', N'Horgásztókezelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 186, NULL, 86, NULL),
            (6194, N'Hulladékfelvásárló és -gazdálkodó', N'Hulladékfelvásárló és -gazdálkodó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 187, NULL, 87, NULL),
            (6195, N'Hulladékgyűjtő és -szállító', N'Hulladékgyűjtő és -szállító', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 188, NULL, 88, NULL),
            (6196, N'Hulladékválogató és -feldolgozó', N'Hulladékválogató és -feldolgozó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 189, NULL, 89, NULL),
            (6197, N'Húskészítmény gyártó', N'Húskészítmény gyártó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 190, NULL, 90, NULL),
            (6198, N'Hűtőipari munkás', N'Hűtőipari munkás', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 191, NULL, 91, NULL),
            (6199, N'Ingatlan-nyilvántartási ügyintéző', N'Ingatlan-nyilvántartási ügyintéző', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 192, NULL, 92, NULL),
            (6200, N'Intézménytakarító', N'Intézménytakarító', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 193, NULL, 93, NULL),
            (6201, N'Ipari olaj- és gáztüzelő-berendezés kezelő', N'Ipari olaj- és gáztüzelő-berendezés kezelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 194, NULL, 94, NULL),
            (6202, N'Jármű- és gépszerelő', N'Jármű- és gépszerelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 195, NULL, 95, NULL),
            (6203, N'Járőr', N'Járőr', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 196, NULL, 96, NULL),
            (6204, N'Jövedéki ügyintéző', N'Jövedéki ügyintéző', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 197, NULL, 97, NULL),
            (6205, N'Kalander gép kezelő', N'Kalander gép kezelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 198, NULL, 98, NULL),
            (6206, N'Keksz- és ostyagyártó', N'Keksz- és ostyagyártó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 199, NULL, 99, NULL),
            (6207, N'Képesített hajós', N'Képesített hajós', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 200, NULL, 100, NULL),
            (6208, N'Képző és iparművészeti program- és projektszervező', N'Képző és iparművészeti program- és projektszervező', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 201, NULL, 101, NULL),
            (6209, N'Kerékpárszerelő', N'Kerékpárszerelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 202, NULL, 102, NULL),
            (6210, N'Kertépítő és -fenntartó', N'Kertépítő és -fenntartó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 203, NULL, 103, NULL),
            (6211, N'Kerti munkás', N'Kerti munkás', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 204, NULL, 104, NULL),
            (6212, N'Kesztyűs', N'Kesztyűs', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 205, NULL, 105, NULL),
            (6213, N'Keveréktakarmány-gyártó', N'Keveréktakarmány-gyártó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 206, NULL, 106, NULL),
            (6214, N'Kézi könyvkötő', N'Kézi könyvkötő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 207, NULL, 107, NULL),
            (6215, N'Kikötői karbantartó', N'Kikötői karbantartó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 208, NULL, 108, NULL),
            (6216, N'Kisteljesítményű kazán fűtő (max. 2 t/h)', N'Kisteljesítményű kazán fűtő (max. 2 t/h)', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 209, NULL, 109, NULL),
            (6217, N'Kistermelői élelmiszerelőállító', N'Kistermelői élelmiszerelőállító', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 210, NULL, 110, NULL),
            (6218, N'Költségvetés-gazdálkodási ügyintéző', N'Költségvetés-gazdálkodási ügyintéző', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 214, NULL, 111, NULL),
            (6219, N'Konyhai kisegítő', N'Konyhai kisegítő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 212, NULL, 112, NULL),
            (6220, N'Konzervgyártó', N'Konzervgyártó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 211, NULL, 113, NULL),
            (6221, N'Környezetvédelmi ügyintéző', N'Környezetvédelmi ügyintéző', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 215, NULL, 114, NULL),
            (6222, N'Kosárfonó', N'Kosárfonó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 213, NULL, 115, NULL),
            (6223, N'Kötő', N'Kötő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 216, NULL, 116, NULL),
            (6224, N'Kötöttáru-összeállító', N'Kötöttáru-összeállító', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 217, NULL, 117, NULL),
            (6225, N'Kulturális rendezvényszervező', N'Kulturális rendezvényszervező', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 218, NULL, 118, NULL),
            (6226, N'Lakástextil-készítő', N'Lakástextil-készítő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 219, NULL, 119, NULL),
            (6227, N'Lóápoló és gondozó', N'Lóápoló és gondozó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 220, NULL, 120, NULL),
            (6228, N'Medencefelügyelő', N'Medencefelügyelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 221, NULL, 121, NULL),
            (6229, N'Médiaművészeti program- és projektszervező', N'Médiaművészeti program- és projektszervező', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 222, NULL, 122, NULL),
            (6230, N'Mélyfúró', N'Mélyfúró', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 223, NULL, 123, NULL),
            (6231, N'Mézeskalács-készítő', N'Mézeskalács-készítő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 224, NULL, 124, NULL),
            (6232, N'Mezőgazdasági munkás', N'Mezőgazdasági munkás', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 225, NULL, 125, NULL),
            (6233, N'Motorfűrész-kezelő', N'Motorfűrész-kezelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 226, NULL, 126, NULL),
            (6234, N'Mozgóképkészítő', N'Mozgóképkészítő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 227, NULL, 127, NULL),
            (6235, N'Mozigépész', N'Mozigépész', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 228, NULL, 128, NULL),
            (6236, N'Műanyag hegesztő', N'Műanyag hegesztő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 231, NULL, 129, NULL),
            (6237, N'Műanyaghajó készítő', N'Műanyaghajó készítő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 232, NULL, 130, NULL),
            (6238, N'Műszakicikk-eladó', N'Műszakicikk-eladó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 233, NULL, 131, NULL),
            (6239, N'Műtárgybecsüs', N'Műtárgybecsüs', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 234, NULL, 132, NULL),
            (6240, N'Műtárgyvédelmi asszisztens', N'Műtárgyvédelmi asszisztens', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 235, NULL, 133, NULL),
            (6241, N'Múzeumi gyűjtemény- és raktárkezelő', N'Múzeumi gyűjtemény- és raktárkezelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 229, NULL, 134, NULL),
            (6242, N'Múzeumi preparátor', N'Múzeumi preparátor', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 230, NULL, 135, NULL),
            (6243, N'Nemszőtt-termék gyártó', N'Nemszőtt-termék gyártó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 236, NULL, 136, NULL),
            (6244, N'Nyílászáró és árnyékolástechnikai szerelő', N'Nyílászáró és árnyékolástechnikai szerelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 237, NULL, 137, NULL),
            (6245, N'Önjáró felülettisztító gépkezelő', N'Önjáró felülettisztító gépkezelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 238, NULL, 138, NULL),
            (6246, N'Őr-járőrtárs', N'Őr-járőrtárs', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 239, NULL, 139, NULL),
            (6247, N'Papír feldolgozó', N'Papír feldolgozó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 240, NULL, 140, NULL),
            (6248, N'Papíralapanyag-gyártó', N'Papíralapanyag-gyártó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 241, NULL, 141, NULL),
            (6249, N'Papírgyártó', N'Papírgyártó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 242, NULL, 142, NULL),
            (6250, N'Parkgondozó', N'Parkgondozó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 243, NULL, 143, NULL),
            (6251, N'Pénztárkezelő és valutapénztáros', N'Pénztárkezelő és valutapénztáros', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 244, NULL, 144, NULL),
            (6252, N'Pénzügyi projektmenedzser-asszisztens', N'Pénzügyi projektmenedzser-asszisztens', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 245, NULL, 145, NULL),
            (6253, N'Pénzügyi ügyintéző', N'Pénzügyi ügyintéző', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 246, NULL, 146, NULL),
            (6254, N'Pirotechnikai terméküzemeltető', N'Pirotechnikai terméküzemeltető', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 247, NULL, 147, NULL),
            (6255, N'Pirotechnikus raktárkezelő', N'Pirotechnikus raktárkezelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 248, NULL, 148, NULL),
            (6256, N'PLC programozó', N'PLC programozó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 249, NULL, 149, NULL),
            (6257, N'Produkciós asszisztens', N'Produkciós asszisztens', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 250, NULL, 150, NULL),
            (6258, N'Raktáros', N'Raktáros', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 251, NULL, 151, NULL),
            (6259, N'Raktárvezető', N'Raktárvezető', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 252, NULL, 152, NULL),
            (6260, N'Regeneráló balneoterápiás masszőr', N'Regeneráló balneoterápiás masszőr', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 253, NULL, 153, NULL),
            (6261, N'Relé jelnyelvi tolmács', N'Relé jelnyelvi tolmács', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 254, NULL, 154, NULL),
            (6262, N'Rendezvénytechnikus', N'Rendezvénytechnikus', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 1700, NULL, 155, NULL),
            (6263, N'Rögzítésvezető', N'Rögzítésvezető', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 255, NULL, 156, NULL),
            (6264, N'Sajtkészítő', N'Sajtkészítő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 256, NULL, 157, NULL),
            (6265, N'Savanyító', N'Savanyító', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 257, NULL, 158, NULL),
            (6266, N'Sírkő és műkőkészítő', N'Sírkő és műkőkészítő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 258, NULL, 159, NULL),
            (6267, N'Sörgyártó', N'Sörgyártó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 259, NULL, 160, NULL),
            (6268, N'Sportoktató (a sportág megjelölésével)', N'Sportoktató (a sportág megjelölésével)', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 260, NULL, 161, NULL),
            (6269, N'Sütőipari és gyorspékségi munkás', N'Sütőipari és gyorspékségi munkás', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 261, NULL, 162, NULL),
            (6270, N'Számítógépes adatrögzítő', N'Számítógépes adatrögzítő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 262, NULL, 163, NULL),
            (6271, N'Számítógépes műszaki rajzoló', N'Számítógépes műszaki rajzoló', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 263, NULL, 164, NULL),
            (6272, N'Számviteli ügyintéző', N'Számviteli ügyintéző', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 264, NULL, 165, NULL),
            (6273, N'Szikvízgyártó', N'Szikvízgyártó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 265, NULL, 166, NULL),
            (6274, N'Színpadi alsógépezet kezelő', N'Színpadi alsógépezet kezelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 266, NULL, 167, NULL),
            (6275, N'Színpadi díszítő, berendező', N'Színpadi díszítő, berendező', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 267, NULL, 168, NULL),
            (6276, N'Szinpadi hangtechnikus', N'Szinpadi hangtechnikus', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 268, NULL, 169, NULL),
            (6277, N'Színpadi segéddíszítő', N'Színpadi segéddíszítő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 269, NULL, 170, NULL),
            (6278, N'Színpadmester', N'Színpadmester', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 270, NULL, 171, NULL),
            (6279, N'Szivattyútelepi duzzasztóműkezelő és fenntartó gépész', N'Szivattyútelepi duzzasztóműkezelő és fenntartó gépész', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 271, NULL, 172, NULL),
            (6280, N'Szobafestő', N'Szobafestő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 272, NULL, 173, NULL),
            (6281, N'Szoftverkódoló', N'Szoftverkódoló', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 273, NULL, 174, NULL),
            (6282, N'Szövő', N'Szövő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 274, NULL, 175, NULL),
            (6283, N'Társadalombiztosítási ügyintéző', N'Társadalombiztosítási ügyintéző', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 275, NULL, 176, NULL),
            (6284, N'Távközlési és informatikai hálózatszerelő', N'Távközlési és informatikai hálózatszerelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 276, NULL, 177, NULL),
            (6285, N'Tejtermékgyártó', N'Tejtermékgyártó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 277, NULL, 178, NULL),
            (6286, N'Térinformatikai ügyintéző', N'Térinformatikai ügyintéző', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 278, NULL, 179, NULL),
            (6287, N'Termékdíj ügyintéző', N'Termékdíj ügyintéző', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 279, NULL, 180, NULL),
            (6288, N'Textiltermék-összeállító', N'Textiltermék-összeállító', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 280, NULL, 181, NULL),
            (6289, N'Tüzelőberendezés égéstermék elvezető készítő', N'Tüzelőberendezés égéstermék elvezető készítő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 281, NULL, 182, NULL),
            (6290, N'Tűzoltó II.', N'Tűzoltó II.', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 282, NULL, 183, NULL),
            (6291, N'Üdítőital-ipari termékgyártó', N'Üdítőital-ipari termékgyártó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 285, NULL, 184, NULL),
            (6292, N'Utazásügyintéző', N'Utazásügyintéző', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 283, NULL, 185, NULL),
            (6293, N'Utómunka asszisztens', N'Utómunka asszisztens', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 284, NULL, 186, NULL),
            (6294, N'Vágó gép kezelő', N'Vágó gép kezelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 286, NULL, 187, NULL),
            (6295, N'Vágóhídi munkás', N'Vágóhídi munkás', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 287, NULL, 188, NULL),
            (6296, N'Vállalkozási ügyintéző', N'Vállalkozási ügyintéző', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 288, NULL, 189, NULL),
            (6297, N'Vámügyintéző', N'Vámügyintéző', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 289, NULL, 190, NULL),
            (6298, N'Vendéglátó eladó', N'Vendéglátó eladó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 290, NULL, 191, NULL),
            (6299, N'Világítástechnikus', N'Világítástechnikus', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 291, NULL, 192, NULL),
            (6300, N'Vincellér', N'Vincellér', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 292, NULL, 193, NULL),
            (6301, N'Virágbolti eladó', N'Virágbolti eladó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 293, NULL, 194, NULL),
            (6302, N'Virágkötő', N'Virágkötő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 294, NULL, 195, NULL),
            (6303, N'Vízkárelhárító', N'Vízkárelhárító', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 295, NULL, 196, NULL),
            (6304, N'Vízminőség vizsgáló', N'Vízminőség vizsgáló', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 296, NULL, 197, NULL),
            (6305, N'Vízműkezelő', N'Vízműkezelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 297, NULL, 198, NULL),
            (6306, N'Vízrajzi állomás üzemeltető', N'Vízrajzi állomás üzemeltető', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 298, NULL, 199, NULL),
            (6307, N'Volfrámelektródás védőgázas ívhegesztő', N'Volfrámelektródás védőgázas ívhegesztő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 299, NULL, 200, NULL),
            (6308, N'Vulkanizáló gép kezelő', N'Vulkanizáló gép kezelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 300, NULL, 201, NULL),
            (6309, N'Zöldség- és gyümölcsfeldolgozó', N'Zöldség- és gyümölcsfeldolgozó', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 301, NULL, 202, NULL),
            (6310, N'Zsaluzóács', N'Zsaluzóács', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 302, NULL, 203, NULL),
            (6311, N'Zsinórpadlás-kezelő', N'Zsinórpadlás-kezelő', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 303, NULL, 204, NULL),
            (6940, N'Gyorséttermi ételeladó (2014)', N'Gyorséttermi ételeladó (2014)', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'T', 1400, NULL, 205, NULL),
            (7722, N'Több részszakképesítés', N'Több részszakképesítés', NULL, NULL, NULL, N'ReszszakkepesitesTipus', 94, N'T', N'F', 100, NULL, 206, NULL),
            (5484, N'Na', N'Na', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 1, NULL, 1, NULL),
            (5485, N'Agrár gépész - XXXI.', N'Agrár gépész', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 131, NULL, 2, NULL),
            (5486, N'Bányászat - VII.', N'Bányászat', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 107, NULL, 3, NULL),
            (5487, N'Egészségügy - I.', N'Egészségügy', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 101, NULL, 4, NULL),
            (5488, N'Egészségügyi technika - II.', N'Egészségügyi technika', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 102, NULL, 5, NULL),
            (5489, N'Élelmiszeripar - XXXVI.', N'Élelmiszeripar', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 136, NULL, 6, NULL),
            (5490, N'Előadóművészet - XLII.', N'Előadóművészet', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 142, NULL, 7, NULL),
            (5491, N'Építőipar - XVI.', N'Építőipar', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 116, NULL, 8, NULL),
            (5492, N'Épületgépészet - VIII.', N'Épületgépészet', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 108, NULL, 9, NULL),
            (5493, N'Erdészet és vadgazdálkodás - XXXII.', N'Erdészet és vadgazdálkodás', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 132, NULL, 10, NULL),
            (5494, N'Faipar - XVIII.', N'Faipar', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 118, NULL, 11, NULL),
            (5495, N'Földmérés - XXXV.', N'Földmérés', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 135, NULL, 12, NULL),
            (5496, N'Gépészet - IX.', N'Gépészet', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 109, NULL, 13, NULL),
            (5497, N'Hang-, film és színháztechnika - VI.', N'Hang-, film és színháztechnika', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 106, NULL, 14, NULL),
            (6825, N'Honvédelem - XLIII.', N'Honvédelem', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 143, NULL, 15, NULL),
            (5498, N'Informatika - XIII.', N'Informatika', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 113, NULL, 16, NULL),
            (5499, N'Képző- és iparművészet - V.', N'Képző- és iparművészet', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 105, NULL, 17, NULL),
            (5500, N'Kereskedelem - XXVI.', N'Kereskedelem', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 126, NULL, 18, NULL),
            (5501, N'Kertészet és parképítés - XXXIV.', N'Kertészet és parképítés', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 134, NULL, 19, NULL),
            (5502, N'Kohászat - X.', N'Kohászat', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 110, NULL, 20, NULL),
            (5503, N'Könnyűipar - XVII.', N'Könnyűipar', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 117, NULL, 21, NULL),
            (5504, N'Környezetvédelem - XXIII.', N'Környezetvédelem', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 123, NULL, 22, NULL),
            (6827, N'Környezetvédelem-vízgazdálkodás - XXIII. (2014)', N'Környezetvédelem - vízgazdálkodás (2014)', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 201, NULL, 23, NULL),
            (5505, N'Közgazdaság - XXIV.', N'Közgazdaság', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 124, NULL, 24, NULL),
            (5506, N'Közlekedés - XXI.', N'Közlekedés', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 121, NULL, 25, NULL),
            (5507, N'Közlekedés, szállítmányozás és logisztika - XL.', N'Közlekedés, szállítmányozás és logisztika', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 140, NULL, 26, NULL),
            (5508, N'Közlekedésépítő - XX.', N'Közlekedésépítő', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 120, NULL, 27, NULL),
            (5509, N'Közlekedésgépész - XXII.', N'Közlekedésgépész', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 122, NULL, 28, NULL),
            (5510, N'Közművelődés - XXXIX.', N'Közművelődés', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 139, NULL, 29, NULL),
            (5511, N'Mezőgazdaság - XXXIII.', N'Mezőgazdaság', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 133, NULL, 30, NULL),
            (5512, N'Nyomdaipar - XIX.', N'Nyomdaipar', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 119, NULL, 31, NULL),
            (5513, N'Optika - XXIX.', N'Optika', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 129, NULL, 32, NULL),
            (5514, N'Pedagógia - IV.', N'Pedagógia', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 104, NULL, 33, NULL),
            (6826, N'Rendészet - XXXVIII. (2014)', N'Rendészet (2014)', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 203, NULL, 34, NULL),
            (5515, N'Rendészet és közszolgálat - XXXVIII.', N'Rendészet és közszolgálat', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 138, NULL, 35, NULL),
            (5516, N'Sport - XXXVII.', N'Sport', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 137, NULL, 36, NULL),
            (5517, N'Szépészet - XXX.', N'Szépészet', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 130, NULL, 37, NULL),
            (5518, N'Szociális - III.', N'Szociális', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 103, NULL, 38, NULL),
            (5519, N'Távközlés - XII.', N'Távközlés', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 112, NULL, 39, NULL),
            (5520, N'Turisztika - XXVIII.', N'Turisztika', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 128, NULL, 40, NULL),
            (5521, N'Ügyvitel - XXV.', N'Ügyvitel', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 125, NULL, 41, NULL),
            (5522, N'Vegyész - XV.', N'Vegyész', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 115, NULL, 42, NULL),
            (5523, N'Vegyipar - XIV.', N'Vegyipar', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 114, NULL, 43, NULL),
            (5524, N'Vendéglátóipar - XXVII.', N'Vendéglátóipar', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 127, NULL, 44, NULL),
            (5525, N'Villamosipar és elektronika - XI.', N'Villamosipar és elektronika', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 111, NULL, 45, NULL),
            (5526, N'Vízügy - XLI.', N'Vízügy', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 141, NULL, 46, NULL),
            (7719, N'-', N'-', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'T', 200, NULL, 47, NULL),
            (7720, N'Több ágazat', N'Több ágazat', NULL, NULL, NULL, N'AgazatTipus', 91, N'T', N'F', 100, NULL, 48, NULL),
            (6084, N'Na', N'Na', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 1, NULL, 1, NULL),
            (6085, N'Egészségügy - 1.', N'Egészségügy', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 10, NULL, 2, NULL),
            (6103, N'Szociális szolgáltatások - 2.', N'Szociális szolgáltatások', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 20, NULL, 3, NULL),
            (6101, N'Oktatás - 3.', N'Oktatás', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 30, NULL, 4, NULL),
            (6099, N'Művészet, közművelődés, kommunikáció - 4.', N'Művészet, közművelődés, kommunikáció', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 40, NULL, 5, NULL),
            (6091, N'Gépészet - 5.', N'Gépészet', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 50, NULL, 6, NULL),
            (6087, N'Elektrotechnika-elektronika - 6.', N'Elektrotechnika-elektronika', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 60, NULL, 7, NULL),
            (6092, N'Informatika - 7.', N'Informatika', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 70, NULL, 8, NULL),
            (6105, N'Vegyipar - 8.', N'Vegyipar', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 80, NULL, 9, NULL),
            (6089, N'Építészet - 9.', N'Építészet', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 90, NULL, 10, NULL),
            (6094, N'Könnyűipar - 10.', N'Könnyűipar', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 100, NULL, 11, NULL),
            (6090, N'Faipar - 11.', N'Faipar', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 110, NULL, 12, NULL),
            (6100, N'Nyomdaipar - 12.', N'Nyomdaipar', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 120, NULL, 13, NULL),
            (6097, N'Közlekedés - 13.', N'Közlekedés', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 130, NULL, 14, NULL),
            (6095, N'Környezetvédelem - 14.', N'Környezetvédelem', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 140, NULL, 15, NULL),
            (6831, N'Környezetvédelem-vízgazdálkodás - 14. (2014)', N'Környezetvédelem - vízgazdálkodás (2014)', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 300, NULL, 16, NULL),
            (6096, N'Közgazdaság - 15.', N'Közgazdaság', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 150, NULL, 17, NULL),
            (6104, N'Ügyvitel - 16.', N'Ügyvitel', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 160, NULL, 18, NULL),
            (6093, N'Kereskedelem-marketing, üzleti adminisztráció - 17.', N'Kereskedelem-marketing, üzleti adminisztráció', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 170, NULL, 19, NULL),
            (6106, N'Vendéglátás-turisztika - 18.', N'Vendéglátás-turisztika', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 180, NULL, 20, NULL),
            (6086, N'Egyéb szolgáltatások - 19.', N'Egyéb szolgáltatások', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 190, NULL, 21, NULL),
            (6098, N'Mezőgazdaság - 20.', N'Mezőgazdaság', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 200, NULL, 22, NULL),
            (6088, N'Élelmiszeripar - 21.', N'Élelmiszeripar', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 210, NULL, 23, NULL),
            (6102, N'Rendészet, honvédelem és közszolgálat - 22.', N'Rendészet, honvédelem és közszolgálat', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 220, NULL, 24, NULL),
            (6107, N'Vízügy - 23.', N'Vízügy', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'T', 230, NULL, 25, NULL),
            (7735, N'Több szakmacsoport', N'Több szakmacsoport', NULL, NULL, NULL, N'SzakmacsoportTipus', 93, N'T', N'F', 5, NULL, 26, NULL),
            (5527, N'Na', N'Na', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1, NULL, 1, NULL),
            (6810, N'Nyomdaipari gépmester - 5421304 (2014)', N'Nyomdaipari gépmester - 5421304 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14140, NULL, 2, NULL),
            (6811, N'Ügyviteli titkár - 5434602 (2014)', N'Ügyviteli titkár - 5434602 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14160, NULL, 3, NULL),
            (5528, N'Abroncsgyártó - 3454301', N'Abroncsgyártó - 3454301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 50, NULL, 4, NULL),
            (5529, N'Ács - 3458201', N'Ács - 3458201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 55, NULL, 5, NULL),
            (5530, N'Adótanácsadó - 5534401', N'Adótanácsadó - 5534401', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 60, NULL, 6, NULL),
            (5531, N'Agrár áruforgalmazó szaktechnikus - 5562101', N'Agrár áruforgalmazó szaktechnikus - 5562101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 65, NULL, 7, NULL),
            (5532, N'Alapellátási közösségi szakápoló - 5572316', N'Alapellátási közösségi szakápoló - 5572316', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 70, NULL, 8, NULL),
            (5533, N'Államháztartási mérlegképes könyvelő - 5534402', N'Államháztartási mérlegképes könyvelő - 5534402', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 75, NULL, 9, NULL),
            (5534, N'Államháztartási ügyintéző - 5434404', N'Államháztartási ügyintéző - 5434404', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 80, NULL, 10, NULL),
            (5535, N'Állattartó szakmunkás - 3462103', N'Állattartó szakmunkás - 3462103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 85, NULL, 11, NULL),
            (5536, N'Állattenyésztő és állategészségügyi technikus - 5462101', N'Állattenyésztő és állategészségügyi technikus - 5462101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 90, NULL, 12, NULL),
            (5537, N'Általános ápolási és egészségügyi asszisztens - 5272001', N'Általános ápolási és egészségügyi asszisztens - 5272001', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 100, NULL, 13, NULL),
            (5538, N'Alternatív gépjárműhajtási technikus - 5552503', N'Alternatív gépjárműhajtási technikus - 5552503', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 110, NULL, 14, NULL),
            (5539, N'Aneszteziológiai szakasszisztens - 5572501', N'Aneszteziológiai szakasszisztens - 5572501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 120, NULL, 15, NULL),
            (6939, N'Ápolási asszisztens - 3272301 (2014)', N'Ápolási asszisztens - 3272301 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14000, NULL, 16, NULL),
            (5540, N'Ápoló - 5572301', N'Ápoló - 5572301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 130, NULL, 17, NULL),
            (5541, N'Aranyműves - 5521101', N'Aranyműves - 5521101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 140, NULL, 18, NULL),
            (6832, N'Vendéglátásszervező-vendéglős - 5481101 (2014)', N'Vendéglátásszervező-vendéglős - 5481101 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14190, NULL, 19, NULL),
            (5542, N'Artista I. (a szakmairány megnevezésével) - 5521202', N'Artista I. (a szakmairány megnevezésével) - 5521202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 150, NULL, 20, NULL),
            (5543, N'Artista II. (a szakmairány megnevezésével) - 5421201', N'Artista II. (a szakmairány megnevezésével) - 5421201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 160, NULL, 21, NULL),
            (5544, N'Asztalos - 3454302', N'Asztalos - 3454302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 170, NULL, 22, NULL),
            (5545, N'Audio- és vizuáltechnikai műszerész - 3552201', N'Audio- és vizuáltechnikai műszerész - 3552201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 180, NULL, 23, NULL),
            (5546, N'Audiológiai asszisztens és hallásakusztikus - 5472502', N'Audiológiai asszisztens és hallásakusztikus - 5472502', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 190, NULL, 24, NULL),
            (5547, N'Autóbuszvezető - 3584101', N'Autóbuszvezető - 3584101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 200, NULL, 25, NULL),
            (5548, N'Autóelektronikai műszerész - 5452501', N'Autóelektronikai műszerész - 5452501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 210, NULL, 26, NULL),
            (5549, N'Autógyártó - 3452101', N'Autógyártó - 3452101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 220, NULL, 27, NULL),
            (5550, N'Automatikai berendezés karbantartó - 3552301', N'Automatikai berendezés karbantartó - 3552301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 230, NULL, 28, NULL),
            (5551, N'Automatikai technikus - 5452301', N'Automatikai technikus - 5452301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 240, NULL, 29, NULL),
            (5552, N'Autószerelő - 5452502', N'Autószerelő - 5452502', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 250, NULL, 30, NULL),
            (5553, N'Autótechnikus - 5552501', N'Autótechnikus - 5552501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 260, NULL, 31, NULL),
            (5554, N'Avionikus - 5452503', N'Avionikus - 5452503', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 270, NULL, 32, NULL),
            (5555, N'Bádogos - 3458202', N'Bádogos - 3458202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 280, NULL, 33, NULL),
            (5556, N'Baleseti helyszínelő - 5586101', N'Baleseti helyszínelő - 5586101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 290, NULL, 34, NULL),
            (5557, N'Bányaművelő - 3454402', N'Bányaművelő - 3454402', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 300, NULL, 35, NULL),
            (5558, N'Bányaművelő technikus - 5454401', N'Bányaművelő technikus - 5454401', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 310, NULL, 36, NULL),
            (5559, N'Becsüs - 5234101', N'Becsüs - 5234101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 320, NULL, 37, NULL),
            (5560, N'Belovagló - 3581301', N'Belovagló - 3581301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 330, NULL, 38, NULL),
            (5561, N'Beszédátviteli rendszerüzemeltető technikus - 5552305', N'Beszédátviteli rendszerüzemeltető technikus - 5552305', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 340, NULL, 39, NULL),
            (5562, N'Betonszerkezet-készítő, betonelem gyártó - 3258201', N'Betonszerkezet-készítő, betonelem gyártó - 3258201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 350, NULL, 40, NULL),
            (5563, N'Biogazdálkodó - 3562102', N'Biogazdálkodó - 3562102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 360, NULL, 41, NULL),
            (5564, N'Biztonságszervező - 5286101', N'Biztonságszervező - 5286101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 370, NULL, 42, NULL),
            (5565, N'Biztosítási tanácsadó - 5534301', N'Biztosítási tanácsadó - 5534301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 380, NULL, 43, NULL),
            (5566, N'Boltvezető - 3534101', N'Boltvezető - 3534101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 390, NULL, 44, NULL),
            (5567, N'Boncmester - 5272501', N'Boncmester - 5272501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 400, NULL, 45, NULL),
            (5568, N'Bor- és pezsgőgyártó szaktechnikus - 5554101', N'Bor- és pezsgőgyártó szaktechnikus - 5554101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 410, NULL, 46, NULL),
            (5569, N'Bőrdíszműves - 3454201', N'Bőrdíszműves - 3454201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 420, NULL, 47, NULL),
            (5570, N'Bőrfeldolgozó-ipari technikus - 5454201', N'Bőrfeldolgozó-ipari technikus - 5454201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 430, NULL, 48, NULL),
            (5571, N'Bronzműves és szoboröntő - 5521102', N'Bronzműves és szoboröntő - 5521102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 440, NULL, 49, NULL),
            (5572, N'Büntetés-végrehajtási egészségügyi főfelügyelő - 5386101', N'Büntetés-végrehajtási egészségügyi főfelügyelő - 5386101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 470, NULL, 50, NULL),
            (5573, N'Büntetés-végrehajtási felügyelő - 5286102', N'Büntetés-végrehajtási felügyelő - 5286102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 480, NULL, 51, NULL),
            (5574, N'Büntetés-végrehajtási főfelügyelő - 5386102', N'Büntetés-végrehajtási főfelügyelő - 5386102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 490, NULL, 52, NULL),
            (5575, N'Büntetés-végrehajtási gazdálkodási főfelügyelő - 5386103', N'Büntetés-végrehajtási gazdálkodási főfelügyelő - 5386103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 500, NULL, 53, NULL),
            (5576, N'Büntetés-végrehajtási igazgatási főfelügyelő - 5386104', N'Büntetés-végrehajtási igazgatási főfelügyelő - 5386104', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 510, NULL, 54, NULL),
            (5577, N'Bűnügyi technikus - 5586102', N'Bűnügyi technikus - 5586102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 520, NULL, 55, NULL),
            (5578, N'Burkoló - 3458213', N'Burkoló - 3458213', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 450, NULL, 56, NULL),
            (5579, N'Bútoripari gyártás-előkészítő, szervező - 5254301', N'Bútoripari gyártás-előkészítő, szervező - 5254301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 460, NULL, 57, NULL),
            (5580, N'CAD-CAM informatikus - 5448101', N'CAD-CAM informatikus - 5448101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 530, NULL, 58, NULL),
            (5581, N'Cipőkészítő - 3454202', N'Cipőkészítő - 3454202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 540, NULL, 59, NULL),
            (5582, N'Citológiai szakasszisztens - 5572503', N'Citológiai szakasszisztens - 5572503', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 550, NULL, 60, NULL),
            (5583, N'CNC gépkezelő - 3552101', N'CNC gépkezelő - 3552101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 560, NULL, 61, NULL),
            (5584, N'Családi gazdálkodó - 3481401', N'Családi gazdálkodó - 3481401', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 610, NULL, 62, NULL),
            (5585, N'Családsegítő asszisztens - 5276101', N'Családsegítő asszisztens - 5276101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 620, NULL, 63, NULL),
            (5586, N'Csecsemő és gyermekápoló - 5572302', N'Csecsemő és gyermekápoló - 5572302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 630, NULL, 64, NULL),
            (5587, N'CT/MR szakasszisztens - 5572516', N'CT/MR szakasszisztens - 5572516', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 570, NULL, 65, NULL),
            (5588, N'Cukor- és édesipari szaktechnikus - 5554102', N'Cukor- és édesipari szaktechnikus - 5554102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 580, NULL, 66, NULL),
            (5589, N'Cukrász - 3481101', N'Cukrász - 3481101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 590, NULL, 67, NULL),
            (5590, N'Cukrászati részlegvezető - 3581104', N'Cukrászati részlegvezető - 3581104', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 600, NULL, 68, NULL),
            (5591, N'Dekoratőr - 5421101', N'Dekoratőr - 5421101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 640, NULL, 69, NULL),
            (5592, N'Demencia gondozó - 3576201', N'Demencia gondozó - 3576201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 650, NULL, 70, NULL),
            (5593, N'Diabetológiai szakápoló és edukátor - 5572303', N'Diabetológiai szakápoló és edukátor - 5572303', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 660, NULL, 71, NULL),
            (5594, N'Diétás cukrász - 3581105', N'Diétás cukrász - 3581105', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 670, NULL, 72, NULL),
            (5595, N'Diétás szakács - 3581103', N'Diétás szakács - 3581103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 680, NULL, 73, NULL),
            (5596, N'Digitális műszaki rajzoló - 5248101', N'Digitális műszaki rajzoló - 5248101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 690, NULL, 74, NULL),
            (5597, N'Díszítő festő - 5521103', N'Díszítő festő - 5521103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 700, NULL, 75, NULL),
            (5598, N'Díszlettervező - 5521104', N'Díszlettervező - 5521104', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 710, NULL, 76, NULL),
            (5599, N'Díszműbádogos - 3358201', N'Díszműbádogos - 3358201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 720, NULL, 77, NULL),
            (5600, N'Díszműkovács - 3421101', N'Díszműkovács - 3421101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 730, NULL, 78, NULL),
            (5601, N'Dísznövénykertész - 3462201', N'Dísznövénykertész - 3462201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 740, NULL, 79, NULL),
            (5602, N'Divat- és stílustervező - 5421102', N'Divat- és stílustervező - 5421102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 750, NULL, 80, NULL),
            (5603, N'Drog- és toxikológiai technikus - 5552401', N'Drog- és toxikológiai technikus - 5552401', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 760, NULL, 81, NULL),
            (5604, N'Édesipari termékgyártó - 3454101', N'Édesipari termékgyártó - 3454101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 770, NULL, 82, NULL),
            (5605, N'Egészségfejlesztési segítő - 3272001', N'Egészségfejlesztési segítő - 3272001', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 780, NULL, 83, NULL),
            (5606, N'Egészségügyi gázmester - 5385301', N'Egészségügyi gázmester - 5385301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 790, NULL, 84, NULL),
            (5607, N'Egészségügyi gyakorlatvezető - 5572001', N'Egészségügyi gyakorlatvezető - 5572001', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 800, NULL, 85, NULL),
            (5608, N'Egészségügyi kártevőírtó és fertőtlenítő - 3285301', N'Egészségügyi kártevőírtó és fertőtlenítő - 3285301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 810, NULL, 86, NULL),
            (5609, N'Egészségügyi technikai asszisztens - 5272002', N'Egészségügyi technikai asszisztens - 5272002', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 820, NULL, 87, NULL),
            (5610, N'Egyéb szervezeti mérlegképes könyvelő - 5534403', N'Egyéb szervezeti mérlegképes könyvelő - 5534403', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 830, NULL, 88, NULL),
            (5611, N'Egyházzenész I. (a szakmairány megjelölésével) - 5521203', N'Egyházzenész I. (a szakmairány megjelölésével) - 5521203', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 840, NULL, 89, NULL),
            (5612, N'Egyházzenész II. (a szakmairány megjelölésével) - 5421202', N'Egyházzenész II. (a szakmairány megjelölésével) - 5421202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 850, NULL, 90, NULL),
            (5613, N'Eladó - 3434101', N'Eladó - 3434101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 860, NULL, 91, NULL),
            (5614, N'Elektromechanikai műszerész - 3452201', N'Elektromechanikai műszerész - 3452201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 870, NULL, 92, NULL),
            (5615, N'Elektromos gép- és készülékszerelő - 3452202', N'Elektromos gép- és készülékszerelő - 3452202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 880, NULL, 93, NULL),
            (5616, N'Elektronikai műszerész - 3452203', N'Elektronikai műszerész - 3452203', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 890, NULL, 94, NULL),
            (5617, N'Elektronikai technikus - 5452302', N'Elektronikai technikus - 5452302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 900, NULL, 95, NULL),
            (5618, N'Elektronikus hozzáférési és magánhálózati rendszerüzemeltető technikus - 5552301', N'Elektronikus hozzáférési és magánhálózati rendszerüzemeltető technikus - 5552301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 910, NULL, 96, NULL),
            (5619, N'Elektronikus műsorközlő és tartalomátviteli rendszerüzemeltető technikus - 5552302', N'Elektronikus műsorközlő és tartalomátviteli rendszerüzemeltető technikus - 5552302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 920, NULL, 97, NULL),
            (5620, N'Elektronikus vagyonvédelmi rendszerszerelő - 3586101', N'Elektronikus vagyonvédelmi rendszerszerelő - 3586101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 930, NULL, 98, NULL),
            (5621, N'Élelmezésvezető - 5281101', N'Élelmezésvezető - 5281101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 940, NULL, 99, NULL),
            (5622, N'Élelmiszeripari analitikus technikus - 5454101', N'Élelmiszeripari analitikus technikus - 5454101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 950, NULL, 100, NULL),
            (5623, N'Élelmiszeripari gépésztechnikus - 5452101', N'Élelmiszeripari gépésztechnikus - 5452101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 960, NULL, 101, NULL),
            (5624, N'Élelmiszeripari technikus - 5454102', N'Élelmiszeripari technikus - 5454102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 970, NULL, 102, NULL),
            (5625, N'Ellenőrzési szakelőadó - 5534404', N'Ellenőrzési szakelőadó - 5534404', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 980, NULL, 103, NULL),
            (5626, N'Emelőgép-ügyintéző - 5258201', N'Emelőgép-ügyintéző - 5258201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 990, NULL, 104, NULL),
            (5627, N'Endoszkópos szakasszisztens - 5572504', N'Endoszkópos szakasszisztens - 5572504', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1000, NULL, 105, NULL),
            (5628, N'Epidemiológiai szakápoló - 5572304', N'Epidemiológiai szakápoló - 5572304', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1010, NULL, 106, NULL),
            (5629, N'Építő- és anyagmozgató gép kezelője (a szakmairány megjelölésével) - 3258202', N'Építő- és anyagmozgató gép kezelője (a szakmairány megjelölésével) - 3258202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1020, NULL, 107, NULL),
            (5630, N'Építő- szállító- és munkagép-szerelő - 3452111', N'Építő- szállító- és munkagép-szerelő - 3452111', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1030, NULL, 108, NULL),
            (5631, N'Építőanyagipari minőségellenőr - 5558204', N'Építőanyagipari minőségellenőr - 5558204', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1040, NULL, 109, NULL),
            (5632, N'Épület- és szerkezetlakatos - 3458203', N'Épület- és szerkezetlakatos - 3458203', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1050, NULL, 110, NULL),
            (5633, N'Épületgépész technikus - 5458201', N'Épületgépész technikus - 5458201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1060, NULL, 111, NULL),
            (5634, N'Épületgépészeti előkészítő - 5258202', N'Épületgépészeti előkészítő - 5258202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1070, NULL, 112, NULL),
            (5635, N'Erdészeti gépésztechnikus - 5452102', N'Erdészeti gépésztechnikus - 5452102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1090, NULL, 113, NULL),
            (5636, N'Erdészeti szakmunkás - 3462301', N'Erdészeti szakmunkás - 3462301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1100, NULL, 114, NULL),
            (5637, N'Erdésztechnikus - 5462302', N'Erdésztechnikus - 5462302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1110, NULL, 115, NULL),
            (5638, N'Ergoterapeuta - 5472603', N'Ergoterapeuta - 5472603', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1120, NULL, 116, NULL),
            (5639, N'Érintésvédelmi szabványossági felülvizsgáló - 3552203', N'Érintésvédelmi szabványossági felülvizsgáló - 3552203', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1130, NULL, 117, NULL),
            (5640, N'Erjedés- és üdítőitalipari szaktechnikus - 5554103', N'Erjedés- és üdítőitalipari szaktechnikus - 5554103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1140, NULL, 118, NULL),
            (5641, N'Erjedés- és üdítőital-ipari termékgyártó - 3454107', N'Erjedés- és üdítőital-ipari termékgyártó - 3454107', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1150, NULL, 119, NULL),
            (5642, N'Erőművi blokkgépész - 5352201', N'Erőművi blokkgépész - 5352201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1160, NULL, 120, NULL),
            (5643, N'Erőművi gépész - 5252201', N'Erőművi gépész - 5252201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1170, NULL, 121, NULL),
            (5644, N'Erősáramú berendezések felülvizsgálója - 3552202', N'Erősáramú berendezések felülvizsgálója - 3552202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1180, NULL, 122, NULL),
            (5645, N'Erősáramú elektrotechnikus - 5452201', N'Erősáramú elektrotechnikus - 5452201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1190, NULL, 123, NULL),
            (5646, N'Ezüstműves - 5521107', N'Ezüstműves - 5521107', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1200, NULL, 124, NULL),
            (7104, N'Élelmiszeripari szakmunkás - 3454102 (2014)', N'Élelmiszeripari szakmunkás - 3454102 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14010, NULL, 125, NULL),
            (5647, N'Faipari technikus - 5454301', N'Faipari technikus - 5454301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1210, NULL, 126, NULL),
            (5648, N'Fegyveres biztonsági őr - 5386105', N'Fegyveres biztonsági őr - 5386105', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1220, NULL, 127, NULL),
            (5649, N'Fegyverműszerész - 5486301', N'Fegyverműszerész - 5486301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1230, NULL, 128, NULL),
            (5650, N'Fehérnemű-készítő és kötöttáru összeállító - 3454203', N'Fehérnemű-készítő és kötöttáru összeállító - 3454203', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1240, NULL, 129, NULL),
            (5651, N'Felnőtt intenzív szakápoló - 5572305', N'Felnőtt intenzív szakápoló - 5572305', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1250, NULL, 130, NULL),
            (5652, N'Felvonóellenőr - 6252101', N'Felvonóellenőr - 6252101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1260, NULL, 131, NULL),
            (5653, N'Felvonószerelő - 3252101', N'Felvonószerelő - 3252101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1270, NULL, 132, NULL),
            (5654, N'Férfi fodrász-borbély - 5281503', N'Férfi fodrász-borbély - 5281503', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1280, NULL, 133, NULL),
            (5655, N'Férfiszabó - 3454204', N'Férfiszabó - 3454204', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1290, NULL, 134, NULL),
            (5656, N'Festő - 5421103', N'Festő - 5421103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1300, NULL, 135, NULL),
            (5657, N'Festő, mázoló, tapétázó - 3458204', N'Festő, mázoló, tapétázó - 3458204', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1310, NULL, 136, NULL),
            (5658, N'Finommechanikai műszerész - 3452102', N'Finommechanikai műszerész - 3452102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1320, NULL, 137, NULL),
            (5659, N'Fitness-wellness instruktor - 5481301', N'Fitness-wellness instruktor - 5481301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1330, NULL, 138, NULL),
            (5660, N'Fizioterápiás asszisztens - 5472503', N'Fizioterápiás asszisztens - 5472503', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1340, NULL, 139, NULL),
            (5661, N'Fluidumkitermelő technikus - 5454402', N'Fluidumkitermelő technikus - 5454402', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1350, NULL, 140, NULL),
            (5662, N'Fő határrendész - 5586103', N'Fő határrendész - 5586103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1460, NULL, 141, NULL),
            (5663, N'Fodrász - 5481501', N'Fodrász - 5481501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1360, NULL, 142, NULL),
            (5664, N'Fogadós - 3481106', N'Fogadós - 3481106', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1370, NULL, 143, NULL),
            (5665, N'Fogászati asszisztens - 5472002', N'Fogászati asszisztens - 5472002', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1380, NULL, 144, NULL),
            (5666, N'Foglalkozásegészségügyi szakápoló - 5572306', N'Foglalkozásegészségügyi szakápoló - 5572306', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1390, NULL, 145, NULL),
            (5667, N'Foglalkozás-szervező - 5576201', N'Foglalkozás-szervező - 5576201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1400, NULL, 146, NULL),
            (5668, N'Fogtechnikus - 5572401', N'Fogtechnikus - 5572401', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1410, NULL, 147, NULL),
            (5669, N'Fogtechnikus gyakornok - 5472401', N'Fogtechnikus gyakornok - 5472401', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1420, NULL, 148, NULL),
            (5670, N'Fogyatékkal élők gondozója - 5214001 (2017)', N'Fogyatékkal élők gondozója - 5214001 (2017)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 17000, NULL, 149, NULL),
            (5671, N'Földmérő, földügyi és térinformatikai technikus - 5458101', N'Földmérő, földügyi és térinformatikai technikus - 5458101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1470, NULL, 150, NULL),
            (5672, N'Földügyi térinformatikai szaktechnikus - 5558101', N'Földügyi térinformatikai szaktechnikus - 5558101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1480, NULL, 151, NULL),
            (5673, N'Formacikk-gyártó - 3454303', N'Formacikk-gyártó - 3454303', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1440, NULL, 152, NULL),
            (5674, N'Fotográfus és fotótermék-kereskedő - 5481001', N'Fotográfus és fotótermék-kereskedő - 5481001', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1450, NULL, 153, NULL),
            (5675, N'Fővilágosító - 5321302', N'Fővilágosító - 5321302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1490, NULL, 154, NULL),
            (5676, N'Fülilleszték készítő - 5572402', N'Fülilleszték készítő - 5572402', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1500, NULL, 155, NULL),
            (5677, N'Gáz- és hőtermelő berendezés-szerelő - 3558201', N'Gáz- és hőtermelő berendezés-szerelő - 3558201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1510, NULL, 156, NULL),
            (5678, N'Gázautószerelő - 5552502', N'Gázautószerelő - 5552502', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1520, NULL, 157, NULL),
            (5679, N'Gazda - 3462101', N'Gazda - 3462101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1530, NULL, 158, NULL),
            (5680, N'Gazdasági informatikus - 5448102', N'Gazdasági informatikus - 5448102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1540, NULL, 159, NULL),
            (5681, N'Gázipari műszaki-biztonsági felülvizsgáló - 3558202', N'Gázipari műszaki-biztonsági felülvizsgáló - 3558202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1550, NULL, 160, NULL),
            (5682, N'Gázipari technikus - 5454403', N'Gázipari technikus - 5454403', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1560, NULL, 161, NULL),
            (5683, N'Gázturbina gépész - 5252202', N'Gázturbina gépész - 5252202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1570, NULL, 162, NULL),
            (5684, N'Gépgyártástechnológiai technikus - 5452103', N'Gépgyártástechnológiai technikus - 5452103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1580, NULL, 163, NULL),
            (5685, N'Gépi forgácsoló - 3452103', N'Gépi forgácsoló - 3452103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1590, NULL, 164, NULL),
            (5686, N'Gépjármű mechatronikus - 3452502', N'Gépjármű mechatronikus - 3452502', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1600, NULL, 165, NULL),
            (5687, N'Gépjármű-építési, szerelési logisztikus - 3484101', N'Gépjármű-építési, szerelési logisztikus - 3484101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1610, NULL, 166, NULL),
            (5688, N'Gépjárműépítő, szerelő - 3452501', N'Gépjárműépítő, szerelő - 3452501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1620, NULL, 167, NULL),
            (5689, N'Geriátriai és krónikus beteg szakápoló - 5572307', N'Geriátriai és krónikus beteg szakápoló - 5572307', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1630, NULL, 168, NULL),
            (5690, N'Gerinchálózati rendszerüzemeltető technikus - 5552303', N'Gerinchálózati rendszerüzemeltető technikus - 5552303', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1640, NULL, 169, NULL),
            (5691, N'Gerontológiai gondozó - 5576202', N'Gerontológiai gondozó - 5576202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1650, NULL, 170, NULL),
            (5692, N'Grafikus - 5421104', N'Grafikus - 5421104', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1660, NULL, 171, NULL),
            (5693, N'Gumiabroncs-javító és kerékkiegyensúlyozó - 3252501', N'Gumiabroncs-javító és kerékkiegyensúlyozó - 3252501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1670, NULL, 172, NULL),
            (5694, N'Gumiipari technikus - 5454302', N'Gumiipari technikus - 5454302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1680, NULL, 173, NULL),
            (5695, N'Gyakorlatos színész (a szakmairány megjelölésével) - 5421203', N'Gyakorlatos színész (a szakmairány megjelölésével) - 5421203', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1690, NULL, 174, NULL),
            (5696, N'Gyakorló ápoló - 5472302', N'Gyakorló ápoló - 5472302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1700, NULL, 175, NULL),
            (5697, N'Gyakorló csecsemő- és gyermekápoló - 5472303', N'Gyakorló csecsemő- és gyermekápoló - 5472303', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1710, NULL, 176, NULL),
            (5698, N'Gyakorló hangszerkészítő és -javító (a hangszercsoport megjelölésével) - 5421501', N'Gyakorló hangszerkészítő és -javító (a hangszercsoport megjelölésével) - 5421501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1720, NULL, 177, NULL),
            (5699, N'Gyakorló klinikai laboratóriumi asszisztens - 5472504', N'Gyakorló klinikai laboratóriumi asszisztens - 5472504', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1730, NULL, 178, NULL),
            (5700, N'Gyakorló mentőápoló - 5472301', N'Gyakorló mentőápoló - 5472301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1740, NULL, 179, NULL),
            (5701, N'Gyakorló szövettani asszisztens - 5472505', N'Gyakorló szövettani asszisztens - 5472505', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1750, NULL, 180, NULL),
            (5702, N'Gyártásvezető - 5321303', N'Gyártásvezető - 5321303', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1760, NULL, 181, NULL),
            (5703, N'Gyártósori gépbeállító - 3452105', N'Gyártósori gépbeállító - 3452105', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1770, NULL, 182, NULL),
            (5704, N'Gyermek intenzív szakápoló - 5572308', N'Gyermek intenzív szakápoló - 5572308', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1780, NULL, 183, NULL),
            (5705, N'Gyermekotthoni asszisztens - 5476101', N'Gyermekotthoni asszisztens - 5476101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1790, NULL, 184, NULL),
            (5706, N'Gyógy- és fűszernövénytermesztő - 3562201', N'Gyógy- és fűszernövénytermesztő - 3562201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1800, NULL, 185, NULL),
            (5707, N'Gyógyászati segédeszköz-forgalmazó - 5272601', N'Gyógyászati segédeszköz-forgalmazó - 5272601', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1810, NULL, 186, NULL),
            (5708, N'Gyógymasszőr - 5472604', N'Gyógymasszőr - 5472604', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1820, NULL, 187, NULL),
            (5709, N'Gyógynövény eladó - 3534102', N'Gyógynövény eladó - 3534102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1830, NULL, 188, NULL),
            (5710, N'Gyógypedagógiai segítő munkatárs - 5414001', N'Gyógypedagógiai segítő munkatárs - 5414001', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1840, NULL, 189, NULL),
            (5711, N'Gyógyszeripari szaktechnikus - 5552405', N'Gyógyszeripari szaktechnikus - 5552405', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1850, NULL, 190, NULL),
            (5712, N'Gyógyszerkészítmény-gyártó - 3452401', N'Gyógyszerkészítmény-gyártó - 3452401', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1860, NULL, 191, NULL),
            (5713, N'Gyógyszerkiadó szakasszisztens - 5572002', N'Gyógyszerkiadó szakasszisztens - 5572002', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1870, NULL, 192, NULL),
            (5714, N'Gyógyszertári asszisztens - 5472003', N'Gyógyszertári asszisztens - 5472003', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1880, NULL, 193, NULL),
            (5715, N'Habilitációs kutya kiképzője - 5281001', N'Habilitációs kutya kiképzője - 5281001', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1890, NULL, 194, NULL),
            (5716, N'Hajós szakmunkás - 3484103', N'Hajós szakmunkás - 3484103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1900, NULL, 195, NULL),
            (5717, N'Hajózási technikus - 5484101', N'Hajózási technikus - 5484101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1910, NULL, 196, NULL),
            (5718, N'Hajózási üzemeltetési vezető - 5284101', N'Hajózási üzemeltetési vezető - 5284101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1920, NULL, 197, NULL),
            (5719, N'Halász, haltenyésztő - 3462401', N'Halász, haltenyésztő - 3462401', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1930, NULL, 198, NULL),
            (5720, N'Hanglemez-bemutató - 5234501', N'Hanglemez-bemutató - 5234501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1940, NULL, 199, NULL),
            (5721, N'Hangmester - 5421301', N'Hangmester - 5421301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1950, NULL, 200, NULL),
            (5722, N'Hangszerkészítő és -javító (a hangszercsoport megjelölésével) - 5521501', N'Hangszerkészítő és -javító (a hangszercsoport megjelölésével) - 5521501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1960, NULL, 201, NULL),
            (5723, N'Hangtárvezető - 5521301', N'Hangtárvezető - 5521301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1970, NULL, 202, NULL),
            (5724, N'Háztartási gépszervíz szaktechnikus - 5552306', N'Háztartási gépszervíz szaktechnikus - 5552306', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1980, NULL, 203, NULL),
            (5725, N'Hegesztő - 3452106', N'Hegesztő - 3452106', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1990, NULL, 204, NULL),
            (5726, N'Hematológiai és transzfuziológiai szakasszisztens - 5572506', N'Hematológiai és transzfuziológiai szakasszisztens - 5572506', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2000, NULL, 205, NULL),
            (5727, N'Hídépítő és -fenntartó technikus - 5458202', N'Hídépítő és -fenntartó technikus - 5458202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2010, NULL, 206, NULL),
            (5728, N'Hobbiállat-tenyésztő és -forgalmazó - 5262101', N'Hobbiállat-tenyésztő és -forgalmazó - 5262101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2020, NULL, 207, NULL),
            (5729, N'Homlokzatépítő és -szerelő - 3258203', N'Homlokzatépítő és -szerelő - 3258203', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2030, NULL, 208, NULL),
            (5730, N'Honvéd altiszt (az ágazat/szakmairány megjelölésével) - 5486302', N'Honvéd altiszt (az ágazat/szakmairány megjelölésével) - 5486302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2040, NULL, 209, NULL),
            (5731, N'Honvéd zászlós (az ágazat/szakmairány megjelölésével) - 5586301', N'Honvéd zászlós (az ágazat/szakmairány megjelölésével) - 5586301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2050, NULL, 210, NULL),
            (5732, N'Hospice szakápoló - 5572309', N'Hospice szakápoló - 5572309', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2070, NULL, 211, NULL),
            (5733, N'Hulladékgazdálkodó szaktechnikus - 5585001', N'Hulladékgazdálkodó szaktechnikus - 5585001', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2080, NULL, 212, NULL),
            (5734, N'Hulladéktelepkezelő - 3285102', N'Hulladéktelepkezelő - 3285102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2090, NULL, 213, NULL),
            (5735, N'Hús- és baromfiipari szaktechnikus - 5554104', N'Hús- és baromfiipari szaktechnikus - 5554104', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2100, NULL, 214, NULL),
            (5736, N'Húsipari termékgyártó - 3454103', N'Húsipari termékgyártó - 3454103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2110, NULL, 215, NULL),
            (5737, N'Hűtéstechnikai berendezés üzemeltető - 3252201', N'Hűtéstechnikai berendezés üzemeltető - 3252201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2120, NULL, 216, NULL),
            (5738, N'Hűtő- és légtechnikai rendszerszerelő - 3458205', N'Hűtő- és légtechnikai rendszerszerelő - 3458205', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2130, NULL, 217, NULL),
            (5739, N'Hűtő-, klíma- és hőszivattyú berendezés-szerelő - 3558203', N'Hűtő-, klíma- és hőszivattyú berendezés-szerelő - 3558203', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2140, NULL, 218, NULL),
            (5740, N'Idegennyelvű ipari és kereskedelmi ügyintéző - 5434701', N'Idegennyelvű ipari és kereskedelmi ügyintéző - 5434701', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2160, NULL, 219, NULL),
            (5741, N'Idegennyelvű ügyfélkapcsolati szakügyintéző - 5534701', N'Idegennyelvű ügyfélkapcsolati szakügyintéző - 5534701', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2170, NULL, 220, NULL),
            (5742, N'Idegenvezető - 5481201', N'Idegenvezető - 5481201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2150, NULL, 221, NULL),
            (5743, N'IFRS mérlegképes könyvelő - 5534405', N'IFRS mérlegképes könyvelő - 5534405', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2180, NULL, 222, NULL),
            (5744, N'Immunhisztokémiai, hisztokémiai és molekuláris biológiai szakasszisztens - 5572507', N'Immunhisztokémiai, hisztokémiai és molekuláris biológiai szakasszisztens - 5572507', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2190, NULL, 223, NULL),
            (5745, N'Infokommunikációs hálózatépítő és üzemeltető - 5448103', N'Infokommunikációs hálózatépítő és üzemeltető - 5448103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2200, NULL, 224, NULL),
            (5746, N'Információrendszer-szervező - 5548102', N'Információrendszer-szervező - 5548102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2210, NULL, 225, NULL),
            (5747, N'Informatikai rendszerüzemeltető - 5448106', N'Informatikai rendszerüzemeltető - 5448106', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2220, NULL, 226, NULL),
            (5748, N'Ingatlankezelő - 5381401', N'Ingatlankezelő - 5381401', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2230, NULL, 227, NULL),
            (5749, N'Ingatlanközvetítő - 5234102', N'Ingatlanközvetítő - 5234102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2240, NULL, 228, NULL),
            (5750, N'Ingatlanvagyon-értékelő és -közvetítő - 5334101', N'Ingatlanvagyon-értékelő és -közvetítő - 5334101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2250, NULL, 229, NULL),
            (5751, N'Intervenciós szakasszisztens - 5572517', N'Intervenciós szakasszisztens - 5572517', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2260, NULL, 230, NULL),
            (5752, N'Ipari alpinista - 3258204', N'Ipari alpinista - 3258204', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2270, NULL, 231, NULL),
            (5753, N'Ipari gépész - 3452104', N'Ipari gépész - 3452104', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2280, NULL, 232, NULL),
            (5754, N'Ipari gumitermék előállító - 3454304', N'Ipari gumitermék előállító - 3454304', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2290, NULL, 233, NULL),
            (5755, N'Ipari szigetelő bádogos - 3258205', N'Ipari szigetelő bádogos - 3258205', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2300, NULL, 234, NULL),
            (5756, N'Ipari üvegműves - 3254301', N'Ipari üvegműves - 3254301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2310, NULL, 235, NULL),
            (5757, N'Irodai informatikus - 5248102', N'Irodai informatikus - 5248102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2320, NULL, 236, NULL),
            (5758, N'Irodai titkár - 5434603', N'Irodai titkár - 5434603', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2330, NULL, 237, NULL),
            (5759, N'IT mentor - 5448201', N'IT mentor - 5448201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2340, NULL, 238, NULL),
            (5760, N'Járműfényező - 3452503', N'Járműfényező - 3452503', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2350, NULL, 239, NULL),
            (5761, N'Járműipari fémalkatrész-gyártó - 3452107', N'Járműipari fémalkatrész-gyártó - 3452107', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2360, NULL, 240, NULL),
            (5762, N'Járműipari karbantartó technikus - 5552307', N'Járműipari karbantartó technikus - 5552307', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2370, NULL, 241, NULL),
            (5763, N'Járműkarosszéria előkészítő, felületbevonó - 3452504', N'Járműkarosszéria előkészítő, felületbevonó - 3452504', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2380, NULL, 242, NULL),
            (5764, N'Járműkarosszéria készítő, szerelő - 3452505', N'Járműkarosszéria készítő, szerelő - 3452505', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2390, NULL, 243, NULL),
            (5765, N'Jazz-zenész I. (a szakmairány megjelölésével) - 5521204', N'Jazz-zenész I. (a szakmairány megjelölésével) - 5521204', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2400, NULL, 244, NULL),
            (5766, N'Jazz-zenész II. (a szakmairány megjelölésével) - 5421204', N'Jazz-zenész II. (a szakmairány megjelölésével) - 5421204', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2410, NULL, 245, NULL),
            (5767, N'Jegyvizsgáló - 5484110', N'Jegyvizsgáló - 5484110', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2420, NULL, 246, NULL),
            (5768, N'Jelmeztervező - 5521105', N'Jelmeztervező - 5521105', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2430, NULL, 247, NULL),
            (5769, N'Jelnyelvi szaktolmács - 5322301', N'Jelnyelvi szaktolmács - 5322301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2440, NULL, 248, NULL),
            (5770, N'Jelnyelvi tolmács - 5222301', N'Jelnyelvi tolmács - 5222301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2450, NULL, 249, NULL),
            (5771, N'Jelző kutya kiképzője - 5381001', N'Jelző kutya kiképzője - 5381001', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2460, NULL, 250, NULL),
            (5772, N'Kádár, bognár - 3254302', N'Kádár, bognár - 3254302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2470, NULL, 251, NULL),
            (5773, N'Kályhás - 3258207', N'Kályhás - 3258207', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2480, NULL, 252, NULL),
            (5774, N'Kardiológiai és angiológiai asszisztens - 5472506', N'Kardiológiai és angiológiai asszisztens - 5472506', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2490, NULL, 253, NULL),
            (5775, N'Karosszérialakatos - 3452506', N'Karosszérialakatos - 3452506', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2500, NULL, 254, NULL),
            (5776, N'Kárpitos - 3454205', N'Kárpitos - 3454205', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2510, NULL, 255, NULL),
            (5777, N'Kaszkadőr (a tevékenység megjelölésével) - 3221201', N'Kaszkadőr (a tevékenység megjelölésével) - 3221201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2520, NULL, 256, NULL),
            (5778, N'Katasztrófavédelmi előadó - 5286103', N'Katasztrófavédelmi előadó - 5286103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2540, NULL, 257, NULL),
            (5779, N'Katasztrófavédelmi főelőadó - 5386106', N'Katasztrófavédelmi főelőadó - 5386106', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2550, NULL, 258, NULL),
            (5780, N'Katasztrófavédelmi referens - 5386107', N'Katasztrófavédelmi referens - 5386107', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2560, NULL, 259, NULL),
            (5781, N'Katasztrófavédelmi szakelőadó - 5286104', N'Katasztrófavédelmi szakelőadó - 5286104', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2570, NULL, 260, NULL),
            (5782, N'Kazángépész (12 t/h felett) - 3352201', N'Kazángépész (12 t/h felett) - 3352201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2580, NULL, 261, NULL),
            (5783, N'Kazánkezelő (2-12 t/h között) - 3252202', N'Kazánkezelő (2-12 t/h között) - 3252202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2590, NULL, 262, NULL),
            (5784, N'Kéményseprő - 3285302', N'Kéményseprő - 3285302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2600, NULL, 263, NULL),
            (5785, N'Kémiai laboratóriumi szakasszisztens - 5572509', N'Kémiai laboratóriumi szakasszisztens - 5572509', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2610, NULL, 264, NULL),
            (5786, N'Kerámia, porcelán készítő (a szakmairány megjelölésével) - 3421102', N'Kerámia, porcelán készítő (a szakmairány megjelölésével) - 3421102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2620, NULL, 265, NULL),
            (5787, N'Kerámiaműves - 5421105', N'Kerámiaműves - 5421105', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2630, NULL, 266, NULL),
            (5788, N'Kereskedelmi képviselő - 5434102', N'Kereskedelmi képviselő - 5434102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2640, NULL, 267, NULL),
            (5789, N'Kereskedelmi, háztartási és vendéglátóipari gépszerelő - 3252102', N'Kereskedelmi, háztartási és vendéglátóipari gépszerelő - 3252102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2650, NULL, 268, NULL),
            (5790, N'Kereskedő - 5434101', N'Kereskedő - 5434101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2660, NULL, 269, NULL),
            (5791, N'Kertész - 3462202', N'Kertész - 3462202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2670, NULL, 270, NULL),
            (5792, N'Kertészeti szaktechnikus - 5562201', N'Kertészeti szaktechnikus - 5562201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2680, NULL, 271, NULL),
            (5793, N'Kézápoló és műkörömépítő - 3281501', N'Kézápoló és műkörömépítő - 3281501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2690, NULL, 272, NULL),
            (5794, N'Kiadványszerkesztő technikus - 5421306', N'Kiadványszerkesztő technikus - 5421306', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2700, NULL, 273, NULL),
            (5795, N'Kis- és középvállalkozások ügyvezetője I - 3534501', N'Kis- és középvállalkozások ügyvezetője I - 3534501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2710, NULL, 274, NULL),
            (5796, N'Kis- és középvállalkozások ügyvezetője II - 5534501', N'Kis- és középvállalkozások ügyvezetője II - 5534501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2720, NULL, 275, NULL),
            (5797, N'Kisfeszültségű csatlakozó- és közvilágítási FAM szerelő - 3552204', N'Kisfeszültségű csatlakozó- és közvilágítási FAM szerelő - 3552204', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2730, NULL, 276, NULL),
            (5798, N'Kisfeszültségű FAM kábelszerelő - 3552205', N'Kisfeszültségű FAM kábelszerelő - 3552205', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2740, NULL, 277, NULL),
            (5799, N'Kisfeszültségű kábelszerelő - 3552206', N'Kisfeszültségű kábelszerelő - 3552206', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2750, NULL, 278, NULL),
            (5800, N'Kisfeszültségű szabadvezeték hálózati FAM szerelő - 3552207', N'Kisfeszültségű szabadvezeték hálózati FAM szerelő - 3552207', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2770, NULL, 279, NULL),
            (5801, N'Kisgyermekgondozó, -nevelő - 5476102', N'Kisgyermekgondozó, -nevelő - 5476102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2780, NULL, 280, NULL),
            (5802, N'Kishajóépítő, -karbantartó - 3454305', N'Kishajóépítő, -karbantartó - 3454305', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2790, NULL, 281, NULL),
            (5803, N'Kistermelői élelmiszerelőállító, falusi vendéglátó - 3454108', N'Kistermelői élelmiszerelőállító, falusi vendéglátó - 3454108', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2800, NULL, 282, NULL),
            (5804, N'Klasszikus zenész I. (a szakmairány megnevezésével) - 5521205', N'Klasszikus zenész I. (a szakmairány megnevezésével) - 5521205', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2810, NULL, 283, NULL),
            (5805, N'Klasszikus zenész II. (a szakmairány megnevezésével) - 5421205', N'Klasszikus zenész II. (a szakmairány megnevezésével) - 5421205', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2820, NULL, 284, NULL),
            (5806, N'Klinikai fogászati higiénikus - 5572511', N'Klinikai fogászati higiénikus - 5572511', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2830, NULL, 285, NULL),
            (5807, N'Klinikai laboratóriumi szakasszisztens - 5572518', N'Klinikai laboratóriumi szakasszisztens - 5572518', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2840, NULL, 286, NULL),
            (5808, N'Klinikai neurofiziológiai asszisztens - 5472507', N'Klinikai neurofiziológiai asszisztens - 5472507', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2850, NULL, 287, NULL),
            (5809, N'Kocsivizsgáló - 5452509', N'Kocsivizsgáló - 5452509', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2860, NULL, 288, NULL),
            (5810, N'Kőfaragó, műköves és épületszobrász - 3458207 (2017)', N'Kőfaragó, műköves és épületszobrász - 3458207 (2017)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 17010, NULL, 289, NULL),
            (5811, N'Kohászati technikus - 5452104', N'Kohászati technikus - 5452104', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2870, NULL, 290, NULL),
            (5812, N'Kőműves - 3458214', N'Kőműves - 3458214', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2920, NULL, 291, NULL),
            (5813, N'Könnyűipari gyártáselőkészítő - 5254201', N'Könnyűipari gyártáselőkészítő - 5254201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2930, NULL, 292, NULL),
            (5814, N'Konyhai részlegvezető - 3581106', N'Konyhai részlegvezető - 3581106', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2880, NULL, 293, NULL),
            (5815, N'Könyvesbolti és antikváriumi kereskedő - 5234103', N'Könyvesbolti és antikváriumi kereskedő - 5234103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2940, NULL, 294, NULL),
            (5816, N'Könyvkötő-nyomtatványfeldolgozó - 3454306', N'Könyvkötő-nyomtatványfeldolgozó - 3454306', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2950, NULL, 295, NULL),
            (5817, N'Környezetvédelmi technikus - 5485001', N'Környezetvédelmi technikus - 5485001', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2960, NULL, 296, NULL),
            (5818, N'Környezetvédelmi-mérés szaktechnikus - 5585002', N'Környezetvédelmi-mérés szaktechnikus - 5585002', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2970, NULL, 297, NULL),
            (5819, N'Korrektor - 5521303', N'Korrektor - 5521303', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2890, NULL, 298, NULL),
            (5820, N'Körzeti megbízott - 5586104', N'Körzeti megbízott - 5586104', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2980, NULL, 299, NULL),
            (5821, N'Közbeszerzési referens - 5234301', N'Közbeszerzési referens - 5234301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2990, NULL, 300, NULL),
            (5822, N'Középfeszültségű FAM szerelő - 3552208 (2016)', N'Középfeszültségű FAM szerelő - 3552208 (2016)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 16000, NULL, 301, NULL),
            (5823, N'Középfeszültségű kábelszerelő - 3552209', N'Középfeszültségű kábelszerelő - 3552209', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3020, NULL, 302, NULL),
            (5824, N'Közlekedésautomatikai műszerész - 5452303', N'Közlekedésautomatikai műszerész - 5452303', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3030, NULL, 303, NULL),
            (5825, N'Kozmetikus - 5481502', N'Kozmetikus - 5481502', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2900, NULL, 304, NULL),
            (5826, N'Közművelődési és közönségkapcsolati szakember (a szakmairány megnevezésével) - 5434502', N'Közművelődési és közönségkapcsolati szakember (a szakmairány megnevezésével) - 5434502', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3040, NULL, 305, NULL),
            (5827, N'Közművelődési szakember I. - 6234501', N'Közművelődési szakember I. - 6234501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3050, NULL, 306, NULL),
            (5828, N'Közösségfejlesztő animátor - 5534502', N'Közösségfejlesztő animátor - 5534502', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3060, NULL, 307, NULL),
            (5829, N'Központifűtés- és gázhálózat rendszerszerelő - 3458209', N'Központifűtés- és gázhálózat rendszerszerelő - 3458209', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3070, NULL, 308, NULL),
            (5830, N'Közszolgálati ügyintéző - 5434501', N'Közszolgálati ügyintéző - 5434501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3100, NULL, 309, NULL),
            (5831, N'Közszolgálati ügykezelő - 5234504', N'Közszolgálati ügykezelő - 5234504', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3110, NULL, 310, NULL),
            (5832, N'Közterület-felügyelő - 5286105', N'Közterület-felügyelő - 5286105', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3080, NULL, 311, NULL),
            (5833, N'Közúti közlekedésüzemvitel-ellátó - 5484102', N'Közúti közlekedésüzemvitel-ellátó - 5484102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3090, NULL, 312, NULL),
            (5834, N'Külszíni bányász - 3254401', N'Külszíni bányász - 3254401', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3150, NULL, 313, NULL),
            (5835, N'Kutyakozmetikus - 3262101', N'Kutyakozmetikus - 3262101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3120, NULL, 314, NULL),
            (5836, N'Kutyavezető-rendőr - 5586105', N'Kutyavezető-rendőr - 5586105', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3130, NULL, 315, NULL),
            (5837, N'Kutyavezető-vagyonőr - 3386101', N'Kutyavezető-vagyonőr - 3386101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3140, NULL, 316, NULL),
            (5838, N'Lábápoló - 3281502', N'Lábápoló - 3281502', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3160, NULL, 317, NULL),
            (5839, N'Lakberendező - 5221102', N'Lakberendező - 5221102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3170, NULL, 318, NULL),
            (5840, N'Lakott-területi fakitermelő - 3562301', N'Lakott-területi fakitermelő - 3562301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3180, NULL, 319, NULL),
            (5841, N'Látszerész és optikai árucikk-kereskedő - 5472508', N'Látszerész és optikai árucikk-kereskedő - 5472508', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3190, NULL, 320, NULL),
            (5842, N'Légi közlekedésüzemvitel-ellátó - 5484103', N'Légi közlekedésüzemvitel-ellátó - 5484103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3200, NULL, 321, NULL),
            (5843, N'Légzőszervi szakápoló - 5572310', N'Légzőszervi szakápoló - 5572310', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3210, NULL, 322, NULL),
            (5844, N'Létesítményi energetikus - 5558201', N'Létesítményi energetikus - 5558201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3220, NULL, 323, NULL),
            (5845, N'Logisztikai és szállítmányozási ügyintéző - 5484111', N'Logisztikai és szállítmányozási ügyintéző - 5484111', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3230, NULL, 324, NULL),
            (5846, N'Lovastúra-vezető - 5281202', N'Lovastúra-vezető - 5281202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3240, NULL, 325, NULL),
            (5847, N'Lovász - 3462102', N'Lovász - 3462102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3250, NULL, 326, NULL),
            (5848, N'Magánnyomozó - 5286106', N'Magánnyomozó - 5286106', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3260, NULL, 327, NULL),
            (5849, N'Magasépítő technikus - 5458203', N'Magasépítő technikus - 5458203', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3270, NULL, 328, NULL),
            (5850, N'Malom- és keveréktakarmány-ipari szaktechnikus - 5554105', N'Malom- és keveréktakarmány-ipari szaktechnikus - 5554105', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3280, NULL, 329, NULL),
            (5851, N'Mechanikus vagyonvédelmi szerelő - 3586102', N'Mechanikus vagyonvédelmi szerelő - 3586102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3290, NULL, 330, NULL),
            (5852, N'Mechatronikai technikus - 5452304', N'Mechatronikai technikus - 5452304', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3300, NULL, 331, NULL),
            (5853, N'Mechatronikus-karbantartó - 3452301', N'Mechatronikus-karbantartó - 3452301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3310, NULL, 332, NULL),
            (5854, N'Megújuló energiatermelő berendezések üzemeltetője - 3552217', N'Megújuló energiatermelő berendezések üzemeltetője - 3552217', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3320, NULL, 333, NULL),
            (5855, N'Méhész - 3262102', N'Méhész - 3262102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3330, NULL, 334, NULL),
            (5856, N'Mélyépítő technikus - 5458204', N'Mélyépítő technikus - 5458204', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3340, NULL, 335, NULL),
            (5857, N'Mentálhigiénés asszisztens - 5576203', N'Mentálhigiénés asszisztens - 5576203', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3350, NULL, 336, NULL),
            (5858, N'Mentőápoló - 5572311', N'Mentőápoló - 5572311', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3370, NULL, 337, NULL),
            (5859, N'Mezőgazdasági gépész - 3452108', N'Mezőgazdasági gépész - 3452108', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3380, NULL, 338, NULL),
            (5860, N'Mezőgazdasági gépésztechnikus - 5452105', N'Mezőgazdasági gépésztechnikus - 5452105', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3390, NULL, 339, NULL),
            (5861, N'Mezőgazdasági gépjavító - 3552102', N'Mezőgazdasági gépjavító - 3552102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3400, NULL, 340, NULL),
            (5862, N'Mezőgazdasági technikus - 5462102', N'Mezőgazdasági technikus - 5462102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3410, NULL, 341, NULL),
            (5863, N'Migrációs ügyintéző - 5234707', N'Migrációs ügyintéző - 5234707', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3420, NULL, 342, NULL),
            (5864, N'Mikrobiológiai szakasszisztens - 5572514', N'Mikrobiológiai szakasszisztens - 5572514', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3430, NULL, 343, NULL),
            (5865, N'Mobilalkalmazás fejlesztő - 5521304', N'Mobilalkalmazás fejlesztő - 5521304', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3440, NULL, 344, NULL),
            (5866, N'Molnár - 3454104', N'Molnár - 3454104', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3450, NULL, 345, NULL),
            (5867, N'Motorkerékpár-szerelő - 3452507', N'Motorkerékpár-szerelő - 3452507', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3460, NULL, 346, NULL),
            (5868, N'Mozgássérültet segítő kutya kiképzője - 5381002', N'Mozgássérültet segítő kutya kiképzője - 5381002', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3470, NULL, 347, NULL),
            (5869, N'Mozgókép- és animációkészítő - 5421303', N'Mozgókép- és animációkészítő - 5421303', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3480, NULL, 348, NULL),
            (5870, N'Mozgóképgyártó - 5221301', N'Mozgóképgyártó - 5221301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3490, NULL, 349, NULL),
            (5871, N'Mozgóképgyártó hangtechnikus - 5321305', N'Mozgóképgyártó hangtechnikus - 5321305', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3500, NULL, 350, NULL),
            (5872, N'Mozgóképterjesztő és -üzemeltető - 5234104', N'Mozgóképterjesztő és -üzemeltető - 5234104', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3510, NULL, 351, NULL),
            (5873, N'Mozgólépcső ellenőr - 6252102', N'Mozgólépcső ellenőr - 6252102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3520, NULL, 352, NULL),
            (5874, N'Mozgólépcső karbantartó-szerelő - 3352101', N'Mozgólépcső karbantartó-szerelő - 3352101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3530, NULL, 353, NULL),
            (5875, N'Műanyagfeldolgozó - 3452109', N'Műanyagfeldolgozó - 3452109', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3570, NULL, 354, NULL),
            (5876, N'Műanyagfeldolgozó technikus - 5452106', N'Műanyagfeldolgozó technikus - 5452106', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3580, NULL, 355, NULL),
            (5877, N'Műbútorasztalos - 3554301', N'Műbútorasztalos - 3554301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3590, NULL, 356, NULL),
            (5878, N'Műemlékfenntartó technikus - 5558202', N'Műemlékfenntartó technikus - 5558202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3600, NULL, 357, NULL),
            (5879, N'Műemléki díszítőszobrász - 3558204', N'Műemléki díszítőszobrász - 3558204', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3610, NULL, 358, NULL),
            (5880, N'Műemléki helyreállító - 3558205', N'Műemléki helyreállító - 3558205', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3620, NULL, 359, NULL),
            (5881, N'Multimédia-alkalmazásfejlesztő - 5521305', N'Multimédia-alkalmazásfejlesztő - 5521305', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3540, NULL, 360, NULL),
            (5882, N'Munkaruha- és védőruha-készítő - 3554201', N'Munkaruha- és védőruha-készítő - 3554201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3550, NULL, 361, NULL),
            (5883, N'Munkavédelmi technikus - 5286201', N'Munkavédelmi technikus - 5286201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3560, NULL, 362, NULL),
            (5884, N'Műszaki informatikus - 5448105', N'Műszaki informatikus - 5448105', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3630, NULL, 363, NULL),
            (5885, N'Műszeres analitikus - 5552403', N'Műszeres analitikus - 5552403', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3640, NULL, 364, NULL),
            (5886, N'Műtárgyvédelmi munkatárs - 5221101', N'Műtárgyvédelmi munkatárs - 5221101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3650, NULL, 365, NULL),
            (5887, N'Műtéti szakasszisztens - 5572515', N'Műtéti szakasszisztens - 5572515', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3660, NULL, 366, NULL),
            (5888, N'Műtőssegéd-gipszmester - 3272501', N'Műtőssegéd-gipszmester - 3272501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3670, NULL, 367, NULL),
            (5889, N'Művészeti és médiafotográfus - 5421110', N'Művészeti és médiafotográfus - 5421110', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3680, NULL, 368, NULL),
            (5890, N'Nefrológiai szakápoló - 5572312', N'Nefrológiai szakápoló - 5572312', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3690, NULL, 369, NULL),
            (5891, N'Népi játék és kismesterségek oktatója - 5321501', N'Népi játék és kismesterségek oktatója - 5321501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3700, NULL, 370, NULL),
            (5892, N'Népi kézműves (a szakmairány megjelölésével) - 3421501', N'Népi kézműves (a szakmairány megjelölésével) - 3421501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3710, NULL, 371, NULL),
            (5893, N'Népzenész I. (a szakmairány megjelölésével) - 5521206', N'Népzenész I. (a szakmairány megjelölésével) - 5521206', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3720, NULL, 372, NULL),
            (5894, N'Népzenész II. (a szakmairány megjelölésével) - 5421206', N'Népzenész II. (a szakmairány megjelölésével) - 5421206', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3730, NULL, 373, NULL),
            (5895, N'Nevelőszülő - 3276101', N'Nevelőszülő - 3276101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3740, NULL, 374, NULL),
            (5896, N'Női szabó - 3454206', N'Női szabó - 3454206', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3760, NULL, 375, NULL),
            (5897, N'Nonprofit menedzser - 5234505', N'Nonprofit menedzser - 5234505', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3750, NULL, 376, NULL),
            (5898, N'Növényvédelmi szaktechnikus - 5562102', N'Növényvédelmi szaktechnikus - 5562102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3770, NULL, 377, NULL),
            (5899, N'Nukleáris környezetvédelmi szaktechnikus - 5585003', N'Nukleáris környezetvédelmi szaktechnikus - 5585003', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3780, NULL, 378, NULL),
            (5900, N'Nukleáris medicina szakasszisztens - 5572519', N'Nukleáris medicina szakasszisztens - 5572519', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3790, NULL, 379, NULL),
            (5901, N'Nyíltvízi- vízimentő - 3281002', N'Nyíltvízi- vízimentő - 3281002', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3800, NULL, 380, NULL),
            (5902, N'Nyilvántartási és okmányügyintéző - 5234503 (2016)', N'Nyilvántartási és okmányügyintéző - 5234503 (2016)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 16010, NULL, 381, NULL),
            (5903, N'Nyomástartóedény-gépész - 3252401', N'Nyomástartóedény-gépész - 3252401', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3810, NULL, 382, NULL),
            (5904, N'Nyomdaipari szaktechnikus - 5521307', N'Nyomdaipari szaktechnikus - 5521307', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3820, NULL, 383, NULL),
            (5905, N'Nyomdaipari technikus - 5421307', N'Nyomdaipari technikus - 5421307', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3830, NULL, 384, NULL),
            (5906, N'Okleveles adóellenőrzési szakértő - 6234401', N'Okleveles adóellenőrzési szakértő - 6234401', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3840, NULL, 385, NULL),
            (5907, N'Okleveles forgalmiadó-szakértő - 6234402', N'Okleveles forgalmiadó-szakértő - 6234402', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3850, NULL, 386, NULL),
            (5908, N'Okleveles jövedelemadó-szakértő - 6234403', N'Okleveles jövedelemadó-szakértő - 6234403', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3860, NULL, 387, NULL),
            (5909, N'Okleveles nemzetköziadó-szakértő - 6234404', N'Okleveles nemzetköziadó-szakértő - 6234404', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3870, NULL, 388, NULL),
            (5910, N'Okmányvizsgáló - 5586106', N'Okmányvizsgáló - 5586106', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3880, NULL, 389, NULL),
            (5911, N'Olvasztár és öntő - 3252103', N'Olvasztár és öntő - 3252103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3890, NULL, 390, NULL),
            (5912, N'Onkológiai szakápoló - 5572313', N'Onkológiai szakápoló - 5572313', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3900, NULL, 391, NULL),
            (5913, N'Öntészeti mintakészítő - 3252105', N'Öntészeti mintakészítő - 3252105', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3980, NULL, 392, NULL),
            (5914, N'Optikai üvegcsiszoló - 3472501', N'Optikai üvegcsiszoló - 3472501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3910, NULL, 393, NULL),
            (5915, N'Órás - 3252104', N'Órás - 3252104', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3920, NULL, 394, NULL),
            (5916, N'Ortopédiai cipész - 3354201', N'Ortopédiai cipész - 3354201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3930, NULL, 395, NULL),
            (5917, N'Ortopédiai kötszerész és fűzőkészítő - 3272601', N'Ortopédiai kötszerész és fűzőkészítő - 3272601', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3940, NULL, 396, NULL),
            (5918, N'Ortopédiai műszerész - 5472602', N'Ortopédiai műszerész - 5472602', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3950, NULL, 397, NULL),
            (5919, N'Orvosi elektronikai technikus - 5552304', N'Orvosi elektronikai technikus - 5552304', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3960, NULL, 398, NULL),
            (5920, N'Ötvös - 5421106', N'Ötvös - 5421106', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3990, NULL, 399, NULL),
            (5921, N'Óvodai dajka - 3214001', N'Óvodai dajka - 3214001', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3970, NULL, 400, NULL),
            (5922, N'Pályázati-támogatási asszisztens - 5234506', N'Pályázati-támogatási asszisztens - 5234506', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4000, NULL, 401, NULL),
            (5923, N'Pantomimes - 5421207', N'Pantomimes - 5421207', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4010, NULL, 402, NULL),
            (5924, N'Papírgyártó és -feldolgozó - 3254303', N'Papírgyártó és -feldolgozó - 3254303', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4020, NULL, 403, NULL),
            (5925, N'Papíripari technikus - 5552404', N'Papíripari technikus - 5552404', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4030, NULL, 404, NULL),
            (5926, N'Parképítő és fenntartó technikus - 5458102', N'Parképítő és fenntartó technikus - 5458102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4040, NULL, 405, NULL),
            (5927, N'Patkolókovács - 3552103', N'Patkolókovács - 3552103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4050, NULL, 406, NULL),
            (5928, N'Pedagógiai- és családsegítő munkatárs - 5414002', N'Pedagógiai- és családsegítő munkatárs - 5414002', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4060, NULL, 407, NULL),
            (5929, N'Pék - 3454105', N'Pék - 3454105', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4070, NULL, 408, NULL),
            (5930, N'Pénzügyi szervezeti mérlegképes könyvelő - 5534406', N'Pénzügyi szervezeti mérlegképes könyvelő - 5534406', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4090, NULL, 409, NULL),
            (5931, N'Pénzügyi termékértékesítő (bank, befektetés, biztosítás) - 5434301', N'Pénzügyi termékértékesítő (bank, befektetés, biztosítás) - 5434301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4100, NULL, 410, NULL),
            (5932, N'Pénzügyi-számviteli ügyintéző - 5434401', N'Pénzügyi-számviteli ügyintéző - 5434401', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4110, NULL, 411, NULL),
            (5933, N'Perioperatív asszisztens - 5472004', N'Perioperatív asszisztens - 5472004', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4120, NULL, 412, NULL),
            (5934, N'Pincér - 3481103', N'Pincér - 3481103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4130, NULL, 413, NULL),
            (5935, N'Pirotechnikus - 5254401', N'Pirotechnikus - 5254401', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4140, NULL, 414, NULL),
            (5936, N'Postai üzleti ügyintéző - 5484109', N'Postai üzleti ügyintéző - 5484109', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4150, NULL, 415, NULL),
            (5937, N'Pszichiátriai gondozó - 5576204', N'Pszichiátriai gondozó - 5576204', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4160, NULL, 416, NULL),
            (5938, N'Pszichiátriai szakápoló és gyógyfoglalkoztató - 5572314', N'Pszichiátriai szakápoló és gyógyfoglalkoztató - 5572314', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4170, NULL, 417, NULL),
            (5939, N'Radiográfiai asszisztens - 5472509', N'Radiográfiai asszisztens - 5472509', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4180, NULL, 418, NULL),
            (5940, N'Rehabilitációs nevelő, segítő - 5476201', N'Rehabilitációs nevelő, segítő - 5476201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4190, NULL, 419, NULL),
            (5941, N'Rekonstrukciós és műemléki festő, mázoló - 3558206', N'Rekonstrukciós és műemléki festő, mázoló - 3558206', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4200, NULL, 420, NULL),
            (5942, N'Rendészeti őr - 3486101', N'Rendészeti őr - 3486101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4210, NULL, 421, NULL),
            (5943, N'Rendészeti szervező (a szakmairány megjelölésével) - 6286101', N'Rendészeti szervező (a szakmairány megjelölésével) - 6286101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4220, NULL, 422, NULL),
            (5944, N'Rendészeti ügyintéző - 5286111', N'Rendészeti ügyintéző - 5286111', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4230, NULL, 423, NULL),
            (5945, N'Rendezvényszervező - 5234507', N'Rendezvényszervező - 5234507', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4240, NULL, 424, NULL),
            (5946, N'Rendőr tiszthelyettes (a szakmairány megjelölésével) - 5486101', N'Rendőr tiszthelyettes (a szakmairány megjelölésével) - 5486101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4250, NULL, 425, NULL),
            (5947, N'Rendőr zászlós - 5586107', N'Rendőr zászlós - 5586107', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4260, NULL, 426, NULL),
            (5948, N'Repülőgép-szerelő - 5452510', N'Repülőgép-szerelő - 5452510', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4270, NULL, 427, NULL),
            (5949, N'Robbanásbiztos berendezés kezelője - 5252203', N'Robbanásbiztos berendezés kezelője - 5252203', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4280, NULL, 428, NULL),
            (5950, N'Ruhaipari technikus - 5454202', N'Ruhaipari technikus - 5454202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4290, NULL, 429, NULL),
            (5951, N'Segédkönyvtáros - 5232201', N'Segédkönyvtáros - 5232201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4300, NULL, 430, NULL),
            (5952, N'Segédlevéltáros és ügykezelő - 5232202', N'Segédlevéltáros és ügykezelő - 5232202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4310, NULL, 431, NULL),
            (5953, N'Segédrestaurátor (a szakmairány megjelölésével) - 5321101', N'Segédrestaurátor (a szakmairány megjelölésével) - 5321101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4320, NULL, 432, NULL),
            (5954, N'Sommelier - 3581101', N'Sommelier - 3581101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4330, NULL, 433, NULL),
            (5955, N'Speciális állatfeldolgozó - 3554101', N'Speciális állatfeldolgozó - 3554101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4340, NULL, 434, NULL),
            (5956, N'Speciális lábápoló - 5381501', N'Speciális lábápoló - 5381501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4350, NULL, 435, NULL),
            (5957, N'Sportedző (a sportág megjelölésével) - 5481302', N'Sportedző (a sportág megjelölésével) - 5481302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4360, NULL, 436, NULL),
            (5958, N'Sportmasszőr - 5572602', N'Sportmasszőr - 5572602', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4370, NULL, 437, NULL),
            (5959, N'Sportszervező, -menedzser - 5281301', N'Sportszervező, -menedzser - 5281301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4380, NULL, 438, NULL),
            (5960, N'Statisztikai és gazdasági ügyintéző - 5246201', N'Statisztikai és gazdasági ügyintéző - 5246201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4390, NULL, 439, NULL),
            (5961, N'Stúdióvezető - 5321306', N'Stúdióvezető - 5321306', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4400, NULL, 440, NULL),
            (5962, N'Sugárterápiás szakasszisztens - 5572520', N'Sugárterápiás szakasszisztens - 5572520', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4410, NULL, 441, NULL),
            (5963, N'Sürgősségi szakápoló - 5572315', N'Sürgősségi szakápoló - 5572315', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4420, NULL, 442, NULL),
            (5964, N'Sütő- és cukrászipari szaktechnikus - 5554106', N'Sütő- és cukrászipari szaktechnikus - 5554106', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4430, NULL, 443, NULL),
            (5965, N'Szakács - 3481104', N'Szakács - 3481104', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4440, NULL, 444, NULL),
            (5966, N'Szakképesített bankreferens - 5534302', N'Szakképesített bankreferens - 5534302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4450, NULL, 445, NULL),
            (5967, N'Szakszolgálati FAM szerelő - 3552210', N'Szakszolgálati FAM szerelő - 3552210', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4460, NULL, 446, NULL),
            (5968, N'Szállodahajós, szállodai személyzet - 5281203', N'Szállodahajós, szállodai személyzet - 5281203', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4470, NULL, 447, NULL),
            (5969, N'Szállodai recepciós - 5281201', N'Szállodai recepciós - 5281201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4480, NULL, 448, NULL),
            (5970, N'Számítógép-szerelő, karbantartó - 3452302', N'Számítógép-szerelő, karbantartó - 3452302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4490, NULL, 449, NULL),
            (5971, N'Szárazépítő - 3458210', N'Szárazépítő - 3458210', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4500, NULL, 450, NULL),
            (5972, N'Személy- és vagyonőr - 3286101', N'Személy- és vagyonőr - 3286101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4510, NULL, 451, NULL),
            (5973, N'Személyi edző - 5581301', N'Személyi edző - 5581301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4520, NULL, 452, NULL),
            (5974, N'Szenvedélybeteg gondozó - 5576205', N'Szenvedélybeteg gondozó - 5576205', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4530, NULL, 453, NULL),
            (5975, N'Szépségtanácsadó - 5281504', N'Szépségtanácsadó - 5281504', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4540, NULL, 454, NULL),
            (5976, N'Szerkesztő műsorvezető - 5321307', N'Szerkesztő műsorvezető - 5321307', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4550, NULL, 455, NULL),
            (5977, N'Szerszámkészítő - 3452110', N'Szerszámkészítő - 3452110', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4560, NULL, 456, NULL),
            (5978, N'Szíjgyártó és nyerges - 3554202', N'Szíjgyártó és nyerges - 3554202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4570, NULL, 457, NULL),
            (5979, N'Színész - 5521201', N'Színész - 5521201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4580, NULL, 458, NULL),
            (5980, N'Színháztechnikus, szcenikus - 5452107', N'Színháztechnikus, szcenikus - 5452107', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4590, NULL, 459, NULL),
            (5981, N'Színpad- és porond technikus - 5452108', N'Színpad- és porond technikus - 5452108', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4600, NULL, 460, NULL),
            (5982, N'Színpadi fénytervező - 5552102', N'Színpadi fénytervező - 5552102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4610, NULL, 461, NULL),
            (5983, N'Színpadi magasépítő - 5552103', N'Színpadi magasépítő - 5552103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4620, NULL, 462, NULL),
            (5984, N'Szobrász - 5421107', N'Szobrász - 5421107', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4630, NULL, 463, NULL),
            (5985, N'Szociális asszisztens - 5476202', N'Szociális asszisztens - 5476202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4640, NULL, 464, NULL),
            (5986, N'Szociális gondozó és ápoló - 3476201', N'Szociális gondozó és ápoló - 3476201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4650, NULL, 465, NULL),
            (5987, N'Szociális szakgondozó - 5476203', N'Szociális szakgondozó - 5476203', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4660, NULL, 466, NULL),
            (5988, N'Szociális, gyermek- és ifjúságvédelmi ügyintéző - 5576206', N'Szociális, gyermek- és ifjúságvédelmi ügyintéző - 5576206', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4670, NULL, 467, NULL),
            (5989, N'Szoftverfejlesztő - 5421305', N'Szoftverfejlesztő - 5421305', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4680, NULL, 468, NULL),
            (5990, N'Szőlész-borász - 3454106', N'Szőlész-borász - 3454106', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4710, NULL, 469, NULL),
            (5991, N'Szórakoztató zenész I. (a szakmairány megjelölésével) - 5521207', N'Szórakoztató zenész I. (a szakmairány megjelölésével) - 5521207', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4690, NULL, 470, NULL),
            (5992, N'Szórakoztató zenész II. (a szakmairány megjelölésével) - 5421208', N'Szórakoztató zenész II. (a szakmairány megjelölésével) - 5421208', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4700, NULL, 471, NULL),
            (5993, N'Szövettani szakasszisztens - 5572521', N'Szövettani szakasszisztens - 5572521', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4720, NULL, 472, NULL),
            (5994, N'Tánc instruktor - 5281302', N'Tánc instruktor - 5281302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4730, NULL, 473, NULL),
            (5995, N'Táncos I. (a szakmairány megjelölésével) - 5521208', N'Táncos I. (a szakmairány megjelölésével) - 5521208', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4740, NULL, 474, NULL),
            (5996, N'Táncos II. (a szakmairány megjelölésével) - 5421209', N'Táncos II. (a szakmairány megjelölésével) - 5421209', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4750, NULL, 475, NULL),
            (5997, N'Társasházkezelő - 5281401', N'Társasházkezelő - 5281401', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4760, NULL, 476, NULL),
            (5998, N'Tartósítóipari szakmunkás - 3454109', N'Tartósítóipari szakmunkás - 3454109', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4770, NULL, 477, NULL),
            (5999, N'Tartósítóipari szaktechnikus - 5554107', N'Tartósítóipari szaktechnikus - 5554107', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4780, NULL, 478, NULL),
            (6000, N'Távérzékelési szaktechnikus - 5558102', N'Távérzékelési szaktechnikus - 5558102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4790, NULL, 479, NULL),
            (6001, N'Távközlési technikus - 5452305', N'Távközlési technikus - 5452305', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4800, NULL, 480, NULL)
                ) x (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
                WHERE NOT EXISTS (SELECT 1 FROM #DICTIONARYITEMBASE d WHERE x.ID = d.ID)
                
    INSERT INTO #DICTIONARYITEMBASE (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
        SELECT * FROM ( VALUES
            (6002, N'Távközlési üzemeltető - 5252301', N'Távközlési üzemeltető - 5252301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4810, NULL, 481, NULL),
            (6003, N'Tehergépkocsi-vezető - 3584102', N'Tehergépkocsi-vezető - 3584102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4820, NULL, 482, NULL),
            (6004, N'Tejipari szakmunkás - 3454110', N'Tejipari szakmunkás - 3454110', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4830, NULL, 483, NULL),
            (6005, N'Tejipari szaktechnikus - 5554108', N'Tejipari szaktechnikus - 5554108', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4840, NULL, 484, NULL),
            (6006, N'Települési környezetvédelmi szaktechnikus - 5585004', N'Települési környezetvédelmi szaktechnikus - 5585004', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4850, NULL, 485, NULL),
            (6007, N'Televíziós kameraman - 5321308', N'Televíziós kameraman - 5321308', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4860, NULL, 486, NULL),
            (6008, N'Temetkezési szolgáltató - 5285301', N'Temetkezési szolgáltató - 5285301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4870, NULL, 487, NULL),
            (6009, N'Térinformatikus - 5548101', N'Térinformatikus - 5548101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4880, NULL, 488, NULL),
            (6010, N'Térképész szaktechnikus - 5558103', N'Térképész szaktechnikus - 5558103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4890, NULL, 489, NULL),
            (6011, N'Természetvédelmi munkatárs - 3285001', N'Természetvédelmi munkatárs - 3285001', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4900, NULL, 490, NULL),
            (6012, N'Természetvédelmi szaktechnikus - 5585005', N'Természetvédelmi szaktechnikus - 5585005', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4910, NULL, 491, NULL),
            (6013, N'Testőr - 3386102', N'Testőr - 3386102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4920, NULL, 492, NULL),
            (6014, N'Tetőfedő - 3458215', N'Tetőfedő - 3458215', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4930, NULL, 493, NULL),
            (6015, N'Textilanyaggyártó - 3254201', N'Textilanyaggyártó - 3254201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4940, NULL, 494, NULL),
            (6016, N'Textilipari technikus - 5454203', N'Textilipari technikus - 5454203', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4950, NULL, 495, NULL),
            (6017, N'Textilműves - 5421108', N'Textilműves - 5421108', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4960, NULL, 496, NULL),
            (6018, N'Textiltisztító és textilszínező - 3254202', N'Textiltisztító és textilszínező - 3254202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4970, NULL, 497, NULL),
            (6019, N'Tisztítás-technológiai szakmunkás - 3285303', N'Tisztítás-technológiai szakmunkás - 3285303', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4980, NULL, 498, NULL),
            (6020, N'Tisztítás-technológiai szolgáltatásvezető - 5285302', N'Tisztítás-technológiai szolgáltatásvezető - 5285302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4990, NULL, 499, NULL),
            (6021, N'Töltőállomás-kezelő - 3234101', N'Töltőállomás-kezelő - 3234101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5000, NULL, 500, NULL),
            (6022, N'Turisztikai szervező, értékesítő - 5481203', N'Turisztikai szervező, értékesítő - 5481203', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5010, NULL, 501, NULL),
            (6023, N'Tűzoltó I. - 5286108', N'Tűzoltó I. - 5286108', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5020, NULL, 502, NULL),
            (6024, N'Tűzoltó szerparancsnok - 5386108', N'Tűzoltó szerparancsnok - 5386108', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5030, NULL, 503, NULL),
            (6025, N'Tűzoltósági referens - 5386109', N'Tűzoltósági referens - 5386109', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5040, NULL, 504, NULL),
            (6026, N'Tűzvédelmi előadó - 5286110', N'Tűzvédelmi előadó - 5286110', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5050, NULL, 505, NULL),
            (6027, N'Tűzvédelmi főelőadó - 5386110', N'Tűzvédelmi főelőadó - 5386110', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5060, NULL, 506, NULL),
            (6028, N'Ügyfélszolgálati ügyintéző - 5284102', N'Ügyfélszolgálati ügyintéző - 5284102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5120, NULL, 507, NULL),
            (6029, N'Uszodamester - 3281001', N'Uszodamester - 3281001', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5070, NULL, 508, NULL),
            (6030, N'Útépítő - 3458211', N'Útépítő - 3458211', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5080, NULL, 509, NULL),
            (6031, N'Útépítő és -fenntartó technikus - 5458205', N'Útépítő és -fenntartó technikus - 5458205', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5090, NULL, 510, NULL),
            (6032, N'Útfenntartó - 3258208', N'Útfenntartó - 3258208', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5100, NULL, 511, NULL),
            (6033, N'Utómunka szakasszisztens - 5321309', N'Utómunka szakasszisztens - 5321309', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5110, NULL, 512, NULL),
            (6034, N'Üvegcsiszoló - 3254304', N'Üvegcsiszoló - 3254304', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5130, NULL, 513, NULL),
            (6035, N'Üveggyártó - 3254305', N'Üveggyártó - 3254305', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5150, NULL, 514, NULL),
            (6036, N'Üvegműves - 5421109', N'Üvegműves - 5421109', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5140, NULL, 515, NULL),
            (6037, N'Üzleti szolgáltatási munkatárs - 5434001', N'Üzleti szolgáltatási munkatárs - 5434001', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5160, NULL, 516, NULL),
            (6038, N'Vadászpuska műves - 5586302', N'Vadászpuska műves - 5586302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5170, NULL, 517, NULL),
            (6039, N'Vadgazdálkodási technikus - 5462501', N'Vadgazdálkodási technikus - 5462501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5180, NULL, 518, NULL),
            (6040, N'Vakvezető kutya kiképzője - 5381003', N'Vakvezető kutya kiképzője - 5381003', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5190, NULL, 519, NULL),
            (6041, N'Vállalkozási és bérügyintéző - 5434402', N'Vállalkozási és bérügyintéző - 5434402', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5200, NULL, 520, NULL),
            (6042, N'Vállalkozási mérlegképes könyvelő - 5534407', N'Vállalkozási mérlegképes könyvelő - 5534407', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5210, NULL, 521, NULL),
            (6043, N'Váltókezelő - 5284102 (2017)', N'Váltókezelő - 5284102 (2017)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 17020, NULL, 522, NULL),
            (6044, N'Vám-, jövedéki- és termékdíj ügyintéző - 5434403', N'Vám-, jövedéki- és termékdíj ügyintéző - 5434403', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5230, NULL, 523, NULL),
            (6045, N'Vas- és acélfeldolgozó - 3252106', N'Vas- és acélfeldolgozó - 3252106', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5240, NULL, 524, NULL),
            (6046, N'Vasútépítő és -fenntartó technikus - 5458206', N'Vasútépítő és -fenntartó technikus - 5458206', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5250, NULL, 525, NULL),
            (6047, N'Vasútforgalmi szolgálattevő - 5484105', N'Vasútforgalmi szolgálattevő - 5484105', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5260, NULL, 526, NULL),
            (6048, N'Vasúti árufuvarozási ügyintéző - 5484106', N'Vasúti árufuvarozási ügyintéző - 5484106', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5270, NULL, 527, NULL),
            (6049, N'Vasúti jármű dízelmotor- és hajtásszerelő - 5452511', N'Vasúti jármű dízelmotor- és hajtásszerelő - 5452511', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5280, NULL, 528, NULL),
            (6050, N'Vasúti jármű szerkezeti és fékrendszer szerelője - 5452512', N'Vasúti jármű szerkezeti és fékrendszer szerelője - 5452512', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5290, NULL, 529, NULL),
            (6051, N'Vasúti jármű villamos rendszereinek szerelője - 5452513', N'Vasúti jármű villamos rendszereinek szerelője - 5452513', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5300, NULL, 530, NULL),
            (6052, N'Vasúti személypénztáros - 5284103', N'Vasúti személypénztáros - 5284103', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5310, NULL, 531, NULL),
            (6053, N'Vasútijármű-technikus és diagnosztikus - 5552504', N'Vasútijármű-technikus és diagnosztikus - 5552504', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5320, NULL, 532, NULL),
            (6054, N'Vegyész technikus - 5452403', N'Vegyész technikus - 5452403', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5330, NULL, 533, NULL),
            (6055, N'Vegyipari rendszerkezelő - 3452402', N'Vegyipari rendszerkezelő - 3452402', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5340, NULL, 534, NULL),
            (6056, N'Vegyipari rendszerüzemeltető szaktechnikus - 5552406', N'Vegyipari rendszerüzemeltető szaktechnikus - 5552406', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5350, NULL, 535, NULL),
            (6057, N'Vendéglátásszervező - 5481101', N'Vendéglátásszervező - 5481101', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5360, NULL, 536, NULL),
            (6058, N'Vendéglátó-üzletvezető - 3581102', N'Vendéglátó-üzletvezető - 3581102', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5370, NULL, 537, NULL),
            (6059, N'Vésnök - 5521108', N'Vésnök - 5521108', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5380, NULL, 538, NULL),
            (6060, N'Vidékfejlesztési szaktechnikus - 5558104', N'Vidékfejlesztési szaktechnikus - 5558104', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5390, NULL, 539, NULL),
            (6061, N'Villamos alállomás kezelő - 3552211', N'Villamos alállomás kezelő - 3552211', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5400, NULL, 540, NULL),
            (6062, N'Villamos berendezés szerelő és üzemeltető - 5252204', N'Villamos berendezés szerelő és üzemeltető - 5252204', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5410, NULL, 541, NULL),
            (6063, N'Villamos elosztóhálózat szerelő, üzemeltető - 3552212', N'Villamos elosztóhálózat szerelő, üzemeltető - 3552212', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5420, NULL, 542, NULL),
            (6064, N'Villamos gép és -készülék üzemeltető - 3552213', N'Villamos gép és -készülék üzemeltető - 3552213', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5430, NULL, 543, NULL),
            (6065, N'Villamos hálózat kezelő - 3552214', N'Villamos hálózat kezelő - 3552214', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5440, NULL, 544, NULL),
            (6066, N'Villamos távvezeték építő, üzemeltető - 3552215', N'Villamos távvezeték építő, üzemeltető - 3552215', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5450, NULL, 545, NULL),
            (6067, N'Villámvédelmi felülvizsgáló - 3552216', N'Villámvédelmi felülvizsgáló - 3552216', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5460, NULL, 546, NULL),
            (6068, N'Villanyszerelő - 3452204', N'Villanyszerelő - 3452204', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5470, NULL, 547, NULL),
            (6069, N'Virágdekoratőr - 3521502', N'Virágdekoratőr - 3521502', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5480, NULL, 548, NULL),
            (6070, N'Virágkötő és virágkereskedő - 3421504', N'Virágkötő és virágkereskedő - 3421504', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5490, NULL, 549, NULL),
            (6071, N'Víz-, csatorna- és közmű- rendszerszerelő - 3458212', N'Víz-, csatorna- és közmű- rendszerszerelő - 3458212', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5500, NULL, 550, NULL),
            (6072, N'Vízépítő szaktechnikus - 5585301', N'Vízépítő szaktechnikus - 5585301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5510, NULL, 551, NULL),
            (6073, N'Vízgazdálkodó technikus - 5485302', N'Vízgazdálkodó technikus - 5485302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5520, NULL, 552, NULL),
            (6074, N'Vízgépészeti és technológiai berendezésszerelő - 3558208', N'Vízgépészeti és technológiai berendezésszerelő - 3558208', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5530, NULL, 553, NULL),
            (6075, N'Vízgépészeti technikus - 5485303', N'Vízgépészeti technikus - 5485303', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5540, NULL, 554, NULL),
            (6076, N'Vizi sportmotor-szerelő - 3552501', N'Vizi sportmotor-szerelő - 3552501', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5550, NULL, 555, NULL),
            (6077, N'Víziközmű technikus - 5485304', N'Víziközmű technikus - 5485304', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5560, NULL, 556, NULL),
            (6078, N'Vízminőség-védelmi szaktechnikus - 5585302', N'Vízminőség-védelmi szaktechnikus - 5585302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5580, NULL, 557, NULL),
            (6079, N'Vízszigetelő, melegburkoló - 3258206', N'Vízszigetelő, melegburkoló - 3258206', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5610, NULL, 558, NULL),
            (6080, N'Vízügyi szakmunkás - 3485302', N'Vízügyi szakmunkás - 3485302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5590, NULL, 559, NULL),
            (6081, N'Vízügyi ügyintéző - 5285303', N'Vízügyi ügyintéző - 5285303', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5600, NULL, 560, NULL),
            (6082, N'Webfejlesztő - 5521302', N'Webfejlesztő - 5521302', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5620, NULL, 561, NULL),
            (6083, N'Zöldség- és gyümölcstermesztő - 3562202', N'Zöldség- és gyümölcstermesztő - 3562202', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5630, NULL, 562, NULL),
            (6712, N'Informatikai rendszergazda - 5448104 (2014)', N'Informatikai rendszergazda - 5448104 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14070, NULL, 563, NULL),
            (6713, N'Vendéglátó eladó - 3481105 (2014)', N'Vendéglátó eladó - 3481105 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14200, NULL, 564, NULL),
            (6714, N'Kiadványszerkesztő - 5421302 (2014)', N'Kiadványszerkesztő - 5421302 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14090, NULL, 565, NULL),
            (6715, N'Mezőgazdasági gazdaasszony, falusi vendéglátó - 3481102 (2014)', N'Mezőgazdasági gazdaasszony, falusi vendéglátó - 3481102 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14130, NULL, 566, NULL),
            (6716, N'Hidegburkoló', N'Hidegburkoló', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 20000, NULL, 567, NULL),
            (6815, N'Erdészeti és vadgazdálkodási technikus - 5462301 (2014)', N'Erdészeti és vadgazdálkodási technikus - 5462301 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14020, NULL, 568, NULL),
            (6816, N'Irodai asszisztens - 5434601 (2014)', N'Irodai asszisztens - 5434601 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14080, NULL, 569, NULL),
            (6814, N'Kőműves és hidegburkoló - 3458208 (2014)', N'Kőműves és hidegburkoló - 3458208 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14100, NULL, 570, NULL),
            (6817, N'Logisztikai ügyintéző - 5434501 (2014)', N'Logisztikai ügyintéző - 5434501 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14120, NULL, 571, NULL),
            (6821, N'Gyakorló kozmetikus - 5281502 (2014)', N'Gyakorló kozmetikus - 5281502 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14060, NULL, 572, NULL),
            (6820, N'Vízügyi technikus - 5485301 (2014)', N'Vízügyi technikus - 5485301 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14220, NULL, 573, NULL),
            (6822, N'Laboratóriumi technikus - 5452401 (2014)', N'Laboratóriumi technikus - 5452401 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14110, NULL, 574, NULL),
            (6823, N'Vegyipari technikus - 5452402 (2014)', N'Vegyipari technikus - 5452402 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14180, NULL, 575, NULL),
            (6828, N'Honvédelmi igazgatási ügyintéző - 5486303', N'Honvédelmi igazgatási ügyintéző - 5486303', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2060, NULL, 576, NULL),
            (6829, N'Víz- és csatornaműkezelő - 3485301 (2014)', N'Víz- és csatornaműkezelő - 3485301 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14210, NULL, 577, NULL),
            (6944, N'Gyakorló fodrász - 5281501 (2014)', N'Gyakorló fodrász - 5281501 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14050, NULL, 578, NULL),
            (6951, N'Gyakorló ápoló - 5272301 (2014)', N'Gyakorló ápoló - 5272301 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14040, NULL, 579, NULL),
            (7369, N'Fodrász - 5581501 (2014)', N'Fodrász - 5581501 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14030, NULL, 580, NULL),
            (7370, N'Szállítmányozási ügyintéző - 5484104 (2014)', N'Szállítmányozási ügyintéző - 5484104 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14150, NULL, 581, NULL),
            (7376, N'Épületszobrász és műköves - 3458216', N'Épületszobrász és műköves - 3458216', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1080, NULL, 582, NULL),
            (7377, N'Pék-cukrász - 3454111', N'Pék-cukrász - 3454111', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 4080, NULL, 583, NULL),
            (7378, N'Vasúti villamos jármű szerelője - 5452507 (2014)', N'Vasúti villamos jármű szerelője - 5452507 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14170, NULL, 584, NULL),
            (7379, N'Kőfaragó - 3458217', N'Kőfaragó - 3458217', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2910, NULL, 585, NULL),
            (7736, N'Fogyatékkal élők gondozója - 3214002', N'Fogyatékkal élők gondozója - 3214002', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 1430, NULL, 586, NULL),
            (7737, N'Honvéd kadét, igazgatási ügykezelő - 5286301', N'Honvéd kadét, igazgatási ügykezelő - 5286301', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2050, NULL, 587, NULL),
            (7738, N'Kaszkadőr szakértő - 5221201', N'Kaszkadőr szakértő - 5221201', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2530, NULL, 588, NULL),
            (7739, N'Kisfeszültségű mérőhelyi FAM szerelő - 3552220', N'Kisfeszültségű mérőhelyi FAM szerelő - 3552220', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 2760, NULL, 589, NULL),
            (7740, N'Középfeszültségű FAM szerelő kombinált technológiával. - 3552219', N'Középfeszültségű FAM szerelő kombinált technológiával. - 3552219', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3000, NULL, 590, NULL),
            (7741, N'Középfeszültségű FAM szerelő távolból végzett technológiával - 3552218', N'Középfeszültségű FAM szerelő távolból végzett technológiával - 3552218', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3010, NULL, 591, NULL),
            (7742, N'Mentésirányító - 5572317', N'Mentésirányító - 5572317', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 3360, NULL, 592, NULL),
            (7743, N'Váltókezelő - 5284104', N'Váltókezelő - 5284104', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5220, NULL, 593, NULL),
            (7744, N'Vízkútfúró - 5258203', N'Vízkútfúró - 5258203', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 5570, NULL, 594, NULL),
            (7745, N'Több szakképesítés', N'Több szakképesítés', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'F', 10, NULL, 595, NULL),
            (7769, N'Vízi közlekedésüzemvitel-ellátó - 5484108 (2014)', N'Vízi közlekedésüzemvitel-ellátó - 5484108 (2014)', NULL, NULL, NULL, N'SzakkepesitesTipus', 92, N'T', N'T', 14215, NULL, 596, NULL),
            (5431, N'Székhely', N'Székhely', NULL, NULL, NULL, N'MukodesiHelyTipus', 90, N'F', N'T', 1, NULL, 1, NULL),
            (5432, N'Tagintézmény', N'Tagintézmény', NULL, NULL, NULL, N'MukodesiHelyTipus', 90, N'F', N'T', 2, NULL, 2, NULL),
            (5433, N'Telephely', N'Telephely', NULL, NULL, NULL, N'MukodesiHelyTipus', 90, N'F', N'T', 3, NULL, 3, NULL),
            (5414, N'Nincs', N'Nincs', NULL, NULL, NULL, N'IKTEszkozhasznalatMod', 87, N'F', N'T', 1, NULL, 1, NULL),
            (5415, N'Saját eszköz', N'Saját eszköz', NULL, NULL, NULL, N'IKTEszkozhasznalatMod', 87, N'F', N'T', 2, NULL, 2, NULL),
            (5416, N'Iskolai eszköz', N'Iskolai eszköz', NULL, NULL, NULL, N'IKTEszkozhasznalatMod', 87, N'F', N'T', 3, NULL, 3, NULL),
            (5417, N'PC - három évnél idősebb', N'PC - három évnél idősebb', NULL, NULL, NULL, N'ElsodlegesIKTEszkoz', 88, N'F', N'T', 1, NULL, 1, NULL),
            (5418, N'PC - három évnél fiatalabb', N'PC - három évnél fiatalabb', NULL, NULL, NULL, N'ElsodlegesIKTEszkoz', 88, N'F', N'T', 2, NULL, 2, NULL),
            (5419, N'Notebook - három évnél idősebb', N'Notebook - három évnél idősebb', NULL, NULL, NULL, N'ElsodlegesIKTEszkoz', 88, N'F', N'T', 3, NULL, 3, NULL),
            (5420, N'Notebook - három évnél fiatalabb', N'Notebook - három évnél fiatalabb', NULL, NULL, NULL, N'ElsodlegesIKTEszkoz', 88, N'F', N'T', 4, NULL, 4, NULL),
            (5421, N'Tablet - három évnél idősebb', N'Tablet - három évnél idősebb', NULL, NULL, NULL, N'ElsodlegesIKTEszkoz', 88, N'F', N'T', 5, NULL, 5, NULL),
            (5422, N'Tablet - három évnél fiatalabb', N'Tablet - három évnél fiatalabb', NULL, NULL, NULL, N'ElsodlegesIKTEszkoz', 88, N'F', N'T', 6, NULL, 6, NULL),
            (5423, N'Egyéb', N'Egyéb', NULL, NULL, NULL, N'ElsodlegesIKTEszkoz', 88, N'F', N'T', 7, NULL, 7, NULL),
            (5424, N'Nincs', N'Nincs', NULL, NULL, NULL, N'WiFiEleres', 89, N'F', N'T', 1, NULL, 1, NULL),
            (5425, N'Nem minden teremben elérhető', N'Nem minden teremben elérhető', NULL, NULL, NULL, N'WiFiEleres', 89, N'F', N'T', 2, NULL, 2, NULL),
            (5426, N'Mindenhol elérhető', N'Mindenhol elérhető', NULL, NULL, NULL, N'WiFiEleres', 89, N'F', N'T', 3, NULL, 3, NULL),
            (5407, N'Nincs', N'Nincs', NULL, NULL, NULL, N'IKTKompetenciaSzint', 86, N'F', N'T', 1, NULL, 1, NULL),
            (5408, N'IKER1', N'IKER1', NULL, NULL, NULL, N'IKTKompetenciaSzint', 86, N'F', N'T', 2, NULL, 2, NULL),
            (5409, N'IKER2', N'IKER2', NULL, NULL, NULL, N'IKTKompetenciaSzint', 86, N'F', N'T', 3, NULL, 3, NULL),
            (5410, N'IKER3', N'IKER3', NULL, NULL, NULL, N'IKTKompetenciaSzint', 86, N'F', N'T', 4, NULL, 4, NULL),
            (5411, N'IKER4', N'IKER4', NULL, NULL, NULL, N'IKTKompetenciaSzint', 86, N'F', N'T', 5, NULL, 5, NULL),
            (5413, N'Na', N'Na', NULL, NULL, NULL, N'IKTKompetenciaSzint', 86, N'F', N'T', 6, NULL, 6, NULL),
            (5404, N'Igen, már bevezettük az e-naplót', N'Igen, már bevezettük az e-naplót', NULL, NULL, NULL, N'ENaploHasznalat', 85, N'F', N'T', 1, NULL, 1, NULL),
            (5405, N'Nem, de a 2017/2018 tanévben bevezetjük', N'Nem, de a 2017/2018 tanévben bevezetjük', NULL, NULL, NULL, N'ENaploHasznalat', 85, N'F', N'T', 2, NULL, 2, NULL),
            (5406, N'Nem és nem is tervezzük a bevezetést a 2017/2018 tanévben', N'Nem és nem is tervezzük a bevezetést a 2017/2018 tanévben', NULL, NULL, NULL, N'ENaploHasznalat', 85, N'F', N'T', 3, NULL, 3, NULL),
            (5429, N'Tervezzük, de nem a 2017/2018 tanévben', N'Tervezzük, de nem a 2017/2018 tanévben', NULL, NULL, NULL, N'ENaploHasznalat', 85, N'F', N'T', 4, NULL, 4, NULL),
            (5262, N'Na', N'Na', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 1, NULL, 1, NULL),
            (5263, N'Normál - normál', N'Normál - normál', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 2, NULL, 2, NULL),
            (5264, N'Normál - egész napos iskola', N'Normál - egész napos iskola', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 3, NULL, 3, NULL),
            (5265, N'Normál - összevont 1-2 évfolyam', N'Normál - összevont 1-2 évfolyam', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 4, NULL, 4, NULL),
            (5266, N'Normál - öszevont 1-3 évfolyam', N'Normál - öszevont 1-3 évfolyam', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 5, NULL, 5, NULL),
            (5267, N'Normál - összevont 1-4 évfolyam', N'Normál - összevont 1-4 évfolyam', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 6, NULL, 6, NULL),
            (5268, N'Gyógypedagógia - mozgásszervi fogyatékos', N'Gyógypedagógia - mozgásszervi fogyatékos', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 7, NULL, 7, NULL),
            (5269, N'Gyógypedagógia - érzékszervi fogyatékos', N'Gyógypedagógia - érzékszervi fogyatékos', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 8, NULL, 8, NULL),
            (5270, N'Gyógypedagógia - értelmi fogyatékos', N'Gyógypedagógia - értelmi fogyatékos', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 9, NULL, 9, NULL),
            (5271, N'Gyógypedagógia - beszédfogyatékos', N'Gyógypedagógia - beszédfogyatékos', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 10, NULL, 10, NULL),
            (5272, N'Gyógypedagógia - halmozottan fogyatékos', N'Gyógypedagógia - halmozottan fogyatékos', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 11, NULL, 11, NULL),
            (5273, N'Gyógypedagógia - autizmus spektrumzavar', N'Gyógypedagógia - autizmus spektrumzavar', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 12, NULL, 12, NULL),
            (5274, N'Gyógypedagógia - egyéb pszichés fejlődés zavarral küzdők', N'Gyógypedagógia - egyéb pszichés fejlődés zavarral küzdők', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 13, NULL, 13, NULL),
            (5275, N'Anyanyelvű nemzetiségi nevelés-oktatás - bolgár', N'Anyanyelvű nemzetiségi nevelés-oktatás - bolgár', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 14, NULL, 14, NULL),
            (5276, N'Anyanyelvű nemzetiségi nevelés-oktatás - görög', N'Anyanyelvű nemzetiségi nevelés-oktatás - görög', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 15, NULL, 15, NULL),
            (5277, N'Anyanyelvű nemzetiségi nevelés-oktatás - horvát', N'Anyanyelvű nemzetiségi nevelés-oktatás - horvát', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 16, NULL, 16, NULL),
            (5278, N'Anyanyelvű nemzetiségi nevelés-oktatás - lengyel', N'Anyanyelvű nemzetiségi nevelés-oktatás - lengyel', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 17, NULL, 17, NULL),
            (5279, N'Anyanyelvű nemzetiségi nevelés-oktatás - német', N'Anyanyelvű nemzetiségi nevelés-oktatás - német', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 18, NULL, 18, NULL),
            (5280, N'Anyanyelvű nemzetiségi nevelés-oktatás - örmény', N'Anyanyelvű nemzetiségi nevelés-oktatás - örmény', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 19, NULL, 19, NULL),
            (5281, N'Anyanyelvű nemzetiségi nevelés-oktatás - romani', N'Anyanyelvű nemzetiségi nevelés-oktatás - romani', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 20, NULL, 20, NULL),
            (5282, N'Anyanyelvű nemzetiségi nevelés-oktatás - beás', N'Anyanyelvű nemzetiségi nevelés-oktatás - beás', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 21, NULL, 21, NULL),
            (5283, N'Anyanyelvű nemzetiségi nevelés-oktatás - román', N'Anyanyelvű nemzetiségi nevelés-oktatás - román', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 22, NULL, 22, NULL),
            (5284, N'Anyanyelvű nemzetiségi nevelés-oktatás - ruszin', N'Anyanyelvű nemzetiségi nevelés-oktatás - ruszin', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 23, NULL, 23, NULL),
            (5285, N'Anyanyelvű nemzetiségi nevelés-oktatás - szerb', N'Anyanyelvű nemzetiségi nevelés-oktatás - szerb', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 24, NULL, 24, NULL),
            (5286, N'Anyanyelvű nemzetiségi nevelés-oktatás - szlovák', N'Anyanyelvű nemzetiségi nevelés-oktatás - szlovák', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 25, NULL, 25, NULL),
            (5287, N'Anyanyelvű nemzetiségi nevelés-oktatás - szlovén', N'Anyanyelvű nemzetiségi nevelés-oktatás - szlovén', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 26, NULL, 26, NULL),
            (5288, N'Anyanyelvű nemzetiségi nevelés-oktatás - ukrán', N'Anyanyelvű nemzetiségi nevelés-oktatás - ukrán', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 27, NULL, 27, NULL),
            (5289, N'Anyanyelvű nemzetiségi nevelés-oktatás - magyar', N'Anyanyelvű nemzetiségi nevelés-oktatás - magyar', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 28, NULL, 28, NULL),
            (5290, N'Anyanyelvű nemzetiségi nevelés-oktatás - lovári', N'Anyanyelvű nemzetiségi nevelés-oktatás - lovári', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 29, NULL, 29, NULL),
            (5291, N'Kétnyelvű nemzetiségi nevelés-oktatás - bolgár', N'Kétnyelvű nemzetiségi nevelés-oktatás - bolgár', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 30, NULL, 30, NULL),
            (5292, N'Kétnyelvű nemzetiségi nevelés-oktatás - görög', N'Kétnyelvű nemzetiségi nevelés-oktatás - görög', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 31, NULL, 31, NULL),
            (5293, N'Kétnyelvű nemzetiségi nevelés-oktatás - horvát', N'Kétnyelvű nemzetiségi nevelés-oktatás - horvát', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 32, NULL, 32, NULL),
            (5294, N'Kétnyelvű nemzetiségi nevelés-oktatás - lengyel', N'Kétnyelvű nemzetiségi nevelés-oktatás - lengyel', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 33, NULL, 33, NULL),
            (5295, N'Kétnyelvű nemzetiségi nevelés-oktatás - német', N'Kétnyelvű nemzetiségi nevelés-oktatás - német', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 34, NULL, 34, NULL),
            (5296, N'Kétnyelvű nemzetiségi nevelés-oktatás - örmény', N'Kétnyelvű nemzetiségi nevelés-oktatás - örmény', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 35, NULL, 35, NULL),
            (5297, N'Kétnyelvű nemzetiségi nevelés-oktatás - romani', N'Kétnyelvű nemzetiségi nevelés-oktatás - romani', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 36, NULL, 36, NULL),
            (5298, N'Kétnyelvű nemzetiségi nevelés-oktatás - beás', N'Kétnyelvű nemzetiségi nevelés-oktatás - beás', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 37, NULL, 37, NULL),
            (5299, N'Kétnyelvű nemzetiségi nevelés-oktatás - román', N'Kétnyelvű nemzetiségi nevelés-oktatás - román', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 38, NULL, 38, NULL),
            (5300, N'Kétnyelvű nemzetiségi nevelés-oktatás - ruszin', N'Kétnyelvű nemzetiségi nevelés-oktatás - ruszin', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 39, NULL, 39, NULL),
            (5301, N'Kétnyelvű nemzetiségi nevelés-oktatás - szerb', N'Kétnyelvű nemzetiségi nevelés-oktatás - szerb', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 40, NULL, 40, NULL),
            (5302, N'Kétnyelvű nemzetiségi nevelés-oktatás - szlovák', N'Kétnyelvű nemzetiségi nevelés-oktatás - szlovák', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 41, NULL, 41, NULL),
            (5303, N'Kétnyelvű nemzetiségi nevelés-oktatás - szlovén', N'Kétnyelvű nemzetiségi nevelés-oktatás - szlovén', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 42, NULL, 42, NULL),
            (5304, N'Kétnyelvű nemzetiségi nevelés-oktatás - ukrán', N'Kétnyelvű nemzetiségi nevelés-oktatás - ukrán', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 43, NULL, 43, NULL),
            (5305, N'Kétnyelvű nemzetiségi nevelés-oktatás - magyar', N'Kétnyelvű nemzetiségi nevelés-oktatás - magyar', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 44, NULL, 44, NULL),
            (5306, N'Kétnyelvű nemzetiségi nevelés-oktatás - lovári', N'Kétnyelvű nemzetiségi nevelés-oktatás - lovári', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 45, NULL, 45, NULL),
            (5307, N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - bolgár', N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - bolgár', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 46, NULL, 46, NULL),
            (5308, N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - görög', N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - görög', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 47, NULL, 47, NULL),
            (5309, N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - horvát', N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - horvát', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 48, NULL, 48, NULL),
            (5310, N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - lengyel', N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - lengyel', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 49, NULL, 49, NULL),
            (5311, N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - német', N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - német', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 50, NULL, 50, NULL),
            (5312, N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - örmény', N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - örmény', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 51, NULL, 51, NULL),
            (5313, N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - romani', N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - romani', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 52, NULL, 52, NULL),
            (5314, N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - beás', N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - beás', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 53, NULL, 53, NULL),
            (5315, N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - román', N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - román', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 54, NULL, 54, NULL),
            (5316, N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - ruszin', N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - ruszin', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 55, NULL, 55, NULL),
            (5317, N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - szerb', N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - szerb', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 56, NULL, 56, NULL),
            (5318, N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - szlovák', N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - szlovák', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 57, NULL, 57, NULL),
            (5319, N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - szlovén', N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - szlovén', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 58, NULL, 58, NULL),
            (5320, N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - ukrán', N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - ukrán', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 59, NULL, 59, NULL),
            (5321, N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - magyar', N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - magyar', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 60, NULL, 60, NULL),
            (5322, N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - lovári', N'Nyelvoktató nemzetiségi nevelés-oktatás (hagyományos) - lovári', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 61, NULL, 61, NULL),
            (5323, N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - bolgár', N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - bolgár', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 62, NULL, 62, NULL),
            (5324, N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - görög', N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - görög', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 63, NULL, 63, NULL),
            (5325, N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - horvát', N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - horvát', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 64, NULL, 64, NULL),
            (5326, N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - lengyel', N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - lengyel', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 65, NULL, 65, NULL),
            (5327, N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - német', N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - német', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 66, NULL, 66, NULL),
            (5328, N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - örmény', N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - örmény', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 67, NULL, 67, NULL),
            (5329, N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - romani', N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - romani', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 68, NULL, 68, NULL),
            (5330, N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - beás', N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - beás', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 69, NULL, 69, NULL),
            (5331, N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - román', N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - román', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 70, NULL, 70, NULL),
            (5332, N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - ruszin', N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - ruszin', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 71, NULL, 71, NULL),
            (5333, N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - szerb', N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - szerb', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 72, NULL, 72, NULL),
            (5334, N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - szlovák', N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - szlovák', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 73, NULL, 73, NULL),
            (5335, N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - szlovén', N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - szlovén', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 74, NULL, 74, NULL),
            (5336, N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - ukrán', N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - ukrán', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 75, NULL, 75, NULL),
            (5337, N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - magyar', N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - magyar', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 76, NULL, 76, NULL),
            (5338, N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - lovári', N'Nyelvoktató nemzetiségi nevelés-oktatás (bővített) - lovári', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 77, NULL, 77, NULL),
            (5339, N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - bolgár', N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - bolgár', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 78, NULL, 78, NULL),
            (5340, N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - görög', N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - görög', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 79, NULL, 79, NULL),
            (5341, N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - horvát', N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - horvát', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 80, NULL, 80, NULL),
            (5342, N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - lengyel', N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - lengyel', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 81, NULL, 81, NULL),
            (5343, N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - német', N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - német', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 82, NULL, 82, NULL),
            (5344, N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - örmény', N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - örmény', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 83, NULL, 83, NULL),
            (5345, N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - romani', N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - romani', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 84, NULL, 84, NULL),
            (5346, N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - beás', N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - beás', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 85, NULL, 85, NULL),
            (5347, N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - román', N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - román', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 86, NULL, 86, NULL),
            (5348, N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - ruszin', N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - ruszin', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 87, NULL, 87, NULL),
            (5349, N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - szerb', N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - szerb', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 88, NULL, 88, NULL),
            (5350, N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - szlovák', N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - szlovák', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 89, NULL, 89, NULL),
            (5351, N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - szlovén', N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - szlovén', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 90, NULL, 90, NULL),
            (5352, N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - ukrán', N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - ukrán', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 91, NULL, 91, NULL),
            (5353, N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - magyar', N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - magyar', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 92, NULL, 92, NULL),
            (5354, N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - lovári', N'Magyar nyelvű roma/cigány nemzetiségi nevelés-oktatás - lovári', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 93, NULL, 93, NULL),
            (5355, N'Kiegészítő nemzetiségi nevelés-oktatás - bolgár', N'Kiegészítő nemzetiségi nevelés-oktatás - bolgár', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 94, NULL, 94, NULL),
            (5356, N'Kiegészítő nemzetiségi nevelés-oktatás - görög', N'Kiegészítő nemzetiségi nevelés-oktatás - görög', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 95, NULL, 95, NULL),
            (5357, N'Kiegészítő nemzetiségi nevelés-oktatás - horvát', N'Kiegészítő nemzetiségi nevelés-oktatás - horvát', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 96, NULL, 96, NULL),
            (5358, N'Kiegészítő nemzetiségi nevelés-oktatás - lengyel', N'Kiegészítő nemzetiségi nevelés-oktatás - lengyel', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 97, NULL, 97, NULL),
            (5359, N'Kiegészítő nemzetiségi nevelés-oktatás - német', N'Kiegészítő nemzetiségi nevelés-oktatás - német', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 98, NULL, 98, NULL),
            (5360, N'Kiegészítő nemzetiségi nevelés-oktatás - örmény', N'Kiegészítő nemzetiségi nevelés-oktatás - örmény', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 99, NULL, 99, NULL),
            (5361, N'Kiegészítő nemzetiségi nevelés-oktatás - romani', N'Kiegészítő nemzetiségi nevelés-oktatás - romani', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 100, NULL, 100, NULL),
            (5362, N'Kiegészítő nemzetiségi nevelés-oktatás - beás', N'Kiegészítő nemzetiségi nevelés-oktatás - beás', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 101, NULL, 101, NULL),
            (5363, N'Kiegészítő nemzetiségi nevelés-oktatás - román', N'Kiegészítő nemzetiségi nevelés-oktatás - román', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 102, NULL, 102, NULL),
            (5364, N'Kiegészítő nemzetiségi nevelés-oktatás - ruszin', N'Kiegészítő nemzetiségi nevelés-oktatás - ruszin', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 103, NULL, 103, NULL),
            (5365, N'Kiegészítő nemzetiségi nevelés-oktatás - szerb', N'Kiegészítő nemzetiségi nevelés-oktatás - szerb', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 104, NULL, 104, NULL),
            (5366, N'Kiegészítő nemzetiségi nevelés-oktatás - szlovák', N'Kiegészítő nemzetiségi nevelés-oktatás - szlovák', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 105, NULL, 105, NULL),
            (5367, N'Kiegészítő nemzetiségi nevelés-oktatás - szlovén', N'Kiegészítő nemzetiségi nevelés-oktatás - szlovén', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 106, NULL, 106, NULL),
            (5368, N'Kiegészítő nemzetiségi nevelés-oktatás - ukrán', N'Kiegészítő nemzetiségi nevelés-oktatás - ukrán', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 107, NULL, 107, NULL),
            (5369, N'Kiegészítő nemzetiségi nevelés-oktatás - magyar', N'Kiegészítő nemzetiségi nevelés-oktatás - magyar', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 108, NULL, 108, NULL),
            (5370, N'Kiegészítő nemzetiségi nevelés-oktatás - lovári', N'Kiegészítő nemzetiségi nevelés-oktatás - lovári', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 109, NULL, 109, NULL),
            (5371, N'Kéttanítási nyelvű nevelés-oktatás - angol', N'Kéttanítási nyelvű nevelés-oktatás - angol', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 110, NULL, 110, NULL),
            (5372, N'Kéttanítási nyelvű nevelés-oktatás - francia', N'Kéttanítási nyelvű nevelés-oktatás - francia', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 111, NULL, 111, NULL),
            (5373, N'Kéttanítási nyelvű nevelés-oktatás - német', N'Kéttanítási nyelvű nevelés-oktatás - német', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 112, NULL, 112, NULL),
            (5374, N'Kéttanítási nyelvű nevelés-oktatás - olasz', N'Kéttanítási nyelvű nevelés-oktatás - olasz', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 113, NULL, 113, NULL),
            (5375, N'Kéttanítási nyelvű nevelés-oktatás - spanyol', N'Kéttanítási nyelvű nevelés-oktatás - spanyol', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 114, NULL, 114, NULL),
            (5376, N'Kéttanítási nyelvű nevelés-oktatás - héber', N'Kéttanítási nyelvű nevelés-oktatás - héber', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 115, NULL, 115, NULL),
            (5377, N'Kéttanítási nyelvű nevelés-oktatás - horvát', N'Kéttanítási nyelvű nevelés-oktatás - horvát', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 116, NULL, 116, NULL),
            (5378, N'Kéttanítási nyelvű nevelés-oktatás - kínai', N'Kéttanítási nyelvű nevelés-oktatás - kínai', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 117, NULL, 117, NULL),
            (5379, N'Kéttanítási nyelvű nevelés-oktatás - lengyel', N'Kéttanítási nyelvű nevelés-oktatás - lengyel', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 118, NULL, 118, NULL),
            (5380, N'Kéttanítási nyelvű nevelés-oktatás - orosz', N'Kéttanítási nyelvű nevelés-oktatás - orosz', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 119, NULL, 119, NULL),
            (5381, N'Kéttanítási nyelvű nevelés-oktatás - román', N'Kéttanítási nyelvű nevelés-oktatás - román', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 120, NULL, 120, NULL),
            (5382, N'Kéttanítási nyelvű nevelés-oktatás - szlovák', N'Kéttanítási nyelvű nevelés-oktatás - szlovák', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 121, NULL, 121, NULL),
            (5383, N'Kéttanítási nyelvű nevelés-oktatás - szlovén', N'Kéttanítási nyelvű nevelés-oktatás - szlovén', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 122, NULL, 122, NULL),
            (5384, N'Alternatív - Meixner', N'Alternatív - Meixner', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 123, NULL, 123, NULL),
            (5385, N'Alternatív - Montessori', N'Alternatív - Montessori', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 124, NULL, 124, NULL),
            (5386, N'Alternatív - Rogers', N'Alternatív - Rogers', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 125, NULL, 125, NULL),
            (5387, N'Alternatív - Zsolnai', N'Alternatív - Zsolnai', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 126, NULL, 126, NULL),
            (5388, N'Alternatív - egyéb', N'Alternatív - egyéb', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 127, NULL, 127, NULL),
            (5389, N'Emelt szintű - magyar nyelv és irodalom', N'Emelt szintű - magyar nyelv és irodalom', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 128, NULL, 128, NULL),
            (5390, N'Emelt szintű - idegen nyelv', N'Emelt szintű - idegen nyelv', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 129, NULL, 129, NULL),
            (5391, N'Emelt szintű - matematika', N'Emelt szintű - matematika', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 130, NULL, 130, NULL),
            (5392, N'Emelt szintű - erkölcstan', N'Emelt szintű - erkölcstan', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 131, NULL, 131, NULL),
            (5393, N'Emelt szintű - környezetismeret', N'Emelt szintű - környezetismeret', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 132, NULL, 132, NULL),
            (5394, N'Emelt szintű - ének-zene', N'Emelt szintű - ének-zene', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 133, NULL, 133, NULL),
            (5395, N'Emelt szintű - vizuális kultúra', N'Emelt szintű - vizuális kultúra', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 134, NULL, 134, NULL),
            (5396, N'Emelt szintű - technika, életvitel és gyakorlat', N'Emelt szintű - technika, életvitel és gyakorlat', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 135, NULL, 135, NULL),
            (5397, N'Emelt szintű - testnevelés és sport', N'Emelt szintű - testnevelés és sport', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 136, NULL, 136, NULL),
            (5398, N'Emelt szintű - informatika', N'Emelt szintű - informatika', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 137, NULL, 137, NULL),
            (5399, N'Emelt szintű - sportiskolai nevelés-oktatás', N'Emelt szintű - sportiskolai nevelés-oktatás', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 138, NULL, 138, NULL),
            (5400, N'Emelt szintű - egyéb', N'Emelt szintű - egyéb', NULL, NULL, NULL, N'TagozatTipus', 83, N'F', N'T', 139, NULL, 139, NULL),
            (5401, N'Felvéve', N'Felvéve', NULL, NULL, NULL, N'FelvetelStatuszaTipus', 84, N'F', N'T', 1, NULL, 1, NULL),
            (5402, N'Nem felvéve', N'Nem felvéve', NULL, NULL, NULL, N'FelvetelStatuszaTipus', 84, N'F', N'T', 2, NULL, 2, NULL),
            (5403, N'Függőben', N'Függőben', NULL, NULL, NULL, N'FelvetelStatuszaTipus', 84, N'F', N'T', 3, NULL, 3, NULL),
            (5427, N'Szakértői vizsg. folyamatban', N'Szakértői vizsg. folyamatban', NULL, NULL, NULL, N'FelvetelStatuszaTipus', 84, N'F', N'T', 4, NULL, 4, NULL),
            (5428, N'Nem jelent meg', N'Nem jelent meg', NULL, NULL, NULL, N'FelvetelStatuszaTipus', 84, N'F', N'T', 5, NULL, 5, NULL),
            (5430, N'Óvodában maradt', N'Óvodában maradt', NULL, NULL, NULL, N'FelvetelStatuszaTipus', 84, N'F', N'T', 6, NULL, 6, NULL),
            (5434, N'Más intézménybe felvéve', N'Más intézménybe felvéve', NULL, NULL, NULL, N'FelvetelStatuszaTipus', 84, N'F', N'T', 7, NULL, 7, NULL),
            (7797, N'Beiratkozás visszavonva', N'Beiratkozás visszavonva', NULL, NULL, NULL, N'FelvetelStatuszaTipus', 84, N'F', N'T', 8, NULL, 8, NULL),
            (5246, N'Havi', N'Havi', NULL, NULL, NULL, N'Idokeret', 80, N'F', N'T', 1, NULL, 1, NULL),
            (5247, N'Negyedéves', N'Negyedéves', NULL, NULL, NULL, N'Idokeret', 80, N'F', N'T', 2, NULL, 2, NULL),
            (5248, N'Féléves', N'Féléves', NULL, NULL, NULL, N'Idokeret', 80, N'F', N'T', 3, NULL, 3, NULL),
            (5249, N'Éves', N'Éves', NULL, NULL, NULL, N'Idokeret', 80, N'F', N'T', 4, NULL, 4, NULL),
            (5250, N'Nincs', N'Nincs', NULL, NULL, NULL, N'OsszegTullepesTipus', 81, N'F', N'T', 1, NULL, 1, NULL),
            (5251, N'Százalék', N'Százalék', NULL, NULL, NULL, N'OsszegTullepesTipus', 81, N'F', N'T', 2, NULL, 2, NULL),
            (5252, N'Összeg', N'Összeg', NULL, NULL, NULL, N'OsszegTullepesTipus', 81, N'F', N'T', 3, NULL, 3, NULL),
            (5229, N'Új', N'Új', NULL, NULL, NULL, N'HibajegyStatusz', 77, N'F', N'T', 1, NULL, 1, NULL),
            (5230, N'Feldolgozás alatt', N'Feldolgozás alatt', NULL, NULL, NULL, N'HibajegyStatusz', 77, N'F', N'T', 2, NULL, 2, NULL),
            (5231, N'Hibajavítás alatt', N'Hibajavítás alatt', NULL, NULL, NULL, N'HibajegyStatusz', 77, N'F', N'T', 3, NULL, 3, NULL),
            (5232, N'Fejlesztés alatt', N'Fejlesztés alatt', NULL, NULL, NULL, N'HibajegyStatusz', 77, N'F', N'T', 4, NULL, 4, NULL),
            (5233, N'Folyamatban', N'Folyamatban', NULL, NULL, NULL, N'HibajegyStatusz', 77, N'F', N'T', 5, NULL, 5, NULL),
            (5234, N'Lezárt', N'Lezárt', NULL, NULL, NULL, N'HibajegyStatusz', 77, N'F', N'T', 6, NULL, 6, NULL),
            (5235, N'Visszajelzésre vár', N'Visszajelzésre vár', NULL, NULL, NULL, N'HibajegyStatusz', 77, N'F', N'T', 7, NULL, 7, NULL),
            (5236, N'Visszavont', N'Visszavont', NULL, NULL, NULL, N'HibajegyStatusz', 77, N'F', N'T', 8, NULL, 8, NULL),
            (5237, N'Alacsony', N'Alacsony', NULL, NULL, NULL, N'Prioritas', 78, N'F', N'T', 1, NULL, 1, NULL),
            (5238, N'Közepes', N'Közepes', NULL, NULL, NULL, N'Prioritas', 78, N'F', N'T', 2, NULL, 2, NULL),
            (5239, N'Magas', N'Magas', NULL, NULL, NULL, N'Prioritas', 78, N'F', N'T', 3, NULL, 3, NULL),
            (5240, N'Kritikus', N'Kritikus', NULL, NULL, NULL, N'Prioritas', 78, N'F', N'T', 4, NULL, 4, NULL),
            (5241, N'Információ', N'Információ', NULL, NULL, NULL, N'Prioritas', 78, N'F', N'T', 5, NULL, 5, NULL),
            (5242, N'Telefon', N'Telefon', NULL, NULL, NULL, N'Forras', 79, N'F', N'T', 1, NULL, 1, NULL),
            (5243, N'Email', N'Email', NULL, NULL, NULL, N'Forras', 79, N'F', N'T', 2, NULL, 2, NULL),
            (5244, N'Hibajegykezelő', N'Hibajegykezelő', NULL, NULL, NULL, N'Forras', 79, N'F', N'T', 3, NULL, 3, NULL),
            (5245, N'Belső teszt', N'Belső teszt', NULL, NULL, NULL, N'Forras', 79, N'F', N'T', 4, NULL, 4, NULL),
            (3139, N'Intézményvezető', N'Intézményvezető', NULL, NULL, NULL, N'SzerzodoSzemelyPozicio', 76, N'F', N'T', 1, NULL, 1, NULL),
            (3140, N'Tagintézmény-vezető', N'Tagintézmény-vezető', NULL, NULL, NULL, N'SzerzodoSzemelyPozicio', 76, N'F', N'T', 2, NULL, 2, NULL),
            (3141, N'Intézményvezető-helyettes', N'Intézményvezető-helyettes', NULL, NULL, NULL, N'SzerzodoSzemelyPozicio', 76, N'F', N'T', 3, NULL, 3, NULL),
            (3142, N'Tagintézmény-vezető helyettes', N'Tagintézmény-vezető helyettes', NULL, NULL, NULL, N'SzerzodoSzemelyPozicio', 76, N'F', N'T', 4, NULL, 4, NULL),
            (3143, N'Intézményi gazdasági munkatárs', N'Intézményi gazdasági munkatárs', NULL, NULL, NULL, N'SzerzodoSzemelyPozicio', 76, N'F', N'T', 5, NULL, 5, NULL),
            (3144, N'Egyéb intézményi munkatárs', N'Egyéb intézményi munkatárs', NULL, NULL, NULL, N'SzerzodoSzemelyPozicio', 76, N'F', N'T', 6, NULL, 6, NULL),
            (3145, N'Tankerületi megbízott', N'Tankerületi megbízott', NULL, NULL, NULL, N'SzerzodoSzemelyPozicio', 76, N'F', N'T', 7, NULL, 7, NULL),
            (3146, N'Tankerületi igazgató', N'Tankerületi igazgató', NULL, NULL, NULL, N'SzerzodoSzemelyPozicio', 76, N'F', N'T', 8, NULL, 8, NULL),
            (3147, N'Egyéb tankerületi munkatárs', N'Egyéb tankerületi munkatárs', NULL, NULL, NULL, N'SzerzodoSzemelyPozicio', 76, N'F', N'T', 9, NULL, 9, NULL),
            (3148, N'Alapítványvezető', N'Alapítványvezető', NULL, NULL, NULL, N'SzerzodoSzemelyPozicio', 76, N'F', N'T', 10, NULL, 10, NULL),
            (3149, N'Szülői munkaközösség elnöke', N'Szülői munkaközösség elnöke', NULL, NULL, NULL, N'SzerzodoSzemelyPozicio', 76, N'F', N'T', 11, NULL, 11, NULL),
            (3150, N'Egyéb személy', N'Egyéb személy', NULL, NULL, NULL, N'SzerzodoSzemelyPozicio', 76, N'F', N'T', 12, NULL, 12, NULL),
            (3115, N'KRÉTA e-napló', N'KRÉTA e-napló', NULL, NULL, NULL, N'ENaploTipus', 73, N'F', N'T', 1, NULL, 1, NULL),
            (3116, N'NEPTUN-Napló (@Napló)', N'NEPTUN-Napló (@Napló)', NULL, NULL, NULL, N'ENaploTipus', 73, N'F', N'T', 2, NULL, 2, NULL),
            (3117, N'Taninform', N'Taninform', NULL, NULL, NULL, N'ENaploTipus', 73, N'F', N'T', 3, NULL, 3, NULL),
            (3118, N'Magiszter', N'Magiszter', NULL, NULL, NULL, N'ENaploTipus', 73, N'F', N'T', 4, NULL, 4, NULL),
            (3119, N'Aromo', N'Aromo', NULL, NULL, NULL, N'ENaploTipus', 73, N'F', N'T', 5, NULL, 5, NULL),
            (3120, N'FÖKIR', N'FÖKIR', NULL, NULL, NULL, N'ENaploTipus', 73, N'F', N'T', 6, NULL, 6, NULL),
            (3121, N'Mayor', N'Mayor', NULL, NULL, NULL, N'ENaploTipus', 73, N'F', N'T', 7, NULL, 7, NULL),
            (3122, N'mozaNapló', N'mozaNapló', NULL, NULL, NULL, N'ENaploTipus', 73, N'F', N'T', 8, NULL, 8, NULL),
            (3123, N'Digitnapló', N'Digitnapló', NULL, NULL, NULL, N'ENaploTipus', 73, N'F', N'T', 9, NULL, 9, NULL),
            (3124, N'F2', N'F2', NULL, NULL, NULL, N'ENaploTipus', 73, N'F', N'T', 10, NULL, 10, NULL),
            (3125, N'SNW enapló', N'SNW enapló', NULL, NULL, NULL, N'ENaploTipus', 73, N'F', N'T', 11, NULL, 11, NULL),
            (3126, N'Petersoft iN@apló', N'Petersoft iN@apló', NULL, NULL, NULL, N'ENaploTipus', 73, N'F', N'T', 12, NULL, 12, NULL),
            (3127, N'DINA napló', N'DINA napló', NULL, NULL, NULL, N'ENaploTipus', 73, N'F', N'T', 13, NULL, 13, NULL),
            (3128, N'7bit IAR', N'7bit IAR', NULL, NULL, NULL, N'ENaploTipus', 73, N'F', N'T', 14, NULL, 14, NULL),
            (3129, N'Saját fejlesztés', N'Saját fejlesztés', NULL, NULL, NULL, N'ENaploTipus', 73, N'F', N'T', 15, NULL, 15, NULL),
            (3130, N'Egyéb', N'Egyéb', NULL, NULL, NULL, N'ENaploTipus', 73, N'F', N'T', 16, NULL, 16, NULL),
            (3135, N'Fenntartói forrás', N'Fenntartói forrás', NULL, NULL, NULL, N'ENaploForras', 75, N'F', N'T', 1, NULL, 1, NULL),
            (3136, N'Alapítványi forrás', N'Alapítványi forrás', NULL, NULL, NULL, N'ENaploForras', 75, N'F', N'T', 2, NULL, 2, NULL),
            (3137, N'Szülői forrás', N'Szülői forrás', NULL, NULL, NULL, N'ENaploForras', 75, N'F', N'T', 3, NULL, 3, NULL),
            (3138, N'Egyéb', N'Egyéb', NULL, NULL, NULL, N'ENaploForras', 75, N'F', N'T', 4, NULL, 4, NULL),
            (3131, N'Intézmény saját szerverén', N'Intézmény saját szerverén', NULL, NULL, NULL, N'ENaploInfra', 74, N'F', N'T', 1, NULL, 1, NULL),
            (3132, N'Alapítványi szerveren', N'Alapítványi szerveren', NULL, NULL, NULL, N'ENaploInfra', 74, N'F', N'T', 2, NULL, 2, NULL),
            (3133, N'Szolgáltató szerverén', N'Szolgáltató szerverén', NULL, NULL, NULL, N'ENaploInfra', 74, N'F', N'T', 3, NULL, 3, NULL),
            (3134, N'Egyéb', N'Egyéb', NULL, NULL, NULL, N'ENaploInfra', 74, N'F', N'T', 4, NULL, 4, NULL),
            (3085, N'Tanulók száma', N'Tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 1, NULL, 1, NULL),
            (3086, N'Tanév kezdete óta távozott tanulók száma', N'Tanév kezdete óta távozott tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 2, NULL, 2, NULL),
            (3087, N'Tantárgyi átlag miatt veszélyeztetett tanulók száma', N'Tantárgyi átlag miatt veszélyeztetett tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 3, NULL, 3, NULL),
            (3088, N'Félévi jegyek miatt veszélyeztetett tanulók száma', N'Félévi jegyek miatt veszélyeztetett tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 4, NULL, 4, NULL),
            (3089, N'Év végi jegyek miatt veszélyeztetett tanulók száma', N'Év végi jegyek miatt veszélyeztetett tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 5, NULL, 5, NULL),
            (3090, N'Mulasztások miatt veszélyeztetett tanulók száma', N'Mulasztások miatt veszélyeztetett tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 6, NULL, 6, NULL),
            (3091, N'Tantárgyi mulasztások miatt veszélyeztetett tanulók száma', N'Tantárgyi mulasztások miatt veszélyeztetett tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 7, NULL, 7, NULL),
            (3092, N'Tantárgyi mulasztások miatt előreláthatóan veszélyeztetett tanulók száma', N'Tantárgyi mulasztások miatt előreláthatóan veszélyeztetett tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 8, NULL, 8, NULL),
            (3093, N'Igazolatlan mulasztások miatt veszélyeztetett tanulók száma', N'Igazolatlan mulasztások miatt veszélyeztetett tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 9, NULL, 9, NULL),
            (3094, N'Igazolatlan mulasztások miatt előreláthatóan veszélyeztetett tanulók száma', N'Igazolatlan mulasztások miatt előreláthatóan veszélyeztetett tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 10, NULL, 10, NULL),
            (3095, N'Alacsony magatartásértékelésű tanulók száma', N'Alacsony magatartásértékelésű tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 11, NULL, 11, NULL),
            (3096, N'Alacsony szorgalomértékelésű tanulók száma', N'Alacsony szorgalomértékelésű tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 12, NULL, 12, NULL),
            (3097, N'Évismétlő tanulók száma', N'Évismétlő tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 13, NULL, 13, NULL),
            (3098, N'Magántanulók száma', N'Magántanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 14, NULL, 14, NULL),
            (3099, N'Magántanulók száma saját döntés alapján', N'Magántanulók száma saját döntés alapján', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 15, NULL, 15, NULL),
            (3100, N'Magántanulóvá vált tanulók száma', N'Magántanulóvá vált tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 16, NULL, 16, NULL),
            (3101, N'Magántanulóvá vált tanulók száma saját döntés alapján', N'Magántanulóvá vált tanulók száma saját döntés alapján', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 17, NULL, 17, NULL),
            (3102, N'Menedékjoggal rendelkező tanulók száma', N'Menedékjoggal rendelkező tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 18, NULL, 18, NULL),
            (3103, N'Jogviszonyt szüneteltető tanulók száma', N'Jogviszonyt szüneteltető tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 19, NULL, 19, NULL),
            (3104, N'Intézeti, állami nevelt tanulók száma', N'Intézeti, állami nevelt tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 20, NULL, 20, NULL),
            (3105, N'Veszélyeztetett tanulók száma', N'Veszélyeztetett tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 21, NULL, 21, NULL),
            (3106, N'Sajátos nevelési igényű (SNI) tanulók száma', N'Sajátos nevelési igényű (SNI) tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 22, NULL, 22, NULL),
            (3107, N'Beilleszkedési nehézségekkel rendelkező tanulók száma', N'Beilleszkedési nehézségekkel rendelkező tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 23, NULL, 23, NULL),
            (3108, N'Magatartási problémával küzdő tanulók száma', N'Magatartási problémával küzdő tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 24, NULL, 24, NULL),
            (3109, N'Tanulási problémával küzdő tanulók száma', N'Tanulási problémával küzdő tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 25, NULL, 25, NULL),
            (3110, N'Hátrányos helyzetű tanulók száma', N'Hátrányos helyzetű tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 26, NULL, 26, NULL),
            (3111, N'Halmozottan hátrányos helyzetű tanulók száma', N'Halmozottan hátrányos helyzetű tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 27, NULL, 27, NULL),
            (3112, N'Rendszeres gyermekvédelmi kedvezményben részesülő tanulók száma', N'Rendszeres gyermekvédelmi kedvezményben részesülő tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 28, NULL, 28, NULL),
            (3113, N'Alapfokú oktatásban résztvevő, tizenhatodik életévét betöltött tanulók száma', N'Alapfokú oktatásban résztvevő, tizenhatodik életévét betöltött tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 29, NULL, 29, NULL),
            (3114, N'A középfokú oktatásba tizenhatodik életévük betöltését követően belépett tanulók száma', N'A középfokú oktatásba tizenhatodik életévük betöltését követően belépett tanulók száma', NULL, NULL, NULL, N'ESLAdatTipus', 72, N'F', N'T', 30, NULL, 30, NULL),
            (1418, N'[na]', N'[na]', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 1, NULL, 1, NULL),
            (1419, N'Felvéve [átvéve, a(z) ... számú határozattal áthelyezve] a(z) (iskola címe) ... iskolába.', N'Felvéve [átvéve, a(z) ... számú határozattal áthelyezve] a(z) (iskola címe) ... iskolába.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 10, NULL, 2, NULL),
            (1420, N'A(z) ... számú fordítással hitelesített bizonyítvány alapján tanulmányait a(z) (betűvel) ... évfolyamon folytatja.', N'A(z) ... számú fordítással hitelesített bizonyítvány alapján tanulmányait a(z) (betűvel) ... évfolyamon folytatja.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 20, NULL, 3, NULL),
            (1421, N'Felvette a(z) (iskola címe) ... iskola.', N'Felvette a(z) (iskola címe) ... iskola.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 30, NULL, 4, NULL),
            (1422, N'Tanulmányait évfolyamismétléssel kezdheti meg, vagy osztályozó vizsga letételével folytathatja.', N'Tanulmányait évfolyamismétléssel kezdheti meg, vagy osztályozó vizsga letételével folytathatja.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 40, NULL, 5, NULL),
            (1423, N'... tantárgyból tanulmányait egyéni továbbhaladás szerint végzi.', N'... tantárgyból tanulmányait egyéni továbbhaladás szerint végzi.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 50, NULL, 6, NULL),
            (1424, N'Mentesítve ... tantárgyból az értékelés és a minősítés alól.', N'Mentesítve ... tantárgyból az értékelés és a minősítés alól.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 60, NULL, 7, NULL),
            (1425, N'... tantárgy ... évfolyamainak követelményeit egy tanévben teljesítette a következők szerint: …', N'... tantárgy ... évfolyamainak követelményeit egy tanévben teljesítette a következők szerint: …', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 70, NULL, 8, NULL),
            (1426, N'Egyes tantárgyak tanórai látogatása alól a 20.../... tanévben felmentve ... miatt. [Osztályozó vizsgát köteles tenni.]', N'Egyes tantárgyak tanórai látogatása alól a 20.../... tanévben felmentve ... miatt. [Osztályozó vizsgát köteles tenni.]', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 80, NULL, 9, NULL),
            (1427, N'Tanulmányait a szülő kérésére (szakértői vélemény alapján) egyéni munkarend keretében folytatja.', N'Tanulmányait a szülő kérésére (szakértői vélemény alapján) egyéni munkarend keretében folytatja.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 90, NULL, 10, NULL),
            (1428, N'Mentesítve a(z) [a tantárgy(ak) neve] ... tantárgy tanulása alól.', N'Mentesítve a(z) [a tantárgy(ak) neve] ... tantárgy tanulása alól.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 100, NULL, 11, NULL),
            (1429, N'Tanulmányi idejének megrövidítése miatt a(z) ... évfolyam tantárgyaiból osztályozó vizsgát köteles tenni.', N'Tanulmányi idejének megrövidítése miatt a(z) ... évfolyam tantárgyaiból osztályozó vizsgát köteles tenni.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 120, NULL, 12, NULL),
            (1430, N'A(z) évfolyamra megállapított tantervi követelményeket a tanulmányi idő megrövidítésével teljesítette.', N'A(z) évfolyamra megállapított tantervi követelményeket a tanulmányi idő megrövidítésével teljesítette.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 130, NULL, 13, NULL),
            (1431, N'A(z) ... tantárgy óráinak látogatása alól felmentve ...-tól ...-ig. [Osztályozó vizsgát köteles tenni.]', N'A(z) ... tantárgy óráinak látogatása alól felmentve ...-tól ...-ig. [Osztályozó vizsgát köteles tenni.]', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 140, NULL, 14, NULL),
            (1432, N'Mulasztása miatt nem osztályozható, a nevelőtestület határozata értelmében osztályozó vizsgát tehet.', N'Mulasztása miatt nem osztályozható, a nevelőtestület határozata értelmében osztályozó vizsgát tehet.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 150, NULL, 15, NULL),
            (1433, N'A nevelőtestület határozata: a(z) (betűvel) ... évfolyamba léphet.', N'A nevelőtestület határozata: a(z) (betűvel) ... évfolyamba léphet.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 160, NULL, 16, NULL),
            (1434, N'A nevelőtestület határozata: iskolai tanulmányait befejezte, tanulmányait a(z) (betűvel) ... évfolyamon folytathatja.', N'A nevelőtestület határozata: iskolai tanulmányait befejezte, tanulmányait a(z) (betűvel) ... évfolyamon folytathatja.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 170, NULL, 17, NULL),
            (1435, N'A tanuló a(z) ... évfolyam követelményeit egy tanítási évnél hosszabb ideig, ... hónap alatt teljesítette.', N'A tanuló a(z) ... évfolyam követelményeit egy tanítási évnél hosszabb ideig, ... hónap alatt teljesítette.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 180, NULL, 18, NULL),
            (1436, N'A(z) ... tantárgyból javítóvizsgát tehet.', N'A(z) ... tantárgyból javítóvizsgát tehet.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 190, NULL, 19, NULL),
            (1437, N'A javítóvizsgán ... tantárgyból ... osztályzatot kapott, ... évfolyamba léphet.', N'A javítóvizsgán ... tantárgyból ... osztályzatot kapott, ... évfolyamba léphet.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 200, NULL, 20, NULL),
            (1438, N'A(z) ... évfolyam követelményeit nem teljesítette, az évfolyamot meg kell ismételnie.', N'A(z) ... évfolyam követelményeit nem teljesítette, az évfolyamot meg kell ismételnie.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 210, NULL, 21, NULL),
            (1439, N'A javítóvizsgán ... tantárgyból elégtelen osztályzatot kapott. Évfolyamot ismételni köteles.', N'A javítóvizsgán ... tantárgyból elégtelen osztályzatot kapott. Évfolyamot ismételni köteles.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 220, NULL, 22, NULL),
            (1440, N'A(z) ... tantárgyból ...-án osztályozó vizsgát tett.', N'A(z) ... tantárgyból ...-án osztályozó vizsgát tett.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 230, NULL, 23, NULL),
            (1441, N'Osztályozó vizsgát tett.', N'Osztályozó vizsgát tett.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 240, NULL, 24, NULL),
            (1442, N'A(z) ... tantárgy alól ... okból felmentve.', N'A(z) ... tantárgy alól ... okból felmentve.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 250, NULL, 25, NULL),
            (1443, N'A(z) ... tanóra alól ... okból felmentve.', N'A(z) ... tanóra alól ... okból felmentve.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 260, NULL, 26, NULL),
            (1444, N'Az osztályozó (beszámoltató, különbözeti, javító-) vizsga letételére ...-ig halasztást kapott.', N'Az osztályozó (beszámoltató, különbözeti, javító-) vizsga letételére ...-ig halasztást kapott.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 270, NULL, 27, NULL),
            (1445, N'Az osztályozó (javító-) vizsgát engedéllyel a(z) ... iskolában független vizsgabizottság előtt tette le.', N'Az osztályozó (javító-) vizsgát engedéllyel a(z) ... iskolában független vizsgabizottság előtt tette le.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 280, NULL, 28, NULL),
            (1446, N'A(z) ... szakképesítés évfolyamán folytatja tanulmányait.', N'A(z) ... szakképesítés évfolyamán folytatja tanulmányait.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 290, NULL, 29, NULL),
            (1447, N'Tanulmányait ... okból megszakította, a tanulói jogviszonya ...-ig szünetel.', N'Tanulmányait ... okból megszakította, a tanulói jogviszonya ...-ig szünetel.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 300, NULL, 30, NULL),
            (1448, N'A tanuló jogviszonya kimaradással megszűnt, a létszámból törölve.', N'A tanuló jogviszonya kimaradással megszűnt, a létszámból törölve.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 310, NULL, 31, NULL),
            (1449, N'A tanuló jogviszonya ... óra igazolatlan mulasztás miatt megszűnt, a létszámból törölve.', N'A tanuló jogviszonya ... óra igazolatlan mulasztás miatt megszűnt, a létszámból törölve.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 320, NULL, 32, NULL),
            (1450, N'A tanuló jogviszonya egészségügyi alkalmasság miatt megszűnt, a létszámból törölve.', N'A tanuló jogviszonya egészségügyi alkalmasság miatt megszűnt, a létszámból törölve.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 330, NULL, 33, NULL),
            (1451, N'A tanuló jogviszonya térítési díj, tandíj fizetési hátralék miatt megszűnt, a létszámból törölve.', N'A tanuló jogviszonya térítési díj, tandíj fizetési hátralék miatt megszűnt, a létszámból törölve.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 340, NULL, 34, NULL),
            (1452, N'A tanuló jogviszonya ... iskolába való átvétel miatt megszűnt, a létszámból törölve.', N'A tanuló jogviszonya ... iskolába való átvétel miatt megszűnt, a létszámból törölve.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 350, NULL, 35, NULL),
            (1453, N'... fegyelmező intézkedésben részesült.', N'... fegyelmező intézkedésben részesült.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 360, NULL, 36, NULL),
            (1454, N'... fegyelmi büntetésben részesült. A büntetés végrehajtása ...-ig felfüggesztve.', N'... fegyelmi büntetésben részesült. A büntetés végrehajtása ...-ig felfüggesztve.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 370, NULL, 37, NULL),
            (1455, N'Tanköteles tanuló igazolatlan mulasztása esetén: A tanuló ... óra igazolatlan mulasztása miatt a szülőt felszólítottam.', N'Tanköteles tanuló igazolatlan mulasztása esetén: A tanuló ... óra igazolatlan mulasztása miatt a szülőt felszólítottam.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 380, NULL, 38, NULL),
            (1456, N'Tanköteles tanuló igazolatlan mulasztása esetén: A tanuló ismételt ... óra igazolatlan mulasztása miatt a szülő ellen szabálysértési eljárást kezdeményeztem.', N'Tanköteles tanuló igazolatlan mulasztása esetén: A tanuló ismételt ... óra igazolatlan mulasztása miatt a szülő ellen szabálysértési eljárást kezdeményeztem.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 390, NULL, 39, NULL),
            (1457, N'Tankötelezettsége megszűnt.', N'Tankötelezettsége megszűnt.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 400, NULL, 40, NULL),
            (1458, N'A ... szót (szavakat) / osztályzato(ka)t ...-ra helyesbítettem.', N'A ... szót (szavakat) / osztályzato(ka)t ...-ra helyesbítettem.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 410, NULL, 41, NULL),
            (1459, N'A bizonyítvány ... lapját téves bejegyzés miatt érvénytelenítettem.', N'A bizonyítvány ... lapját téves bejegyzés miatt érvénytelenítettem.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 420, NULL, 42, NULL),
            (1460, N'Ezt a póttörzslapot a(z) ... következtében elvesztett (megsemmisült) eredeti helyett ... adatai (adatok) alapján állítottam ki.', N'Ezt a póttörzslapot a(z) ... következtében elvesztett (megsemmisült) eredeti helyett ... adatai (adatok) alapján állítottam ki.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 430, NULL, 43, NULL),
            (1461, N'Ezt a bizonyítványmásodlatot az elveszett (megsemmisült) eredeti helyett ... adatai (adatok) alapján állítottam ki.', N'Ezt a bizonyítványmásodlatot az elveszett (megsemmisült) eredeti helyett ... adatai (adatok) alapján állítottam ki.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 440, NULL, 44, NULL),
            (1462, N'A bizonyítványt ... kérelmére a ... számú bizonyítvány alapján, téves bejegyzés miatt állítottam ki.', N'A bizonyítványt ... kérelmére a ... számú bizonyítvány alapján, téves bejegyzés miatt állítottam ki.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 450, NULL, 45, NULL),
            (1463, N'Pótbizonyítvány. Igazolom, hogy név ..., anyja neve ... a(z) ... iskola ... szak (szakmai, speciális osztály, két tanítási nyelvű osztály, tagozat) ... évfolyamát a(z) ... tanévben eredményesen elvégezte.', N'Pótbizonyítvány. Igazolom, hogy név ..., anyja neve ... a(z) ... iskola ... szak (szakmai, speciális osztály, két tanítási nyelvű osztály, tagozat) ... évfolyamát a(z) ... tanévben eredményesen elvégezte.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 460, NULL, 46, NULL),
            (1464, N'Érettségi vizsgát tehet.', N'Érettségi vizsgát tehet.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 470, NULL, 47, NULL),
            (1465, N'Gyakorlati képzésről mulasztását ...-tól ...-ig pótolhatja.', N'Gyakorlati képzésről mulasztását ...-tól ...-ig pótolhatja.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 480, NULL, 48, NULL),
            (1466, N'Beírtam a(z) ... iskola első osztályába.', N'Beírtam a(z) ... iskola első osztályába.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 490, NULL, 49, NULL),
            (1467, N'Ezt a naplót ... tanítási nappal (órával) lezártam.', N'Ezt a naplót ... tanítási nappal (órával) lezártam.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 500, NULL, 50, NULL),
            (1468, N'Ezt az osztályozó naplót ... azaz ... (betűvel) osztályozott tanulóval lezártam.', N'Ezt az osztályozó naplót ... azaz ... (betűvel) osztályozott tanulóval lezártam.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 510, NULL, 51, NULL),
            (1469, N'Igazolom, hogy a tanuló a .../... tanévben ... óra közösségi szolgálatot teljesített.', N'Igazolom, hogy a tanuló a .../... tanévben ... óra közösségi szolgálatot teljesített.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 520, NULL, 52, NULL),
            (1470, N'A tanuló teljesítette a rendes érettségi vizsga megkezdéséhez szükséges közösségi szolgálatot.', N'A tanuló teljesítette a rendes érettségi vizsga megkezdéséhez szükséges közösségi szolgálatot.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 530, NULL, 53, NULL),
            (1471, N'… (nemzetiség megnevezése) kiegészítő nemzetiségi tanulmányait a nyolcadik/tizenkettedik évfolyamon befejezte.', N'… (nemzetiség megnevezése) kiegészítő nemzetiségi tanulmányait a nyolcadik/tizenkettedik évfolyamon befejezte.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 540, NULL, 54, NULL),
            (1472, N'Az összefüggő nyári gyakorlat teljesítését követően magasabb osztályba léphet.', N'Az összefüggő nyári gyakorlat teljesítését követően magasabb osztályba léphet.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'F', 55, NULL, 55, NULL),
            (1473, N'A 10. évfolyam követelményeit nem teljesítette. Az évfolyamot a(z) ... ágazati szakközépiskola 10. évfolyamán megismételheti.', N'A 10. évfolyam követelményeit nem teljesítette. Az évfolyamot a(z) ... ágazati szakközépiskola 10. évfolyamán megismételheti.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'F', 56, NULL, 56, NULL),
            (1474, N'Az ágazati szakközépiskola 9. évfolyamán előírt összefüggő szakmai gyakorlatot a 12. évfolyam végéig teljesíteni köteles.', N'Az ágazati szakközépiskola 9. évfolyamán előírt összefüggő szakmai gyakorlatot a 12. évfolyam végéig teljesíteni köteles.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'F', 57, NULL, 57, NULL),
            (1475, N'A 20 hónapos köznevelési HÍD II. program első évének tanulmányi követelményeit teljesítette.', N'A 20 hónapos köznevelési HÍD II. program első évének tanulmányi követelményeit teljesítette.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 550, NULL, 58, NULL),
            (1476, N'A nevelőtestület határozata: a megadott évfolyamba léphet.', N'A nevelőtestület határozata: a megadott évfolyamba léphet.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'F', 59, NULL, 59, NULL),
            (1477, N'A nevelőtestület határozata: iskolai tanulmányait befejezte, tanulmányait a megadott évfolyamon folytathatja.', N'A nevelőtestület határozata: iskolai tanulmányait befejezte, tanulmányait a megadott évfolyamon folytathatja.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'F', 60, NULL, 60, NULL),
            (1479, N'Tanulmányait a megjegyzésben szereplő okból megszakította, a tanulói jogviszonya a megjegyzésben szereplő napig (dátum) szünetel.', N'Tanulmányait a megjegyzésben szereplő okból megszakította, a tanulói jogviszonya a megjegyzésben szereplő napig (dátum) szünetel.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'F', 61, NULL, 61, NULL),
            (1481, N'A tanuló jogviszonya a megjegyzésben szereplő óraszámú igazolatlan mulasztás miatt megszűnt, a létszámból törölve.', N'A tanuló jogviszonya a megjegyzésben szereplő óraszámú igazolatlan mulasztás miatt megszűnt, a létszámból törölve.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'F', 62, NULL, 62, NULL),
            (1483, N'A tanuló jogviszonya térítési díj fizetési hátralék miatt megszűnt, a létszámból törölve.', N'A tanuló jogviszonya térítési díj fizetési hátralék miatt megszűnt, a létszámból törölve.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'F', 63, NULL, 63, NULL),
            (1484, N'A tanuló jogviszonya tandíj fizetési hátralék miatt megszűnt, a létszámból törölve.', N'A tanuló jogviszonya tandíj fizetési hátralék miatt megszűnt, a létszámból törölve.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'F', 64, NULL, 64, NULL),
            (1485, N'A tanuló jogviszonya a megadott iskolába való átvétel miatt megszűnt, a létszámból törölve.', N'A tanuló jogviszonya a megadott iskolába való átvétel miatt megszűnt, a létszámból törölve.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'F', 65, NULL, 65, NULL),
            (1486, N'A megadott évfolyamot az 1993. évi LXXIX. törvény 72. §-ának (4) bekezdésében foglaltak alapján megismételte.', N'A megadott évfolyamot az 1993. évi LXXIX. törvény 72. §-ának (4) bekezdésében foglaltak alapján megismételte.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'F', 66, NULL, 66, NULL),
            (1487, N'A tanuló az adott évfolyam követelményeit egy tanítási évnél hosszabb ideig, a megjegyzésben rögzített hónap alatt teljesítette.', N'A tanuló az adott évfolyam követelményeit egy tanítási évnél hosszabb ideig, a megjegyzésben rögzített hónap alatt teljesítette.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'F', 67, NULL, 67, NULL),
            (1488, N'Tanulmányait évfolyamismétléssel kezdheti meg, illetve osztályozó vizsga letételével folytathatja', N'Tanulmányait évfolyamismétléssel kezdheti meg, illetve osztályozó vizsga letételével folytathatja', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'F', 68, NULL, 68, NULL),
            (1489, N'Az első évfolyam követelményeit nem teljesítette, munkája előkészítőnek minősül, tanulmányait az első évfolyamon folytathatja.', N'Az első évfolyam követelményeit nem teljesítette, munkája előkészítőnek minősül, tanulmányait az első évfolyamon folytathatja.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'F', 69, NULL, 69, NULL),
            (1490, N'Átvéve a megadott iskolába.', N'Átvéve a megadott iskolába.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'F', 70, NULL, 70, NULL),
            (1491, N'Téves felvitel.', N'Téves felvitel.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'F', 71, NULL, 71, NULL),
            (1492, N'Felvételt nem nyert tanulók törlése', N'Felvételt nem nyert tanulók törlése', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'F', 72, NULL, 72, NULL),
            (6498, N'Felvéve a ... kollégiumba (externátusi elhelyezésre) a(z) ... tanévre.', N'Felvéve a ... kollégiumba (externátusi elhelyezésre) a(z) ... tanévre.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 1000, NULL, 73, NULL),
            (6499, N'A kollégiumi tagsága (externátusi elhelyezése) ... miatt megszűnt.', N'A kollégiumi tagsága (externátusi elhelyezése) ... miatt megszűnt.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 1010, NULL, 74, NULL),
            (6501, N'... fegyelmi büntetésben részesült.', N'... fegyelmi büntetésben részesült.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'T', N'T', 1030, NULL, 75, NULL),
            (8416, N'Felvéve [átvéve, a(z) … számú határozattal áthelyezve] a(z) (szakképző intézmény címe) … szakképző intézménybe.', N'Felvéve [átvéve, a(z) … számú határozattal áthelyezve] a(z) (szakképző intézmény címe) … szakképző intézménybe.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2000, NULL, 76, NULL),
            (8417, N'Felvette a(z) (szakképző intézmény címe) … szakképző intézmény.', N'Felvette a(z) (szakképző intézmény címe) … szakképző intézmény.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2010, NULL, 77, NULL),
            (8418, N'… tantárgyból tanulmányait egyéni előrehaladás szerint végzi.', N'… tantárgyból tanulmányait egyéni előrehaladás szerint végzi.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2020, NULL, 78, NULL),
            (8419, N'Egyes tantárgyak látogatása alól a …/… tanévben felmentve … miatt. Kiegészülhet: Osztályozó vizsgát köteles tenni.', N'Egyes tantárgyak látogatása alól a …/… tanévben felmentve … miatt. Kiegészülhet: Osztályozó vizsgát köteles tenni.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2030, NULL, 79, NULL),
            (8420, N'Tanulmányait egyéni tanulmányi rendben folytatja.', N'Tanulmányait egyéni tanulmányi rendben folytatja.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2040, NULL, 80, NULL),
            (8421, N'Mentesítve a(z) [a tantárgy(ak) neve] … tantárgy tanulása alól. A mentesítés oka: …', N'Mentesítve a(z) [a tantárgy(ak) neve] … tantárgy tanulása alól. A mentesítés oka: …', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2050, NULL, 81, NULL),
            (8422, N'A(z) ... tantárgy látogatása alól felmentve ...-tól ...-ig. [Osztályozó vizsgát köteles tenni.]', N'A(z) ... tantárgy látogatása alól felmentve ...-tól ...-ig. [Osztályozó vizsgát köteles tenni.]', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2060, NULL, 82, NULL),
            (8423, N'Mulasztása miatt nem osztályozható, az oktatói testület határozata értelmében osztályozó vizsgát tehet.', N'Mulasztása miatt nem osztályozható, az oktatói testület határozata értelmében osztályozó vizsgát tehet.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2070, NULL, 83, NULL),
            (8424, N'Az oktatói testület határozata: a(z) ... évfolyamba léphet.', N'Az oktatói testület határozata: a(z) ... évfolyamba léphet.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2080, NULL, 84, NULL),
            (8425, N'A(z) ... tantárgyból javítóvizsgát tehet. A javítóvizsgán ... tantárgyból ... osztályzatot kapott, a(z) ... évfolyamba léphet.', N'A(z) ... tantárgyból javítóvizsgát tehet. A javítóvizsgán ... tantárgyból ... osztályzatot kapott, a(z) ... évfolyamba léphet.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2090, NULL, 85, NULL),
            (8426, N'A ... évfolyam követelményeit nem teljesítette. Évfolyamot ismételni köteles.', N'A ... évfolyam követelményeit nem teljesítette. Évfolyamot ismételni köteles.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2100, NULL, 86, NULL),
            (8427, N'A(z) ... foglalkozás alól ... okból felmentve.', N'A(z) ... foglalkozás alól ... okból felmentve.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2110, NULL, 87, NULL),
            (8428, N'A tanuló tanulói jogviszonya kimaradással megszűnt, a létszámból törölve.', N'A tanuló tanulói jogviszonya kimaradással megszűnt, a létszámból törölve.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2120, NULL, 88, NULL),
            (8429, N'A tanuló tanulói jogviszonya ... foglalkozás igazolatlan mulasztás miatt megszűnt, a létszámból törölve.', N'A tanuló tanulói jogviszonya ... foglalkozás igazolatlan mulasztás miatt megszűnt, a létszámból törölve.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2130, NULL, 89, NULL),
            (8430, N'A tanuló tanulói jogviszonya egészségügyi alkalmasság miatt megszűnt, a létszámból törölve.', N'A tanuló tanulói jogviszonya egészségügyi alkalmasság miatt megszűnt, a létszámból törölve.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2140, NULL, 90, NULL),
            (8431, N'A tanuló tanulói jogviszonya térítési díj, tandíj fizetési hátralék miatt megszűnt, a létszámból törölve.', N'A tanuló tanulói jogviszonya térítési díj, tandíj fizetési hátralék miatt megszűnt, a létszámból törölve.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2150, NULL, 91, NULL),
            (8432, N'A tanuló tanulói jogviszonya ... iskolába való átvétel miatt megszűnt, a létszámból törölve.', N'A tanuló tanulói jogviszonya ... iskolába való átvétel miatt megszűnt, a létszámból törölve.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2160, NULL, 92, NULL),
            (8433, N'Tanköteles tanuló igazolatlan mulasztása esetén: A tanuló ... foglalkozás igazolatlan mulasztása miatt a tanuló törvényes képviselőjét felszólítottam.', N'Tanköteles tanuló igazolatlan mulasztása esetén: A tanuló ... foglalkozás igazolatlan mulasztása miatt a tanuló törvényes képviselőjét felszólítottam.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2170, NULL, 93, NULL),
            (8434, N'Tanköteles tanuló igazolatlan mulasztása esetén: A tanuló ismételt ... foglalkozás igazolatlan mulasztása miatt a tanuló törvényes képviselője ellen szabálysértési eljárást kezdeményeztem.', N'Tanköteles tanuló igazolatlan mulasztása esetén: A tanuló ismételt ... foglalkozás igazolatlan mulasztása miatt a tanuló törvényes képviselője ellen szabálysértési eljárást kezdeményeztem.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2180, NULL, 94, NULL),
            (8435, N'Pótbizonyítvány. Igazolom, hogy név ..., anyja neve ... a(z) ... szakképző intézmény... szak (szakmai, speciális osztály, két tanítási nyelvű osztály, tagozat) ... évfolyamát a(z) ... tanévben eredményesen elvégezte.', N'Pótbizonyítvány. Igazolom, hogy név ..., anyja neve ... a(z) ... szakképző intézmény... szak (szakmai, speciális osztály, két tanítási nyelvű osztály, tagozat) ... évfolyamát a(z) ... tanévben eredményesen elvégezte.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2190, NULL, 95, NULL),
            (8436, N'A szakirányú oktatásról mulasztását ...-tól ...-ig pótolhatja.', N'A szakirányú oktatásról mulasztását ...-tól ...-ig pótolhatja.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2200, NULL, 96, NULL),
            (8437, N'Ezt a naplót ... tanítási nappal lezártam.', N'Ezt a naplót ... tanítási nappal lezártam.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2210, NULL, 97, NULL),
            (8438, N'Igazolom, hogy a tanuló a .../... tanévben ... óra közösségi szolgálatot teljesített. (Szkr.)', N'Igazolom, hogy a tanuló a .../... tanévben ... óra közösségi szolgálatot teljesített. (Szkr.)', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2220, NULL, 98, NULL),
            (8439, N'A tanuló teljesítette a rendes érettségi vizsga megkezdéséhez szükséges közösségi szolgálatot. (Szkr.)', N'A tanuló teljesítette a rendes érettségi vizsga megkezdéséhez szükséges közösségi szolgálatot. (Szkr.)', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2230, NULL, 99, NULL),
            (8440, N'A ... megszerzésére irányuló vizsgán elkövetett szabálytalanság miatt a(z) ... mellett működő vizsgabizottság a vizsgától eltiltotta. Javítóvizsgát tehet.', N'A ... megszerzésére irányuló vizsgán elkövetett szabálytalanság miatt a(z) ... mellett működő vizsgabizottság a vizsgától eltiltotta. Javítóvizsgát tehet.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2240, NULL, 100, NULL),
            (8441, N'A ... megszerzésére irányuló vizsgán igazolható okból nem jelent meg vagy azt nem tudta befejezni. Pótlóvizsgát tehet.', N'A ... megszerzésére irányuló vizsgán igazolható okból nem jelent meg vagy azt nem tudta befejezni. Pótlóvizsgát tehet.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2250, NULL, 101, NULL),
            (8442, N'A ... megszerzésére irányuló vizsgán igazolható ok nélkül nem jelent meg vagy azt megszakította. Javítóvizsgát tehet.', N'A ... megszerzésére irányuló vizsgán igazolható ok nélkül nem jelent meg vagy azt megszakította. Javítóvizsgát tehet.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2260, NULL, 102, NULL),
            (8443, N'A ... megszerzésére irányuló vizsgán a ... vizsgatevékenység esetében elégtelen teljesítményt nyújtott. Javítóvizsgát tehet.', N'A ... megszerzésére irányuló vizsgán a ... vizsgatevékenység esetében elégtelen teljesítményt nyújtott. Javítóvizsgát tehet.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2270, NULL, 103, NULL),
            (8444, N'A ... vizsgatevékenység érdemjegyét a(z) ... szakmai versenyen elért eredménye alapján kapta.', N'A ... vizsgatevékenység érdemjegyét a(z) ... szakmai versenyen elért eredménye alapján kapta.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2280, NULL, 104, NULL),
            (8445, N'A ... vizsgatevékenység teljesítése alól a képzési és kimeneti követelményekben foglaltak alapján mentesítve. A ... vizsgatevékenység érdemjegye a(z) ... év ... hó ... napján tett ... vizsga eredménye alapján került megállapításra.', N'A ... vizsgatevékenység teljesítése alól a képzési és kimeneti követelményekben foglaltak alapján mentesítve. A ... vizsgatevékenység érdemjegye a(z) ... év ... hó ... napján tett ... vizsga eredménye alapján került megállapításra.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2290, NULL, 105, NULL),
            (8446, N'A vizsgán/javítóvizsgán a vizsgakövetelményeknek megfelelt, ... szakma/részszakma megszerzését igazoló bizonyítványt kapott.', N'A vizsgán/javítóvizsgán a vizsgakövetelményeknek megfelelt, ... szakma/részszakma megszerzését igazoló bizonyítványt kapott.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2300, NULL, 106, NULL),
            (8447, N'A vizsgán a vizsgakövetelményeknek megfelelt, ... szakma/részszakma megszerzését igazoló bizonyítványa az érettségi vizsga letételét követően adható ki.', N'A vizsgán a vizsgakövetelményeknek megfelelt, ... szakma/részszakma megszerzését igazoló bizonyítványa az érettségi vizsga letételét követően adható ki.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2310, NULL, 107, NULL),
            (8448, N'A ... dátummal kiállított, ... számú érettségi bizonyítványa alapján a szakma/részszakma megszerzését igazoló bizonyítványa ... dátummal kiadásra került.', N'A ... dátummal kiállított, ... számú érettségi bizonyítványa alapján a szakma/részszakma megszerzését igazoló bizonyítványa ... dátummal kiadásra került.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2320, NULL, 108, NULL),
            (8449, N'A vizsgán a(z) ... vizsgatevékenysége(ke)t eredményesen teljesítette, vizsgaeredménye a vizsgatárgy eredményes teljesítését követően kerül megállapításra.', N'A vizsgán a(z) ... vizsgatevékenysége(ke)t eredményesen teljesítette, vizsgaeredménye a vizsgatárgy eredményes teljesítését követően kerül megállapításra.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2330, NULL, 109, NULL),
            (8450, N'A ... dátummal kiállított, ... számú érettségi bizonyítványa vagy a(z) ... számú érettségi törzslapkivonata alapján a vizsgakövetelményeknek megfelelt, ... dátummal ... szakma/ részszakma megszerzését igazoló bizonyítványt kapott.', N'A ... dátummal kiállított, ... számú érettségi bizonyítványa vagy a(z) ... számú érettségi törzslapkivonata alapján a vizsgakövetelményeknek megfelelt, ... dátummal ... szakma/ részszakma megszerzését igazoló bizonyítványt kapott.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2340, NULL, 110, NULL),
            (8451, N'A ... vizsgatevékenység esetében a felmentés alapjául szolgáló dokumentumokat határidőig nem mutatta be, a szakmai vizsga eredménytelen.', N'A ... vizsgatevékenység esetében a felmentés alapjául szolgáló dokumentumokat határidőig nem mutatta be, a szakmai vizsga eredménytelen.', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 2350, NULL, 111, NULL),
            (8452, N'Mentesítés oka: ...', N'Mentesítés oka: ...', NULL, NULL, NULL, N'SorolasOkaTipus', 41, N'F', N'T', 110, NULL, 112, NULL),
            (6763, N'Na', N'Na', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 1, NULL, 1, NULL),
            (3073, N'egyéb', N'egyéb', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 50, NULL, 2, NULL),
            (3069, N'osztályfőnök', N'osztályfőnök', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 5, NULL, 3, NULL),
            (3071, N'diákönkormányzati munkát segítő', N'diákönkormányzati munkát segítő', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 15, NULL, 4, NULL),
            (3070, N'munkaközösség vezető', N'munkaközösség vezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 10, NULL, 5, NULL),
            (6483, N'osztályfőnök és diákönkormányzati munkát segítő', N'osztályfőnök és diákönkormányzati munkát segítő', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 25, NULL, 6, NULL),
            (3072, N'osztályfőnök és munkaközösség vezető', N'osztályfőnök és munkaközösség vezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 20, NULL, 7, NULL),
            (6484, N'diákönkormányzati munkát segítő és munkaközösség vezető', N'diákönkormányzati munkát segítő és munkaközösség vezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 30, NULL, 8, NULL),
            (6485, N'osztályfőnök és diákönkormányzati munkát segítő és munkaközösség vezető', N'osztályfőnök és diákönkormányzati munkát segítő és munkaközösség vezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 35, NULL, 9, NULL),
            (6971, N'osztályfőnök és egyéb', N'osztályfőnök és egyéb', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 55, NULL, 10, NULL),
            (6972, N'munkaközösség vezető és egyéb', N'munkaközösség vezető és egyéb', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 60, NULL, 11, NULL),
            (6973, N'diákönkormányzati munkát segítő és egyéb', N'diákönkormányzati munkát segítő és egyéb', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 65, NULL, 12, NULL),
            (7798, N'tanszakvezető', N'tanszakvezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 70, NULL, 13, NULL),
            (7799, N'tanszakvezető és munkaközösség vezető', N'tanszakvezető és munkaközösség vezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 75, NULL, 14, NULL),
            (9106, N'tanszakvezető és osztályfőnök', N'tanszakvezető és osztályfőnök', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 15, NULL, 15, NULL),
            (9107, N'tanszakvezető és diákönkormányzati munkát segítő', N'tanszakvezető és diákönkormányzati munkát segítő', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 16, NULL, 16, NULL),
            (9108, N'tanszakvezető, osztályfőnök, diákönkormányzati munkát segítő és munkaközösség vezető', N'tanszakvezető, osztályfőnök, diákönkormányzati munkát segítő és munkaközösség vezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 17, NULL, 17, NULL),
            (9109, N'tanszakvezető, osztályfőnök és diákönkormányzati munkát segítő', N'tanszakvezető, osztályfőnök és diákönkormányzati munkát segítő', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 18, NULL, 18, NULL),
            (9110, N'tanszakvezető, osztályfőnök és munkaközösségvezető', N'tanszakvezető, osztályfőnök és munkaközösségvezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 19, NULL, 19, NULL),
            (9111, N'tanszakvezető, diákönkormányzati munkát segítő és munkaközösségvezető', N'tanszakvezető, diákönkormányzati munkát segítő és munkaközösségvezető', NULL, NULL, NULL, N'MunkaidoKedvezmenyOka', 71, N'F', N'T', 20, NULL, 20, NULL),
            (3040, N'Na', N'Na', NULL, NULL, NULL, N'BesorolasiFokozatTipus', 70, N'F', N'T', 1, NULL, 1, NULL),
            (3044, N'gyakornok', N'gyakornok', NULL, NULL, NULL, N'BesorolasiFokozatTipus', 70, N'F', N'T', 2, NULL, 2, NULL),
            (3045, N'pedagógus I.', N'pedagógus I.', NULL, NULL, NULL, N'BesorolasiFokozatTipus', 70, N'F', N'T', 3, NULL, 3, NULL),
            (3046, N'pedagógus II.', N'pedagógus II.', NULL, NULL, NULL, N'BesorolasiFokozatTipus', 70, N'F', N'T', 4, NULL, 4, NULL),
            (3048, N'mesterpedagógus', N'mesterpedagógus', NULL, NULL, NULL, N'BesorolasiFokozatTipus', 70, N'F', N'T', 5, NULL, 5, NULL),
            (3047, N'kutatótanár', N'kutatótanár', NULL, NULL, NULL, N'BesorolasiFokozatTipus', 70, N'F', N'T', 6, NULL, 6, NULL),
            (3032, N'teljes munkaidős', N'teljes munkaidős', NULL, NULL, NULL, N'PedagogusStatusz', 69, N'F', N'T', 1, NULL, 1, NULL),
            (3033, N'részmunkaidős', N'részmunkaidős', NULL, NULL, NULL, N'PedagogusStatusz', 69, N'F', N'T', 2, NULL, 2, NULL),
            (3034, N'óraadó', N'óraadó', NULL, NULL, NULL, N'PedagogusStatusz', 69, N'F', N'T', 3, NULL, 3, NULL),
            (3035, N'tartósan távollévő', N'tartósan távollévő', NULL, NULL, NULL, N'PedagogusStatusz', 69, N'F', N'T', 4, NULL, 4, NULL),
            (3036, N'nem pedagógus', N'nem pedagógus', NULL, NULL, NULL, N'PedagogusStatusz', 69, N'F', N'T', 5, NULL, 5, NULL),
            (3024, N'Csak adatszolgáltatás', N'Csak adatszolgáltatás', NULL, NULL, NULL, N'KretaTipus', 68, N'F', N'T', 1, NULL, 1, NULL),
            (3025, N'Teljes KRÉTA', N'Teljes KRÉTA', NULL, NULL, NULL, N'KretaTipus', 68, N'F', N'T', 2, NULL, 2, NULL),
            (1756, N'Nincs beállítva', N'Nincs beállítva', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 1, NULL, 1, NULL),
            (1758, N'Adminisztrátor', N'Adminisztrátor', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 3, NULL, 2, NULL),
            (1760, N'Napló', N'Napló', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 5, NULL, 3, NULL),
            (1761, N'e-Ellenőrző', N'e-Ellenőrző', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 6, NULL, 4, NULL),
            (7371, N'Pénzügyi Admin', N'Pénzügyi Admin', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 9, NULL, 5, NULL),
            (7372, N'Pénztáros', N'Pénztáros', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 10, NULL, 6, NULL),
            (7373, N'Aláíró', N'Aláíró', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 11, NULL, 7, NULL),
            (7374, N'eÜgyintéző', N'eÜgyintéző', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 12, NULL, 8, NULL),
            (7698, N'Gazdasági Ügyintéző', N'Gazdasági Ügyintéző', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 13, NULL, 9, NULL),
            (7702, N'Fókusz Monitoring', N'Fókusz Monitoring', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 14, NULL, 10, NULL),
            (7703, N'Étkezés Admin', N'Étkezés Admin', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 15, NULL, 11, NULL),
            (7704, N'Étkezés Ellenőrző', N'Étkezés Ellenőrző', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 16, NULL, 12, NULL),
            (7705, N'Terembérlet Jelenlét Ellenőrző', N'Terembérlet Jelenlét Ellenőrző', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 17, NULL, 13, NULL),
            (7781, N'Alkalmazott', N'Alkalmazott', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 18, NULL, 14, NULL),
            (7789, N'Iskola-egészségügyi koordinátor', N'Iskola-egészségügyi koordinátor', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 19, NULL, 15, NULL),
            (7790, N'Védőnő', N'Védőnő', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 20, NULL, 16, NULL),
            (7791, N'Iskolaorvos', N'Iskolaorvos', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 21, NULL, 17, NULL),
            (7792, N'Iskola-egészségügyi asszisztens', N'Iskola-egészségügyi asszisztens', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 22, NULL, 18, NULL),
            (7793, N'Iskola-egészségügyi lekérdező', N'Iskola-egészségügyi lekérdező', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 23, NULL, 19, NULL),
            (8753, N'Belépőkártya Admin', N'Belépőkártya Admin', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 24, NULL, 20, NULL),
            (8755, N'KRÉTA Leltározásban érintett', N'KRÉTA Leltározásban érintett', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 25, NULL, 21, NULL),
            (8824, N'Könyvtáros', N'Könyvtáros', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 25, NULL, 22, NULL),
            (8828, N'TESZEK - Eset kezelő', N'TESZEK - Eset kezelő', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 27, NULL, 23, NULL),
            (8829, N'TESZEK - Eszköz menedzser', N'TESZEK - Eszköz menedzser', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 29, NULL, 24, NULL),
            (8961, N'Kérdőív kitöltő - Kérdőív kezelő', N'Kérdőív kitöltő - Kérdőív kezelő', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 30, NULL, 25, NULL),
            (9001, N'Duális képzőhelyi adminisztrátor', N'Duális képzőhelyi adminisztrátor', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 31, NULL, 26, NULL),
            (9061, N'TESZEK - Szerződés menedzser', N'TESZEK - Szerződés menedzser', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 32, NULL, 27, NULL),
            (9062, N'TESZEK - Eset jóváhagyó', N'TESZEK - Eset jóváhagyó', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 33, NULL, 28, NULL),
            (9063, N'TESZEK - Eset közreműködő', N'TESZEK - Eset közreműködő', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 34, NULL, 29, NULL),
            (9064, N'TESZEK - Munkavégző', N'TESZEK - Munkavégző', NULL, NULL, NULL, N'SzerepkorTipus', 67, N'F', N'T', 35, NULL, 30, NULL),
            (1694, N'Kilépés', N'Exit', NULL, NULL, NULL, N'SystemSettingsControlType', 62, N'T', N'T', 1, NULL, 1, NULL),
            (1695, N'Jelszó', N'Password', NULL, NULL, NULL, N'SystemSettingsControlType', 62, N'T', N'T', 2, NULL, 2, NULL),
            (1696, N'Felhasználónév', N'Username', NULL, NULL, NULL, N'SystemSettingsControlType', 62, N'T', N'T', 3, NULL, 3, NULL),
            (1697, N'Bejelentkezés', N'Login', NULL, NULL, NULL, N'SystemSettingsControlType', 62, N'T', N'T', 4, NULL, 4, NULL),
            (1698, N'Nyelv', N'Language', NULL, NULL, NULL, N'SystemSettingsControlType', 62, N'T', N'T', 5, NULL, 5, NULL),
            (1699, N'Nyilvántartások', NULL, NULL, NULL, NULL, N'SystemSettingsControlType', 62, N'T', N'T', 6, NULL, 6, NULL),
            (2884, N'TimePicker', N'TimePicker', NULL, NULL, NULL, N'SystemSettingsControlType', 62, N'T', N'T', 7, NULL, 7, NULL),
            (2886, N'NumericTextBox', N'NumericTextBox', NULL, NULL, NULL, N'SystemSettingsControlType', 62, N'T', N'T', 8, NULL, 8, NULL),
            (5442, N'MultiSelect', N'MultiSelect', NULL, NULL, NULL, N'SystemSettingsControlType', 62, N'T', N'T', 9, NULL, 9, NULL),
            (1655, N'Mulasztás', N'Mulasztás', NULL, NULL, NULL, N'MulasztasStatisztikaTipus', 60, N'F', N'T', 1, NULL, 1, NULL),
            (1656, N'Igazolt', N'Igazolt', NULL, NULL, NULL, N'MulasztasStatisztikaTipus', 60, N'F', N'T', 2, NULL, 2, NULL),
            (1657, N'Egyéb', N'Egyéb', NULL, NULL, NULL, N'MulasztasStatisztikaTipus', 60, N'F', N'T', 3, NULL, 3, NULL),
            (1658, N'Orvosi igazolás', N'Orvosi igazolás', NULL, NULL, NULL, N'MulasztasStatisztikaTipus', 60, N'F', N'T', 4, NULL, 4, NULL),
            (1659, N'Szülői igazolás', N'Szülői igazolás', NULL, NULL, NULL, N'MulasztasStatisztikaTipus', 60, N'F', N'T', 5, NULL, 5, NULL),
            (1660, N'Hivatalos távollét', N'Hivatalos távollét', NULL, NULL, NULL, N'MulasztasStatisztikaTipus', 60, N'F', N'T', 6, NULL, 6, NULL),
            (1661, N'Iskolaérdekű távollét', N'Iskolaérdekű távollét', NULL, NULL, NULL, N'MulasztasStatisztikaTipus', 60, N'F', N'T', 7, NULL, 7, NULL),
            (1662, N'Igazolatlan', N'Igazolatlan', NULL, NULL, NULL, N'MulasztasStatisztikaTipus', 60, N'F', N'T', 8, NULL, 8, NULL),
            (1663, N'Késés', N'Késés', NULL, NULL, NULL, N'MulasztasStatisztikaTipus', 60, N'F', N'T', 9, NULL, 9, NULL),
            (1664, N'Késés igazolt', N'Késés igazolt', NULL, NULL, NULL, N'MulasztasStatisztikaTipus', 60, N'F', N'T', 10, NULL, 10, NULL),
            (1665, N'Késés igazolatlan', N'Késés igazolatlan', NULL, NULL, NULL, N'MulasztasStatisztikaTipus', 60, N'F', N'T', 11, NULL, 11, NULL),
            (1666, N'Tanuló sajátos nevelési igényének, szociális és tanügyi adatainak kezelése', N'Tanuló sajátos nevelési igényének, szociális és tanügyi adatainak kezelése', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 1, NULL, 1, NULL),
            (1667, N'Tanuló vallási adatainak kezelése', N'Tanuló vallási adatainak kezelése', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 2, NULL, 2, NULL),
            (1668, N'Tanulók osztályok közötti átsorolása esetén az összes csoportból való kiléptetését is automatikusan elvégezze a program', N'Tanulók osztályok közötti átsorolása esetén az összes csoportból való kiléptetését is automatikusan elvégezze a program', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 3, NULL, 3, NULL),
            (1669, N'A felhasználók nem változtathatják meg a jelszavukat az elektronikus ellenőrzőben', N'A felhasználók nem változtathatják meg a jelszavukat az elektronikus ellenőrzőben', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 4, NULL, 4, NULL),
            (1670, N'A tanulók értékeléseinél az értékelések súlyszámainak megjelenítése', N'A tanulók értékeléseinél az értékelések súlyszámainak megjelenítése', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 5, NULL, 5, NULL),
            (1671, N'Pedagógusok saját órarendjeiket módosíthatják az elektronikus naplóban', N'Pedagógusok saját órarendjeiket módosíthatják az elektronikus naplóban', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 6, NULL, 6, NULL),
            (1672, N'A haladási napló csak az alábbiak szerint tölthető ki (utólag naplózni)', N'A haladási napló csak az alábbiak szerint tölthető ki (utólag naplózni)', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 7, NULL, 7, NULL),
            (1673, N'A haladási naplóba óránként csak egy tanóra rögzíthető', N'A haladási naplóba óránként csak egy tanóra rögzíthető', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 8, NULL, 8, NULL),
            (1674, N'Elmaradt tanítási óra rögzítésének tiltása', N'Elmaradt tanítási óra rögzítésének tiltása', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 9, NULL, 9, NULL),
            (1675, N'Pedagógusok csak az előírt helyettesítéseiket regisztrálhatják a haladási naplóban', N'Pedagógusok csak az előírt helyettesítéseiket regisztrálhatják a haladási naplóban', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 10, NULL, 10, NULL),
            (1677, N'Osztályfőnök módosíthatja osztályába járó tanulók adatait', N'Osztályfőnök módosíthatja osztályába járó tanulók adatait', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 12, NULL, 11, NULL),
            (1678, N'Osztályfőnök megtekintheti/módosíthatja osztályába járó tanulók szociális, tanügyi és SNI/BTM adatait', N'Osztályfőnök megtekintheti/módosíthatja osztályába járó tanulók szociális, tanügyi és SNI/BTM adatait', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 13, NULL, 12, NULL),
            (1679, N'Osztályfőnök rögzíthet évközi záradékot az osztályába járó tanulók számára', N'Osztályfőnök rögzíthet évközi záradékot az osztályába járó tanulók számára', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 14, NULL, 13, NULL),
            (7699, N'Osztályfőnök rögzíthet ki- vagy átsorolási záradékot az osztályába járó tanulók számára', N'Osztályfőnök rögzíthet ki- vagy átsorolási záradékot az osztályába járó tanulók számára', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 14, NULL, 14, NULL),
            (1680, N'Osztályfőnök kezelheti az osztályába járó tanulók vallási adatait', N'Osztályfőnök kezelheti az osztályába járó tanulók vallási adatait', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 15, NULL, 15, NULL),
            (1764, N'Osztályfőnök módosíthatja osztályába járó tanulók sportolói adatait', N'Osztályfőnök módosíthatja osztályába járó tanulók sportolói adatait', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 16, NULL, 16, NULL),
            (1681, N'Értékelésnél a naplójegyek átlaga megjelenik', N'Értékelésnél a naplójegyek átlaga megjelenik', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 17, NULL, 17, NULL),
            (1682, N'Magántanuló / Egyéni munkarend óralátogatás alóli mentességének kezelése', N'Magántanuló / Egyéni munkarend óralátogatás alóli mentességének kezelése', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 18, NULL, 18, NULL),
            (1683, N'Az iskolaérdekű távollét típusú igazolások beleszámítanak a tanuló összes mulasztásába', N'Az iskolaérdekű távollét típusú igazolások beleszámítanak a tanuló összes mulasztásába', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 19, NULL, 19, NULL),
            (1684, N'Osztályfőnök számára tiltott a mulasztási adatok módosítása', N'Osztályfőnök számára tiltott a mulasztási adatok módosítása', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 20, NULL, 20, NULL),
            (1685, N'A foglalkozások mulasztási adatai megadásánál a párhuzamos órákkal való ütközés vizsgálata', N'A foglalkozások mulasztási adatai megadásánál a párhuzamos órákkal való ütközés vizsgálata', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 21, NULL, 21, NULL),
            (1686, N'Értesítés Tanév rendi változtatásról esetek', N'Értesítés Tanév rendi változtatásról esetek', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 22, NULL, 22, NULL),
            (1687, N'Értesítés Tanév rendi változtatásról időpont', N'Értesítés Tanév rendi változtatásról időpont', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 23, NULL, 23, NULL),
            (1688, N'Érdemjegy beírás értesítés késleltetése', N'Érdemjegy beírás értesítés késleltetése', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 24, NULL, 24, NULL),
            (1689, N'Értesítés hiányzó órabeírásról késleltetése', N'Értesítés hiányzó órabeírásról késleltetése', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 25, NULL, 25, NULL),
            (1690, N'Értesítés bukásra álló tanulókról', N'Értesítés bukásra álló tanulókról', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 26, NULL, 26, NULL),
            (1691, N'Értesítés bukásra álló végzős tanulókról', N'Értesítés bukásra álló végzős tanulókról', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 27, NULL, 27, NULL),
            (1692, N'Értesítés nyílt napról', N'Értesítés nyílt napról', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 28, NULL, 28, NULL),
            (1693, N'Értesítés óralátogatásról', N'Értesítés óralátogatásról', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 29, NULL, 29, NULL),
            (3074, N'Helyettesítéskor a felügyelet alapértelmezetten sorszámozandó', N'Helyettesítéskor a felügyelet alapértelmezetten sorszámozandó', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 30, NULL, 30, NULL),
            (3075, N'Tanítási óra sorszámozás módja', N'Tanítási óra sorszámozás módja', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 31, NULL, 31, NULL),
            (3078, N'Osztályok és osztálybontott csoportjaik óráinak közös számozása', N'Osztályok és osztálybontott csoportjaik óráinak közös számozása', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 32, NULL, 32, NULL),
            (1755, N'Adminisztrátor értesítése órarendi tanuló ütközésről', N'Adminisztrátor értesítése órarendi tanuló ütközésről', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 33, NULL, 33, NULL),
            (1818, N'Tanuló sportolói adatainak kezelése', N'Tanuló sportolói adatainak kezelése', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 34, NULL, 34, NULL),
            (2874, N'Szakszerű helyettesítés esetén az elszámolt óraszám', N'Szakszerű helyettesítés esetén az elszámolt óraszám', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 35, NULL, 35, NULL),
            (2875, N'Nem szakszerű (felügyelet) helyettesítés esetén az elszámolt óraszám', N'Nem szakszerű (felügyelet) helyettesítés esetén az elszámolt óraszám', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 36, NULL, 36, NULL),
            (2876, N'Óraösszevonásos helyettesítés esetén az elszámolt óraszám', N'Óraösszevonásos helyettesítés esetén az elszámolt óraszám', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 37, NULL, 37, NULL),
            (2877, N'Lehet-e helyettesítés túlóra?', N'Lehet-e helyettesítés túlóra?', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 38, NULL, 38, NULL),
            (2882, N'Az elektronikus napló megjelenésének kezdőidőpontja', N'Az elektronikus napló megjelenésének kezdőidőpontja', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 39, NULL, 39, NULL),
            (2883, N'Az elektronikus napló megjelenésének záróidőpontja', N'Az elektronikus napló megjelenésének záróidőpontja', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 40, NULL, 40, NULL),
            (2885, N'Tanuló felhasználói név generálás módja', N'Tanuló felhasználói név generálás módja', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 41, NULL, 41, NULL),
            (3006, N'Tanár vehet-e fel órát magának?', N'Tanár vehet-e fel órát magának?', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 42, NULL, 42, NULL),
            (3015, N'Lehet órát felvenni tantárgyfelosztás nélkül', N'Lehet órát felvenni tantárgyfelosztás nélkül', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 43, NULL, 43, NULL),
            (3037, N'Napirend felvitele esetében csak osztály vagy tanórai célú csoport állítható be', N'Napirend felvitele esetében csak osztály vagy tanórai célú csoport állítható be', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 44, NULL, 44, NULL),
            (3039, N'Egy státuszra jutó kötelező óraszám', N'Egy státuszra jutó kötelező óraszám', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 45, NULL, 45, NULL),
            (3076, N'Értékelések dátumának minimuma', N'Értékelések dátumának minimuma', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 46, NULL, 46, NULL),
            (5222, N'Értékelések oldalon lévő szűrők beállítása', N'Értékelések oldalon lévő szűrők beállítása', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 47, NULL, 47, NULL),
            (3079, N'Heti vagy havi naplózárás', N'Heti vagy havi naplózárás', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 48, NULL, 48, NULL),
            (3080, N'Havi naplózárás napja', N'Havi naplózárás napja', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 49, NULL, 49, NULL),
            (3081, N'Heti naplózárás napja', N'Heti naplózárás napja', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 50, NULL, 50, NULL),
            (3082, N'Elektronikus napló zárásának dátuma', N'Elektronikus napló zárásának dátuma', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 51, NULL, 51, NULL),
            (3083, N'Kötött munkaidő neveléssel-oktatással le nem kötött rész megjelenésének kezdőidőpontja', N'Kötött munkaidő neveléssel-oktatással le nem kötött rész megjelenésének kezdőidőpontja', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 52, NULL, 52, NULL),
            (3084, N'Kötött munkaidő neveléssel-oktatással le nem kötött rész megjelenésének záróidőpontja', N'Kötött munkaidő neveléssel-oktatással le nem kötött rész megjelenésének záróidőpontja', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 53, NULL, 53, NULL),
            (5223, N'Magatartás/szorgalom értékelések oldalon lévő szűrők beállítása', N'Magatartás/szorgalom értékelések oldalon lévő szűrők beállítása', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 54, NULL, 54, NULL),
            (5225, N'Tanuló felhasználói jelszó generálás módja', N'Tanuló felhasználói jelszó generálás módja', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 55, NULL, 55, NULL),
            (5226, N'Gondviselő felhasználói név generálás módja', N'Gondviselő felhasználói név generálás módja', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 56, NULL, 56, NULL),
            (5227, N'Gondviselő felhasználói jelszó generálás módja', N'Gondviselő felhasználói jelszó generálás módja', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 57, NULL, 57, NULL),
            (5261, N'Tevékenység felvitelekor ütközés figyelése', N'Tevékenység felvitelekor ütközés figyelése', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 58, NULL, 58, NULL),
            (5436, N'Osztályfőnök adhat-e értékelést osztályának minden tantárgyból', N'Osztályfőnök adhat-e értékelést osztályának minden tantárgyból', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 59, NULL, 59, NULL),
            (5437, N'Az óra sorszámozását a naplózó pedagógus eldöntheti', N'Az óra sorszámozását a naplózó pedagógus eldöntheti', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 60, NULL, 60, NULL),
            (5441, N'Alkalmazottak láthatják-e a diákok adatait', N'Alkalmazottak láthatják-e a diákok adatait', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 61, NULL, 61, NULL),
            (5438, N'Félévi jegyek megjelenésének dátuma', N'Félévi jegyek megjelenésének dátuma', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 62, NULL, 62, NULL),
            (5439, N'Év végi jegyek megjelenésének dátuma', N'Év végi jegyek megjelenésének dátuma', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 63, NULL, 63, NULL),
            (5440, N'Év végi jegyek megjelenésének dátuma (végzős évfolyam esetén)', N'Év végi jegyek megjelenésének dátuma (végzős évfolyam esetén)', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 64, NULL, 64, NULL),
            (5449, N'Tanóra naplózásánál mulasztás százalék számítás', N'Tanóra naplózásánál mulasztás százalék számítás', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 65, NULL, 65, NULL),
            (6717, N'Naplózáskor terembefogadóképesség ellenőrzés', N'Naplózáskor terembefogadóképesség ellenőrzés', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 66, NULL, 66, NULL),
            (6764, N'Tanórához tartozó csoportok', N'Tanórához tartozó csoportok', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 67, NULL, 67, NULL),
            (6765, N'Tanéven belül egy tanuló csak egy évfolyamot végezhet', N'Tanéven belül egy tanuló csak egy évfolyamot végezhet', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 68, NULL, 68, NULL),
            (6766, N'A naplóban csak a jelenlegi osztályában szerzett értékelés/mulasztás adatai jelennek meg', N'A naplóban csak a jelenlegi osztályában szerzett értékelés/mulasztás adatai jelennek meg', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 69, NULL, 69, NULL),
            (6767, N'E-mail értesítés küldése a jelszó generálásakor', N'E-mail értesítés küldése a jelszó generálásakor', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 70, NULL, 70, NULL),
            (6768, N'A lista – beállítástól függően – az összes osztályt/csoportot megjeleníti, vagy csak azokat az osztályokat/csoportokat amelyeket a tanár tanít.', N'A lista – beállítástól függően – az összes osztályt/csoportot megjeleníti, vagy csak azokat az osztályokat/csoportokat amelyeket a tanár tanít.', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 71, NULL, 71, NULL),
            (6769, N'A csoportvezetők láthatják a csoportjukba sorolt diákok adatait.', N'A csoportvezetők láthatják a csoportjukba sorolt diákok adatait.', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 72, NULL, 72, NULL),
            (6770, N'A tanóra célú csoportokba sorolt diákok is jelenjenek meg a csoportvezetői felületen.', N'A tanóra célú csoportokba sorolt diákok is jelenjenek meg a csoportvezetői felületen.', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 73, NULL, 73, NULL),
            (6830, N'Sikeres művelet visszajelzés kikapcsolása', N'Sikeres művelet visszajelzés kikapcsolása', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 74, NULL, 74, NULL),
            (6852, N'Intézmény adatkezelési beállítások', N'Intézmény adatkezelési beállítások', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 75, NULL, 75, NULL),
            (6854, N'Magatartás és szorgalom értékelést csak osztályfőnök adhat', N'Magatartás és szorgalom értékelést csak osztályfőnök adhat', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 76, NULL, 76, NULL),
            (6941, N'Teremütközés figyelés', N'Teremütközés figyelés', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 77, NULL, 77, NULL),
            (7094, N'Diák általi házi feladatok rögzítésének lehetősége', N'Diák általi házi feladatok rögzítésének lehetősége', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 78, NULL, 78, NULL),
            (7106, N'Egybefüggő szakmai gyakorlat kezdete', N'Egybefüggő szakmai gyakorlat kezdete', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 79, NULL, 79, NULL),
            (7107, N'Egybefüggő szakmai gyakorlat vége', N'Egybefüggő szakmai gyakorlat vége', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 80, NULL, 80, NULL),
            (7108, N'Tanár vehet-e fel órát magának az egybefüggő szakmai gyakorlat ideje alatt?', N'Tanár vehet-e fel órát magának az egybefüggő szakmai gyakorlat ideje alatt?', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 81, NULL, 81, NULL),
            (7130, N'Mennyi tanítási óra esetén tekintse a rendszer érvényesnek a tanítási napot', N'Mennyi tanítási óra esetén tekintse a rendszer érvényesnek a tanítási napot', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 82, NULL, 82, NULL),
            (7131, N'Értékelések, mulasztások, feljegyzések kezelése osztályváltás esetén', N'Értékelések, mulasztások, feljegyzések kezelése osztályváltás esetén', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 83, NULL, 83, NULL),
            (7132, N'Értékelések, mulasztások, feljegyzések kezelése évfolyamváltás esetén', N'Értékelések, mulasztások, feljegyzések kezelése évfolyamváltás esetén', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 84, NULL, 84, NULL),
            (7360, N'Tanári órarendek láthatóságának tiltása', N'Tanári órarendek láthatóságának tiltása', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 85, NULL, 85, NULL),
            (7361, N'A tanórák témája megtekinthető az elektronikus ellenőrzőben', N'A tanórák témája megtekinthető az elektronikus ellenőrzőben', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 86, NULL, 86, NULL),
            (7362, N'I Negyedéves jegyek megjelenésének dátuma', N'I Negyedéves jegyek megjelenésének dátuma', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 87, NULL, 87, NULL),
            (7363, N'II Negyedéves jegyek megjelenésének dátuma', N'II Negyedéves jegyek megjelenésének dátuma', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 88, NULL, 88, NULL),
            (7364, N'III Negyedéves jegyek megjelenésének dátuma', N'III Negyedéves jegyek megjelenésének dátuma', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 89, NULL, 89, NULL),
            (7365, N'IV Negyedéves jegyek megjelenésének dátuma', N'IV Negyedéves jegyek megjelenésének dátuma', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 90, NULL, 90, NULL),
            (7375, N'Csökkentett gondviselők kezelése', N'Csökkentett gondviselők kezelése', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 91, NULL, 91, NULL),
            (7380, N'Értékelések megjelenése az Ellenőrző modulban', N'Értékelések megjelenése az Ellenőrző modulban', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 92, NULL, 92, NULL),
            (7563, N'Osztályátlag megjelenítése az Ellenőrző (szülő, diák) felületeken', N'Osztályátlag megjelenítése az Ellenőrző (szülő, diák) felületeken', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 93, NULL, 93, NULL),
            (7564, N'Fogadóórák menü megjelenítése', N'Fogadóórák menü megjelenítése', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 93, NULL, 94, NULL),
            (7565, N'Alapértelmezett ellátott feladat', N'Alapértelmezett ellátott feladat', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 94, NULL, 95, NULL),
            (7568, N'Egyéni csoportok automatikus létrehozása', N'Egyéni csoportok automatikus létrehozása', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 95, NULL, 96, NULL),
            (7650, N'Fogadóóra kezdőidőpontja', N'Fogadóóra kezdőidőpontja', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 96, NULL, 97, NULL),
            (7651, N'Fogadóóra záróidőpontja', N'Fogadóóra záróidőpontja', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 97, NULL, 98, NULL),
            (7661, N'Napi számonkérési limit', N'Napi számonkérési limit', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 98, NULL, 99, NULL),
            (7662, N'Értékelés módok beállítása a napi limithez', N'Értékelés módok beállítása a napi limithez', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 99, NULL, 100, NULL),
            (7663, N'Limit figyelmeztetési / tiltási beállítás', N'Limit figyelmeztetési / tiltási beállítás', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 100, NULL, 101, NULL),
            (7680, N'Értékeléseknél megjelenő speciális gombok láthatósága', N'Értékeléseknél megjelenő speciális gombok láthatósága', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 101, NULL, 102, NULL),
            (7683, N'Foglalkozás rögzítésének engedélyezése hétvégére', N'Foglalkozás rögzítésének engedélyezése hétvégére', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 102, NULL, 103, NULL),
            (7700, N'Az osztályfőnök módosíthatja-e a tanulók mulasztási adatait', N'Az osztályfőnök módosíthatja-e a tanulók mulasztási adatait', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 103, NULL, 104, NULL),
            (7718, N'Csoportvezetők igazolási jogosultsága', N'Csoportvezetők igazolási jogosultsága', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 104, NULL, 105, NULL),
            (7707, N'Szakszerű helyettesítés százalékos értéke', N'Szakszerű helyettesítés százalékos értéke', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 105, NULL, 106, NULL),
            (7708, N'Nem szakszerű helyettesítés százalékos értéke', N'Nem szakszerű helyettesítés százalékos értéke', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 106, NULL, 107, NULL),
            (7709, N'Óraösszevonás százalékos értéke', N'Óraösszevonás százalékos értéke', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 107, NULL, 108, NULL),
            (7710, N'Összevont felügyelet százalékos értéke', N'Összevont felügyelet százalékos értéke', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 108, NULL, 109, NULL),
            (7746, N'Lázár Ervin Program kezelése', N'Lázár Ervin Program kezelése', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 109, NULL, 110, NULL),
            (7770, N'Bukás veszélyre figyelmeztetés határa', N'Bukás veszélyre figyelmeztetés határa', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 110, NULL, 111, NULL),
            (7771, N'Szakgimnáziumok modulon kívül is tudják kezelni az osztályokat/csoportokat és tanulókat', N'Szakgimnáziumok modulon kívül is tudják kezelni az osztályokat/csoportokat és tanulókat', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 111, NULL, 112, NULL),
            (7784, N'Nemzetiségi dokumentumokhoz szükséges további nyelvek', N'Nemzetiségi dokumentumokhoz szükséges további nyelvek', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 112, NULL, 113, NULL),
            (7785, N'Távollét bejelentésre jogosult szerepkörök', N'Távollét bejelentésre jogosult szerepkörök', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 113, NULL, 114, NULL),
            (7786, N'Órarendi elemek befejező időpontja az "Órahossz" adat alapján', N'Órarendi elemek befejező időpontja az "Órahossz" adat alapján', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 114, NULL, 115, NULL),
            (7800, N'Értesítések küldése a mobil alkalmazások számára a tanulók házi feladatokhoz írt hozzászólásairól', N'Értesítések küldése a mobil alkalmazások számára a tanulók házi feladatokhoz írt hozzászólásairól', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 115, NULL, 116, NULL),
            (7801, N'Alapértelmezett státusz naplózások esetében', N'Alapértelmezett státusz naplózások esetében', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 116, NULL, 117, NULL),
            (7802, N'Összes dokumentum elérésének engedélyezése szaktanárok és osztályfőnökök részére', N'Összes dokumentum elérésének engedélyezése szaktanárok és osztályfőnökök részére', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 117, NULL, 118, NULL),
            (7803, N'Hetesek megjelenítése az Elektronikus ellenőrzőben', N'Hetesek megjelenítése az Elektronikus ellenőrzőben', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 118, NULL, 119, NULL),
            (7804, N'IsCsokkentettAdminJog', N'IsCsokkentettAdminJog', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'F', 119, NULL, 120, NULL),
            (7805, N'Magatartás értékelés szabályozása', N'Magatartás értékelés szabályozása', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 120, NULL, 121, NULL),
            (7890, N'Nem tanórai célú csoportok értékelési lehetősége', N'Nem tanórai célú csoportok értékelési lehetősége', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 121, NULL, 122, NULL),
            (7892, N'Helyettesítés láthatósága az ellenőrzőben', N'Helyettesítés láthatósága az ellenőrzőben', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 123, NULL, 123, NULL),
            (7891, N'Elektronikus napló zárásának dátuma beállítás érvényességi köre', N'Elektronikus napló zárásának dátuma beállítás érvényességi köre', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 124, NULL, 124, NULL),
            (8388, N'Igazolások naplózárása', N'Igazolások naplózárása', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 126, NULL, 125, NULL),
            (8400, N'Lezárt elektronikus naplók', N'Lezárt elektronikus naplók', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 127, NULL, 126, NULL),
            (8614, N'Értékelésnél a naplójegyek havi átlaga megjelenik', N'Értékelésnél a naplójegyek havi átlaga megjelenik', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 128, NULL, 127, NULL),
            (8682, N'Órarendi elemek rögzítésének előfeltétele', N'Órarendi elemek rögzítésének előfeltétele', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 129, NULL, 128, NULL),
            (8702, N'Mulasztási százalék számítás osztályváltás után', N'Mulasztási százalék számítás osztályváltás után', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 130, NULL, 129, NULL),
            (8703, N'Kötött munkaidő megjelenítése', N'Kötött munkaidő megjelenítése', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 131, NULL, 130, NULL),
            (8825, N'Kötött munkaidő neveléssel-oktatással le nem kötött rész kezelésének tiltása', N'Kötött munkaidő neveléssel-oktatással le nem kötött rész kezelésének tiltása', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 43, NULL, 131, NULL),
            (8826, N'Tanulók és Gondviselők szerkeszthetik saját elérhetőségi adataikat', N'Tanulók és Gondviselők szerkeszthetik saját elérhetőségi adataikat', NULL, NULL, NULL, N'RendszerBeallitasTipus', 61, N'F', N'T', 132, NULL, 132, NULL),
            (1651, N'Osztályszintű', N'Osztályszintű', NULL, NULL, NULL, N'KizaroCsoportTipus', 59, N'F', N'T', 1, NULL, 1, NULL),
            (1652, N'Több osztályszintű', N'Több osztályszintű', NULL, NULL, NULL, N'KizaroCsoportTipus', 59, N'F', N'T', 2, NULL, 2, NULL),
            (1653, N'Évfolyam szintű', N'Évfolyam szintű', NULL, NULL, NULL, N'KizaroCsoportTipus', 59, N'F', N'T', 3, NULL, 3, NULL),
            (1654, N'Intézményi szintű', N'Intézményi szintű', NULL, NULL, NULL, N'KizaroCsoportTipus', 59, N'F', N'T', 4, NULL, 4, NULL),
            (1596, N'Tanév rend változás', N'Tanév rend változás', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 1, NULL, 1, NULL),
            (1597, N'Jegybeírás', N'Jegybeírás', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 2, NULL, 2, NULL),
            (1598, N'Késés', N'Késés', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 3, NULL, 3, NULL),
            (1599, N'Hiányzás', N'Hiányzás', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 4, NULL, 4, NULL),
            (1600, N'Magas százalékos hiányzás', N'Magas százalékos hiányzás', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 5, NULL, 5, NULL),
            (1601, N'Tárgy jegyek átlaga', N'Tárgy jegyek átlaga', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 6, NULL, 6, NULL),
            (1602, N'Friss feljegyzés', N'Friss feljegyzés', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 7, NULL, 7, NULL),
            (1603, N'Közösségi szolgálat állapot', N'Közösségi szolgálat állapot', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 8, NULL, 8, NULL),
            (1604, N'Közösségi szolgálat betöltése', N'Közösségi szolgálat betöltése', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 9, NULL, 9, NULL),
            (1605, N'Közösségi szolgálat figyelmeztetés', N'Közösségi szolgálat figyelmeztetés', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 10, NULL, 10, NULL),
            (1606, N'Szülői értekezlet', N'Szülői értekezlet', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 11, NULL, 11, NULL),
            (1607, N'Tanári értekezlet', N'Tanári értekezlet', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 12, NULL, 12, NULL),
            (1608, N'Hiányzó óra naplózás', N'Hiányzó óra naplózás', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 13, NULL, 13, NULL),
            (1609, N'Tanárnak bukásra álló tanulók', N'Tanárnak bukásra álló tanulók', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 14, NULL, 14, NULL),
            (1610, N'Osztályfőnöknek bukásra álló tanulók', N'Osztályfőnöknek bukásra álló tanulók', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 15, NULL, 15, NULL),
            (1611, N'Tanár óra helyettesítés', N'Tanár óra helyettesítés', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 16, NULL, 16, NULL),
            (1612, N'Tanár óra helyettesítő', N'Tanár óra helyettesítő', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 17, NULL, 17, NULL),
            (1613, N'Tanuló óra helyettesítés', N'Tanuló óra helyettesítés', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 18, NULL, 18, NULL),
            (1614, N'Nyílt nap', N'Nyílt nap', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 19, NULL, 19, NULL),
            (1615, N'Tanári óralátogatás tartása', N'Tanári óralátogatás tartása', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 20, NULL, 20, NULL),
            (1616, N'Tanárnak óráján óralátogatás', N'Tanárnak óráján óralátogatás', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 21, NULL, 21, NULL),
            (1617, N'Osztályba új tanuló', N'Osztályba új tanuló', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 22, NULL, 22, NULL),
            (1618, N'Csoportba új tanuló', N'Csoportba új tanuló', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 23, NULL, 23, NULL),
            (1619, N'Osztályból tanuló kikerülés', N'Osztályból tanuló kikerülés', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 24, NULL, 24, NULL),
            (1620, N'Csoportból tanuló kikerülés', N'Csoportból tanuló kikerülés', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 25, NULL, 25, NULL),
            (1621, N'Tanítási óra elmaradás', N'Tanítási óra elmaradás', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 26, NULL, 26, NULL),
            (1622, N'Új pénzügyi tétel', N'Új pénzügyi tétel', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 27, NULL, 27, NULL),
            (1623, N'Diákigazolvány állapot', N'Diákigazolvány állapot', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 28, NULL, 28, NULL),
            (1624, N'Nyomtatvány állapot', N'Nyomtatvány állapot', NULL, NULL, NULL, N'ErtesitesTipus', 57, N'F', N'T', 29, NULL, 29, NULL),
            (1593, N'Kék', N'Kék', NULL, NULL, NULL, N'ErtesitesSzinTipus', 56, N'F', N'T', 1, NULL, 1, NULL),
            (1594, N'Zöld', N'Zöld', NULL, NULL, NULL, N'ErtesitesSzinTipus', 56, N'F', N'T', 2, NULL, 2, NULL),
            (1595, N'Piros', N'Piros', NULL, NULL, NULL, N'ErtesitesSzinTipus', 56, N'F', N'T', 3, NULL, 3, NULL),
            (1625, N'Egyéb', N'Egyéb', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 1, NULL, 1, NULL),
            (1626, N'foglalkozások, tanítási órák előkészítése', N'foglalkozások, tanítási órák előkészítése', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 2, NULL, 2, NULL),
            (1627, N'a gyermekek, tanulók teljesítményének értékelése', N'a gyermekek, tanulók teljesítményének értékelése', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 3, NULL, 3, NULL),
            (1628, N'az intézmény kulturális és sportéletének, versenyeknek, a szabadidő hasznos eltöltésének megszervezése', N'az intézmény kulturális és sportéletének, versenyeknek, a szabadidő hasznos eltöltésének megszervezése', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 4, NULL, 4, NULL),
            (1629, N'heti egy órát meghaladó részében a tanulók nevelési-oktatási intézményen belüli önszerveződésének segítésével összefüggő feladatok végrehajtása', N'heti egy órát meghaladó részében a tanulók nevelési-oktatási intézményen belüli önszerveződésének segítésével összefüggő feladatok végrehajtása', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 5, NULL, 5, NULL),
            (1630, N'előre tervezett beosztás szerint vagy alkalomszerűen gyermekek, tanulók, tanórai és egyéb foglalkozásnak nem minősülő felügyelete', N'előre tervezett beosztás szerint vagy alkalomszerűen gyermekek, tanulók, tanórai és egyéb foglalkozásnak nem minősülő felügyelete', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 6, NULL, 6, NULL),
            (1631, N'a tanuló- és gyermekbalesetek megelőzésével kapcsolatos feladatok végrehajtása', N'a tanuló- és gyermekbalesetek megelőzésével kapcsolatos feladatok végrehajtása', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 7, NULL, 7, NULL),
            (1632, N'a gyermek- és ifjúságvédelemmel összefüggő feladatok végrehajtása', N'a gyermek- és ifjúságvédelemmel összefüggő feladatok végrehajtása', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 8, NULL, 8, NULL),
            (1633, N'eseti helyettesítés', N'eseti helyettesítés', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 9, NULL, 9, NULL),
            (1634, N'a pedagógiai tevékenységhez kapcsolódó ügyviteli tevékenység', N'a pedagógiai tevékenységhez kapcsolódó ügyviteli tevékenység', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 10, NULL, 10, NULL),
            (1635, N'az intézményi dokumentumok készítése, vezetése', N'az intézményi dokumentumok készítése, vezetése', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 11, NULL, 11, NULL),
            (1636, N'a szülőkkel történő kapcsolattartás, szülői értekezlet, fogadóóra megtartása', N'a szülőkkel történő kapcsolattartás, szülői értekezlet, fogadóóra megtartása', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 12, NULL, 12, NULL),
            (1637, N'az osztályfőnöki munkával összefüggő tevékenység heti két órát meghaladó része', N'az osztályfőnöki munkával összefüggő tevékenység heti két órát meghaladó része', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 13, NULL, 13, NULL),
            (1638, N'pedagógusjelölt, gyakornok szakmai segítése, mentorálása', N'pedagógusjelölt, gyakornok szakmai segítése, mentorálása', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 14, NULL, 14, NULL),
            (1639, N'a nevelőtestület, a szakmai munkaközösség munkájában történő részvétel', N'a nevelőtestület, a szakmai munkaközösség munkájában történő részvétel', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 15, NULL, 15, NULL),
            (1640, N'a munkaközösség-vezetés heti két órát meghaladó része', N'a munkaközösség-vezetés heti két órát meghaladó része', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 16, NULL, 16, NULL),
            (1641, N'az intézményfejlesztési feladatokban való közreműködés', N'az intézményfejlesztési feladatokban való közreműködés', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 17, NULL, 17, NULL),
            (1642, N'környezeti neveléssel összefüggő feladatok ellátása', N'környezeti neveléssel összefüggő feladatok ellátása', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 18, NULL, 18, NULL),
            (1643, N'iskolai szertár fejlesztése, karbantartása', N'iskolai szertár fejlesztése, karbantartása', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 19, NULL, 19, NULL),
            (1644, N'hangszerkarbantartás megszervezése', N'hangszerkarbantartás megszervezése', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 20, NULL, 20, NULL),
            (1645, N'különböző feladatellátási helyekre történő alkalmazás esetében a köznevelési intézmény telephelyei közötti utazás', N'különböző feladatellátási helyekre történő alkalmazás esetében a köznevelési intézmény telephelyei közötti utazás', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 21, NULL, 21, NULL),
            (1646, N'a pedagógiai program célrendszerének megfelelő, az éves munkatervben rögzített, tanórai vagy egyéb foglalkozásnak nem minősülő feladat ellátása', N'a pedagógiai program célrendszerének megfelelő, az éves munkatervben rögzített, tanórai vagy egyéb foglalkozásnak nem minősülő feladat ellátása', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 22, NULL, 22, NULL),
            (6514, N'a gyakornok felkészülése a minősítő vizsgára', N'a gyakornok felkészülése a minősítő vizsgára', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 23, NULL, 23, NULL),
            (6515, N'pedagógus-továbbképzésben való részvétel', N'pedagógus-továbbképzésben való részvétel', NULL, NULL, NULL, N'TevekenysegTipus', 58, N'T', N'T', 24, NULL, 24, NULL),
            (6462, N'Na', N'Na', NULL, NULL, NULL, N'EletpalyamodellFokozatTipus', 53, N'F', N'T', 1, NULL, 1, NULL),
            (1566, N'Gyakornok', N'Gyakornok', NULL, NULL, NULL, N'EletpalyamodellFokozatTipus', 53, N'F', N'T', 2, NULL, 2, NULL),
            (1567, N'Pedagógus I.', N'Pedagógus I.', NULL, NULL, NULL, N'EletpalyamodellFokozatTipus', 53, N'F', N'T', 3, NULL, 3, NULL),
            (1568, N'Pedagógus II.', N'Pedagógus II.', NULL, NULL, NULL, N'EletpalyamodellFokozatTipus', 53, N'F', N'T', 4, NULL, 4, NULL),
            (1569, N'Mesterpedagógus', N'Mesterpedagógus', NULL, NULL, NULL, N'EletpalyamodellFokozatTipus', 53, N'F', N'T', 5, NULL, 5, NULL),
            (1570, N'Kutatótanár', N'Kutatótanár', NULL, NULL, NULL, N'EletpalyamodellFokozatTipus', 53, N'F', N'T', 6, NULL, 6, NULL),
            (6521, N'Na', N'Na', NULL, NULL, NULL, N'EletpalyamodellStatuszok', 54, N'F', N'T', 1, NULL, 1, NULL),
            (6522, N'megfelelt', N'megfelelt', NULL, NULL, NULL, N'EletpalyamodellStatuszok', 54, N'F', N'T', 2, NULL, 2, NULL),
            (6523, N'újabb minősítés szükséges', N'újabb minősítés szükséges', NULL, NULL, NULL, N'EletpalyamodellStatuszok', 54, N'F', N'T', 3, NULL, 3, NULL),
            (6524, N'nem felelt meg', N'nem felelt meg', NULL, NULL, NULL, N'EletpalyamodellStatuszok', 54, N'F', N'T', 4, NULL, 4, NULL),
            (6525, N'Mesterpedagógus fokozatba lép', N'Mesterpedagógus fokozatba lép', NULL, NULL, NULL, N'EletpalyamodellStatuszok', 54, N'F', N'T', 5, NULL, 5, NULL),
            (6526, N'Pedagógus II. fokozatban marad', N'Pedagógus II. fokozatban marad', NULL, NULL, NULL, N'EletpalyamodellStatuszok', 54, N'F', N'T', 6, NULL, 6, NULL),
            (6527, N'Kutatótanár fokozatba lép', N'Kutatótanár fokozatba lép', NULL, NULL, NULL, N'EletpalyamodellStatuszok', 54, N'F', N'T', 7, NULL, 7, NULL),
            (6528, N'Mesterpedagógus fokozatban marad', N'Mesterpedagógus fokozatban marad', NULL, NULL, NULL, N'EletpalyamodellStatuszok', 54, N'F', N'T', 8, NULL, 8, NULL),
            (1558, N'Rossz', N'Poor', N'Nicht angemessen', NULL, NULL, N'MagatartasErtekelesTipus', 50, N'F', N'T', 1, NULL, 1, NULL),
            (1557, N'Változó', N'Could do better', N'Veränderlich', NULL, NULL, N'MagatartasErtekelesTipus', 50, N'F', N'T', 2, NULL, 2, NULL),
            (1556, N'Jó', N'Good', N'Gut', NULL, NULL, N'MagatartasErtekelesTipus', 50, N'F', N'T', 3, NULL, 3, NULL),
            (1555, N'Példás', N'Exemplary', N'Vorbildlich', NULL, NULL, N'MagatartasErtekelesTipus', 50, N'F', N'T', 4, NULL, 4, NULL),
            (1562, N'Hanyag', N'Poor', N'Nachlässig', NULL, NULL, N'SzorgalomErtekelesTipus', 51, N'F', N'T', 1, NULL, 1, NULL),
            (1561, N'Változó', N'Could do better', N'Veränderlich', NULL, NULL, N'SzorgalomErtekelesTipus', 51, N'F', N'T', 2, NULL, 2, NULL),
            (1560, N'Jó', N'Good', N'Gut', NULL, NULL, N'SzorgalomErtekelesTipus', 51, N'F', N'T', 3, NULL, 3, NULL),
            (1559, N'Példás', N'Exemplary', N'Vorbildlich', NULL, NULL, N'SzorgalomErtekelesTipus', 51, N'F', N'T', 4, NULL, 4, NULL),
            (1534, N'Na', N'Na', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 1, NULL, 1, NULL),
            (6315, N'Szaktanári dicséret', N'Szaktanári dicséret', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 2, NULL, 2, NULL),
            (5450, N'Szakoktatói dicséret', N'Szakoktatói dicséret', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 3, NULL, 3, NULL),
            (5451, N'Szaktárgyi dicséret', N'Szaktárgyi dicséret', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 4, NULL, 4, NULL),
            (6312, N'Szaktanári figyelmeztetés', N'Szaktanári figyelmeztetés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 5, NULL, 5, NULL),
            (5460, N'Gyakorlati oktatói figyelmeztetés', N'Gyakorlati oktatói figyelmeztetés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 6, NULL, 6, NULL),
            (6313, N'Szaktanári intés', N'Szaktanári intés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 7, NULL, 7, NULL),
            (6314, N'Szaktanári megrovás', N'Szaktanári megrovás', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 8, NULL, 8, NULL),
            (6319, N'Osztályfőnöki dicséret', N'Osztályfőnöki dicséret', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 9, NULL, 9, NULL),
            (6316, N'Osztályfőnöki figyelmeztetés', N'Osztályfőnöki figyelmeztetés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 10, NULL, 10, NULL),
            (6317, N'Osztályfőnöki intés', N'Osztályfőnöki intés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 11, NULL, 11, NULL),
            (6318, N'Osztályfőnöki megrovás', N'Osztályfőnöki megrovás', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 12, NULL, 12, NULL),
            (5456, N'Igazgatóhelyettesi dicséret', N'Igazgatóhelyettesi dicséret', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 13, NULL, 13, NULL),
            (5467, N'Igazgatóhelyettesi figyelmeztetés', N'Igazgatóhelyettesi figyelmeztetés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 14, NULL, 14, NULL),
            (5468, N'Igazgatóhelyettesi intés', N'Igazgatóhelyettesi intés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 15, NULL, 15, NULL),
            (5469, N'Igazgatóhelyettesi megrovás', N'Igazgatóhelyettesi megrovás', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 16, NULL, 16, NULL),
            (5457, N'Intézményvezetői dicséret', N'Intézményvezetői dicséret', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 17, NULL, 17, NULL),
            (6323, N'Igazgatói dicséret', N'Igazgatói dicséret', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 18, NULL, 18, NULL),
            (6320, N'Igazgatói figyelmeztetés', N'Igazgatói figyelmeztetés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 19, NULL, 19, NULL),
            (6321, N'Igazgatói intés', N'Igazgatói intés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 20, NULL, 20, NULL),
            (6322, N'Igazgatói megrovás', N'Igazgatói megrovás', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 21, NULL, 21, NULL),
            (5473, N'Igazgatói szigorú megrovás', N'Igazgatói szigorú megrovás', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 22, NULL, 22, NULL),
            (5458, N'Nevelőtestületi dicséret', N'Nevelőtestületi dicséret', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 23, NULL, 23, NULL),
            (5470, N'Nevelőtestületi figyelmeztetés', N'Nevelőtestületi figyelmeztetés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 24, NULL, 24, NULL),
            (5471, N'Nevelőtestületi intés', N'Nevelőtestületi intés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 25, NULL, 25, NULL),
            (5472, N'Nevelőtestületi megrovás', N'Nevelőtestületi megrovás', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 26, NULL, 26, NULL),
            (5454, N'Nevelőtanári dicséret', N'Nevelőtanári dicséret', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 27, NULL, 27, NULL),
            (5461, N'Nevelőtanári figyelmeztetés', N'Nevelőtanári figyelmeztetés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 28, NULL, 28, NULL),
            (5462, N'Nevelőtanári intés', N'Nevelőtanári intés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 29, NULL, 29, NULL),
            (5463, N'Nevelőtanári megrovás', N'Nevelőtanári megrovás', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 30, NULL, 30, NULL),
            (5459, N'Napközis nevelői figyelmeztetés', N'Napközis nevelői figyelmeztetés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 31, NULL, 31, NULL),
            (5455, N'Csoportnevelői dicséret', N'Csoportnevelői dicséret', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 32, NULL, 32, NULL),
            (5464, N'Csoportnevelői figyelmeztetés', N'Csoportnevelői figyelmeztetés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 33, NULL, 33, NULL),
            (5465, N'Csoportnevelői intés', N'Csoportnevelői intés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 34, NULL, 34, NULL),
            (5466, N'Csoportnevelői megrovás', N'Csoportnevelői megrovás', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 35, NULL, 35, NULL),
            (5453, N'Diákönkormányzat-vezetői dicséret', N'Diákönkormányzat-vezetői dicséret', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 36, NULL, 36, NULL),
            (1546, N'A közösségi szolgálat teljesítésével kapcsolatos adatok', N'A közösségi szolgálat teljesítésével kapcsolatos adatok', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 37, NULL, 37, NULL),
            (6326, N'Figyelmeztetés bukásra állásról', N'Figyelmeztetés bukásra állásról', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 38, NULL, 38, NULL),
            (6327, N'Figyelmeztetés gyenge tanulmányi teljesítményről', N'Figyelmeztetés gyenge tanulmányi teljesítményről', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 39, NULL, 39, NULL),
            (6328, N'Tájékoztatás', N'Tájékoztatás', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 40, NULL, 40, NULL),
            (5452, N'Általános tanulmányi dicséret', N'Általános tanulmányi dicséret', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 41, NULL, 41, NULL),
            (1537, N'Baleset', N'Baleset', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 42, NULL, 42, NULL),
            (1536, N'Dicséret', N'Dicséret', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 43, NULL, 43, NULL),
            (1538, N'Egészségügyi', N'Egészségügyi', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 44, NULL, 44, NULL),
            (5482, N'Elektronikus üzenet', N'Elektronikus üzenet', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 45, NULL, 45, NULL),
            (1539, N'Évismétlés', N'Évismétlés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 46, NULL, 46, NULL),
            (5483, N'Faliújság bejegyzés', N'Faliújság bejegyzés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 47, NULL, 47, NULL),
            (1535, N'Fegyelmi', N'Fegyelmi', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 48, NULL, 48, NULL),
            (6324, N'Felszereléshiány', N'Felszereléshiány', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 49, NULL, 49, NULL),
            (6325, N'Házi feladat hiány', N'Házi feladat hiány', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 50, NULL, 50, NULL),
            (1541, N'Javítóvizsga', N'Javítóvizsga', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 51, NULL, 51, NULL),
            (6491, N'Osztályozó vizsgára való kiírás', N'Osztályozó vizsgára való kiírás', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 52, NULL, 52, NULL),
            (1540, N'Kimaradás', N'Kimaradás', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 53, NULL, 53, NULL),
            (1543, N'Határozat', N'Határozat', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 54, NULL, 54, NULL),
            (1542, N'Szakmai mentesség (nem hivatalos)', N'Szakmai mentesség (nem hivatalos)', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 55, NULL, 55, NULL),
            (1544, N'Szakmai gyakorlaton vesz részt (határozat)', N'Szakmai gyakorlaton vesz részt (határozat)', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 56, NULL, 56, NULL),
            (1553, N'Törzslapon megjelenő egyéb feljegyzés vagy határozat', N'Törzslapon megjelenő egyéb feljegyzés vagy határozat', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 57, NULL, 57, NULL),
            (1545, N'Feljegyzések a személyi adatok változásához (törzslapon megjelenik)', N'Feljegyzések a személyi adatok változásához (törzslapon megjelenik)', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 58, NULL, 58, NULL),
            (1547, N'Félévi bizonyítványban megjelenő tantestületi feljegyzés, határozat (dicséret, felmentés stb.)', N'Félévi bizonyítványban megjelenő tantestületi feljegyzés, határozat (dicséret, felmentés stb.)', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 59, NULL, 59, NULL),
            (1548, N'Év végi bizonyítványban megjelenő tantestületi feljegyzés, határozat (dicséret, felmentés stb.)', N'Év végi bizonyítványban megjelenő tantestületi feljegyzés, határozat (dicséret, felmentés stb.)', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 60, NULL, 60, NULL),
            (1549, N'I. negyedéves bizonyítványban megjelenő tantestületi feljegyzés, határozat (dicséret, felmentés stb.)', N'I. negyedéves bizonyítványban megjelenő tantestületi feljegyzés, határozat (dicséret, felmentés stb.)', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 61, NULL, 61, NULL),
            (1550, N'II. negyedéves bizonyítványban megjelenő tantestületi feljegyzés, határozat (dicséret, felmentés stb.)', N'II. negyedéves bizonyítványban megjelenő tantestületi feljegyzés, határozat (dicséret, felmentés stb.)', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 62, NULL, 62, NULL),
            (1551, N'III. negyedéves bizonyítványban megjelenő tantestületi feljegyzés, határozat (dicséret, felmentés stb.)', N'III. negyedéves bizonyítványban megjelenő tantestületi feljegyzés, határozat (dicséret, felmentés stb.)', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 63, NULL, 63, NULL),
            (1552, N'IV. negyedéves bizonyítványban megjelenő tantestületi feljegyzés, határozat (dicséret, felmentés stb.)', N'IV. negyedéves bizonyítványban megjelenő tantestületi feljegyzés, határozat (dicséret, felmentés stb.)', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 64, NULL, 64, NULL),
            (5477, N'Áthelyezés másik osztályba, tanulócsoportba vagy iskolába (fegyelmi büntetés)', N'Áthelyezés másik osztályba, tanulócsoportba vagy iskolába (fegyelmi büntetés)', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 65, NULL, 65, NULL),
            (5478, N'Áthelyezés másik szobába, tanulócsoportba (fegyelmi büntetés)', N'Áthelyezés másik szobába, tanulócsoportba (fegyelmi büntetés)', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 66, NULL, 66, NULL),
            (5476, N'Meghatározott kedvezmények, juttatások csökkentése, megvonása (fegyelmi büntetés)', N'Meghatározott kedvezmények, juttatások csökkentése, megvonása (fegyelmi büntetés)', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 67, NULL, 67, NULL),
            (5474, N'Megrovás (fegyelmi büntetés)', N'Megrovás (fegyelmi büntetés)', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 68, NULL, 68, NULL),
            (5475, N'Szigorú megrovás (fegyelmi büntetés)', N'Szigorú megrovás (fegyelmi büntetés)', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 69, NULL, 69, NULL),
            (5479, N'Eltiltás az adott iskolában a tanév folytatásától (fegyelmi büntetés)', N'Eltiltás az adott iskolában a tanév folytatásától (fegyelmi büntetés)', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 70, NULL, 70, NULL),
            (5481, N'Kizárás (fegyelmi büntetés)', N'Kizárás (fegyelmi büntetés)', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 71, NULL, 71, NULL),
            (5480, N'Kizárás az iskolából (fegyelmi büntetés)', N'Kizárás az iskolából (fegyelmi büntetés)', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 72, NULL, 72, NULL),
            (6870, N'Első figyelmeztetés késések miatt', N'Első figyelmeztetés késések miatt', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 73, NULL, 73, NULL),
            (6871, N'Figyelmeztetés közösségi programról való távolmaradás miatt', N'Figyelmeztetés közösségi programról való távolmaradás miatt', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 74, NULL, 74, NULL),
            (6873, N'Intézményvezetői figyelmeztetés', N'Intézményvezetői figyelmeztetés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 76, NULL, 75, NULL),
            (6874, N'Intézményvezetői intés', N'Intézményvezetői intés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 77, NULL, 76, NULL),
            (6875, N'Intézményvezetői megrovás', N'Intézményvezetői megrovás', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 78, NULL, 77, NULL),
            (6872, N'Szaktanári tanulmányi figyelmeztetés', N'Szaktanári tanulmányi figyelmeztetés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 79, NULL, 78, NULL),
            (6906, N'Tagozatvezetői figyelmeztetés', N'Tagozatvezetői figyelmeztetés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 80, NULL, 79, NULL),
            (6907, N'Tagozatvezetői intés', N'Tagozatvezetői intés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 81, NULL, 80, NULL),
            (6945, N'Kollégiumvezetői dicséret', N'Kollégiumvezetői dicséret', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 81, NULL, 81, NULL),
            (6946, N'Kollégiumvezetői figyelmeztetés', N'Kollégiumvezetői figyelmeztetés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 82, NULL, 82, NULL),
            (6947, N'Kollégiumvezetői intés', N'Kollégiumvezetői intés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 83, NULL, 83, NULL),
            (6970, N'Szakoktatói figyelmeztetés', N'Szakoktatói figyelmeztetés', NULL, NULL, NULL, N'EsemenyTipus', 48, N'T', N'T', 84, NULL, 84, NULL),
            (1530, N'Orvosi igazolás', N'Orvosi igazolás', NULL, NULL, NULL, N'IgazolasTipus', 47, N'T', N'T', 100, NULL, 1, NULL),
            (1531, N'Szülői igazolás', N'Szülői igazolás', NULL, NULL, NULL, N'IgazolasTipus', 47, N'T', N'T', 110, NULL, 2, NULL),
            (1532, N'Hivatalos távollét', N'Hivatalos távollét', NULL, NULL, NULL, N'IgazolasTipus', 47, N'T', N'T', 130, NULL, 3, NULL),
            (1533, N'Iskolaérdekű távollét', N'Iskolaérdekű távollét', NULL, NULL, NULL, N'IgazolasTipus', 47, N'T', N'T', 120, NULL, 4, NULL),
            (6494, N'Szolgáltatói igazolás', N'Szolgáltatói igazolás', NULL, NULL, NULL, N'IgazolasTipus', 47, N'T', N'T', 180, NULL, 5, NULL),
            (6495, N'Iskolai engedély', N'Iskolai engedély', NULL, NULL, NULL, N'IgazolasTipus', 47, N'T', N'T', 160, NULL, 6, NULL),
            (6967, N'Pályaválasztási célú igazolás', N'Pályaválasztási célú igazolás', NULL, NULL, NULL, N'IgazolasTipus', 47, N'T', N'T', 150, NULL, 7, NULL),
            (6834, N'Kikérő', N'Kikérő', NULL, NULL, NULL, N'IgazolasTipus', 47, N'T', N'T', 140, NULL, 8, NULL),
            (6853, N'Táppénz', N'Táppénz', NULL, NULL, NULL, N'IgazolasTipus', 47, N'T', N'T', 190, NULL, 9, NULL),
            (1529, N'Egyéb igazolás', N'Egyéb', NULL, NULL, NULL, N'IgazolasTipus', 47, N'T', N'T', 200, NULL, 10, NULL),
            (8830, N'Hatósági házi karantén', N'Hatósági házi karantén', NULL, NULL, NULL, N'IgazolasTipus', 47, N'T', N'T', 170, NULL, 11, NULL),
            (9000, N'Szabadság', N'Szabadság', NULL, NULL, NULL, N'IgazolasTipus', 47, N'T', N'F', 12, NULL, 12, NULL),
            (9115, N'Pótlás', N'Pótlás', NULL, NULL, NULL, N'IgazolasTipus', 47, N'T', N'F', 13, NULL, 13, NULL),
            (9116, N'Betegszabadság', N'Betegszabadság', NULL, NULL, NULL, N'IgazolasTipus', 47, N'T', N'F', 14, NULL, 14, NULL),
            (1497, N'Na', N'Na', NULL, NULL, NULL, N'MulasztasTipus', 43, N'F', N'T', 1, NULL, 1, NULL),
            (1498, N'Jelenlét', N'Jelenlét', NULL, NULL, NULL, N'MulasztasTipus', 43, N'F', N'T', 2, NULL, 2, NULL),
            (1499, N'Késés', N'Késés', NULL, NULL, NULL, N'MulasztasTipus', 43, N'F', N'T', 3, NULL, 3, NULL),
            (1500, N'Hiányzás', N'Hiányzás', NULL, NULL, NULL, N'MulasztasTipus', 43, N'F', N'T', 4, NULL, 4, NULL),
            (1817, N'Üres', N'Üres', NULL, NULL, NULL, N'MulasztasTipus', 43, N'F', N'T', 5, NULL, 5, NULL),
            (9084, N'Szabadság', N'Szabadság', NULL, NULL, NULL, N'MulasztasTipus', 43, N'F', N'T', 7, NULL, 6, NULL),
            (9112, N'Táppénz', N'Táppénz', NULL, NULL, NULL, N'MulasztasTipus', 43, N'F', N'T', 8, NULL, 7, NULL),
            (9113, N'Betegszabadság', N'Betegszabadság', NULL, NULL, NULL, N'MulasztasTipus', 43, N'F', N'T', 9, NULL, 8, NULL),
            (9114, N'Pótlás', N'Pótlás', NULL, NULL, NULL, N'MulasztasTipus', 43, N'F', N'T', 6, NULL, 9, NULL),
            (1506, N'Na', N'Na', NULL, NULL, NULL, N'ErtekelesMod', 45, N'T', N'T', 1, NULL, 1, NULL),
            (1507, N'Írásbeli témazáró dolgozat', N'Írásbeli témazáró dolgozat', NULL, NULL, NULL, N'ErtekelesMod', 45, N'T', N'T', 2, NULL, 2, NULL),
            (1508, N'Írásbeli röpdolgozat', N'Írásbeli röpdolgozat', NULL, NULL, NULL, N'ErtekelesMod', 45, N'T', N'T', 3, NULL, 3, NULL),
            (1509, N'Szóbeli felelet', N'Szóbeli felelet', NULL, NULL, NULL, N'ErtekelesMod', 45, N'T', N'T', 4, NULL, 4, NULL),
            (1510, N'Beszámoló', N'Beszámoló', NULL, NULL, NULL, N'ErtekelesMod', 45, N'T', N'T', 5, NULL, 5, NULL)
                ) x (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
                WHERE NOT EXISTS (SELECT 1 FROM #DICTIONARYITEMBASE d WHERE x.ID = d.ID)
                
    INSERT INTO #DICTIONARYITEMBASE (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
        SELECT * FROM ( VALUES
            (1511, N'Gyakorlati feladat', N'Gyakorlati feladat', NULL, NULL, NULL, N'ErtekelesMod', 45, N'T', N'T', 6, NULL, 6, NULL),
            (1512, N'Kisérettségi', N'Kisérettségi', NULL, NULL, NULL, N'ErtekelesMod', 45, N'T', N'T', 7, NULL, 7, NULL),
            (1513, N'Házi feladat', N'Házi feladat', NULL, NULL, NULL, N'ErtekelesMod', 45, N'T', N'T', 8, NULL, 8, NULL),
            (1514, N'Házi dolgozat', N'Házi dolgozat', NULL, NULL, NULL, N'ErtekelesMod', 45, N'T', N'T', 9, NULL, 9, NULL),
            (1515, N'Projektmunka', N'Projektmunka', NULL, NULL, NULL, N'ErtekelesMod', 45, N'T', N'T', 10, NULL, 10, NULL),
            (1516, N'Órai munka', N'Órai munka', NULL, NULL, NULL, N'ErtekelesMod', 45, N'T', N'T', 11, NULL, 11, NULL),
            (6490, N'Másik intézményből hozott értékelés', N'Másik intézményből hozott értékelés', NULL, NULL, NULL, N'ErtekelesMod', 45, N'T', N'T', 12, NULL, 12, NULL),
            (8373, N'Órai feladat', N'Órai feladat', NULL, NULL, NULL, N'ErtekelesMod', 45, N'T', N'T', 13, NULL, 13, NULL),
            (8374, N'E-learning feladat', N'E-learning feladat', NULL, NULL, NULL, N'ErtekelesMod', 45, N'T', N'T', 14, NULL, 14, NULL),
            (8375, N'Teszt feladat', N'Teszt feladat', NULL, NULL, NULL, N'ErtekelesMod', 45, N'T', N'T', 15, NULL, 15, NULL),
            (8376, N'Feladatlap', N'Feladatlap', NULL, NULL, NULL, N'ErtekelesMod', 45, N'T', N'T', 16, NULL, 16, NULL),
            (8756, N'Nyelvi feladat', N'Nyelvi feladat', NULL, NULL, NULL, N'ErtekelesMod', 45, N'T', N'T', 17, NULL, 17, NULL),
            (1505, N'Jeles(5)', N'Excellent (5)', N'Sehr gut (5)', NULL, NULL, N'OsztalyzatTipus', 44, N'F', N'T', 1, NULL, 5, NULL),
            (1504, N'Jó(4)', N'Good (4)', N'Gut (4)', NULL, NULL, N'OsztalyzatTipus', 44, N'F', N'T', 2, NULL, 4, NULL),
            (1503, N'Közepes(3)', N'Satisfactory (3)', N'Befriedigend (3)', NULL, NULL, N'OsztalyzatTipus', 44, N'F', N'T', 3, NULL, 3, NULL),
            (1502, N'Elégséges(2)', N'Pass (2)', N'Ausreichend (2)', NULL, NULL, N'OsztalyzatTipus', 44, N'F', N'T', 4, NULL, 2, NULL),
            (1501, N'Elégtelen(1)', N'Fail (1)', N'Ungenügend (1)', NULL, NULL, N'OsztalyzatTipus', 44, N'F', N'T', 5, NULL, 1, NULL),
            (1517, N'Na', N'Na', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'F', 1, NULL, 1, NULL),
            (1518, N'Évközi jegy/értékelés', N'Évközi jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 20, NULL, 2, NULL),
            (1519, N'Félévi jegy/értékelés', N'Félévi jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 30, NULL, 3, NULL),
            (1520, N'Év végi jegy/értékelés', N'Év végi jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 40, NULL, 4, NULL),
            (1521, N'Modulzáró vizsga', N'Modulzáró vizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 600, NULL, 5, NULL),
            (1522, N'I. né. jegy/értékelés', N'I. né. jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 50, NULL, 6, NULL),
            (1523, N'III. né. jegy/értékelés', N'III. né. jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 70, NULL, 7, NULL),
            (1524, N'II. né. jegy/értékelés', N'II. né. jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 60, NULL, 8, NULL),
            (1525, N'IV. né. jegy/értékelés', N'IV. né. jegy/értékelés', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 80, NULL, 9, NULL),
            (1526, N'Pótvizsga', N'Pótvizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 610, NULL, 10, NULL),
            (1527, N'Osztályozó vizsga', N'Osztályozó vizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 100, NULL, 11, NULL),
            (1528, N'Különbözeti vizsga', N'Különbözeti vizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 110, NULL, 12, NULL),
            (6916, N'Pótló vizsga', N'Pótló vizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 120, NULL, 13, NULL),
            (6917, N'Javítóvizsga', N'Javítóvizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 130, NULL, 14, NULL),
            (6918, N'Beszámoltató vizsga', N'Beszámoltató vizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 140, NULL, 15, NULL),
            (7897, N'Ágazati alapvizsga', N'Ágazati alapvizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'F', 200, NULL, 16, NULL),
            (7898, N'Szakmai vizsga', N'Szakmai vizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'F', 210, NULL, 17, NULL),
            (7899, N'Művészeti alapvizsga', N'Művészeti alapvizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 300, NULL, 18, NULL),
            (7900, N'Művészeti záróvizsga', N'Művészeti záróvizsga', NULL, NULL, NULL, N'ErtekelesTipus', 46, N'T', N'T', 310, NULL, 19, NULL),
            (1493, N'Na', N'Na', NULL, NULL, NULL, N'HelyettesitesTipus', 42, N'F', N'T', 1, NULL, 1, NULL),
            (1494, N'Szakszerű helyettesítés', N'Szakszerű helyettesítés', NULL, NULL, NULL, N'HelyettesitesTipus', 42, N'F', N'T', 2, NULL, 2, NULL),
            (1495, N'Nem szakszerű helyettesítés (felügyelet)', N'Nem szakszerű helyettesítés (felügyelet)', NULL, NULL, NULL, N'HelyettesitesTipus', 42, N'F', N'T', 3, NULL, 3, NULL),
            (1496, N'Óraösszevonás', N'Óraösszevonás', NULL, NULL, NULL, N'HelyettesitesTipus', 42, N'F', N'T', 4, NULL, 4, NULL),
            (7367, N'Összevont felügyelet', N'Összevont felügyelet', NULL, NULL, NULL, N'HelyettesitesTipus', 42, N'F', N'T', 5, NULL, 5, NULL),
            (1407, N'Na', N'Na', NULL, NULL, NULL, N'HetNapjaTipus', 40, N'F', N'T', 1, NULL, 0, NULL),
            (1408, N'Hétfő', N'Hétfő', NULL, NULL, NULL, N'HetNapjaTipus', 40, N'F', N'T', 2, NULL, 1, NULL),
            (1409, N'Kedd', N'Kedd', NULL, NULL, NULL, N'HetNapjaTipus', 40, N'F', N'T', 3, NULL, 2, NULL),
            (1410, N'Szerda', N'Szerda', NULL, NULL, NULL, N'HetNapjaTipus', 40, N'F', N'T', 4, NULL, 3, NULL),
            (1411, N'Csütörtök', N'Csütörtök', NULL, NULL, NULL, N'HetNapjaTipus', 40, N'F', N'T', 5, NULL, 4, NULL),
            (1412, N'Péntek', N'Péntek', NULL, NULL, NULL, N'HetNapjaTipus', 40, N'F', N'T', 6, NULL, 5, NULL),
            (1413, N'Szombat', N'Szombat', NULL, NULL, NULL, N'HetNapjaTipus', 40, N'F', N'T', 7, NULL, 6, NULL),
            (1414, N'Vasárnap', N'Vasárnap', NULL, NULL, NULL, N'HetNapjaTipus', 40, N'F', N'T', 8, NULL, 7, NULL),
            (1554, N'Minden héten', N'Minden héten', NULL, NULL, NULL, N'HetiRendTipus', 49, N'T', N'T', 1, NULL, 1, NULL),
            (1384, N'Na', N'Na', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 1, NULL, 1, NULL),
            (1385, N'Tanítási nap', N'Tanítási nap', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 2, NULL, 2, NULL),
            (1386, N'Munkaszüneti nap', N'Munkaszüneti nap', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 3, NULL, 3, NULL),
            (1387, N'Tanítás nélküli munkanap', N'Tanítás nélküli munkanap', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 4, NULL, 4, NULL),
            (1388, N'Szülői értekezlet (tanítás nélkül)', N'Szülői értekezlet (tanítás nélkül)', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 5, NULL, 5, NULL),
            (1389, N'Egész napos kirándulás', N'Egész napos kirándulás', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 6, NULL, 6, NULL),
            (1390, N'Fogadónap (tanítás nélküli)', N'Fogadónap (tanítás nélküli)', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 7, NULL, 7, NULL),
            (1391, N'Szabadnap (intézményvezetői hatáskörben)', N'Szabadnap (intézményvezetői hatáskörben)', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 8, NULL, 8, NULL),
            (1392, N'Tanítási szünet', N'Tanítási szünet', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 9, NULL, 9, NULL),
            (1393, N'Rövidített órákat tartalmazó tanítási nap', N'Rövidített órákat tartalmazó tanítási nap', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 10, NULL, 10, NULL),
            (1394, N'Első tanítási nap', N'Első tanítási nap', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 11, NULL, 11, NULL),
            (1395, N'Utolsó tanítási nap', N'Utolsó tanítási nap', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 12, NULL, 12, NULL),
            (1396, N'Ünnepnap', N'Ünnepnap', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 13, NULL, 13, NULL),
            (1397, N'Téli szünet', N'Téli szünet', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 14, NULL, 14, NULL),
            (1398, N'Tavaszi szünet', N'Tavaszi szünet', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 15, NULL, 15, NULL),
            (1399, N'Őszi szünet', N'Őszi szünet', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 16, NULL, 16, NULL),
            (1400, N'Első félév vége', N'Első félév vége', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 17, NULL, 17, NULL),
            (1401, N'Vizsganap', N'Vizsganap', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 18, NULL, 18, NULL),
            (1402, N'Utolsó tanítási nap a végzős évfolyamokon', N'Utolsó tanítási nap a végzős évfolyamokon', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 19, NULL, 19, NULL),
            (1403, N'I. negyedév vége', N'I. negyedév vége', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 20, NULL, 20, NULL),
            (1404, N'III. negyedév vége', N'III. negyedév vége', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 21, NULL, 21, NULL),
            (1405, N'Egybefüggő szakmai gyakorlati nap', N'Egybefüggő szakmai gyakorlati nap', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 22, NULL, 22, NULL),
            (1406, N'Rendkívüli tanítási nap', N'Rendkívüli tanítási nap', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 23, NULL, 23, NULL),
            (1649, N'Nyílt nap', N'Nyílt nap', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 24, NULL, 24, NULL),
            (6833, N'Pihenőnap', N'Pihenőnap', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 25, NULL, 25, NULL),
            (7600, N'Utolsó tanítási nap a végzős évfolyamokon (rendészeti iskolákban)', N'Utolsó tanítási nap a végzős évfolyamokon (rendészeti iskolákban)', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 26, NULL, 26, NULL),
            (7601, N'Utolsó tanítási nap a végzős évfolyamokon (honvédelmi iskolákban)', N'Utolsó tanítási nap a végzős évfolyamokon (honvédelmi iskolákban)', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 27, NULL, 27, NULL),
            (7602, N'Utolsó tanítási nap a végzős évfolyamokon (keresztféléves képzésben)', N'Utolsó tanítási nap a végzős évfolyamokon (keresztféléves képzésben)', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 28, NULL, 28, NULL),
            (7603, N'Utolsó tanítási nap a végzős évfolyamokon (részszakképesítéses oktatásban)', N'Utolsó tanítási nap a végzős évfolyamokon (részszakképesítéses oktatásban)', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 29, NULL, 29, NULL),
            (7604, N'Első tanítási nap (keresztféléves képzésben)', N'Első tanítási nap (keresztféléves képzésben)', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 30, NULL, 30, NULL),
            (7605, N'Első félév vége (keresztféléves képzésben)', N'Első félév vége (keresztféléves képzésben)', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 31, NULL, 31, NULL),
            (8407, N'Rendkívüli tanítási szünet', N'Rendkívüli tanítási szünet', NULL, NULL, NULL, N'NapTipus', 39, N'T', N'T', 32, NULL, 32, NULL),
            (1338, N'Na', N'Na', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 1, NULL, 1, NULL),
            (1339, N'Tanóra', N'Tanóra', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 2, NULL, 2, NULL),
            (1340, N'Napközi', N'Napközi', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 3, NULL, 3, NULL),
            (1341, N'Tanulószoba', N'Tanulószoba', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 4, NULL, 4, NULL),
            (1342, N'Rehabilitáció', N'Rehabilitáció', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 5, NULL, 5, NULL),
            (1343, N'Zeneiskolai egyéni', N'Zeneiskolai egyéni', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 6, NULL, 6, NULL),
            (1344, N'Gyógytestnevelés', N'Gyógytestnevelés', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 7, NULL, 7, NULL),
            (1345, N'Szakkör', N'Szakkör', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 8, NULL, 8, NULL),
            (1346, N'Érdeklődési kör', N'Érdeklődési kör', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 9, NULL, 9, NULL),
            (1347, N'Önképzőkör', N'Önképzőkör', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 10, NULL, 10, NULL),
            (1348, N'Énekkar', N'Énekkar', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 11, NULL, 11, NULL),
            (1349, N'Művészeti csoport', N'Művészeti csoport', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 12, NULL, 12, NULL),
            (1350, N'Iskolai sportkör', N'Iskolai sportkör', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 13, NULL, 13, NULL),
            (1351, N'Tanulmányi verseny', N'Tanulmányi verseny', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 14, NULL, 14, NULL),
            (1352, N'Szakmai verseny', N'Szakmai verseny', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 15, NULL, 15, NULL),
            (1353, N'Kulturális verseny', N'Kulturális verseny', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 16, NULL, 16, NULL),
            (1354, N'Házi bajnokság', N'Házi bajnokság', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 17, NULL, 17, NULL),
            (1355, N'Iskolák közötti verseny', N'Iskolák közötti verseny', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 18, NULL, 18, NULL),
            (1356, N'Iskolák közötti bajnokság', N'Iskolák közötti bajnokság', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 19, NULL, 19, NULL),
            (1357, N'Diáknap', N'Diáknap', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 20, NULL, 20, NULL),
            (1358, N'Tanulmányi kirándulás', N'Tanulmányi kirándulás', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 21, NULL, 21, NULL),
            (1359, N'Környezeti nevelés', N'Környezeti nevelés', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 22, NULL, 22, NULL),
            (1360, N'Kulturális rendezvény', N'Kulturális rendezvény', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 23, NULL, 23, NULL),
            (1361, N'Sportrendezvény', N'Sportrendezvény', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 24, NULL, 24, NULL),
            (1362, N'Felzárkóztató foglalkozás', N'Felzárkóztató foglalkozás', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 25, NULL, 25, NULL),
            (1363, N'Fejlesztő foglalkozás', N'Fejlesztő foglalkozás', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 26, NULL, 26, NULL),
            (1364, N'Tehetséggondozó foglalkozás', N'Tehetséggondozó foglalkozás', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 27, NULL, 27, NULL),
            (1365, N'Konzultáció', N'Konzultáció', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 28, NULL, 28, NULL),
            (1366, N'Speciális ismeretek átadása', N'Speciális ismeretek átadása', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 29, NULL, 29, NULL),
            (1367, N'Kiegészítő ismeretek átadása', N'Kiegészítő ismeretek átadása', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 30, NULL, 30, NULL),
            (1368, N'Tanóra (osztálybontásos)', N'Tanóra (osztálybontásos)', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 31, NULL, 31, NULL),
            (1369, N'Idegennyelvi többletóra', N'Idegennyelvi többletóra', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 32, NULL, 32, NULL),
            (1370, N'Habilitáció', N'Habilitáció', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 33, NULL, 33, NULL),
            (1371, N'Egyéni foglalkozás', N'Egyéni foglalkozás', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 34, NULL, 34, NULL),
            (1372, N'Óvódai foglalkozás', N'Óvódai foglalkozás', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 35, NULL, 35, NULL),
            (1373, N'Kollégiumi foglalkozás', N'Kollégiumi foglalkozás', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 36, NULL, 36, NULL),
            (1374, N'Kollégiumi ügyelet hétközben', N'Kollégiumi ügyelet hétközben', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 37, NULL, 37, NULL),
            (1375, N'Kollégiumi ügyelet hétvégén', N'Kollégiumi ügyelet hétvégén', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 38, NULL, 38, NULL),
            (1376, N'Diákotthoni foglalkozás', N'Diákotthoni foglalkozás', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 39, NULL, 39, NULL),
            (1377, N'Diákotthoni ügyelet hétközben', N'Diákotthoni ügyelet hétközben', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 40, NULL, 40, NULL),
            (1378, N'Diákotthoni ügyelet hétvégén', N'Diákotthoni ügyelet hétvégén', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 41, NULL, 41, NULL),
            (1379, N'Gyermekotthoni foglalkozás', N'Gyermekotthoni foglalkozás', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 42, NULL, 42, NULL),
            (1380, N'Gyermekotthoni ügyelet hétközben', N'Gyermekotthoni ügyelet hétközben', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 43, NULL, 43, NULL),
            (1381, N'Gyermekotthoni ügyelet hétvégén', N'Gyermekotthoni ügyelet hétvégén', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 44, NULL, 44, NULL),
            (1382, N'Egyéb foglalkozás', N'Egyéb foglalkozás', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 45, NULL, 45, NULL),
            (1383, N'Logopédiai foglalkozás', N'Logopédiai foglalkozás', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 46, NULL, 46, NULL),
            (1647, N'Szülői értekezlet', N'Szülői értekezlet', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 47, NULL, 47, NULL),
            (1648, N'Tanári értekezlet', N'Tanári értekezlet', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 48, NULL, 48, NULL),
            (8999, N'Duális foglalkozás', N'Duális foglalkozás', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'F', 49, NULL, 49, NULL),
            (9078, N'Apáczai mentori foglalkozás', N'Apáczai mentori foglalkozás', NULL, NULL, NULL, N'FoglalkozasTipus', 38, N'F', N'T', 50, NULL, 50, NULL),
            (1197, N'Na', N'Na', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1, NULL, 1, NULL),
            (1198, N'Magyar nyelv és irodalom', N'Magyar nyelv és irodalom', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1000, NULL, 2, NULL),
            (1200, N'Nemzetiségi nyelv és irodalom', N'Nemzetiségi nyelv és irodalom', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 4000, NULL, 3, NULL),
            (1201, N'Két tanítási nyelvű célnyelv', N'Két tanítási nyelvű célnyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1280, NULL, 4, NULL),
            (1202, N'Angol nyelv', N'Angol nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1200, NULL, 5, NULL),
            (1203, N'Német nyelv', N'Német nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1210, NULL, 6, NULL),
            (1204, N'Francia nyelv', N'Francia nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1230, NULL, 7, NULL),
            (1205, N'Olasz nyelv', N'Olasz nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1240, NULL, 8, NULL),
            (1206, N'Spanyol nyelv', N'Spanyol nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1250, NULL, 9, NULL),
            (6726, N'Báb- és színművészet', N'Báb- és színművészet', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1570, NULL, 10, NULL),
            (6727, N'Első idegen nyelv - Angol nyelv', N'Első idegen nyelv - Angol nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1300, NULL, 11, NULL),
            (6728, N'Első idegen nyelv - Egyéb idegen nyelv', N'Első idegen nyelv - Egyéb idegen nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1370, NULL, 12, NULL),
            (6729, N'Első idegen nyelv - Francia nyelv', N'Első idegen nyelv - Francia nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1330, NULL, 13, NULL),
            (6730, N'Első idegen nyelv - Latin nyelv', N'Első idegen nyelv - Latin nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1360, NULL, 14, NULL),
            (6731, N'Első idegen nyelv - Német nyelv', N'Első idegen nyelv - Német nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1310, NULL, 15, NULL),
            (6732, N'Első idegen nyelv - Olasz nyelv', N'Első idegen nyelv - Olasz nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1340, NULL, 16, NULL),
            (6733, N'Első idegen nyelv - Orosz nyelv', N'Első idegen nyelv - Orosz nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1320, NULL, 17, NULL),
            (6734, N'Első idegen nyelv - Spanyol nyelv', N'Első idegen nyelv - Spanyol nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1350, NULL, 18, NULL),
            (6735, N'Ember és természet', N'Ember és természet', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1100, NULL, 19, NULL),
            (6736, N'Képző- és iparművészet', N'Képző- és iparművészet', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1560, NULL, 20, NULL),
            (6737, N'Második idegen nyelv - Angol nyelv', N'Második idegen nyelv - Angol nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1400, NULL, 21, NULL),
            (6738, N'Második idegen nyelv - Egyéb idegen nyelv', N'Második idegen nyelv - Egyéb idegen nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1470, NULL, 22, NULL),
            (6739, N'Második idegen nyelv - Francia nyelv', N'Második idegen nyelv - Francia nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1430, NULL, 23, NULL),
            (6740, N'Második idegen nyelv - Latin nyelv', N'Második idegen nyelv - Latin nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1460, NULL, 24, NULL),
            (6741, N'Második idegen nyelv - Német nyelv', N'Második idegen nyelv - Német nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1410, NULL, 25, NULL),
            (6742, N'Második idegen nyelv - Olasz nyelv', N'Második idegen nyelv - Olasz nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1440, NULL, 26, NULL),
            (6743, N'Második idegen nyelv - Orosz nyelv', N'Második idegen nyelv - Orosz nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1420, NULL, 27, NULL),
            (6744, N'Második idegen nyelv - Spanyol nyelv', N'Második idegen nyelv - Spanyol nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1450, NULL, 28, NULL),
            (6745, N'Szakmai elmélet (Környezetvédelem - 14.)', N'Szakmai elmélet (Környezetvédelem - 14.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10140, NULL, 29, NULL),
            (6746, N'Szakmai elmélet (Közgazdaság - 15.)', N'Szakmai elmélet (Közgazdaság - 15.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10150, NULL, 30, NULL),
            (6747, N'Szakmai elmélet (Rendészet, honvédelem és közszolgálat - 22.)', N'Szakmai elmélet (Rendészet, honvédelem és közszolgálat - 22.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10220, NULL, 31, NULL),
            (6748, N'Szakmai elmélet (Vízügy - 23.)', N'Szakmai elmélet (Vízügy - 23.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10230, NULL, 32, NULL),
            (6749, N'Szakmai gyakorlat (Környezetvédelem - 14.)', N'Szakmai gyakorlat (Környezetvédelem - 14.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11140, NULL, 33, NULL),
            (6750, N'Szakmai gyakorlat (Közgazdaság - 15.)', N'Szakmai gyakorlat (Közgazdaság - 15.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11150, NULL, 34, NULL),
            (6751, N'Szakmai gyakorlat (Rendészet, honvédelem és közszolgálat - 22.)', N'Szakmai gyakorlat (Rendészet, honvédelem és közszolgálat - 22.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11220, NULL, 35, NULL),
            (6752, N'Szakmai gyakorlat (Vízügy - 23.)', N'Szakmai gyakorlat (Vízügy - 23.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11230, NULL, 36, NULL),
            (6753, N'Táncművészet', N'Táncművészet', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1550, NULL, 37, NULL),
            (6754, N'Történelem', N'Történelem', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1030, NULL, 38, NULL),
            (6755, N'Életvitel és gyakorlat', N'Életvitel és gyakorlat', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1600, NULL, 39, NULL),
            (1207, N'Orosz nyelv', N'Orosz nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1220, NULL, 40, NULL),
            (1208, N'Latin nyelv', N'Latin nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1260, NULL, 41, NULL),
            (1209, N'Egyéb idegen nyelv', N'Egyéb idegen nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1270, NULL, 42, NULL),
            (1210, N'Matematika', N'Matematika', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1010, NULL, 43, NULL),
            (1212, N'Filozófia', N'Filozófia', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1070, NULL, 44, NULL),
            (1213, N'Etika, hit- és erkölcstan, hittan', N'Etika, hit- és erkölcstan, hittan', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1040, NULL, 45, NULL),
            (1217, N'Hon- és népismeret', N'Hon- és népismeret', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1050, NULL, 46, NULL),
            (1218, N'Természetismeret', N'Természetismeret', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1130, NULL, 47, NULL),
            (1219, N'Fizika', N'Fizika', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1140, NULL, 48, NULL),
            (1220, N'Kémia', N'Kémia', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1150, NULL, 49, NULL),
            (1221, N'Biológia', N'Biológia', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1160, NULL, 50, NULL),
            (1222, N'Környezetismeret', N'Környezetismeret', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1110, NULL, 51, NULL),
            (1223, N'Földünk – környezetünk', N'Földünk – környezetünk', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1170, NULL, 52, NULL),
            (1224, N'Ének-zene', N'Ének-zene', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1510, NULL, 53, NULL),
            (1225, N'Dráma és tánc', N'Dráma és tánc', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1530, NULL, 54, NULL),
            (1226, N'Vizuális kultúra', N'Vizuális kultúra', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1520, NULL, 55, NULL),
            (1228, N'Mozgóképkultúra és médiaismeret', N'Mozgóképkultúra és médiaismeret', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1710, NULL, 56, NULL),
            (1230, N'Informatika', N'Informatika', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1700, NULL, 57, NULL),
            (1235, N'Testnevelés és sport', N'Testnevelés és sport', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1800, NULL, 58, NULL),
            (1236, N'Pályaorientáció, szakmai alapozás', N'Pályaorientáció, szakmai alapozás', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 2000, NULL, 59, NULL),
            (1237, N'Szakmai orientáció', N'Szakmai orientáció', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 2010, NULL, 60, NULL),
            (1238, N'Szakmacsoportos alapozás', N'Szakmacsoportos alapozás', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 2020, NULL, 61, NULL),
            (1239, N'Szakmai elmélet', N'Szakmai elmélet', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10000, NULL, 62, NULL),
            (1240, N'Szakmai gyakorlat', N'Szakmai gyakorlat', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11000, NULL, 63, NULL),
            (1241, N'Zeneművészet', N'Zeneművészet', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1540, NULL, 64, NULL),
            (1242, N'Egyéb', N'Egyéb', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 20000, NULL, 65, NULL),
            (1243, N'Óvodai feladat', N'Óvodai feladat', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 999, NULL, 66, NULL),
            (1244, N'Kollégiumi feladat', N'Kollégiumi feladat', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 6030, NULL, 67, NULL),
            (1245, N'Diákotthoni feladat', N'Diákotthoni feladat', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 6010, NULL, 68, NULL),
            (1246, N'Gyermekotthoni feladat', N'Gyermekotthoni feladat', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 6020, NULL, 69, NULL),
            (1247, N'Szakértői bizottsági tevékenység', N'Szakértői bizottsági tevékenység', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 13000, NULL, 70, NULL),
            (1248, N'Osztályfőnöki, élet- és pályatervezés', N'Osztályfőnöki, élet- és pályatervezés', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1900, NULL, 71, NULL),
            (1249, N'Szakmai elmélet (Egészségügy - 1.)', N'Szakmai elmélet (Egészségügy - 1.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10010, NULL, 72, NULL),
            (1250, N'Szakmai elmélet (Szociális szolgáltatások - 2.)', N'Szakmai elmélet (Szociális szolgáltatások - 2.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10020, NULL, 73, NULL),
            (1251, N'Szakmai elmélet (Oktatás - 3.)', N'Szakmai elmélet (Oktatás - 3.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10030, NULL, 74, NULL),
            (1252, N'Szakmai elmélet (Művészet, közművelődés, kommunikáció - 4.)', N'Szakmai elmélet (Művészet, közművelődés, kommunikáció - 4.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10040, NULL, 75, NULL),
            (1253, N'Szakmai elmélet (Gépészet - 5.)', N'Szakmai elmélet (Gépészet - 5.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10050, NULL, 76, NULL),
            (1254, N'Szakmai elmélet (Elektrotechnika-elektronika - 6.)', N'Szakmai elmélet (Elektrotechnika-elektronika - 6.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10060, NULL, 77, NULL),
            (1255, N'Szakmai elmélet (Informatika - 7.)', N'Szakmai elmélet (Informatika - 7.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10070, NULL, 78, NULL),
            (1256, N'Szakmai elmélet (Vegyipar - 8.)', N'Szakmai elmélet (Vegyipar - 8.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10080, NULL, 79, NULL),
            (1257, N'Szakmai elmélet (Építészet - 9.)', N'Szakmai elmélet (Építészet - 9.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10090, NULL, 80, NULL),
            (1258, N'Szakmai elmélet (Könnyűipar - 10.)', N'Szakmai elmélet (Könnyűipar - 10.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10100, NULL, 81, NULL),
            (1259, N'Szakmai elmélet (Faipar - 11.)', N'Szakmai elmélet (Faipar - 11.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10110, NULL, 82, NULL),
            (1260, N'Szakmai elmélet (Nyomdaipar - 12.)', N'Szakmai elmélet (Nyomdaipar - 12.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10120, NULL, 83, NULL),
            (1261, N'Szakmai elmélet (Közlekedés - 13.)', N'Szakmai elmélet (Közlekedés - 13.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10130, NULL, 84, NULL),
            (1263, N'Szakmai elmélet (Ügyvitel - 16.)', N'Szakmai elmélet (Ügyvitel - 16.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10160, NULL, 85, NULL),
            (1264, N'Szakmai elmélet (Kereskedelem-marketing, üzleti adminisztráció - 17.)', N'Szakmai elmélet (Kereskedelem-marketing, üzleti adminisztráció - 17.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10170, NULL, 86, NULL),
            (1265, N'Szakmai elmélet (Vendéglátás-turisztika - 18.)', N'Szakmai elmélet (Vendéglátás-turisztika - 18.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10180, NULL, 87, NULL),
            (1266, N'Szakmai elmélet (Egyéb szolgáltatások - 19.)', N'Szakmai elmélet (Egyéb szolgáltatások - 19.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10190, NULL, 88, NULL),
            (1267, N'Szakmai elmélet (Mezőgazdaság - 20.)', N'Szakmai elmélet (Mezőgazdaság - 20.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10200, NULL, 89, NULL),
            (1268, N'Szakmai elmélet (Élelmiszeripar - 21.)', N'Szakmai elmélet (Élelmiszeripar - 21.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 10210, NULL, 90, NULL),
            (1269, N'Szakmai gyakorlat (Egészségügy - 1.)', N'Szakmai gyakorlat (Egészségügy - 1.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11010, NULL, 91, NULL),
            (1270, N'Szakmai gyakorlat (Szociális szolgáltatások - 2.)', N'Szakmai gyakorlat (Szociális szolgáltatások - 2.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11020, NULL, 92, NULL),
            (1271, N'Szakmai gyakorlat (Oktatás - 3.)', N'Szakmai gyakorlat (Oktatás - 3.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11030, NULL, 93, NULL),
            (1272, N'Szakmai gyakorlat (Művészet, közművelődés, kommunikáció - 4.)', N'Szakmai gyakorlat (Művészet, közművelődés, kommunikáció - 4.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11040, NULL, 94, NULL),
            (1273, N'Szakmai gyakorlat (Gépészet - 5.)', N'Szakmai gyakorlat (Gépészet - 5.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11050, NULL, 95, NULL),
            (1274, N'Szakmai gyakorlat (Elektrotechnika-elektronika - 6.)', N'Szakmai gyakorlat (Elektrotechnika-elektronika - 6.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11060, NULL, 96, NULL),
            (1275, N'Szakmai gyakorlat (Informatika - 7.)', N'Szakmai gyakorlat (Informatika - 7.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11070, NULL, 97, NULL),
            (1276, N'Szakmai gyakorlat (Vegyipar - 8.)', N'Szakmai gyakorlat (Vegyipar - 8.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11080, NULL, 98, NULL),
            (1277, N'Szakmai gyakorlat (Építészet - 9.)', N'Szakmai gyakorlat (Építészet - 9.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11090, NULL, 99, NULL),
            (1278, N'Szakmai gyakorlat (Könnyűipar - 10.)', N'Szakmai gyakorlat (Könnyűipar - 10.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11100, NULL, 100, NULL),
            (1279, N'Szakmai gyakorlat (Faipar - 11.)', N'Szakmai gyakorlat (Faipar - 11.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11110, NULL, 101, NULL),
            (1280, N'Szakmai gyakorlat (Nyomdaipar - 12.)', N'Szakmai gyakorlat (Nyomdaipar - 12.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11120, NULL, 102, NULL),
            (1281, N'Szakmai gyakorlat (Közlekedés - 13.)', N'Szakmai gyakorlat (Közlekedés - 13.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11130, NULL, 103, NULL),
            (1283, N'Szakmai gyakorlat (Ügyvitel - 16.)', N'Szakmai gyakorlat (Ügyvitel - 16.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11160, NULL, 104, NULL),
            (1284, N'Szakmai gyakorlat (Kereskedelem-marketing, üzleti adminisztráció - 17.)', N'Szakmai gyakorlat (Kereskedelem-marketing, üzleti adminisztráció - 17.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11170, NULL, 105, NULL),
            (1285, N'Szakmai gyakorlat (Vendéglátás-turisztika - 18.)', N'Szakmai gyakorlat (Vendéglátás-turisztika - 18.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11180, NULL, 106, NULL),
            (1286, N'Szakmai gyakorlat (Egyéb szolgáltatások - 19.)', N'Szakmai gyakorlat (Egyéb szolgáltatások - 19.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11190, NULL, 107, NULL),
            (1287, N'Szakmai gyakorlat (Mezőgazdaság - 20.)', N'Szakmai gyakorlat (Mezőgazdaság - 20.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11200, NULL, 108, NULL),
            (1288, N'Szakmai gyakorlat (Élelmiszeripar - 21.)', N'Szakmai gyakorlat (Élelmiszeripar - 21.)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 11210, NULL, 109, NULL),
            (1293, N'Társadalmi, állampolgári és gazdasági ismeretek', N'Társadalmi, állampolgári és gazdasági ismeretek', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1060, NULL, 110, NULL),
            (1294, N'Művészetek', N'Művészetek', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1500, NULL, 111, NULL),
            (5221, N'Napközi', N'Napközi', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 6000, NULL, 112, NULL),
            (6706, N'Szakmai elmélet (közgazdaságtan)', N'Szakmai elmélet (közgazdaságtan)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 20000, NULL, 113, NULL),
            (6707, N'Szakmai gyakorlat (közgadaságtan)', N'Szakmai gyakorlat (közgadaságtan)', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 20000, NULL, 114, NULL),
            (6708, N'Ember és társadalom', N'Ember és társadalom', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1020, NULL, 115, NULL),
            (7688, N'Szorgalom', N'Szorgalom', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1545, NULL, 116, NULL),
            (7747, N'Természettudomány', N'Természettudomány', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1120, NULL, 117, NULL),
            (7748, N'Két tanítási nyelvű célnyelv - Angol nyelv', N'Két tanítási nyelvű célnyelv - Angol nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1281, NULL, 118, NULL),
            (7749, N'Két tanítási nyelvű célnyelv - Német nyelv', N'Két tanítási nyelvű célnyelv - Német nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1282, NULL, 119, NULL),
            (7750, N'Két tanítási nyelvű célnyelv - Orosz nyelv', N'Két tanítási nyelvű célnyelv - Orosz nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1283, NULL, 120, NULL),
            (7751, N'Két tanítási nyelvű célnyelv - Francia nyelv', N'Két tanítási nyelvű célnyelv - Francia nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1284, NULL, 121, NULL),
            (7752, N'Két tanítási nyelvű célnyelv - Olasz nyelv', N'Két tanítási nyelvű célnyelv - Olasz nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1285, NULL, 122, NULL),
            (7753, N'Két tanítási nyelvű célnyelv - Spanyol nyelv', N'Két tanítási nyelvű célnyelv - Spanyol nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1286, NULL, 123, NULL),
            (7754, N'Két tanítási nyelvű célnyelv - Egyéb idegen nyelv', N'Két tanítási nyelvű célnyelv - Egyéb idegen nyelv', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1287, NULL, 124, NULL),
            (8271, N'Történelem és állampolgári ismeretek', N'Történelem és állampolgári ismeretek', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1025, NULL, 125, NULL),
            (8272, N'Természettudomány és földrajz', N'Természettudomány és földrajz', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1105, NULL, 126, NULL),
            (8273, N'Technológia', N'Technológia', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1610, NULL, 127, NULL),
            (8274, N'Testnevelés és egészségfejlesztés', N'Testnevelés és egészségfejlesztés', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 1810, NULL, 128, NULL),
            (8275, N'Szakmai elmélet (Bányászat és kohászat - 01 (2020))', N'Szakmai elmélet (Bányászat és kohászat - 01 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8010, NULL, 129, NULL),
            (8276, N'Szakmai elmélet (Egészségügyi technika - 02 (2020))', N'Szakmai elmélet (Egészségügyi technika - 02 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8020, NULL, 130, NULL),
            (8277, N'Szakmai elmélet (Egészségügy - 03 (2020))', N'Szakmai elmélet (Egészségügy - 03 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8030, NULL, 131, NULL),
            (8278, N'Szakmai elmélet (Elektronika és elektrotechnika - 04 (2020))', N'Szakmai elmélet (Elektronika és elektrotechnika - 04 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8040, NULL, 132, NULL),
            (8279, N'Szakmai elmélet (Élelmiszeripar - 05 (2020))', N'Szakmai elmélet (Élelmiszeripar - 05 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8050, NULL, 133, NULL),
            (8280, N'Szakmai elmélet (Építőipar - 06 (2020))', N'Szakmai elmélet (Építőipar - 06 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8060, NULL, 134, NULL),
            (8281, N'Szakmai elmélet (Épületgépészet - 07 (2020))', N'Szakmai elmélet (Épületgépészet - 07 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8070, NULL, 135, NULL),
            (8282, N'Szakmai elmélet (Fa- és bútoripar - 08 (2020))', N'Szakmai elmélet (Fa- és bútoripar - 08 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8080, NULL, 136, NULL),
            (8283, N'Szakmai elmélet (Gazdálkodás és menedzsment - 09 (2020))', N'Szakmai elmélet (Gazdálkodás és menedzsment - 09 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8090, NULL, 137, NULL),
            (8284, N'Szakmai elmélet (Gépészet - 10 (2020))', N'Szakmai elmélet (Gépészet - 10 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8100, NULL, 138, NULL),
            (8285, N'Szakmai elmélet (Honvédelem - 11 (2020))', N'Szakmai elmélet (Honvédelem - 11 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8110, NULL, 139, NULL),
            (8286, N'Szakmai elmélet (Informatika és távközlés - 12 (2020))', N'Szakmai elmélet (Informatika és távközlés - 12 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8120, NULL, 140, NULL),
            (8287, N'Szakmai elmélet (Kereskedelem - 13 (2020))', N'Szakmai elmélet (Kereskedelem - 13 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8130, NULL, 141, NULL),
            (8288, N'Szakmai elmélet (Környezetvédelem és vízügy - 14 (2020))', N'Szakmai elmélet (Környezetvédelem és vízügy - 14 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8140, NULL, 142, NULL),
            (8289, N'Szakmai elmélet (Közlekedés és szállítmányozás - 15 (2020))', N'Szakmai elmélet (Közlekedés és szállítmányozás - 15 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8150, NULL, 143, NULL),
            (8290, N'Szakmai elmélet (Kreatív - 16 (2020))', N'Szakmai elmélet (Kreatív - 16 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8160, NULL, 144, NULL),
            (8291, N'Szakmai elmélet (Mezőgazdaság és erdészet - 17 (2020))', N'Szakmai elmélet (Mezőgazdaság és erdészet - 17 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8170, NULL, 145, NULL),
            (8292, N'Szakmai elmélet (Rendészet és közszolgálat - 18 (2020))', N'Szakmai elmélet (Rendészet és közszolgálat - 18 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8180, NULL, 146, NULL),
            (8293, N'Szakmai elmélet (Specializált gép- és járműgyártás - 19 (2020))', N'Szakmai elmélet (Specializált gép- és járműgyártás - 19 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8190, NULL, 147, NULL),
            (8294, N'Szakmai elmélet (Sport - 20 (2020))', N'Szakmai elmélet (Sport - 20 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8200, NULL, 148, NULL),
            (8295, N'Szakmai elmélet (Szépészet - 21 (2020))', N'Szakmai elmélet (Szépészet - 21 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8210, NULL, 149, NULL),
            (8296, N'Szakmai elmélet (Szociális - 22 (2020))', N'Szakmai elmélet (Szociális - 22 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8220, NULL, 150, NULL),
            (8297, N'Szakmai elmélet (Turizmus-vendéglátás - 23 (2020))', N'Szakmai elmélet (Turizmus-vendéglátás - 23 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8230, NULL, 151, NULL),
            (8298, N'Szakmai elmélet (Vegyipar - 24 (2020))', N'Szakmai elmélet (Vegyipar - 24 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8240, NULL, 152, NULL),
            (8299, N'Szakmai gyakorlat (Bányászat és kohászat - 01 (2020))', N'Szakmai gyakorlat (Bányászat és kohászat - 01 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8510, NULL, 153, NULL),
            (8300, N'Szakmai gyakorlat (Egészségügyi technika - 02 (2020))', N'Szakmai gyakorlat (Egészségügyi technika - 02 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8520, NULL, 154, NULL),
            (8301, N'Szakmai gyakorlat (Egészségügy - 03 (2020))', N'Szakmai gyakorlat (Egészségügy - 03 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8530, NULL, 155, NULL),
            (8302, N'Szakmai gyakorlat (Elektronika és elektrotechnika - 04 (2020))', N'Szakmai gyakorlat (Elektronika és elektrotechnika - 04 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8540, NULL, 156, NULL),
            (8303, N'Szakmai gyakorlat (Élelmiszeripar - 05 (2020))', N'Szakmai gyakorlat (Élelmiszeripar - 05 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8550, NULL, 157, NULL),
            (8304, N'Szakmai gyakorlat (Építőipar - 06 (2020))', N'Szakmai gyakorlat (Építőipar - 06 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8560, NULL, 158, NULL),
            (8305, N'Szakmai gyakorlat (Épületgépészet - 07 (2020))', N'Szakmai gyakorlat (Épületgépészet - 07 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8570, NULL, 159, NULL),
            (8306, N'Szakmai gyakorlat (Fa- és bútoripar - 08 (2020))', N'Szakmai gyakorlat (Fa- és bútoripar - 08 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8580, NULL, 160, NULL),
            (8307, N'Szakmai gyakorlat (Gazdálkodás és menedzsment - 09 (2020))', N'Szakmai gyakorlat (Gazdálkodás és menedzsment - 09 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8590, NULL, 161, NULL),
            (8308, N'Szakmai gyakorlat (Gépészet - 10 (2020))', N'Szakmai gyakorlat (Gépészet - 10 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8600, NULL, 162, NULL),
            (8309, N'Szakmai gyakorlat (Honvédelem - 11 (2020))', N'Szakmai gyakorlat (Honvédelem - 11 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8610, NULL, 163, NULL),
            (8310, N'Szakmai gyakorlat (Informatika és távközlés - 12 (2020))', N'Szakmai gyakorlat (Informatika és távközlés - 12 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8620, NULL, 164, NULL),
            (8311, N'Szakmai gyakorlat (Kereskedelem - 13 (2020))', N'Szakmai gyakorlat (Kereskedelem - 13 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8630, NULL, 165, NULL),
            (8312, N'Szakmai gyakorlat (Környezetvédelem és vízügy - 14 (2020))', N'Szakmai gyakorlat (Környezetvédelem és vízügy - 14 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8640, NULL, 166, NULL),
            (8313, N'Szakmai gyakorlat (Közlekedés és szállítmányozás - 15 (2020))', N'Szakmai gyakorlat (Közlekedés és szállítmányozás - 15 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8650, NULL, 167, NULL),
            (8314, N'Szakmai gyakorlat (Kreatív - 16 (2020))', N'Szakmai gyakorlat (Kreatív - 16 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8660, NULL, 168, NULL),
            (8315, N'Szakmai gyakorlat (Mezőgazdaság és erdészet - 17 (2020))', N'Szakmai gyakorlat (Mezőgazdaság és erdészet - 17 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8670, NULL, 169, NULL),
            (8316, N'Szakmai gyakorlat (Rendészet és közszolgálat - 18 (2020))', N'Szakmai gyakorlat (Rendészet és közszolgálat - 18 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8680, NULL, 170, NULL),
            (8317, N'Szakmai gyakorlat (Specializált gép- és járműgyártás - 19 (2020))', N'Szakmai gyakorlat (Specializált gép- és járműgyártás - 19 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8690, NULL, 171, NULL),
            (8318, N'Szakmai gyakorlat (Sport - 20 (2020))', N'Szakmai gyakorlat (Sport - 20 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8700, NULL, 172, NULL),
            (8319, N'Szakmai gyakorlat (Szépészet - 21 (2020))', N'Szakmai gyakorlat (Szépészet - 21 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8710, NULL, 173, NULL),
            (8320, N'Szakmai gyakorlat (Szociális - 22 (2020))', N'Szakmai gyakorlat (Szociális - 22 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8720, NULL, 174, NULL),
            (8321, N'Szakmai gyakorlat (Turizmus-vendéglátás - 23 (2020))', N'Szakmai gyakorlat (Turizmus-vendéglátás - 23 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8730, NULL, 175, NULL),
            (8322, N'Szakmai gyakorlat (Vegyipar - 24 (2020))', N'Szakmai gyakorlat (Vegyipar - 24 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 8740, NULL, 176, NULL),
            (8323, N'Szakmai', N'Szakmai', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9000, NULL, 177, NULL),
            (8324, N'Szakmai (Bányászat és kohászat - 01 (2020))', N'Szakmai (Bányászat és kohászat - 01 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9010, NULL, 178, NULL),
            (8325, N'Szakmai (Egészségügyi technika - 02 (2020))', N'Szakmai (Egészségügyi technika - 02 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9020, NULL, 179, NULL),
            (8326, N'Szakmai (Egészségügy - 03 (2020))', N'Szakmai (Egészségügy - 03 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9030, NULL, 180, NULL),
            (8327, N'Szakmai (Elektronika és elektrotechnika - 04 (2020))', N'Szakmai (Elektronika és elektrotechnika - 04 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9040, NULL, 181, NULL),
            (8328, N'Szakmai (Élelmiszeripar - 05 (2020))', N'Szakmai (Élelmiszeripar - 05 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9050, NULL, 182, NULL),
            (8329, N'Szakmai (Építőipar - 06 (2020))', N'Szakmai (Építőipar - 06 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9060, NULL, 183, NULL),
            (8330, N'Szakmai (Épületgépészet - 07 (2020))', N'Szakmai (Épületgépészet - 07 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9070, NULL, 184, NULL),
            (8331, N'Szakmai (Fa- és bútoripar - 08 (2020))', N'Szakmai (Fa- és bútoripar - 08 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9080, NULL, 185, NULL),
            (8332, N'Szakmai (Gazdálkodás és menedzsment - 09 (2020))', N'Szakmai (Gazdálkodás és menedzsment - 09 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9090, NULL, 186, NULL),
            (8333, N'Szakmai (Gépészet - 10 (2020))', N'Szakmai (Gépészet - 10 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9100, NULL, 187, NULL),
            (8334, N'Szakmai (Honvédelem - 11 (2020))', N'Szakmai (Honvédelem - 11 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9110, NULL, 188, NULL),
            (8335, N'Szakmai (Informatika és távközlés - 12 (2020))', N'Szakmai (Informatika és távközlés - 12 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9120, NULL, 189, NULL),
            (8336, N'Szakmai (Kereskedelem - 13 (2020))', N'Szakmai (Kereskedelem - 13 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9130, NULL, 190, NULL),
            (8337, N'Szakmai (Környezetvédelem és vízügy - 14 (2020))', N'Szakmai (Környezetvédelem és vízügy - 14 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9140, NULL, 191, NULL),
            (8338, N'Szakmai (Közlekedés és szállítmányozás - 15 (2020))', N'Szakmai (Közlekedés és szállítmányozás - 15 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9150, NULL, 192, NULL),
            (8339, N'Szakmai (Kreatív - 16 (2020))', N'Szakmai (Kreatív - 16 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9160, NULL, 193, NULL),
            (8340, N'Szakmai (Mezőgazdaság és erdészet - 17 (2020))', N'Szakmai (Mezőgazdaság és erdészet - 17 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9170, NULL, 194, NULL),
            (8341, N'Szakmai (Rendészet és közszolgálat - 18 (2020))', N'Szakmai (Rendészet és közszolgálat - 18 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9180, NULL, 195, NULL),
            (8342, N'Szakmai (Specializált gép- és járműgyártás - 19 (2020))', N'Szakmai (Specializált gép- és járműgyártás - 19 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9190, NULL, 196, NULL),
            (8343, N'Szakmai (Sport - 20 (2020))', N'Szakmai (Sport - 20 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9200, NULL, 197, NULL),
            (8344, N'Szakmai (Szépészet - 21 (2020))', N'Szakmai (Szépészet - 21 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9210, NULL, 198, NULL),
            (8345, N'Szakmai (Szociális - 22 (2020))', N'Szakmai (Szociális - 22 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9220, NULL, 199, NULL),
            (8346, N'Szakmai (Turizmus-vendéglátás - 23 (2020))', N'Szakmai (Turizmus-vendéglátás - 23 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9230, NULL, 200, NULL),
            (8347, N'Szakmai (Vegyipar - 24 (2020))', N'Szakmai (Vegyipar - 24 (2020))', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 9240, NULL, 201, NULL),
            (8789, N'Hit- és erkölcstan', NULL, NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 1044, NULL, 202, NULL),
            (8790, N'Etika', NULL, NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'F', 1042, NULL, 203, NULL),
            (8821, N'Dobbantó', N'Dobbantó', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 20010, NULL, 204, NULL),
            (8822, N'Orientáció', N'Orientáció', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 20020, NULL, 205, NULL),
            (8823, N'Műhelyiskola', N'Műhelyiskola', NULL, NULL, NULL, N'TargyKategoriaTipus', 36, N'T', N'T', 20030, NULL, 206, NULL),
            (1296, N'Na', N'Na', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 1, NULL, 1, NULL),
            (1297, N'Kis', N'Kis', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 1010, NULL, 2, NULL),
            (1298, N'Középső', N'Középső', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 1020, NULL, 3, NULL),
            (6470, N'Nagy középső', N'Nagy középső', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 1030, NULL, 4, NULL),
            (1299, N'Nagy', N'Nagy', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 1040, NULL, 5, NULL),
            (1300, N'Óvodai vegyes', N'Óvodai vegyes', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 1050, NULL, 6, NULL),
            (1301, N'Előképző 1', N'Előképző 1.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 1060, NULL, 7, NULL),
            (1302, N'Előképző 2', N'Előképző 2.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 1070, NULL, 8, NULL),
            (1303, N'1', N'1.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2110, NULL, 9, NULL),
            (1304, N'2', N'2.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2120, NULL, 10, NULL),
            (1305, N'3', N'3.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2130, NULL, 11, NULL),
            (1306, N'4', N'4.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2140, NULL, 12, NULL),
            (1307, N'5', N'5.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2150, NULL, 13, NULL),
            (1308, N'6', N'6.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2160, NULL, 14, NULL),
            (1309, N'7', N'7.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2170, NULL, 15, NULL),
            (1310, N'8', N'8.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2180, NULL, 16, NULL),
            (1311, N'9', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2310, NULL, 17, NULL),
            (1312, N'10', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2410, NULL, 18, NULL),
            (1313, N'11', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2510, NULL, 19, NULL),
            (1314, N'12', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2610, NULL, 20, NULL),
            (1315, N'13', N'13.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2710, NULL, 21, NULL),
            (1330, N'5/Ny', N'5/Ny', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3010, NULL, 22, NULL),
            (1331, N'7/Ny', N'7/Ny', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3020, NULL, 23, NULL),
            (1332, N'9/Ny', N'9/Ny', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3040, NULL, 24, NULL),
            (1333, N'9/Kny', N'9/Kny', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3120, NULL, 25, NULL),
            (1334, N'5/N', N'5/N', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3210, NULL, 26, NULL),
            (1335, N'7/N', N'7/N', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3220, NULL, 27, NULL),
            (1336, N'9/N', N'9/N', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3230, NULL, 28, NULL),
            (3019, N'9/AJKP', N'9/AJKP', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3310, NULL, 29, NULL),
            (3018, N'9/AJTP', N'9/AJTP', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3320, NULL, 30, NULL),
            (3020, N'9/E', N'9/E', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3420, NULL, 31, NULL),
            (6471, N'KH/1/1', N'KH', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3510, NULL, 32, NULL),
            (6472, N'KH/2/1/C', N'KH', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3520, NULL, 33, NULL),
            (6473, N'KH/2/1/A', N'KH', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3530, NULL, 34, NULL),
            (6474, N'KH/2/2/A', N'KH', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3540, NULL, 35, NULL),
            (6475, N'KH/2/1/B', N'KH', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3550, NULL, 36, NULL),
            (6476, N'KH/2/2/B', N'KH', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3560, NULL, 37, NULL),
            (6477, N'SZH/1', N'SZH/1', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3610, NULL, 38, NULL),
            (6478, N'SZH/2', N'SZH/2', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3620, NULL, 39, NULL),
            (6479, N'SZH/1/A', N'SZH/1', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3630, NULL, 40, NULL),
            (6480, N'SZH/2/A', N'SZH/2', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3640, NULL, 41, NULL),
            (6481, N'SZH/1/B', N'SZH/1', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3650, NULL, 42, NULL),
            (6482, N'SZH/2/B', N'SZH/2', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3660, NULL, 43, NULL),
            (1324, N'1/8', N'1/8.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4020, NULL, 44, NULL),
            (1316, N'1/9', N'1/9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4140, NULL, 45, NULL),
            (1323, N'2/10', N'2/10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4150, NULL, 46, NULL),
            (1325, N'3/11', N'3/11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4160, NULL, 47, NULL),
            (6520, N'4/12', N'4/12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4170, NULL, 48, NULL),
            (6469, N'5/13', N'5/13.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4190, NULL, 49, NULL),
            (1317, N'1/11', N'1/11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4210, NULL, 50, NULL),
            (1318, N'2/12', N'2/12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4220, NULL, 51, NULL),
            (1322, N'3/13', N'3/13.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4230, NULL, 52, NULL),
            (1319, N'1/13', N'1/13.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4320, NULL, 53, NULL),
            (1320, N'2/14', N'2/14.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4340, NULL, 54, NULL),
            (1321, N'3/15', N'3/15.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4350, NULL, 55, NULL),
            (6468, N'4/16', N'4/16.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4360, NULL, 56, NULL),
            (1326, N'1/12', N'1/12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4420, NULL, 57, NULL),
            (1337, N'1/15', N'1/15.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4440, NULL, 58, NULL),
            (3016, N'Ksz/11', N'Ksz/11', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4520, NULL, 59, NULL),
            (3017, N'Ksz/12', N'Ksz/12', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4540, NULL, 60, NULL),
            (3021, N'Szél/12/1', N'Szél/12/1', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 640, NULL, 61, NULL),
            (3022, N'Szél/12/2', N'Szél/12/2', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 650, NULL, 62, NULL),
            (6702, N'9/S', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4610, NULL, 63, NULL),
            (6703, N'10/S', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4620, NULL, 64, NULL),
            (6704, N'11/S', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4630, NULL, 65, NULL),
            (6705, N'12/S', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4640, NULL, 66, NULL),
            (6818, N'Kk 12', N'Kk 12', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4710, NULL, 67, NULL),
            (6819, N'Kk 13', N'Kk 13', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4720, NULL, 68, NULL),
            (6908, N'Összevont 1-2', N'Összevont 1-2', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 5020, NULL, 69, NULL),
            (6909, N'Összevont 1-3', N'Összevont 1-3', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 5030, NULL, 70, NULL),
            (6910, N'Összevont 1-4', N'Összevont 1-4', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 5040, NULL, 71, NULL),
            (6911, N'Összevont 2-3', N'Összevont 2-3', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 5050, NULL, 72, NULL),
            (6912, N'Összevont 2-4', N'Összevont 2-4', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 5060, NULL, 73, NULL),
            (6913, N'Összevont 3-4', N'Összevont 3-4', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 5070, NULL, 74, NULL),
            (7109, N'7 gimnázium', N'7.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2190, NULL, 75, NULL),
            (7110, N'8 gimnázium', N'8.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2200, NULL, 76, NULL),
            (7111, N'9 szakgimnázium', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2320, NULL, 77, NULL),
            (7112, N'9 gimnázium', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2330, NULL, 78, NULL),
            (7113, N'9 szakiskola', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2340, NULL, 79, NULL),
            (7114, N'9 készségfejlesztő iskola', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2350, NULL, 80, NULL),
            (7115, N'10 szakgimnázium', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2420, NULL, 81, NULL),
            (7116, N'10 gimnázium', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2430, NULL, 82, NULL),
            (7117, N'10 szakiskola', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2440, NULL, 83, NULL),
            (7118, N'10 készségfejlesztő iskola', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2450, NULL, 84, NULL),
            (7119, N'11 szakgimnázium', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2520, NULL, 85, NULL),
            (7120, N'11 gimnázium', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2530, NULL, 86, NULL),
            (7121, N'11 szakiskola', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2540, NULL, 87, NULL),
            (7122, N'11 készségfejlesztő iskola', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2550, NULL, 88, NULL),
            (7123, N'12 szakgimnázium', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2620, NULL, 89, NULL),
            (7124, N'12 gimnázium', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2630, NULL, 90, NULL),
            (7125, N'12 szakiskola', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2640, NULL, 91, NULL),
            (7126, N'12 készségfejlesztő iskola', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2650, NULL, 92, NULL),
            (7127, N'9/Ny gimnázium nyelvi előkészítő', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3050, NULL, 93, NULL),
            (7128, N'9/Kny gimnázium két tanítási nyelvű előkészítő', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 3130, NULL, 94, NULL),
            (7129, N'Kollégium', N'Kollégium', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 6010, NULL, 95, NULL),
            (7366, N'Összevont osztály', N'Összevont', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 5010, NULL, 96, NULL),
            (7862, N'1/9 4-es szintű szakképző iskola', N'1/9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4141, N'002', 97, NULL),
            (7863, N'2/10 4-es szintű szakképző iskola', N'2/10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4151, N'003', 98, NULL),
            (7864, N'3/11 4-es szintű szakképző iskola', N'3/11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4161, N'003', 99, NULL),
            (7865, N'4/12 4-es szintű szakképző iskola (4 éves)', N'4/12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 463, N'003', 100, NULL),
            (7866, N'Ksz/11 4-es szintű szakképző iskola közismeret nélküli', N'Ksz/11', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4521, N'002', 101, NULL),
            (7867, N'Ksz/12 4-es szintű szakképző iskola közismeret nélküli', N'Ksz/12', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4522, N'003', 102, NULL),
            (7868, N'Szakképzést előkészítő évfolyam', N'9/Oe', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4010, N'004', 103, NULL),
            (7869, N'9/Ny technikum nyelvi előkészítő', N'9/Ny', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3042, N'001', 104, NULL),
            (7870, N'9/Kny technikum két tanítási nyelvű előkészítő', N'9/Kny', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3122, N'001', 105, NULL),
            (7871, N'9. technikum', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2322, N'001', 106, NULL),
            (7872, N'9. technikum (két tanítási nyelvű)', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2324, N'001', 107, NULL),
            (7873, N'10. technikum', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2422, N'001', 108, NULL),
            (7874, N'10. technikum (két tanítási nyelvű)', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2424, N'001', 109, NULL),
            (7875, N'11. technikum', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2522, N'003', 110, NULL),
            (7876, N'11. technikum (két tanítási nyelvű)', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2524, N'003', 111, NULL),
            (7877, N'12. technikum', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2622, N'003', 112, NULL),
            (7878, N'12. technikum (két tanítási nyelvű)', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2624, N'003', 113, NULL),
            (7879, N'13. technikum', N'13.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 477, N'003', 114, NULL),
            (7880, N'13. technikum (két tanítási nyelvű)', N'13.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 478, N'003', 115, NULL),
            (7881, N'14. technikum (6 éves képzés)', N'14.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 479, N'003', 116, NULL),
            (7882, N'14. technikum (két tanítási nyelvű) (6 éves képzés)', N'14.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 480, N'003', 117, NULL),
            (7883, N'1/13 5-ös szintű technikum', N'1/13.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4322, N'001', 118, NULL),
            (7884, N'2/14 5-ös szintű technikum', N'2/14.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4342, N'003', 119, NULL),
            (7889, N'Műhelyiskola osztály', N'M', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 550, N'005', 120, NULL),
            (7901, N'1/12 35-ös szintű kifutó ráépülés szakképző iskola', N'1/12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4421, NULL, 121, NULL),
            (7902, N'Kk 12 szakképző iskola két éves érettségire felkészítő', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4711, NULL, 122, NULL),
            (7903, N'Kk 13 szakképző iskola két éves érettségire felkészítő', N'13.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4721, NULL, 123, NULL),
            (7904, N'Dobbantó előkészítő', N'E/D', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4000, N'004', 124, NULL),
            (7905, N'9/Ny szakgimnázium NKT nyelvi előkészítő', N'9/Ny', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 3044, NULL, 125, NULL),
            (7906, N'9. szakgimnázium NKT', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2326, NULL, 126, NULL),
            (7907, N'10. szakgimnázium NKT', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2426, NULL, 127, NULL),
            (7908, N'11. szakgimnázium NKT', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2526, NULL, 128, NULL),
            (7909, N'12. szakgimnázium NKT', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 2626, NULL, 129, NULL),
            (7910, N'5/13 5-ös szintű technikum beszámításos egy éves', N'5/13.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4192, NULL, 130, NULL),
            (7911, N'1/15 55-ös szintű kifutó ráépülés technikum', N'1/15.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4441, NULL, 131, NULL),
            (8394, N'Ksz/13 4-es szintű szakképző iskola közismeret nélküli', N'Ksz/13', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4560, N'003', 132, NULL),
            (8395, N'3/15 5-ös szintű technikum', N'3/15.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 4345, N'003', 133, NULL),
            (8396, N'9. szakgimnázium NKT (két tanítási nyelvű)', N'9.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2328, NULL, 134, NULL),
            (8397, N'10. szakgimnázium NKT (két tanítási nyelvű)', N'10.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2428, NULL, 135, NULL),
            (8398, N'11. szakgimnázium NKT (két tanítási nyelvű)', N'11.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2528, NULL, 136, NULL),
            (8399, N'12. szakgimnázium NKT (két tanítási nyelvű)', N'12.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 2628, NULL, 137, NULL),
            (8401, N'Fejlesztő nevelés-oktatás', N'Fejlesztő nevelés-oktatás', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'T', 6020, NULL, 138, NULL),
            (8408, N'Ksz/13', N'Ksz/13', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4545, NULL, 139, NULL),
            (8409, N'Ksz/14', N'Ksz/14', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4550, NULL, 140, NULL),
            (8769, N'1/13 szakgimnázium (művészeti, pedagógiai, közművelődési)', N'1/13.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4324, NULL, 141, NULL),
            (8770, N'2/14 szakgimnázium (művészeti, pedagógiai, közművelődési)', N'2/14.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4344, NULL, 142, NULL),
            (8771, N'3/15 szakgimnázium (művészeti, pedagógiai, közművelődési)', N'3/15.', NULL, NULL, NULL, N'EvfolyamTipus', 37, N'T', N'F', 4347, NULL, 143, NULL),
            (1156, N'Na', N'Na', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 1, NULL, 1, NULL),
            (1157, N'Óvoda', N'Óvoda', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 10, NULL, 2, NULL),
            (1158, N'Általános iskola', N'Általános iskola', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 20, NULL, 3, NULL),
            (1159, N'Alapfokú művészetoktatás', N'Alapfokú művészetoktatás', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 140, NULL, 4, NULL),
            (1160, N'Gimnázium', N'Gimnázium', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 30, NULL, 5, NULL),
            (1161, N'Szakközépiskola', N'Szakközépiskola', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 90, NULL, 6, NULL),
            (1162, N'Szakiskola', N'Szakiskola', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 100, NULL, 7, NULL),
            (5435, N'Szakgimnázium', N'Szakgimnázium', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 40, NULL, 8, NULL),
            (1165, N'Kollégium', N'Kollégium', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 130, NULL, 9, NULL),
            (1166, N'Gyermekotthoni feladatok', N'Gyermekotthoni feladatok', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 200, NULL, 10, NULL),
            (1173, N'Konduktív pedagógiai ellátás', N'Konduktív pedagógiai ellátás', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 190, NULL, 11, NULL),
            (1176, N'Felnőttképzés', N'Felnőttképzés', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 1000, NULL, 12, NULL),
            (6463, N'Felnőttoktatás', N'Felnőttoktatás', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 120, NULL, 13, NULL),
            (1177, N'Óvoda (gyógypedagógia)', N'Óvoda (gyógypedagógia)', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 170, NULL, 14, NULL),
            (1178, N'Általános iskola (gyógypedagógia)', N'Általános iskola (gyógypedagógia)', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 180, NULL, 15, NULL),
            (1180, N'Készségfejlesztő iskola', N'Készségfejlesztő iskola', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 110, NULL, 16, NULL),
            (1195, N'Pedagógiai szakszolgálat', N'Pedagógiai szakszolgálat', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 160, NULL, 17, NULL),
            (1196, N'Kiegészítő nemzetiségi nyelvoktatás', N'Kiegészítő nemzetiségi nyelvoktatás', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 350, NULL, 18, NULL),
            (6570, N'Fejlesztő nevelés-oktatás', N'Fejlesztő nevelés-oktatás', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 150, NULL, 19, NULL),
            (6711, N'Utazó gyógypedagógusi hálózat', N'Utazó gyógypedagógusi hálózat', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 210, NULL, 20, NULL),
            (7664, N'Művészeti szakgimnázium', N'Művészeti szakgimnázium', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 50, NULL, 21, NULL),
            (7760, N'Köznevelési és Szakképzési Hídprogram', N'Köznevelési és Szakképzési Hídprogram', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 230, NULL, 22, NULL),
            (7761, N'Utazó konduktori hálózat', N'Utazó konduktori hálózat', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 220, NULL, 23, NULL),
            (7762, N'Óvoda (nemzetiségi)', N'Óvoda (nemzetiségi)', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 1100, NULL, 24, NULL),
            (7763, N'Általános iskola (nemzetiségi)', N'Általános iskola (nemzetiségi)', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 1110, NULL, 25, NULL),
            (7764, N'Gimnázium (nemzetiségi)', N'Gimnázium (nemzetiségi)', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 1120, NULL, 26, NULL),
            (7765, N'Szakgimnázium (nemzetiségi)', N'Szakgimnázium (nemzetiségi)', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 1130, NULL, 27, NULL),
            (7766, N'Szakközépiskola (nemzetiségi)', N'Szakközépiskola (nemzetiségi)', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 1140, NULL, 28, NULL),
            (7767, N'Kollégium (nemzetiségi)', N'Kollégium (nemzetiségi)', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 1150, NULL, 29, NULL),
            (7759, N'Pedagógiai-szakmai szolgáltatás', N'Pedagógiai-szakmai szolgáltatás', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 500, NULL, 30, NULL),
            (7894, N'Szakgimnázium (művészeti, pedagógiai, közművelődési)', N'Szakgimnázium', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 60, NULL, 31, NULL),
            (7895, N'Technikum', N'Technikum', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 70, NULL, 32, NULL),
            (7896, N'Szakképző iskola', N'Szakképző iskola', NULL, NULL, NULL, N'OktatasiNevelesiFeladat', 35, N'T', N'T', 80, NULL, 33, NULL),
            (1136, N'Na', N'Na', NULL, NULL, NULL, N'ErtelmiFogyatekosTipus', 31, N'F', N'T', 1, NULL, 1, NULL),
            (1137, N'Nem értelmi fogyatékos', N'Nem értelmi fogyatékos', NULL, NULL, NULL, N'ErtelmiFogyatekosTipus', 31, N'F', N'T', 2, NULL, 2, NULL),
            (1138, N'Autista', N'Autista', NULL, NULL, NULL, N'ErtelmiFogyatekosTipus', 31, N'F', N'T', 3, NULL, 3, NULL),
            (1139, N'Egyéb pszichés fejlődési zavar', N'Egyéb pszichés fejlődési zavar', NULL, NULL, NULL, N'ErtelmiFogyatekosTipus', 31, N'F', N'T', 4, NULL, 4, NULL),
            (1140, N'Enyhe értelmi fogyatékos', N'Enyhe értelmi fogyatékos', NULL, NULL, NULL, N'ErtelmiFogyatekosTipus', 31, N'F', N'T', 5, NULL, 5, NULL),
            (1141, N'Középsúlyos értelmi fogyatékos', N'Középsúlyos értelmi fogyatékos', NULL, NULL, NULL, N'ErtelmiFogyatekosTipus', 31, N'F', N'T', 6, NULL, 6, NULL),
            (1142, N'Súlyos értelmi fogyatékos', N'Súlyos értelmi fogyatékos', NULL, NULL, NULL, N'ErtelmiFogyatekosTipus', 31, N'F', N'T', 7, NULL, 7, NULL),
            (1143, N'Na', N'Na', NULL, NULL, NULL, N'HallasserultTipus', 32, N'F', N'T', 1, NULL, 1, NULL),
            (1144, N'Nem hallássérült', N'Nem hallássérült', NULL, NULL, NULL, N'HallasserultTipus', 32, N'F', N'T', 2, NULL, 2, NULL),
            (1145, N'Nagyothalló', N'Nagyothalló', NULL, NULL, NULL, N'HallasserultTipus', 32, N'F', N'T', 3, NULL, 3, NULL),
            (1146, N'Siket', N'Siket', NULL, NULL, NULL, N'HallasserultTipus', 32, N'F', N'T', 4, NULL, 4, NULL),
            (1147, N'Na', N'Na', NULL, NULL, NULL, N'LatasserultTipus', 33, N'F', N'T', 1, NULL, 1, NULL),
            (1148, N'Nem látássérült', N'Nem látássérült', NULL, NULL, NULL, N'LatasserultTipus', 33, N'F', N'T', 2, NULL, 2, NULL),
            (1149, N'Gyengénlátó', N'Gyengénlátó', NULL, NULL, NULL, N'LatasserultTipus', 33, N'F', N'T', 3, NULL, 3, NULL),
            (1150, N'Vak', N'Vak', NULL, NULL, NULL, N'LatasserultTipus', 33, N'F', N'T', 4, NULL, 4, NULL),
            (1106, N'Na', N'Na', NULL, NULL, NULL, N'EtkezesiKedvezmenyTipus', 28, N'F', N'T', 1, NULL, 1, NULL),
            (1107, N'Nem étkező', N'Nem étkező', NULL, NULL, NULL, N'EtkezesiKedvezmenyTipus', 28, N'F', N'T', 2, NULL, 2, NULL),
            (1108, N'Nem kedvezményes (egyszeri étkezés)', N'Nem kedvezményes (egyszeri étkezés)', NULL, NULL, NULL, N'EtkezesiKedvezmenyTipus', 28, N'F', N'T', 3, NULL, 3, NULL),
            (1109, N'50%-os kedvezmény (egyszeri étkezés)', N'50%-os kedvezmény (egyszeri étkezés)', NULL, NULL, NULL, N'EtkezesiKedvezmenyTipus', 28, N'F', N'T', 4, NULL, 4, NULL),
            (1110, N'100%-os kedvezmény (egyszeri étkezés)', N'100%-os kedvezmény (egyszeri étkezés)', NULL, NULL, NULL, N'EtkezesiKedvezmenyTipus', 28, N'F', N'T', 5, NULL, 5, NULL),
            (1111, N'Nem kedvezményes (kétszeri étkezés)', N'Nem kedvezményes (kétszeri étkezés)', NULL, NULL, NULL, N'EtkezesiKedvezmenyTipus', 28, N'F', N'T', 6, NULL, 6, NULL),
            (1112, N'50%-os kedvezmény (kétszeri étkezés)', N'50%-os kedvezmény (kétszeri étkezés)', NULL, NULL, NULL, N'EtkezesiKedvezmenyTipus', 28, N'F', N'T', 7, NULL, 7, NULL),
            (1113, N'100%-os kedvezmény (kétszeri étkezés)', N'100%-os kedvezmény (kétszeri étkezés)', NULL, NULL, NULL, N'EtkezesiKedvezmenyTipus', 28, N'F', N'T', 8, NULL, 8, NULL),
            (1114, N'Nem kedvezményes (háromszori étkezés)', N'Nem kedvezményes (háromszori étkezés)', NULL, NULL, NULL, N'EtkezesiKedvezmenyTipus', 28, N'F', N'T', 9, NULL, 9, NULL),
            (1115, N'50%-os kedvezmény (háromszori étkezés)', N'50%-os kedvezmény (háromszori étkezés)', NULL, NULL, NULL, N'EtkezesiKedvezmenyTipus', 28, N'F', N'T', 10, NULL, 10, NULL),
            (1116, N'100%-os kedvezmény (háromszori étkezés)', N'100%-os kedvezmény (háromszori étkezés)', NULL, NULL, NULL, N'EtkezesiKedvezmenyTipus', 28, N'F', N'T', 11, NULL, 11, NULL),
            (1117, N'Nem kedvezményes (ötszöri étkezés)', N'Nem kedvezményes (ötszöri étkezés)', NULL, NULL, NULL, N'EtkezesiKedvezmenyTipus', 28, N'F', N'T', 12, NULL, 12, NULL),
            (1118, N'50%-os kedvezmény (ötszöri étkezés)', N'50%-os kedvezmény (ötszöri étkezés)', NULL, NULL, NULL, N'EtkezesiKedvezmenyTipus', 28, N'F', N'T', 13, NULL, 13, NULL),
            (1119, N'100%-os kedvezmény (ötszöri étkezés)', N'100%-os kedvezmény (ötszöri étkezés)', NULL, NULL, NULL, N'EtkezesiKedvezmenyTipus', 28, N'F', N'T', 14, NULL, 14, NULL),
            (1120, N'Na', N'Na', NULL, NULL, NULL, N'HatranyosHelyzetTipus', 29, N'F', N'T', 1, NULL, 1, NULL),
            (1121, N'Nem hátrányos helyzetű', N'Nem hátrányos helyzetű', NULL, NULL, NULL, N'HatranyosHelyzetTipus', 29, N'F', N'T', 2, NULL, 2, NULL),
            (1122, N'Hátrányos helyzetű', N'Hátrányos helyzetű', NULL, NULL, NULL, N'HatranyosHelyzetTipus', 29, N'F', N'T', 3, NULL, 3, NULL),
            (1123, N'Halmozottan hátrányos helyzetű', N'Halmozottan hátrányos helyzetű', NULL, NULL, NULL, N'HatranyosHelyzetTipus', 29, N'F', N'T', 4, NULL, 4, NULL),
            (8791, N'Hátrányos helyzetű - Halmozottan hátrányos helyzetű', N'Hátrányos helyzetű - Halmozottan hátrányos helyzetű', NULL, NULL, NULL, N'HatranyosHelyzetTipus', 29, N'T', N'F', 10, NULL, 5, NULL),
            (1124, N'Na', N'Na', NULL, NULL, NULL, N'Vallas', 30, N'T', N'T', 1, NULL, 1, NULL),
            (1125, N'Római katolikus', N'Római katolikus', NULL, NULL, NULL, N'Vallas', 30, N'T', N'T', 2, NULL, 2, NULL),
            (1126, N'Görög katolikus', N'Görög katolikus', NULL, NULL, NULL, N'Vallas', 30, N'T', N'T', 3, NULL, 3, NULL),
            (1127, N'Református', N'Református', NULL, NULL, NULL, N'Vallas', 30, N'T', N'T', 4, NULL, 4, NULL),
            (1128, N'Evangélikus', N'Evangélikus', NULL, NULL, NULL, N'Vallas', 30, N'T', N'T', 5, NULL, 5, NULL),
            (1129, N'Baptista', N'Baptista', NULL, NULL, NULL, N'Vallas', 30, N'T', N'T', 6, NULL, 6, NULL),
            (1130, N'Unitárius', N'Unitárius', NULL, NULL, NULL, N'Vallas', 30, N'T', N'T', 7, NULL, 7, NULL),
            (1131, N'Egyéb protestáns', N'Egyéb protestáns', NULL, NULL, NULL, N'Vallas', 30, N'T', N'T', 8, NULL, 8, NULL),
            (1132, N'Ortodox', N'Ortodox', NULL, NULL, NULL, N'Vallas', 30, N'T', N'T', 9, NULL, 9, NULL),
            (1133, N'Izraelita', N'Izraelita', NULL, NULL, NULL, N'Vallas', 30, N'T', N'T', 10, NULL, 10, NULL),
            (1134, N'Hit gyülekezete', N'Hit gyülekezete', NULL, NULL, NULL, N'Vallas', 30, N'T', N'T', 11, NULL, 11, NULL),
            (1135, N'Pünkösdi', N'Pünkösdi', NULL, NULL, NULL, N'Vallas', 30, N'T', N'T', 12, NULL, 12, NULL),
            (6516, N'Iszlám', N'Iszlám', NULL, NULL, NULL, N'Vallas', 30, N'T', N'T', 13, NULL, 13, NULL),
            (6517, N'Buddhista', N'Buddhista', NULL, NULL, NULL, N'Vallas', 30, N'T', N'T', 14, NULL, 14, NULL),
            (6518, N'Jehova Tanúi', N'Jehova Tanúi', NULL, NULL, NULL, N'Vallas', 30, N'T', N'T', 15, NULL, 15, NULL),
            (6519, N'Egyéb', N'Egyéb', NULL, NULL, NULL, N'Vallas', 30, N'T', N'T', 16, NULL, 16, NULL),
            (1095, N'Na', N'Na', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 1, NULL, 1, NULL),
            (1096, N'Nappali rendszerű iskolai oktatás', N'Nappali rendszerű iskolai oktatás', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 10, NULL, 2, NULL),
            (1097, N'Felnőttoktatás', N'Felnőttoktatás', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 20, NULL, 3, NULL),
            (1099, N'Bentlakásos intézményi ellátás', N'Bentlakásos intézményi ellátás', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 90, NULL, 4, NULL),
            (1103, N'Esti iskolai oktatás', N'Esti iskolai oktatás', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 30, NULL, 5, NULL),
            (1101, N'Felnőttképzés', N'Felnőttképzés', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 240, NULL, 6, NULL),
            (5446, N'Felnőttoktatás esti rendszerű', N'Esti rendszerű felnőttoktatás', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 60, NULL, 7, NULL),
            (5447, N'Felnőttoktatás levelező rendszerű', N'Levelező rendszerű felnőttoktatás', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 70, NULL, 8, NULL),
            (5445, N'Felnőttoktatás nappali rendszerű', N'Nappali rendszerű felnőttoktatás', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 50, NULL, 9, NULL),
            (1102, N'Iskolarendszeren kívüli egyéb tevékenység', N'Iskolarendszeren kívüli egyéb tevékenység', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 220, NULL, 10, NULL),
            (1104, N'Levelező iskolai oktatás', N'Levelező iskolai oktatás', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 40, NULL, 11, NULL),
            (1098, N'Óvodai nevelés', N'Óvodai nevelés', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 100, NULL, 12, NULL),
            (5448, N'Speciális szakképzés', N'Speciális szakképzés', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 210, NULL, 13, NULL),
            (1100, N'Szakértői bizottsági tevékenység', N'Szakértői bizottsági tevékenység', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 230, NULL, 14, NULL),
            (1105, N'Távoktatásos képzés', N'Távoktatásos képzés', NULL, NULL, NULL, N'TanterviJelleg', 27, N'T', N'T', 200, NULL, 15, NULL),
            (8827, N'Felnőttképzési jogviszony munkarend nélkül', N'Munkarend nélküli oktatás', NULL, NULL, NULL, N'TanterviJelleg', 27, N'F', N'T', 80, NULL, 16, NULL),
            (1090, N'Na', N'Na', NULL, NULL, NULL, N'TestnevelesTipus', 26, N'F', N'T', 1, NULL, 1, NULL),
            (1091, N'Normál testnevelés', N'Normál testnevelés', NULL, NULL, NULL, N'TestnevelesTipus', 26, N'F', N'T', 2, NULL, 2, NULL),
            (1092, N'Könnyített testnevelés', N'Könnyített testnevelés', NULL, NULL, NULL, N'TestnevelesTipus', 26, N'F', N'T', 3, NULL, 3, NULL),
            (1093, N'Gyógytestnevelés', N'Gyógytestnevelés', NULL, NULL, NULL, N'TestnevelesTipus', 26, N'F', N'T', 4, NULL, 4, NULL),
            (1094, N'Felmentett', N'Felmentett', NULL, NULL, NULL, N'TestnevelesTipus', 26, N'F', N'T', 5, NULL, 5, NULL),
            (5228, N'Mozgásnevelés', N'Mozgásnevelés', NULL, NULL, NULL, N'TestnevelesTipus', 26, N'F', N'T', 6, NULL, 6, NULL),
            (1074, N'Na', N'Na', NULL, NULL, NULL, N'EszkozJelleg', 23, N'F', N'T', 1, NULL, 1, NULL),
            (1075, N'Egyedi', N'Egyedi', NULL, NULL, NULL, N'EszkozJelleg', 23, N'F', N'T', 2, NULL, 2, NULL),
            (1076, N'Tömeges', N'Tömeges', NULL, NULL, NULL, N'EszkozJelleg', 23, N'F', N'T', 3, NULL, 3, NULL),
            (1077, N'Na', N'Na', NULL, NULL, NULL, N'EszkozCel', 24, N'F', N'T', 1, NULL, 1, NULL),
            (1079, N'Tanári felkészülésre használt', N'Tanári felkészülésre használt', NULL, NULL, NULL, N'EszkozCel', 24, N'F', N'T', 2, NULL, 2, NULL),
            (1078, N'Oktatási célra használt, nem akadálymentesített', N'Oktatási célra használt, nem akadálymentesített', NULL, NULL, NULL, N'EszkozCel', 24, N'F', N'T', 3, NULL, 3, NULL),
            (1084, N'Oktatási célra használt, részben akadálymentesített', N'Oktatási célra használt, részben akadálymentesített', NULL, NULL, NULL, N'EszkozCel', 24, N'F', N'T', 4, NULL, 4, NULL),
            (1085, N'Oktatási célra használt, akadálymentesített', N'Oktatási célra használt, akadálymentesített', NULL, NULL, NULL, N'EszkozCel', 24, N'F', N'T', 5, NULL, 5, NULL),
            (1083, N'Közösségi célra használt', N'Közösségi célra használt', NULL, NULL, NULL, N'EszkozCel', 24, N'F', N'T', 6, NULL, 6, NULL),
            (1080, N'Adminisztrációs célra használt', N'Adminisztrációs célra használt', NULL, NULL, NULL, N'EszkozCel', 24, N'F', N'T', 7, NULL, 7, NULL),
            (1081, N'Ügyviteli célra használt', N'Ügyviteli célra használt', NULL, NULL, NULL, N'EszkozCel', 24, N'F', N'T', 8, NULL, 8, NULL),
            (1082, N'Egyéb célra használt', N'Egyéb célra használt', NULL, NULL, NULL, N'EszkozCel', 24, N'F', N'T', 9, NULL, 9, NULL),
            (1086, N'Na', N'Na', NULL, NULL, NULL, N'MertekegysegTipus', 25, N'F', N'T', 1, NULL, 1, NULL),
            (1087, N'Darab', N'Darab', NULL, NULL, NULL, N'MertekegysegTipus', 25, N'F', N'T', 2, NULL, 2, NULL),
            (1088, N'Kilogramm', N'Kilogramm', NULL, NULL, NULL, N'MertekegysegTipus', 25, N'F', N'T', 3, NULL, 3, NULL),
            (1089, N'Liter', N'Liter', NULL, NULL, NULL, N'MertekegysegTipus', 25, N'F', N'T', 4, NULL, 4, NULL),
            (6567, N'Méter', N'Méter', NULL, NULL, NULL, N'MertekegysegTipus', 25, N'F', N'T', 5, NULL, 5, NULL),
            (6568, N'Sorozat', N'Sorozat', NULL, NULL, NULL, N'MertekegysegTipus', 25, N'F', N'T', 6, NULL, 6, NULL),
            (6569, N'Doboz', N'Doboz', NULL, NULL, NULL, N'MertekegysegTipus', 25, N'F', N'T', 7, NULL, 7, NULL),
            (985, N'Na', N'Na', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 1, NULL, 1, NULL),
            (986, N'Fővárosi önkormányzat', N'Fővárosi önkormányzat', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 2, NULL, 2, NULL),
            (987, N'Fővárosi kerületi önkormányzat', N'Fővárosi kerületi önkormányzat', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 3, NULL, 3, NULL),
            (988, N'Állami szerv', N'Állami szerv', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 4, NULL, 4, NULL),
            (989, N'Egyházi jogi személy', N'Egyházi jogi személy', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 5, NULL, 5, NULL),
            (990, N'Alapítvány', N'Alapítvány', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 6, NULL, 6, NULL),
            (991, N'Egyéni vállalkozó', N'Egyéni vállalkozó', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 7, NULL, 7, NULL),
            (992, N'Egyéb', N'Egyéb', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 8, NULL, 8, NULL),
            (993, N'Községi önkormányzat', N'Községi önkormányzat', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 9, NULL, 9, NULL),
            (994, N'Városi önkormányzat', N'Városi önkormányzat', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 10, NULL, 10, NULL),
            (995, N'Megyei jogú városi önkormányzat', N'Megyei jogú városi önkormányzat', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 11, NULL, 11, NULL),
            (996, N'Megyei önkormányzat', N'Megyei önkormányzat', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 12, NULL, 12, NULL),
            (997, N'Helyi önkormányzati társulás', N'Helyi önkormányzati társulás', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 13, NULL, 13, NULL),
            (998, N'Kisebbségi önkormányzat', N'Kisebbségi önkormányzat', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 14, NULL, 14, NULL),
            (999, N'Gazdálkodó szervezet', N'Gazdálkodó szervezet', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 15, NULL, 15, NULL),
            (1000, N'Felsőoktatási intézet', N'Felsőoktatási intézet', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 16, NULL, 16, NULL),
            (1001, N'Közalapítvány', N'Közalapítvány', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 17, NULL, 17, NULL),
            (1002, N'Közhasznú társaság', N'Közhasznú társaság', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 18, NULL, 18, NULL),
            (1003, N'Egyesület', N'Egyesület', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 19, NULL, 19, NULL),
            (1004, N'Fenntartói társulás', N'Fenntartói társulás', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 20, NULL, 20, NULL),
            (1005, N'Települési kisebbségi önkormányzat', N'Települési kisebbségi önkormányzat', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 21, NULL, 21, NULL),
            (1006, N'Országos kisebbségi önkormányzat', N'Országos kisebbségi önkormányzat', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 22, NULL, 22, NULL),
            (1007, N'Területi kisebbségi önkormányzat', N'Területi kisebbségi önkormányzat', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 23, NULL, 23, NULL),
            (1008, N'Korlátolt felelősségű társaság', N'Korlátolt felelősségű társaság', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 24, NULL, 24, NULL),
            (1009, N'Szövetkezet', N'Szövetkezet', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 25, NULL, 25, NULL),
            (1010, N'Részvénytársaság', N'Részvénytársaság', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 26, NULL, 26, NULL),
            (1011, N'Állami felsőoktatási intézmény', N'Állami felsőoktatási intézmény', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 27, NULL, 27, NULL),
            (1012, N'Egyházi felsőoktatási intézmény', N'Egyházi felsőoktatási intézmény', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 28, NULL, 28, NULL),
            (1013, N'Egyéb felsőoktatási intézmény', N'Egyéb felsőoktatási intézmény', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 29, NULL, 29, NULL),
            (1014, N'Közhasznú alapítvány', N'Közhasznú alapítvány', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 30, NULL, 30, NULL),
            (1015, N'Közhasznú egyesület', N'Közhasznú egyesület', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 31, NULL, 31, NULL),
            (1016, N'Önkormányzatok fenntartói társulása', N'Önkormányzatok fenntartói társulása', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 32, NULL, 32, NULL),
            (1017, N'Nem önkormányzatok fenntartói társulása', N'Nem önkormányzatok fenntartói társulása', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 33, NULL, 33, NULL),
            (1018, N'Többcélú kistérségi társulás', N'Többcélú kistérségi társulás', NULL, NULL, NULL, N'FenntartoTipus', 19, N'F', N'T', 34, NULL, 34, NULL),
            (1031, N'Na', N'Na', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 1, NULL, 1, NULL),
            (1032, N'Óvodai csoport', N'Óvodai csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 100, NULL, 2, NULL),
            (1034, N'Iskolai csoport (tanórai célú)', N'Iskolai csoport (tanórai célú)', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 110, NULL, 3, NULL),
            (1035, N'Napközis csoport', N'Napközis csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 420, NULL, 4, NULL),
            (1036, N'Tanulószobai csoport', N'Tanulószobai csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 500, NULL, 5, NULL),
            (1037, N'Szakköri csoport', N'Szakköri csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 470, NULL, 6, NULL),
            (1038, N'Érdeklődési kör', N'Érdeklődési kör', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 190, NULL, 7, NULL),
            (1039, N'Önképzőkör', N'Önképző kör', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 430, NULL, 8, NULL),
            (1040, N'Énekkar', N'Énekkar', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 180, NULL, 9, NULL),
            (1041, N'Művészeti csoport', N'Művészeti csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 410, NULL, 10, NULL),
            (1042, N'Iskolai sportkör', N'Iskolai sportkör', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 310, NULL, 11, NULL),
            (1065, N'Gyógytestnevelési csoport', N'Gyógytestnevelési csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 170, NULL, 12, NULL),
            (1061, N'Idegennyelvi többletórai', N'Idegennyelvi többletórai', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 390, NULL, 13, NULL),
            (1062, N'Fakultációs csoport', N'Fakultációs csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 210, NULL, 14, NULL),
            (1043, N'Egyéb, szervezett tanórán kívüli csoport', N'Egyéb, szervezett tanórán kívüli csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 280, NULL, 15, NULL),
            (1044, N'Felzárkóztató foglalkozási csoport', N'Felzárkóztató foglalkozási csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 300, NULL, 16, NULL),
            (1045, N'Fejlesztő foglalkozási csoport', N'Fejlesztő foglalkozási csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 200, NULL, 17, NULL),
            (1046, N'Tehetséggondozó csoport', N'Tehetséggondozó csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 160, NULL, 18, NULL),
            (1047, N'Konzultációs csoport', N'Konzultációs csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 240, NULL, 19, NULL),
            (1048, N'Egyéb, nem kötelező (választható) foglalkozási csoport', N'Egyéb, nem kötelező (választható) foglalkozási csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 220, NULL, 20, NULL),
            (1049, N'Kollégiumi ügyeleti csoport – hétköznap', N'Kollégiumi ügyeleti csoport - hétközben', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 510, NULL, 21, NULL),
            (1050, N'Kollégiumi ügyeleti csoport – pihenő-, munkaszüneti nap', N'Kollégiumi ügyeleti csoport - hétvégén', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 380, NULL, 22, NULL),
            (1051, N'Kollégiumi foglalkozási csoport', N'Kollégiumi foglalkozási csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 150, NULL, 23, NULL),
            (1052, N'Diákotthoni ügyeleti csoport - hétközben', N'Diákotthoni ügyeleti csoport - hétközben', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 360, NULL, 24, NULL),
            (1053, N'Diákotthoni ügyeleti csoport - hétvégén', N'Diákotthoni ügyeleti csoport - hétvégén', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 370, NULL, 25, NULL),
            (1054, N'Diákotthoni csoport', N'Diákotthoni csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 320, NULL, 26, NULL),
            (1055, N'Gyermekotthoni ügyeleti csoport - hétközben', N'Gyermekotthoni ügyeleti csoport - hétközben', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 130, NULL, 27, NULL),
            (1056, N'Gyermekotthoni ügyeleti csoport - hétvégén', N'Gyermekotthoni ügyeleti csoport - hétvégén', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 140, NULL, 28, NULL),
            (1057, N'Gyermekotthoni csoport', N'Gyermekotthoni csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 120, NULL, 29, NULL),
            (1058, N'Korai fejlesztés', N'Korai fejlesztés', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 260, NULL, 30, NULL),
            (1059, N'Fejlesztő felkészítés', N'Fejlesztő felkészítés', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 270, NULL, 31, NULL),
            (1060, N'Szakértői vizsgálat', N'Szakértői vizsgálat', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 250, NULL, 32, NULL),
            (1063, N'Logopédiai csoport', N'Logopédiai csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 460, NULL, 33, NULL),
            (1064, N'Egyéni foglalkozási csoport', N'Egyéni foglalkozási csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 400, NULL, 34, NULL),
            (1066, N'Pénzügyi csoport', N'Pénzügyi csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 450, NULL, 35, NULL),
            (1067, N'Alapf. müv. okt. zeneművészeti csoport - egyéni főtanszak', N'Alapf. müv. okt. zeneművészeti csoport - egyéni főtanszak', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 620, NULL, 36, NULL),
            (1068, N'Alapf. müv. okt. zeneművészeti csoport - csoportos főtanszak', N'Alapf. müv. okt. zeneművészeti csoport - csoportos főtanszak', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 570, NULL, 37, NULL),
            (1069, N'Alapf. müv. okt. képző- és iparművészeti csoport', N'Alapf. müv. okt. képző- és iparművészeti csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 530, NULL, 38, NULL),
            (1070, N'Alapf. müv. okt. táncművészeti csoport', N'Alapf. müv. okt. táncművészeti csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 550, NULL, 39, NULL),
            (1071, N'Alapf. müv. okt. szín- és bábművészeti csoport', N'Alapf. müv. okt. szín- és bábművészeti csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 540, NULL, 40, NULL),
            (1072, N'Fejlesztő iskolai csoport', N'Fejlesztő iskolai csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 230, NULL, 41, NULL),
            (1073, N'Egybefüggő szakmai gyakorlati csoport', N'Egybefüggő szakmai gyakorlati csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 440, NULL, 42, NULL),
            (5224, N'Utazó gyógypedagógus csoportja', N'Utazó gyógypedagógus csoportja', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 520, NULL, 43, NULL),
            (5443, N'Szakmai gyakorlati csoport', N'Szakmai gyakorlati csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 480, NULL, 44, NULL),
            (6756, N'Alapf. műv. okt. zeneművészeti csoport - csoportos kötelezően választható', N'Alapf. műv. okt. zeneművészeti csoport - csoportos kötelezően választható', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 590, NULL, 45, NULL),
            (6757, N'Alapf. műv. okt. zeneművészeti csoport - egyéni kötelezően választható', N'Alapf. műv. okt. zeneművészeti csoport - egyéni kötelezően választható', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 640, NULL, 46, NULL),
            (6758, N'Alapf. műv. okt. zeneművészeti csoport - csoportos kötelező', N'Alapf. műv. okt. zeneművészeti csoport - csoportos kötelező', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 580, NULL, 47, NULL),
            (6759, N'Alapf. műv. okt. zeneművészeti csoport - egyéni kötelező', N'Alapf. műv. okt. zeneművészeti csoport - egyéni kötelező', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 630, NULL, 48, NULL),
            (6760, N'Alapf. műv. okt. zeneművészeti csoport - csoportos választható', N'Alapf. műv. okt. zeneművészeti csoport - csoportos választható', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 600, NULL, 49, NULL),
            (6761, N'Alapf. műv. okt. zeneművészeti csoport - egyéni választható', N'Alapf. műv. okt. zeneművészeti csoport - egyéni választható', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 650, NULL, 50, NULL),
            (6877, N'Hittan csoport', N'Hittan csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 290, NULL, 51, NULL),
            (7684, N'Kollégiumi ügyeleti csoport', N'Kollégiumi ügyeleti csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 340, NULL, 52, NULL),
            (7685, N'Kollégiumi ügyeleti csoport – hétköznap éjszaka', N'Kollégiumi ügyeleti csoport – hétköznap éjszaka', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 365, NULL, 53, NULL),
            (7686, N'Kollégiumi pedagógiai felügyeleti csoport', N'Kollégiumi pedagógiai felügyeleti csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 330, NULL, 54, NULL),
            (7725, N'Kollégiumi ügyeleti csoport – pihenő-, munkaszüneti nap éjszaka', N'Kollégiumi ügyeleti csoport – pihenő-, munkaszüneti nap éjszaka', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 375, NULL, 55, NULL),
            (7726, N'Szakértői bizottsági tevékenység (pedagógiai szakszolgálat)', N'Szakértői bizottsági tevékenység', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 710, NULL, 56, NULL),
            (7727, N'Nevelési tanácsadás (pedagógiai szakszolgálat)', N'Nevelési tanácsadás', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 720, NULL, 57, NULL),
            (7728, N'Konduktív pedagógiai ellátás (pedagógiai szakszolgálat)', N'Konduktív pedagógiai ellátás', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 750, NULL, 58, NULL),
            (7729, N'Iskolapszichológiai, óvodapszichológiai ellátás (pedagógiai szakszolgálat)', N'Iskolapszichológiai, óvodapszichológiai ellátás', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 770, NULL, 59, NULL),
            (7730, N'Logopédiai ellátás (pedagógiai szakszolgálat)', N'Logopédiai ellátás', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 730, NULL, 60, NULL),
            (7731, N'Gyógytestnevelés (pedagógiai szakszolgálat)', N'Gyógytestnevelés', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 760, NULL, 61, NULL),
            (7732, N'Gyógypedagógiai tanácsadás, korai fejlesztés és gondozás (pedagógiai szakszolgálat)', N'Gyógypedagógiai tanácsadás, korai fejlesztés és gondozás', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 700, NULL, 62, NULL),
            (7733, N'Kiemelten tehetséges gyermekek, tanulók gondozása (pedagógiai szakszolgálat)', N'Kiemelten tehetséges gyermekek, tanulók gondozása', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 780, NULL, 63, NULL),
            (7734, N'Továbbtanulási, pályaválasztási tanácsadás (pedagógiai szakszolgálat)', N'Továbbtanulási, pályaválasztási tanácsadás', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 740, NULL, 64, NULL),
            (7806, N'Könyvtár csoport', N'Könyvtár csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 395, NULL, 65, NULL),
            (7807, N'Egész napos iskola csoport', N'Egész napos iskola csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'T', 145, NULL, 66, NULL),
            (8831, N'Duális képzési csoport', N'Duális képzési csoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'F', 790, NULL, 67, NULL),
            (9079, N'Apáczai mentorcsoport', N'Apáczai mentorcsoport', NULL, NULL, NULL, N'CsoportTipus', 22, N'T', N'F', 800, NULL, 68, NULL),
            (972, N'Na', N'Na', NULL, NULL, NULL, N'RokonsagiFok', 18, N'T', N'T', 1, NULL, 1, NULL),
            (973, N'Édesapa', N'Édesapa', NULL, NULL, NULL, N'RokonsagiFok', 18, N'T', N'T', 2, NULL, 2, NULL),
            (974, N'Édesanya', N'Édesanya', NULL, NULL, NULL, N'RokonsagiFok', 18, N'T', N'T', 3, NULL, 3, NULL),
            (975, N'Nevelő apa', N'Nevelő apa', NULL, NULL, NULL, N'RokonsagiFok', 18, N'T', N'T', 4, NULL, 4, NULL),
            (976, N'Nevelő anya', N'Nevelő anya', NULL, NULL, NULL, N'RokonsagiFok', 18, N'T', N'T', 5, NULL, 5, NULL),
            (977, N'Testvér', N'Testvér', NULL, NULL, NULL, N'RokonsagiFok', 18, N'T', N'T', 6, NULL, 6, NULL),
            (978, N'Nagyszülő', N'Nagyszülő', NULL, NULL, NULL, N'RokonsagiFok', 18, N'T', N'T', 7, NULL, 7, NULL),
            (979, N'Gyám', N'Gyám', NULL, NULL, NULL, N'RokonsagiFok', 18, N'T', N'T', 8, NULL, 8, NULL),
            (980, N'Hivatásos nevelőszülő', N'Hivatásos nevelőszülő', NULL, NULL, NULL, N'RokonsagiFok', 18, N'T', N'T', 9, NULL, 9, NULL),
            (981, N'Gyermekotthon vezetője', N'Gyermekotthon vezetője', NULL, NULL, NULL, N'RokonsagiFok', 18, N'T', N'T', 10, NULL, 10, NULL),
            (982, N'Egyéb rokoni kapcsolat', N'Egyéb rokoni kapcsolat', NULL, NULL, NULL, N'RokonsagiFok', 18, N'T', N'T', 11, NULL, 11, NULL),
            (983, N'Egyéb nem rokoni kapcsolat', N'Egyéb nem rokoni kapcsolat', NULL, NULL, NULL, N'RokonsagiFok', 18, N'T', N'T', 12, NULL, 12, NULL),
            (984, N'Gondnok', N'Gondnok', NULL, NULL, NULL, N'RokonsagiFok', 18, N'T', N'T', 13, NULL, 13, NULL),
            (7571, N'Kollégiumi nevelő', N'Kollégiumi nevelő', NULL, NULL, NULL, N'RokonsagiFok', 18, N'T', N'T', 14, NULL, 14, NULL),
            (6812, N'fejlesztő pedagógus (*)', N'fejlesztő pedagógus (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 20, NULL, 1, NULL),
            (6813, N'pedagógus (ágazatspecifikus szociális területen)', N'Pedagógus (ágazatspecifikus szociális területen)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1900, NULL, 2, NULL),
            (566, N'NOKS ápoló (*)', N'NOKS ápoló (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 500, NULL, 3, NULL),
            (639, N'családgondozó', N'Családgondozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1130, NULL, 4, NULL),
            (637, N'családsegítő', N'Családsegítő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1140, NULL, 5, NULL),
            (560, N'NOKS dajka (*)', N'NOKS dajka (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1165, NULL, 6, NULL),
            (574, N'fűtő', N'Fűtő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1340, NULL, 7, NULL),
            (569, N'EGYÉB gazdasági dolgozó (*)', N'EGYÉB gazdasági dolgozó (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 800, NULL, 8, NULL),
            (600, N'Gondnok', N'Gondnok', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1360, NULL, 9, NULL),
            (636, N'gondozó', N'Gondozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1370, NULL, 10, NULL),
            (561, N'NOKS gondozónő és takarító (*)', N'NOKS gondozónő és takarító (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 520, NULL, 11, NULL),
            (558, N'gyermek- és ifjúságvédelmi felügyelő', N'Gyermek- és ifjúságvédelmi felügyelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1400, NULL, 12, NULL),
            (563, N'NOKS gyermek- és ifjúságvédelmi felügyelő (*)', N'NOKS gyermek- és ifjúságvédelmi felügyelő (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 530, NULL, 13, NULL),
            (635, N'gyermekfelügyelő', N'Gyermekfelügyelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1410, NULL, 14, NULL),
            (638, N'gyermekvédelmi asszisztens', N'Gyermekvédelmi asszisztens', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1450, NULL, 15, NULL),
            (642, N'gyermekvédelmi ügyintéző', N'Gyermekvédelmi ügyintéző', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1480, NULL, 16, NULL),
            (559, N'NOKS gyógypedagógiai asszisztens (*)', N'NOKS gyógypedagógiai asszisztens (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 540, NULL, 17, NULL),
            (597, N'gyógypedagógus (*)', N'gyógypedagógus (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 50, NULL, 18, NULL),
            (633, N'NOKS gyógytornász (*)', N'NOKS gyógytornász (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1497, NULL, 19, NULL),
            (567, N'NOKS hangszerkarbantartó (*)', N'NOKS hangszerkarbantartó (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1505, NULL, 20, NULL),
            (584, N'NOKS iskolatitkár (*)', N'NOKS iskolatitkár (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 595, NULL, 21, NULL),
            (573, N'karbantartó', N'Karbantartó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1600, NULL, 22, NULL),
            (572, N'EGYÉB kisegítő dolgozó (*)', N'EGYÉB kisegítő dolgozó (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 810, NULL, 23, NULL),
            (598, N'konduktor (*)', N'konduktor (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1547, NULL, 24, NULL),
            (3026, N'NOKS könyvtáros (*)', N'NOKS könyvtáros (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 600, NULL, 25, NULL),
            (582, N'EGYÉB könyvtáros asszisztens (*)', N'EGYÉB könyvtáros asszisztens (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1665, NULL, 26, NULL),
            (553, N'könyvtárostanár (tanító)', N'könyvtárostanár (tanító)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 140, NULL, 27, NULL),
            (581, N'EGYÉB könyvtáros technikus (*)', N'EGYÉB könyvtáros technikus (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1667, NULL, 28, NULL),
            (589, N'NOKS laboráns (*)', N'NOKS laboráns (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 610, NULL, 29, NULL),
            (555, N'gyógypedagógus (logopédus) (*)', N'gyógypedagógus (logopédus) (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1715, NULL, 30, NULL),
            (591, N'munkaügyi előadó', N'Munkaügyi előadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1760, NULL, 31, NULL),
            (571, N'EGYÉB műszaki dolgozó (*)', N'EGYÉB műszaki dolgozó (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 850, NULL, 32, NULL),
            (565, N'NOKS műszaki vezető (*)', N'NOKS műszaki vezető (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 620, NULL, 33, NULL),
            (548, N'Na', N'Na', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1, NULL, 34, NULL),
            (552, N'nevelő', N'Nevelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1800, NULL, 35, NULL),
            (641, N'növendékügyi előadó', N'Növendékügyi előadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1830, NULL, 36, NULL),
            (586, N'EGYÉB oktatástechnikus (*)', N'EGYÉB oktatástechnikus (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 860, NULL, 37, NULL),
            (577, N'orvos', N'Orvos', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1850, NULL, 38, NULL),
            (595, N'óvodapedagógus (*)', N'óvodapedagógus (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1873, NULL, 39, NULL),
            (583, N'NOKS óvodatitkár (*)', N'NOKS óvodatitkár (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1877, NULL, 40, NULL),
            (585, N'NOKS pedagógiai asszisztens (*)', N'NOKS pedagógiai asszisztens (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 640, NULL, 41, NULL),
            (562, N'NOKS pedagógiai felügyelő (*)', N'NOKS pedagógiai felügyelő (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1895, NULL, 42, NULL),
            (576, N'portás', N'Portás', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1920, NULL, 43, NULL),
            (578, N'pszichológus (pedagógiai szakszolgálat) (*)', N'pszichológus (pedagógiai szakszolgálat) (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1945, NULL, 44, NULL),
            (594, N'NOKS rendszergazda (*)', N'NOKS rendszergazda (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 670, NULL, 45, NULL),
            (564, N'NOKS szabadidő-szervező (*)', N'NOKS szabadidő-szervező (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 680, NULL, 46, NULL),
            (557, N'NOKS szakorvos (*)', N'NOKS szakorvos (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2067, NULL, 47, NULL),
            (587, N'EGYÉB számítógép-kezelő (*)', N'EGYÉB számítógép-kezelő (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2085, NULL, 48, NULL),
            (588, N'EGYÉB számítógép rendszerprogramozó (*)', N'EGYÉB számítógép rendszerprogramozó (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2087, NULL, 49, NULL),
            (592, N'személyzeti előadó', N'Személyzeti előadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2100, NULL, 50, NULL),
            (579, N'szociális munkás', N'szociális munkás', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 705, NULL, 51, NULL),
            (580, N'szociálpedagógus (*)', N'szociálpedagógus (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2155, NULL, 52, NULL),
            (599, N'takarító', N'Takarító', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2160, NULL, 53, NULL),
            (568, N'NOKS úszómester (*)', N'NOKS úszómester (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 710, NULL, 54, NULL),
            (640, N'utógondozó', N'Utógondozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2290, NULL, 55, NULL),
            (570, N'EGYÉB ügyviteli dolgozó (*)', N'EGYÉB ügyviteli dolgozó (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 890, NULL, 56, NULL),
            (590, N'EGYÉB ügyviteli gépkezelő (*)', N'EGYÉB ügyviteli gépkezelő (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 900, NULL, 57, NULL),
            (6329, N'addiktológiai konzultáns', N'addiktológiai konzultáns', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1000, NULL, 58, NULL),
            (6330, N'addiktológiai koordinátor', N'addiktológiai koordinátor', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1010, NULL, 59, NULL),
            (6331, N'adminisztrátor', N'adminisztrátor', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1020, NULL, 60, NULL),
            (6332, N'általános ápoló', N'általános ápoló', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1030, NULL, 61, NULL),
            (6333, N'általános titkár, titkárnő', N'általános titkár, titkárnő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1040, NULL, 62, NULL),
            (6334, N'anyagnyilvántartó', N'anyagnyilvántartó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1050, NULL, 63, NULL),
            (6335, N'ápolási asszisztens', N'ápolási asszisztens', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1060, NULL, 64, NULL),
            (6336, N'ápoló kizárólagosan sajátos nevelési igényű tanulókat ellátó kollégiumban', N'ápoló kizárólagosan sajátos nevelési igényű tanulókat ellátó kollégiumban', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1070, NULL, 65, NULL),
            (6337, N'ápoló kollégiumban', N'ápoló kollégiumban', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1080, NULL, 66, NULL),
            (6338, N'asszisztens', N'asszisztens', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1090, NULL, 67, NULL),
            (6339, N'bér- és társadalombiztosítási ügyintéző', N'bér- és társadalombiztosítási ügyintéző', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1100, NULL, 68, NULL),
            (6340, N'bérelszámoló', N'bérelszámoló', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1110, NULL, 69, NULL),
            (6341, N'betanított munkás', N'betanított munkás', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1120, NULL, 70, NULL),
            (6342, N'csecsemő- és gyermekápoló', N'csecsemő- és gyermekápoló', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1150, NULL, 71, NULL),
            (6343, N'csecsemő- és gyermekgyógyászat szakorvos', N'csecsemő- és gyermekgyógyászat szakorvos', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1160, NULL, 72, NULL),
            (6344, N'diétás nővér', N'diétás nővér', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1170, NULL, 73, NULL),
            (6345, N'egyéb irodai és ügyviteli munkakör', N'egyéb irodai és ügyviteli munkakör', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1180, NULL, 74, NULL),
            (6346, N'egyéb kisegítő', N'egyéb kisegítő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1190, NULL, 75, NULL),
            (6347, N'egyéb ügyintéző', N'egyéb ügyintéző', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1200, NULL, 76, NULL),
            (6348, N'együttnevelést segítő pedagógus (*)', N'együttnevelést segítő pedagógus (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1210, NULL, 77, NULL),
            (6349, N'éjjeliőr', N'éjjeliőr', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1220, NULL, 78, NULL),
            (6350, N'élelmezésvezető', N'élelmezésvezető', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1230, NULL, 79, NULL),
            (6351, N'elhelyezési ügyintéző', N'elhelyezési ügyintéző', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1240, NULL, 80, NULL),
            (6352, N'ének-zene tanár (*)', N'ének-zene tanár (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 15, NULL, 81, NULL),
            (6353, N'esetfelelős', N'esetfelelős', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1260, NULL, 82, NULL),
            (6354, N'esetmenedzser', N'esetmenedzser', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1270, NULL, 83, NULL),
            (6355, N'falugondnok', N'falugondnok', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1280, NULL, 84, NULL),
            (6356, N'foglalkoztatás-szervező', N'foglalkoztatás-szervező', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1300, NULL, 85, NULL),
            (6357, N'foglalkoztatás-vezető', N'foglalkoztatás-vezető', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1310, NULL, 86, NULL),
            (6358, N'foglalkoztató', N'foglalkoztató', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1320, NULL, 87, NULL),
            (6359, N'fogyatékosság típusának megfelelő szakorvos', N'fogyatékosság típusának megfelelő szakorvos', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1330, NULL, 88, NULL),
            (6360, N'gépkocsivezető', N'gépkocsivezető', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1350, NULL, 89, NULL),
            (6361, N'gimnáziumi tanár (*)', N'gimnáziumi tanár (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 30, NULL, 90, NULL),
            (6362, N'gondozó (irányító gondozó)', N'gondozó (irányító gondozó)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1380, NULL, 91, NULL),
            (6363, N'gyermek- és ifjúsági pszichiátria szakorvos', N'gyermek- és ifjúsági pszichiátria szakorvos', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1390, NULL, 92, NULL),
            (6364, N'gyermekgondozó', N'gyermekgondozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1420, NULL, 93, NULL),
            (6365, N'gyermekneurológia szakorvos', N'gyermekneurológia szakorvos', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1430, NULL, 94, NULL),
            (6366, N'gyermekotthon vezető', N'gyermekotthon vezető', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1440, NULL, 95, NULL),
            (6367, N'gyermekvédelmi gyám', N'gyermekvédelmi gyám', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1460, NULL, 96, NULL),
            (6368, N'gyermekvédelmi szakértői bizottság családgondozója', N'gyermekvédelmi szakértői bizottság családgondozója', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1470, NULL, 97, NULL),
            (6369, N'gyógypedagógiai/fejlesztőpedagógiai tanácsadó', N'gyógypedagógiai/fejlesztőpedagógiai tanácsadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1490, NULL, 98, NULL),
            (6370, N'gyógytestnevelő tanár (pedagógiai szakszolgálat) (*)', N'gyógytestnevelő tanár (pedagógiai szakszolgálat) (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1495, NULL, 99, NULL),
            (6371, N'gyors- és gépíró', N'gyors- és gépíró', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1500, NULL, 100, NULL),
            (6372, N'helyettes szülői tanácsadó', N'helyettes szülői tanácsadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1510, NULL, 101, NULL),
            (6373, N'hivatalsegéd', N'hivatalsegéd', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1520, NULL, 102, NULL),
            (6374, N'humán erőgazdálkodási ügyintéző', N'humán erőgazdálkodási ügyintéző', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1530, NULL, 103, NULL),
            (6375, N'idegennyelvtanár (*)', N'idegennyelvtanár (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 90, NULL, 104, NULL),
            (6376, N'iratkezelő', N'iratkezelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1540, NULL, 105, NULL),
            (6377, N'iskola- és óvodapszichológia koordinátora (*)', N'iskola- és óvodapszichológia koordinátora (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1545, NULL, 106, NULL),
            (6378, N'iskolai szociális munkás', N'iskolai szociális munkás', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1550, NULL, 107, NULL),
            (6379, N'javítóintézeti ápoló', N'javítóintézeti ápoló', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1560, NULL, 108, NULL),
            (6380, N'javítóintézeti rendész', N'javítóintézeti rendész', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1570, NULL, 109, NULL),
            (6381, N'javítóintézeti utógondozó', N'javítóintézeti utógondozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1580, NULL, 110, NULL),
            (6382, N'NOKS jelmez- és viselettáros (*)', N'NOKS jelmez- és viselettáros (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1585, NULL, 111, NULL),
            (6383, N'jogász', N'jogász', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1590, NULL, 112, NULL),
            (6384, N'kazánfűtő', N'kazánfűtő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1610, NULL, 113, NULL),
            (6385, N'kézbesítő', N'kézbesítő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1620, NULL, 114, NULL),
            (6386, N'kísérő támogatást biztosító munkatárs', N'kísérő támogatást biztosító munkatárs', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1630, NULL, 115, NULL),
            (6387, N'kisgyermeknevelő', N'kisgyermeknevelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1640, NULL, 116, NULL),
            (6388, N'NOKS kollégiumi titkár (*)', N'NOKS kollégiumi titkár (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1545, NULL, 117, NULL),
            (6389, N'konyhai kisegítő', N'konyhai kisegítő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1650, NULL, 118, NULL),
            (6390, N'könyvelő', N'könyvelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1660, NULL, 119, NULL),
            (6392, N'közismereti tantárgyat nemzetiségi nyelven oktató tanár (*)', N'közismereti tantárgyat nemzetiségi nyelven oktató tanár (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 150, NULL, 120, NULL),
            (6393, N'közismereti tantárgyat oktató középiskolai tanár (szakgimnázium) (*)', N'közismereti tantárgyat oktató középiskolai tanár (Szakgimnázium) (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 160, NULL, 121, NULL),
            (6394, N'közismereti tantárgyat oktató tanár (szakközépiskola) (*)', N'közismereti tantárgyat oktató tanár (Szakközépiskola) (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 170, NULL, 122, NULL),
            (6395, N'közösségi gondozó', N'közösségi gondozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1670, NULL, 123, NULL),
            (6396, N'közösségi koordinátor', N'közösségi koordinátor', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1680, NULL, 124, NULL),
            (6397, N'közvetítő', N'közvetítő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1690, NULL, 125, NULL),
            (6398, N'leltárellenőr', N'leltárellenőr', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1700, NULL, 126, NULL),
            (6399, N'leltározó', N'leltározó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1710, NULL, 127, NULL),
            (6400, N'mentálhigiénés munkatárs', N'mentálhigiénés munkatárs', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1720, NULL, 128, NULL),
            (6401, N'módszertani munkatárs', N'módszertani munkatárs', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1730, NULL, 129, NULL),
            (6402, N'mozgásterapeuta', N'mozgásterapeuta', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1740, NULL, 130, NULL),
            (6403, N'munkaoktató', N'munkaoktató', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1750, NULL, 131, NULL),
            (6404, N'EGYÉB munkaügyi, személyzeti előadó (*)', N'EGYÉB munkaügyi, személyzeti előadó (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 840, NULL, 132, NULL),
            (6405, N'munkavezető', N'munkavezető', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1770, NULL, 133, NULL),
            (6406, N'nemzetiségi nyelvtanár (*)', N'nemzetiségi nyelvtanár (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1775, NULL, 134, NULL),
            (6407, N'nemzetiségi óvodapedagógus (*)', N'nemzetiségi óvodapedagógus (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1780, NULL, 135, NULL),
            (6408, N'nemzetiségi tanító (*)', N'nemzetiségi tanító (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1790, NULL, 136, NULL),
            (6409, N'nevelőszülői hálózat szakmai vezetője', N'nevelőszülői hálózat szakmai vezetője', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1810, NULL, 137, NULL),
            (6410, N'nevelőszülői tanácsadó', N'nevelőszülői tanácsadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1820, NULL, 138, NULL),
            (6411, N'nyilvántartó pénztáros', N'nyilvántartó pénztáros', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1840, NULL, 139, NULL),
            (6412, N'orvos konzultáns', N'orvos konzultáns', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1860, NULL, 140, NULL),
            (6413, N'otthonvezető javítóintézetben', N'otthonvezető javítóintézetben', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1870, NULL, 141, NULL),
            (6414, N'iskolapszichológus (*)', N'Iskolapszichológus (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 110, NULL, 142, NULL),
            (6415, N'őr/telepőr', N'őr/telepőr', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1880, NULL, 143, NULL),
            (6416, N'örökbefogadási tanácsadó', N'örökbefogadási tanácsadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1890, NULL, 144, NULL),
            (6418, N'pénzügyi ügyintéző', N'pénzügyi ügyintéző', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1910, NULL, 145, NULL),
            (6419, N'pszichiáter', N'pszichiáter', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1930, NULL, 146, NULL),
            (6420, N'pszichológiai tanácsadó', N'pszichológiai tanácsadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1940, NULL, 147, NULL),
            (6421, N'NOKS pszichopedagógus (*)', N'NOKS pszichopedagógus (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 660, NULL, 148, NULL),
            (6422, N'raktárkezelő', N'raktárkezelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1950, NULL, 149, NULL),
            (6423, N'raktáros', N'raktáros', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1960, NULL, 150, NULL),
            (6424, N'rendész', N'rendész', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1970, NULL, 151, NULL),
            (6425, N'segédápoló', N'segédápoló', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1980, NULL, 152, NULL),
            (6426, N'segédgondozó', N'segédgondozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1990, NULL, 153, NULL),
            (6427, N'segédmunkás', N'segédmunkás', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2000, NULL, 154, NULL),
            (6428, N'statisztikai ügyintéző', N'statisztikai ügyintéző', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2010, NULL, 155, NULL),
            (6429, N'szakápoló', N'szakápoló', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2020, NULL, 156, NULL),
            (6430, N'szakasszisztens', N'szakasszisztens', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2030, NULL, 157, NULL),
            (6431, N'szakgondozó', N'szakgondozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2040, NULL, 158, NULL),
            (6432, N'szakmai elméleti tantárgyat oktató tanár (*)', N'szakmai elméleti tantárgyat oktató tanár (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 230, NULL, 159, NULL),
            (6433, N'szakmai tanár, szakoktató, gyakorlati oktató', N'szakmai tanár, szakoktató, gyakorlati oktató', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 405, NULL, 160, NULL),
            (6434, N'szakmai tantárgyat tanító tanár (*)', N'szakmai tantárgyat tanító tanár (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2045, NULL, 161, NULL),
            (6435, N'szakmunkás', N'szakmunkás', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2060, NULL, 162, NULL),
            (6436, N'szakoktató (*)', N'szakoktató (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2065, NULL, 163, NULL),
            (6437, N'szakorvos kizárólag sajátos nevelési igényű gyermekeket nevelő óvodában', N'szakorvos kizárólag sajátos nevelési igényű gyermekeket nevelő óvodában', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2070, NULL, 164, NULL),
            (6438, N'NOKS szakszolgálati titkár (*)', N'NOKS szakszolgálati titkár (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2075, NULL, 165, NULL),
            (6439, N'szaktanácsadó', N'szaktanácsadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2080, NULL, 166, NULL),
            (6440, N'számviteli ügyintéző', N'számviteli ügyintéző', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2090, NULL, 167, NULL),
            (6441, N'szociális gondozó', N'szociális gondozó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2110, NULL, 168, NULL),
            (6442, N'szociális munkatárs/szakértő', N'szociális munkatárs/szakértő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2130, NULL, 169, NULL),
            (6443, N'szociális segítő', N'szociális segítő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2140, NULL, 170, NULL),
            (6445, N'szociális ügyintéző', N'szociális ügyintéző', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2150, NULL, 171, NULL),
            (6446, N'tanácsadó', N'tanácsadó', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2170, NULL, 172, NULL),
            (6447, N'tanácsadó pedagógus (*)', N'tanácsadó pedagógus (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2175, NULL, 173, NULL),
            (6448, N'tanár', N'tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2180, NULL, 174, NULL),
            (6449, N'tanár a szakmai elméleti oktatásban', N'tanár a szakmai elméleti oktatásban', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2190, NULL, 175, NULL),
            (6450, N'tanító, tanár', N'tanító, tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2200, NULL, 176, NULL),
            (6451, N'tanyagondnok', N'tanyagondnok', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2210, NULL, 177, NULL),
            (6452, N'technikus', N'technikus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2220, NULL, 178, NULL),
            (6453, N'tehetséggondozó koordinátor (*)', N'tehetséggondozó koordinátor (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2225, NULL, 179, NULL)
                ) x (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
                WHERE NOT EXISTS (SELECT 1 FROM #DICTIONARYITEMBASE d WHERE x.ID = d.ID)
                
    INSERT INTO #DICTIONARYITEMBASE (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
        SELECT * FROM ( VALUES
            (6454, N'tejkonyhavezető', N'tejkonyhavezető', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2230, NULL, 180, NULL),
            (6455, N'telefon-, telex- és telefaxkezelő', N'telefon-, telex- és telefaxkezelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2240, NULL, 181, NULL),
            (6456, N'terápiás munkatárs', N'terápiás munkatárs', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2250, NULL, 182, NULL),
            (6457, N'testnevelő (*)', N'testnevelő (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 415, NULL, 183, NULL),
            (6458, N'utcai szociális munkás', N'utcai szociális munkás', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2270, NULL, 184, NULL),
            (6459, N'utcai szolgálat koordinátor', N'utcai szolgálat koordinátor', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2280, NULL, 185, NULL),
            (6460, N'fejlesztőpedagógus', N'Fejlesztőpedagógus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1290, NULL, 186, NULL),
            (6461, N'kollégiumi nevelőtanár (*)', N'kollégiumi nevelőtanár (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 120, NULL, 187, NULL),
            (6709, N'külső gyakorlati képzőhely (*)', N'külső gyakorlati képzőhely (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 180, NULL, 188, NULL),
            (6710, N'hittanoktató (*)', N'Hittanoktató (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1515, NULL, 189, NULL),
            (6725, N'tanító (*)', N'tanító (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2195, NULL, 190, NULL),
            (6914, N'általános iskolai tanár (*)', N'általános iskolai tanár (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 10, NULL, 191, NULL),
            (6915, N'óvodapszichológus (*)', N'óvodapszichológus (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1875, NULL, 192, NULL),
            (6968, N'szakmai tanár (*)', N'szakmai tanár (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 240, NULL, 193, NULL),
            (6969, N'gyakorlati oktató (*)', N'gyakorlati oktató (*)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 40, NULL, 194, NULL),
            (7356, N'NOKS pedagógiai felügyelő', N'NOKS pedagógiai felügyelő', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 650, NULL, 195, NULL),
            (7772, N'könyvtárostanár', N'könyvtárostanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 142, NULL, 196, NULL),
            (7773, N'könyvtárostanító', N'könyvtárostanító', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 144, NULL, 197, NULL),
            (7774, N'utazó gyógypedagógus', N'utazó gyógypedagógus', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 310, NULL, 198, NULL),
            (7775, N'utazó konduktor', N'utazó konduktor', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 320, NULL, 199, NULL),
            (7783, N'lelkész', N'lelkész', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 1695, NULL, 200, NULL),
            (8349, N'idegennyelv-tantágykategóriát oktató oktató (*)', N'idegennyelv-tantágykategóriát oktató oktató', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 92, NULL, 201, NULL),
            (8350, N'könyvtáros oktató (kizárólag könyvtárosi feladatokat lát el) (*)', N'könyvtáros oktató (kizárólag könyvtárosi feladatokat lát el)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 136, NULL, 202, NULL),
            (8351, N'könyvtáros oktató (oktatói feladatokat is ellát) (*)', N'könyvtáros oktató (oktatói feladatokat is ellát)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 138, NULL, 203, NULL),
            (8352, N'közismereti tantárgyat oktató középiskolai oktató (*)', N'közismereti tantárgyat oktató középiskolai oktató', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 158, NULL, 204, NULL),
            (8353, N'közismereti tantárgyat oktató oktató (*)', N'közismereti tantárgyat oktató oktató', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 168, NULL, 205, NULL),
            (8354, N'szakmai oktató (felsőfokú végzettséggel) (*)', N'szakmai oktató (felsőfokú végzettséggel)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 232, NULL, 206, NULL),
            (8355, N'szakmai oktató (középfokú végzettséggel) (*)', N'szakmai oktató (középfokú végzettséggel)', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'F', 234, NULL, 207, NULL),
            (8372, N'Iskolaőr', N'Iskolaőr', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2300, NULL, 208, NULL),
            (8736, N'közismereti tantárgyat oktató középiskola', N'közismereti tantárgyat oktató középiskola', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2301, NULL, 209, NULL),
            (8737, N'közismereti tantárgyat oktató szakgimnáziumi tanár', N'közismereti tantárgyat oktató szakgimnáziumi tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2302, NULL, 210, NULL),
            (8738, N'közismereti tantárgyat oktató tanár', N'közismereti tantárgyat oktató tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2303, NULL, 211, NULL),
            (8739, N'művészeti szakmai tantárgyat oktató tanár', N'művészeti szakmai tantárgyat oktató tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2304, NULL, 212, NULL),
            (8740, N'szakmai tantárgyat oktató tanár', N'szakmai tantárgyat oktató tanár', NULL, NULL, NULL, N'MunkakorTipus', 12, N'T', N'T', 2305, NULL, 213, NULL),
            (493, N'Na', N'Na', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 1, NULL, 1, NULL),
            (509, N'magyar', N'Hungarian', N'magyar', NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 2, NULL, 2, NULL),
            (494, N'afgán', N'Afghan', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 10, NULL, 3, NULL),
            (495, N'albán', N'Albanian', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 20, NULL, 4, NULL),
            (534, N'angol', N'Angol', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 30, NULL, 5, NULL),
            (530, N'arab', N'Arab', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 40, NULL, 6, NULL),
            (496, N'azeri', N'Azeri', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 50, NULL, 7, NULL),
            (498, N'bolgár', N'Bulgarian', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 60, NULL, 8, NULL),
            (501, N'cseh', N'Czech', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 70, NULL, 9, NULL),
            (502, N'dán', N'Danish', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 80, NULL, 10, NULL),
            (542, N'Egyéb', N'Egyéb', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 90, NULL, 11, NULL),
            (537, N'eszperanto', N'Eszperanto', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 100, NULL, 12, NULL),
            (503, N'észt', N'Esthonian', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 110, NULL, 13, NULL),
            (504, N'finn', N'Finn', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 120, NULL, 14, NULL),
            (505, N'francia', N'French', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 130, NULL, 15, NULL),
            (508, N'görög', N'Greek', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 140, NULL, 16, NULL),
            (506, N'grúz', N'Georgian', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 150, NULL, 17, NULL),
            (512, N'héber', N'Héber', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 160, NULL, 18, NULL),
            (510, N'hindi', N'Hindi', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 170, NULL, 19, NULL),
            (519, N'holland', N'Dutch', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 180, NULL, 20, NULL),
            (500, N'horvát', N'Croatian', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 190, NULL, 21, NULL),
            (3031, N'Ismeretlen', N'Ismeretlen', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 1000, NULL, 22, NULL),
            (514, N'japán', N'Japanese', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 210, NULL, 23, NULL),
            (539, N'kazah', N'Kazakh', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 220, NULL, 24, NULL),
            (541, N'kikuju (kenya)', N'Kikuju (kenya)', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 230, NULL, 25, NULL),
            (499, N'kínai', N'Chinese', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 240, NULL, 26, NULL),
            (515, N'koreai', N'Korean', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 250, NULL, 27, NULL),
            (538, N'latin', N'Latin', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 260, NULL, 28, NULL),
            (521, N'lengyel', N'Polish', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 270, NULL, 29, NULL),
            (516, N'lett', N'Latvian', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 280, NULL, 30, NULL),
            (517, N'litván', N'Lithuanian', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 290, NULL, 31, NULL),
            (533, N'macedón', N'Macedonian', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 300, NULL, 32, NULL),
            (518, N'mongol', N'Mongolian', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 310, NULL, 33, NULL),
            (507, N'német', N'German', N'német', NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 320, NULL, 34, NULL),
            (520, N'norvég', N'Norwegian', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 330, NULL, 35, NULL),
            (513, N'olasz', N'Italian', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 340, NULL, 36, NULL),
            (497, N'örmény', N'Armenian', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 350, NULL, 37, NULL),
            (524, N'orosz', N'Russian', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 360, NULL, 38, NULL),
            (511, N'perzsa', N'Perzsa', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 370, NULL, 39, NULL),
            (522, N'portugál', N'Portugese', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 380, NULL, 40, NULL),
            (523, N'román', N'Romanian', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 390, NULL, 41, NULL),
            (527, N'spanyol', N'Spanish', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 400, NULL, 42, NULL),
            (528, N'svéd', N'Swedish', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 410, NULL, 43, NULL),
            (536, N'szerb', N'Serb', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 420, NULL, 44, NULL),
            (525, N'szlovák', N'Slovak', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 430, NULL, 45, NULL),
            (526, N'szlovén', N'Slovene', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 440, NULL, 46, NULL),
            (529, N'thai', N'Thai', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 450, NULL, 47, NULL),
            (531, N'török', N'Turkish', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 460, NULL, 48, NULL),
            (532, N'ukrán', N'Ukrán', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 470, NULL, 49, NULL),
            (535, N'üzbég', N'Üzbég', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 480, NULL, 50, NULL),
            (540, N'vietnami', N'Vietnami', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 490, NULL, 51, NULL),
            (7782, N'indonéz', N'indonéz', NULL, NULL, NULL, N'Anyanyelv', 10, N'T', N'T', 200, NULL, 52, NULL),
            (543, N'Na', N'Na', NULL, NULL, NULL, N'IgazolvanyTipus', 11, N'F', N'T', 1, NULL, 1, NULL),
            (544, N'Személyi igazolvány', N'Személyi igazolvány', NULL, NULL, NULL, N'IgazolvanyTipus', 11, N'F', N'T', 2, NULL, 2, NULL),
            (545, N'Ideiglenes személyi igazolvány', N'Ideiglenes személyi igazolvány', NULL, NULL, NULL, N'IgazolvanyTipus', 11, N'F', N'T', 3, NULL, 3, NULL),
            (546, N'Útlevél', N'Útlevél', NULL, NULL, NULL, N'IgazolvanyTipus', 11, N'F', N'T', 4, NULL, 4, NULL),
            (547, N'Tartózkodási engedély', N'Tartózkodási engedély', NULL, NULL, NULL, N'IgazolvanyTipus', 11, N'F', N'T', 5, NULL, 5, NULL),
            (6496, N'Vezetői engedély', N'Vezetői engedély', NULL, NULL, NULL, N'IgazolvanyTipus', 11, N'F', N'T', 6, NULL, 6, NULL),
            (6497, N'Lakcímet igazoló hatósági igazolvány', N'Lakcímet igazoló hatósági igazolvány', NULL, NULL, NULL, N'IgazolvanyTipus', 11, N'F', N'T', 7, NULL, 7, NULL),
            (643, N'Na', N'Na', NULL, NULL, NULL, N'MunkaviszonyTipus', 13, N'T', N'T', 1, NULL, 1, NULL),
            (644, N'közalkalmazotti jogviszony (határozatlan időre)', N'közalkalmazotti jogviszony (határozatlan időre)', NULL, NULL, NULL, N'MunkaviszonyTipus', 13, N'T', N'T', 2, NULL, 2, NULL),
            (645, N'közalkalmazotti jogviszony (határozott időre)', N'közalkalmazotti jogviszony (határozott időre)', NULL, NULL, NULL, N'MunkaviszonyTipus', 13, N'T', N'T', 3, NULL, 3, NULL),
            (651, N'munkaviszony (határozott időre)', N'munkaviszony (határozott időre)', NULL, NULL, NULL, N'MunkaviszonyTipus', 13, N'T', N'T', 5, NULL, 4, NULL),
            (652, N'munkaviszony (határozatlan időre)', N'munkaviszony (határozatlan időre)', NULL, NULL, NULL, N'MunkaviszonyTipus', 13, N'T', N'T', 4, NULL, 5, NULL),
            (653, N'polgári jogviszony - vállalkozási jogviszony', N'polgári jogviszony - vállalkozási jogviszony', NULL, NULL, NULL, N'MunkaviszonyTipus', 13, N'T', N'F', 7, NULL, 6, NULL),
            (654, N'polgári jogviszony - megbízási jogviszony', N'polgári jogviszony - megbízási jogviszony', NULL, NULL, NULL, N'MunkaviszonyTipus', 13, N'T', N'T', 6, NULL, 7, NULL),
            (6762, N'egyházi szolgálati jogviszony', N'egyházi szolgálati jogviszony', NULL, NULL, NULL, N'MunkaviszonyTipus', 13, N'T', N'T', 8, NULL, 8, NULL),
            (7706, N'szolgálati jogviszony', N'szolgálati jogviszony', NULL, NULL, NULL, N'MunkaviszonyTipus', 13, N'T', N'F', 9, NULL, 9, NULL),
            (8455, N'honvédelmi alkalmazotti jogviszony', N'honvédelmi alkalmazotti jogviszony', NULL, NULL, NULL, N'MunkaviszonyTipus', 13, N'T', N'F', 10, NULL, 10, NULL),
            (70, N'Na', N'Na', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 1, NULL, 1, NULL),
            (77, N'3D metszet', N'3D metszet', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 2, NULL, 2, NULL),
            (82, N'Applikációs készlet', N'Applikációs készlet', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 3, NULL, 3, NULL),
            (71, N'Applikációs táblai készlet', N'Applikációs táblai készlet', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 4, NULL, 4, NULL),
            (92, N'CD lemez (audió)', N'CD lemez (audió)', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 5, NULL, 5, NULL),
            (105, N'CD-ROM, Multimédia oktató program', N'CD-ROM, Multimédia oktató program', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 6, NULL, 6, NULL),
            (93, N'Diasorozat, diafilm', N'Diasorozat, diafilm', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 7, NULL, 7, NULL),
            (83, N'Didaktikus játék', N'Didaktikus játék', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 8, NULL, 8, NULL),
            (94, N'Dinamikus optikai ábra', N'Dinamikus optikai ábra', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 9, NULL, 9, NULL),
            (106, N'DVD film', N'DVD film', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 10, NULL, 10, NULL),
            (116, N'Egyéb számítástechnikai eszköz', N'Egyéb számítástechnikai eszköz', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 11, NULL, 11, NULL),
            (95, N'Falikép, poszter, tabló', N'Falikép, poszter, tabló', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 12, NULL, 12, NULL),
            (96, N'Falitérkép', N'Falitérkép', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 13, NULL, 13, NULL),
            (74, N'Földgömb, éggömb', N'Földgömb, éggömb', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 14, NULL, 14, NULL),
            (107, N'Fotó CD', N'Fotó CD', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 15, NULL, 15, NULL),
            (108, N'Hálózati multimédia oktató program, adatbázis', N'Hálózati multimédia oktató program, adatbázis', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 16, NULL, 16, NULL),
            (91, N'Hangkazetta', N'Hangkazetta', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 17, NULL, 17, NULL),
            (90, N'Hanglemez', N'Hanglemez', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 18, NULL, 18, NULL),
            (101, N'Hangosított diasorozat', N'Hangosított diasorozat', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 19, NULL, 19, NULL),
            (112, N'Hangszer', N'Hangszer', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 20, NULL, 20, NULL),
            (97, N'Írásvetítő transzparens sorozat', N'Írásvetítő transzparens sorozat', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 21, NULL, 21, NULL),
            (104, N'Képlemez', N'Képlemez', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 22, NULL, 22, NULL),
            (75, N'Kísérleti eszköz', N'Kísérleti eszköz', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 23, NULL, 23, NULL),
            (72, N'Kőzet gyűjtemény', N'Kőzet gyűjtemény', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 24, NULL, 24, NULL),
            (85, N'Laboratóriumi készlet', N'Laboratóriumi készlet', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 25, NULL, 25, NULL),
            (115, N'Lapolvasó (szkenner)', N'Lapolvasó (szkenner)', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 26, NULL, 26, NULL),
            (86, N'Logikai készlet', N'Logikai készlet', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 27, NULL, 27, NULL),
            (76, N'Makett', N'Makett', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 28, NULL, 28, NULL),
            (87, N'Manipulációs eszköz', N'Manipulációs eszköz', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 29, NULL, 29, NULL),
            (79, N'Mérőeszköz, műszer', N'Mérőeszköz, műszer', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 30, NULL, 30, NULL),
            (99, N'Mikroszkopikus metszet', N'Mikroszkopikus metszet', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 31, NULL, 31, NULL),
            (117, N'Monitor', N'Monitor', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 32, NULL, 32, NULL),
            (80, N'Növény-, állat preparátum', N'Növény-, állat preparátum', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 33, NULL, 33, NULL),
            (73, N'Növény gyűjtemény', N'Növény gyűjtemény', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 34, NULL, 34, NULL),
            (114, N'Nyomtató', N'Nyomtató', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 35, NULL, 35, NULL),
            (98, N'Nyomtatott/foto képsorozat', N'Nyomtatott/foto képsorozat', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 36, NULL, 36, NULL),
            (111, N'Oktatócsomag, programcsomag', N'Oktatócsomag, programcsomag', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 37, NULL, 37, NULL),
            (102, N'Oktatófilm', N'Oktatófilm', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 38, NULL, 38, NULL),
            (6493, N'Projektor', N'Projektor', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 39, NULL, 39, NULL),
            (100, N'Síkmodell írásvetítőhöz', N'Síkmodell írásvetítőhöz', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 40, NULL, 40, NULL),
            (84, N'Speciális kísérleti eszköz', N'Speciális kísérleti eszköz', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 41, NULL, 41, NULL),
            (88, N'Speciális mérőeszköz', N'Speciális mérőeszköz', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 42, NULL, 42, NULL),
            (113, N'Számítógép', N'Számítógép', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 43, NULL, 43, NULL),
            (109, N'Számítógépes oktató, gyakorló, teszt program', N'Számítógépes oktató, gyakorló, teszt program', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 44, NULL, 44, NULL),
            (110, N'Számítógépes szimulációs program', N'Számítógépes szimulációs program', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 45, NULL, 45, NULL),
            (118, N'Szoftver', N'Szoftver', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 46, NULL, 46, NULL),
            (78, N'Tanári modell', N'Tanári modell', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 47, NULL, 47, NULL),
            (89, N'Tanulói modell', N'Tanulói modell', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 48, NULL, 48, NULL),
            (81, N'Termék-minta gyűjtemény', N'Termék-minta gyűjtemény', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 49, NULL, 49, NULL),
            (103, N'Videó kazetta', N'Videó kazetta', NULL, NULL, NULL, N'EszkozTipus', 6, N'T', N'T', 50, NULL, 50, NULL),
            (119, N'Na', N'Na', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 1, NULL, 1, NULL),
            (120, N'Számítógép - Hálózati szerver 3 évesnél újabb', N'Számítógép - Hálózati szerver 3 évesnél újabb', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 2, NULL, 2, NULL),
            (121, N'Számítógép - Hálózati szerver 3 évesnél régebbi', N'Számítógép - Hálózati szerver 3 évesnél régebbi', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 3, NULL, 3, NULL),
            (122, N'Számítógép - Személyi számítógép PC I. kategória', N'Számítógép - Személyi számítógép PC I. kategória', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 4, NULL, 4, NULL),
            (123, N'Számítógép - Személyi számítógép PC II. kategória', N'Számítógép - Személyi számítógép PC II. kategória', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 5, NULL, 5, NULL),
            (124, N'Számítógép - Személyi számítógép PC III. kategória', N'Számítógép - Személyi számítógép PC III. kategória', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 6, NULL, 6, NULL),
            (125, N'Számítógép - Személyi számítógép PC IV. kategória', N'Számítógép - Személyi számítógép PC IV. kategória', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 7, NULL, 7, NULL),
            (126, N'Monitor - CRT max 15 méret', N'Monitor - CRT max 15 méret', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 8, NULL, 8, NULL),
            (127, N'Monitor - CRT 15-17 méret', N'Monitor - CRT 15-17 méret', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 9, NULL, 9, NULL),
            (128, N'Monitor - CRT min 17 méret', N'Monitor - CRT min 17 méret', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 10, NULL, 10, NULL),
            (129, N'Monitor - LCD max 15 méret', N'Monitor - LCD max 15 méret', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 11, NULL, 11, NULL),
            (130, N'Monitor - LCD 15-17 méret', N'Monitor - LCD 15-17 méret', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 12, NULL, 12, NULL),
            (131, N'Monitor - LCD min 17 méret', N'Monitor - LCD min 17 méret', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 13, NULL, 13, NULL),
            (132, N'Nyomtató - Mátrixnyomtató', N'Nyomtató - Mátrixnyomtató', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 14, NULL, 14, NULL),
            (133, N'Nyomtató - Tintasugaras', N'Nyomtató - Tintasugaras', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 15, NULL, 15, NULL),
            (134, N'Nyomtató - Lézernyomtató normál', N'Nyomtató - Lézernyomtató normál', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 16, NULL, 16, NULL),
            (135, N'Nyomtató - Lézernyomtató nagysebességű', N'Nyomtató - Lézernyomtató nagysebességű', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 17, NULL, 17, NULL),
            (179, N'Nyomtató - Multifunkciós', N'Nyomtató - Multifunkciós', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 18, NULL, 18, NULL),
            (170, N'PROJEKTOR (mobil)', N'PROJEKTOR (mobil)', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 19, NULL, 19, NULL),
            (6492, N'PROJEKTOR (fixen telepített)', N'PROJEKTOR (fixen telepített)', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 20, NULL, 20, NULL),
            (176, N'Aktív tábla (fixen telepített)', N'Aktív tábla (fixen telepített)', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 21, NULL, 21, NULL),
            (177, N'Aktív tábla (mobil)', N'Aktív tábla (mobil)', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 22, NULL, 22, NULL),
            (136, N'Szoftver - Szerver operációs rendszer:MS WINDOWS', N'Szoftver - Szerver operációs rendszer:MS WINDOWS', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 23, NULL, 23, NULL),
            (137, N'Szoftver - Szerver operációs rendszer:LINUX', N'Szoftver - Szerver operációs rendszer:LINUX', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 24, NULL, 24, NULL),
            (138, N'Szoftver - Szerver operációs rendszer:NOVELL NETWARE', N'Szoftver - Szerver operációs rendszer:NOVELL NETWARE', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 25, NULL, 25, NULL),
            (139, N'Szoftver - Szerver operációs rendszer:UNIX', N'Szoftver - Szerver operációs rendszer:UNIX', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 26, NULL, 26, NULL),
            (140, N'Szoftver - Szerver operációs rendszer:egyéb', N'Szoftver - Szerver operációs rendszer:egyéb', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 27, NULL, 27, NULL),
            (141, N'Szoftver - PC operációs rendszer:MS WINDOWS', N'Szoftver - PC operációs rendszer:MS WINDOWS', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 28, NULL, 28, NULL),
            (142, N'Szoftver - PC operációs rendszer:LINUX', N'Szoftver - PC operációs rendszer:LINUX', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 29, NULL, 29, NULL),
            (143, N'Szoftver - PC operációs rendszer:MS DOS', N'Szoftver - PC operációs rendszer:MS DOS', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 30, NULL, 30, NULL),
            (144, N'Szoftver - PC operációs rendszer:MAC OS', N'Szoftver - PC operációs rendszer:MAC OS', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 31, NULL, 31, NULL),
            (145, N'Szoftver - PC operációs rendszer:egyéb', N'Szoftver - PC operációs rendszer:egyéb', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 32, NULL, 32, NULL),
            (146, N'Szoftver - Irodai programcsomag', N'Szoftver - Irodai programcsomag', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 33, NULL, 33, NULL),
            (147, N'Szoftver - Grafikai programcsomag', N'Szoftver - Grafikai programcsomag', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 34, NULL, 34, NULL),
            (148, N'Szoftver - Rendszerprogramok, segédprogramok', N'Szoftver - Rendszerprogramok, segédprogramok', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 35, NULL, 35, NULL),
            (149, N'Szoftver - Fejlesztőeszközök', N'Szoftver - Fejlesztőeszközök', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 36, NULL, 36, NULL),
            (150, N'Szoftver - Oktatóprogramok, bemutatóprogramok', N'Szoftver - Oktatóprogramok, bemutatóprogramok', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 37, NULL, 37, NULL),
            (151, N'Szoftver - Játékprogramok', N'Szoftver - Játékprogramok', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 38, NULL, 38, NULL),
            (152, N'Szoftver - egyéb programok', N'Szoftver - egyéb programok', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 39, NULL, 39, NULL),
            (153, N'Egyéb technikai eszköz - Billentyűzet', N'Egyéb technikai eszköz - Billentyűzet', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 40, NULL, 40, NULL),
            (154, N'Egyéb technikai eszköz - Egér(mechanikus)', N'Egyéb technikai eszköz - Egér(mechanikus)', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 41, NULL, 41, NULL),
            (155, N'Egyéb technikai eszköz - Egér(optikai)', N'Egyéb technikai eszköz - Egér(optikai)', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 42, NULL, 42, NULL),
            (156, N'Egyéb technikai eszköz - CD-R', N'Egyéb technikai eszköz - CD-R', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 43, NULL, 43, NULL),
            (157, N'Egyéb technikai eszköz - CD-RW 700', N'Egyéb technikai eszköz - CD-RW 700', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 44, NULL, 44, NULL),
            (158, N'Egyéb technikai eszköz - DVD-R', N'Egyéb technikai eszköz - DVD-R', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 45, NULL, 45, NULL),
            (159, N'Egyéb technikai eszköz - DVD-RW', N'Egyéb technikai eszköz - DVD-RW', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 46, NULL, 46, NULL),
            (160, N'Egyéb technikai eszköz - Digitális fényképezőgép', N'Egyéb technikai eszköz - Digitális fényképezőgép', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 47, NULL, 47, NULL),
            (161, N'Egyéb technikai eszköz - Digitális kamera', N'Egyéb technikai eszköz - Digitális kamera', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 48, NULL, 48, NULL),
            (162, N'Egyéb technikai eszköz - PEN drive', N'Egyéb technikai eszköz - PEN drive', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 49, NULL, 49, NULL),
            (163, N'Egyéb technikai eszköz - MODEM (külső)', N'Egyéb technikai eszköz - MODEM (külső)', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 50, NULL, 50, NULL),
            (164, N'Egyéb technikai eszköz - Mikrofon', N'Egyéb technikai eszköz - Mikrofon', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 51, NULL, 51, NULL),
            (165, N'Egyéb technikai eszköz - Hangszóró', N'Egyéb technikai eszköz - Hangszóró', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 52, NULL, 52, NULL),
            (166, N'Egyéb technikai eszköz - Lapolvasó', N'Egyéb technikai eszköz - Lapolvasó', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 53, NULL, 53, NULL),
            (167, N'Egyéb technikai eszköz - Szünetmentes táp', N'Egyéb technikai eszköz - Szünetmentes táp', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 54, NULL, 54, NULL),
            (168, N'Egyéb technikai eszköz - WEB kamera', N'Egyéb technikai eszköz - WEB kamera', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 55, NULL, 55, NULL),
            (169, N'Egyéb technikai eszköz - Videómagnó', N'Egyéb technikai eszköz - Videómagnó', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 56, NULL, 56, NULL),
            (171, N'Egyéb technikai eszköz - Fénymásoló', N'Egyéb technikai eszköz - Fénymásoló', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 57, NULL, 57, NULL),
            (172, N'Egyéb technikai eszköz - CNC gép', N'Egyéb technikai eszköz - CNC gép', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 58, NULL, 58, NULL),
            (173, N'Vizuál tábla', N'Vizuál tábla', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 59, NULL, 59, NULL),
            (174, N'Egyéb technikai eszköz - egyéb számítástechnikai eszköz', N'Egyéb technikai eszköz - egyéb számítástechnikai eszköz', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 60, NULL, 60, NULL),
            (175, N'Egyéb technikai eszköz - egyéb technikai eszköz', N'Egyéb technikai eszköz - egyéb technikai eszköz', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 61, NULL, 61, NULL),
            (178, N'Egyéb technikai eszköz - Videókamera', N'Egyéb technikai eszköz - Videókamera', NULL, NULL, NULL, N'EszkozKategoria', 7, N'F', N'T', 62, NULL, 62, NULL),
            (10, N'Iskolai könyvtár', N'Iskolai könyvtár', NULL, NULL, NULL, N'IngyenesTankonyvEllatas', 2, N'F', N'T', 1, NULL, 1, NULL),
            (11, N'Saját tulajdonba adás', N'Saját tulajdonba adás', NULL, NULL, NULL, N'IngyenesTankonyvEllatas', 2, N'F', N'T', 2, NULL, 2, NULL),
            (12, N'Vegyes(saját tulajdonba adás és iskolai könyvtár is)', N'Vegyes(saját tulajdonba adás és iskolai könyvtár is)', NULL, NULL, NULL, N'IngyenesTankonyvEllatas', 2, N'F', N'T', 3, NULL, 3, NULL),
            (180, N'Na', N'Na', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 1, NULL, 1, NULL),
            (181, N'Osztályterem', N'Osztályterem', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 2, NULL, 2, NULL),
            (182, N'Óvodai csoportszoba', N'Óvodai csoportszoba', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 3, NULL, 3, NULL),
            (183, N'Könyvtár', N'Könyvtár', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 4, NULL, 4, NULL),
            (184, N'Szakterem', N'Szakterem', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 5, NULL, 5, NULL),
            (185, N'Tanulócsoportos foglalkozásra kialakított kisterem', N'Tanulócsoportos foglalkozásra kialakított kisterem', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 6, NULL, 6, NULL),
            (186, N'Egyéb, az oktatás célját szolgáló helyiség', N'Egyéb, az oktatás célját szolgáló helyiség', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 7, NULL, 7, NULL),
            (187, N'Napközis szoba/tanulószoba', N'Napközis szoba/tanulószoba', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 8, NULL, 8, NULL),
            (188, N'Kollégiumi hálószoba', N'Kollégiumi hálószoba', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 9, NULL, 9, NULL),
            (189, N'Tanári munkaszoba', N'Tanári munkaszoba', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 10, NULL, 10, NULL),
            (190, N'Tornaszoba', N'Tornaszoba', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 11, NULL, 11, NULL),
            (191, N'Tornaterem', N'Tornaterem', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 12, NULL, 12, NULL),
            (192, N'Sportpálya', N'Sportpálya', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 13, NULL, 13, NULL),
            (193, N'Saját úszómedence', N'Saját úszómedence', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 14, NULL, 14, NULL),
            (194, N'Szakmai gyakorlóterem', N'Szakmai gyakorlóterem', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 15, NULL, 15, NULL),
            (195, N'Szakmai laboratórium', N'Szakmai laboratórium', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 16, NULL, 16, NULL),
            (196, N'Demonstrációs terem', N'Demonstrációs terem', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 17, NULL, 17, NULL),
            (197, N'Iskolai tanműhely (egyéb)', N'Iskolai tanműhely (egyéb)', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 18, NULL, 18, NULL),
            (198, N'Tangazdaság', N'Tangazdaság', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 19, NULL, 19, NULL),
            (199, N'Taniroda', N'Taniroda', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 20, NULL, 20, NULL),
            (200, N'Tanudvar', N'Tanudvar', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 21, NULL, 21, NULL),
            (201, N'Tanbolt', N'Tanbolt', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 22, NULL, 22, NULL),
            (202, N'Tankonyha', N'Tankonyha', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 23, NULL, 23, NULL),
            (203, N'Iskolai tanműhelyi képzési helyek', N'Iskolai tanműhelyi képzési helyek', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 24, NULL, 24, NULL),
            (204, N'Iskolán kívüli tanműhely', N'Iskolán kívüli tanműhely', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 25, NULL, 25, NULL),
            (205, N'Iskolán kívüli tanműhelyi képzési helye', N'Iskolán kívüli tanműhelyi képzési helye', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 26, NULL, 26, NULL),
            (206, N'Egyéb iskolai gyakorlati képzési helyszín', N'Egyéb iskolai gyakorlati képzési helyszín', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 27, NULL, 27, NULL),
            (207, N'Kisterem', N'Kisterem', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 28, NULL, 28, NULL),
            (208, N'Szertár', N'Szertár', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 29, NULL, 29, NULL),
            (209, N'Úszómedence', N'Úszómedence', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 30, NULL, 30, NULL),
            (210, N'Iskolai tanműhely (tangazdaság)', N'Iskolai tanműhely (tangazdaság)', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 31, NULL, 31, NULL),
            (211, N'Iskolai tanműhely (taniroda)', N'Iskolai tanműhely (taniroda)', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 32, NULL, 32, NULL),
            (212, N'Iskolai tanműhely (tanudvar)', N'Iskolai tanműhely (tanudvar)', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 33, NULL, 33, NULL),
            (213, N'Iskolai tanműhely (tanbolt)', N'Iskolai tanműhely (tanbolt)', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 34, NULL, 34, NULL),
            (214, N'Iskolai tanműhely (tankonyha)', N'Iskolai tanműhely (tankonyha)', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 35, NULL, 35, NULL),
            (215, N'Nem oktatási célra szolgáló helyiség', N'Nem oktatási célra szolgáló helyiség', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 36, NULL, 36, NULL),
            (2923, N'Saját sportpálya', N'Saját sportpálya', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 37, NULL, 37, NULL),
            (2924, N'Általános szertár', N'Általános szertár', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 38, NULL, 38, NULL),
            (2925, N'Gyermekágy/fektető tároló helyiség', N'Gyermekágy/fektető tároló helyiség', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 39, NULL, 39, NULL),
            (2926, N'Logopédiai foglalkoztató, egyéni fejlesztő szoba', N'Logopédiai foglalkoztató, egyéni fejlesztő szoba', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 40, NULL, 40, NULL),
            (2927, N'Óvodapszichológusi helyiség', N'Óvodapszichológusi helyiség', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 41, NULL, 41, NULL),
            (2928, N'Játszóudvar', N'Játszóudvar', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 42, NULL, 42, NULL),
            (2929, N'Intézményvezetői iroda', N'Intézményvezetői iroda', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 43, NULL, 43, NULL),
            (2930, N'Intézményvezető-helyettesi iroda', N'Intézményvezető-helyettesi iroda', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 44, NULL, 44, NULL),
            (2931, N'Tagintézmény-, intézményegységvezető-helyettes iroda', N'Tagintézmény-, intézményegységvezető-helyettes iroda', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 45, NULL, 45, NULL),
            (2932, N'Óvodatitkári iroda', N'Óvodatitkári iroda', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 46, NULL, 46, NULL),
            (2933, N'Raktár (általános célú)', N'Raktár (általános célú)', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 47, NULL, 47, NULL),
            (2934, N'Többcélú helyiség', N'Többcélú helyiség', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 48, NULL, 48, NULL),
            (2935, N'Orvosi szoba', N'Orvosi szoba', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 49, NULL, 49, NULL),
            (2936, N'Elkülönítő szoba', N'Elkülönítő szoba', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 50, NULL, 50, NULL),
            (2937, N'Gyermeköltöző', N'Gyermeköltöző', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 51, NULL, 51, NULL),
            (2938, N'Gyermekmosdó, wc helyiség', N'Gyermekmosdó, wc helyiség', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 52, NULL, 52, NULL),
            (2939, N'Felnőtt öltöző', N'Felnőtt öltöző', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 53, NULL, 53, NULL),
            (2940, N'Felnőtt mosdó', N'Felnőtt mosdó', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 54, NULL, 54, NULL),
            (2941, N'Felnőtt wc helyiség', N'Felnőtt wc helyiség', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 55, NULL, 55, NULL),
            (2942, N'Felnőtt zuhanyzó', N'Felnőtt zuhanyzó', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 56, NULL, 56, NULL),
            (2943, N'Mosó, vasaló helyiség', N'Mosó, vasaló helyiség', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 57, NULL, 57, NULL),
            (2944, N'Szárító helyiség', N'Szárító helyiség', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 58, NULL, 58, NULL),
            (2945, N'Felnőtt étkező', N'Felnőtt étkező', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 59, NULL, 59, NULL),
            (2946, N'Főzőkonyha', N'Főzőkonyha', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 60, NULL, 60, NULL),
            (2947, N'Melegítő konyha', N'Melegítő konyha', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 61, NULL, 61, NULL),
            (2948, N'Tálaló-mosogató', N'Tálaló-mosogató', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 62, NULL, 62, NULL),
            (2949, N'Szárazáru raktár', N'Szárazáru raktár', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 63, NULL, 63, NULL),
            (2950, N'Földesárú raktár', N'Földesárú raktár', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 64, NULL, 64, NULL),
            (2951, N'Éléskamra', N'Éléskamra', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 65, NULL, 65, NULL),
            (2952, N'Karbantartó műhely', N'Karbantartó műhely', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 66, NULL, 66, NULL),
            (2953, N'Kerekesszék tároló', N'Kerekesszék tároló', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 67, NULL, 67, NULL),
            (2954, N'Ételhulladék tároló', N'Ételhulladék tároló', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 68, NULL, 68, NULL),
            (2955, N'Műhely', N'Műhely', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 69, NULL, 69, NULL),
            (2956, N'Iskolapszichológusi szoba', N'Iskolapszichológusi szoba', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 70, NULL, 70, NULL),
            (2957, N'Gyógytestnevelési/erőnléti terem', N'Gyógytestnevelési/erőnléti terem', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 71, NULL, 71, NULL),
            (2958, N'Iskolatitkári iroda', N'Iskolatitkári iroda', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 72, NULL, 72, NULL),
            (2959, N'Könyvtárszoba', N'Könyvtárszoba', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 73, NULL, 73, NULL),
            (2960, N'Könyvraktár', N'Könyvraktár', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 74, NULL, 74, NULL),
            (2961, N'Kiszolgáló helyiségek', N'Kiszolgáló helyiségek', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 75, NULL, 75, NULL),
            (2962, N'Sportszertár', N'Sportszertár', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 76, NULL, 76, NULL),
            (2963, N'Aula (előtér, közösségi tér)', N'Aula (előtér, közösségi tér)', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 77, NULL, 77, NULL),
            (2964, N'Porta, portáshelyiség', N'Porta, portáshelyiség', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 78, NULL, 78, NULL),
            (2965, N'Ebédlő', N'Ebédlő', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 79, NULL, 79, NULL),
            (2966, N'Teakonyha', N'Teakonyha', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 80, NULL, 80, NULL),
            (2967, N'Személyzeti öltöző', N'Személyzeti öltöző', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 81, NULL, 81, NULL),
            (2968, N'Személyzeti wc helyiség', N'Személyzeti wc helyiség', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 82, NULL, 82, NULL),
            (2969, N'Tanulói wc helyiség', N'Tanulói wc helyiség', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 83, NULL, 83, NULL),
            (2970, N'Technikai alkalmazotti mosdó-zuhanyzó, wc helyiség', N'Technikai alkalmazotti mosdó-zuhanyzó, wc helyiség', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 84, NULL, 84, NULL),
            (2971, N'Élelmiszerhulladék-tároló', N'Élelmiszerhulladék-tároló', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 85, NULL, 85, NULL),
            (2972, N'Testedző szoba', N'Testedző szoba', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 86, NULL, 86, NULL),
            (2973, N'Hálószoba, hálóterem', N'Hálószoba, hálóterem', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 87, NULL, 87, NULL),
            (2974, N'Stúdió', N'Stúdió', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 88, NULL, 88, NULL),
            (2975, N'Kollégiumi sportudvar', N'Kollégiumi sportudvar', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 89, NULL, 89, NULL),
            (2976, N'Kollégiumi titkári iroda', N'Kollégiumi titkári iroda', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 90, NULL, 90, NULL),
            (2977, N'Ügyeletes nevelői szoba', N'Ügyeletes nevelői szoba', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 91, NULL, 91, NULL),
            (2978, N'Éjszakai gyermekfelügyelői szoba', N'Éjszakai gyermekfelügyelői szoba', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 92, NULL, 92, NULL),
            (2979, N'Tanári pihenő', N'Tanári pihenő', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 93, NULL, 93, NULL),
            (2980, N'Nővérszoba', N'Nővérszoba', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 94, NULL, 94, NULL),
            (2981, N'Vendégszoba', N'Vendégszoba', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 95, NULL, 95, NULL),
            (2982, N'Társalgó (látogatófogadó)', N'Társalgó (látogatófogadó)', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 96, NULL, 96, NULL),
            (2983, N'Tanulói vizesblokk, fürdőszoba', N'Tanulói vizesblokk, fürdőszoba', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 97, NULL, 97, NULL),
            (2984, N'Tisztítószer, takarítóeszközök és -gépek tárolója', N'Tisztítószer, takarítóeszközök és -gépek tárolója', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 98, NULL, 98, NULL),
            (2985, N'Ágyneműraktár', N'Ágyneműraktár', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 99, NULL, 99, NULL),
            (2986, N'Mosókonyha (tanulói)', N'Mosókonyha (tanulói)', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 100, NULL, 100, NULL),
            (2987, N'Szárítóhelyiség (tanulói)', N'Szárítóhelyiség (tanulói)', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 101, NULL, 101, NULL),
            (2988, N'Vasaló- és fehérnemű-javító helyiség (tanulói)', N'Vasaló- és fehérnemű-javító helyiség (tanulói)', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 102, NULL, 102, NULL),
            (2989, N'Szeméttároló', N'Szeméttároló', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 103, NULL, 103, NULL),
            (2990, N'Tanulószoba (felkészülő szoba)', N'Tanulószoba (felkészülő szoba)', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 104, NULL, 104, NULL),
            (2991, N'Szakköri, diákköri szoba', N'Szakköri, diákköri szoba', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 105, NULL, 105, NULL),
            (2992, N'Rendezvényterem', N'Rendezvényterem', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 106, NULL, 106, NULL),
            (2993, N'Betegszoba', N'Betegszoba', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 107, NULL, 107, NULL),
            (2994, N'Személyzeti mosdó-zuhanyzó', N'Személyzeti mosdó-zuhanyzó', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 108, NULL, 108, NULL),
            (2995, N'Wc helyiség és mosdó mozgáskorlátozottak számára felszerelve', N'Wc helyiség és mosdó mozgáskorlátozottak számára felszerelve', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 109, NULL, 109, NULL),
            (2996, N'Számítástechnika terem', N'Számítástechnika terem', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 110, NULL, 110, NULL),
            (2997, N'Nevelőtestületi szoba', N'Nevelőtestületi szoba', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 111, NULL, 111, NULL),
            (6504, N'Tagintézmény-, intézményegységvezető iroda', N'Tagintézmény-, intézményegységvezető iroda', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 112, NULL, 112, NULL),
            (6505, N'Tanüzem', N'Tanüzem', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 113, NULL, 113, NULL),
            (6506, N'Felszolgáló terem', N'Felszolgáló terem', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 114, NULL, 114, NULL),
            (6507, N'Ruhatár', N'Ruhatár', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 115, NULL, 115, NULL),
            (6508, N'Titkárság', N'Titkárság', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 116, NULL, 116, NULL),
            (6509, N'Szakmai titkárság', N'Szakmai titkárság', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 117, NULL, 117, NULL),
            (6510, N'Tanügyi iroda', N'Tanügyi iroda', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 118, NULL, 118, NULL),
            (6511, N'Iktató', N'Iktató', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 119, NULL, 119, NULL),
            (6512, N'Irattár', N'Irattár', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 120, NULL, 120, NULL),
            (6513, N'Bútor raktár', N'Bútor raktár', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 121, NULL, 121, NULL),
            (7673, N'Folyosó', N'Folyosó', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 122, NULL, 122, NULL),
            (7796, N'Külső, nem saját helység', N'Külső, nem saját helység', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 123, NULL, 123, NULL),
            (9076, N'Duális helyszín', N'Duális helyszín', NULL, NULL, NULL, N'TeremTipus', 8, N'T', N'T', 124, NULL, 124, NULL),
            (657, N'Na', N'Na', NULL, NULL, NULL, N'MagantanulosagOka', 14, N'F', N'T', 1, NULL, 1, NULL),
            (658, N'Saját döntése alapján', N'Saját döntése alapján', NULL, NULL, NULL, N'MagantanulosagOka', 14, N'F', N'T', 2, NULL, 2, NULL),
            (659, N'Testi, érzékszervi fogyatékosság miatt (SZB szakvélemény alapján)', N'Testi, érzékszervi fogyatékosság miatt (SZB szakvélemény alapján)', NULL, NULL, NULL, N'MagantanulosagOka', 14, N'F', N'T', 3, NULL, 3, NULL),
            (660, N'Értelmi fogyatékosság miatt (SZB szakvélemény alapján)', N'Értelmi fogyatékosság miatt (SZB szakvélemény alapján', NULL, NULL, NULL, N'MagantanulosagOka', 14, N'F', N'T', 4, NULL, 4, NULL),
            (661, N'Egyéb fogyatékosság miatt (SZB szakvélemény alapján)', N'Egyéb fogyatékosság miatt (SZB szakvélemény alapján', NULL, NULL, NULL, N'MagantanulosagOka', 14, N'F', N'T', 5, NULL, 5, NULL),
            (662, N'Szakorvosi javaslat alapján', N'Szakorvosi javaslat alapján', NULL, NULL, NULL, N'MagantanulosagOka', 14, N'F', N'T', 6, NULL, 6, NULL),
            (663, N'Szülők tartós külföldi szolgálata miatt (szülő kérésére)', N'Szülők tartós külföldi szolgálata miatt (szülő kérésére)', NULL, NULL, NULL, N'MagantanulosagOka', 14, N'F', N'T', 7, NULL, 7, NULL),
            (664, N'Tanuló egyéb rendkívüli körülményei miatt (szülő kérésére)', N'Tanuló egyéb rendkívüli körülményei miatt (szülő kérésére)', NULL, NULL, NULL, N'MagantanulosagOka', 14, N'F', N'T', 8, NULL, 8, NULL),
            (665, N'Tanköteles koron túl', N'Tanköteles koron túl', NULL, NULL, NULL, N'MagantanulosagOka', 14, N'F', N'T', 9, NULL, 9, NULL),
            (6566, N'Egyéb okok miatt', N'Egyéb okok miatt', NULL, NULL, NULL, N'MagantanulosagOka', 14, N'F', N'T', 10, NULL, 10, NULL),
            (1575, N'Január', N'Január', NULL, NULL, NULL, N'Honapok', 55, N'F', N'T', 1, NULL, 1, NULL),
            (1576, N'Február', N'Február', NULL, NULL, NULL, N'Honapok', 55, N'F', N'T', 2, NULL, 2, NULL),
            (1577, N'Március', N'Március', NULL, NULL, NULL, N'Honapok', 55, N'F', N'T', 3, NULL, 3, NULL),
            (1578, N'Április', N'Április', NULL, NULL, NULL, N'Honapok', 55, N'F', N'T', 4, NULL, 4, NULL),
            (1579, N'Május', N'Május', NULL, NULL, NULL, N'Honapok', 55, N'F', N'T', 5, NULL, 5, NULL),
            (1580, N'Június', N'Június', NULL, NULL, NULL, N'Honapok', 55, N'F', N'T', 6, NULL, 6, NULL),
            (1581, N'Július', N'Július', NULL, NULL, NULL, N'Honapok', 55, N'F', N'T', 7, NULL, 7, NULL),
            (1582, N'Augusztus', N'Augusztus', NULL, NULL, NULL, N'Honapok', 55, N'F', N'T', 8, NULL, 8, NULL),
            (1583, N'Szeptember', N'Szeptember', NULL, NULL, NULL, N'Honapok', 55, N'F', N'T', 9, NULL, 9, NULL),
            (1584, N'Október', N'Október', NULL, NULL, NULL, N'Honapok', 55, N'F', N'T', 10, NULL, 10, NULL),
            (1585, N'November', N'November', NULL, NULL, NULL, N'Honapok', 55, N'F', N'T', 11, NULL, 11, NULL),
            (1586, N'December', N'December', NULL, NULL, NULL, N'Honapok', 55, N'F', N'T', 12, NULL, 12, NULL),
            (1151, N'Első igénylés', N'Első igénylés', NULL, NULL, NULL, N'DiakigazolvanyIgenyTipus', 34, N'F', N'T', 1, NULL, 1, NULL),
            (1152, N'Adatváltozás miatt', N'Adatváltozás miatt', NULL, NULL, NULL, N'DiakigazolvanyIgenyTipus', 34, N'F', N'T', 2, NULL, 2, NULL),
            (1153, N'Elveszett', N'Elveszett', NULL, NULL, NULL, N'DiakigazolvanyIgenyTipus', 34, N'F', N'T', 3, NULL, 3, NULL),
            (1154, N'Új igénylés téves adatok miatt', N'Új igénylés téves adatok miatt', NULL, NULL, NULL, N'DiakigazolvanyIgenyTipus', 34, N'F', N'T', 4, NULL, 4, NULL),
            (1155, N'Előző diákigazolvány sérülése miatt', N'Előző diákigazolvány sérülése miatt', NULL, NULL, NULL, N'DiakigazolvanyIgenyTipus', 34, N'F', N'T', 5, NULL, 5, NULL),
            (911, N'Na', N'Na', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 1, NULL, 1, NULL),
            (6529, N'allé', N'allé', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 2, NULL),
            (6530, N'alsó rakpart', N'alsó rakpart', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 3, NULL),
            (6531, N'alsósor', N'alsósor', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 4, NULL),
            (912, N'árok', N'árok', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 5, NULL),
            (913, N'átjáró', N'átjáró', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 6, NULL),
            (914, N'bástya', N'bástya', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 7, NULL),
            (915, N'bástyája', N'bástyája', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 8, NULL),
            (6532, N'bekötőút', N'bekötőút', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 9, NULL),
            (916, N'dűlő', N'dűlő', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 10, NULL),
            (917, N'dűlőút', N'dűlőút', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 11, NULL),
            (918, N'elágazás', N'elágazás', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 12, NULL),
            (919, N'erdősor', N'erdősor', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 13, NULL),
            (920, N'fasor', N'fasor', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 14, NULL),
            (921, N'fasora', N'fasora', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 15, NULL),
            (6533, N'felső rakpart', N'felső rakpart', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 16, NULL),
            (6534, N'felsősor', N'felsősor', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 17, NULL),
            (922, N'forduló', N'forduló', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 18, NULL),
            (6535, N'főtér', N'főtér', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 19, NULL),
            (923, N'főút', N'főút', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 20, NULL),
            (924, N'gát', N'gát', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 21, NULL),
            (925, N'hajóállomás', N'hajóállomás', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 22, NULL),
            (6536, N'határ', N'határ', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 23, NULL),
            (926, N'határsor', N'határsor', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 24, NULL),
            (927, N'határút', N'határút', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 25, NULL),
            (928, N'hegy', N'hegy', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 26, NULL),
            (929, N'helyrajzi szám', N'helyrajzi szám', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 27, NULL),
            (930, N'hídfő', N'hídfő', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 28, NULL),
            (6537, N'ipartelep', N'ipartelep', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 29, NULL),
            (931, N'játszótér', N'játszótér', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 30, NULL),
            (932, N'kert', N'kert', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 31, NULL),
            (6538, N'kertsor', N'kertsor', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 32, NULL),
            (933, N'kikötő', N'kikötő', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 33, NULL),
            (934, N'kilátó', N'kilátó', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 34, NULL),
            (6539, N'környék', N'környék', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 35, NULL),
            (935, N'körönd', N'körönd', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 36, NULL),
            (936, N'körtér', N'körtér', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 37, NULL),
            (937, N'körvasútsor', N'körvasútsor', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 38, NULL),
            (938, N'körút', N'körút', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 39, NULL),
            (6541, N'korzó', N'korzó', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 40, NULL),
            (939, N'köz', N'köz', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 41, NULL),
            (6542, N'lakópark', N'lakópark', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 42, NULL),
            (940, N'lakótelep', N'lakótelep', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 43, NULL),
            (941, N'lejtő', N'lejtő', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 44, NULL),
            (6543, N'lépcsősor', N'lépcsősor', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 45, NULL),
            (942, N'liget', N'liget', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 46, NULL),
            (6544, N'major', N'major', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 47, NULL),
            (6545, N'majorság', N'majorság', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 48, NULL),
            (943, N'lépcső', N'lépcső', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 49, NULL),
            (944, N'mélyút', N'mélyút', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 50, NULL),
            (6546, N'negyed', N'negyed', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 51, NULL),
            (6547, N'oldal', N'oldal', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 52, NULL),
            (6548, N'országút', N'országút', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 53, NULL),
            (945, N'park', N'park', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 54, NULL),
            (946, N'parkja', N'parkja', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 55, NULL),
            (947, N'part', N'part', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 56, NULL),
            (948, N'piac', N'piac', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 57, NULL),
            (949, N'pihenő', N'pihenő', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 58, NULL),
            (6549, N'pincesor', N'pincesor', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 59, NULL),
            (950, N'puszta', N'puszta', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 60, NULL),
            (951, N'rakpart', N'rakpart', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 61, NULL),
            (952, N'repülőtér', N'repülőtér', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 62, NULL),
            (953, N'rét', N'rét', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 63, NULL),
            (6550, N'sikátor', N'sikátor', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 64, NULL),
            (954, N'sor', N'sor', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 65, NULL),
            (955, N'sugárút', N'sugárút', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 66, NULL),
            (6551, N'szállás', N'szállás', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 67, NULL),
            (6552, N'szektor', N'szektor', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 68, NULL),
            (6553, N'szél', N'szél', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 69, NULL),
            (6554, N'szer', N'szer', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 70, NULL),
            (956, N'sziget', N'sziget', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 71, NULL),
            (7570, N'szőlő', N'szőlő', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 72, NULL),
            (6555, N'szőlőhegy', N'szőlőhegy', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 73, NULL),
            (6556, N'tag', N'tag', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 74, NULL),
            (6558, N'tető', N'tető', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 75, NULL),
            (6559, N'üdülősor', N'üdülősor', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 76, NULL),
            (6560, N'üdülőtelep', N'üdülőtelep', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 77, NULL),
            (957, N'sétaút', N'sétaút', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 78, NULL),
            (958, N'sétány', N'sétány', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 79, NULL),
            (959, N'telep', N'telep', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 80, NULL),
            (960, N'tere', N'tere', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 81, NULL),
            (961, N'tér', N'tér', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 82, NULL),
            (962, N'ösvény', N'ösvény', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 83, NULL),
            (963, N'udvar', N'udvar', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 84, NULL),
            (964, N'utca', N'utca', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 85, NULL),
            (965, N'út', N'út', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 86, NULL),
            (966, N'útja', N'útja', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 87, NULL),
            (6561, N'vár', N'vár', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 88, NULL),
            (6562, N'várkert', N'várkert', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 89, NULL),
            (6563, N'város', N'város', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 90, NULL),
            (6564, N'villasor', N'villasor', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 91, NULL),
            (967, N'üdülőpart', N'üdülőpart', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 92, NULL),
            (968, N'völgy', N'völgy', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 93, NULL),
            (969, N'zug', N'zug', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 94, NULL),
            (970, N'tanya', N'tanya', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 95, NULL),
            (971, N'Egyéb', N'Egyéb', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 96, NULL),
            (5444, N'kapu', N'kapu', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 97, NULL),
            (6835, N'tömb', N'tömb', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 98, NULL),
            (6961, N'Üres', N'Üres', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 3, NULL, 99, NULL),
            (7859, N'lakónegyed', N'lakónegyed', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'F', N'T', 2, NULL, 100, NULL),
            (8820, N'domb', N'domb', NULL, NULL, NULL, N'KozteruletJelleg', 17, N'T', N'T', 2, NULL, 101, NULL),
            (6486, N'Folyamatban', N'Folyamatban', NULL, NULL, NULL, N'DiakigazolvanyIgenyStatusz', 52, N'F', N'T', 1, NULL, 1, NULL),
            (6487, N'Adathiány miatt visszautasítva', N'Adathiány miatt visszautasítva', NULL, NULL, NULL, N'DiakigazolvanyIgenyStatusz', 52, N'F', N'T', 2, NULL, 2, NULL),
            (6488, N'Elkészült, postázásra vár', N'Elkészült, postázásra vár', NULL, NULL, NULL, N'DiakigazolvanyIgenyStatusz', 52, N'F', N'T', 3, NULL, 3, NULL),
            (6489, N'Elkészült, postázva', N'Elkészült, postázva', NULL, NULL, NULL, N'DiakigazolvanyIgenyStatusz', 52, N'F', N'T', 4, NULL, 4, NULL),
            (1563, N'Érvénytelen', N'Érvénytelen', NULL, NULL, NULL, N'DiakigazolvanyIgenyStatusz', 52, N'F', N'T', 5, NULL, 5, NULL),
            (1564, N'Selejtezve', N'Selejtezve', NULL, NULL, NULL, N'DiakigazolvanyIgenyStatusz', 52, N'F', N'T', 6, NULL, 6, NULL),
            (765, N'Magyarország', N'Hungary', N'Magyarország', NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 10, NULL, 1, NULL),
            (666, N'Afganisztán', N'Afghanistan', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 20, NULL, 2, NULL),
            (667, N'Albánia', N'Albania', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 30, NULL, 3, NULL),
            (669, N'Algéria', N'Algeria', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 40, NULL, 4, NULL),
            (827, N'Amerikai Csendes-óceáni-szigetek', N'Pacific Islands Americans', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 50, NULL, 5, NULL),
            (894, N'Amerikai Egyesült Államok', N'United States of America', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 60, NULL, 6, NULL),
            (670, N'Amerikai Szamoa', N'American Samoa', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 70, NULL, 7, NULL),
            (895, N'Amerikai Virgin-szigetek', N'U.S. Virgin Islands', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 80, NULL, 8, NULL),
            (671, N'Andorra', N'Andorra', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 90, NULL, 9, NULL),
            (672, N'Angola', N'Angola', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 100, NULL, 10, NULL),
            (850, N'Anguilla', N'Anguilla', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 110, NULL, 11, NULL),
            (668, N'Antarktisz', N'Antarctic', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 120, NULL, 12, NULL),
            (673, N'Antigua és Barbuda', N'Antigua and Barbuda', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 130, NULL, 13, NULL),
            (882, N'Arab Emírségek', N'United Arab Emirates', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 140, NULL, 14, NULL),
            (675, N'Argentína', N'Argentina', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 150, NULL, 15, NULL),
            (816, N'Aruba', N'Aruba', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 160, NULL, 16, NULL),
            (676, N'Ausztrália', N'Australia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 170, NULL, 17, NULL),
            (677, N'Ausztria', N'Austria', N'Ausztria', NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 180, NULL, 18, NULL),
            (674, N'Azerbajdzsán', N'Azerbaijan', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 190, NULL, 19, NULL),
            (678, N'Bahama-szigetek', N'The Bahamas', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 200, NULL, 20, NULL),
            (679, N'Bahrein', N'Bahrain', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 210, NULL, 21, NULL),
            (680, N'Banglades', N'Bangladesh', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 220, NULL, 22, NULL),
            (682, N'Barbados', N'Barbados', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 230, NULL, 23, NULL),
            (683, N'Belgium', N'Belgium', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 240, NULL, 24, NULL),
            (691, N'Belize', N'Belize', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 250, NULL, 25, NULL),
            (724, N'Benin', N'Benin', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 260, NULL, 26, NULL),
            (684, N'Bermuda', N'Bermuda', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 270, NULL, 27, NULL),
            (685, N'Bhután', N'Bhutan', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 280, NULL, 28, NULL),
            (840, N'Bissau-Guinea', N'Guinea-Bissau', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 290, NULL, 29, NULL),
            (686, N'Bolívia', N'Bolivia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 300, NULL, 30, NULL),
            (687, N'Bosznia-Hercegovina', N'Bosnia and Herzegovina', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 310, NULL, 31, NULL),
            (688, N'Botswana', N'Botswana', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 320, NULL, 32, NULL),
            (689, N'Bouvet-sziget', N'Bouvet Island', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 330, NULL, 33, NULL),
            (690, N'Brazília', N'Brazil', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 340, NULL, 34, NULL),
            (692, N'Brit Indiai-óceáni terület', N'British Indian Ocean Territory', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 350, NULL, 35, NULL),
            (694, N'Brit Virgin-szigetek', N'British Virgin Islands', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 360, NULL, 36, NULL),
            (695, N'Brunei', N'Bruneian', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 370, NULL, 37, NULL),
            (696, N'Bulgária', N'Bulgaria', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 380, NULL, 38, NULL),
            (896, N'Burkina Faso', N'Burkina Faso', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 390, NULL, 39, NULL),
            (698, N'Burundi', N'Burundian', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 400, NULL, 40, NULL),
            (708, N'Chile', N'Chile', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 410, NULL, 41, NULL),
            (722, N'Ciprus', N'Cyprus', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 420, NULL, 42, NULL),
            (714, N'Comore-szigetek', N'Comoros', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 430, NULL, 43, NULL),
            (718, N'Cook-szigetek', N'Cook Islands', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 440, NULL, 44, NULL),
            (719, N'Costa Rica', N'Costa Rica', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 450, NULL, 45, NULL),
            (707, N'Csád', N'Chad', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 460, NULL, 46, NULL),
            (723, N'Csehország', N'Czech Republic', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 470, NULL, 47, NULL),
            (725, N'Dánia', N'Denmark', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 480, NULL, 48, NULL),
            (865, N'Dél-afrikai Köztársaság', N'South Africa', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 490, NULL, 49, NULL),
            (736, N'Déli-Georgia és Déli-Sandwich-szi', N'South Georgia and the South Sandwich Islands', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 500, NULL, 50, NULL),
            (726, N'Dominikai Közösség', N'Dominica', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 510, NULL, 51, NULL),
            (727, N'Dominikai Köztársaság', N'Dominican Republic', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 520, NULL, 52, NULL),
            (744, N'Dzsibuti', N'Djiboutian', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 530, NULL, 53, NULL),
            (728, N'Ecuador', N'Ecuador', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 540, NULL, 54, NULL),
            (730, N'Egyenlítői-Guinea', N'Equatorial Guinea', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 550, NULL, 55, NULL),
            (891, N'Egyiptom', N'Egypt', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 560, NULL, 56, NULL),
            (774, N'Elefántcsontpart', N'Ivory Coast', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 570, NULL, 57, NULL),
            (732, N'Eritrea', N'Eritrea', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 580, NULL, 58, NULL),
            (826, N'Északi-Mariana-szigetek', N'Northern Mariana Islands', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 590, NULL, 59, NULL),
            (733, N'Észtország', N'Estonia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 600, NULL, 60, NULL),
            (731, N'Etiópia', N'Ethiopia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 610, NULL, 61, NULL),
            (735, N'Falkland-szigetek (Malvinas)', N'Falkland Islands', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 620, NULL, 62, NULL),
            (699, N'Fehéroroszország', N'Belarus', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 630, NULL, 63, NULL),
            (734, N'Feröer', N'Faroe Islands', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 640, NULL, 64, NULL),
            (737, N'Fidzsi-szigetek', N'Fiji', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 650, NULL, 65, NULL),
            (738, N'Finnország', N'Finland', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 660, NULL, 66, NULL),
            (743, N'Francia déli területek', N'French Southern and Antarctic Lands', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 670, NULL, 67, NULL),
            (741, N'Francia Guyana', N'French Guiana', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 680, NULL, 68, NULL),
            (742, N'Francia Polinézia', N'French Polynesia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 690, NULL, 69, NULL),
            (740, N'Franciaország', N'France', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 700, NULL, 70, NULL),
            (739, N'Franciaország, Európai', N'Franciaország, Európai', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 710, NULL, 71, NULL),
            (836, N'Fülöp-szigetek', N'Philippines', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 720, NULL, 72, NULL),
            (745, N'Gabon', N'Gabon', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 730, NULL, 73, NULL),
            (747, N'Gambia', N'The Gambia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 740, NULL, 74, NULL),
            (749, N'Ghána', N'Ghana', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 750, NULL, 75, NULL),
            (750, N'Gibraltár', N'Gibraltar', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 760, NULL, 76, NULL),
            (752, N'Görögország', N'Greece', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 770, NULL, 77, NULL),
            (754, N'Grenada', N'Grenada', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 780, NULL, 78, NULL),
            (753, N'Grönland', N'Greenland', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 790, NULL, 79, NULL),
            (746, N'Grúzia', N'Georgia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 800, NULL, 80, NULL),
            (755, N'Guadeloupe', N'Guadeloupe', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 810, NULL, 81, NULL),
            (756, N'Guam', N'Guam', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 820, NULL, 82, NULL),
            (757, N'Guatemala', N'Guatemala', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 830, NULL, 83, NULL),
            (758, N'Guinea', N'Guinea', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 840, NULL, 84, NULL),
            (759, N'Guyana', N'Guyana', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 850, NULL, 85, NULL),
            (760, N'Haiti', N'Haitian', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 860, NULL, 86, NULL),
            (761, N'Heard-sziget és McDonald-szigetek', N'Heard Island and McDonald Islands', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 870, NULL, 87, NULL),
            (815, N'Holland Antillák', N'Netherland Antilles', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 880, NULL, 88, NULL),
            (814, N'Hollandia', N'Netherlands', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 890, NULL, 89, NULL),
            (763, N'Honduras', N'Honduras', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 900, NULL, 90, NULL),
            (764, N'Hongkong', N'Hong Kong', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 910, NULL, 91, NULL),
            (720, N'Horvátország', N'Croatia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 920, NULL, 92, NULL),
            (767, N'India', N'India', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 930, NULL, 93, NULL),
            (768, N'Indonézia', N'Indonesia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 940, NULL, 94, NULL),
            (770, N'Irak', N'Iraq', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 950, NULL, 95, NULL),
            (769, N'Irán', N'Iran', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 960, NULL, 96, NULL),
            (771, N'Írország', N'Ireland', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 970, NULL, 97, NULL),
            (766, N'Izland', N'Iceland', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 980, NULL, 98, NULL),
            (772, N'Izrael', N'Israel', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 990, NULL, 99, NULL),
            (775, N'Jamaica', N'Jamaica', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1000, NULL, 100, NULL),
            (776, N'Japán', N'Japanese', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1010, NULL, 101, NULL),
            (902, N'Jemen', N'Yemen', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1020, NULL, 102, NULL),
            (7701, N'Jersey', N'Jersey', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1030, NULL, 103, NULL),
            (778, N'Jordánia', N'Jordan', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1040, NULL, 104, NULL),
            (903, N'Jugoszlávia', N'Yugoslavia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1050, NULL, 105, NULL),
            (704, N'Kajmán-szigetek', N'Cayman Islands', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1060, NULL, 106, NULL),
            (700, N'Kambodzsa', N'Cambodia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1070, NULL, 107, NULL),
            (701, N'Kamerun', N'Cameroon', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1080, NULL, 108, NULL),
            (702, N'Kanada', N'Canada', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1090, NULL, 109, NULL),
            (711, N'Karácsony-sziget', N'Christmas Island', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1100, NULL, 110, NULL),
            (843, N'Katar', N'Qatar', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1110, NULL, 111, NULL),
            (777, N'Kazahsztán', N'Kazakhstan', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1120, NULL, 112, NULL),
            (841, N'Kelet-Timor', N'East Timor', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1130, NULL, 113, NULL),
            (779, N'Kenya', N'Kenya', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1140, NULL, 114, NULL),
            (709, N'Kína', N'China', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1150, NULL, 115, NULL),
            (783, N'Kirgizisztán', N'Kyrgyzstan', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1160, NULL, 116, NULL),
            (751, N'Kiribati', N'Kiribatian', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1170, NULL, 117, NULL),
            (712, N'Kókusz (Keeling)-szigetek', N'Cocos (Keeling) Islands', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1180, NULL, 118, NULL),
            (713, N'Kolumbia', N'Colombia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1190, NULL, 119, NULL),
            (716, N'Kongó', N'Congo', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1200, NULL, 120, NULL),
            (717, N'Kongói Demokratikus Köztársaság', N'Democratic Republic of the Congo', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1210, NULL, 121, NULL),
            (781, N'Koreai Köztársaság', N'South Korea', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1220, NULL, 122, NULL),
            (780, N'Koreai Népi Demokratikus Köztársa', N'North Korea', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1230, NULL, 123, NULL),
            (7687, N'Koszovó', N'Kosovo', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1240, NULL, 124, NULL),
            (705, N'Közép-afrikai Köztársaság', N'Central African Republic', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1250, NULL, 125, NULL),
            (721, N'Kuba', N'Cuba', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1260, NULL, 126, NULL),
            (782, N'Kuvait', N'Kuwait', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1270, NULL, 127, NULL),
            (784, N'Laosz', N'Laos', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1280, NULL, 128, NULL),
            (838, N'Lengyelország', N'Poland', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1290, NULL, 129, NULL),
            (786, N'Lesotho', N'Lesotho', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1300, NULL, 130, NULL),
            (787, N'Lettország', N'Latvia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1310, NULL, 131, NULL),
            (785, N'Libanon', N'Lebanon', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1320, NULL, 132, NULL),
            (788, N'Libéria', N'Liberia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1330, NULL, 133, NULL),
            (789, N'Líbia', N'Libya', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1340, NULL, 134, NULL),
            (790, N'Liechtenstein', N'Liechtenstein', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1350, NULL, 135, NULL),
            (791, N'Litvánia', N'Lithuania', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1360, NULL, 136, NULL),
            (792, N'Luxemburg', N'Luxembourg', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1370, NULL, 137, NULL),
            (890, N'Macedónia', N'Macedonia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1380, NULL, 138, NULL),
            (794, N'Madagaszkár', N'Madagascar', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1390, NULL, 139, NULL),
            (793, N'Makaó', N'Macau', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1400, NULL, 140, NULL),
            (796, N'Malajzia', N'Malaysia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1410, NULL, 141, NULL),
            (795, N'Malawi', N'Malawian', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1420, NULL, 142, NULL),
            (797, N'Maldív-szigetek', N'Maldives', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1430, NULL, 143, NULL),
            (798, N'Mali', N'Malian', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1440, NULL, 144, NULL),
            (799, N'Málta', N'Malta', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1450, NULL, 145, NULL),
            (808, N'Marokkó', N'Morocco', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1460, NULL, 146, NULL),
            (829, N'Marshall-szigetek', N'Marshall Islands', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1470, NULL, 147, NULL),
            (800, N'Martinique', N'Martinique', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1480, NULL, 148, NULL),
            (801, N'Mauritánia', N'Mauritania', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1490, NULL, 149, NULL),
            (802, N'Mauritius', N'Mauritius', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1500, NULL, 150, NULL),
            (715, N'Mayotte', N'Mayotte', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1510, NULL, 151, NULL),
            (803, N'Mexikó', N'Mexico', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1520, NULL, 152, NULL),
            (697, N'Mianmar, Burma', N'Myanmar, Burma', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1530, NULL, 153, NULL),
            (828, N'Mikronézia', N'Micronesia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1540, NULL, 154, NULL),
            (806, N'Moldova', N'Moldova', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1550, NULL, 155, NULL),
            (804, N'Monaco', N'Monaco', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1560, NULL, 156, NULL),
            (805, N'Mongólia', N'Mongolia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1570, NULL, 157, NULL),
            (7572, N'Montenegró', N'Montenegro', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1580, NULL, 158, NULL),
            (807, N'Montserrat', N'Montserrat', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1590, NULL, 159, NULL),
            (809, N'Mozambik', N'Mozambique', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1600, NULL, 160, NULL),
            (892, N'Nagy-Britannia', N'Great Britain', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1610, NULL, 161, NULL),
            (811, N'Namíbia', N'Namibia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1620, NULL, 162, NULL),
            (812, N'Nauru', N'Nauru', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1630, NULL, 163, NULL),
            (748, N'Németország', N'Germany', N'Németország', NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1640, NULL, 164, NULL),
            (813, N'Nepál', N'Nepal', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1650, NULL, 165, NULL),
            (820, N'Nicaragua', N'Nicaragua', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1660, NULL, 166, NULL),
            (821, N'Niger', N'Niger', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1670, NULL, 167, NULL),
            (822, N'Nigéria', N'Nigeria', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1680, NULL, 168, NULL),
            (823, N'Niue', N'Niue', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1690, NULL, 169, NULL),
            (824, N'Norfolk-sziget', N'Norfolk Island', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1700, NULL, 170, NULL),
            (825, N'Norvégia', N'Norway', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1710, NULL, 171, NULL),
            (868, N'Nyugat-Szahara', N'Western Sahara', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1720, NULL, 172, NULL),
            (901, N'Nyugat-Szamoa', N'Western Samoa', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1730, NULL, 173, NULL),
            (773, N'Olaszország', N'Italy', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1740, NULL, 174, NULL),
            (810, N'Omán', N'Oman', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1750, NULL, 175, NULL),
            (681, N'Örményország', N'Armenia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1760, NULL, 176, NULL),
            (846, N'Oroszország', N'Russia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1770, NULL, 177, NULL),
            (831, N'Pakisztán', N'Pakistan', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1780, NULL, 178, NULL),
            (830, N'Palau', N'Palau', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1790, NULL, 179, NULL),
            (832, N'Panama', N'Panama', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1800, NULL, 180, NULL),
            (833, N'Pápua Új-Guinea', N'Papua New Guinea', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1810, NULL, 181, NULL),
            (834, N'Paraguay', N'Paraguay', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1820, NULL, 182, NULL),
            (835, N'Peru', N'Peru', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1830, NULL, 183, NULL),
            (837, N'Pitcairn-szigetek', N'Pitcairn Islands', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1840, NULL, 184, NULL),
            (839, N'Portugália', N'Portugal', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1850, NULL, 185, NULL),
            (842, N'Puerto Rico', N'Puerto Rico', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1860, NULL, 186, NULL),
            (844, N'Réunion', N'Réunion', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1870, NULL, 187, NULL),
            (845, N'Románia', N'Romania', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1880, NULL, 188, NULL),
            (847, N'Ruanda', N'Rwanda', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1890, NULL, 189, NULL),
            (849, N'Saint Christopher and Nevis', N'Saint Kitts and Nevis', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1900, NULL, 190, NULL),
            (851, N'Saint Lucia', N'Saint Lucia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1910, NULL, 191, NULL),
            (852, N'Saint-Pierre és Miquelon', N'Saint Pierre and Miquelon', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1920, NULL, 192, NULL),
            (853, N'Saint Vincent', N'Saint Vincent and the Grenadines', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1930, NULL, 193, NULL),
            (693, N'Salamon-szigetek', N'Solomon Islands', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1940, NULL, 194, NULL),
            (729, N'Salvador', N'El Salvador', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1950, NULL, 195, NULL),
            (854, N'San Marino', N'San Marino', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1960, NULL, 196, NULL),
            (855, N'Sao Tomé és Príncipe', N'Sao Tome and Principe', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1970, NULL, 197, NULL),
            (858, N'Seychelle-szigetek', N'Seychelles', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1980, NULL, 198, NULL),
            (859, N'Sierra Leone', N'Sierra Leone', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1990, NULL, 199, NULL),
            (867, N'Spanyolország', N'Spain', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2000, NULL, 200, NULL),
            (706, N'Srí Lanka', N'Sri Lanka', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2010, NULL, 201, NULL),
            (870, N'Suriname', N'Suriname', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2020, NULL, 202, NULL),
            (874, N'Svájc', N'Switzerland', N'Svájc', NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2030, NULL, 203, NULL),
            (871, N'Svatbard', N'Svalbard', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2040, NULL, 204, NULL),
            (873, N'Svédország', N'Sweden', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2050, NULL, 205, NULL),
            (856, N'Szaúd-Arábia', N'Saudi Arabia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2060, NULL, 206, NULL),
            (857, N'Szenegál', N'Senegal', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2070, NULL, 207, NULL),
            (848, N'Szent Ilona', N'Saint Helena', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2080, NULL, 208, NULL),
            (905, N'Szerbia', N'Serbia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2090, NULL, 209, NULL),
            (7105, N'Szerbia és Montenegró', N'Serbia and Montenegro', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2100, NULL, 210, NULL),
            (860, N'Szingapúr', N'Singapore', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2110, NULL, 211, NULL),
            (875, N'Szíria', N'Syria', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2120, NULL, 212, NULL),
            (861, N'Szlovákia', N'Slovakia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2130, NULL, 213, NULL),
            (863, N'Szlovénia', N'Slovenia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2140, NULL, 214, NULL),
            (864, N'Szomália', N'Somalia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2150, NULL, 215, NULL),
            (7368, N'Szovjetunió', N'Soviet Union', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2160, NULL, 216, NULL),
            (869, N'Szudán', N'Sudan', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2170, NULL, 217, NULL),
            (872, N'Szváziföld', N'Eswatini', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2180, NULL, 218, NULL),
            (876, N'Tádzsikisztán', N'Tajikistan', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2190, NULL, 219, NULL),
            (710, N'Tajvan, Kína tartománya', N'Taiwan', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2200, NULL, 220, NULL),
            (893, N'Tanzánia', N'Tanzania', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2210, NULL, 221, NULL),
            (877, N'Thaiföld', N'Tahiland', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2220, NULL, 222, NULL),
            (878, N'Togó', N'Togo', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2230, NULL, 223, NULL),
            (879, N'Tokelau-szigetek', N'Tokelau', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2240, NULL, 224, NULL),
            (880, N'Tonga', N'Tonga', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2250, NULL, 225, NULL),
            (884, N'Törökország', N'Turkey', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2260, NULL, 226, NULL),
            (881, N'Trinidad és Tobago', N'Trinidad and Tobago', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2270, NULL, 227, NULL),
            (883, N'Tunézia', N'Tunisia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2280, NULL, 228, NULL),
            (885, N'Türkmenisztán', N'Turkmenistan', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2290, NULL, 229, NULL),
            (886, N'Turks és Caicos-szigetek', N'Turks and Caicos Islands', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2300, NULL, 230, NULL),
            (887, N'Tuvalu', N'Tuvalu', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2310, NULL, 231, NULL),
            (888, N'Uganda', N'Uganda', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2320, NULL, 232, NULL),
            (817, N'Új-Kaledónia', N'New Caledonia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2330, NULL, 233, NULL),
            (819, N'Új-Zéland', N'New-Zealand', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2340, NULL, 234, NULL),
            (889, N'Ukrajna', N'Ukraine', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2350, NULL, 235, NULL),
            (897, N'Uruguay', N'Uruguay', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2360, NULL, 236, NULL),
            (898, N'Üzbegisztán', N'Uzbekistan', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2370, NULL, 237, NULL),
            (818, N'Vanuatu', N'Vanuatu', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2380, NULL, 238, NULL),
            (762, N'Vatikán', N'Vatican City', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2390, NULL, 239, NULL),
            (899, N'Venezuela', N'Venezuela', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2400, NULL, 240, NULL),
            (862, N'Vietnam', N'Vietnam', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2410, NULL, 241, NULL),
            (900, N'Wallis és Futuna', N'Wallis and Futuna', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2420, NULL, 242, NULL),
            (904, N'Zambia', N'Zambia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2430, NULL, 243, NULL),
            (866, N'Zimbabwe', N'Zimbabwe', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2440, NULL, 244, NULL),
            (703, N'Zöld-foki Köztársaság', N'Cape Verde', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 2450, NULL, 245, NULL),
            (8402, N'Csehszlovákia', N'Csehszlovákia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 472, NULL, 246, NULL),
            (8458, N'Egyesült Királyság', N'United Kingdom', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 555, NULL, 247, NULL),
            (8746, N'Cseh Köztársaság', N'Cseh Köztársaság', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 468, NULL, 248, NULL),
            (8747, N'Egyesült Nemzetek Szervezete', N'Egyesült Nemzetek Szervezete', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 557, NULL, 249, NULL),
            (8748, N'Európai Unió', N'Európai Unió', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 615, NULL, 250, NULL),
            (8749, N'Moldávia', N'Moldávia', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1545, NULL, 251, NULL),
            (8750, N'NATO', N'NATO', NULL, NULL, NULL, N'OrszagTipus', 15, N'T', N'T', 1625, NULL, 252, NULL),
            (8751, N'Egyesült Arab Emírségek', N'Egyesült Arab Emírségek', NULL, NULL, NULL, N'OrszagTipus', 15, N'F', N'T', 141, NULL, 253, NULL),
            (8752, N'Palesztina', N'Palesztina', NULL, NULL, NULL, N'OrszagTipus', 15, N'F', N'T', 1771, NULL, 254, NULL),
            (18, N'Feldolgozásra vár', N'Feldolgozásra vár', NULL, NULL, NULL, N'EsemenyStatusz', 4, N'F', N'T', 1, NULL, 1, NULL),
            (23, N'Sikertelen feldolgozás időtúllépés miatt', N'Sikertelen feldolgozás időtúllépés miatt', NULL, NULL, NULL, N'EsemenyStatusz', 4, N'F', N'T', 2, NULL, 2, NULL),
            (29, N'Sikeres feldolgozás', N'Sikeres feldolgozás', NULL, NULL, NULL, N'EsemenyStatusz', 4, N'F', N'T', 3, NULL, 3, NULL),
            (30, N'Sikertelen feldolgozás', N'Sikertelen feldolgozás', NULL, NULL, NULL, N'EsemenyStatusz', 4, N'F', N'T', 4, NULL, 4, NULL),
            (1024, N'Na', N'Na', NULL, NULL, NULL, N'EmailTipus', 21, N'F', N'T', 1, NULL, 1, NULL),
            (1025, N'Hivatalos', N'Hivatalos', NULL, NULL, NULL, N'EmailTipus', 21, N'F', N'T', 2, NULL, 2, NULL),
            (1026, N'Magán', N'Magán', NULL, NULL, NULL, N'EmailTipus', 21, N'F', N'T', 3, NULL, 3, NULL),
            (7716, N'Publikus', N'Publikus', NULL, NULL, NULL, N'EmailTipus', 21, N'F', N'T', 4, NULL, 4, NULL),
            (1019, N'Ismeretlen', N'Ismeretlen', NULL, NULL, NULL, N'TelefonTipus', 20, N'F', N'T', 1, NULL, 1, NULL),
            (1020, N'Vezetékes', N'Vezetékes', NULL, NULL, NULL, N'TelefonTipus', 20, N'F', N'T', 2, NULL, 2, NULL),
            (1021, N'Mobil', N'Mobil', NULL, NULL, NULL, N'TelefonTipus', 20, N'F', N'T', 3, NULL, 3, NULL),
            (1022, N'Munkahelyi', N'Munkahelyi', NULL, NULL, NULL, N'TelefonTipus', 20, N'F', N'T', 4, NULL, 4, NULL),
            (1023, N'Fax', N'Fax', NULL, NULL, NULL, N'TelefonTipus', 20, N'F', N'T', 5, NULL, 5, NULL),
            (7715, N'Publikus', N'Publikus', NULL, NULL, NULL, N'TelefonTipus', 20, N'F', N'T', 6, NULL, 6, NULL),
            (216, N'Na', N'Na', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 1, NULL, 1, NULL),
            (217, N'magyar', N'Hungarian', N'magyar', NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 2, NULL, 2, NULL),
            (218, N'afgán', N'Afghan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 3, NULL, 3, NULL),
            (219, N'Aland-szigeteki', N'Alan Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 4, NULL, 4, NULL),
            (220, N'albán', N'Albanian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 5, NULL, 5, NULL),
            (221, N'Algériai', N'Algerian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 6, NULL, 6, NULL),
            (459, N'Amerikai', N'American', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 7, NULL, 7, NULL),
            (490, N'Amerikai csendes-ó.szk-i', N'Amerikai csendes-ó.szk-i', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 8, NULL, 8, NULL),
            (222, N'Amerikai szamoai', N'American Samoan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 9, NULL, 9, NULL),
            (223, N'Amerikai virgin-szk-i', N'Virgin Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 10, NULL, 10, NULL),
            (224, N'Andorrai', N'Andorran', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 11, NULL, 11, NULL),
            (225, N'Angolai', N'Angolan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 12, NULL, 12, NULL),
            (226, N'Anguillai', N'Anguillan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 13, NULL, 13, NULL),
            (227, N'Antarktiszi', N'Antarctic', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 14, NULL, 14, NULL),
            (228, N'Antiguai és barbudai', N'Antiguan and Barbudan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 15, NULL, 15, NULL),
            (276, N'Arab emirátusi', N'Emirati', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 16, NULL, 16, NULL),
            (229, N'Argentin', N'Argentine', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 17, NULL, 17, NULL),
            (230, N'Arubai', N'Aruban', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 18, NULL, 18, NULL),
            (231, N'Ausztrál', N'Australian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 19, NULL, 19, NULL),
            (233, N'Azerbajdzsán', N'Azerbaijani', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 20, NULL, 20, NULL),
            (234, N'Bahama-szigeteki', N'Bahamian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 21, NULL, 21, NULL),
            (235, N'Bahraini', N'Bahraini', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 22, NULL, 22, NULL),
            (236, N'Bangladesi', N'Bangladeshi', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 23, NULL, 23, NULL),
            (237, N'Barbadosi', N'Barbadian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 24, NULL, 24, NULL),
            (239, N'Belga', N'Belgian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 25, NULL, 25, NULL),
            (240, N'Belizei', N'Belizean', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 26, NULL, 26, NULL),
            (238, N'Belorusz', N'Belarusian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 27, NULL, 27, NULL),
            (241, N'Benini', N'Beninese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 28, NULL, 28, NULL),
            (242, N'Bermudai', N'Bermudian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 29, NULL, 29, NULL),
            (243, N'Bhutáni', N'Bhutanese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 30, NULL, 30, NULL),
            (244, N'Bissau-guineai', N'Bissau-Guinean', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 31, NULL, 31, NULL),
            (251, N'bolgár', N'Bulgarian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 32, NULL, 32, NULL),
            (245, N'Bolíviai', N'Bolivian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 33, NULL, 33, NULL),
            (246, N'Bosznia-hercegovinai', N'Bosnian and Herzegovinian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 34, NULL, 34, NULL),
            (247, N'Botswanai', N'Motswana', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 35, NULL, 35, NULL),
            (248, N'Bouvet-szigeti', N'Bouvet Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 36, NULL, 36, NULL),
            (249, N'Brazil', N'Brazilian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 37, NULL, 37, NULL),
            (377, N'Brit', N'British', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 38, NULL, 38, NULL),
            (486, N'Brit indiai-óceáni területi', N'Brittish Indian Oceaner', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 39, NULL, 39, NULL),
            (471, N'Brit virgin-szigeteki', N'British Virgin Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 40, NULL, 40, NULL),
            (250, N'Brunei', N'Bruneian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 41, NULL, 41, NULL),
            (252, N'Burkina fasoi', N'Burkinabe', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 42, NULL, 42, NULL),
            (253, N'Burmai', N'Burmese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 43, NULL, 43, NULL),
            (254, N'Burundi', N'Burundian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 44, NULL, 44, NULL),
            (255, N'Canton-enderbury-szigeteki', N'Candon Enderbury Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 45, NULL, 45, NULL),
            (256, N'Chilei', N'Chilean', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 46, NULL, 46, NULL),
            (257, N'Ciprusi', N'Cypriot', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 47, NULL, 47, NULL),
            (259, N'Comorre-szigeteki', N'Comoran', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 48, NULL, 48, NULL),
            (260, N'Cook-szigeteki', N'Cook Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 49, NULL, 49, NULL),
            (261, N'Costa ricai', N'Costa Rican', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 50, NULL, 50, NULL),
            (262, N'Csádi', N'Chadian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 51, NULL, 51, NULL),
            (263, N'cseh', N'Czech', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 52, NULL, 52, NULL),
            (264, N'Csehszlovák', N'Czechoslovakian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 53, NULL, 53, NULL),
            (265, N'Dahomeyi', N'Dahoman', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 54, NULL, 54, NULL),
            (266, N'Dán', N'Dane', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 55, NULL, 55, NULL),
            (267, N'Dél-afrikai', N'South African', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 56, NULL, 56, NULL),
            (269, N'Dél-koreai', N'South Korean', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 57, NULL, 57, NULL),
            (268, N'Déli-georgiai', N'South Georgian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 58, NULL, 58, NULL),
            (271, N'Dominikai közösségi', N'Dominican', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 59, NULL, 59, NULL),
            (272, N'Dominikai köztársasági', N'Dominican', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 60, NULL, 60, NULL),
            (472, N'Dronning Maud-földi', N'Dronning Mauder', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 61, NULL, 61, NULL),
            (273, N'Dzsibuti', N'Djiboutian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 62, NULL, 62, NULL),
            (274, N'Ecuadori', N'Ecuadorian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 63, NULL, 63, NULL),
            (275, N'Egyenlítői-guineai', N'Equatorial Guinean', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 64, NULL, 64, NULL),
            (277, N'Egyiptomi', N'Egyptian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 65, NULL, 65, NULL),
            (278, N'Elefántcsontparti', N'Ivorian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 66, NULL, 66, NULL),
            (279, N'Eritreai', N'Eritrean', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 67, NULL, 67, NULL),
            (281, N'Észak-koreai', N'North Korean', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 68, NULL, 68, NULL),
            (280, N'Észak-marinai-szigeteki', N'North Marin Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 69, NULL, 69, NULL),
            (282, N'észt', N'Esthonian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 70, NULL, 70, NULL),
            (283, N'Etióp', N'Ethiopian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 71, NULL, 71, NULL),
            (284, N'Falkland-szigeteki', N'Falkland Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 72, NULL, 72, NULL),
            (286, N'Felső-voltai', N'Felso Voltaire', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 73, NULL, 73, NULL),
            (285, N'Feröeri', N'Faroese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 74, NULL, 74, NULL),
            (287, N'Fidzsi-szigeteki', N'Fijian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 75, NULL, 75, NULL),
            (288, N'finn', N'Finn', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 76, NULL, 76, NULL),
            (291, N'Francia', N'Frenchman, Frenchwoman', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 77, NULL, 77, NULL),
            (487, N'Francia déli területeki (Francia Déli- és Antarktiszi-Területek)', N'Francia déli területeki (Francia Déli- és Antarktiszi-Területek)', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 78, NULL, 78, NULL),
            (488, N'Francia déli területeki (Francia Déli Területek)', N'Francia déli területeki (Francia Déli Területek)', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 79, NULL, 79, NULL),
            (289, N'Francia guyanai', N'French Guianese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 80, NULL, 80, NULL),
            (290, N'Francia polinéziai', N'French Polynesian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 81, NULL, 81, NULL),
            (292, N'Fülöp-szigeteki', N'Filipino', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 82, NULL, 82, NULL),
            (293, N'Gaboni', N'Gabonese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 83, NULL, 83, NULL),
            (294, N'Gambiai', N'Gambian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 84, NULL, 84, NULL),
            (295, N'Ghánai', N'Ghanaian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 85, NULL, 85, NULL),
            (473, N'Gibraltári', N'Gibraltarian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 86, NULL, 86, NULL),
            (296, N'Gilbert és Ellice-szigeteki', N'Gilbert and Ellice Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 87, NULL, 87, NULL),
            (297, N'görög', N'Greek', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 88, NULL, 88, NULL),
            (298, N'Grenadai', N'Grenadian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 89, NULL, 89, NULL),
            (299, N'Grönlandi', N'Greenlander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 90, NULL, 90, NULL),
            (300, N'grúz', N'Georgian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 91, NULL, 91, NULL),
            (301, N'Guadeloupei', N'Guadeloupian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 92, NULL, 92, NULL),
            (302, N'Guami', N'Guamanian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 93, NULL, 93, NULL),
            (303, N'Guatemalai', N'Guatemalan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 94, NULL, 94, NULL),
            (304, N'Guernseyi', N'Channel Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 95, NULL, 95, NULL),
            (305, N'Guineai', N'Guinean', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 96, NULL, 96, NULL),
            (306, N'Guyanai', N'Guianese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 97, NULL, 97, NULL),
            (307, N'Haiti', N'Haitian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 98, NULL, 98, NULL),
            (308, N'Heard-szigeti', N'Heard Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 99, NULL, 99, NULL),
            (310, N'Holland', N'Dutchman, Dutchwoman', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 100, NULL, 100, NULL),
            (309, N'Holland antilláki', N'Dutch Antillean', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 101, NULL, 101, NULL),
            (311, N'Hondurasi', N'Honduran', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 102, NULL, 102, NULL),
            (312, N'Hongkongi', N'Hongkonger', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 103, NULL, 103, NULL),
            (491, N'Hontalan', N'Stateless', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 104, NULL, 104, NULL),
            (313, N'horvát', N'Croatian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 105, NULL, 105, NULL),
            (314, N'Indiai', N'Indian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 106, NULL, 106, NULL),
            (315, N'Indonéz', N'Indonesian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 107, NULL, 107, NULL),
            (318, N'Ír', N'Irishman, Irishwoman', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 108, NULL, 108, NULL),
            (316, N'Iraki', N'Iraqi', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 109, NULL, 109, NULL),
            (317, N'Iráni', N'Iranian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 110, NULL, 110, NULL),
            (485, N'Ismeretlen', N'Ismeretlen', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 111, NULL, 111, NULL),
            (319, N'Izlandi', N'Icelandic', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 112, NULL, 112, NULL),
            (320, N'Izraeli', N'Israeli', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 113, NULL, 113, NULL),
            (321, N'Jamaikai', N'Jamaican', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 114, NULL, 114, NULL),
            (322, N'japán', N'Japanese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 115, NULL, 115, NULL),
            (323, N'Jemeni', N'Yemeni', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 116, NULL, 116, NULL),
            (270, N'Jemeni (Demokratikus Jemen)', N'Yemeni', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 117, NULL, 117, NULL),
            (324, N'Jerseyi', N'Channel Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 118, NULL, 118, NULL),
            (474, N'Johnston-szigeti', N'Johnston Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 119, NULL, 119, NULL),
            (325, N'Jordán', N'Jordanian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 120, NULL, 120, NULL),
            (326, N'Jugoszláv', N'Yugoslavian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 121, NULL, 121, NULL),
            (327, N'Kajmán-szigeteki', N'Caymanian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 122, NULL, 122, NULL),
            (328, N'Kambodzsai', N'Cambodian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 123, NULL, 123, NULL),
            (329, N'Kameruni', N'Cameroonian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 124, NULL, 124, NULL),
            (330, N'Kanadai', N'Canadian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 125, NULL, 125, NULL),
            (331, N'Karácsony-szigeti', N'Christmas Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 126, NULL, 126, NULL),
            (404, N'Katari', N'Qatari', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 127, NULL, 127, NULL),
            (332, N'kazah', N'Kazakh', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 128, NULL, 128, NULL),
            (333, N'Kelet-timori (2002.05.20-ig)', N'East Timorese (until 20-05-2002)', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 129, NULL, 129, NULL),
            (489, N'Kelet-timori (2002.05.20-tól)', N'Timorese (since 20-05-2002)', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 130, NULL, 130, NULL),
            (334, N'Kenyai', N'Kenyan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 131, NULL, 131, NULL),
            (335, N'kínai', N'Chinese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 132, NULL, 132, NULL),
            (336, N'Kirgiz', N'Kyrgyz', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 133, NULL, 133, NULL),
            (337, N'Kiribati', N'Kiribatian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 134, NULL, 134, NULL),
            (258, N'Kókusz-szigeteki', N'Cocus Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 135, NULL, 135, NULL),
            (338, N'Kolumbiai', N'Colombian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 136, NULL, 136, NULL),
            (339, N'Kongói köztársasági', N'Congolese republican', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 137, NULL, 137, NULL),
            (340, N'Kongói (zairei)', N'Congolese (Zairean)', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 138, NULL, 138, NULL),
            (481, N'koreai', N'Korean', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 139, NULL, 139, NULL),
            (482, N'Koszovói', N'Kosovar, Kosovac', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 140, NULL, 140, NULL),
            (341, N'Közép-afrikai', N'Central African', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 141, NULL, 141, NULL),
            (342, N'Kubai', N'Cuban', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 142, NULL, 142, NULL),
            (343, N'Kuvaiti', N'Kuwaiti', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 143, NULL, 143, NULL),
            (344, N'Laoszi', N'Laotian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 144, NULL, 144, NULL),
            (345, N'Lengyel', N'Pole', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 145, NULL, 145, NULL),
            (346, N'Lesothoi', N'Mosotho', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 146, NULL, 146, NULL),
            (347, N'lett', N'Latvian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 147, NULL, 147, NULL),
            (348, N'Libanoni', N'Lebanese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 148, NULL, 148, NULL),
            (349, N'Libériai', N'Liberian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 149, NULL, 149, NULL),
            (350, N'Líbiai', N'Libyan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 150, NULL, 150, NULL),
            (351, N'Liechtensteini', N'Liechtensteiner', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 151, NULL, 151, NULL),
            (352, N'litván', N'Lithuanian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 152, NULL, 152, NULL),
            (353, N'Luxemburgi', N'Luxembourger', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 153, NULL, 153, NULL),
            (354, N'macedón', N'Macedonian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 154, NULL, 154, NULL),
            (355, N'Madagaszkári', N'Malagasy', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 155, NULL, 155, NULL),
            (356, N'Makaói', N'Macanese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 156, NULL, 156, NULL)
                ) x (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
                WHERE NOT EXISTS (SELECT 1 FROM #DICTIONARYITEMBASE d WHERE x.ID = d.ID)
                
    INSERT INTO #DICTIONARYITEMBASE (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
        SELECT * FROM ( VALUES
            (357, N'Malajziai', N'Malaysian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 157, NULL, 157, NULL),
            (358, N'Malawi', N'Malawian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 158, NULL, 158, NULL),
            (359, N'Maldív-szigeteki', N'Maldivian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 159, NULL, 159, NULL),
            (360, N'Mali', N'Malian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 160, NULL, 160, NULL),
            (361, N'Máltai', N'Maltese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 161, NULL, 161, NULL),
            (475, N'Man-szigeti', N'Manxman, Manxwoman', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 162, NULL, 162, NULL),
            (362, N'Marokkói', N'Moroccan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 163, NULL, 163, NULL),
            (363, N'Marschall-szigeteki', N'Marshallese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 164, NULL, 164, NULL),
            (364, N'Martiniquei', N'Martiniquais', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 165, NULL, 165, NULL),
            (365, N'Mauritániai', N'Mauritanian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 166, NULL, 166, NULL),
            (366, N'Mauritiusi', N'Mauritian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 167, NULL, 167, NULL),
            (367, N'Mayottei', N'Mahorais', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 168, NULL, 168, NULL),
            (368, N'Mexikói', N'Mexician', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 169, NULL, 169, NULL),
            (369, N'Mianmari (burmai)', N'Burmese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 170, NULL, 170, NULL),
            (476, N'Midway-szigeteki', N'Midway Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 171, NULL, 171, NULL),
            (370, N'Mikronéziai', N'Micronesian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 172, NULL, 172, NULL),
            (371, N'Moldovai', N'Moldovan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 173, NULL, 173, NULL),
            (372, N'Monacoi', N'Monacan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 174, NULL, 174, NULL),
            (373, N'mongol', N'Mongolian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 175, NULL, 175, NULL),
            (375, N'Monserrati', N'Montserratian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 176, NULL, 176, NULL),
            (374, N'Montenegrói', N'Montenegrin', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 177, NULL, 177, NULL),
            (376, N'Mozambiki', N'Mozambican', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 178, NULL, 178, NULL),
            (378, N'Namíbiai', N'Namibian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 179, NULL, 179, NULL),
            (379, N'Naurui', N'Nauruan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 180, NULL, 180, NULL),
            (380, N'NDK-beli', N'NDK-beli', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 181, NULL, 181, NULL),
            (381, N'német', N'German', N'német', NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 182, NULL, 182, NULL),
            (382, N'Nepáli', N'Nepali', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 183, NULL, 183, NULL),
            (383, N'Nicaraguai', N'Nicaraguan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 184, NULL, 184, NULL),
            (384, N'Nigeri', N'Nigerien', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 185, NULL, 185, NULL),
            (385, N'Nigériai', N'Nigerian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 186, NULL, 186, NULL),
            (386, N'Niuei', N'Niuean', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 187, NULL, 187, NULL),
            (387, N'Norfolk-szigeti', N'Norfolk Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 188, NULL, 188, NULL),
            (388, N'norvég', N'Norwegian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 189, NULL, 189, NULL),
            (389, N'NSZK-beli', N'NSZK-beli', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 190, NULL, 190, NULL),
            (390, N'Nyugat-szaharai', N'Sahrawi', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 191, NULL, 191, NULL),
            (391, N'olasz', N'Italian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 192, NULL, 192, NULL),
            (392, N'Ománi', N'Omani', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 193, NULL, 193, NULL),
            (393, N'örmény', N'Armenian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 194, NULL, 194, NULL),
            (394, N'orosz', N'Russian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 195, NULL, 195, NULL),
            (232, N'Osztrák', N'Austrian', N'austrian', NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 196, NULL, 196, NULL),
            (395, N'Pakisztáni', N'Pakistani', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 197, NULL, 197, NULL),
            (396, N'Palaui', N'Palauan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 198, NULL, 198, NULL),
            (477, N'Palesztin (1948 előtt)', N'Palestinian (before 1948)', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 199, NULL, 199, NULL),
            (492, N'Palesztin (1988 után)', N'Palestinian (after 1988)', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 200, NULL, 200, NULL),
            (397, N'Panamai', N'Panamanian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 201, NULL, 201, NULL),
            (478, N'Panamai (Panamai Csatornaövezet)', N'Panamanian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 202, NULL, 202, NULL),
            (398, N'Pápua új-guineai', N'Papua New Guinean', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 203, NULL, 203, NULL),
            (399, N'Paraguayi', N'Paraguayan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 204, NULL, 204, NULL),
            (400, N'Perui', N'Peruvian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 205, NULL, 205, NULL),
            (401, N'Pitcairn-szigeteki', N'Pitcairn Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 206, NULL, 206, NULL),
            (402, N'portugál', N'Portugese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 207, NULL, 207, NULL),
            (403, N'Puerto ricoi', N'Puerto Rican', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 208, NULL, 208, NULL),
            (405, N'Reunioni', N'Reunionese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 209, NULL, 209, NULL),
            (406, N'Rhodéziai', N'Rhodezer', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 210, NULL, 210, NULL),
            (407, N'román', N'Romanian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 211, NULL, 211, NULL),
            (408, N'Ruandai', N'Rwandan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 212, NULL, 212, NULL),
            (483, N'Saint Barthélemy-i', N'Saint Barthelemy', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 213, NULL, 213, NULL),
            (410, N'Saint kitts és nevisi', N'Kittitian and Nevisian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 214, NULL, 214, NULL),
            (411, N'Saint luciai', N'Saint Lucian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 215, NULL, 215, NULL),
            (484, N'Saint Martin-i', N'Saint Martiner', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 216, NULL, 216, NULL),
            (413, N'Saint pierrei és miquelon', N'Frenchman, Frenchwoman', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 217, NULL, 217, NULL),
            (412, N'Saint vincenti', N'Saint Vincentian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 218, NULL, 218, NULL),
            (414, N'Salamon-szigeteki', N'Solomon Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 219, NULL, 219, NULL),
            (415, N'Salvadori', N'Salvadoran', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 220, NULL, 220, NULL),
            (416, N'San marinói', N'Sammarinese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 221, NULL, 221, NULL),
            (417, N'Sao toméi és príncipei', N'Sao Tomean', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 222, NULL, 222, NULL),
            (418, N'Seychelle-szigeteki', N'Seychellois', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 223, NULL, 223, NULL),
            (419, N'Sierra leonei', N'Sierra Leonean', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 224, NULL, 224, NULL),
            (421, N'Spanyol', N'Spaniard', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 225, NULL, 225, NULL),
            (422, N'Sri lankai (ceyloni)', N'Sri Lankan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 226, NULL, 226, NULL),
            (423, N'Surinamei', N'Surinamer', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 227, NULL, 227, NULL),
            (424, N'Svájci', N'Swiss', N'swiss', NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 228, NULL, 228, NULL),
            (425, N'Svalbardi (spitzbergáki)', N'Svalbarder and Spitzbergaker', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 229, NULL, 229, NULL),
            (426, N'Svéd', N'Swede', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 230, NULL, 230, NULL),
            (427, N'Szamoai', N'Samoan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 231, NULL, 231, NULL),
            (428, N'Szaúdi', N'Saudi', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 232, NULL, 232, NULL),
            (429, N'Szenegáli', N'Senegalese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 233, NULL, 233, NULL),
            (409, N'Szent-ilonai', N'Saint Helenian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 234, NULL, 234, NULL),
            (430, N'szerb', N'Serb', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 235, NULL, 235, NULL),
            (431, N'Szerbia-montenegrói', N'Serb, Montenegrin', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 236, NULL, 236, NULL),
            (420, N'Szikkimi', N'Sikkimmer', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 237, NULL, 237, NULL),
            (432, N'Szingapúri', N'Singaporean', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 238, NULL, 238, NULL),
            (433, N'Szír', N'Syrian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 239, NULL, 239, NULL),
            (434, N'szlovák', N'Slovak', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 240, NULL, 240, NULL),
            (435, N'szlovén', N'Slovene', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 241, NULL, 241, NULL),
            (436, N'Szomáliai', N'Somali', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 242, NULL, 242, NULL),
            (479, N'Szomáliparti', N'Somali', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 243, NULL, 243, NULL),
            (437, N'Szovjet', N'Soviet', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 244, NULL, 244, NULL),
            (438, N'Szudáni', N'Sudanese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 245, NULL, 245, NULL),
            (439, N'Szváziföldi', N'Swazi', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 246, NULL, 246, NULL),
            (440, N'Tádzsik', N'Tajikistani', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 247, NULL, 247, NULL),
            (441, N'Tajvani', N'Taiwan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 248, NULL, 248, NULL),
            (442, N'Tanzániai', N'Tanzanian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 249, NULL, 249, NULL),
            (443, N'Thaiföldi', N'Thai', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 250, NULL, 250, NULL),
            (444, N'Togói', N'Togolese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 251, NULL, 251, NULL),
            (445, N'Tokelau-szigeteki', N'Tokelauan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 252, NULL, 252, NULL),
            (446, N'Tongai', N'Tongan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 253, NULL, 253, NULL),
            (447, N'Török', N'Turk', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 254, NULL, 254, NULL),
            (448, N'Trinidadi és tobagoi', N'Trinidadian and Tobagonian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 255, NULL, 255, NULL),
            (449, N'Tunéziai', N'Tunisian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 256, NULL, 256, NULL),
            (450, N'Türkmén', N'Turkmen', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 257, NULL, 257, NULL),
            (451, N'Turks és Caicos-szk-i', N'Turks and Caicos Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 258, NULL, 258, NULL),
            (452, N'Tuvalui', N'Tuvaluan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 259, NULL, 259, NULL),
            (453, N'Ugandai', N'Ugandan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 260, NULL, 260, NULL),
            (454, N'Új-hebridáki', N'New Hebridaker', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 261, NULL, 261, NULL),
            (455, N'Új-kaledóniai', N'New Caledonian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 262, NULL, 262, NULL),
            (456, N'Új-zélandi', N'New Zealander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 263, NULL, 263, NULL),
            (457, N'ukrán', N'Ukrán', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 264, NULL, 264, NULL),
            (458, N'Uruguayi', N'Uruguayan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 265, NULL, 265, NULL),
            (460, N'üzbég', N'Üzbég', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 266, NULL, 266, NULL),
            (461, N'Vanuatui', N'Ni-Vanuatu', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 267, NULL, 267, NULL),
            (462, N'Vatikáni', N'Vatican', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 268, NULL, 268, NULL),
            (463, N'Venezuelai', N'Venezuelan', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 269, NULL, 269, NULL),
            (465, N'Vietnami (1976.07.01-ig)', N'Vietnamese (until 01-07-1976)', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 270, NULL, 270, NULL),
            (464, N'Vietnami (1976.07.02-tól)', N'Vietnamese (since 02-07-1976)', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 271, NULL, 271, NULL),
            (480, N'Wake-szigeti', N'Wake Islander', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 272, NULL, 272, NULL),
            (466, N'Wallisi és futunai', N'Wallis and Futuna Islanders', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 273, NULL, 273, NULL),
            (467, N'Zaire-i', N'Zairen', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 274, NULL, 274, NULL),
            (468, N'Zambiai', N'Zambian', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 275, NULL, 275, NULL),
            (469, N'Zimbabvei', N'Zimbabwean', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 276, NULL, 276, NULL),
            (470, N'Zöld-foki köztársasági', N'Cape Verdean', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 277, NULL, 277, NULL),
            (8456, N'Egyesült arab emírségeki', N'Egyesült arab emírségeki', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 278, NULL, 278, NULL),
            (8457, N'Vietnámi', N'Vietnamese', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 279, NULL, 279, NULL),
            (8741, N'Moldáv', N'Moldáv', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 280, NULL, 280, NULL),
            (8742, N'Montenegró', N'Montenegró', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 281, NULL, 281, NULL),
            (8743, N'Palesztinai', N'Palesztinai', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 282, NULL, 282, NULL),
            (8744, N'San Marino-i', N'San Marino-i', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 283, NULL, 283, NULL),
            (8745, N'Szerb-montenegrói', N'Szerb-montenegrói', NULL, NULL, NULL, N'Allampolgarsag', 9, N'T', N'T', 284, NULL, 284, NULL),
            (906, N'Na', N'Na', NULL, NULL, NULL, N'CimTipus', 16, N'T', N'T', 1, NULL, 1, NULL),
            (907, N'Állandó lakcím', N'Állandó lakcím', NULL, NULL, NULL, N'CimTipus', 16, N'T', N'T', 2, NULL, 2, NULL),
            (908, N'Ideiglenes lakcím', N'Ideiglenes lakcím', NULL, NULL, NULL, N'CimTipus', 16, N'T', N'T', 3, NULL, 3, NULL),
            (909, N'Tartózkodási hely', N'Tartózkodási hely', NULL, NULL, NULL, N'CimTipus', 16, N'T', N'T', 4, NULL, 4, NULL),
            (910, N'Intézménycím', N'Intézménycím', NULL, NULL, NULL, N'CimTipus', 16, N'T', N'T', 5, NULL, 5, NULL),
            (1565, N'Nincs adat', N'Nincs adat', NULL, NULL, NULL, N'Nem', 1, N'F', N'T', 1, NULL, 1, NULL),
            (4, N'Férfi', N'Férfi', NULL, NULL, NULL, N'Nem', 1, N'F', N'T', 2, NULL, 2, NULL),
            (5, N'Nő', N'Nő', NULL, NULL, NULL, N'Nem', 1, N'F', N'T', 3, NULL, 3, NULL)
                ) x (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR)
                WHERE NOT EXISTS (SELECT 1 FROM #DICTIONARYITEMBASE d WHERE x.ID = d.ID)
                

    -- A 'Default' elemek vege
    

    SET IDENTITY_INSERT [dbo].[T_DICTIONARYITEMBASE] ON
	INSERT INTO dbo.T_DICTIONARYITEMBASE (ID, C_NAME, C_NAME_1, C_NAME_2, C_NAME_3, C_NAME_4, C_TYPE, C_DICTIONARYTYPEID, C_PROTECTED, C_VISIBLE, C_ORDER, C_CODE, C_VALUE, C_COLOR, C_INTEZMENYID, C_TANEVID, TOROLT, SERIAL, LASTCHANGED, CREATED, MODIFIER, CREATOR, ELOZOTANEVIREKORDID)
		SELECT tdib.ID, tdib.C_NAME, tdib.C_NAME_1, tdib.C_NAME_2, tdib.C_NAME_3, tdib.C_NAME_4, tdib.C_TYPE, tdib.C_DICTIONARYTYPEID, tdib.C_PROTECTED, tdib.C_VISIBLE, tdib.C_ORDER, tdib.C_CODE, tdib.C_VALUE, tdib.C_COLOR, tn.IntezmenyId, tn.TanevId, 'F', 0, GETDATE(), GETDATE(), NULL, NULL,tdib.ID
		FROM #DICTIONARYITEMBASE tdib
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.T_DICTIONARYITEMBASE dib WHERE dib.ID = tdib.ID AND dib.C_INTEZMENYID = tn.IntezmenyId AND dib.C_TANEVID = tn.TanevId)

    SET IDENTITY_INSERT [dbo].[T_DICTIONARYITEMBASE] OFF

    -- DictionaryItem elemek 

    INSERT INTO [dbo].[T_NYUGDIJTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_NYUGDIJTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 186 

    INSERT INTO [dbo].[T_DUALISHELYSZINTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_DUALISHELYSZINTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 185 

    INSERT INTO [dbo].[T_AGAZATALAPVIZSGAEREDTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_AGAZATALAPVIZSGAEREDTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 181 

    INSERT INTO [dbo].[T_JOGVKELETKEZESEJOGCIMTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_JOGVKELETKEZESEJOGCIMTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 184 

    INSERT INTO [dbo].[T_AGAZATALAPOKTMEGNEVTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_AGAZATALAPOKTMEGNEVTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 180 

    INSERT INTO [dbo].[T_TBJOGVISZONYTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_TBJOGVISZONYTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 183 

    INSERT INTO [dbo].[T_JOGVMEGSZUNESEJOGCIMTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_JOGVMEGSZUNESEJOGCIMTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 182 

    INSERT INTO [dbo].[T_ESZKOZIGENYLESSTATUSZ] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ESZKOZIGENYLESSTATUSZ] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 179 

    INSERT INTO [dbo].[T_RESZSZAKMATIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_RESZSZAKMATIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 178 

    INSERT INTO [dbo].[T_AGAZATRESZSZAKMATIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_AGAZATRESZSZAKMATIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 176 

    INSERT INTO [dbo].[T_SZAKMARESZSZAKMATIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_SZAKMARESZSZAKMATIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 177 

    INSERT INTO [dbo].[T_FOGYATEKOSSAGTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_FOGYATEKOSSAGTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 175 

    INSERT INTO [dbo].[T_TARTJOGOKMANYTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_TARTJOGOKMANYTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 173 

    INSERT INTO [dbo].[T_TARTOZKODASJOGCIMTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_TARTOZKODASJOGCIMTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 170 

    INSERT INTO [dbo].[T_TKJOGALAPTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_TKJOGALAPTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 167 

    INSERT INTO [dbo].[T_FELEKEZETTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_FELEKEZETTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 169 

    INSERT INTO [dbo].[T_NEMZETISEGTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_NEMZETISEGTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 168 

    INSERT INTO [dbo].[T_TANSZAKTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_TANSZAKTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 172 

    INSERT INTO [dbo].[T_MUFAJTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_MUFAJTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 171 

    INSERT INTO [dbo].[T_DIGTAMESZKOZTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_DIGTAMESZKOZTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 166 

    INSERT INTO [dbo].[T_DIGPLATFORMTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_DIGPLATFORMTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 165 

    INSERT INTO [dbo].[T_DIGESZKOZTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_DIGESZKOZTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 164 

    INSERT INTO [dbo].[T_SZAKKEPESITESNKTTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_SZAKKEPESITESNKTTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 162 

    INSERT INTO [dbo].[T_TANULMANYITERULETNKTTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_TANULMANYITERULETNKTTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 161 

    INSERT INTO [dbo].[T_TULAJDONTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_TULAJDONTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 160 

    INSERT INTO [dbo].[T_SZAKIRANYNKTTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_SZAKIRANYNKTTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 163 

    INSERT INTO [dbo].[T_ORAFILETIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ORAFILETIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 159 

    INSERT INTO [dbo].[T_SZIRSTATTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_SZIRSTATTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 158 

    INSERT INTO [dbo].[T_ALKALMAZOTTMUNKAKORTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ALKALMAZOTTMUNKAKORTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 157 

    INSERT INTO [dbo].[T_EGYEBTEVEKENYSEGEKTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_EGYEBTEVEKENYSEGEKTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 155 

    INSERT INTO [dbo].[T_AGAZATUJSZKTTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_AGAZATUJSZKTTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 149 

    INSERT INTO [dbo].[T_SZAKMATIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_SZAKMATIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 150 

    INSERT INTO [dbo].[T_SZAKMAIRANYTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_SZAKMAIRANYTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 151 

    INSERT INTO [dbo].[T_JUTTATASTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_JUTTATASTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 154 

    INSERT INTO [dbo].[T_JOGVISZONYTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_JOGVISZONYTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 152 

    INSERT INTO [dbo].[T_BANKSZAMLATULAJDONOSTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_BANKSZAMLATULAJDONOSTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 153 

    INSERT INTO [dbo].[T_AMIJELLEMZOTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_AMIJELLEMZOTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 148 

    INSERT INTO [dbo].[T_TANTERVIJELLEMZOTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_TANTERVIJELLEMZOTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 147 

    INSERT INTO [dbo].[T_ARANYJANOSPROGRAMTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ARANYJANOSPROGRAMTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 145 

    INSERT INTO [dbo].[T_SNITANULONEVELESIFORMATIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_SNITANULONEVELESIFORMATIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 144 

    INSERT INTO [dbo].[T_BESZERZESIIGENYSATUSZ] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_BESZERZESIIGENYSATUSZ] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 142 

    INSERT INTO [dbo].[T_AMIFOTARGY] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_AMIFOTARGY] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 141 

    INSERT INTO [dbo].[T_AMITANTARGYKOTELEZOSEG] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_AMITANTARGYKOTELEZOSEG] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 140 

    INSERT INTO [dbo].[T_AMITAGOZAT] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_AMITAGOZAT] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 139 

    INSERT INTO [dbo].[T_MUVESZETIAG] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_MUVESZETIAG] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 137 

    INSERT INTO [dbo].[T_ZENEMUVESZETIAGTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ZENEMUVESZETIAGTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 138 

    INSERT INTO [dbo].[T_ESLTANTARGYKATEGORIA] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ESLTANTARGYKATEGORIA] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 136 

    INSERT INTO [dbo].[T_TAVOLLETTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_TAVOLLETTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 134 

    INSERT INTO [dbo].[T_TAVOLLETSTATUSZ] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_TAVOLLETSTATUSZ] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 135 

    INSERT INTO [dbo].[T_DOKUMENTUMELEMISZINT] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_DOKUMENTUMELEMISZINT] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 133 

    INSERT INTO [dbo].[T_OKTNEVELESIKATEGORIA] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_OKTNEVELESIKATEGORIA] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 132 

    INSERT INTO [dbo].[T_DOKUMENTUMTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_DOKUMENTUMTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 131 

    INSERT INTO [dbo].[T_DOKUMENTUMSTATUSZ] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_DOKUMENTUMSTATUSZ] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 128 

    INSERT INTO [dbo].[T_DOKUMENTUMKULCSSZOTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_DOKUMENTUMKULCSSZOTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 130 

    INSERT INTO [dbo].[T_DOKUMENTUMKATEGORIA] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_DOKUMENTUMKATEGORIA] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 129 

    INSERT INTO [dbo].[T_ERTMONDATBANKTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ERTMONDATBANKTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 127 

    INSERT INTO [dbo].[T_KKKEPESITESJELLTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_KKKEPESITESJELLTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 119 

    INSERT INTO [dbo].[T_KKVEGZETTSEGTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_KKVEGZETTSEGTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 115 

    INSERT INTO [dbo].[T_KKAMINEPZENETIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_KKAMINEPZENETIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 122 

    INSERT INTO [dbo].[T_KKAMIELEKTROAKUTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_KKAMIELEKTROAKUTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 124 

    INSERT INTO [dbo].[T_KKAMIVEGZETTSEGTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_KKAMIVEGZETTSEGTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 116 

    INSERT INTO [dbo].[T_KKAMITERULETTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_KKAMITERULETTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 118 

    INSERT INTO [dbo].[T_KKAMIJAZZZENETIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_KKAMIJAZZZENETIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 123 

    INSERT INTO [dbo].[T_KKTANITOMUVTERTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_KKTANITOMUVTERTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 125 

    INSERT INTO [dbo].[T_KKAMIKLASSZIKUSTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_KKAMIKLASSZIKUSTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 121 

    INSERT INTO [dbo].[T_KKGYOGYPEDTERTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_KKGYOGYPEDTERTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 126 

    INSERT INTO [dbo].[T_KKKEPESITESTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_KKKEPESITESTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 117 

    INSERT INTO [dbo].[T_KKSZAKTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_KKSZAKTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 114 

    INSERT INTO [dbo].[T_KKAMIZENEMUVKATTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_KKAMIZENEMUVKATTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 120 

    INSERT INTO [dbo].[T_KIRSZOLGALTATASTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_KIRSZOLGALTATASTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 113 

    INSERT INTO [dbo].[T_SZAKTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_SZAKTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 112 

    INSERT INTO [dbo].[T_GIMNAZIUMIEVFOLYAMTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_GIMNAZIUMIEVFOLYAMTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 111 

    INSERT INTO [dbo].[T_LICENCEFUNKCIOKTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_LICENCEFUNKCIOKTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 109 

    INSERT INTO [dbo].[T_PENZNEMTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_PENZNEMTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 108 

    INSERT INTO [dbo].[T_MEROHELYTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_MEROHELYTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 106 

    INSERT INTO [dbo].[T_KOZMUSZAMLATIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_KOZMUSZAMLATIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 107 

    INSERT INTO [dbo].[T_HITTANTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_HITTANTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 105 

    INSERT INTO [dbo].[T_VEGZETTSEGTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_VEGZETTSEGTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 103 

    INSERT INTO [dbo].[T_ORATULAJDONSAGTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ORATULAJDONSAGTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 102 

    INSERT INTO [dbo].[T_KOZOSSEGISZOLGALATTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_KOZOSSEGISZOLGALATTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 101 

    INSERT INTO [dbo].[T_VEZETOIORASZAMOKTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_VEZETOIORASZAMOKTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 96 

    INSERT INTO [dbo].[T_FOGLALKOZTATASTIPUSA] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_FOGLALKOZTATASTIPUSA] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 95 

    INSERT INTO [dbo].[T_ERETTSEGISZINT] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ERETTSEGISZINT] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 97 

    INSERT INTO [dbo].[T_ERETTSEGITIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ERETTSEGITIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 98 

    INSERT INTO [dbo].[T_ERETTSEGITANTARGY] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ERETTSEGITANTARGY] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 99 

    INSERT INTO [dbo].[T_RESZSZAKKEPESITESTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_RESZSZAKKEPESITESTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 94 

    INSERT INTO [dbo].[T_AGAZATTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_AGAZATTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 91 

    INSERT INTO [dbo].[T_SZAKMACSOPORTTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_SZAKMACSOPORTTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 93 

    INSERT INTO [dbo].[T_SZAKKEPESITESTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_SZAKKEPESITESTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 92 

    INSERT INTO [dbo].[T_MUKODESIHELYTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_MUKODESIHELYTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 90 

    INSERT INTO [dbo].[T_IKTESZKOZHASZNALATMOD] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_IKTESZKOZHASZNALATMOD] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 87 

    INSERT INTO [dbo].[T_ELSODLEGESIKTESZKOZ] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ELSODLEGESIKTESZKOZ] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 88 

    INSERT INTO [dbo].[T_WIFIELERES] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_WIFIELERES] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 89 

    INSERT INTO [dbo].[T_IKTKOMPETENCIASZINT] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_IKTKOMPETENCIASZINT] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 86 

    INSERT INTO [dbo].[T_ENAPLOHASZNALAT] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ENAPLOHASZNALAT] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 85 

    INSERT INTO [dbo].[T_TAGOZATTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_TAGOZATTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 83 

    INSERT INTO [dbo].[T_FELVETELSTATUSZATIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_FELVETELSTATUSZATIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 84 

    INSERT INTO [dbo].[T_IDOKERET] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_IDOKERET] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 80 

    INSERT INTO [dbo].[T_OSSZEGTULLEPESTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_OSSZEGTULLEPESTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 81 

    INSERT INTO [dbo].[T_HIBAJEGYSTATUSZ] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_HIBAJEGYSTATUSZ] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 77 

    INSERT INTO [dbo].[T_PRIORITAS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_PRIORITAS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 78 

    INSERT INTO [dbo].[T_FORRAS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_FORRAS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 79 

    INSERT INTO [dbo].[T_SZERZODOSZEMELYPOZICIO] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_SZERZODOSZEMELYPOZICIO] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 76 

    INSERT INTO [dbo].[T_ENAPLOTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ENAPLOTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 73 

    INSERT INTO [dbo].[T_ENAPLOFORRAS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ENAPLOFORRAS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 75 

    INSERT INTO [dbo].[T_ENAPLOINFRA] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ENAPLOINFRA] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 74 

    INSERT INTO [dbo].[T_ESLADATTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ESLADATTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 72 

    INSERT INTO [dbo].[T_SOROLASOKATIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_SOROLASOKATIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 41 

    INSERT INTO [dbo].[T_MUNKAIDOKEDVEZMENYOKA] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_MUNKAIDOKEDVEZMENYOKA] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 71 

    INSERT INTO [dbo].[T_BESOROLASIFOKOZATTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_BESOROLASIFOKOZATTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 70 

    INSERT INTO [dbo].[T_PEDAGOGUSSTATUSZ] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_PEDAGOGUSSTATUSZ] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 69 

    INSERT INTO [dbo].[T_KRETATIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_KRETATIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 68 

    INSERT INTO [dbo].[T_SZEREPKORTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_SZEREPKORTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 67 

    INSERT INTO [dbo].[T_SYSTEMSETTINGSCONTROLTYPE] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_SYSTEMSETTINGSCONTROLTYPE] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 62 

    INSERT INTO [dbo].[T_MULASZTASSTATISZTIKATIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_MULASZTASSTATISZTIKATIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 60 

    INSERT INTO [dbo].[T_RENDSZERBEALLITASTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_RENDSZERBEALLITASTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 61 

    INSERT INTO [dbo].[T_KIZAROCSOPORTTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_KIZAROCSOPORTTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 59 

    INSERT INTO [dbo].[T_ERTESITESTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ERTESITESTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 57 

    INSERT INTO [dbo].[T_ERTESITESSZINTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ERTESITESSZINTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 56 

    INSERT INTO [dbo].[T_TEVEKENYSEGTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_TEVEKENYSEGTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 58 

    INSERT INTO [dbo].[T_ELETPALYAMODELLFOKOZATTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ELETPALYAMODELLFOKOZATTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 53 

    INSERT INTO [dbo].[T_ELETPALYAMODELLSTATUSZOK] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ELETPALYAMODELLSTATUSZOK] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 54 

    INSERT INTO [dbo].[T_MAGATARTASERTEKELESTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_MAGATARTASERTEKELESTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 50 

    INSERT INTO [dbo].[T_SZORGALOMERTEKELESTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_SZORGALOMERTEKELESTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 51 

    INSERT INTO [dbo].[T_ESEMENYTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ESEMENYTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 48 

    INSERT INTO [dbo].[T_IGAZOLASTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_IGAZOLASTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 47 

    INSERT INTO [dbo].[T_MULASZTASTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_MULASZTASTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 43 

    INSERT INTO [dbo].[T_ERTEKELESMOD] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ERTEKELESMOD] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 45 

    INSERT INTO [dbo].[T_OSZTALYZATTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_OSZTALYZATTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 44 

    INSERT INTO [dbo].[T_ERTEKELESTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ERTEKELESTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 46 

    INSERT INTO [dbo].[T_HELYETTESITESTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_HELYETTESITESTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 42 

    INSERT INTO [dbo].[T_HETNAPJATIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_HETNAPJATIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 40 

    INSERT INTO [dbo].[T_HETIRENDTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_HETIRENDTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 49 

    INSERT INTO [dbo].[T_NAPTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_NAPTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 39 

    INSERT INTO [dbo].[T_FOGLALKOZASTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_FOGLALKOZASTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 38 

    INSERT INTO [dbo].[T_TARGYKATEGORIATIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_TARGYKATEGORIATIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 36 

    INSERT INTO [dbo].[T_EVFOLYAMTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_EVFOLYAMTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 37 

    INSERT INTO [dbo].[T_OKTATASINEVELESIFELADAT] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_OKTATASINEVELESIFELADAT] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 35 

    INSERT INTO [dbo].[T_ERTELMIFOGYATEKOSTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ERTELMIFOGYATEKOSTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 31 

    INSERT INTO [dbo].[T_HALLASSERULTTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_HALLASSERULTTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 32 

    INSERT INTO [dbo].[T_LATASSERULTTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_LATASSERULTTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 33 

    INSERT INTO [dbo].[T_ETKEZESIKEDVEZMENYTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ETKEZESIKEDVEZMENYTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 28 

    INSERT INTO [dbo].[T_HATRANYOSHELYZETTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_HATRANYOSHELYZETTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 29 

    INSERT INTO [dbo].[T_VALLAS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_VALLAS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 30 

    INSERT INTO [dbo].[T_TANTERVIJELLEG] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_TANTERVIJELLEG] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 27 

    INSERT INTO [dbo].[T_TESTNEVELESTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_TESTNEVELESTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 26 

    INSERT INTO [dbo].[T_ESZKOZJELLEG] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ESZKOZJELLEG] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 23 

    INSERT INTO [dbo].[T_ESZKOZCEL] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ESZKOZCEL] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 24 

    INSERT INTO [dbo].[T_MERTEKEGYSEGTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_MERTEKEGYSEGTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 25 

    INSERT INTO [dbo].[T_FENNTARTOTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_FENNTARTOTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 19 

    INSERT INTO [dbo].[T_CSOPORTTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_CSOPORTTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 22 

    INSERT INTO [dbo].[T_ROKONSAGIFOK] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ROKONSAGIFOK] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 18 

    INSERT INTO [dbo].[T_MUNKAKORTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_MUNKAKORTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 12 

    INSERT INTO [dbo].[T_ANYANYELV] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ANYANYELV] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 10 

    INSERT INTO [dbo].[T_IGAZOLVANYTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_IGAZOLVANYTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 11 

    INSERT INTO [dbo].[T_MUNKAVISZONYTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_MUNKAVISZONYTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 13 

    INSERT INTO [dbo].[T_ESZKOZTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ESZKOZTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 6 

    INSERT INTO [dbo].[T_ESZKOZKATEGORIA] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ESZKOZKATEGORIA] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 7 

    INSERT INTO [dbo].[T_INGYENESTANKONYVELLATAS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_INGYENESTANKONYVELLATAS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 2 

    INSERT INTO [dbo].[T_TEREMTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_TEREMTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 8 

    INSERT INTO [dbo].[T_MAGANTANULOSAGOKA] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_MAGANTANULOSAGOKA] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 14 

    INSERT INTO [dbo].[T_HONAPOK] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_HONAPOK] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 55 

    INSERT INTO [dbo].[T_DIAKIGAZOLVANYIGENYTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_DIAKIGAZOLVANYIGENYTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 34 

    INSERT INTO [dbo].[T_KOZTERULETJELLEG] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_KOZTERULETJELLEG] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 17 

    INSERT INTO [dbo].[T_DIAKIGAZOLVANYIGENYSTATUSZ] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_DIAKIGAZOLVANYIGENYSTATUSZ] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 52 

    INSERT INTO [dbo].[T_ORSZAGTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ORSZAGTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 15 

    INSERT INTO [dbo].[T_ESEMENYSTATUSZ] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ESEMENYSTATUSZ] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 4 

    INSERT INTO [dbo].[T_EMAILTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_EMAILTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 21 

    INSERT INTO [dbo].[T_TELEFONTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_TELEFONTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 20 

    INSERT INTO [dbo].[T_ALLAMPOLGARSAG] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_ALLAMPOLGARSAG] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 9 

    INSERT INTO [dbo].[T_CIMTIPUS] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_CIMTIPUS] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 16 

    INSERT INTO [dbo].[T_NEM] ([ID], [C_ALINTEZMENYID], [C_ALTANEVID])
	    SELECT tdib.ID, tn.IntezmenyId, tn.TanevId
	    FROM #DICTIONARYITEMBASE tdib 
            CROSS JOIN #Tanevek tn
        WHERE NOT EXISTS (SELECT 1 FROM dbo.[T_NEM] di WHERE di.ID = tdib.ID AND di.C_ALINTEZMENYID = tn.IntezmenyId AND di.C_ALTANEVID = tn.TanevId)
	        AND tdib.C_DICTIONARYTYPEID = 1  DROP TABLE IF EXISTS #DICTIONARYITEMBASE

END